/*
JMP Clinical Analytical Process

Name: Adverse Events Bayesian Hierarchical Model
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath TrtUse PopulationSet IncSF IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL AETermLevel AEGroupLevel  
TrtVar SubjectFilter WhereStmt WhereForReports AEWhereStmt AESerOnly TEAEOnly OutPath ServerOutPath BModelType BTimeScale
RefArm TestArm nBurnIn nPostSamp nThin NMarkov BaySeed SeedVal VolcanoXCat ColorTheme AEBayesianHModel_OutData MacroPath AEBayesianHModelScript
hp_mu_G00 hp_mu_T00 hp_tau2_G00 hp_tau2_T00 hp_A_G hp_A_T hp_A_G00 hp_A_T00 hp_B_G hp_B_T hp_B_G00 hp_B_T00 hp_L_A hp_L_B TrtOffSet MergeSuppDom OutName IgnoreTEAEFlags; 

*ProcessBody;

%macro AEBayesianHModel;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur","TreatmentUse","PopulationFlags","AETermValues"}}$
$INCD: {"AE"}$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CMP: RSYS$ $UE: DM.RFSTDTC and DM.RFENDTC and AE.AESTDTC$
   $NOTES: Analysis currently fixed to TEAEs. DM.RFSTDTC and DM.RFENDTC are used to calculate exposure to first event or total time for no events.$
$CMP: OSYS$ $UE: AE.AETRTEM or AE.TRTEMFL$  $NOTES: used for treatment emergent flags if available$   
$CVS_STOP$
*/

*initialization and argument checking;
%let WhereForReports=&WhereStmt;
%let ProcessName = AEBayesianHModel; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%let TEAEOnly = Treatment emergent adverse events;

%if ^%sysfunc(findw(&domains, AE)) %then %do;
   %put [MS_S_0011];
   %put ERROR: The AE or ADAE data set does not exist for this study.;
   %let exiterror=1;
   %goto exit;
%end;
   
%AESerOnlyToAEWhereStmt;

%if %upcase(&BaySeed) = RANDOMLY GENERATE  %then %do;
   data _null_;
     val = ranuni(0) * 100000;
     val = floor(val);
     if val < 1 then val = 1;
     else if val > 99999 then val = 99999;
     call symput("SeedVal", left(trim(val)));
   run;
%end;

%let trOddsRatio = %nrbquote(%LocaleGet(OddsRatio_Text));
%let trHazardRatio = %nrbquote(%LocaleGet(HazardRatio_Text));
%let trDifferenceinProportions = %nrbquote(%LocaleGet(DifferenceinProportions_Text));
%let trDifferenceinHazardRates = %nrbquote(%LocaleGet(DifferenceinHazardRates_Text));
%let trMCMCDiagnostics = %nrbquote(%LocaleGet(MCMCDiagnostics_Text));

%if %upcase(&BModelType) = LOGISTIC %then %let BStat = &trOddsRatio;
%else %if %upcase(&BModelType) = LOG LINEAR %then %let BStat = &trHazardRatio;

%if %upcase(&BModelType) = LOGISTIC %then %let BStat2 = &trDifferenceinProportions;
%else %if %upcase(&BModelType) = LOG LINEAR %then %let BStat2 = &trDifferenceinHazardRates;

*** Get Variable Based on Drop Down Label ***;
%GETMEDDRAVAR(label = &AETermLevel, column = CatName);
%GETMEDDRAVAR(label = &AEGroupLevel, column = GroupVar);

*** Check if Catname null ***;
%if ^%sysfunc(findw(&AEVARS, &CatName)) %then %do;
   %put [MS_S_0017];
   %put ERROR: Selected AE term variable has no non-missing values.  Please select an alternate variable.;
   %let exiterror = 1;
   %goto exit;
%end;

*** Check if Group null ***;
%if ^%sysfunc(findw(&AEVARS, &GroupVar)) %then %do;
   %put NOTE: Selected AE grouping variable has no non-missing values.  Grouping variable is set to term variable.;
   %let GroupVar = &CatName;
%end;

*** Ensure that the Grouping Variable is at least the level of Term Variable ***;
%COMPAREMEDDRA(term = &CatName, group = &GroupVar);
%if &exiterror %then %goto exit;

*** Check if Date Missing ***;
%if ^%sysfunc(findw(&AEVARS, AESTDTC)) %then %do;
   %put [MS_S_0018];
   %put ERROR: AESTDTC is missing.;
   %let exiterror = 1;
   %goto exit;
%end;

%LOADDATA(domlist = AE,mergesupp = &MergeSuppDom, checkreview=1);
%include "&MacroPath./TEAEOTFU.sas";
%if &exiterror %then %goto exit;       

%if %length(&AEWhereStmt) %then %do;
   data ae_ws;
      set &in_ae;
      where %unquote(&AEWhereStmt);
   run;
   %if &syserr > 4 %then %do;
      %put [MS_S_0019];
      %put ERROR: The specified filter &AEWhereStmt is invalid.  Change the filter to be valid for the domain you are considering.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %NObsVars(ae_ws);
   %if ^&nobs %then %do;
      %put [MS_S_0020];
      %put ERROR: No observations satisfy &AEWhereStmt..;
      %put Please relax the filter.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %let in_ae = ae_ws;
%end; 

proc sort data = &in_ae out = ae;
   by usubjid &GroupVar &CatName aestdtc;
run;

%let EventLabel = &CatName;
%CheckLabel(ae,EventLabel);
%let GroupLabel = &GroupVar;
%CheckLabel(ae,GroupLabel);

proc sort data = &in_adsl out = adsl;
   by usubjid;
run;
%if %sysfunc(findw(&ADSLVARS, RFSTDTC)) %then %let adslst = RFSTDTC;
%else %let adslst =;
%if %sysfunc(findw(&ADSLVARS, RFENDTC)) %then %let adslen = RFENDTC;
%else %let adslen =;

%if (^%length(&adslst) or ^%length(&adslen)) and %sysfunc(findw(&domains, DM)) %then %do;
   %if %sysfunc(findw(&DMVARS, RFSTDTC)) %then %let dmst = RFSTDTC;
   %else %let dmst =;
   %if %sysfunc(findw(&DMVARS, RFENDTC)) %then %let dmen = RFENDTC;
   %else %let dmen =;

   %if %length(&adslst) %then %let dmst =;
   %else %if %length(&dmst) and ^%length(&adslst) %then %let adslst = RFSTDTC;  
   
   %if %length(&adslen) %then %let dmen =;
   %else %if %length(&dmen) and ^%length(&adslen) %then %let adslen = RFENDTC;  

   %if %length(&dmst) or %length(&dmen) %then %do;
      %LOADDATA(domlist = DM,mergesupp = 0,checkreview=0);
      proc sort data = &in_dm out = dm(keep = usubjid &dmst &dmen);
         by usubjid;
      run;
      
      data adsl;
         merge adsl(in = ina) dm;
         by usubjid;
         if ina;
      run;
      
      proc datasets nolist;
         delete dm;
      quit;
   %end;
%end;

%if (^%length(&adslst) or ^%length(&adslen)) %then %do;
   %if ^%length(&adslst) %then %let adslst = RFSTDTC;
   %if ^%length(&adslen) %then %let adslen = RFENDTC;
      %put [MS_S_0021];
   %put ERROR: ADSL, ADDM, or DM is missing the following required variables: &adslst  &adslen;
   %let exiterror = 1;
   %goto exit;
%end;

*** Get Treatment Levels ***;
proc sql noprint;         
   select distinct &TrtName
      into :Treatments separated by '~'
      from adsl
      order by &TrtName;
quit;

%let TrtCnt = 1;
%do %while(%length(%scan(%bquote(&Treatments),&TrtCnt,~)));
   %let TrtCnt = %eval(&TrtCnt+1);
%end;
%let TrtCnt = %eval(&TrtCnt-1);
%if &TrtCnt = 1 %then %do;
   %put [MS_S_0022];
   %put ERROR: There is only a single level for the treatment variable specified.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&RefArm) %then %do;
   %if %sysfunc(findw(%bquote(~%upcase(&Treatments)~),PLACEBO, ~)) %then %let RefArm =  Placebo;
   %else %let RefArm = %scan(%bquote(&Treatments), -1, ~);
%end;
%else %do;
   %if ^%sysfunc(findw(%bquote(~%upcase(&Treatments)~), %upcase(&RefArm), ~)) %then %do;
      %put [MS_S_0023];
      %put ERROR: The specified reference treatment &RefArm does not exist.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end;
%put Reference Arm: &RefArm;

%if ^%length(&TestArm) %then %do;
   %let TestArm = %scan(%bquote(&Treatments), 1, ~);
   %if %quote(%upcase(&TestArm)) = PLACEBO %then %let TestArm = %scan(%bquote(&Treatments), 2, ~); 
%end;
%else %do;
   %if ^%sysfunc(findw(~%upcase(&Treatments)~, %upcase(&TestArm), ~)) %then %do;
      %put [MS_S_0024];
      %put ERROR: The specified test treatment &TestArm does not exist.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end; 
%put Test Arm: &TestArm;

data ae;
   merge adsl(in = ina keep = usubjid &TrtName where = (&TrtName in ("&RefArm","&TestArm"))) ae(keep = usubjid &CatName &GroupVar aestdtc in = inb);
   by usubjid;
   if ina and inb;
run;
%NObsVars(ae);
%if ^&nobs %then %do;
   %put [MS_S_0025];
   %put ERROR: The selected subjects have no adverse events.;
   %let exiterror = 1;
   %goto exit;
%end;

data ae;
   set ae;
   by usubjid &GroupVar &CatName aestdtc;
   if first.&CatName;
run;

proc sql noprint;
   create table event(keep = &GroupVar &CatName) as
      select distinct &GroupVar, &CatName
      from ae;
   create table fill as
      select *
      from adsl(keep = usubjid &TrtName RFSTDTC RFENDTC), event;       
   create table fill2 as
      select *
      from fill natural left join ae;
quit;

proc datasets nolist;
   delete event fill;
run;

data fill2 (drop = aestdtc rfstdtc rfendtc);
   set fill2;
   if not missing(aestdtc) then do;
      %calcStudyDay(aestdtc,risktime);
      /*
      risktime = %DATE9(var = aestdtc) - %DATE9(var = rfstdtc);
      */
      count = 1;
   end;
   else do;
      %calcStudyDay(rfendtc,risktime);
      /*
      risktime = %DATE9(var = rfendtc) - %DATE9(var = rfstdtc);
      */
      count = 0;  
   end;
   total = 1;
   risktime = max(0,risktime-1);
   %if %upcase(&BTimeScale) = WEEKS %then %do;
      %let trBTimeScale = %nrbquote(%LocaleGet(Weeks_Text));
      risktime = risktime / 7;
   %end;
   %else %if %upcase(&BTimeScale) = MONTHS %then %do;
      %let trBTimeScale = %nrbquote(%LocaleGet(Months_Text));
      risktime = risktime / 30.4375;
   %end;
   %else %if %upcase(&BTimeScale) = YEARS %then %do;
      %let trBTimeScale = %nrbquote(%LocaleGet(Years_Text));
      risktime = risktime / 365.25;
   %end;
   %else %if %upcase(&BTimeScale) = DAYS %then %do;
      %let trBTimeScale = %nrbquote(%LocaleGet(Days_Text));
   %end; 
   %else %do;
   	%let trBTimeScale =;
   %end;
run;

/*
data outlib.fill2;
   set fill2;
run;

%put ERROR: Not!;
%let exiterror = 1;
%goto exit;
*/

proc means sum noprint nway data = fill2;
   class &GroupVar &CatName &TrtName;
   var count total risktime;
   output out = stats(drop = _freq_ _type_) sum = count total risktime;
run;

proc datasets nolist;
   delete fill2;
run;

%global numAE numSoc;

data stats;
   set stats end = eof;
   by &GroupVar &CatName;
   retain group_num;
   if _n_ = 1 then group_num = 1;
   else if first.&GroupVar then group_num = group_num + 1;
   if eof then call symput("numSoc", trim(left(group_num)));
run;

data stats;
   set stats end = eof;
   by &GroupVar &CatName;
   retain event_num;
   if _n_ = 1 then event_num = 1;
   else if first.&CatName then event_num = event_num + 1;
   if eof then call symput("numAE", trim(left(event_num)));   
run;

data stats;
   set stats;
   by &GroupVar &CatName;
   if first.&GroupVar then do;
      call symputx("_Group"||left(group_num),trim(&GroupVar));
   end;
   if first.&CatName then do;
      call symputx("_Event"||left(event_num),trim(&CatName));
   end;
run;

data stats;
   set stats;
   if upcase(&TrtName) = "%upcase(&RefArm)" then trt = 0;
   else if upcase(&TrtName) = "%upcase(&TestArm)" then trt = 1;
run;

%put Number of Events = &numAE; 
%put Number of Classes = &numSoc;

%let mu_G00 = &hp_mu_G00;
%let mu_T00 = &hp_mu_T00;
%let tau2_G00 = &hp_tau2_G00;
%let tau2_T00 = &hp_tau2_T00;
%let A_G = &hp_A_G;
%let A_T = &hp_A_T;
%let A_G00 = &hp_A_G00;
%let A_T00 = &hp_A_T00;
%let B_G = &hp_B_G;
%let B_T = &hp_B_T;
%let B_G00 = &hp_B_G00;
%let B_T00 = &hp_B_T00;
%let L_A = &hp_L_A;
%let L_B = &hp_L_B;

*ods trace on / listing;
ods trace off;
%macro MCMC(suffix = , seedme = );
   %let trChain = %nrbquote(%LocaleGet(Chain_Text));
   %let trSeed = %nrbquote(%LocaleGet(Seed_Text));
   title "&trChain #&Suffix, &trSeed = &seedme";
   proc mcmc data = stats seed = &seedme nmc = &nPostSamp nbi = &nBurnIn thin = &nThin dic monitor = (_parms_ T_1-T_&numAE D_1-D_&numAE) 
                          diagnostics = none statistics(alpha = (0.01 0.05 0.10)) outpost = outpost&suffix;
      %if &suffix = 1 %then %do; 
         ods output PostIntervals = PostIntervals(where = (substr(Parameter,1,2) in ("T_","D_")));                        
      %end;
      array T_[&numAE] T_1 - T_&numAE;
      array D_[&numAE] D_1 - D_&numAE;
      parms mu_G0;
      parms tau2_G0;
      parms mu_T0;
      parms tau2_T0;
      parms A_P;
      parms B_P;

      *** Hyperpriors for Gammas ***;
      prior mu_G0 ~ normal(&mu_G00, prec = %sysevalf(1/&tau2_G00));
      prior tau2_G0 ~ gamma(&A_G00, iscale = &B_G00);
      *** Hyperpriors for Thetas ***;
      prior mu_T0 ~ normal(&mu_T00, prec = %sysevalf(1/&tau2_T00));
      prior tau2_T0 ~ gamma(&A_T00, iscale = &B_T00); 
      *** Hyperpriors for Pis ***;
      prior A_P ~ expon(iscale = &L_A, lower = 1);
      prior B_P ~ expon(iscale = &L_B, lower = 1);

      random mu_Ga ~ normal(mu_G0, prec = tau2_G0) subject = group_num monitor = (mu_Ga);
      random tau_Ga ~ gamma(&A_G, iscale = &B_G) subject = group_num monitor = (tau_Ga);
      random G ~ normal(mu_Ga, prec = tau_Ga) subject = event_num monitor = (G);

      random mu_Ta ~ normal(mu_T0, prec = tau2_T0) subject = group_num monitor = (mu_Ta);
      random tau_Ta ~ gamma(&A_T, iscale = &B_T) subject = group_num monitor = (tau_Ta);    
      random T1 ~ normal(mu_Ta, prec = tau_Ta) subject = event_num monitor = (T1);

      random P ~ beta(A_P, B_P) subject = group_num monitor = (P);
      random Bi ~ binary(P) subject = event_num monitor =(Bi);

      T_[event_num] = (1 - Bi) * T1;

      lp = G + T_[event_num] * trt;
      %if %upcase(&BModelType) = LOGISTIC %then %do;
         ptc = logistic(lp);
         D_[event_num] = exp(G + T_[event_num]) / (1 + exp(G + T_[event_num])) - exp(G) / (1 + exp(G));        
         model count ~ binomial(n = total, p = ptc);
      %end;
      %else %if %upcase(&BModelType) = LOG LINEAR %then %do;
         ptc = exp(lp);
         D_[event_num] = exp(G + T_[event_num]) - exp(G);         
         model count ~ poisson(mean = ptc*risktime);     
      %end;
   run;
    %if &syserr > 4 %then %do;
        %let exiterror = 1;
    %end;
%mend MCMC;

%let SeedAdd = 0;
%do i = 1 %to &NMarkov;
   %MCMC(suffix = &i, seedme = %eval(&SeedVal+&SeedAdd));
    %if &exiterror = 1 %then %do;
        %goto exit;
    %end;
   %let SeedAdd = %eval(&SeedAdd+1000);
%end;

%let includeds =;
%if %length(&AEBayesianHModel_OutData)>18 %then %let AEBayesianHModel_OutData = %substr(&AEBayesianHModel_OutData,1,18);
%if %length(&AEBayesianHModel_OutData) %then %let Outname = &AEBayesianHModel_OutData._;
%else %let Outname = &AEBayesianHModel_OutData;
%let AEBayesianHModel_OutData =;
%let AEBayesianHModel_OutData_post = ;    
%PrepOutDS(AEBayesianHModel_OutData_post,OutDataName_post,&OutName.BHM,_post);
%let includeds = &includeds &OutDataName_post..sas7bdat,;

data outpost;
   set 
   %do i = 1 %to &NMarkov;
      outpost&i(in = in&i)
   %end;
   ;
   if in1 then chain = 1;
   %do i = 2 %to &NMarkov;
      else if in&i then chain = &i;
   %end;
   if in1 then seed = &SeedVal;
   %let SeedAdd = 1000;
   %do i = 2 %to &NMarkov;
      else if in&i then seed = %eval(&SeedVal+(&i-1)*&SeedAdd);
   %end; 
run;

data &AEBayesianHModel_OutData_post;
   set outpost;   
   label
   %do i = 1 %to &numSoc;
      %let temp = %bquote(&&_Group&i);
      mu_Ga_&i = "mu_Ga_&i (&temp)"
      Tau_Ga_&i = "Tau_Ga_&i (&temp)"
      mu_Ta_&i = "mu_Ta_&i (&temp)"
      Tau_Ta_&i = "Tau_Ta_&i (&temp)"
      P_&i = "P_&i (&temp)"
      Bi_&i = "Bi_&i (&temp)"
   %end; 
   %do i = 1 %to &numAE;
      %let temp = %bquote(&&_Event&i);
      T_&i = "T_&i (&temp)"
      T1_&i = "T1_&i (&temp)"
      G_&i = "G_&i (&temp)"
      D_&i = "D_&i (&temp)"
   %end;
   ;
run;

proc datasets nolist;
   delete outpost;
quit;

**** Odds Ratio or Hazard Ratio ****;

data chain1;
   set &AEBayesianHModel_OutData_post;
   where chain = 1;
   keep iteration chain T_1 - T_&numAE;
run;

proc transpose data = chain1 out = chain1(drop = _label_);
   by iteration chain;
   var T_1 - T_&numAE;
run;

data chain1;
   set chain1;
   event_num = scan(_name_,2,"_") + 0;
run;

proc sort data = chain1;
   by event_num;
run;

%let trNumberofSubjectswithEvent = %nrbquote(%LocaleGet(NumberofSubjectswithEvent_Text));
%let trTotalNumberofSubjects = %nrbquote(%LocaleGet(TotalNumberofSubjects_Text));
%let trXofExposure = %nrbquote(%LocaleGet(XofExposure_Text));
%let trTABTSofExposure = %nrbquote(%LocaleFill(%nrbquote(&trXofExposure),f1=%nrbquote(&TestArm &trBTimeScale)));
%let trRABTSofExposure = %nrbquote(%LocaleFill(%nrbquote(&trXofExposure),f1=%nrbquote(&RefArm &trBTimeScale)));

data stats(drop = trt &TrtName);
   merge stats(where = (upcase(&TrtName) = "%upcase(&TestArm)") rename = (count = t_count total = t_total risktime = t_risktime))
         stats(where = (upcase(&TrtName) = "%upcase(&RefArm)") rename = (count = r_count total = r_total risktime = r_risktime));
   by &GroupVar group_num &CatName event_num;
   label t_count = "&TestArm &trNumberofSubjectswithEvent"
         t_total = "&TestArm &trTotalNumberofSubjects"
         r_count = "&RefArm &trNumberofSubjectswithEvent"
         r_total = "&RefArm &trTotalNumberofSubjects";
   %if %upcase(&BModelType) = LOG LINEAR %then %do;
      label t_risktime = "&trTABTSofExposure" r_risktime = "&trRABTSofExposure"
   %end;
   %else %do;
      drop t_risktime r_risktime;
   %end;
run;

%let AEBayesianHModel_OutData_chain1 = ;     
%PrepOutDS(AEBayesianHModel_OutData_chain1,OutDataName_chain1,&OutName.BHM,_chain1);
%let includeds = &includeds &OutDataName_chain1..sas7bdat,;

%let trXvsX = %nrbquote(%LocaleGet(XvsX_Text));
%let trTAvsRA = %nrbquote(%LocaleFill(%nrbquote(&trXvsX), f1=%nrbquote(&TestArm), f2=%nrbquote(&RefArm)));
%let trTotalNumberofEvents = %nrbquote(%LocaleGet(TotalNumberofEvents_Text));
%let trRawX = %nrbquote(%LocaleGet(RawX_Text));
%let trRawBstat = %nrbquote(%LocaleFill(%nrbquote(&trRawX), f1=%nrbquote(&Bstat)));

data chain1;
   merge chain1(rename = (col1 = statistic)) stats;
   by event_num;
   count = t_count + r_count;
   statistic = exp(statistic);
   if statistic > 1 then postexceed = 1;
   else if statistic > . then postexceed = 0;
   %if %upcase(&BModelType) = LOGISTIC %then %do;
      rawstat = ((t_count/t_total) * (1 - r_count/r_total)) / ((r_count/r_total) * (1 - t_count/t_total));
   %end;
   %else %do;
      rawstat = (t_count/t_risktime) / (r_count/r_risktime);
   %end;
   label statistic = "&BStat: &trTAvsRA" count = "&trTotalNumberofEvents" &GroupVar = "&GroupLabel" &CatName = "&EventLabel" rawstat = "&trRawBstat";
   drop _name_ group_num;
run;

title "";
proc means noprint nway data = chain1;
   class &GroupVar &CatName event_num count t_count r_count t_total r_total rawstat / missing;
   var statistic postexceed;
   output out = postexceed(drop = _type_) mean = postmean postexceed median = postmedian postexmed;
run;

proc means min max noprint data = chain1;
   var statistic;
   output out = minmaxstat(drop = _type_) min = min max = max median = median;   
run;

data _null_;
   set minmaxstat;
   call symput("minstatval",trim(left(min)));
   call symput("maxstatval",trim(left(max)));
   call symput("medstatval",trim(left(median)));
run;

data &AEBayesianHModel_OutData_chain1;
   set chain1;
   drop postexceed;
run;

proc datasets nolist;
   delete chain1 minmaxstat;
quit;

%let AEBayesianHModel_OutData_exceed = ;     
%PrepOutDS(AEBayesianHModel_OutData_exceed,OutDataName_exceed,&OutName.BHM,_exceed);
%let includeds = &includeds &OutDataName_exceed..sas7bdat,;

%let trPosteriorMeanofX = %nrbquote(%LocaleGet(PosteriorMeanofX_Text));
%let trPosteriorMeanofBstat = %nrbquote(%LocaleFill(%nrbquote(&trPosteriorMeanofX), f1=%nrbquote(&Bstat)));
%let trExceedanceProb = %nrbquote(%LocaleGet(ExceedanceProb_Text));
%let trPosteriorMedianofX = %nrbquote(%LocaleGet(PosteriorMedianofX_Text));
%let trPosteriorMedianofBstat = %nrbquote(%LocaleFill(%nrbquote(&trPosteriorMedianofX), f1=%nrbquote(&Bstat)));
%let trNumberofPosteriorSamples = %nrbquote(%LocaleGet(NumberofPosteriorSamples_Text));

data &AEBayesianHModel_OutData_exceed;
   set postexceed(rename = (_freq_ = numsamps) drop = postexmed);
   label postmean = "&trPosteriorMeanofBstat " postexceed = "&trExceedanceProb" postmedian = "&trPosteriorMedianofBstat"
         numsamps = "&trNumberofPosteriorSamples" &GroupVar = "&GroupLabel" &CatName = "&EventLabel";
run;

%let AEBayesianHModel_OutData_exceedo = ;    
%PrepOutDS(AEBayesianHModel_OutData_exceedo,OutDataName_exceedo,&OutName.BHM,_exceedo);
%let includeds = &includeds &OutDataName_exceedo..sas7bdat,;

*** data set of only exceedance probability ***;
data &AEBayesianHModel_OutData_exceedo(keep = &GroupVar &CatName postexceed);
   set postexceed(rename = (_freq_ = numsamps) drop = postexmed);
   label postmean = "&trPosteriorMeanofBstat" postexceed = "&trExceedanceProb" postmedian = "&trPosteriorMedianofBstat"
         numsamps = "&trNumberofPosteriorSamples" &GroupVar = "&GroupLabel" &CatName = "&EventLabel";
run;

%let ExceedLabel = postexceed;
%CheckLabel(&AEBayesianHModel_OutData_exceed,ExceedLabel);
%let PMeanLabel = postmean;
%CheckLabel(&AEBayesianHModel_OutData_exceed,PMeanLabel);
%let PMedianLabel = postmedian;
%CheckLabel(&AEBayesianHModel_OutData_exceed,PMedianLabel);
%let RawLabel = rawstat;
%CheckLabel(&AEBayesianHModel_OutData_exceed,RawLabel);

proc datasets nolist;
   delete postexceed;
quit;

data PostIntervals1;
   set PostIntervals(where = (substr(Parameter,1,2) = "T_"));
   event_num = scan(Parameter,2,"_") + 0;
   CredibleLower = exp(CredibleLower);
   CredibleUpper = exp(CredibleUpper);
   HPDLower = exp(HPDLower);
   HPDUpper = exp(HPDUpper);
run;

data pints(drop = alpha parameter);
   merge PostIntervals1(where = (alpha = 0.100) rename = (CredibleLower = ETLower90 CredibleUpper = ETUpper90 HPDLower = HPDLower90 HPDUpper = HPDUpper90))
         PostIntervals1(where = (alpha = 0.050) rename = (CredibleLower = ETLower95 CredibleUpper = ETUpper95 HPDLower = HPDLower95 HPDUpper = HPDUpper95))
         PostIntervals1(where = (alpha = 0.010) rename = (CredibleLower = ETLower99 CredibleUpper = ETUpper99 HPDLower = HPDLower99 HPDUpper = HPDUpper99))       
   ;
   by event_num;  
run;

data &AEBayesianHModel_OutData_exceed(drop = event_num);
   merge &AEBayesianHModel_OutData_exceed pints;
   by event_num;
run;

proc datasets nolist;
   delete PostIntervals1;
quit;

**** Difference in Proportions or Hazard Rates ****;

data chain1d;
   set &AEBayesianHModel_OutData_post;
   where chain = 1;
   keep iteration chain D_1 - D_&numAE;
run;

proc transpose data = chain1d out = chain1d(drop = _label_);
   by iteration chain;
   var D_1 - D_&numAE;
run;

data chain1d;
   set chain1d;
   event_num = scan(_name_,2,"_") + 0;
run;

proc sort data = chain1d;
   by event_num;
run;

%let AEBayesianHModel_OutData_chain1d = ;    
%PrepOutDS(AEBayesianHModel_OutData_chain1d,OutDataName_chain1d,&OutName.BHM,_chain1d);
%let includeds = &includeds &OutDataName_chain1d..sas7bdat,;

%let trTotalNumberofEvents = %nrbquote(%LocaleGet(TotalNumberofEvents_Text));
%let trRawX = %nrbquote(%LocaleGet(RawX_Text));
%let trRawBstat2 = %nrbquote(%LocaleFill(%nrbquote(&trRawX), f1=%nrbquote(&Bstat2)));

data chain1d;
   merge chain1d(rename = (col1 = statistic)) stats;
   by event_num;
   count = t_count + r_count;
   if statistic > 0 then postexceed = 1;
   else if statistic > . then postexceed = 0;
   %if %upcase(&BModelType) = LOGISTIC %then %do;
      rawstat = (t_count/t_total) - (r_count/r_total);
   %end;
   %else %do;
      rawstat = (t_count/t_risktime) - (r_count/r_risktime);
   %end;
   label statistic = "&BStat2: &TestArm - &RefArm" count = "&trTotalNumberofEvents" &GroupVar = "&GroupLabel" &CatName = "&EventLabel" rawstat = "&trRawBstat2";
   drop _name_ group_num;
run;

title "";
proc means noprint nway data = chain1d;
   class &GroupVar &CatName event_num count t_count r_count t_total r_total rawstat / missing;
   var statistic postexceed;
   output out = postexceedd(drop = _type_) mean = postmean postexceed median = postmedian postexmed;
run;

proc means min max noprint data = chain1d;
   var statistic;
   output out = minmaxstatd(drop = _type_) min = min max = max median = median;  
run;

data _null_;
   set minmaxstatd;
   call symput("minstatvald",trim(left(min)));
   call symput("maxstatvald",trim(left(max)));
   call symput("medstatvald",trim(left(median)));
run;

data &AEBayesianHModel_OutData_chain1d;
   set chain1d;
   drop postexceed;
run;

proc datasets nolist;
   delete chain1d minmaxstatd;
quit;

%let AEBayesianHModel_OutData_exceedd = ;    
%PrepOutDS(AEBayesianHModel_OutData_exceedd,OutDataName_exceedd,&OutName.BHM,_exceedd);
%let includeds = &includeds &OutDataName_exceedd..sas7bdat,;

%let trPosteriorMeanofBstat2 = %nrbquote(%LocaleFill(%nrbquote(&trPosteriorMeanofX), f1=%nrbquote(&Bstat2)));
%let trPosteriorMedianofBstat2 = %nrbquote(%LocaleFill(%nrbquote(&trPosteriorMedianofX), f1=%nrbquote(&Bstat2)));

data &AEBayesianHModel_OutData_exceedd;
   set postexceedd(rename = (_freq_ = numsamps) drop = postexmed);
   label postmean = "&trPosteriorMeanofBstat2" postexceed = "&trExceedanceProb" postmedian = "&trPosteriorMedianofBstat2"
         numsamps = "&trNumberofPosteriorSamples" &GroupVar = "&GroupLabel" &CatName = "&EventLabel";
run;

%let AEBayesianHModel_OutData_exceedp = ;    
%PrepOutDS(AEBayesianHModel_OutData_exceedp,OutDataName_exceedp,&OutName.BHM,_exceedp);
%let includeds = &includeds &OutDataName_exceedp..sas7bdat,;

*** data set of only exceedance probability ***;
data &AEBayesianHModel_OutData_exceedp(keep = &GroupVar &CatName postexceed);
   set postexceedd(rename = (_freq_ = numsamps) drop = postexmed);
   label postmean = "&trPosteriorMeanofBstat2" postexceed = "&trExceedanceProb" postmedian = "&trPosteriorMedianofBstat2"
         numsamps = "&trNumberofPosteriorSamples" &GroupVar = "&GroupLabel" &CatName = "&EventLabel";
run;

%let ExceedLabel = postexceed;
%CheckLabel(&AEBayesianHModel_OutData_exceed,ExceedLabel);
%let PMeanLabel = postmean;
%CheckLabel(&AEBayesianHModel_OutData_exceed,PMeanLabel);
%let PMedianLabel = postmedian;
%CheckLabel(&AEBayesianHModel_OutData_exceed,PMedianLabel);
%let PMeanLabel2 = postmean;
%CheckLabel(&AEBayesianHModel_OutData_exceedd,PMeanLabel2);
%let PMedianLabel2 = postmedian;
%CheckLabel(&AEBayesianHModel_OutData_exceedd,PMedianLabel2);
%let RawLabel2 = rawstat;
%CheckLabel(&AEBayesianHModel_OutData_exceedd,RawLabel2);

proc datasets nolist;
   delete stats postexceedd;
quit;

data PostIntervalsd;
   set PostIntervals(where = (substr(Parameter,1,2) = "D_"));
   event_num = scan(Parameter,2,"_") + 0;
run;

data pintsd(drop = alpha parameter);
   merge PostIntervalsd(where = (alpha = 0.100) rename = (CredibleLower = ETLower90 CredibleUpper = ETUpper90 HPDLower = HPDLower90 HPDUpper = HPDUpper90))
         PostIntervalsd(where = (alpha = 0.050) rename = (CredibleLower = ETLower95 CredibleUpper = ETUpper95 HPDLower = HPDLower95 HPDUpper = HPDUpper95))
         PostIntervalsd(where = (alpha = 0.010) rename = (CredibleLower = ETLower99 CredibleUpper = ETUpper99 HPDLower = HPDLower99 HPDUpper = HPDUpper99))       
   ;
   by event_num;  
run;

data &AEBayesianHModel_OutData_exceedd(drop = event_num);
   merge &AEBayesianHModel_OutData_exceedd pintsd;
   by event_num;
run;

data &AEBayesianHModel_OutData_post;
   set &AEBayesianHModel_OutData_post(drop = D_1-D_&numAE);
run;

********************************;

proc datasets nolist;
   delete PostIntervals PostIntervalsd;
quit;

%put "SYSERR = &syserr";

%if &syserr > 4 %then %do;
    %let exiterror = 1;
    %goto exit;
%end;


* create JSL file;
%PrepOutFile(AEBayesianHModelScript,AEBayesianHModelScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &AEBayesianHModelScript;
%let FullAPName=%str(Adverse Events Bayesian Hierarchical Model);

* put JSL statements;
data _null_;
   file "&AEBayesianHModelScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:NoteDomain = ""AE"";";      
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:PopulationSet=""&PopulationSet"");";
   put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&AEWhereStmt)]\"");";
   put "try(objRefNS:includestring=""Adverse Events: "");";
   
   %let DomainWhereStmt=&AEWhereStmt;
   %if %length(&WhereForReports) %then %do;
      put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
      put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
      %if %length(&DomainWhereStmt) %then %do;
      put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditionalFilter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;
   %end;
   %else %if %length(&DomainWhereStmt) %then %do;
      put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
   %end; 
   
   put "objRefNS:TabsSectionFlag = 1;";
   put "objRefNS:Stat = ""&BStat"";";
   put "objRefNS:Stat2 = ""&BStat2"";";
   put "objRefNS:MinVal = &minstatval;";
   put "objRefNS:MaxVal = &maxstatval;";
   put "objRefNS:MedVal = &medstatval;";
   put "objRefNS:MinVal2 = &minstatvald;";
   put "objRefNS:MaxVal2 = &maxstatvald;";
   put "objRefNS:MedVal2 = &medstatvald;";
   put "objRefNS:EventVar = ""&CatName"";";
   put "objRefNS:GroupVar = ""&GroupVar"";"; 
   put "objRefNS:Event = ""&EventLabel"";";
   put "objRefNS:Group = ""&GroupLabel"";";
   put "objRefNS:TestArm = ""&TestArm"";";
   put "objRefNS:RefArm = ""&RefArm"";";
   put "objRefNS:PMean = ""&PMeanLabel"";";
   put "objRefNS:PMedian = ""&PMedianLabel"";";
   put "objRefNS:PMean2 = ""&PMeanLabel2"";";
   put "objRefNS:PMedian2 = ""&PMedianLabel2"";";
   put "objRefNS:Raw = ""&RawLabel"";";
   put "objRefNS:Raw2 = ""&RawLabel2"";";
   put "objRefNS:post = ""&OutDataName_post..sas7bdat"";";
   put "objRefNS:exceed = ""&OutDataName_exceed..sas7bdat"";";
   put "objRefNS:exceedo = ""&OutDataName_exceedo..sas7bdat"";";
   put "objRefNS:chain1 = ""&OutDataName_chain1..sas7bdat"";";
   put "objRefNS:chain2 = ""&AEBayesianHModel_OutData_chain1"";";
   put "objRefNS:exceed2 = ""&AEBayesianHModel_OutData_exceed"";";
   put "objRefNS:exceedo2 = ""&AEBayesianHModel_OutData_exceedo"";";
   put "objRefNS:exceedd = ""&OutDataName_exceedd..sas7bdat"";";
   put "objRefNS:exceedp = ""&OutDataName_exceedp..sas7bdat"";";
   put "objRefNS:chain1d = ""&OutDataName_chain1d..sas7bdat"";";
   put "objRefNS:chain1d2 = ""&AEBayesianHModel_OutData_chain1d"";";
   put "objRefNS:exceedd2 = ""&AEBayesianHModel_OutData_exceedd"";";
   put "objRefNS:exceedp2 = ""&AEBayesianHModel_OutData_exceedp"";"; 
   put "objRefNS:outlib = ""&OutPath"";";
   put "objRefNS:update_exceed_TextEditBox = Expr(";
   put "    value = Num(objRefNS:exceedsize);";
   put "    objRefNS:Exceed_T<<SetText(char(num(value)));";
   put "    objRefNS:Exceed_T<<Reshow;";
   put "    objRefNS:exceedsize = value;";
   put ");";
   put "objRefNS:update_exceed_SliderBox = Expr(";
   put "    objRefNS:exceedsize = num(objRefNS:Exceed_T<<GetText);";
   put "    objRefNS:Exceed_S << Reshow;";
   put "    objRefNS:update_exceed_TextEditBox;";
   put ");  ";
   put "objRefNS:exceedsize = objRefNS:MedVal;";   
   put "objRefNS:update_exceed_TextEditBox2 = Expr(";
   put "    value = Num(objRefNS:exceedsize2);";
   put "    objRefNS:Exceed_T2<<SetText(char(num(value)));";
   put "    objRefNS:Exceed_T2<<Reshow;";
   put "    objRefNS:exceedsize2 = value;";
   put ");";
   put "objRefNS:update_exceed_SliderBox2 = Expr(";
   put "    objRefNS:exceedsize2 = num(objRefNS:Exceed_T2<<GetText);";
   put "    objRefNS:Exceed_S2 << Reshow;";
   put "    objRefNS:update_exceed_TextEditBox2;";
   put ");  ";
   put "objRefNS:exceedsize2 = objRefNS:MedVal2;"; 
   put "Include(""$CLINICAL_HOME\JSLFiles\BayesianHModelFunctions.jsl"");";
   put "include(""$LS_HOME\JSLFILES\DrillDownReport.JSL"");";

   put "objRefNS:Hier=Function({},{default local},";
   put "   objRefNS:VolcanoData1=open(""&ClientOutPath""||objRefNS:exceed,invisible);";
   put "   HlistBox(";
   put "       objRefNS:Filter = objRefNS:VolcanoData1 << Data Filter(<<Conditional(1),";
   put "            Add Filter(";
   put "               Columns(:Name(""&GroupLabel""), :Name(""&EventLabel""), :Name(""&ExceedLabel""))"; 
   put "            ),";
   put "            Mode( Show(1), Include(1) )";  
   put "       ), objRefNS:BubbleTab = VListBox(";
   %if %length(&TEAEText) %then %do;
      put "objRefNS:TEAEText = TextBox(""&TEAEText"", <<Set Wrap(1000)), Text Box(""""),";
   %end;
   %if %upcase(&BModelType) = LOG LINEAR %then %do;
      put "       TextBox(locale:fill(locale:get(""ExposureismeasuredinX_Text""), {""&trBTimeScale""}),<< Font Color(5)),";
      put "       TextBox(""""),";
   %end;
   put"           objRefNS:BayesVolcano(""&PMeanLabel"",""&ExceedLabel"","">"",1, 1),";
   put "             TextBox(""  ""),";
   put "             objRefNS:_operator = Radio Box( {""<"",""<="",""="","">="","">""},<<Set(5)),";
   put "             TextBox(""  ""),";
   put "             HListBox(";
   put "                objRefNS:Exceed_T = TextEditBox(char(num(objRefNS:exceedsize)),";
   put "                   <<Script(objRefNS:update_exceed_SliderBox)";
   put "                ),";
   put "                objRefNS:Exceed_S = SliderBox(objRefNS:MinVal,objRefNS:MaxVal,objRefNS:exceedsize,";
   put "                   objRefNS:update_exceed_TextEditBox";
   put "                ),";
   put "                objRefNS:Exceed_S << SetValue(num(objRefNS:exceedsize));";
   put "                objRefNS:Exceed_S << SetWidth(100);";
   put "             ),";
   put "             TextBox(""  ""),";
   put "             Button Box( locale:get(""UpdateExceedanceProbability_Text""),";
   put "                objRefNS:ExceedValue = num(objRefNS:Exceed_T<<Get Text);";
   put "                operator = objRefNS:_operator << Get;";
   put "                If(operator == 1, operatortxt = ""<"",";
   put "                   operator == 2, operatortxt = ""<="",";
   put "                   operator == 3, operatortxt = ""="",";
   put "                   operator == 4, operatortxt = "">="",";
   put "                   operator == 5, operatortxt = "">"");";
   put "                objRefNS:outpath = ""&ClientOutPath"";";
   put "                objRefNS:UpdateData(operatortxt,objRefNS:ExceedValue, 1);";
   put "                objRefNS:ExTemp = open(""&ClientOutPath""||objRefNS:exceedo,invisible);";
   put "                objRefNS:VolcanoData1 << Update(";
   put "                      With( objRefNS:ExTemp ),";
   put "                      Match Columns( :Name(""&GroupLabel"") = :Name(""&GroupLabel""), :Name(""&EventLabel"") = :Name(""&EventLabel"")),";
   put "                      Add Columns from Update table( :Name(""&ExceedLabel"") )";
   put "                );";
   put "                Close(objRefNS:ExTemp,nosave);";
   put "                Try(objRefNS:Volcano1 << Delete;);";   
   put "                objRefNS:BubbleTab << Prepend(objRefNS:BayesVolcano(""&PMeanLabel"",""&ExceedLabel"",operatortxt, objRefNS:ExceedValue, 1););";
   put "             ),";
   put "          ),";
   put "   );";
   put ");";
   put "InsertInto(objRefNS:DataList,{{""objRefNS:VolcanoData1""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&Bstat""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:fill(locale:get(""VolcanoPlotforX_Text""), {""&Bstat""})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:Hier()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   
   put "objRefNS:Hier2=Function({},{default local},";
   put "   objRefNS:VolcanoData2=open(""&ClientOutPath""||objRefNS:exceedd,invisible);";
   put "   HlistBox(";
   put "       objRefNS:Filter = objRefNS:VolcanoData2 << Data Filter(<<Conditional(1),";
   put "            Add Filter(";
   put "               Columns(:Name(""&GroupLabel""), :Name(""&EventLabel""), :Name(""&ExceedLabel""))"; 
   put "            ),";
   put "            Mode( Show(1), Include(1) )";  
   put "       ),objRefNS:BubbleTab2 = VListBox(";
   %if %length(&TEAEText) %then %do;
      put "objRefNS:TEAEText = TextBox(""&TEAEText"", <<Set Wrap(1000)), Text Box(""""),";
   %end;
   %if %upcase(&BModelType) = LOG LINEAR %then %do;
      put "       TextBox(locale:fill(locale:get(""ExposureismeasuredinX_Text""), {""&trBTimeScale""}),<< Font Color(5)),";
      put "       TextBox(""""),";
   %end;
   put"           objRefNS:BayesVolcano(""&PMeanLabel2"",""&ExceedLabel"","">"",0, 2),";
   put "             TextBox(""  ""),";
   put "             objRefNS:_operator2 = Radio Box( {""<"",""<="",""="","">="","">""},<<Set(5)),";
   put "             TextBox(""  ""),";
   put "             HListBox(";
   put "                objRefNS:Exceed_T2 = TextEditBox(char(num(objRefNS:exceedsize2)),";
   put "                   <<Script(objRefNS:update_exceed_SliderBox2)";
   put "                ),";
   put "                objRefNS:Exceed_S2 = SliderBox(objRefNS:MinVal2,objRefNS:MaxVal2,objRefNS:exceedsize2,";
   put "                   objRefNS:update_exceed_TextEditBox2";
   put "                ),";
   put "                objRefNS:Exceed_S2 << SetValue(num(objRefNS:exceedsize2));";
   put "                objRefNS:Exceed_S2 << SetWidth(100);";
   put "             ),";
   put "             TextBox(""  ""),";
   put "             Button Box( locale:get(""UpdateExceedanceProbability_Text""),";
   put "                objRefNS:ExceedValue = num(objRefNS:Exceed_T2<<Get Text);";
   put "                operator = objRefNS:_operator2 << Get;";
   put "                If(operator == 1, operatortxt = ""<"",";
   put "                   operator == 2, operatortxt = ""<="",";
   put "                   operator == 3, operatortxt = ""="",";
   put "                   operator == 4, operatortxt = "">="",";
   put "                   operator == 5, operatortxt = "">"");";
   put "                objRefNS:UpdateData(operatortxt,objRefNS:ExceedValue, 2);";
   put "                objRefNS:ExTemp2 = open(""&ClientOutPath""||objRefNS:exceedp,invisible);";
   put "                objRefNS:VolcanoData2 << Update(";
   put "                      With( objRefNS:ExTemp2 ),";
   put "                      Match Columns( :Name(""&GroupLabel"") = :Name(""&GroupLabel""), :Name(""&EventLabel"") = :Name(""&EventLabel"")),";
   put "                      Add Columns from Update table( :Name(""&ExceedLabel"") )";
   put "                );";
   put "                Close(objRefNS:ExTemp2,nosave);";
   put "                Try(objRefNS:Volcano2 << Delete;);";   
   put "                objRefNS:BubbleTab2 << Prepend(objRefNS:BayesVolcano(""&PMeanLabel2"",""&ExceedLabel"",operatortxt, objRefNS:ExceedValue, 2););";
   put "             ),";
   put "          ),";
   put "   );";
   put ");";
   put "InsertInto(objRefNS:DataList,{{""objRefNS:VolcanoData2""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&Bstat2""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:fill(locale:get(""VolcanoPlotforX_Text""), {""&Bstat2""})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:Hier2()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";   


   put "objRefNS:Hyper=Function({},{default local},";
   put "   VlistBox(";
   put "       TextBox(locale:get(""AssumedconstantsforHyperparamsBasedOnDialOpts_Text"") || "":"",<< font color(5),<<Set Wrap(900)),";
   put "       TextBox("" ""),";
   put "       TextBox(locale:get(""mu_G00_Text"") || "" = &hp_mu_G00"", << font color(5)),";   
   put "       TextBox(locale:get(""mu_T00_Text"") || "" = &hp_mu_T00"", << font color(5)),";
   put "       TextBox(locale:get(""tau2_G00_Text"") || "" = &hp_tau2_G00"", << font color(5)),";
   put "       TextBox(locale:get(""tau2_T00_Text"") || "" = &hp_tau2_T00"", << font color(5)),";
   put "       TextBox(locale:get(""A_G_Text"") || "" = &hp_A_G"", << font color(5)),";
   put "       TextBox(locale:get(""A_T_Text"") || "" = &hp_A_T"", << font color(5)),";
   put "       TextBox(locale:get(""A_G00_Text"") || "" = &hp_A_G00"", << font color(5)),";
   put "       TextBox(locale:get(""A_T00_Text"") || "" = &hp_A_T00"", << font color(5)),";
   put "       TextBox(locale:get(""B_G_Text"") || "" = &hp_B_G"", << font color(5)),";
   put "       TextBox(locale:get(""B_T_Text"") || "" = &hp_B_T"", << font color(5)),";
   put "       TextBox(locale:get(""B_G00_Text"") || "" = &hp_B_G00"", << font color(5)),";
   put "       TextBox(locale:get(""B_T00_Text"") || "" = &hp_B_T00"", << font color(5)),";
   put "       TextBox(locale:get(""L_A_Text"") || "" = &hp_L_A"", << font color(5)),";
   put "       TextBox(locale:get(""L_B_Text"") || "" = &hp_L_B"", << font color(5)),";
   put "   );";
   put ");";
   put "InsertInto(objRefNS:DataList,{{""""}});";
   put "InsertInto(objRefNS:DataNameList,{{""""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""Hyperparameters_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:Hyper()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";      
   
   put "objRefNS:MCDiag=Function({},{default local},";
   put "   objRefNS:MCMCData=open(""&ClientOutPath""||objRefNS:post,invisible);";
   put "   Column(objRefNS:MCMCData, ""iteration"") << Set Property( ""Time Frequency"", Numeric );";
   put "   objRefNS:MCMCTab = VListBox(objRefNS:MCMCFunc()";
   put "   );";
   put ");";
   put "InsertInto(objRefNS:DataList,{{""objRefNS:MCMCData""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trMCMCDiagnostics""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""MCMCDiagnostics_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:MCDiag()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,0);";
      
   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionIconList ={{}};";
   put "objRefNS:ActionSelectionCountList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";
   put "objRefNS:ActionButtonsperRow = {1};";
     
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectEventsthenClick_Text""))});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ForestPlotsofCredibleIntervals_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:BForestPlot(""95"")));";      
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:VolcanoData1"",""objRefNS:VolcanoData2""}});";   
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.ForestPlot.gif"");";
   put "InsertInto(objRefNS:ActionSelectionCountList[1],1);";

   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowEvents_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:Showevents()));";      
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:VolcanoData1"",""objRefNS:VolcanoData2""}});";   
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.DataSetHighlightRows.gif"");";
   put "InsertInto(objRefNS:ActionSelectionCountList[1],1);";
   put "objRefNS:ActionRequirementFlagList = {{0, 0}};";

   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:TEAEText,objRefNS:Volcano1},{objRefNS:TEAEText,objRefNS:Volcano2}});";   
run;

%let includeds = %substr(%quote(&includeds), 1, %eval(%length(&includeds) - 1));
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &AEBayesianHModelScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AEBayesianHModel;
%AEBayesianHModel;

************************************;
************************************;
************************************;