/*
JMP Clinical Analytical Process

Name: Adverse Events Resolution Screen
Authors: Geoff Mann and Russ Wolfinger, SAS Institute Inc.

Description: 
   This process screens all adverse events or MedDRA terms by performing CMH tests on
   all 2 x 2 tables constructed from event Resolution and treatment arm.  Output is a table
   of multiplicity-adjusted p-values and an accompanying volcano plot of relative risk.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"EventsO","CrossoverPeriods","AETermValues","TreatmentEmergentEvents"}}$
$INCD: {"AE"}$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CMP: CPV.{"Determine Resolved / Not Resolved Status using:","Values coded in outcome of adverse event"}$ $UE: AE.AEOUT$ 
$CMP: CPV.{"Determine Resolved / Not Resolved Status using:","Adverse event duration compared to a reference value"}$ $UE: AE.AEDUR or 
   (AE.AEENDY and AE.AESTDY) or (AE.AEENDY and AE.AESTDTC and  DM.RFSTDTC) or (AE.AESTDY and AE.AEENDTC and  DM.RFSTDTC) or (AE.AESTDTC and AE.AEENDTC and DM.RFSTDTC)$ 
   $NOTES:  AEDUR will be calculated from other timing variables for relative study day if not found.$
$CMP: CPV.{"Determine Resolved / Not Resolved Status using:","Adverse event end day compared to a reference day"}$ $UE:(AE.AEENDTC or AE.AEENDY) and DM.RFENDTC and DM.RFSTDTC$ 
$CVS_STOP$
*/


*options mprint mlogic symbolgen;
*ProcessVariables;

%global StudyName InPath AInPath TrtUse ResolveUsing ReferenceDay IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL AETermLevel AEGroupLevel
TrtVar DesignByVars SubjectFilter WhereStmt WhereForReports DSWhereStmt AEWhereStmt AESerOnly TEAEOnly TimeWindows TimeScale StrataVars
OutPath ServerOutPath 
VolcanoXCat DoubleFDR MultipleTestingMethod Alpha NegLog10pCutoff ResolutionTimeWindows  ScreenControlValue
ColorTheme AEResolutionScreen_OutData AEResolutionScreen_GOutData MacroPath AEResolutionScreenScript ResolutionScreenScript TrtOffSet MergeSuppDom IgnoreTEAEFlags;

*ProcessBody;

%macro AEResolutionScreen;

*initialization and argument checking;
%let ProcessName = AEResolutionScreen; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
%LOADDATA(domlist=AE,mergesupp=&MergeSuppDom,checkreview=1);

%let WhereForReports = %nrbquote(&WhereStmt);

%if %length(&DesignByVars) and &DoubleFDR = yes %then %do;
   %put [MS_S_0027];
   %put ERROR: By Variables are currently not available when performing Double FDR.;
   %let exiterror = 1;
   %goto exit;
%end;

%AESerOnlyToAEWhereStmt;

%include "&MacroPath./TEAEOTFU.sas";
%if &exiterror %then %goto exit;       

%if %length(&AEWhereStmt) %then %do;
   data ae_ws;
      set &in_ae;
      where %unquote(&AEWhereStmt);
   run;
   %if &syserr > 4 %then %do;
      %put [MS_S_0019];
      %put ERROR: The specified filter &AEWhereStmt is invalid.  Change the filter to be valid for the domain you are considering.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %NObsVars(ae_ws);
   %if ^&nobs %then %do;
      %put [MS_S_0020];
      %put ERROR: No observations satisfy &AEWhereStmt..;
      %put Please relax the filter.;
      %let exiterror = 1;
      %goto exit;
   %end; 
   %let in_ae = ae_ws;
%end; 

*** Get Variable Based on Drop Down Label ***;
%GETMEDDRAVAR(label = &AETermLevel, column = CatName);
%GETMEDDRAVAR(label = &AEGroupLevel, column = GroupVar);

*** Check if Catname null ***;
%if ^%sysfunc(findw(&AEVARS, &CatName)) %then %do;
   %put [MS_S_0017];
   %put ERROR: Selected AE term variable has no non-missing values.  Please select an alternate variable.;
   %let exiterror = 1;
   %goto exit;
%end;

*** Check if Group null ***;
%if ^%sysfunc(findw(&AEVARS, &GroupVar)) %then %do;
   %put NOTE: Selected AE grouping variable has no non-missing values.  Grouping variable is set to term variable.;
   %let GroupVar = &CatName;
%end;

*** Ensure that the Grouping Variable is at least the level of Term Variable ***;
%COMPAREMEDDRA(term = &CatName, group = &GroupVar);
%if &exiterror %then %goto exit;

%let domain = ae;
%let ddat = &in_ae;
* add numeric suffix;
%if ^%symexist(_nThePro_) %then %let _nThePro_ = 001;
%let suffix = aer_&_nThePro_;
%let CIPrefix = AER;

%let _tid_ = %sysfunc(open(&ddat));
%let vn = %sysfunc(varnum(&_tid_,&CatName));
%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
%if ^%length(%trim(&CatLabel)) %then %let CatLabel = &CatName;
%put CatLabel = &CatLabel;
%let _tid_ = %sysfunc(close(&_tid_));

%let ResolutionScreenScript=&AEResolutionScreenScript;
%include "&MacroPath./TreatmentLevels.sas" / nosource;
%if &exiterror %then %goto exit;
      
%if %index(%upcase(&ResolveUsing),DURATION) and ^%length(&ReferenceDay) %then %let ReferenceDay = 1;
%else %if %index(%upcase(&ResolveUsing),END DAY) and ^%length(&ReferenceDay) %then %let ReferenceDay = RFENDY;
%else %if %index(%upcase(&ResolveUsing),VALUES) %then %do;
   %let vn = %sysfunc(findw(&AEVARS, AEOUT));
   %if ^&vn %then %do;
      %put [MS_S_0045];
      %put ERROR: Variable AEOUT not found in &domain data set.;
      %let exiterror = 1;
   %end;
%end;
%if &exiterror %then %goto exit;

%let trRecovered = %nrbquote(%LocaleGet(Recovered_Text));
%let trResolved = %nrbquote(%LocaleGet(Resolved_Text));
%let trNot = %nrbquote(%LocaleGet(Not_Text));

* compute STDY, ENDY, RFENDY and resolution indicator by comparing the latter two;
data dm;
   set &in_dm;
   keep USUBJID rfstdtc rfendtc;
run;

proc sort data=&ddat out=&domain;
   by USUBJID;
run;

proc sort data=dm;
   by USUBJID;
run;

data &domain;
   merge &domain dm;
   by USUBJID;
   if missing(&CatName) then delete;
   %calculateStudyDays(ae);                           
   %calcStudyDay(rfendtc,RFENDY);
   %if %index(%upcase(&ResolveUsing),DURATION) %then %do;
      if AEDUR > &ReferenceDay then NOT_RESOLVED = 1;
      else NOT_RESOLVED = 0;
   %end;
   %else %if %index(%upcase(&ResolveUsing),END DAY) %then %do;
      if ENDY > RFENDY then NOT_RESOLVED = 1;
      else NOT_RESOLVED = 0;  
   %end;
   %else %do;
      if ((kindex(kupcase(AEOUT),kupcase("&trResolved")) or kindex(kupcase(AEOUT),kupcase("&trRecovered"))) and 
         not kindex(kupcase(AEOUT),kupcase("&trNot"))) or ((index(upcase(AEOUT),"RESOLVED") or index(upcase(AEOUT),"RECOVERED")) and 
         not index(upcase(AEOUT),"NOT")) then NOT_RESOLVED = 0;
      else NOT_RESOLVED = 1;
   %end;
run; 

* incorporate time windows if specified;
%AddTimeWindows(domain=ae,indat=&domain,outdat=&domain,twb=&TimeWindows,tvar=&TimeScale);
%if &exiterror %then %goto exit;

* handle crossover design by adding treatment as an id in the incidence matrix and USUBJID as a strata variable in the CMH test;
%if &NTrtPeriods > 1 %then %do;
   proc sort data=&domain;
      by USUBJID;
   run;
   %AssignCOTreatment(&domain, &domain, &domain.STDTC);
   %let cidvar = USUBJID Period Trt;
   %let StrataVars = USUBJID &StrataVars;
   proc sort data=&domain;
      by &cidvar;
   run;
   proc sort data=OutLib.adsl_&suffix;
      by &cidvar;
   run;
%end;
%else %do;
   %let cidvar = USUBJID;
%end;

* join with adsl; 
data OutLib.&suffix;
   merge &domain OutLib.adsl_&suffix;
   by &cidvar;
   if missing(&TrtName) then delete;
run;

* create counts to be used in tree map;
%createCounts(&domain,KeepVars=&CatKeepVars);

* set up and call column enrichment process, which does the testing and p-value adjustment;
%global InData AnnoVar ASigVars BSigVars CSigVars Delimiter Delim2 IncidenceScreen_BOutData IncidenceScreen_GOutData
WhereStmt NBins RankEm MultipleTestingMethods NSample PThresh NHits Seed IncidenceScreen_DesOutData;

%let InData = &OutPath.&suffix..sas7bdat;
%let AnnoVar = &CatName;
%let ASigVars = ;
%let CSigVars = ;
%let BSigVars = ;
%do i=1 %to %eval(&nlev-1);
   %let BSigVars = &BSigVars &TrtName&i;
%end;
%let Delimiter=%nrbquote(" / ");
%let Delim2=%nrbquote();
%let MultipleTestingMethods= &MultipleTestingMethod;
%let NSample= 10000;
%let Seed=%nrbquote(12345);
%let PThresh= 1;
%let NHits= 3;
%let NBins= 7;
%let RankEm= No;
%let WhereStmt=%nrbquote();
%let IncidenceScreen_DesOutData=&suffix;
%let IncidenceScreen_BOutData=&AEResolutionScreen_OutData;
%let IncidenceScreen_GOutData=&AEResolutionScreen_GOutData;
%let ScreenType=Resolution;
%let CESuffix=s;
%let CGSuffix=g;
%let TreeData=&domain._counts;
%let TreeSize=Count;
%let AlreadyStacked = 1;
%let ResponseVar = NOT_RESOLVED;

%put CLINICAL_HOME = &CLINICAL_HOME;

%include "&CLINICAL_HOME.\ProcessLibrary\IncidenceScreen.sas"; 

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AEResolutionScreen;

* run the macro;
%AEResolutionScreen;