/*
JMP Clinical Analytical Process

Name: Adverse Events Time To Event
Authors: Geoff Mann and Russ Wolfinger, SAS Institute Inc.

Description: 
   This process screens all adverse events by performing log-rank and Wilcoxon tests
   between treatment groups.  The time to first occurence of the adverse event is used as the
   response,
*/

/*
-----------------------------------------------------------
Copyright (C) 2009 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"Events","TreatmentEmergentEvents"}}$
$INCD: {"AE"}$
$CMP: RSYS$ $UE: AE.AEDECOD$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName InPath AInPath TrtUse DesignByVars ResponseType SubjectFilter PropThresh
WhereStmt WhereForReports AEWhereStmt AESerOnly TEAEOnly ScreenControlValue TimeWindows TimeScale OutPath ServerOutPath 
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL 
Distribution ClassVars FixedEffects RandomEffects GConv
ColorTheme AETimeToEvent_OutData MacroPath AETimeToEventScript TrtVar TrtOffSet MergeSuppDom AETermLevel IgnoreTEAEFlags;

*ProcessBody;

%macro AETimeToEvent;

*initialization and argument checking;
%let ProcessName = AETimeToEvent; 

%let WhereForReports=&WhereStmt;

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
%LOADDATA(domlist=AE,mergesupp=&MergeSuppDom,checkreview=1);

%let trAdverseEventsTimetoEvent = %nrbquote(%LocaleGet(AdverseEventsTimetoEvent_Text));

%let WhereForReports = %nrbquote(&WhereStmt);

* add numeric suffix;
%if ^%symexist(_nThePro_) %then %let _nThePro_ = 001;
%let suffix = tte_&_nThePro_;

* determine number of treatment levels;
%let NeedTwoLevels = 0;
%include "&MacroPath./TreatmentLevels.sas" / nosource;
%let NeedTwoLevels = 1;
%if &exiterror %then %goto exit;

* clean up the outlib.adsl_tte dataset (S1104632);
%if %sysfunc(exist(outlib.adsl_tte)) %then %do;
   %RemoveData(adsl_tte,_Lib=outlib);
%end;

%if &nlev=1 and ^%index(&ResponseType,Time) %then %do;
   %put [MS_S_0048];
   %put ERROR: The Response Type must be Time to first event when there is only one treatment arm.;
   %let exiterror = 1;
   %goto exit;
%end;

%AESerOnlyToAEWhereStmt;

%include "&MacroPath./TEAEOTFU.sas";
%if &exiterror %then %goto exit;    
   
%if %length(&AEWhereStmt) %then %do;
   data ae_ws;
      set &in_ae;
      where %unquote(&AEWhereStmt);
   run;
   %if &syserr > 4 %then %do;
      %put [MS_S_0019];
      %put ERROR: The specified filter &AEWhereStmt is invalid.  Change the filter to be valid for the domain you are considering.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %NObsVars(ae_ws);
   %if ^&nobs %then %do;
      %put [MS_S_0049];
      %put ERROR: No adverse events satisfy where statements or other filtering.;
      %let exiterror = 1;
      %goto exit;
   %end;    
   %let in_ae = ae_ws;
%end; 

*** Get Variable Based on Drop Down Label ***;
%if ^%symexist(AETermLevel) %then %let AETermLevel = Dictionary-Derived Term;
%if ^%length(&AETermLevel) %then %let AETermLevel = Dictionary-Derived Term;
%put AETermLevel = &AETermLevel;
%GETMEDDRAVAR(label = &AETermLevel, column = CatName);

*** Check if Catname null ***;
%if ^%sysfunc(findw(&AEVARS, &CatName)) %then %do;
   %put [MS_S_0017];
   %put ERROR: Selected AE term variable has no non-missing values.  Please select an alternate variable.;
   %let exiterror = 1;
   %goto exit;
%end;

%let _tid_ = %sysfunc(open(&in_ae));
%let vn = %sysfunc(varnum(&_tid_,&CatName));
%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
%if ^%length(%trim(&CatLabel)) %then %let CatLabel = &CatName;
%put CatLabel = &CatLabel;
%let _tid_ = %sysfunc(close(&_tid_));

* add subjects with no adverse events;
data aes;
   set &in_ae;
   keep USUBJID;
run;

proc sort data=aes nodupkey;
   by USUBJID;
run;

data aes;
   set aes;
   SEVERITY = 1;
run;

data asn;
   set &in_adsl;
   * delete subjects with missing by variables;
   %let i = 1;
   %do %while(%length(%scan(&DesignByVars,&i)));
      if missing(%scan(&DesignByVars,&i)) then delete;
      %let i=%eval(&i+1);
   %end;
   keep USUBJID;
run;

proc sort data=asn;
   by USUBJID;
run;

data asn;
   set asn;
   USUBJNUM = _n_;
run;

%NObsVars(asn);
%let ns = &nobs;
%put ns = &ns;

data asub;
   merge asn aes;
   by USUBJID;
run;

%let firstae = ;
data _null_;
   set &in_ae;
   if not missing(&CatName) then call symput("firstae",trim(left(&CatName)));
   stop;
run;

data asub;
   set asub;
   if missing(SEVERITY) then do;
      &CatName = "&firstae";
      AESTDY = -999;
   end;
   else delete;
   keep USUBJID &CatName AESTDY AESTDTC;
run;

data ae;
   set &in_ae asub;
run;

* incorporate time windows if specified;
%AddTimeWindows(domain=ae,indat=ae,outdat=ae,twb=&TimeWindows,tvar=&TimeScale);
%if &exiterror %then %goto exit;

* merge subject number;
proc sort data=ae out=ae;
   by USUBJID &CatName;
run;

data ae;
   merge ae asn;
   by USUBJID;
run;

proc sort data=ae;
   by &CatName &CatKeepVars USUBJNUM;
run;

%if &PropThresh > 0 %then %do;
   proc means data=ae noprint;
      by &CatName;
      var AESTDY;
      output out=aep max=AESTDY;
   run;
   /*
   data outlib.aep;
      set aep;
   run;
   */
   %NObsVars(&in_adsl);
   data aep;
      set aep;
      if _FREQ_/&nobs <= &PropThresh/100 then delete;
      keep &CatName;
   run;
   data ae;
      merge ae aep(in=pin);
      by &CatName;
      if pin;
   run;
%end;

/*
data outlib.ae;
   set ae;
run;
*/

* if processing all events, find max time and total count for each subject;
* otherwise take min over repeated measurements as the time of first event;
%if %index(&ResponseType,all) %then %do;
   proc means data=ae noprint;
      by &CatName &CatKeepVars USUBJNUM;
      var AESTDY;
      output out=aem max=AESTDY;
   run;
%end;
%else %do;
   proc means data=ae noprint;
      by &CatName &CatKeepVars USUBJNUM;
      var AESTDY;
      output out=aem min=AESTDY;
   run;
%end;
/*
data outlib.aem;
   set aem;
run;
*/
* transpose to create one obs per subject per ae;
proc transpose data=aem out=aemt(drop=_name_ _label_) prefix=S_;
   by &CatName &CatKeepVars;
   var AESTDY;
   id USUBJNUM;
run;

* stack to create by groups;
data aemts;
   set aemt;
   array s[*] S_1-S_&ns;
   do i=1 to dim(s);
      if s[i]=-999 then s[i] = .;
      AESTDY = s[i];
      USUBJNUM = i;
      output;
   end;
   keep &CatName &CatKeepVars AESTDY USUBJNUM;
run;

* merge subject-level data;
data td;
   set &in_adsl;
   %let i = 1;
   %do %while(%length(%scan(&DesignByVars,&i)));
      if missing(%scan(&DesignByVars,&i)) then delete;
      %let i=%eval(&i+1);
   %end;
   %include "&MacroPath./TRTDURDY.sas";
   keep USUBJID &DesignByVars TRTDURDY &TrtName SITEID AGE SEX RACE;
run;

proc sql;
   create table aemts1 as
      select *
      from aemts d1, asn d2
      where d1.USUBJNUM=d2.USUBJNUM;
quit;

%let AETTE_OutData_Day1 = ;
%PrepOutDS(AETTE_OutData_Day1,AETTE_OutDataName_Day1,ae,_days1);
proc sql;
   create table OutLib.&AETTE_OutDataName_Day1 as
      select *
      from aemts1 d1, td d2
      where d1.USUBJID=d2.USUBJID;
quit;

%if &syserr>4 %then %goto exit;

%if &TimeScale = Weeks %then %do;
   %let TTEName = Week;
   %let trTimeScale = %nrbquote(%LocaleGet(Weeks_Text));
   %let trTTEName = %nrbquote(%LocaleGet(Week_Text));
%end;
%else %do;
   %let TTEName = Day;
   %let trTimeScale = %nrbquote(%LocaleGet(Days_Text));
   %let trTTEName = %nrbquote(%LocaleGet(Day_Text));
%end;

%PrepOutDS(AETimeToEvent_OutData,OutDataName,ae,_tte);

%let trCENSOR = %nrbquote(%LocaleGet(Censor_Text));
%let trMaxKMDiff = %nrbquote(%LocaleGet(MaxKMDiff_Text));
%let trNegLog10p = %nrbquote(%LocaleGet(Neglog10pvalue_Text));
%let trSignedLog10p = %nrbquote(%LocaleGet(Signedlog10pvalue_Text));
%let trLogRank = %nrbquote(%LocaleGet(LogRank_Text));
%let trover = %nrbquote(%LocaleGet(over_Text));
%let trLIDR = %nrbquote(%LocaleGet(LogIncidenceDensityRatio_Text));

%if %index(&ResponseType,Time) %then %do;
   * create time-to-event and censor variables;
   data OutLib.&AETTE_OutDataName_Day1;
      set OutLib.&AETTE_OutDataName_Day1;
      if missing(AESTDY) then do;
         &TTEName = TRTDURDY;
         CENSOR = 1;
      end;
      else do;
         &TTEName = AESTDY;
         CENSOR = 0;
      end;
      %if &TimeScale = Weeks %then %do;
         &TTEName = &TTEName / 7;
      %end;
      label &TTEName = "&trTTEName";
      /*Why does the line below affect the results in Chinese, or in English if we label it anything other than "Censor"? */
      /*label CENSOR = "&trCENSOR"; */
   run;

   proc sort data=OutLib.&AETTE_OutDataName_Day1;
      by &CatName &CatKeepVars &DesignByVars;
   run;
   
   data OutLib.&OutDataName;
      if (0);
   run;

   %let AETTE_OutData_smu = ;
   %PrepOutDS(AETTE_OutData_smu,AETTE_OutDataName_smu,ae,_smu);
   * call proc lifetest;
   ods exclude all;
   ods noresults;

   proc lifetest data=OutLib.&AETTE_OutDataName_Day1 outsurv=ae_surv1;
      by &CatName &CatKeepVars &DesignByVars;
      time &TTEName*CENSOR(1);
      strata &TrtName;
      * strata SITEID / group=&TrtName;
      ods output homtests=OutLib.&OutDataName means=outlib.&AETTE_OutDataName_smu;
   run;

   ods exclude none;
   ods results;

   %if &syserr>4 %then %goto exit;
   
   %if (&AESerOnly = Yes) %then %do;
   	%nobsvars(OutLib.&OutDataName);
   	%if ^&nvars %then %do;
   		%put ERROR: There is an insufficient number of subjects within each level of serious adverse events to perform this analysis.;
   		%let exiterror = 1; 
   		%goto exit;
   	%end;
   %end;
   /* 
   data outlib.tests;
      set outlib.&OutDataName;
   run;
   */

   %NObsVars(ae_surv1);
   %let nobs0 = &nobs;
   data means;
      set outlib.&AETTE_OutDataName_smu;
      where not missing(mean);
   run;
   %NObsVars(means);
   %if ^&nobs0 or ^&nobs %then %do;
      %put [MS_S_0050];
      %put ERROR: Time-to-event analysis produced no results because of insufficient or invalid data.;
      %put;
      %put There may not be enough data for each level of AEDECOD to produce valid tests, or;
      %put TRTSDT, TRTSDTM, TRTEDT, TRTEDTM, RFSTDTC and/or RFENDTC may not exist in the adsl data set.;
      %let exiterror = 1;
      %goto exit;
   %end;

   %if &nlev > 1 %then %do;
      * compute maximum KM vertical distance statistic;
      proc sort data=ae_surv1 out=surv;
         where not missing(&CatName);
         by &CatName &DesignByVars &TrtName &TTEName;
      run;

      * determine last day for all arms for which we have a survival estimate;
      proc means data=surv noprint;
         by &CatName &DesignByVars &TrtName;
         var &TTEName;
         output out=dmax max=DMAX;
      run;

      proc means data=dmax noprint;
         by &CatName &DesignByVars;
         var DMAX;
         output out=dlast min=DLAST;
      run;

      data surv;
         merge surv dlast;
         by &CatName &DesignByVars;
         if &TTEName > DLAST then delete;
      run;

      * fill in missing data;
      data surv;
         set surv;
         retain LAGSURV 1;
         LAGSURV = lag(SURVIVAL);
         if missing(SURVIVAL) then SURVIVAL=LAGSURV;
      run;
/*
      data outlib.surv;
         set surv;
      run;
*/
      proc sort data=surv out=surv;
         by &CatName &DesignByVars &TTEName &TrtName;
      run;

      proc means data=surv noprint;
         by &CatName &DesignByVars &TTEName &TrtName;
         var SURVIVAL;
         output out=survm mean=SURVIVAL;
      run;
/*    
      data outlib.survm;
         set survm;
      run;
*/    
      * prepend all values of trt with a blank catname so transposed variables are in order;
      data survmp;
         set survm(obs=1);
         &CatName = " ";
         %do i=1 %to &nlev;
            &TrtName = "&&tlev&i";
            output;
         %end;
      run;
      
      data survmp;
         set survmp survm;
      run;
         
      /*
      data outlib.survmp;
         set survmp;
      run;
      */
      
      %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) ^= EN %then %do;
      * if Trtname is other language, there is no way to do transpose;
         proc sort data=survmp;
            by &TrtName;
         run;
         data survmp;
            set survmp;
            by &TrtName;
            retain TrtNameTmp 0;
            if first.&TrtName then TrtNameTmp+1;
         run;
         proc sort data=survmp;
            by &CatName &DesignByVars &TTEName;
         run;
         proc transpose data=survmp out=survmt(drop=_name_ _label_);
            by &CatName &DesignByVars &TTEName;
            var SURVIVAL;
            id TrtNameTmp;
            idlabel &TrtName;
         run;
      %end;
      %else %do;
         proc transpose data=survmp out=survmt(drop=_name_ _label_);
            by &CatName &DesignByVars &TTEName;
            var SURVIVAL;
            id &TrtName;
            idlabel &TrtName;
         run;
      %end;

      %if &syserr>4 %then %goto exit;
      
      * remove blank catname row created above;
      data survmt;
         set survmt;
         if _n_ = 1 then delete;
      run;

      data survmt1;
         set survmt(obs=1);
         drop &CatName &DesignByVars &TTEName;
      run;
/*    
      data outlib.survmt1;
         set survmt1;
      run;
*/
      %let _sid_ = %sysfunc(open(survmt1));
      %let nslev = %sysfunc(attrn(&_sid_,NVARS));
      %put nslev = &nslev;
      %put plev = &plev;
      %put tlev&plev = &&tlev&plev;
      
      /* kemicl - S1009529  support specified reference level or autodetect Placebo from TreatmentLevels Macro */
      %let rlev = 1;
      %do i=1 %to &nslev;
         /* %put i=&i v=%sysfunc(varlabel(&_sid_,&i)); */
         %if %qupcase(%sysfunc(varlabel(&_sid_,&i))) = %qupcase(&&tlev&plev) %then %let rlev = &i;
      %end;
      /*
      %let rlev = &plev;
      */
      %put rlev = &rlev;

      * construct differences of KM survival curves;
      data survmt;
         set survmt;
         %do i=1 %to &nslev;  
            retain _LAG&i 1;
            _TLAG = lag(%sysfunc(varname(&_sid_,&i)));
            if not missing(_TLAG) then _LAG&i = _TLAG;
            if missing(%sysfunc(varname(&_sid_,&i))) then 
               %sysfunc(varname(&_sid_,&i)) = _LAG&i;
         %end;    
         %let d = 1;
         %do i=1 %to &nslev;
            %if &i ^= &rlev %then %do;
               _Diff&d = %sysfunc(varname(&_sid_,&rlev)) - %sysfunc(varname(&_sid_,&i));
               label _Diff&d = "%sysfunc(varlabel(&_sid_,&i)) - %sysfunc(varlabel(&_sid_,&rlev))";
               %let d = %eval(&d + 1);
            %end;
         %end;
      run;

      /*
      data outlib.survmt;
         set survmt;
      run;
      */
      
      %let nd = %eval(&nslev-1);
      %put nd = &nd;
      proc means data=survmt noprint;
         by &CatName  &DesignByVars;
         %if (&nd = 1) %then %do;
            var _Diff1;
         %end;
         %else %do;
            var _Diff1-_Diff&nd;
         %end;
         output out=survmtm(drop=_type_ _freq_)
            min=%do d=1 %to &nd; minDiff&d %end;
            max=%do d=1 %to &nd; maxDiff&d %end;
         ;
      run;

      /*
      data outlib.survmt;
         set survmt;
      run;

      data outlib.survmtm;
         set survmtm;
      run;
      */

      data survmtm;
         set survmtm;
         %do d=1 %to &nd;
            if abs(minDiff&d) > maxDiff&d then MaxKMDiff&d = minDiff&d;
            else MaxKMDiff&d = maxDiff&d;
            drop minDiff&d maxDiff&d;
         %end;
         %let d = 1;
         %do i=1 %to &nslev;
            %if &i ^= &rlev %then %do;
               label MaxKMDiff&d = "&trMaxKMDiff (%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&i)))) - %qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&rlev)))))";
               %let d = %eval(&d + 1);
            %end;
         %end;
      run;
   
      data outlib.&OutDataName;
         merge outlib.&OutDataName survmtm;
         by &CatName &DesignByVars;
         NegLog10p = -log10(ProbChiSq);
         if (abs(MaxKMDiff1) < 1e-6) then SignedLog10p = 0;
         else SignedLog10p = min(6.001,NegLog10p)*abs(MaxKMDiff1)/MaxKMDiff1;
         label 
            NegLog10p = "&trNegLog10p"
            SignedLog10p = "&trSignedLog10p"
         ;
         if Test = "-2Log(LR)" or Test = "-2Log(LR)*" or missing(NegLog10p) then delete;
      run;
      
      * add dummy values so color scale is always the same;
      data dummy;
         set OutLib.&OutDataName(obs=1);
         Test = symget('trLogRank');
         &CatName = "_L";
         ChiSq = .;
         DF = .;
         ProbChiSq = .;
         MaxKMDiff1 = .;
         NegLog10p = .;
         SignedLog10p = -6;
         output;
         &CatName = "_H";
         SignedLog10p = 6;
         output;
      run;
      data OutLib.&OutDataName;
         set OutLib.&OutDataName dummy;
      run;

      %if &syserr>4 %then %goto exit;
      
      data b1;
         set outlib.&OutDataName;
         if Test = "Log-Rank" or Test = symget('trLogRank');
         Raw_p = ProbChiSq;
         keep Raw_p;
      run;
      
   %end;
%end;
%else %do;
   * incidence density calculations;
   * create time-to-event and event variables;
   data OutLib.&AETTE_OutDataName_Day1;
      set OutLib.&AETTE_OutDataName_Day1;
      if missing(AESTDY) then do;
         &TTEName = TRTDURDY;
         EVENT = 0;
      end;
      else do;
         &TTEName = AESTDY;
         EVENT = 1;
      end;
      %if &TimeScale = Weeks %then %do;
         &TTEName = &TTEName / 7;
      %end;
   run;
%*put ERROR: ResponseType=&ResponseType;
   %if %index(&ResponseType,first) %then %do;
      proc sort data=OutLib.&AETTE_OutDataName_Day1;
         by &CatName &CatKeepVars &DesignByVars &TrtName;
      run;
      
      data agg;
         set OutLib.&AETTE_OutDataName_Day1;
         NEvent = Event;
         TotalTime = &TTEName;
      run;
      
      /*
      proc means data=OutLib.&AETTE_OutDataName_Day1 noprint;
         by &CatName &CatKeepVars &DesignByVars &TrtName;
         var EVENT &TTEName;
         output out=agg sum=NEvent TotalTime;
      run;
      */
   %end; 
   %else %do;
      proc sort data=OutLib.&AETTE_OutDataName_Day1;
         by &CatName &CatKeepVars USUBJNUM;
      run;

      * transpose counts to create one obs per subject per ae;
      proc transpose data=aem out=aemtf(drop=_name_ _label_) prefix=S_;
         by &CatName &CatKeepVars;
         var _FREQ_;
         id USUBJNUM;
      run;

      * stack to create by groups;
      data aemtfs;
         set aemtf;
         array s[*] S_1-S_&ns;
         do i=1 to dim(s);
            if missing(s[i]) then s[i] = 0;
            NEvent = s[i];
            USUBJNUM = i;
            output;
         end;
         keep &CatName &CatKeepVars NEvent USUBJNUM;
      run;

      data agg;
         merge OutLib.&AETTE_OutDataName_Day1 aemtfs;
         by &CatName &CatKeepVars USUBJNUM;
         TotalTime = &TTEName;
      run;
      
      %if %length(&DesignByVars) %then %do;
         proc sort data=agg;
            by &CatName &CatKeepVars &DesignByVars;
         run;
      %end;
   %end;
      
   data agg;
      set agg;
      LogTotalTime = log(max(TotalTime,1));
   run;

   %let AETTE_OutData_agg = ;
   %PrepOutDS(AETTE_OutData_agg,AETTE_OutDataName_agg,ae,_agg);
   data outlib.&AETTE_OutDataName_agg;
      set agg;
   run;
   
   %if &Distribution = Negative Binomial %then %let Distribution = NB;

   * call proc genmod;
   ods exclude all;
   ods noresults;

   proc glimmix data=agg ic=pq namelen=200;
      by &CatName &CatKeepVars &DesignByVars;
      class &TrtName &ClassVars;
      NLOPTIONS gconv=%sysevalf(&GConv/100000000);
      model NEvent = &TrtName &FixedEffects / dist=&Distribution link=log offset=LogTotalTime;
      %if %length(&RandomEffects) %then %do;
         random &RandomEffects;
      %end;
      lsmeans &TrtName / diff;
      ods output diffs=diffs fitstatistics=fitstatistics tests3=tests3
         %if %length(&RandomEffects) %then %do;
            covparms=covparms 
         %end;
      ;
   run;
   %let PValue = Probt;
   
   /*
   proc genmod data=agg;
      by &CatName &CatKeepVars &DesignByVars;
      class &TrtName sex race;
      model NEvent = &TrtName / dist=poisson link=log offset=LogTotalTime;
      lsmeans &TrtName / diff;
      ods output diffs=diffs;
   run;
   %let PValue = Probz;
   */

   ods exclude none;
   ods results;

   %if &syserr>4 %then %goto exit;
/* 
   data outlib.diffs;
      set diffs;
   run;
*/ 

   proc sort data=agg out=s1 nodupkey;
      by &TrtName;
   run;

   proc transpose data=s1 out=s2(drop=_name_ _label_);
      var NEvent;
      id &TrtName;
      idlabel &TrtName;
   run;
/*
   data outlib.s2;
      set s2;
   run;
*/
   %let _sid_ = %sysfunc(open(s2));
   %let nslev = %sysfunc(attrn(&_sid_,NVARS));
   %let nd = %eval(&nslev-1);

   /* kemicl - S1009529  support specified reference level or autodetect Placebo from Treatment Levels Macro*/
   %let rlev = &plev;
   
   %put rlev = &rlev;
   %put rlev name = %sysfunc(varname(&_sid_,&rlev));
   %if &rlev = 1 %then %let olev = 2;
   %else %let olev = 1;
   
   data diffs;
      set diffs;
      length Ratio $ 250;
      if trim(left(&TrtName)) ne "%sysfunc(varlabel(&_sid_,&rlev))" and 
         trim(left(_&TrtName)) ne "%sysfunc(varlabel(&_sid_,&rlev))" then delete;
      * make sure reference level is always second;
      if &TrtName = "%sysfunc(varlabel(&_sid_,&rlev))" then do;
         _temp = &TrtName;
         &TrtName = _&TrtName;
         _&TrtName = _temp;
         Estimate = -Estimate;
         drop _temp;
      end;
      Ratio = trim(left(&TrtName)) || " " || symget('trover') || " " || trim(left(_&TrtName));
      /*
      if &PValue > 0.999 then &PValue = .;
      */
      if missing(&PValue) or missing(tValue) then NegLog10p = .;
      else if &PValue < 1e-64 then NegLog10p = 64;
      else NegLog10p = -log10(&PValue);
      if missing(NegLog10p) then delete;
      if (abs(Estimate) < 1e-6) then SignedLog10p = 0;
      else SignedLog10p = min(6.001,NegLog10p)*abs(Estimate)/Estimate;
      label 
         Estimate = "&trLIDR"
         NegLog10p = "&trNegLog10p"
         SignedLog10p = "&trSignedLog10p"
      ;
   run;
   
   /*
   data outlib.diffs1;
      set diffs;
   run;
   */

   * add dummy values so color scale is always the same;
   data dummy;
      set diffs(obs=1);
      &CatName = "_L";
      Estimate = .;
      &PValue = .;
      NegLog10p = .;
      SignedLog10p = -6;
      output;
      &CatName = "_H";
      SignedLog10p = 6;
      output;
   run;
   data  OutLib.&OutDataName;
      set diffs dummy;
   run;
   
   data b1;
      set diffs;
      Raw_p = &PValue;
      if missing(Raw_p) then delete;
      keep Raw_p;
   run;

%end;

/*
%NObsVars(outlib.&OutDataName);
%if ^&nobs %then %do;
   %put ERROR: All tests produced missing values.  There may be insufficient data to perform this analysis.;
   %let exiterror = 1;
   %goto exit;
%end;
*/

%if ^%symexist(nslev) %then %do; * avoid error for the case of one arm (S1066597 TC 8/19/14);
   %let nslev = 1;
%end;

* compute multiple testing cutoff;
%if &nslev > 1 %then %do;
   %let MultipleTestingMethod = FDR;
   %let Alpha = 0.05;
   %put Computing p-value adjustment: &MultipleTestingMethod;

   %include "&MacroPath.\MultipleTestingAdjustment.sas";

   %AdjPValueName(method=&MultipleTestingMethod);
   %MultipleTestingAdjustment(data=b1,method=&MultipleTestingMethod,out=b1);

   %if &syserr > 4 or ^&nobs %then %goto exit;

   %put Alpha = &Alpha;
   %put ntests = &ntests;
   %put mtpvar = &mtpvar;

   data _null_;
      cutoff = -log10(&Alpha/&ntests);
      call symput("NegLog10pcutoff",trim(left(round(cutoff,0.01))));
   run;

   %put Initial -log10 cutoff (Bonferroni) = &NegLog10pcutoff;

   proc sort data=b1;
      by Raw_p;
   run;

   data _null_;
      set b1;
      if not missing(&mtpvar) and (&mtpvar <= &Alpha) then do;
         cutoff = -log10(raw_p);
         call symput("NegLog10pcutoff",trim(left(round(cutoff-0.01,0.01)))); 
      end;
      else stop;
   run;
   %put NegLog10pcutoff = &NegLog10pcutoff;
%end;

%let odata = &AETimeToEvent_OutData;
%include "&MacroPath./DesignByVars.sas";
%put ndbv = &ndbv;
%put TotLvls  = &TotLvls;
%put TotalCLvl_DBV = &TotalCLvl_DBV;

* extract effects for drill-down, code from MixedModelMacros.sas;
%let ByVars = &CatName &CatKeepVars &DesignByVars;
%let nby = %sysfunc(countw(&ByVars));
%let LastByVar = %scan(&ByVars,&nby,' ');
%if ^%index(&ResponseType,Time) %then %do;
   data t3check;
      set tests3;
      if _n_ > 1 and (&LastByVar ne lag(&LastByVar)) then do;
         stop;
      end;
   run;
   data t3check;
      set t3check end=_last;
      call symputx('fef'||trim(left(_n_)),effect);
      if _last then do;
         call symputx('nfef',trim(left(_n_)));
      end;
   run;
   %put nfef = &nfef;
   %do i=1 %to &nfef;
      %put fef&i = &&fef&i;
   %end;
%end;



* create JSL code;
%PrepOutFile(AETimeToEventScript,AETimeToEventScriptName,&StudyName &ProcessName Script.jsl);

* create JSL file;
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &AETimeToEventScript;
%let FullAPName=%str(AE Time to Event);
%let reportString=%nrbquote();

data _null_;
   file "&AETimeToEventScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put '//!';
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:clientoutpath = ""&ClientOutPath"";";
   put "objRefNS:clientoutpath2 = ""&ClientOutPath"";";
   put "objRefNS:serveroutpath = ""&ServerOutPath"";";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList= {};";
   put "objRefNS:NoteDomain = ""AE"";";      
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:PopulationSet=""&PopulationSet"");";
   put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&AEWhereStmt)]\"");";
   put "try(objRefNS:includestring=""Adverse Events: "");";
   %let DomainWhereStmt=&AEWhereStmt;
   %if %length(&WhereForReports) %then %do;
      put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
      put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
      %if %length(&DomainWhereStmt) %then %do;
      put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;
   %end;
   %else %if %length(&DomainWhereStmt) %then %do;
      put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
   %end;
   put "objRefNS:catname = ""&CatName"";";
   put "objRefNS:catlabel = ""&CatLabel"";";
   put "jg:trt = ""&TrtLabel"";";
   %if ^%index(&ResponseType,Time) %then %do;
      put "objRefNS:agg_dataset = ""&AETTE_OutDataName_agg"";";
   %end;
   
   /* put drill down into action button */
   %if %index(&ResponseType,Time) %then %do;
      put "jg:lsindex = 3;";
      put "include(""$JMP_HOME\LifeSciences\JSLFiles\Category Drill Down Expressions.jsl"");";
      put "objRefNS:CatExpr = Expr(";
      put "  Current Data Table(objRefNS:tted);";
      put "  objRefNS:CatSaveSelected;";
      put "  ttename = ""&trTTEName"";";
      put "  ae_days1 = ""&AETTE_OutDataName_Day1"";";
      put "  objRefNS:CatKaplanMeier;";
      put ");";
      put "objRefNS:ActionButtonDesc = {Expr(locale:get(""SelectPointsthenClick_Text""))};";
      put "objRefNS:ActionButtonList = {{Expr(locale:get(""KaplanMeierandHazardPlots_Text""))}};";
      put "objRefNS:ActionCodeList = {{objRefNS:CatExpr}};";
      put "objRefNS:ActionRequirementFlagList = {{1}};"; 
      put "objRefNS:ActionIconList ={{""&IconPath.LifeTables.gif""}};";
      put "objRefNS:ActionSelectionCountList ={{1}};";
      put "objRefNS:ActionDataTableList = {{{""objRefNS:tted""}}};";      
   %end;
   %else %if &Distribution=Poisson and ^%length(&RandomEffects) %then %do;
      put "jg:lsindex = 3;";
      put "include(""$JMP_HOME\LifeSciences\JSLFiles\Category Drill Down Expressions.jsl"");";
      put "objRefNS:effects = {" @@;
      %do i=1 %to &nfef;
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         put " &&fef&i " @@;
      %end; 
      /*
      %let i=1;
      %do %while(%length(%scan(&RandomEffects,&i,' ')));
         %let fei = %scan(&RandomEffects,&i,' ');
         %if &nfef or %length(&FixedEffects) or (&i > 1) %then %do;
            put ", " @@;
         %end;
         put "&fei %str(&) Random" @@;
         %let i=%eval(&i+1);
      %end;
      */
      put "};";
      put "objRefNS:CatExpr = Expr(";
      put "  Current Data Table(objRefNS:tted);";
      put "  objRefNS:CatSaveSelected;";
      put "  objRefNS:CatIncidenceDensity;";
      put ");";
      put "objRefNS:ActionButtonDesc = {Expr(locale:get(""SelectPointsthenClick_Text""))};";
      put "objRefNS:ActionButtonList = {{Expr(locale:get(""FitIncidenceDensityModel_Text""))}};";
      put "objRefNS:ActionCodeList = {{objRefNS:CatExpr}};";
      put "objRefNS:ActionRequirementFlagList = {{1}};";       
      put "objRefNS:ActionIconList ={{""&IconPath.FitModel.gif""}};";
      put "objRefNS:ActionSelectionCountList ={{1}};";
      put "objRefNS:ActionDataTableList = {{{""objRefNS:tted""}}};";      
   %end;

   
   * when there is only one arm, plot means and standard errors;
   %if &nslev = 1 %then %do;
      put "objRefNS:msfunc = Function({}, {default local},";
      put " if(IsEmpty(objRefNS:tted),";
      put "  objRefNS:tted = Open(""&ClientOutPath.&AETTE_OutDataName_smu..sas7bdat"",invisible);";   
      put "  Column(""&CatLabel"") << Label(1);";
      put "  , //else";
      put "  Current Data Table(objRefNS:tted);";
      put " );"; 
      put "objRefNS:dbv = {};";
      %do i=1 %to &ndbv;
         put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
      %end;
      put " objRefNS:dbvt = objRefNS:dbv;";  
      put " VListBox(";
      put "    Graph Builder(";
      put "       Show Control Panel( 0 ),";
      put "       Variables( X( :Mean ), Y( :Standard Error ) ),";
      %if &ndbv %then %do;
         put "       Group Y(eval(objRefNS:dbvt)),";
      %end;
      put "       Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
      put "       SendToReport(";
      put "          Dispatch(";
      put "             {},";
      put "             ""Mean"",";
      put "             ScaleBox,";
      put "             {Scale( ""Log"" )}";
      put "          ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""Standard Error"",";
      put "             ScaleBox,";
      put "             {Scale( ""Log"" )}";
      put "          )";
      put "       )";
      put "    );";
      put " )";
      put ");";
      put "InsertInto(objRefNS:DataList,{{""objRefNS:tted""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trAdverseEventsTimetoEvent""}});";
      put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""SurvivalMeansandStandardErrors_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:msfunc()));";
      put "InsertInto(objRefNS:TabOpenList,1);";
      /*
      put "InsertInto(objRefNS:CreateReportList,{{&reportstring}});";
      */
   %end;
   %else %do;
      put "objRefNS:cutoff = &NegLog10pcutoff;";
      put "objRefNS:fsize = 366;";
      put 'objRefNS:msize = 4;';
      put "objRefNS:lhratio = 1.22;";
      put "objRefNS:vpfunc = Function({}, {default local},";
      put " if(IsEmpty(objRefNS:tted),";
      put "  objRefNS:tted = Open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";   /* first for data list */
      put "  objRefNS:tted << Color By Column(:Name(""&trSignedLog10p""),ColorTheme(""&ColorTheme""));";
      put "  Column(""&CatLabel"") << Label(1);";
      put "  for each row(";
      put "   if (:Name(""&trNegLog10p"") >= objRefNS:cutoff,labeled(rowstate())=1);";
      put "  );"; 
      put "  , //else";
      put "  Current Data Table(objRefNS:tted);";
      put " );";  /*end if stmnt for dt */
      put "objRefNS:dbv = {};";
      %do i=1 %to &ndbv;
         put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
      %end;
      %if %index(&ResponseType,Time) %then %do;
         %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %do;
            put " cols=objRefNS:tted << GetColumnNames(); ";
            put " for (i=1,i<=NItems(cols),i++,";
            put " ci = Column(cols[i]);";
            put " saslb = char(ci << GetProperty(""SAS Label""));";
            put " sasnm = char(ci << GetProperty(""SAS Name""));";
            put " if(length(saslb) & sasnm == ""Test"",colNW = saslb);";
            put ");";
            put " objRefNS:dbvt = Insert(objRefNS:dbv,Column(colNW));";
         %end;
         %else %do;
            put " objRefNS:dbvt = Insert(objRefNS:dbv,Column(""Test"" /*locale:get(""Test_Text"")*/));";
         %end;
      %end;
      %else %do;
         put " objRefNS:dbvt = objRefNS:dbv;";  
      %end;
      
      %let trResponseTypeL = %nrbquote(%LocaleGet(ResponseType_Text));
      /* Re-translate the Response Type chosen */
      %let trResponseType = &ResponseType;
      %if %upcase(&trResponseType) = %str(TIME TO FIRST EVENT) %then
         %let trResponseType = %nrbquote(%LocaleGet(Timetofirstevent_Text));
      %else %if %upcase(&trResponseType) = %str(INCIDENCE DENSITY BASED ON FIRST EVENT ONLY) %then
         %let trResponseType = %nrbquote(%LocaleGet(Incidencedensitybasedonfirsteventonly_Text));
      %else %if %upcase(&trResponseType) = %str(INCIDENCE DENSITY BASED ON ALL EVENTS) %then
         %let trResponseType = %nrbquote(%LocaleGet(Incidencedensitybasedonallevents_Text));
      /* End of re-translate Response Type chosen */
      /* Re-translate the Multiple Testing Method */
      %let trMTM = %nrbquote(%LocaleEZX(%nrbquote(&MultipleTestingMethod), EIA=1));
      /* End of re-translate Multiple Testing Method */
      
      put " VListBox(";
      %if %length(&TEAEText) %then %do;
         put "objRefNS:TEAEText = TextBox("" &TEAEText"", <<Set Wrap(1000)),";
      %end;      
      put "   objRefNS:ResponseType=TextBox("" &trResponseTypeL = &trResponseType""),";
      put "   objRefNS:FDR=TextBox("" "" || locale:fill(locale:get(""XlinedrawnatMinusLog10pX_Text""), {""&trMTM"", char(objRefNS:cutoff)})),";
      put "   HListBox(";
      put "      VListBox(";
      put '         HListBox(TextBox(" " || locale:get("FrameSize_Text")),';
      put '            frameSizeSlider = SliderBox(num(100),num(700),objRefNS:fsize,';
      %do d=1 %to &nd;
         put "                objRefNS:tbivrep&d = (objRefNS:tbiv&d<<report);";  
         %if %upcase(&SplitResults)=YES %then %do;
            %do i = 1 %to &TotalCLvl_DBV;
               %if %index(&ResponseType,Time) %then %do;
                  put "               objRefNS:tbivrep&d[2*(&i-1)+1][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
                  put "               objRefNS:tbivrep&d[2*(&i-1)+2][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
               %end;
               %else %do;
                  put "               objRefNS:tbivrep&d[&i][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
               %end;
            %end;
         %end;
         %else %do;
            put "              objRefNS:tbivrep&d[1][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
            %if %index(&ResponseType,Time) %then %do;
               put "              objRefNS:tbivrep&d[2][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
            %end;
         %end;
      %end;
      put '            )';
      put '         ),';
      put '         HListBox(TextBox(" " || locale:get("MarkerSize_Text")),';
      put '            markerSizeSlider = SliderBox(num(-1),num(10.5),objRefNS:msize,';
      %do d=1 %to &nd;
         put "                objRefNS:tbivrep&d = (objRefNS:tbiv&d<<report);";
         %if %upcase(&SplitResults)=YES %then %do;
            %do i = 1 %to &TotalCLvl_DBV;
               %if %index(&ResponseType,Time) %then %do;
                  put "              objRefNS:tbivrep&d[2*(&i-1)+1][framebox(1)]<<MarkerSize(objRefNS:msize);";
                  put "              objRefNS:tbivrep&d[2*(&i-1)+2][framebox(1)]<<MarkerSize(objRefNS:msize);";
               %end;
               %else %do;
                  put "              objRefNS:tbivrep&d[&i][framebox(1)]<<MarkerSize(objRefNS:msize);";
               %end;
            %end;
         %end;
         %else %do;
            put "               objRefNS:tbivrep&d[1][framebox(1)]<<MarkerSize(objRefNS:msize);";
            %if %index(&ResponseType,Time) %then %do;
               put "               objRefNS:tbivrep&d[2][framebox(1)]<<MarkerSize(objRefNS:msize);";
            %end;
         %end;
      %end;
      put '            )';
      put '         ),';
      put "      ),";
      put "   ),";

      put " objRefNS:GraphBoxes = VListBox(";
      %do i=1 %to &nslev;
         %if &i ^= &rlev %then %do;
            %if &i < &rlev %then %let d = &i;
            %else %let d = %eval(&i-1);
            %if %index(&ResponseType,Time) %then %do;
               put '   LineUpBox(NCol(2),';
            %end;
            put "      objRefNS:tbiv&d = Bivariate(";
            %if %index(&ResponseType,Time) %then %do;
               put "         Y(:Name(""&trNegLog10p"")),";
               put "         X(:Name(""&trMaxKMDiff (%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&i)))) - %qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&rlev)))))"")),";
            %end;
            %else %do;
               put "         Y(:Name(""&trNegLog10p"")),";
               put "         X(:Name(""&trLIDR"")),"; 
               put "         Where(:Ratio==""%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&i)))) &trover %qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&rlev))))""),";
            %end;
            put "         By(eval(objRefNS:dbvt)),";
            put '         SendToReport(';
            put '            Dispatch(';
            put '               {},';
            put '               "2",';
            put '               ScaleBox,';
            put '               {Scale( Linear ), Format( "Best" ),';
            put '               Add Ref Line( objRefNS:cutoff, Dashed, Red )}';
            put '            )';
            put '         )';
            put '      ),';
            %if %index(&ResponseType,Time) %then %do;
               put '   ),';
            %end;
            %if %upcase(&SplitResults)=YES %then %do;
               %do j = 1 %to &TotalCLvl_DBV;
                  put "   objRefNS:tbivrep&d = (objRefNS:tbiv&d<<report);";               
                  %if %index(&ResponseType,Time) %then %do;
                     put "   objRefNS:tbivrep&d[2*(&j-1)+1][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
                     put "   objRefNS:tbivrep&d[2*(&j-1)+1][framebox(1)]<<MarkerSize(objRefNS:msize);";
                     put "   objRefNS:tbivrep&d[2*(&j-1)+1][framebox(1)]<<Marker Drawing Mode(Outlined);";                  
                     put "   objRefNS:tbivrep&d[2*(&j-1)+2][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
                     put "   objRefNS:tbivrep&d[2*(&j-1)+2][framebox(1)]<<MarkerSize(objRefNS:msize);";
                     put "   objRefNS:tbivrep&d[2*(&j-1)+2][framebox(1)]<<Marker Drawing Mode(Outlined);";
                  %end;
                  %else %do;
                     put "   objRefNS:tbivrep&d[&j][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
                     put "   objRefNS:tbivrep&d[&j][framebox(1)]<<MarkerSize(objRefNS:msize);";
                     put "   objRefNS:tbivrep&d[&j][framebox(1)]<<Marker Drawing Mode(Outlined);";                  
                  %end;                  
               /* kemicl - added for legend */
                  put "    try(";
                  put "         objRefNS:RedVar&d = ""%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&i))))"";";
                  put "         objRefNS:BlueVar&d = ""%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&rlev))))"";";
                  put "         //show(objRefNS:RedVar&d, objRefNS:BlueVar&d);";
                  put "         objRefNS:colorgb = Substitute(Expr(Graph Box(";
                  put "           Frame Size(objRefNS:fsize*objRefNS:lhratio, 0.083*objRefNS:fsize),";
                  put "           X Scale(0,120),";
                  put "           Y Scale(0,3),";
                  put "           SuppressAxes,";
                  put "           Text Color(""black""); Text Size(8);";
                  put "           Text(Right Justified, {56,1.5}, ""Larger in ""||_bv1_); ";
                  put "           For(_inc_=0,_inc_ < 1,_inc_+=.05,";
                  put "                      xinc = 47.5+25*_inc_;";
                  put "                      Fill Color(";
                  put "                            cc=Heat Color(";
                  put "                               _inc_,";
                  put "                               <<""&ColorTheme"" ";
                  put "                            )";
                  put "                      );";        
                  put "                      Rect(xinc -0.75 , 1.3,xinc +0.75, 0.2,1);";
                  put "           );";
                  put "           Text Color(""black""); Text Size(8);";
                  put "           Text({64,1.5}, locale:fill(locale:get(""LargerinX_Text""), {_bv2_}));";
                  put "         )), Expr(_bv1_), objRefNS:BlueVar&d, Expr(_bv2_), objRefNS:RedVar&d);";
                  %if %index(&ResponseType,Time) %then %do;
                     put "         Ochild&d.1 = objRefNS:tbivrep&d[2*(&j-1)+1][outlinebox(1)]<< Child();";
                     put "         Ochild&d.1 << Sib Append(objRefNS:colorgb, Vertical);";                     
                     put "         Ochild&d.2 = objRefNS:tbivrep&d[2*(&j-1)+2][outlinebox(1)]<< Child();";                  
                     put "         Ochild&d.2 << Sib Append(objRefNS:colorgb, Vertical);";
                  %end;
                  %else %do;
                     put "         Ochild&d.1 = objRefNS:tbivrep&d[&j][outlinebox(1)]<< Child();";
                     put "         Ochild&d.1 << Sib Append(objRefNS:colorgb, Vertical);";  
                  %end;
                  put "       ); ";/* close try for legend*/               
               %end;
            %end;
            %else %do;
               put "   objRefNS:tbivrep&d = (objRefNS:tbiv&d<<report);";         
               put "   objRefNS:tbivrep&d[1][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
               put "   objRefNS:tbivrep&d[1][framebox(1)]<<MarkerSize(objRefNS:msize);";
               put "   objRefNS:tbivrep&d[1][framebox(1)]<<Marker Drawing Mode(Outlined);";
               %if %index(&ResponseType,Time) %then %do;
                  put "   objRefNS:tbivrep&d[2][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize);";
                  put "   objRefNS:tbivrep&d[2][framebox(1)]<<MarkerSize(objRefNS:msize);";
                  put "   objRefNS:tbivrep&d[2][framebox(1)]<<Marker Drawing Mode(Outlined);";
               %end;
               /* kemicl - added for legend */
               /* put "       try("; */
               put "         objRefNS:RedVar&d = ""%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&i))))"";";
               put "         objRefNS:BlueVar&d = ""%qleft(%nrbquote(%sysfunc(varlabel(&_sid_,&rlev))))"";";
               put "        // show(objRefNS:RedVar&d, objRefNS:BlueVar&d);";
               put "         objRefNS:colorgb = Substitute(Expr(Graph Box(";
               put "           Frame Size(objRefNS:fsize*objRefNS:lhratio, 0.083*objRefNS:fsize),";
               put "           X Scale(0,120),";
               put "           Y Scale(0,3),";
               put "           SuppressAxes,";
               put "           Text Color(""black""); Text Size(8);";
               put "           Text(Right Justified, {56,1.5}, locale:fill(locale:get(""LargerinX_Text""), {_bv1_}));   ";
               put "           For(_inc_=0,_inc_ < 1,_inc_+=.05,";
               put "                      xinc = 47.5+25*_inc_;";
               put "                      Fill Color(";
               put "                            cc=Heat Color(";
               put "                               _inc_,";
               put "                               <<""&ColorTheme"" ";
               put "                            )";
               put "                      );";        
               put "                      Rect(xinc -0.75 , 1.3,xinc +0.75, 0.2,1);";
               put "           );";
               put "           Text Color(""black""); Text Size(8);";
               put "           Text({64,1.5},  locale:fill(locale:get(""LargerinX_Text""), {_bv2_}));";
               put "         )), Expr(_bv1_), objRefNS:BlueVar&d, Expr(_bv2_), objRefNS:RedVar&d);";
               %if %index(&ResponseType,Time) %then %do;
                  put "         Ochild&d.1 = objRefNS:tbivrep&d[1][outlinebox(1)]<< Child();";
                  put "         Ochild&d.1 << Sib Append(objRefNS:colorgb, Vertical);";   
                  put "         Ochild&d.2 = objRefNS:tbivrep&d[2][outlinebox(1)]<< Child();";
                  put "         Ochild&d.2 << Sib Append(objRefNS:colorgb, Vertical);";
               %end;
               %else %do;
                  put "         Ochild&d.1 = objRefNS:tbivrep&d[1]<< Child();";
                  put "         Ochild&d.1 << Sib Append(objRefNS:colorgb, Vertical);";   
               %end;
               /* put "       );"; */ /* close try for legend*/
            %end;
         %end;
      %end;
      put " )";  /*end graph containing vlistbox*/

      put ');'; /* end VlistBox*/
      /*put 'vp << SetWindowIcon("$JMP_HOME\LifeSciences\Documentation\Icons\ANOVAPlot.bmp");';*/
      put ");";

      put "InsertInto(objRefNS:DataList,{{""objRefNS:tted""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trAdverseEventsTimetoEvent""}});";
      put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""VolcanoPlots_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:vpfunc()));";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:TEAEText,objRefNS:ResponseType,objRefNS:FDR,objRefNS:GraphBoxes}});";
   %end;
run;

%let outds = &OutDataName..sas7bdat,&AETTE_OutDataName_Day1..sas7bdat,&AETTE_OutDataName_smu..sas7bdat;
%if ^%index(&ResponseType,Time) %then %do;
   %let outds = &outds,&AETTE_OutDataName_agg..sas7bdat;
%end;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &AETimeToEventScriptName, "&outds", text/plain);

%exit:;

%if %symexist(_sid_) %then %do;
   %if &_sid_ %then %do;
      %let _sid_ = %sysfunc(close(&_sid_));
   %end;
%end;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AETimeToEvent;

* run the macro;
%AETimeToEvent;


