/*
JMP Clinical Analytical Process

Name: Birthdays and Initials
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   This process identifies subjects that might be identical based on birthdays or initials.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName BirthdayInitialsScript BirthdayInitials_OutData 
        OutPath ServerOutPath PopulationSet WhereStmt WhereForReport MatchFl BDWindow SexMatch RaceMatch EthnicMatch SiteCount;

*ProcessBody;

%macro BirthdayInitials;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and (DM.BRTHDTC or (SC.USUBJID and SC.SCTESTCD and (SC.SCSTRESC or SC.SCORRES)))$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: CPV.{"Match subjects based on:","Birthday"}$ $UE: DM.BRTHDTC$
$CMP: CPV.{"Match subjects based on:","Initials"}$ $UE: SC.USUBJID and SC.SCTESTCD and (SC.SCSTRESC or SC.SCORRES)$ $NOTES: SC.SCTESTCD with a value of SUBJINIT. $
$CMP: OPV.{"Sex","Yes"}$ $UE: DM.SEX$ $NOTES: If available, this variable is used, and shown in output. Otherwise, this option is ignored.$
$CMP: OPV.{"Race","Yes"}$ $UE: DM.RACE$ $NOTES: If available, this variable is used, and shown in output. Otherwise, this option is ignored.$
$CMP: OPV.{"Ethnicity","Yes"}$ $UE: DM.ETHNIC$ $NOTES: If available, this variable is used, and shown in output. Otherwise, this option is ignored.$
$CMP: OOUT$ $UE: VS.VSTESTCD and VS.VSSTRESN$ $NOTES: VS.VSTESTCD = HEIGHT, WEIGHT, BMI to summarize in output.$ 
$CMP: OOUT$ $UE: VS.VSSTRESU $ $NOTES: Units used if available.$ 
$CMP: OOUT$ $UE: SC.USUBJID and SC.SCTESTCD and (SC.SCSTRESC or SC.SCORRES)$ $NOTES: SC.SCTESTCD = SUBJINIT to summarize in output for Birthday match.$ 
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = BirthdayInitials; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%OUTNAMETRUNC(outdat = BirthdayInitials_OutData, truncnum = 19);
%DQFTRANS;

%let trBirthdaysandInitials = %nrbquote(%LocaleGet(BirthdaysandInitials_Text));

%let WhereForReport=%nrbquote(&WhereStmt);

%if ^%length(&BDWindow) %then %let BDWindow = 0;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&DMVARS), BRTHDTC)) and %upcase(&MatchFl) = BIRTHDAY %then %do;
   %put [MS_S_0054];
   %put ERROR: Subject birthday is not available as a matching criteria.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), BRTHDTC)) = 0 and %sysfunc(findw(%upcase(&DMVARS), BRTHDTC)) > 0 %then %do;
   proc sort data = &in_adsl out = subject;
      by usubjid;
   run;

   %LOADDATA(domlist = DM,mergesupp = 0,checkreview=0);
   proc sort data = &in_dm out = dm;
      by usubjid;
   run;

   data subject;
      merge subject(in = ina) dm(keep = usubjid BRTHDTC);
      by usubjid;
      if ina;
   run;
%end;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), BRTHDTC)) %then %do;
   proc sort data = &in_adsl out = subject;
      by usubjid;
   run;
%end;
%else %do;
   proc sort data = &in_adsl out = subject;
      by usubjid;
   run;
%end;

%let SiteCount = 1;
%GETSITELISTS;
%if &exiterror %then %goto exit;

proc sort data = subject;
   by siteid;
run;

data subject;
   merge subject(in = ina) siteids;
   by siteid;
   if ina;
run;

proc sort data = subject;
   by usubjid;
run;

%if %upcase(&SexMatch) = YES and ^%sysfunc(findw(%upcase(&ADSLVARS), SEX)) %then %let SexMatch = No;
%if %upcase(&RaceMatch) = YES and ^%sysfunc(findw(%upcase(&ADSLVARS), RACE)) %then %let RaceMatch = No;
%if %upcase(&EthnicMatch) = YES and ^%sysfunc(findw(%upcase(&ADSLVARS), ETHNIC)) %then %let EthnicMatch = No;

%let MatchAlso=;
%if %upcase(&SexMatch) = YES or %upcase(&RaceMatch) = YES or %upcase(&EthnicMatch) = YES %then %do;
  %if %upcase(&SexMatch) = YES %then %let MatchAlso = &MatchAlso &trSex;
  %if %upcase(&RaceMatch) = YES %then %let MatchAlso = &MatchAlso &trRace;
  %if %upcase(&EthnicMatch) = YES %then %let MatchAlso = &MatchAlso &trEthnicity;
  %let MatchAlso = %sysfunc(transtrn(&MatchAlso, %str( ), %str(, )));
%end;

*** Remove possible vars from ADSL ***;
data subject;
   set subject;
   %if %sysfunc(findw(%upcase(&ADSLVARS), WEIGHT)) %then %do;
      drop weight;
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), HEIGHT)) %then %do;
      drop height;
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), BMI)) %then %do;
      drop bmi;
   %end; 
run;

%let initfl = ;
*** Initials ***;
%if %sysfunc(indexw(&domains, SC, ' ')) %then %do;
   %LOADDATA(domlist = SC,mergesupp = 0,checkreview=0);
   proc sort data = &in_sc out = sc(where = (upcase(sctestcd) in ("SUBJINIT")));
      by usubjid sctestcd;
   run;
   
   data sc;
      set sc;
      by usubjid sctestcd;
      if first.sctestcd;
      %if %sysfunc(indexw(&SCVARS, SCSTRESC, ' ')) %then %do;
         value = upcase(SCSTRESC);
      %end;
      %else %do;
         value = upcase(SCORRES);
      %end;                
   run;
   %NObsVars(sc);

   %if &nobs %then %do;
      %let initfl = sc;
    
      proc transpose data = sc out = sc(drop = _name_);
         by usubjid;
         id sctestcd;
         idlabel sctest;
         var value;
      run;     
   %end;
%end;

%if ^%length(&initfl) and %upcase(&MatchFl) = INITIALS %then %do;
   %put [MS_S_0055];
   %put ERROR: Subject initials are not available as a matching criteria.;
   %let exiterror = 1;
   %goto exit;
%end;

%let htwtfl = ;
*** Get Height and Weight ***;
%if %sysfunc(indexw(&domains, VS, ' ')) %then %do;
   %LOADDATA(domlist = VS,mergesupp = 0,checkreview=0);
   %if %sysfunc(indexw(&VSVARS, VISITNUM, ' ')) %then %let visitnum = visitnum;
   %else %let visitnum = ;

   proc sort data = &in_vs out = vs(where = (upcase(vstestcd) in ("HEIGHT","WEIGHT","BMI")));
      by usubjid vstestcd &visitnum;
   run;
   
   data vs;
      set vs;
      by usubjid vstestcd &visitnum;
      if first.vstestcd;
      %if %sysfunc(indexw(&VSVARS, VSSTRESU, ' ')) %then %do;
         value = catx(" ", VSSTRESN, VSSTRESU);
      %end;
      %else %do;
         value = VSSTRESN;
      %end;
   run;
   %NObsVars(vs);

   %if &nobs %then %do;
      %let htwtfl = vs;
    
      proc transpose data = vs out = vs(drop = _name_);
         by usubjid;
         id vstestcd;
         idlabel vstest;
         var value;
      run;     
   %end;
%end;

%if %length(&htwtfl) or %length(&initfl) %then %do;
   data subject;
      merge subject(in = ina) &htwtfl &initfl;
      by usubjid;
      if ina;
   run;
%end;

%if %upcase(&MatchFl) = BIRTHDAY %then %do;  
   data subject;
      set subject;
      %DATE9(var=BRTHDTC,return=BRTHDT);
      label BRTHDT = "&trBirthday";
   run;
   %let MatchVar = BRTHDT;
%end;
%else %if %upcase(&MatchFl) = INITIALS %then %do;
   %let MatchVar = SUBJINIT;
%end;

proc sql noprint;
   create table match(rename = (&MatchVar = MatchVar)) as
      select distinct &MatchVar
      %if %upcase(&SexMatch) = YES %then ,sex;
      %if %upcase(&RaceMatch) = YES %then ,race;
      %if %upcase(&EthnicMatch) = YES %then ,ethnic;
      from subject
      order by &MatchVar
      %if %upcase(&SexMatch) = YES %then ,sex;
      %if %upcase(&RaceMatch) = YES %then ,race;      
      %if %upcase(&EthnicMatch) = YES %then ,ethnic;
      ;     
quit;

%if %upcase(&MatchFl) = BIRTHDAY %then %do;
   proc sql noprint;
      create table subject2 as
         select *
         from match as aa, subject as bb  
         where not missing(aa.MatchVar) and not missing(bb.&MatchVar) and abs(aa.MatchVar - bb.&MatchVar) <= &BDWindow  
         %if %upcase(&SexMatch) = YES %then %do; and aa.sex = bb.sex %end;
         %if %upcase(&RaceMatch) = YES %then %do; and aa.race = bb.race %end;
         %if %upcase(&EthnicMatch) = YES %then %do; and aa.ethnic = bb.ethnic %end;
         order by aa.MatchVar 
         %if %upcase(&SexMatch) = YES %then ,sex;
         %if %upcase(&RaceMatch) = YES %then ,race;
         %if %upcase(&EthnicMatch) = YES %then ,ethnic;        
         ,bb.usubjid;
      create table subject3(drop = count) as    
         select *, count(MatchVar) as count 
         from subject2 
         group by MatchVar
         %if %upcase(&SexMatch) = YES %then ,sex;
         %if %upcase(&RaceMatch) = YES %then ,race;         
         %if %upcase(&EthnicMatch) = YES %then ,ethnic;
         having count >= 2
         order by MatchVar
         %if %upcase(&SexMatch) = YES %then ,sex;
         %if %upcase(&RaceMatch) = YES %then ,race;
         %if %upcase(&EthnicMatch) = YES %then ,ethnic;        
         ,usubjid;
   quit;
   
   proc datasets nolist;
      delete subject2 subject;
      change subject3 = subject;
   quit;
%end;
%else %if %upcase(&MatchFl) = INITIALS %then %do;
   proc sql noprint;
      create table subject2 as
         select *
         from match as aa, subject as bb  
         where (substr(aa.MatchVar,1,1) = substr(bb.&MatchVar,1,1) and substr(aa.MatchVar,3,1) = substr(bb.&MatchVar,3,1))
         %if %upcase(&SexMatch) = YES %then %do; and aa.sex = bb.sex %end;
         %if %upcase(&RaceMatch) = YES %then %do; and aa.race = bb.race %end;
         %if %upcase(&EthnicMatch) = YES %then %do; and aa.ethnic = bb.ethnic %end;
         order by aa.MatchVar
         %if %upcase(&SexMatch) = YES %then ,sex;
         %if %upcase(&RaceMatch) = YES %then ,race;         
         %if %upcase(&EthnicMatch) = YES %then ,ethnic;
         ,bb.usubjid;
   quit;

   data subject2;
      set subject2;
      if upcase(substr(MatchVar,2,1)) in ("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z") then do;
         if substr(&MatchVar,2,1) in ("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z") then do;
            if substr(&MatchVar,2,1) ne substr(MatchVar,2,1) then delete;      
         end;
      end;
   run;

   proc sql noprint;
      create table subject3(drop = count) as    
         select *, count(MatchVar) as count 
         from subject2 
         group by MatchVar
         %if %upcase(&SexMatch) = YES %then ,sex;
         %if %upcase(&RaceMatch) = YES %then ,race;         
         %if %upcase(&EthnicMatch) = YES %then ,ethnic;
         having count >= 2
         order by MatchVar
         %if %upcase(&SexMatch) = YES %then ,sex;
         %if %upcase(&RaceMatch) = YES %then ,race;                  
         %if %upcase(&EthnicMatch) = YES %then ,ethnic;
         ,usubjid;
   quit;
   
   proc datasets nolist;
      delete subject2 subject;
      change subject3 = subject;
   quit;
%end;

%PrepOutDS(BirthdayInitials_OutData,OutDataName, &Outname.BrthInit);

data &BirthdayInitials_OutData;
   set subject;
   label MatchVar = "&trMVV";
run;
%NObsVars(&BirthdayInitials_OutData);
  
%let NumSubs = &nobs;
  
proc datasets nolist;
   delete subject;
quit;

%let SUBJECTLabel = USUBJID;
%CheckLabel(&BirthdayInitials_OutData,SUBJECTLabel);
%let MATCHLabel = MatchVar;
%CheckLabel(&BirthdayInitials_OutData,MATCHLabel);
%let SITElabel = SITEID;
%CheckLabel(&BirthdayInitials_OutData,SITElabel);
%if %length(&initfl) %then %do;
   %let INITLabel = SUBJINIT;
   %CheckLabel(&BirthdayInitials_OutData,INITLabel);
%end;
%let rschk = %sysfunc(open(&BirthdayInitials_OutData));
%let bchk = %sysfunc(varnum(&rschk,BRTHDTC));
%let rchk = %sysfunc(varnum(&rschk,RACE));
%let echk = %sysfunc(varnum(&rschk,ETHNIC));
%let schk = %sysfunc(varnum(&rschk,SEX));
%let hchk = %sysfunc(varnum(&rschk,HEIGHT));
%let wchk = %sysfunc(varnum(&rschk,WEIGHT));
%let rschk = %sysfunc(close(&rschk));
%if &bchk %then %do;
   %let BRTHLabel = BRTHDTC;
   %CheckLabel(&BirthdayInitials_OutData,BRTHLabel);
%end;
%if &rchk %then %do;
   %let RACELabel = RACE;
   %CheckLabel(&BirthdayInitials_OutData,RACELabel);
%end;
%if &echk %then %do;
   %let ETHNICLabel = ETHNIC;
   %CheckLabel(&BirthdayInitials_OutData,ETHNICLabel);
%end;
%if &schk %then %do;
   %let SEXLabel = SEX;
   %CheckLabel(&BirthdayInitials_OutData,SEXLabel);
%end;
%if &hchk %then %do;
   %let HTLabel = HEIGHT;
   %CheckLabel(&BirthdayInitials_OutData,HTLabel);
%end;
%if &wchk %then %do;
   %let WTLabel = WEIGHT;
   %CheckLabel(&BirthdayInitials_OutData,WTLabel);
%end;

%if %upcase(&MatchFl) = BIRTHDAY %then %let CapMatch = Birthday;
%else %let CapMatch = Initials;

%if %upcase(&SexMatch) = YES and ^%sysfunc(findw(%upcase(&ADSLVARS), SEX)) %then %let SexMatch = No;
%if %upcase(&RaceMatch) = YES and ^%sysfunc(findw(%upcase(&ADSLVARS), RACE)) %then %let RaceMatch = No;
%if %upcase(&EthnicMatch) = YES and ^%sysfunc(findw(%upcase(&ADSLVARS), ETHNIC)) %then %let EthnicMatch = No;

%let GroupVars = Column(""&MATCHLabel"");
%if %upcase(&SexMatch) = YES and &schk %then %do;
   %let GroupVars = &GroupVars , Column(""&SEXLabel"");
%end;
%if %upcase(&RaceMatch) = YES and &schk %then %do;
   %let GroupVars = &GroupVars , Column(""&RACELabel"");
%end;
%if %upcase(&EthnicMatch) = YES and &schk %then %do;
   %let GroupVars = &GroupVars , Column(""&ETHNICLabel"");
%end;
%let GroupVars = &GroupVars, Column(""&SUBJECTLabel""), Column(""&SITELabel"");
%if %length(&initfl) %then %do;
   %let GroupVars = &GroupVars , Column(""&INITLabel"");
%end;
%if &bchk %then %do;
   %let GroupVars = &GroupVars , Column(""&BRTHLabel"");
%end;
%if %upcase(&SexMatch) = NO and &schk %then %do;
   %let GroupVars = &GroupVars , Column(""&SEXLabel"");
%end;
%if %upcase(&RaceMatch) = NO and &rchk %then %do;
   %let GroupVars = &GroupVars , Column(""&RACELabel"");
%end;
%if %upcase(&EthnicMatch) = NO and &echk %then %do;
   %let GroupVars = &GroupVars , Column(""&ETHNICLabel"");
%end;
%if &hchk %then %do;
   %let GroupVars = &GroupVars , Column(""&HTLabel"");
%end;
%if &wchk %then %do;
   %let GroupVars = &GroupVars , Column(""&WTLabel"");
%end;

* create JSL file;
%PrepOutFile(BirthdayInitialsScript,BirthdayInitialsScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &BirthdayInitialsScript;
%let FullAPName=%str(Birthdays and Initials);

* put JSL statements;
data _null_;
   file "&BirthdayInitialsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   
   put "if(""&CapMatch"" == ""Birthday"", objRefNS:trCapMatch = locale:get(""Birthday_Text""),";
   put " ""&CapMatch"" == ""Initials"", objRefNS:trCapMatch = locale:get(""Initials_Text""),";
   put " objRefNS:trCapMatch = ""&CapMatch"");";

   %if &NumSubs %then %do;
      put "    objRefNS:dmd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      put "    Column(""&SubjectLabel"") << Label(1);";
   %end;
   
   put "objRefNS:ddw=Function({},{default local},";   
   %if &NumSubs %then %do;
      put "    objRefNS:DupSubs = V List Box (";
      put "    objRefNS:Blue=textbox(locale:get(""DependingOnBWorMISubjsCanAppMoreOneMatchBin_Text""),<<Set Wrap(900),<< Font Color(5)),";
      %if %length(&MatchAlso) %then %do;
         put "    objRefNS:Two=textbox(locale:get(""Alsomatchingusing_Text"")||"" &MatchAlso"",<<Set Wrap(900),<< Font Color(5)),";
      %end;
      put "    textbox(""""),";
      put "    H List Box(";
      put "    objRefNS:tab=Tabulate(";
      put "       Show Control Panel( 0 ),";
      put "       Include missing for grouping columns( 1 ),";
      put "       Add Table(";
      put "          Row Table(";
      put "             Grouping Columns(";
      put "              &GroupVars";
      put "             )";
      put "          )";
      put "       ),";
      put "    SendToReport(";
      put "       Dispatch(";
      put "          {},";
      put "          ""Tabulate"",";
      put "          OutlineBox,";
      put "          {Set Title( locale:fill(locale:get(""ListofSubjectswithDuplicatesforX_Text""),{objRefNS:trCapMatch}) )}";
      put "       )";
      put "    )";
      put "    )";
      put "    ))";
   %end;
   %else %do;
      put "    objRefNS:DupSubs = V List Box (";
      put "       objRefNS:Blue=textbox(locale:fill(locale:get(""TherearenoduplicatesubjectsbasedonX_Text""), {objRefNS:trCapMatch}),<<Set Wrap(900),<< Font Color(5)),";
      put "                   jg:messageHelp(""TherearenoduplicatesubjectsbasedonX_Text"")";
      put "    );";
   %end;
   put ");";

   %if &NumSubs %then %do;   
      put "    objRefNS:CustomPanel = Expr(objRefNS:Filter = objRefNS:dmd << Data Filter(<<Conditional(1),";
      put "       Add Filter(Columns(""&Sitelabel"",""&MATCHLabel"")),";
      put "       Mode( Show(1), Include(1) )";
      put "    ));";   
   %end;
   
   %if &NumSubs %then %do;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trBirthdaysandInitials""}});";
   %end;
   %else %do;
      put "objRefNS:ShowDataTables = 0;";
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
   %end;
   put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""ListofSubjectswithDuplicatesforX_Text""),{objRefNS:trCapMatch})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddw()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   
   put "objRefNS:fpdd = evalexpr(Function({},{default local},  this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); reviewRefNS = namespace(reportRefNS:getParent()); createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< newContext); createFilterFunc(reviewRefNS << getName);));";  
   put "objRefNS:cpdd = evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); clusterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""); clusterFunc(this:getParent());));";
   put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   put "objRefNS:spdd =  Function({}, {},";
   put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";  
   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";

   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectSubjectsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ppdd()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ClusterSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:cpdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""CreateSubjectFilter_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:fpdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dmd""}});";

   put "objRefNS:ActionSelectionCountList = {{1,1,3,1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"",""&IconPath.HierarchicalClustering.gif"",""&IconPath.AddFilter.gif""}};"; 
   put "objRefNS:ActionRequirementFlagList = {{3, 0, 3, 2}};";
   
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:Blue,objRefNS:tab}});";
   %if %length(&MatchAlso) %then %do;
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:Two}});"; 
   %end; 

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &BirthdayInitialsScriptName, &OutDataName..sas7bdat, text/plain);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend BirthdayInitials;
%BirthdayInitials;

************************************;
************************************;
************************************;