/*
JMP Clinical Analytical Process

Name: ClusterSubjectsAcrossSite
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

Performs Cluster subjects across study sites within race, gender and country (if requested).
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName ClusterSubAcrossScript ClusterSubAcross_OutData OutPath 
        ServerOutPath PopulationSet MissMax FindDistFl InterDistFl EventDistFl WhereStmt WhereForReport DelMissFl HierClustMethod ClustSexFl 
        ClustRaceFl ClustCountryFl AgeSexFl ResType FindTests AllFind SubgroupCount EventIntInc;

*ProcessBody;

%macro ClusterSubjectsAcrossSite;

/*    
$CVS_START$
$ID: Requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and (DM.RFSTDTC or DM.DMDTC) and ((FINDINGS.USUBJID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTESTCD and FINDINGS.xxTEST and FINDINGS.VISITNUM) or
          (EVENTS.USUBJID and EVENTS.xxDECOD) or (INTERVENTIONS.USUBJID and INTERVENTIONS.xxDECOD))$ $NOTES: RFSTDTC or DMDTC to determine duration of trial.$ 
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: CPV.{"Age and Sex","Yes"}$ $UE: DM.AGE and DM.SEX $
$CMP: CPV.{"Sex","Yes"}$ $UE: DM.SEX $
$CMP: CPV.{"Race","Yes"}$ $UE: DM.RACE $
$CMP: CPV.{"Country","Yes"}$ $UE: DM.COUNTRY $
$CMP: OOUT$ $UE: VS.VSTESTCD and (VS.VSSTRESN or VS.VSORRES)$ $NOTES: VSTESTCD = HEIGHT or WEIGHT to summarize differences between subjects. Depends on dialog option$ 
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$ 
$CVS_STOP$

$CVS_START$
$ID: Optional Findings$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"PreTreatmentFindingsO"}}$
$CMP: CPV.{"Findings domains","Yes"}$ $UE: FINDINGS.USUBJID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTESTCD and FINDINGS.xxTEST and FINDINGS.VISITNUM$ $NOTES: Only domains with these criteria are used$
$CMP: OSYS$ $UE: FINDINGS.VISIT$ $NOTES: To remove UNSCHEDULED visits, if available$ 
$CMP: OSYS$ $UE: FINDINGS.xxDTC$ $NOTES: Will be used to limit findings to pretreatment, if available.$ 
$CMP: OSYS$ $UE: FINDINGS.xxTPTNUM$ $NOTES: Will be used to get findings by time point also, if available.$ 
$CMP: CPV.{"Analyze findings using:","Numeric results in standard units"}$ $UE: FINDINGS.xxSTRESN$ $NOTES: Only domains with these criteria are used$
$CMP: CPV.{"Analyze findings using:","Results in original units"}$ $UE: FINDINGS.xxORRES$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$

$CVS_START$
$ID: Optional Events$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"PreTreatmentEventsO","EventOccur"}}$
$CMP: CPV.{"Event domains","Yes"}$ $UE: EVENTS.USUBJID and EVENTS.xxDECOD$ $NOTES: Only domains with these criteria are used$
$CMP: OSYS$ $UE: EVENTS.xxSTDTC$ $NOTES: Will be used to limit events to pretreatment, if available.$ 
$CVS_STOP$

$CVS_START$
$ID: Optional Interventions$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"PreTreatmentInterventionsO","InterventionOccur"}}$
$CMP: CPV.{"Intervention domains","Yes"}$ $UE: INTERVENTIONS.USUBJID and INTERVENTIONS.xxDECOD$ $NOTES: Only domains with these criteria are used$
$CMP: OSYS$ $UE: INTERVENTIONS.xxSTDTC$ $NOTES: Will be used to limit interventions to pretreatment, if available.$ 
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = ClusterSubjectsAcrossSite; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds =;
%OUTNAMETRUNC(outdat = ClusterSubAcross_OutData, truncnum = 14);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);
%if &AgeSexFl = No and &EventDistFl = No and &InterDistFl = No and &FindDistFl = No %then %do;
   %put [MS_S_0056];
   %put ERROR: At least one set of variables must be selected for analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

%if &SubgroupCount = 1 %then %do;
   %put ERROR: At least 2 subjects are required to cluster within subgroup.;
   %let exiterror = 1;
%end;

%CHECKPRES(domain = ADSL, var = AGE);
%CHECKPRES(domain = ADSL, var = SEX);
%CHECKPRES(domain = ADSL, var = RACE);

%let trSite = %nrbquote(%LocaleGet(Site_Text));
%let trBetweenSubjectDistancesbyCov = %nrbquote(%LocaleGet(BetweenSubjectDistancesbyCovariateSubgroup_Text));
%let trMinimumBetweenSubDistCov = %nrbquote(%LocaleGet(MinimumBetweenSubjectDistancesforeachCovSubgrp_Text));
%let trBetweenSubjectDistances = %nrbquote(%LocaleGet(BetweenSubjectDistances_Text));
%let trDistanceMatrix = %nrbquote(%LocaleGet(DistanceMatrix_Text));

%if %upcase(&ClustSexFl) = NO and %upcase(&ClustRaceFl) = NO and %upcase(&ClustCountryFl) = NO %then %let OverAll = YES;
%else %let OverAll = NO;

%let yeardiff =;
%let yeardifffl =;
%if %sysfunc(findw(&domains, DM, ' ')) %then %do;
   %LOADDATA(domlist = DM,mergesupp = 0,checkreview=0);

   %if %sysfunc(findw(%upcase(&DMVARS), RFSTDTC, ' ')) %then %let flagdate = RFSTDTC;
   %else %if %sysfunc(findw(%upcase(&DMVARS), DMDTC, ' ')) %then %let flagdate = DMDTC;
   %else %let flagdate = ;

   %if %length(&flagdate) %then %do;
      data dm2(keep = &flagdate);
         set &in_dm;
      run;

      proc sort nodupkey data = dm2 out = dm2;
         by &flagdate;
      run;
      
      data dm2(keep = first last);
         set dm2(where = (not missing(&flagdate))) end = eof;
         retain first last;
         if _n_ = 1 then first = &flagdate;
         if eof then last = &flagdate;
         if eof;
      run;
      
      data dm2;
         set dm2;
         yeardiff = 1 + substr(last,1,4) - substr(first,1,4);
         call symput("yeardiff",trim(left(yeardiff)));
         if yeardiff = 1 then call symput("yeardifffl","&tryear");
         else call symput("yeardifffl","&tryears");
      run;

      proc datasets nolist;
         delete dm2;
      quit;   
   %end;
%end;

%let _tid_ = %sysfunc(open(&in_adsl));
%let vn = %sysfunc(varnum(&_tid_,USUBJID));
%let USubLen = %sysfunc(varlength(&_tid_,&vn));
%let sn = %sysfunc(varnum(&_tid_,SITEID));
%let SiteIDLen = %sysfunc(varlength(&_tid_,&sn));
%let _tid_ = %sysfunc(close(&_tid_));

%if ^%length(&findingsdomains) and %upcase(&FindDistFl) = YES %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&interventionsdomains) and %upcase(&InterDistFl) = YES %then %do;
   %put [MS_S_0058];
   %put ERROR: No Interventions domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&eventsdomains) and %upcase(&EventDistFl) = YES %then %do;
   %put [MS_S_0059];
   %put ERROR: No Events domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM, ' ')) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID, ' ')) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA(varlist = arm &AGE &SEX &RACE);
%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);

data subject;
   set subject;
   int = 1;
run;

proc means sum nway noprint data = subject;
   class 
      %if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
      %if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
      %if %upcase(&ClustCountryFl) = YES %then %do; country %end;
   ;
   var int;
   output out = covgrp(drop = _TYPE_ _FREQ_) sum = count;
run;

proc sort data = covgrp;
   by descending count;
run;

data covgrp;
   set covgrp;
   covid = catx(", ", 
      %if &Overall = NO %then %do;
         %if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; catx(" ","&trSex =",sex), %end;
         %if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; catx(" ","&trRace =",race), %end;       
         %if %upcase(&ClustCountryFl) = YES %then %do; catx(" ","&trCountry =",country), %end;       
         " "
      %end;
      %else %do;
         "&trAllSubjects"
      %end;
   );
   label covid = "&trCovariateGroupID";
run;

%if &Overall = NO %then %do;
   proc sort data = subject;
      by 
         %if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
         %if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
         %if %upcase(&ClustCountryFl) = YES %then %do; country %end;
      ;
   run;

   proc sort data = covgrp;
      by 
         %if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
         %if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
         %if %upcase(&ClustCountryFl) = YES %then %do; country %end;
      ;
   run;

   data subject;
      merge subject covgrp(drop = count);
      by
         %if %upcase(&ClustSexFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %do; sex %end;
         %if %upcase(&ClustRaceFl) = YES and %sysfunc(findw(%upcase(&ADSLVARS), RACE, ' ')) %then %do; race %end;
         %if %upcase(&ClustCountryFl) = YES %then %do; country %end;   
      ;
   run;

   proc sort data = subject out = subject(drop = int);
      by usubjid;
   run;
%end;
%else %do;
   data subject(drop = int);
      set subject;
      if _n_ = 1 then set covgrp(drop = count);
   run;
%end;

%let htwtobs = 0;
%let heightflag = 0;
%let weightflag = 0;
%if %sysfunc(findw(&domains, VS, ' ')) %then %do;
   %let temp = VS;
   
   %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %let variable = &temp.STRESN;
   %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %let variable = &temp.ORRES;
   
   %if %sysfunc(indexw(&&&temp.VARS, &variable, ' ')) %then %do;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.DTC, ' ')) %then %let dtc = &temp.DTC;
      %else %let dtc = ;

      %if %sysfunc(indexw(&&&temp.VARS, &temp.test, ' ')) %then %let testlab = &temp.test;
      %else %let testlab = ;

      %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
      proc sort data = &&in_&temp(where = (not missing (&variable))) out = &temp.2(keep = usubjid &temp.testcd &testlab &variable &dtc);
         by usubjid &temp.testcd &dtc;
      run;
      
      %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;                  
         %let ORRESLabel = &variable;
         %CheckLabel(&temp,ORRESLabel);

         data &temp.2(where = (not missing (&variable)));
            set &temp.2(rename = (&variable = cval));
            &variable = input(cval,best12.);
            drop cval;
            label &variable = "&ORRESLabel";
         run;     
      %end;      

      *** Delete unscheduled records ***;
      %if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
         data &temp.2(drop = visit);
            set &temp.2;
            if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
         run;
      %end;

      *** if possible, limit measurements to those prior to dosing ***;
      %if %length(&dtc) %then %do;
         %TREATMENTEMERGENT(dataset = &temp.2, domain = &temp, date = &temp.DTC);

         data &temp.2(drop = &dtc _TRTEMRG);
            set &temp.2;
            where _TRTEMRG = 0;
         run;
      %end;

      ods exclude all;
      ods noresults;
      proc means noprint nway missing data = &temp.2;
         class usubjid &temp.testcd &testlab;
         var &variable;
         output out = &temp.2(drop = _type_ _freq_) mean = &variable;
         where upcase(&temp.testcd) in ("HEIGHT","WEIGHT");
      run;
      ods select all;
      ods results;      

      proc transpose data = &temp.2 out = htwt(drop = _name_ _label_);
         by usubjid;
         var &variable;
         id &temp.testcd;
         %if %length(&testlab) %then %do;
            idlabel &testlab;
         %end;
      run;     
      %NObsVars(htwt);
      %let htwtobs = &nobs;

      proc datasets nolist;
         delete &temp.2;
      quit;    
   %end;
%end;

%if &htwtobs %then %do;
   data subject;
      merge subject(in = ina) htwt;
      by usubjid;
      if ina;
   run;

   %let _tid_ = %sysfunc(open(subject));
   %let HeightFlag = %sysfunc(varnum(&_tid_,HEIGHT));
   %let WeightFlag = %sysfunc(varnum(&_tid_,WEIGHT));
   %let _tid_ = %sysfunc(close(&_tid_));  
%end;

proc sql noprint;
   create table subpairs as
      select aa.usubjid, bb.usubjid2, aa.covid, aa.siteid, bb.siteid2 %if %length(&AGE) %then %do; , aa.age, bb.age2 %end; 
             %if &heightflag %then %do; , height , height2 %end; %if &weightflag %then %do; , weight ,weight2 %end;
      from subject as aa, subject(rename = (usubjid = USUBJID2 covid = covid2 siteid = siteid2 %if %length(&AGE) %then %do; age = age2 %end; 
             %if &heightflag %then %do; height = height2 %end; %if &weightflag %then %do; weight = weight2 %end;)) as bb   
      where aa.usubjid < bb.usubjid2 and covid = covid2
      order by aa.usubjid, bb.usubjid2;
quit;

data subpairs;
   retain USUBJID siteid USUBJID2 siteid2 covid siteflag %if %length(&AGE) %then %do; agediff %end; %if &heightflag %then %do; heightdiff %end; %if &weightflag %then %do; weightdiff %end;;
   length siteflag $ 20;
   set subpairs;
   if siteid = siteid2 then siteflag = "&trWithin";
   else siteflag = "&trBetween";
   label siteflag = "&trSiteFlag";
   *drop siteid siteid2;
   %if %length(&AGE) %then %do;
      agediff = abs(age-age2);
      drop age age2;
      label agediff = "&trAgeDifference";
   %end;   
   %if &heightflag %then %do; 
      heightdiff = abs(height-height2);
      drop height height2;
      label heightdiff = "&trHeightDifference";   
   %end; 
   %if &weightflag %then %do; 
      weightdiff = abs(weight-weight2);
      drop weight weight2;
      label weightdiff = "&trWeightDifference";      
   %end;
   label siteid = "&SITEIDlabel" siteid2 = "&SITEIDlabel 2";
run;

data subject;
   set subject;
   %if &heightflag %then %do; 
      drop height;
   %end; 
   %if &weightflag %then %do; 
      drop weight;
   %end;   
run;

%let availabledomains = ;
%if &FindDistFl = Yes and %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &findingsdomains, flist = &FindTests);
%end;
%GETFINDINGS(predose = 1);
%if &EventDistFl = Yes %then %do;
   %INTEVENT(list = &eventsdomains, predose = 1);
%end;
%if &InterDistFl = Yes %then %do;
   %INTEVENT(list = &interventionsdomains, predose = 1); 
%end;

%if ^%length(&availabledomains) and ((^%length(&AGE) and ^%length(&SEX) and &AgeSexFl = Yes) or &AgeSexFl = No) %then %do;
   %put ERROR: No domains meet the necessary criteria for the analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

data all;
   merge subject(in = ina) &availabledomains;
   by usubjid;
   if ina;
   %if %length(&SEX) %then %do;
      if sex = "M" or sex = "&trM" then MALE = 1;
      else if sex = "F" or sex = "&trF" then MALE = 0;
      drop sex;
      label male = "&trMaleGender";
   %end;
   %if &AgeSexFl = No %then %do;
      %if %length(&AGE) %then %do;
         drop age;
      %end;
      %if %length(&SEX) %then %do;
         drop male;
      %end;
   %end;
run;
%NObsVars(all);
%let TotalSubs = &nobs;

proc means nmiss noprint data = all;
   output out = nmiss(drop = _type_ _freq_) nmiss = / autoname;
run;

proc transpose data = nmiss out = nmiss;
run;

data nmiss;
   set nmiss(rename = (col1 = nummiss) where = (_name_ ne "SiteSamp_NMiss")) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 6);
   pctmiss = round(nummiss / &TotalSubs * 100, 0.01);
   %if %upcase(&DelMissFl) = YES %then %do; 
      if pctmiss > 0 then missflag = 2;
   %end;
   %else %do;
      if pctmiss >= &MissMax then missflag = 2;
   %end;
   else if pctmiss ne 0 then missflag = 1;
   else if pctmiss = 0 then missflag = 0;
   if eof then call symput("TotalVars", trim(left(_n_)));
   label _name_ = "&trVariable" _label_ = "&trLabel" nummiss = "&trNOSMV" pctmiss = "&trPOSMV" missflag = "&trMissFlag";
run;

%let DegFree = 0;
*** degree of freeom is number of variables contributing to maha scores;
data _null_;
   set nmiss(where = (missflag in (0,1))) end = eof;
   if eof then call symput("DegFree", trim(left(_n_)));
run;

%let RemovedVars = ;
data _null_;
   set nmiss(where = (missflag = 2)) end = eof;
   if eof then call symput("RemovedVars", trim(left(_n_)));
run;

%if ^%length(&RemovedVars) %then %let RemovedVars = 0;
%if ^%eval(&TotalVars - &RemovedVars) %then %do;
   %put [MS_S_0060];
   %put ERROR: No variables remain for analysis after deleting variables with at least &MissMax percent missing data.;
   %let exiterror = 1;
   %goto exit;
%end;

%let MissMaxVars = ;
%let MissMaxLabels = ;
%let MissLabels = ;
proc sql noprint;
   select _name_ 
      into :MissMaxVars separated by ' '
      from nmiss
      where missflag = 2
      order by _name_;
quit;

data all;
   set all(drop = &MissMaxVars);
   subject = cats("S",_n_);
run;

%let ClusterSubAcross_OutData_sum =;
%PrepOutDS(ClusterSubAcross_OutData_sum,OutDataName_sum, &Outname.csass_sum);
%let includeds = &includeds &OutDataName_sum..sas7bdat,;
   
data &ClusterSubAcross_OutData_sum;
   set all(drop = subject) end = eof;
run;

%let SiteList = ;
proc sql noprint;
   select distinct covid 
      into :SiteList separated by '~'
      from covgrp(where = (count >= &SubgroupCount))
      order by covid;
quit;

%if ^%length(&SiteList) %then %do;
   %put ERROR: There are no subgroups available. Analysis cannot be performed.;
   %put ERROR: Try relaxing the requirement on the minimum number of subjects per subgroup.;
   %let exiterror = 1;
   %goto exit;
%end;

proc means var nway noprint data = all;
   class covid;
   output out = novar(drop = _type_ _freq_) var = / autoname;
run;

proc transpose data = novar out = novar;
   by covid;
run;

proc sql noprint;
   create table SiteVarCountList as
   select distinct covid, count(covid) as countvars 
      from novar
      group by covid
      order by covid;
quit;

data novar;
   set novar(rename = (col1 = var) where = (var in (.,0))) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 4);
run;

proc sql noprint;
   create table SiteNoVarCountList as
   select distinct covid, count(covid) as countnovar
      from novar
      group by covid
      order by covid;
quit;

data SiteVarCountList;
    merge SiteVarCountList SiteNoVarCountList;
    by covid;
run;

proc datasets nolist;
   delete SiteNoVarCountList;
quit;

proc sort data = all;
   by subject;
run;


%let SitesNoVariability =;
%let firstflag = 0;      

%let i = 1;
%do %while(%length(%scan(%nrbquote(&SiteList),&i,'~')));
   %let tempsite = %scan(%nrbquote(&SiteList),&i,'~');   
   %let tempsiten = &i; 
   %let SiteConstantVars = ;  
   proc sql noprint;
      select _name_ 
         into :SiteConstantVars separated by ' '
         from novar
         where covid = "&tempsite"
         order by _name_;
   quit;

   %let varok = 1;
   data _null_;
      set SiteVarCountList(where = (covid = "&tempsite"));
      if countvars = countnovar then do;
         call symput("varok",trim(left(0)));
      end;
   run;
   
   %if ^&varok %then %let SitesNoVariability = &SitesNoVariability~&tempsite;
    
   %if &varok %then %do;   
      *** Defect S1325220 ***;
      proc distance data = all(drop = &SiteConstantVars) shape = square method = euclid out = distance;
         var interval(_numeric_ / std = std);
         *copy usubjid covid siteid arm country;
         id subject;
         where covid = "&tempsite";
      run;
            
      proc sort data = distance;
         by subject;
      run;
      
      data distance;
         merge distance(in = ina) all(keep = usubjid covid subject siteid arm country);
         by subject;
         if ina;
      run;
      
      proc sort data = distance;
         by usubjid subject covid siteid arm country;
      run;          

      /*
      proc sql noprint;
         select usubjid 
            into :SiteClusterVars separated by ' '
            from distance
            order by usubjid;
      quit;

      %let SiteClusterVars&tempsiten = ;

      %let j = 1;
      %do %while(%length(%scan(&SiteClusterVars,&j,' ')));
         %let temp = %scan(%upcase(&SiteClusterVars),&j," ");      
         %let SiteClusterVars&tempsiten = &&SiteClusterVars&tempsiten %bquote(:Name(""&temp"")),;
         %let j = %eval(&j+1);
      %end; */

      proc transpose data = distance out = distance;
         by usubjid subject covid siteid arm country;
      run;

      proc sql noprint;
         create table siteids as
            select distinct subnum, usubjid2
            from all(rename = (subject = subnum usubjid = USUBJID2))
            order by subnum;     
         create table distance2 as
            select * 
            from distance as aa left join siteids as bb
            on aa._name_ = bb.subnum
            order by usubjid, usubjid2;
      quit;

      proc datasets nolist;
         delete siteids distance;
         change distance2 = distance;
      quit;

      proc sql noprint;
         create table distancepairs(drop = subnum subject _name_ rename = (col1 = Distance)) as
            select *, min(col1) as mindist
            from subpairs(where = (covid = "&tempsite")) natural left join distance
            order by usubjid, usubjid2;      
      quit;

      data mindisttemp(drop = mindist);
         set distancepairs(where = (distance = mindist));
         if _n_ = 1;
      run;

      %if ^&firstflag %then %do;
         data mindist;
            set mindisttemp;
         run;
      %end;
      %else %do;   
         proc append force base = mindist data = mindisttemp;
         run;
      %end;

      data distancepairs;
         set distancepairs(drop = mindist);
         label distance = "&trDistance" usubjid = "&SUBJECTlabel" usubjid2 = "&SUBJECTlabel 2" covid = "&trCovariateGroupID";
      run;

      %if ^&firstflag %then %do;
         data alldistancepairs;
            set distancepairs;
         run;
      %end;
      %else %do;   
         proc append force base = alldistancepairs data = distancepairs;
         run;
      %end;

      %if ^&firstflag %then %let firstflag = 1;

      %if &i = 1 %then %do;   
         %let SUBJECTLabel2 = USUBJID2;
         %CheckLabel(distancepairs,SUBJECTLabel2);
      %end;

      proc transpose data = distance out = distance(drop = _name_);
         var col1;
         by usubjid siteid arm country;
         id _name_;
         idlabel usubjid2;
         format usubjid2;
      run;

      %let ClusterSubAcross_OutData_&tempsiten = ;
      %let ClusterSubAcross_OutDataP_&tempsiten = ;
      %PrepOutDS(ClusterSubAcross_OutData_&tempsiten,OutDataName_&tempsiten, &Outname.csass, _&tempsiten);
      %PrepOutDS(ClusterSubAcross_OutDataP_&tempsiten,OutDataNameP_&tempsiten, &Outname.csass_p, _&tempsiten);

      %let includeds = &includeds &&&OutDataName_&tempsiten...sas7bdat, &&&OutDataNameP_&tempsiten...sas7bdat,;

      data &&&ClusterSubAcross_OutData_&tempsiten;
         set distance;
         drop siteid;
         label usubjid = "&SUBJECTLabel";
      run;

      data &&&ClusterSubAcross_OutDataP_&tempsiten;
         set distancepairs;
         label usubjid = "&SUBJECTLabel" siteid = "&SITEIDlabel";
      run;

      proc datasets nolist;
         delete distance distancepairs mindisttemp;    
      quit;
   %end; 
   %let i = %eval(&i+1);
%end; 

%if %sysfunc(exist(mindist)) %then %do;
   proc sort data = mindist;
      by distance;
   run;

   %let ClusterSubAcross_OutData_min = ;     
   %PrepOutDS(ClusterSubAcross_OutData_min,OutDataName_min, &Outname.csass,_mindist);
   %let includeds = &includeds &OutDataName_min..sas7bdat,;

   data &ClusterSubAcross_OutData_min;
      set mindist end = eof;
      if _n_ = 1 then call symput("MinDistSite", trim(covid));
      label distance = "&trDistance" usubjid = "&SUBJECTlabel" usubjid2 = "&SUBJECTlabel 2" covid = "&trCovariateGroupID";
   run;
%end;

%if %sysfunc(exist(alldistancepairs)) %then %do;
   %let ClusterSubAcross_OutData_all = ;     
   %PrepOutDS(ClusterSubAcross_OutData_all,OutDataName_all,&Outname.csass,_alldist);
   %let includeds = &includeds &OutDataName_all..sas7bdat,;

   data &ClusterSubAcross_OutData_all;
      set alldistancepairs end = eof;
      label siteid = "&SITEIDlabel";
   run;

   proc datasets nolist;
      delete alldistancepairs;
   quit;
%end;

%let COVIDlabel = COVID;
%CheckLabel(all,COVIDlabel);

proc datasets nolist;
   delete all;
quit;    

* create JSL file;
%PrepOutFile(ClusterSubAcrossScript,ClusterSubAcrossScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ClusterSubAcrossScript;
%let FullAPName=%str(Cluster Subjects Across Study Sites);

%let FilterVars =;
%if %length(&AGE) %then %do;
   %let FilterVars = &FilterVars.:&trAgeDifference,;
%end;
%if &heightflag %then %do;
   %let FilterVars = &FilterVars.:&trHeightDifference,;
%end;
%if &weightflag %then %do;
   %let FilterVars = &FilterVars.:&trWeightDifference,;
%end;

* put JSL statements;
data _null_;
   file "&ClusterSubAcrossScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:TabsSectionFlag = 1;";

   put "objRefNS:Select=Function({},{default local},";
   put "ctable = Current Data Table();";
   put "If(Is Empty(ctable),";
   put "   New Window(locale:get(""JMPClinicalMessage_Text""), showmenu(0),showtoolbars(0),";
   put "      tb = TextBox(locale:get(""ErrorToRunOpenInFocus_Text"")),";
   put "      tb << Font Color(3),";
   put "             jg:messageHelp(""ErrorToRunOpenInFocus_Text"")";
   put "   );";
   put "   Throw()";
   put ");";
   put "subname = ""&SUBJECTLabel"";";
   put "subname2 = ""&SUBJECTLabel 2"";";
   put "foundit = 1;";
   put "foundit2 = 1;";
   put "try(scol = Column(subname),foundit = 0);";
   put "try(scol2 = Column(subname2),foundit2 = 0);";
   put "selsub = {};";
   put "nsel = 0;";
   put " for each row(";
   put "    if(selected(rowstate())==1,";
   put "       if(!Contains(selsub,scol()[]),";
   put "          nsel++;";
   put "          InsertInto(selsub,scol()[])";
   put "       );";
   put "       if(foundit2 == 1,";
   put "          if(!Contains(selsub,scol2()[]),";
   put "             nsel++;";
   put "             InsertInto(selsub,scol2()[])";
   put "          );";
   put "       );";
   put " );";     
   put ");";
   put "Try(objRefNS:alldat = open(""&ClientOutPath.&OutDataName_sum..sas7bdat"", invisible);";
   put "   objRefNS:alldat << Select Where( Contains(selsub, Column(subname)[]));";
   put "   Current Data Table(objRefNS:alldat);";
   put " );";
   put ");";

   %if %sysfunc(exist(mindist)) %then %do;   
      put "objRefNS:fbox0=Function({},{default local},";
      put "objRefNS:mindist = open(""&ClientOutPath.&OutDataName_min..sas7bdat"",invisible);";
      put "objRefNS:alldist = open(""&ClientOutPath.&OutDataName_all..sas7bdat"",invisible);";
      put "Column(objRefNS:mindist, ""&COVIDlabel"") << Label(1);";
      put "Column(objRefNS:alldist, ""&SubjectLabel"") << Label(1);";
      put "Column(objRefNS:alldist, ""&SubjectLabel2"") << Label(1);";
      put "objRefNS:alldist << Color or Mark by Column(:&trSiteFlag, Color Theme(""JMP Default""));";
      put "objRefNS:allDistgraphs=VlistBox(";
      %if %length(&yeardiff) %then %do;
         put "TextBox(locale:fill(locale:get(""EnrollmentOccOverXAPersThatEnrMultTimesMsg_Text""), {""&yeardiff &yeardifffl""}),<<Set Wrap(900),<< Font Color(5)),";
         put "TextBox(""""),";
      %end;
      put "HListBox(Data Filter Context Box(";
      put "HListBox(objRefNS:alldist << Data Filter(<<Conditional(1),local, Mode( Select( 0 ), Show( 1 ), Include( 1 ) ),Add Filter(columns( &FilterVars :&trSiteFlag,:&SITEIDlabel,:&SITEIDlabel 2))),";
      put "    Platform(objRefNS:alldist, objRefNS:G1 = Graph Builder(";
      put "       Automatic Recalc(1),";
      put "       Auto Stretching( 0 ),"; 
      put "       Show Control Panel( 0 ),";
      put "       Show Legend( 0 ),";
      put "       Variables( X( :&COVIDlabel ), Y( :&trDistance )),";
      put "       Elements(";
      put "          Box Plot(";
      put "             X,";
      put "             Y,";
      put "             Legend( 2 ),";
      put "             Jitter( 1 ),";
      put "             Outliers( 1 ),";
      put "             Box Style( ""Solid"" )";
      put "          ),";
      put "       ),";
      put "       SendToReport(";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             OutlineBox,";
      put "             {Set Title( locale:get(""BetweenSubjectDistancesbyCovariateSubgroup_Text"") )}";
      put "          ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""&COVIDlabel"",";
      put "             ScaleBox,";
      put "             {Show Major Ticks( 0 ), Rotated Labels( ""Angled"" )}";
      put "          ),";
      put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             FrameBox,";
      put "             {Marker Size( 2 ), Transparency( 0.75 )}";
      put "          )";
      put "       )";
      put "    )))),";
      put "    objRefNS:G2 = objRefNS:mindist << Graph Builder(";
      put "       Size( 353, 497 ),";
      put "       Automatic Recalc(1),";
      put "       Auto Stretching( 0 ),"; 
      put "       Show Control Panel( 0 ),";
      put "       Show Legend( 0 ),";
      put "       Variables( Y( :&trDistance )),";
      put "       Elements(";
      put "          Box Plot(";
      put "             Y,";
      put "             Legend( 2 ),";
      put "             Jitter( 1 ),";
      put "             Outliers( 0 ),";
      put "             Box Style( ""Outlier"" )";
      put "          ),";
      put "          Points( Y,Legend( 3 ), Jitter( 1 ) )";
      put "       ),";
      put "       SendToReport(";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             OutlineBox,";
      put "             {Set Title( locale:get(""MinimumBetweenSubjectDistancesforeachCovSubgrp_Text"") )}";
      put "          ),";
      put "          Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
      put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             FrameBox,";
      put "             {Marker Size( 2 ), Transparency( 0.75 )}";
      put "          )";
      put "       )";
      put "    ),";     
      put ");";
      put "));";
      put "InsertInto(objRefNS:DataList,{{""objRefNS:mindist"",""objRefNS:alldist""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trMinimumBetweenSubDistCov"",""&trBetweenSubjectDistancesbyCov""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""BetweenSubjectDistanceSummary_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox0()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:G1, objRefNS:G2}});";
   %end;

   %let i = 1;
   %let distdatlist = ;
   %let pairdatlist = ;
   %let firstdomdat = 0;    
   %let tFlag = 0;
   %do %while(%length(%scan(%nrbquote(&SiteList),&i,'~')));
      %let tempsite = %scan(%nrbquote(&SiteList),&i,'~');   
      %let tempsiten = &i; 
      %if ^%index(%bquote(&SitesNoVariability), %bquote(&tempsite)) %then %do;
         put "objRefNS:fbox&tempsiten=Function({},{default local},";
         put "objRefNS:ddist&tempsiten = open(""&ClientOutPath.&&&OutDataName_&tempsiten...sas7bdat"",invisible);";
         put "objRefNS:dpair&tempsiten = open(""&ClientOutPath.&&&OutDataNameP_&tempsiten...sas7bdat"",invisible);"; 
         put "objRefNS:dpair&tempsiten << Color or Mark by Column(:&trSiteFlag, Color Theme(""JMP Default""));";
         put "Column(objRefNS:dpair&tempsiten, ""&SubjectLabel"") << Label(1);";
         put "Column(objRefNS:dpair&tempsiten, ""&SubjectLabel2"") << Label(1);";
         
         %if ^&firstdomdat %then %do;
            %let distdatlist = ""objRefNS:ddist&tempsiten"";
            %let pairdatlist = ""objRefNS:dpair&tempsiten"";
            %let firstdomdat = 1;
         %end;
         %else %do;
            %let distdatlist = &distdatlist,""objRefNS:ddist&tempsiten"";            
            %let pairdatlist = &pairdatlist,""objRefNS:dpair&tempsiten"";            
         %end;
         
         put "objRefNS:Dm&tempsiten=VListBox(";    
         %if %quote(&tempsite) = %quote(&MinDistSite) %then %do;        
            put "textbox(locale:get(""ThisCovSubgrpHasTheMinBetwSubjDist_Text""),<<Set Wrap(900),<< Font Color(5)),"; 
            put "textbox("""",<<Set Wrap(900),<< Font Color(5)),"; 
         %end; 
         put "HListBox(Data Filter Context Box(";
         put "HListBox(objRefNS:dpair&tempsiten << Data Filter(<<Conditional(1),local, Mode( Select( 0 ), Show( 1 ), Include( 1 ) ),Add Filter(columns( &FilterVars :&trSiteFlag,:&SITEIDlabel,:&SITEIDlabel 2))),";
         put "    Platform(objRefNS:dpair&tempsiten, objRefNS:G1_&i = Graph Builder(";
         put "       Size( 353, 497 ),";
         put "       Automatic Recalc(1),";
         put "       Auto Stretching( 0 ),";      
         put "       Show Control Panel( 0 ),";
         put "       Show Legend( 0 ),";
         put "       Variables( Y( :&trDistance )),";
         put "       Elements(";
         put "          Box Plot(";
         put "             Y,";
         put "             Legend( 2 ),";
         put "             Jitter( 1 ),";
         put "             Outliers( 0 ),";
         put "             Box Style( ""Outlier"" )";
         put "          ),";
         put "          Points( Y, Legend( 3 ), Jitter( 1 ) )";
         put "       ),";
         put "       SendToReport(";
         put "          Dispatch(";
         put "             {},";
         put "             ""Graph Builder"",";
         put "             OutlineBox,";
         put "             {Set Title( locale:get(""BetweenSubjectDistances_Text"") )}";
         put "          ),";
         put "          Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
         put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
         put "          Dispatch(";
         put "             {},";
         put "             ""Graph Builder"",";
         put "             FrameBox,";
         put "             {Marker Size( 2 ), Transparency( 0.75 )}";
         put "          )";
         put "       )";
         put "    )))),";
         put "    SiteClusterVars = objRefNS:ddist&tempsiten << Get Column Names(Numeric, string);";
         put "    Eval(Parse(EvalInsert(""\[objRefNS:G2_&i = objRefNS:ddist&tempsiten << Hierarchical Cluster(";
         put "       Hide Distance Messages(1),";
         put "       Automatic Recalc(1),";
         put "       Y(^SiteClusterVars^),";
         put "       Label( :&SUBJECTLabel ),";
         put "       Method( ""&HierClustMethod"" ),";
         put "       Standardize( 0 ),";
         put "       Distance Matrix( 1 ),";
         put "       Color Clusters( 1 ),";
         put "       Distance Graph( 0 ),";
         put "       Legend( 1 ),";
         put "       Dendrogram Scale( ""Distance Scale"" ),";
         put "       Two Way Clustering,";
         put "       Color Map( {"""", {{255, 0, 0}, {255, 255, 255}}} ),";
         put "       SendToReport(";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox,";
         put "          {Frame Size( 45, 352 )}";
         put "       ),";
         put "       Dispatch( {""Dendrogram""}, """", CellPlotBox( 2 ), {Frame Size( 349, 352 ), Color Theme({"""", {{255, 0, 0}, {255, 255, 255}}})} ),";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox( 2 ),";
         put "          {Frame Size( 200, 352 )}";
         put "       ),";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox( 3 ),";
         put "          {Frame Size( 349, 155 )}";
         put "       ),";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox( 4 ),";
         put "          {Frame Size( 349, 59 )}";
         put "       ))";           
         put "    );]\"")))";
         put ")));";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:ddist&tempsiten"", ""objRefNS:dpair&tempsiten""}});";
         put "InsertInto(objRefNS:DataNameList,{{""&tempsite - &trDistanceMatrix"",""&tempsite - &trBetweenSubjectDistances""}});";
         put "InsertInto(objRefNS:TabButtonList, expr(""&tempsite""));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox&tempsiten()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:G1_&i,objRefNS:G2_&i}});";
         
         %if %quote(&tempsite) = %quote(&MinDistSite) %then %do;
            put "InsertInto(objRefNS:TabOpenList,1);";
         %end;
         %else %do;
            put "InsertInto(objRefNS:TabOpenList,0);";
            %let tFlag = 1;
         %end;
      %end;
      %else %do;
         put "objRefNS:fbox&tempsiten=Function({},{default local},";
         put "VListBox(TextBox(locale:get(""AllVarsAvForAnHadNoVarWiThisCovGrpThisImpMsg_Text""),<<Set Wrap(900),<< Font Color(5)));"; 
         put ");";
         put "InsertInto(objRefNS:DataList,{{""""}});";
         put "InsertInto(objRefNS:DataNameList,{{""""}});";
         put "InsertInto(objRefNS:TabButtonList, expr(""&tempsite""));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox&tempsiten()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:G1_&i,objRefNS:G2_&i}});";
      %end;
      %let i = %eval(&i+1);
   %end;
   %if &tFlag=1 %then %do;
      put "objRefNS:TabsSectionFlag = 1;";
   %end;
  
   put "objRefNS:ppdd = evalexpr(Function({},{default local},";
   put "objRefNS:Select();";
   put "this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here""); Close(this:alldat, nosave);));";  
   put "objRefNS:spdd =  Function({}, {},";
   put "objRefNS:Select();";
   put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""); Close(objRefNS:alldat, nosave););";
   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";


   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ppdd()));";
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:alldat"",""objRefNS:mindist"",""objRefNS:alldist"",&pairdatlist,&distdatlist}});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:alldat"",""objRefNS:mindist"",""objRefNS:alldist"",&pairdatlist,&distdatlist}});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowRowsinHeatMap_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowRow()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&pairdatlist}});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""SubsetClusteringRows_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:SubsetClustering()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&pairdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""RevertClustering_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:RevertClustering()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&distdatlist,&pairdatlist}});";

   put "objRefNS:ActionSelectionCountList = {{1,1,1,1,1}};";   
   put "objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"",""&IconPath.ShowRowsInHeatMap.gif"",""&IconPath.SubsetClustering.gif"",""&IconPath.RevertClustering.gif""}};";
   put "objRefNS:ActionRequirementFlagList = {{3, 0, 0, 0, 0}};";
run;   
   
%let includeds = %substr(%quote(&includeds), 1, %eval(%length(&includeds) - 1));
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ClusterSubAcrossScriptName, "&includeds", yes);
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DataFromX_Text)),
   %nrbquote(%LocaleGet(&ProcessName._Label)))), outlib,  &OutDataName_sum, no);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ClusterSubjectsAcrossSite;
%ClusterSubjectsAcrossSite;

************************************;
************************************;
************************************;