/*
JMP Clinical Analytical Process

Name: ClusterSubjectsStudySite
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

Performs Cluster subjects within study sites.
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName ClusterSubjectsScript ClusterSubjectsStudySite_OutData OutPath 
        ServerOutPath PopulationSet MissMax FindDistFl InterDistFl EventDistFl WhereStmt WhereForReport DelMissFl HierClustMethod AgeSexFl ResType FindTests AllFind SiteCount EventIntInc;

*ProcessBody;

%macro ClusterSubjectsStudySite;

/*    
$CVS_START$
$ID: Requirements$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and ((FINDINGS.USUBJID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTESTCD and FINDINGS.xxTEST and FINDINGS.VISITNUM) or
          (EVENTS.USUBJID and EVENTS.xxDECOD) or (INTERVENTIONS.USUBJID and INTERVENTIONS.xxDECOD))$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: CPV.{"Age and Sex","Yes"}$ $UE: DM.AGE and DM.SEX $
$CVS_STOP$

$CVS_START$
$ID: Optional Findings$
$CMP: CPV.{"Findings domains","Yes"}$ $UE: FINDINGS.USUBJID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTESTCD and FINDINGS.xxTEST and FINDINGS.VISITNUM$ $NOTES: Only domains with these criteria are used$
$CMP: OSYS$ $UE: FINDINGS.VISIT$ $NOTES: To remove UNSCHEDULED visits, if available$ 
$CMP: OSYS$ $UE: FINDINGS.xxTPTNUM$ $NOTES: Will be used to get findings by time point also, if available.$
$CMP: CPV.{"Analyze findings using:","Numeric results in standard units"}$ $UE: FINDINGS.xxSTRESN$ $NOTES: Only domains with these criteria are used$
$CMP: CPV.{"Analyze findings using:","Results in original units"}$ $UE: FINDINGS.xxORRES$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$

$CVS_START$
$ID: Optional Events$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"EventOccur"}}$
$CMP: CPV.{"Event domains","Yes"}$ $UE: EVENTS.USUBJID and EVENTS.xxDECOD$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$

$CVS_START$
$ID: Optional Interventions$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"InterventionOccur"}}$
$CMP: CPV.{"Intervention domains","Yes"}$ $UE: INTERVENTIONS.USUBJID and INTERVENTIONS.xxDECOD$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = ClusterSubjectsStudySite; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds =;
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);

%if &AgeSexFl = No and &EventDistFl = No and &InterDistFl = No and &FindDistFl = No %then %do;
   %put [MS_S_0061];
   %put ERROR: At least one domain type must be selected for analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

%CHECKPRES(domain = ADSL, var = AGE);
%CHECKPRES(domain = ADSL, var = SEX);

%let trSite = %nrbquote(%LocaleGet(Site_Text));
%let trBetweenSubjectDistancesbySite = %nrbquote(%LocaleGet(BetweenSubjectDistancesbySite_Text));
%let trMinimumBetweenSubDist = %nrbquote(%LocaleGet(MinimumBetweenSubjectDistancesforEachSite_Text));
%let trBetweenSubjectDistances = %nrbquote(%LocaleGet(BetweenSubjectDistances_Text));
%let trDistanceMatrix = %nrbquote(%LocaleGet(DistanceMatrix_Text));

%let _tid_ = %sysfunc(open(&in_adsl));
%let vn = %sysfunc(varnum(&_tid_,USUBJID));
%let USubLen = %sysfunc(varlength(&_tid_,&vn));
%let _tid_ = %sysfunc(close(&_tid_));

%if ^%length(&findingsdomains) and %upcase(&FindDistFl) = YES %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&interventionsdomains) and %upcase(&InterDistFl) = YES %then %do;
   %put [MS_S_0058];
   %put ERROR: No Interventions domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&eventsdomains) and %upcase(&EventDistFl) = YES %then %do;
   %put [MS_S_0059];
   %put ERROR: No Events domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA(varlist = arm &AGE &SEX);
%GETSITELISTS(cerrflg = 1);
%if &exiterror %then %goto exit;

%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);

proc sql noprint;
   create table subpairs as
      select aa.usubjid, bb.usubjid2, aa.siteid
      from subject as aa, subject(rename = (usubjid = USUBJID2 siteid = siteid2)) as bb   
      where aa.usubjid < bb.usubjid2 and siteid = siteid2   
      order by aa.usubjid, bb.usubjid2;
quit;

%let availabledomains = ;
%if &FindDistFl = Yes and %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &findingsdomains, flist = &FindTests);
%end;
%GETFINDINGS(predose = 0);
%if &EventDistFl = Yes %then %do;
   %INTEVENT(list = &eventsdomains, predose = 0);
%end;
%if &InterDistFl = Yes %then %do;
   %INTEVENT(list = &interventionsdomains, predose = 0); 
%end;

%if ^%length(&availabledomains) and ((^%length(&AGE) and ^%length(&SEX) and &AgeSexFl = Yes) or &AgeSexFl = No) %then %do;
   %put ERROR: No domains meet the necessary criteria for the analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

data all;
   merge subject(in = ina) &availabledomains;
   by usubjid;
   if ina;
   %if %length(&SEX) %then %do;
      if sex = "M" or sex = "&trM" then MALE = 1;
      else if sex = "F" or sex = "&trF" then MALE = 0;
      drop sex;
      label male = "&trMaleGender";
   %end;
   %if &AgeSexFl = No %then %do;
      %if %length(&AGE) %then %do;
         drop age;
      %end;
      %if %length(&SEX) %then %do;
         drop male;
      %end;
   %end;   
run;
%NObsVars(all);
%let TotalSubs = &nobs;

proc means nmiss noprint data = all;
   output out = nmiss(drop = _type_ _freq_) nmiss = / autoname;
run;

proc transpose data = nmiss out = nmiss;
run;

data nmiss;
   set nmiss(rename = (col1 = nummiss) where = (_name_ ne "SiteSamp_NMiss")) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 6);
   pctmiss = round(nummiss / &TotalSubs * 100, 0.01);
   %if %upcase(&DelMissFl) = YES %then %do; 
      if pctmiss > 0 then missflag = 2;
   %end;
   %else %do;
      if pctmiss >= &MissMax then missflag = 2;
   %end;
   else if pctmiss ne 0 then missflag = 1;
   else if pctmiss = 0 then missflag = 0;
   if eof then call symput("TotalVars", trim(left(_n_)));
   label _name_ = "&trVariable" _label_ = "&trLabel" nummiss = "&trNOSMV" pctmiss = "&trPOSMV" missflag = "&trMissFlag";
run;

*** degree of freeom is number of variables contributing to maha scores;
%let DegFree = 0;
data _null_;
   set nmiss(where = (missflag in (0,1))) end = eof;
   if eof then call symput("DegFree", trim(left(_n_)));
run;

%let RemovedVars = ;
data _null_;
   set nmiss(where = (missflag = 2)) end = eof;
   if eof then call symput("RemovedVars", trim(left(_n_)));
run;

%if ^%length(&RemovedVars) %then %let RemovedVars = 0;
%if ^%eval(&TotalVars - &RemovedVars) %then %do;
   %put [MS_S_0060];
   %put ERROR: No variables remain for analysis after deleting variables with at least &MissMax percent missing data.;
   %let exiterror = 1;
   %goto exit;
%end;

%let MissMaxVars = ;
proc sql noprint;
   select _name_ 
      into :MissMaxVars separated by ' '
      from nmiss
      where missflag = 2
      order by _name_;
quit;

data all;
   set all(drop = &MissMaxVars);
   subject = cats("S",_n_);
run;

%let nodashes = 1;
%if %index(&SiteList,-) %then %let nodashes = 0;

%if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = en and &nodashes = 1 %then %do;
   %OUTNAMETRUNC(outdat = ClusterSubjectsStudySite_OutData, truncnum = 14);
%end;
%else %do;
   %let Lengthit = %eval(32-10-&SiteIDLen);
   %if %eval(&Lengthit<0) %then %let Lengthit = 0;
   %let ClusterSubjectsStudySite_OutData = ;
   %OUTNAMETRUNC(outdat = ClusterSubjectsStudySite_OutData, truncnum = 14);
%end;

%let ClusterSubjects_OutData_sum = ;      
%PrepOutDS(ClusterSubjects_OutData_sum,OutDataName_sum, &Outname.cswss_sum);
%let includeds = &includeds &OutDataName_sum..sas7bdat,;
   
data &ClusterSubjects_OutData_sum;
   set all(drop = subject) end = eof;
run;

proc means var nway noprint data = all;
   class siteid;
   output out = novar(drop = _type_ _freq_) var = / autoname;
run;

proc transpose data = novar out = novar;
   by siteid;
run;

proc sql noprint;
   create table SiteVarCountList as
   select distinct siteid, count(siteid) as countvars 
      from novar
      group by siteid
      order by siteid;
quit;

data novar;
   set novar(rename = (col1 = var) where = (var in (.,0))) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 4);
run;

proc sql noprint;
   create table SiteNoVarCountList as
   select distinct siteid, count(siteid) as countnovar
      from novar
      group by siteid
      order by siteid;
quit;

data SiteVarCountList;
    merge SiteVarCountList SiteNoVarCountList;
    by siteid;
run;

proc datasets nolist;
   delete SiteNoVarCountList;
quit;

%let SitesNoVariability =;
%let firstflag = 0;

proc sort data = all;
   by subject;
run;      

%let i = 1;
%do %while(%length(%scan(&SiteList,&i,' ')));
   %let tempsite = %scan(%upcase(&SiteList),&i," ");
   %let tempsitesize = %scan(%upcase(&SiteSamp),&i," ");
   /* %let nodashtempsite = %sysfunc(tranwrd(&tempsite, -, _));*/
   %let nodashtempsite = %sysfunc(compress(&tempsite, -));
   %let SiteConstantVars = ;  
   proc sql noprint;
      select _name_ 
         into :SiteConstantVars separated by ' '
         from novar
         where siteid = "&tempsite"
         order by _name_;
   quit;

   %let varok = 1;
   data _null_;
      set SiteVarCountList(where = (siteid = "&tempsite"));
      if countvars = countnovar then do;
         call symput("varok",trim(left(0)));
      end;
   run;
   
   %if ^&varok %then %let SitesNoVariability = &SitesNoVariability &tempsite;
  
   %if &varok %then %do;
      *** Defect S1325220 ***;
      proc distance data = all(drop = &SiteConstantVars) shape = square method = euclid out = distance;
         var interval(_numeric_ / std = std);
         *copy usubjid siteid arm country;
         id subject;
         where siteid = "&tempsite";
      run;
      
      proc sort data = distance;
         by subject;
      run;
      
      data distance;
         merge distance(in = ina) all(keep = usubjid subject siteid arm country);
         by subject;
         if ina;
      run;
      
      proc sort data = distance;
         by usubjid subject siteid arm country;
      run;      

      /*
      proc sql noprint;
         select usubjid 
            into :SiteClusterVars separated by ' '
            from distance
            order by usubjid;
      quit;

      %let SiteClusterVars&nodashtempsite = ;
      
      %let j = 1;
      %do %while(%length(%scan(&SiteClusterVars,&j,' ')));
         %let temp = %scan(%upcase(&SiteClusterVars),&j," ");
         %let SiteClusterVars&nodashtempsite = &&SiteClusterVars&nodashtempsite %bquote(:Name(""&temp"")),;
         %let j = %eval(&j+1);
      %end;
      
      */

      proc transpose data = distance out = distance;
         by usubjid subject siteid arm country;
      run;

      proc sql noprint;
         create table siteids2 as
            select distinct subnum, usubjid2
            from all(rename = (subject = subnum usubjid = USUBJID2))
            order by subnum;     
         create table distance2 as
            select * 
            from distance as aa left join siteids2 as bb
            on aa._name_ = bb.subnum
            order by usubjid, usubjid2;
      quit;

      proc datasets nolist;
         delete siteids2 distance;
         change distance2 = distance;
      quit;

      proc sql noprint;
         create table distancepairs(drop = subnum subject _name_ rename = (col1 = Distance)) as
            select *, min(col1) as mindist
            from subpairs(where = (siteid = "&tempsite")) natural left join distance
            order by usubjid, usubjid2;      
      quit;
      
      data distancepairs;
         set distancepairs;
         SITESAMP = &tempsitesize;
         label sitesamp = "&trSiteSampleSize";
      run;

      data mindisttemp(drop = mindist);
         set distancepairs(where = (distance = mindist));
         if _n_ = 1;
      run;

      %if ^&firstflag %then %do;
         data mindist;
            set mindisttemp;
         run;
      %end;
      %else %do;   
         proc append force base = mindist data = mindisttemp;
         run;
      %end;

      data distancepairs;
         set distancepairs(drop = mindist);
         label distance = "&trDistance" usubjid = "&SUBJECTlabel" usubjid2 = "&SUBJECTlabel 2" siteid = "&SITEIDlabel";
      run;

      %if ^&firstflag %then %do;
         data alldistancepairs;
            set distancepairs;
         run;
      %end;
      %else %do;   
         proc append force base = alldistancepairs data = distancepairs;
         run;
      %end;

      %if ^&firstflag %then %let firstflag = 1;

      %if &i = 1 %then %do;   
         %let SUBJECTLabel2 = USUBJID2;
         %CheckLabel(distancepairs,SUBJECTLabel2);
      %end;

      proc transpose data = distance out = distance(drop = _name_);
         var col1;
         by usubjid siteid arm country;
         id _name_;
         idlabel usubjid2;
      run;

      %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = en and &nodashes = 1 %then %do;
         %let C_OutData_&tempsite = ;     
         %let C_OutDataP_&tempsite = ;    

         %PrepOutDS(C_OutData_&tempsite,OutDataName_&tempsite, &Outname.cswss, _&tempsite);
         %PrepOutDS(C_OutDataP_&tempsite,OutDataNameP_&tempsite, &Outname.cswss_p, _&tempsite);

         %let includeds = &includeds &&&OutDataName_&tempsite...sas7bdat, &&&OutDataNameP_&tempsite...sas7bdat,;
   
         data &&&C_OutData_&tempsite;
               set distance;
               label usubjid = "&SUBJECTLabel";
            run;

         data &&&C_OutDataP_&tempsite;
               set distancepairs;
               label usubjid = "&SUBJECTLabel";
            run;
       %end;
       %else %do;
         %let C_OutData_&i = ;      
         %let C_OutDataP_&i = ;     

         %PrepOutDS(C_OutData_&i,OutDataName_&i, &Outname.cswss, _&nodashtempsite);
         %PrepOutDS(C_OutDataP_&i,OutDataNameP_&i, &Outname.cswss_p, _&nodashtempsite);

         %let includeds = &includeds &&&OutDataName_&i...sas7bdat, &&&OutDataNameP_&i...sas7bdat,;

         data &&&C_OutData_&i;
            set distance;
            label usubjid = "&SUBJECTLabel";
         run;

         data &&&C_OutDataP_&i;
            set distancepairs;
            label usubjid = "&SUBJECTLabel";
         run;
      %end;

      proc datasets nolist;
         delete distance distancepairs mindisttemp;    
      quit;
   %end; 
   %let i = %eval(&i+1);
%end; 

%if %sysfunc(exist(mindist)) %then %do;
   proc sort data = mindist;
      by distance;
   run;

   %let ClusterSubjects_OutData_min = ;      
   %PrepOutDS(ClusterSubjects_OutData_min,OutDataName_min, &Outname.cswss,_mindist);
   %let includeds = &includeds &OutDataName_min..sas7bdat,;

   data &ClusterSubjects_OutData_min;
      set mindist end = eof;
      if _n_ = 1 then call symput("MinDistSite", trim(siteid));
      label distance = "&trDistance" usubjid = "&SUBJECTlabel" usubjid2 = "&SUBJECTlabel 2" siteid = "&SITEIDlabel";
   run;
%end;

%if %sysfunc(exist(alldistancepairs)) %then %do;
   %let ClusterSubjects_OutData_all = &ClusterSubjectsStudySite_OutData;      
   %PrepOutDS(ClusterSubjects_OutData_all,OutDataName_all, &Outname.cswss,_alldist);
   %let includeds = &includeds &OutDataName_all..sas7bdat,;

   data &ClusterSubjects_OutData_all;
      set alldistancepairs end = eof;    
   run;

   proc datasets nolist;
      delete alldistancepairs;
   quit;
%end;

proc datasets nolist;
   delete all;
quit;    

* create JSL file;
%PrepOutFile(ClusterSubjectsScript,ClusterSubjectsScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ClusterSubjectsScript;
%let FullAPName=%str(Cluster Subjects Within Study Site);

* put JSL statements;
data _null_;
   file "&ClusterSubjectsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:TabsSectionFlag = 1;";
   put "objRefNS:distdatlist = {};";
   put "objRefNS:pairdatlist = {};";
   
   %let firstdomdat = 0;      
   %let i = 1;
   %do %while(%length(%scan(&SiteList,&i,' ')));
      %let tempsite = %scan(%upcase(&SiteList),&i," ");
      %let nodashtempsite = %sysfunc(compress(&tempsite, -));
      %if ^%index(%bquote(&SitesNoVariability), %bquote(&tempsite)) %then %do;         
         put "Insert Into(objRefNS:distdatlist, ""objRefNS:ddist&nodashtempsite"");";
         put "Insert Into(objRefNS:pairdatlist, ""objRefNS:dpair&nodashtempsite"");";
      %end;
      %let i = %eval(&i+1);
   %end;

   put "objRefNS:Select=Function({},{default local},";
   put "ctable = Current Data Table();";
   put "If(Is Empty(ctable),";
   put "   New Window(locale:get(""JMPClinicalMessage_Text""), showmenu(0),showtoolbars(0),";
   put "      tb = TextBox(locale:get(""ErrorToRunOpenInFocus_Text"")),";
   put "      tb << Font Color(3),";
   put "             jg:messageHelp(""ErrorToRunOpenInFocus_Text"")";
   put "   );";
   put "   Throw()";
   put ");";
   put "subname = ""&SUBJECTLabel"";";
   put "subname2 = ""&SUBJECTLabel 2"";";
   put "foundit = 1;";
   put "foundit2 = 1;";
   put "try(scol = Column(subname),foundit = 0);";
   put "try(scol2 = Column(subname2),foundit2 = 0);";
   put "selsub = {};";
   put "nsel = 0;";
   put " for each row(";
   put "    if(selected(rowstate())==1,";
   put "       if(!Contains(selsub,scol()[]),";
   put "          nsel++;";
   put "          InsertInto(selsub,scol()[])";
   put "       );";
   put "       if(foundit2 == 1,";
   put "          if(!Contains(selsub,scol2()[]),";
   put "             nsel++;";
   put "             InsertInto(selsub,scol2()[])";
   put "          );";
   put "       );";
   put " );";     
   put ");";
   put "Try(objRefNS:alldat = open(""&ClientOutPath.&OutDataName_sum..sas7bdat"", invisible);";
   put "   objRefNS:alldat << Select Where( Contains(selsub, Column(subname)[]));";
   put "   Current Data Table(objRefNS:alldat);";   
   put " );";
   put ");";

   %if %sysfunc(exist(mindist)) %then %do;   
      put "objRefNS:fbox0=Function({},{default local},";
      put "objRefNS:mindist = open(""&ClientOutPath.&OutDataName_min..sas7bdat"",invisible);";
      put "objRefNS:alldist = open(""&ClientOutPath.&OutDataName_all..sas7bdat"",invisible);";
      put "Column(objRefNS:mindist, ""&SITEIDlabel"") << Label(1);";
      put "Column(objRefNS:mindist, ""&trSiteSampleSize"") << Label(1);";
      put "Column(objRefNS:alldist, ""&SubjectLabel"") << Label(1);";
      put "Column(objRefNS:alldist, ""&SubjectLabel2"") << Label(1);";
      put "Column(objRefNS:alldist, ""&trSiteSampleSize"") << Label(1);";
      put "objRefNS:allDistgraphs=HListBox(";
      put "    objRefNS:alldist << Graph Builder(";
      put "       Automatic Recalc(1),";
      put "       Auto Stretching( 0 ),"; 
      put "       Show Control Panel( 0 ),";
      put "       Show Legend( 0 ),";
      put "       Variables( X( :&SITEIDlabel ), Y( :&trDistance )),";
      put "       Elements(";
      put "          Box Plot(";
      put "             X,";
      put "             Y,";
      put "             Legend( 2 ),";
      put "             Jitter( 1 ),";
      put "             Outliers( 1 ),";
      put "             Box Style( ""Solid"" )";
      put "          ),";
      put "       ),";
      put "       SendToReport(";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             OutlineBox,";
      put "             {Set Title( locale:get(""BetweenSubjectDistancesbySite_Text"") )}";
      put "          ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""&SITEIDlabel"",";
      put "             ScaleBox,";
      put "             {Show Major Ticks( 0 ), Rotated Labels( ""Automatic"" )}";
      put "          ),";
      put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             FrameBox,";
      put "             {Marker Size( 2 ), Transparency( 0.75 )}";
      put "          )";
      put "       )";
      put "    ),";
      put "    objRefNS:mindist << Graph Builder(";
      put "       Size( 353, 497 ),";
      put "       Automatic Recalc(1),";
      put "       Lock Scales( 1 ),";
      put "       Auto Stretching( 0 ),"; 
      put "       Show Control Panel( 0 ),";
      put "       Show Legend( 0 ),";
      put "       Variables( Y( :&trDistance )),";
      put "       Elements(";
      put "          Box Plot(";
      put "             Y,";
      put "             Legend( 2 ),";
      put "             Jitter( 1 ),";
      put "             Outliers( 0 ),";
      put "             Box Style( ""Outlier"" )";
      put "          ),";
      put "          Points( Y,Legend( 3 ), Jitter( 1 ) )";
      put "       ),";
      put "         Local Data Filter(";
      put "            Add Filter(";
      put "               columns( :&SITEIDlabel ),";
      put "            )";
      put "         ),";
      put "       SendToReport(";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             OutlineBox,";
      put "             {Set Title( locale:get(""MinimumBetweenSubjectDistancesforEachSite_Text"") )}";
      put "          ),";
      put "          Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
      put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "          Dispatch(";
      put "             {},";
      put "             ""Graph Builder"",";
      put "             FrameBox,";
      put "             {Marker Size( 2 ), Transparency( 0.75 )}";
      put "          )";
      put "       )";
      put "    ),";     
      put ");";
      put ");";
      put "InsertInto(objRefNS:DataList,{{""objRefNS:mindist"",""objRefNS:alldist""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trMinimumBetweenSubDist"",""&trBetweenSubjectDistancesbySite""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""BetweenSubjectDistanceSummary_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox0()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:allDistgraphs}});";
   %end;

   %let firstdomdat = 0;      
   %let i = 1;
   %let tFlag = 0;
   %do %while(%length(%scan(&SiteList,&i,' ')));
      %let tempsite = %scan(%upcase(&SiteList),&i," ");
      %let nodashtempsite = %sysfunc(compress(&tempsite, -));
      %if ^%index(%bquote(&SitesNoVariability), %bquote(&tempsite)) %then %do;
         put "objRefNS:fbox&nodashtempsite=Function({},{default local},";
         %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = en and &nodashes = 1 %then %do;
            put "objRefNS:ddist&nodashtempsite = open(""&ClientOutPath.&&&OutDataName_&tempsite...sas7bdat"",invisible);";
            put "objRefNS:dpair&nodashtempsite = open(""&ClientOutPath.&&&OutDataNameP_&tempsite...sas7bdat"",invisible);";   
         %end;
         %else %do;
            put "objRefNS:ddist&nodashtempsite = open(""&ClientOutPath.&&&OutDataName_&i...sas7bdat"",invisible);";
            put "objRefNS:dpair&nodashtempsite = open(""&ClientOutPath.&&&OutDataNameP_&i...sas7bdat"",invisible);"; 
         %end;
         put "Column(objRefNS:dpair&nodashtempsite, ""&SubjectLabel"") << Label(1);";
         put "Column(objRefNS:dpair&nodashtempsite, ""&SubjectLabel2"") << Label(1);";
         
         put "objRefNS:Dm&nodashtempsite=VListBox(";     
         %if &tempsite = &MinDistSite %then %do;         
            put "textbox(locale:get(""Thissitehastheminimumbetweensubjectdistance_Text""),<<Set Wrap(900),<< Font Color(5)),"; 
            put "textbox("""",<<Set Wrap(900),<< Font Color(5)),"; 
         %end; 
         put "HListBox(";
         put "    objRefNS:dpair&nodashtempsite << Graph Builder(";
         put "       Size( 353, 497 ),";
         put "       Automatic Recalc(1),";
         put "       Auto Stretching( 0 ),";      
         put "       Show Control Panel( 0 ),";
         put "       Show Legend( 0 ),";
         put "       Variables( Y( :&trDistance )),";
         put "       Elements(";
         put "          Box Plot(";
         put "             Y,";
         put "             Legend( 2 ),";
         put "             Jitter( 1 ),";
         put "             Outliers( 0 ),";
         put "             Box Style( ""Outlier"" )";
         put "          ),";
         put "          Points( Y, Legend( 3 ), Jitter( 1 ) )";
         put "       ),";
         put "       SendToReport(";
         put "          Dispatch(";
         put "             {},";
         put "             ""Graph Builder"",";
         put "             OutlineBox,";
         put "             {Set Title( locale:get(""BetweenSubjectDistances_Text"") )}";
         put "          ),";
         put "          Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
         put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
         put "          Dispatch(";
         put "             {},";
         put "             ""Graph Builder"",";
         put "             FrameBox,";
         put "             {Marker Size( 2 ), Transparency( 0.75 )}";
         put "          )";
         put "       )";
         put "    ),";
         put "    SiteClusterVars = objRefNS:ddist&nodashtempsite << Get Column Names(Numeric, string);";
         put "    Eval(Parse(EvalInsert(""\[objRefNS:ddist&nodashtempsite << Hierarchical Cluster(";
         put "       Automatic Recalc(1),";
         put "       Hide Distance Messages(1),";        
         put "       Y(^SiteClusterVars^),";
         put "       Label( :&SUBJECTLabel ),";
         put "       Method( ""&HierClustMethod"" ),";
         put "       Standardize( 0 ),";
         put "       Distance Matrix( 1 ),";
         put "       Color Clusters( 1 ),";
         put "       Distance Graph( 0 ),";
         put "       Legend( 1 ),";
         put "       Dendrogram Scale( ""Distance Scale"" ),";
         put "       Two Way Clustering,";
         put "       Color Map( {"""", {{255, 0, 0}, {255, 255, 255}}} ),";
         put "       SendToReport(";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox,";
         put "          {Frame Size( 45, 352 )}";
         put "       ),";
         put "       Dispatch( {""Dendrogram""}, """", CellPlotBox( 2 ), {Frame Size( 349, 352 ), Color Theme({"""", {{255, 0, 0}, {255, 255, 255}}})} ),";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox( 2 ),";
         put "          {Frame Size( 200, 352 )}";
         put "       ),";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox( 3 ),";
         put "          {Frame Size( 349, 155 )}";
         put "       ),";
         put "       Dispatch(";
         put "          {""Dendrogram""},";
         put "          ""Clust Dendro"",";
         put "          FrameBox( 4 ),";
         put "          {Frame Size( 349, 59 )}";
         put "       ))";           
         put "    );]\"")))";
         put ")));";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:fbox&nodashtempsite}});";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:ddist&nodashtempsite"", ""objRefNS:dpair&nodashtempsite""}});";
         put "InsertInto(objRefNS:DataNameList,{{""&trSite &nodashtempsite - &trDistanceMatrix"",""&trSite &nodashtempsite - &trBetweenSubjectDistances""}});";
         put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""SiteXDistanceMatrix_Text""),{""&tempsite""})));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox&nodashtempsite();));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         %if &tempsite = &MinDistSite %then %do;
            put "InsertInto(objRefNS:TabOpenList,1);";
         %end;
         %else %do;
            put "InsertInto(objRefNS:TabOpenList,0);";
            %let tFlag = 1;
         %end;
      %end;
      %else %do;
         put "objRefNS:fbox&nodashtempsite=Function({},{default local},";
         put "VListBox(TextBox(locale:get(""AllVarsAvForAnHadNoVarWiThisSiteThisImpMsg_Text""),<<Set Wrap(900),<< Font Color(5)));"; 
         put ");";
         put "InsertInto(objRefNS:DataList,{{""""}});";
         put "InsertInto(objRefNS:DataNameList,{{""""}});";
         put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""SiteXDistanceMatrix_Text""),{""&tempsite""})));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox&nodashtempsite()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
      %end;
      %let i = %eval(&i+1);
   %end;
   %if &tFlag=1 %then %do;
      put "objRefNS:TabsSectionFlag = 1;";
   %end;
  
   put "objRefNS:ppdd = evalexpr(Function({},{default local},";
   put "objRefNS:Select();";
   put "this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   put "objRefNS:spdd =  Function({}, {},";
   put "objRefNS:Select();";
   put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""); Close(objRefNS:alldat, nosave););";
   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";

   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";
   
   put "objRefNS:list1 = {""objRefNS:alldat"",""objRefNS:mindist"",""objRefNS:alldist""}||objRefNS:pairdatlist||objRefNS:distdatlist;";
   put "objRefNS:list1 = Eval(Parse(EvalInsert(""\[{^objRefNS:list1^}]\"")));";
   put "objRefNS:list2 = objRefNS:pairdatlist||objRefNS:distdatlist;";
   put "objRefNS:list2 = Eval(Parse(EvalInsert(""\[{^objRefNS:list2^}]\"")));";
   put "objRefNS:list3 = Eval(Parse(EvalInsert(""\[{^objRefNS:pairdatlist^}]\"")));";
   
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ppdd()));";
   put "InsertInto(objRefNS:ActionDataTableList[1], objRefNS:list1);";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], objRefNS:list1);";   
   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowRowsinHeatMap_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowRow()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], objRefNS:list3);";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""SubsetClusteringRows_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:SubsetClustering()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], objRefNS:list3);";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""RevertClustering_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:RevertClustering()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], objRefNS:list2);";

   put "objRefNS:ActionSelectionCountList = {{1,1,1,1,1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"",""&IconPath.ShowRowsInHeatMap.gif"",""&IconPath.SubsetClustering.gif"",""&IconPath.RevertClustering.gif""}};";
   put "objRefNS:ActionRequirementFlagList = {{3, 0, 0, 0, 0}};";
run;
  
%let includeds = %substr(%quote(&includeds), 1, %eval(%length(&includeds) - 1));
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ClusterSubjectsScriptName, "&includeds", yes);
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DataFromX_Text)),
   %nrbquote(%LocaleGet(&ProcessName._Label)))), outlib,  &OutDataName_sum, no);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ClusterSubjectsStudySite;
%ClusterSubjectsStudySite;

************************************;
************************************;
************************************;