/*
JMP Clinical Analytical Process

Name: CombineStudies
Author: Li Li, SAS Institute Inc.
Support: Li Li, SAS Institute Inc.
History:
   saslii 02Oct13 initial coding
   saslii May15 reorganize the dialog to combine as many studies as users want. 
Description: 
This process defines the default settings
for JMP Clinical. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath AInPath CombStudyList Comberror CombDomains DefaultServerOutPath 
	ArchiveSASCode ResolveSASCode TransportCheck CombineStudiesScript SelectedDom
	OutPath ServerOutPath MacroPath _slash_ ShowResultFlag SnpComp CombSuppDom pdfFileFlag
	SDTMDom ADaMDom;

*ProcessBody;

%macro CombineStudies;

%let ProcessName=CombineStudies;

options nofmterr;
*options mprint mlogic symbolgen;

* create output folder under home folder;

%if %length(&StudyName) > 32 %then %do;
	%put [MS_S_0001];
	%put ERROR: The study name must be 32 characters or fewer.;
	%let exiterror = 1;
	%goto exit;
%end;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%let OutPath = %jg_getPath(/user/clinical/output,,1);
%let OutName = &StudyName;

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CSTMacros.sas" / nosource;
%if &exiterror %then %goto exit;

%let NewStudyName = &StudyName;
%put newstudyname=&NewStudyName;
%CheckNewStudyName;
%let CombDomains=;
%let pdfFileFlag=No;
%let unmatchFlag=0;
%let SDTMDomFlag=0;
%let ADaMDomFlag=0;
%let SDTMFFlag=No;
%let ADaMFFlag=No;

%if ^%length(&AInPath) & ^%length(&InPath) %then %do;
	%put [MS_S_0002];
	%put ERROR: Neither the SDTM folder or ADaM folder is specified.;
	%put At least one of these folder must be specified to run JMPC Clinical processes.;
	%let exiterror = 1;
%end;
%if &exiterror %then %goto exit;

%if %quote(%upcase(&AInPath))=  %quote(%upcase(&InPath)) %then %do;
	%put [MS_S_0003];
	%put ERROR: The SDTM and ADaM folder paths cannot be the same.;
	%let exiterror = 1;
%end;

%if %length(&InPath) %then %do;
	*check if the directory exists;
	%local rc fileref;
	%let rc = %sysfunc(filename(fileref,&InPath)) ; 
	%if ^%sysfunc(fexist(&fileref))  %then %do;
		%put [MS_S_0004];
	 	%put ERROR: SDTM folder &InPath specification is invalid.;
	 	%let exiterror = 1;
	%end;
%end;
%if %length(&AInPath) %then %do;
	*check if the directory exists;
	%local rc fileref;
	%let rc = %sysfunc(filename(fileref,&AInPath)) ; 
	%if ^%sysfunc(fexist(&fileref))  %then  %do;
		%put [MS_S_0005];
		%put ERROR: ADaM folder &AInPath specification is invalid.;
	 	%let exiterror = 1;
	%end;
%end;
%if &exiterror %then %goto exit;

*check if the directory includes any sas dataset;
%if %length(&InPath) %then %do;
	%FileFilter(&InPath,sasdatalist,_ext="sas7bdat");
	%NObsVars(sasdatalist);
	%if &nObs >0 %then %do;
		%let SDTMDomFlag=1;
	%end;
%end;
%if %length(&AInPath) %then %do;
	%FileFilter(&AInPath,sasdatalist,_ext="sas7bdat");
	%NObsVars(sasdatalist);
	%if &nObs >0 %then %do;
		%let ADaMDomFlag=1;
	%end;
%end;

libname Userlib "%jg_getPath(/user/clinical/studies,,1)";
%let metaData = metadata_study_source;

* check if the folder specified is already used by other study;
data _null_;
	set Userlib.&metaData;
   _F_ = 'metaSInFile'||trim(left(_n_));
   _A_ = 'metaAInFile'||trim(left(_n_));
   call symput(_F_,trim(SDTMFolder));
   call symput(_A_,trim(ADAMFolder));
   call symput ("TInobs",trim(_n_));
run;
%do _i_ = 1 %to &TInobs;
	%if %length(&InPath) %then %do;
		%if %qupcase(&&metaSInFile&_i_)=%qupcase(&InPath) %then %do;
			%put [MS_S_0062];
			%put ERROR: The SDTM folder you specified is already used by other study. Please specify a different folder for the new combined study.;
			%let exiterror = 1;
			%if &exiterror %then %goto exit;
		%end;
	%end;
	%if %length(&AInPath) %then %do;
		%if %qupcase(&&metaAInFile&_i_)=%qupcase(&AInPath) %then %do;
			%put [MS_S_0063];
			%put ERROR: The ADaM folder you specified is already used by other study. Please specify a different folder for the new combined study.;
			%let exiterror = 1;
			%if &exiterror %then %goto exit;
		%end;
	%end;
%end;
* get all of the study names which are used to combine studies;
%put CombStudyList=&CombStudyList;
%let studyNameList =;
%let i = 1;
%do %while(%length(%scan(&CombStudyList,&i,'|')));
   %let CombStudyName&i = %scan(&CombStudyList,&i,'|');
   %let symlist=%nrstr(- . + @ # ~);
   %do j = 1 %to 6;
   	%let tempsymlist = %scan(&symlist,&j,' ');
   	%if &j=1 %then %do;
   		%let comStudyUnderS&i =%sysfunc(transtrn(&&CombStudyName&i, &tempsymlist, _));
   	%end;
   	%else %do;
   		%let comStudyUnderS&i =%sysfunc(transtrn(&&comStudyUnderS&i, &tempsymlist, _));
   	%end;
   %end;
   %let comStudyUnderS&i =%sysfunc(transtrn(&&comStudyUnderS&i, %str( ), _));
   %let studyNameList=&studyNameList &&CombStudyName&i,;
   %put comStudyUnderS&i=&&comStudyUnderS&i;
	%let i = %eval(&i+1);	
%end;
%let totalStudies=%eval(&i-1);
%do i=1 %to &totalStudies;
	%if &i=1 %then %do;
		%let studyNameList=&&CombStudyName&i;
	%end;
	%else %if (&i>1 & &i < &totalStudies) %then %do;
		%let studyNameList=&studyNameList, &&CombStudyName&i;
	%end;
	%else %do;
		%let studyNameList=&studyNameList and &&CombStudyName&i;
	%end;
%end;
* throw an error message if there is no more than 2 studies;
%if &totalStudies <2 %then %do;
         %put ERROR: At least two studies are needed to combine studies.;
         %let exiterror = 1;
         %if &exiterror %then %goto exit;
%end;

* retrieve SDTM and ADAM folder information for combining studies;
data _null_;
	set Userlib.&metaData;
   %do _i_ =1 %to &totalStudies;
	   if StudyName = "&&CombStudyName&_i_" then do;
         folder1='SFolder'||trim(left(&_i_));
         folder2='AFolder'||trim(left(&_i_));
         call symputx(folder1,SDTMFolder);
         call symputx(folder2,ADAMFolder); 
	   end;
   %end;
run;
%put SFolder1=&SFolder1 SFolder2=&SFolder2 AFolder1=&AFolder1 AFolder2=&AFolder2;

%if ^%symexist(Server) %then %let Server = Local;

%do _i_ =1 %to &totalStudies;
   libname SFld&_i_ "&&SFolder&_i_";
   libname AFld&_i_ "&&AFolder&_i_";
   %let adslpresent&_i_ =;
%end;

%do _i_ =1 %to &totalStudies;
   %let comStudyUnder&_i_ = %sysfunc(transtrn(&&CombStudyName&_i_, %str( ), _));
   data alldomains&_i_;
      infile "%jg_getPath(/user/clinical/metadata/base, ALLDOMAINS.txt, 0, 0, 1, &&comStudyUnder&_i_)" delimiter = '~' MISSOVER DSD lrecl=32767;
   	length domain $8. flag $4.;
   	input domain flag;
   run;
%end;

%if ^&CombSuppDom & (^%length(&SDTMDom) & ^%length(&ADaMDom)) %then %do;
	%do _i_ =1 %to &totalStudies;
		data alldomains&_i_;
			set alldomains&_i_;
			if index(domain,"SUPP") then delete;
		run;
	%end;
%end;
%let SelectedDomFlag = 0;
%if %length(&SDTMDom) %then %do;
	%let CombDomains=&CombDomains &SDTMDom;
	%let SelectedDomFlag = 1;
	%if ^%length(&ADaMDom) %then %do;
		%do _i_ =1 %to &totalStudies;
			data domtemp;
				set alldomains&_i_;
				if flag="SDTM" then delete;
			run;
			proc sql noprint;
      		select domain 
      		into :adamDomTmp separated by ' '
      		from domtemp;
      	quit;
      	%if %symexist(adamDomTmp) %then %do;
      		%let CombDomains=&CombDomains &adamDomTmp;
      	%end;
		%end;
	%end;
%end;
%if %length(&ADaMDom) %then %do;
	%let CombDomains=&CombDomains &ADaMDom;
	%let SelectedDomFlag = 1;
	%if ^%length(&SDTMDom) %then %do;
		%do _i_ =1 %to &totalStudies;
			data domtemp;
				set alldomains&_i_;
				if flag="ADAM" then delete;
			run;
			proc sql noprint;
      		select domain 
      		into :sdtmDomTmp separated by ' '
      		from domtemp;
      	quit;
      	%if %symexist(sdtmDomTmp) %then %do;
      		%let CombDomains=&CombDomains &sdtmDomTmp;
      	%end;
		%end;
	%end;
%end;
%if ^%length(&CombDomains) %then %do;
	%let allDomList=;
   %do _i_ =1 %to &totalStudies;
      proc sort data=alldomains&_i_;
	   	by domain;
	   run;	
		
		%let allDomStudy&_i_=;
		* get domain list for each study;
   	proc sql noprint;
      	select domain 
      	into :allDomStudy&_i_ separated by ' '
      	from alldomains&_i_;
   	quit;
   	%*put allDomStudy&_i_=&&allDomStudy&_i_;
   	
      %if &_i_ = 1 %then %do;
         data alldomains;
            set alldomains&_i_;
         run;
      %end;
      %else %do;
         proc sort data=alldomains;
            by domain;
         run;  
         data alldomains;
            merge alldomains(in=ina) alldomains&_i_(in=inb);
            by domain;
            if (ina or inb);  
         run;  
      %end;
   %end;
	data _null_;
		set alldomains;
		if flag="SDTM" then call symput ("SDTMFFlag",trim(left("Yes")));
		if flag="ADAM" then call symput ("ADaMFFlag",trim(left("Yes")));
	run;
   proc sql noprint;
      select domain 
      into :allDomList separated by '|'
      from alldomains;
      %if &SDTMFFlag = Yes %then %do;
      select domain
      into :SDTMDomA separated by ' '
      from alldomains(where=(flag="SDTM"));
      %end;
      %if &ADaMFFlag = Yes %then %do;
      select domain
      into :ADaMDomA separated by ' '
      from alldomains(where= (flag="ADAM"));
      %end;
   quit;
	%let CombDomains=&allDomList;

	%if ^%length(&SDTMDom) %then %let SDTMDom =&SDTMDomA;
	%if ^%length(&ADaMDom) %then %let ADaMDom =&ADaMDomA;	
%end;
%else %do;
	%let CombDomains2=;
	%let i=1;
	%do %while(%length(%scan(&CombDomains,&i,' ')));
		%let combTemp = %scan(&CombDomains,&i,' ');
		%if &i=1 %then %do;
			%let CombDomains2=&combTemp;
		%end;
		%else %do;
			%let CombDomains2=&CombDomains2|&combTemp;
		%end;
		%let i = %eval(&i+1);
	%end;
	%let CombDomains = &CombDomains2;
	   
   data seldomain;
   	length domain $8.;
   	%let i = 1;
   	%do %while(%length(%scan(&CombDomains,&i,'|')));
   		%let temp = %scan(&CombDomains,&i,'|');
   		domain ="&temp";
   		output;
   		%let i = %eval(&i+1);
   	%end;
   run;
	proc sort data=seldomain;
		by domain;
	run;	

	%do _i_ =1 %to &totalStudies;
      proc sort data=alldomains&_i_;
	   	by domain;
	   run;	

		data alldomains&_i_;
			merge seldomain(in=ina) alldomains&_i_(in=inb);
			by domain;
			if (ina and inb);
		run;

		%let allDomStudy&_i_=;
		* get domain list for each study;
   	proc sql noprint;
      	select domain 
      	into :allDomStudy&_i_ separated by ' '
      	from alldomains&_i_;

   	quit;
   	
      %if &_i_ = 1 %then %do;
         data alldomains;
            set alldomains&_i_;
         run;
      %end;
      %else %do;
         proc sort data=alldomains;
            by domain;
         run;  
         data alldomains;
            merge alldomains(in=ina) alldomains&_i_(in=inb);
            by domain;
            if (ina or inb);  
         run;  
      %end;
   %end;
	data _null_;
		set alldomains;
		if flag="SDTM" then call symput ("SDTMFFlag",trim(left("Yes")));
		if flag="ADAM" then call symput ("ADaMFFlag",trim(left("Yes")));
	run;   	
	*if customers selected domains, and also checked combine supp domain;
	%if &CombSuppDom %then %do;
   	%do _i_ =1 %to &totalStudies;
   	   data alldomainsSupp&_i_;
   	      set alldomains&_i_;
		   	if index(domain,"SUPP");
		   run;
   
/*   	   %if &_i_ = 1 %then %do;
   	       data alldomains;
   	          set alldomains&_i_;
   	      run;
   	   %end; */
	      data alldomains;
  	         merge alldomains(in=ina) alldomainsSupp&_i_(in=inb);
	      	by domain;
	      	if (ina or inb);  
	      run;	
   	%end;
		proc sql noprint;
			select domain 
			into :allDomList separated by '|'
			from alldomains;
		quit;
		%let CombDomains= &allDomList;
	%end;
%end;

%put CombDomains=&CombDomains;
%put SDTMFFlag=&SDTMFFlag ADaMFFlag=&ADaMFFlag;

%if &SDTMFFlag=No %then %let InPath="";
%if &ADaMFFlag=No %then %let AInPath="";
* define libnames;
%if %length(&InPath) & &SDTMFFlag=Yes %then %do;
	libname SDTMLib "&InPath";
%end;
%if %length(&AInPath) & &ADaMFFlag=Yes %then %do;
	libname ADAMLib "&AInPath";
%end;

%if ^%length(&CombDomains) %then %do;
	%put [MS_S_0064];
	%put ERROR: There is no common domain between studies &CombStudyName1 and &CombStudyName2 to combine.;
	%put ERROR: Please choose different studies to combine.;
	%let exiterror = 1;
	%goto exit;
%end;

*use selected domains to combine studies;
%if %length(&CombDomains) %then %do;
	%let i = 1;
	%let domlist=;
	%do %while(%length(%scan(&CombDomains,&i,'|')));
		%let dom&i = %scan(&CombDomains,&i,'|');
		%let domlist= &domlist &&dom&i;
		%let i = %eval(&i+1);
	%end;		

	data _null_;
      set alldomains;
		%let i = 1;
		%do %while(%length(%scan(&CombDomains,&i,'|')));
			%put dom&i==&&dom&i;
			if domain = "&&dom&i" then do;
				j=&i;
				name='domainFolder'||trim(left(j));
				call symput (name,flag);
			end;
			%let i = %eval(&i+1);
		%end;
	run;
/*	%let i = 1;
	%do %while(%length(%scan(&CombDomains,&i,'|')));
		%put i= &i domain =&&dom&i domainFolder=&&domainFolder&i;
		%let i = %eval(&i+1);
	%end;
*/
	* compare domain var list for all studies, if there is missing variable in one study, throw an error message;
	* get the domain information for combine study 1, e.g., VAR list for each domain;
   %do _i_ =1 %to &totalStudies;
      %let mbc = %jg_getPath(/user/clinical/metadata/base, CSTMacroVars.sas, 0, 0, 1, &&CombStudyName&_i_);
      %include "&mbc" / nosource;
      %if %sysfunc(findw(&domains, DM)) = 0 and %length(&adslpresent) %then %do;
	   	%let domains = &domains;
	     	%global in_DM;
	   	%let in_DM = &in_ADSL;
	   	%global DMVARS;
	   	%let DMVARS = &ADSLVARS;
	   %end;
      %let combDomain&_i_=&domains;
      %let combSuppDomain&_i_=&suppdomains;
      %let eventsdomains&_i_=&eventsdomains;
      %let findingsdomains&_i_=&findingsdomains;
      %let interventionsdomains&_i_=&interventionsdomains;
      %put NOTE: Available Domains for &&CombStudyName&_i_: &&combDomain&_i_;
      %put NOTE: Domains with Supplemental Qualifiers for &&CombStudyName&_i_: &&combSuppDomain&_i_;
      %put NOTE: Events Domains for &&CombStudyName&_i_: &&eventsdomains&_i_;
      %put NOTE: Findings Domains for &&CombStudyName&_i_: &&findingsdomains&_i_;
      %put NOTE: Interventions Domains for &&CombStudyName&_i_: &&interventionsdomains&_i_;
	   %if %length(&adslpresent) %then %do;
         %put NOTE: ADSL for &&CombStudyName&_i_ Available;
         %let adslpresent&_i_=&adslpresent;
	   %end;
      %else %put NOTE: ADSL for &&CombStudyName&_i_ NOT Available;

     	%let j=1;
     	%let sdtmflag=0;
     	%let adamflag=0;
      
      %put CombDomains=&CombDomains;
     	%do %while(%length(%scan(&CombDomains,&j,'|')));
     		* check if the two studies have the similar columns;
     		%let temp = %scan(&CombDomains,&j,'|');
         
         %if &&domainFolder&j=SDTM %then %do;
            %let domfolder=&&SFolder&_i_;
            %let domfile=&domfolder.&temp..sas7bdat;
         %end;
         %else %do;
            %let domfolder=&&AFolder&_i_;
            %let domfile=&domfolder.&temp..sas7bdat;
         %end;
         %if &_i_=1 %then %do;
            %global &temp.Cflag;
            %let &temp.Cflag=0;
            %let &temp.Mflag=0;
            %let &temp.Aflag=0;
            %let &temp.AllDomains=;
            %let &temp.Allnumber =0;
            %let &temp.NoDomains=;
            %let &temp.Nonumber =0;
         %end;
         %put domfile=&domfile;
         * combine only when the study has the corresponding domain;
         %if %sysfunc(findw(&&&allDomStudy&_i_, &temp)) & ^%sysfunc(fileexist(&domfile)) = 0 %then %do;
         	%let &temp.AllDomains= &&&temp.AllDomains &&comStudyUnderS&_i_;
         	%let &temp.Allnumber =%eval(&&&temp.Allnumber +1);
     	   	%let i=1;
     	   	data a&temp;
     	   		length Variable $10.;
     	   		%do %while(%length(%scan(&&&temp.VARS,&i,' ')));
     	   			%if %index(&&&temp, SUPP) %then %do;
     	   				%let temp3=%substr(&temp,5);
     	   				%let temp3=&temp3.SUPP;
     	   				%let temp2=%scan(&&&temp3.VARS,&i,' ');
     	   			%end;
     	   			%else %do;
     	   				%let temp2=%scan(&&&temp.VARS,&i,' ');
     	   			%end;
     	   			Variable="&temp2";
     	   			output;
     	   			%let i=&i+1;
     	   		%end;
     	   	run;
     	   	proc sort data=a&temp;
     	   		by Variable;
     	   	run;
     	   	
   	   		data aVar&temp;
     	   			set a&temp;
     	   			Domain="&temp";
						&&comStudyUnderS&_i_ =1;
						label &&comStudyUnderS&_i_="&&CombStudyName&_i_";
     	   		run;

            %if &&&temp.Cflag=0  %then %do;
               data m&temp;
                  set a&temp;
               run;
              	data t&temp;
						set aVar&temp;
               run;
               %let &temp.Cflag=1;
            %end;
            %else %do;
               proc sort data=m&temp;
                  by Variable;
               run;
               data m&temp;
                  merge m&temp(in=ina) a&temp(in=inb);
                  by Variable;
                  if (ina and not inb) or (inb and not ina); 
               run;
            %end;
            %NObsVars(a&temp);
            %let nvars2 = &nObs;
            %NObsVars(m&temp);

            %if &nObs > 0 & &_i_ >1 & &Comberror=No %then %do;
               proc sql noprint;
                  select Variable
                  into :varDifflist separated by ' '
                  from m&temp;
               quit;
               %let unmatchFlag =1;
               %let pdfFileFlag=Yes;
               data t&temp;
               	merge t&temp aVar&temp;
               	by Variable;
               run;
            %end;
            %else %do;
               data m&temp;
                  set a&temp;
               run;
					data t&temp;
 						merge t&temp aVar&temp;
						by Variable;
					run;
					%if &nObs > 0 & &_i_ >1 & &Comberror=Yes %then %do;
						%let pdfFileFlag=Yes;
					%end;
            %end;
				%if &unmatchFlag =0 %then %do;
           *combine studies in SDTM folder;
            %if &&domainFolder&j = SDTM & %length(&InPath) & &SDTMFFlag=Yes %then %do;
               %let tempLen=;
               * the column length for SFld1 and SFld2 are different, then create problem S1025523;
               %if &&&temp.Mflag=0 %then %do;
                  libname slib "&&SFolder&_i_";
                  %let tid = %sysfunc(open(slib.&temp));
                  %let nv_out1 = %sysfunc(attrn(&tid,NVARS)); 
                  %do i = 1 %to &nv_out1;
                     %let &temp._name&i = %sysfunc(VARNAME(&tid,&i));
                     %let &temp._len&i = %sysfunc(VARLEN(&tid,&i));
                     %let &temp._type&i = %sysfunc(VARTYPE(&tid,&i));
                     %*put &temp._name&i=&&&temp._name&i  _type&i=&&&temp._type&i _len&i=&&&temp._len&i;
                  %end;
                  %let tid = %sysfunc(close(&tid));        
                  
               %end;
               %else %do;
                  libname slib "&&SFolder&_i_";
                  %let tid = %sysfunc(open(slib.&temp));
                  %let nv_out2 = %sysfunc(attrn(&tid,NVARS)); 
                  %do i = 1 %to &nv_out2;
                     %let &temp._nameC&i = %sysfunc(VARNAME(&tid,&i));
                     %let &temp._lenC&i = %sysfunc(VARLEN(&tid,&i));
                     %let &temp._typeC&i = %sysfunc(VARTYPE(&tid,&i));
                     %*put &temp._nameC&i=&&&temp._nameC&i  &temp._typeC&i=&&&temp._typeC&i &temp._lenC&j=&&&temp._lenC&i;
                  %end;
                  %let tid = %sysfunc(close(&tid));   
               %end;

               * compare the string length between two files and use the longer length for merging;
               %if &&&temp.Mflag>0 %then %do;
                 %do i = 1 %to &nv_out1;
                    %if %nrbquote(&&&temp._type&i) = C %then %do;
                       %do _j_ = 1 %to &nv_out2;
                          %if %nrbquote(&&&temp._name&i) = %nrbquote(&&&temp._nameC&_j_) %then %do;
                             %if &&&temp._len&i > &&&temp._lenC&_j_ %then %do;
                                %let tempLen=&tempLen &&&temp._name&i $ &&&temp._len&i;
                             %end;
                             %else %if &&&temp._len&i < &&&temp._lenC&_j_ %then %do;
                                %let tempLen=&tempLen &&&temp._name&i $ &&&temp._lenC&_j_;
                                %let &temp._len&i =&&&temp._lenC&_j_;
                             %end;
                          %end;
                       %end;
                    %end;
                 %end;
               %end;
               %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) eq %nrbquote(zh) %then %do;
                  %if &&&temp.Mflag=0 %then %do;
                     data SDTMLib.&temp;
                        set slib.&temp(encoding=ANY);
                     run;
                     %let &temp.Mflag=1;
                  %end;
                  %else %do;
                     data SDTMLib.&temp;
                        %if %length(&tempLen) %then length &tempLen;;
                        set SDTMLib.&temp(encoding=ANY) slib.&temp(encoding=ANY);
                     run;
                  %end;
               %end;
               %else %do;
                  %if &&&temp.Mflag=0 %then %do;
                     data SDTMLib.&temp;
                        set slib.&temp;
                     run;
                     %let &temp.Mflag=1;
                  %end;
                  %else %do;
                     data SDTMLib.&temp;
                        %if %length(&tempLen) %then length &tempLen;;
                        set SDTMLib.&temp slib.&temp;
                     run;
                  %end;
               %end;
            %end; /* end of SDTM folder combining */
            %else %if &&domainFolder&j=ADAM & %length(&AInPath) & &ADaMFFlag=Yes %then %do;
               %let tempLen=;
               * the column length for AFld1 and AFld2 are different, then create problem S1025523;
               %if &&&temp.Aflag=0 %then %do;
                  libname alib "&&AFolder&_i_";
                  %let tid = %sysfunc(open(alib.&temp));
                  %let nv_out1 = %sysfunc(attrn(&tid,NVARS)); 
                  %put nv_out1=&nv_out1  &&AFld&_i_.&temp;
                  
                  %do i = 1 %to &nv_out1;
                     %let &temp._name&i = %sysfunc(VARNAME(&tid,&i));
                     %let &temp._len&i = %sysfunc(VARLEN(&tid,&i));
                     %let &temp._type&i = %sysfunc(VARTYPE(&tid,&i));
                     %put &temp._name&i=&&&temp._name&i  _type&i=&&&temp._type&i _len&i=&&&temp._len&i;
                  %end;
                  %let tid = %sysfunc(close(&tid));
               %end; 
               %else %do;    
                  libname alib "&&AFolder&_i_";
                  %let tid = %sysfunc(open(alib.&temp));
                  %let nv_out2 = %sysfunc(attrn(&tid,NVARS)); 
                  %do _j_ = 1 %to &nv_out2;
                     %let &temp._nameC&_j_ = %sysfunc(VARNAME(&tid,&_j_));
                     %let &temp._lenC&_j_ = %sysfunc(VARLEN(&tid,&_j_));
                     %let &temp._typeC&_j_ = %sysfunc(VARTYPE(&tid,&_j_));
                     %put &temp._nameC&_j_=&&&temp._nameC&_j_  &temp._typeC&_j_=&&&temp._typeC&_j_ &temp._lenC&_j_=&&&temp._lenC&_j_;
                  %end;
                  %let tid = %sysfunc(close(&tid));   
               %end;
      
               %let USUBflg=0;

               * compare the string length between two files and use the longer length for merging;
               %if &&&temp.Aflag>0 %then %do;
                  %do i = 1 %to &nv_out1;
                     %if %nrbquote(&&&temp._type&i) = C %then %do;
                        %do _j_ = 1 %to &nv_out2;
                           %if %nrbquote(&&&temp._name&i) = %nrbquote(&&&temp._nameC&_j_) %then %do;
                              %if &&&temp._len&i > &&&temp._lenC&_j_ %then %do;
                                 %let tempLen=&tempLen &&&temp._name&i $ &&&temp._len&i;
                              %end;
                              %else %if &&&temp._len&i < &&&temp._lenC&_j_ %then %do;
                                 %let tempLen=&tempLen &&&temp._name&i $ &&&temp._lenC&_j_;
                                 %let &temp._len&i =&&&temp._lenC&_j_;
                              %end;
                              %if %nrbquote(&&_nameC&_j_)=USUBJID %then %do;
                                 %let USUBflg=1;
                              %end;
                           %end;
                        %end;
                     %end;
                  %end;
               %end;
               %if &&&temp.Aflag=0 %then %do;
                   data ADAMLib.&temp;
                     set alib.&temp;
                   run;
                    %let &temp.Aflag=1;
               %end;
               %else %do;
                  data ADAMLib.&temp;
                     %if %length(&tempLen) %then length &tempLen;;
                     set ADAMLib.&temp alib.&temp;
                  run;
               %end;
            %end; /* end of ADaM combining */ 
            %end; /* end of unmatchFlag */
				%if &unmatchFlag %then %do;
					%put [MS_S_0065];
					%put ERROR: Variables &varDifflist for domain %upcase(&temp) are only in one of the studies. Studies can not be combined.;
					%put ERROR: Study &studyNameList can not be combined. All columns in all data sets must match ;
					%put ERROR: Please check the summary table for combined studies: &OutPath.&StudyName SummaryCombineStudies.pdf ;
					%put ERROR: If you want to continue to combine, please check the checkbox for Attempt study combination when study variables dont match.;
					%let exiterror = 1;
					%goto exit;
				%end;
          %end;/* end of domain checking*/
          %else %do;
          	%let &temp.NoDomains=&&&temp.NoDomains &&CombStudyName&_i_;
          	%let &temp.Nonumber = %eval(&&&temp.Nonumber+1);
          %end;
         
         %let j = %eval(&j+1);
      %end; /* end of j loop; */
   %end; /* end of %do loop;*/
   
   %let j=1;
   *check the uniqueness for each domain;
   %do %while(%length(%scan(&CombDomains,&j,'|')));
      * check if the two studies have the similar columns;
      %let temp = %scan(&CombDomains,&j,'|');

      %if &&domainFolder&j=SDTM & %upcase(&temp)=DM & %length(&InPath) %then %do;
         data SDTMLib.&temp;
          set SDTMLib.&temp;
         run;
                     
         %NObsVars(SDTMLib.&temp);
         %let sdtmfobs = &nObs;
         proc sort data=SDTMLib.&temp nodupkey out=sdtmNodup;
            by USUBJID;
         run;
         %NObsVars(sdtmNodup);
         %let sdtmfobs2 = &nObs;
         %put sdtmfobs=&sdtmfobs sdtmfobs2=&sdtmfobs2;
         %if &sdtmfobs ^= &sdtmfobs2 %then %do;
            %put WARNING: After combining studies &studyNameList, the new study &StudyName contains subjects with duplicate USUBJID.;
         %end;
      %end;   
      
      %else %if &&domainFolder&j=ADAM & %length(&AInPath) %then %do;
      *concatenate STUDYID to USUBJID for the new study;
         data ADAMLib.&temp;
            set ADAMLib.&temp;
         run;
      %end;
         
      %if %upcase(&temp)=ADSL & %length(&AInPath) %then %do;
         %NObsVars(ADAMLib.&temp);
         %let sdtmfobs = &nObs;
         proc sort data=ADAMLib.&temp nodupkey out=sdtmNodup;
            by USUBJID;
         run;
         %NObsVars(sdtmNodup);
         %let sdtmfobs2 = &nObs;
         %if &sdtmfobs ^= &sdtmfobs2 %then %do;
            %put WARNING: After combining studies &studyNameList, the new study &StudyName contains subjects with duplicate USUBJID.;
         %end;
      %end;
      %let j = %eval(&j+1);
   %end;
%end;
%*put pdfFileFlag=&pdfFileFlag;
%if &pdfFileFlag = Yes %then %do;
	options orientation = landscape nodate missing = "" sysprint="";
	ods escapechar = "^";
	ods listing close;
	ods pdf file = "&OutPath.&StudyName SummaryCombineStudies.pdf" style = journal;
	
   * print out avaiable domains for each study;
   %do _i_ =1 %to &totalStudies;
   	 %let j=1;
   	%do %while(%length(%scan(&CombDomains,&j,'|')));
	  		* check if the two studies have the similar columns;
     		%let temp = %scan(&CombDomains,&j,'|');
     		%*put &temp.AllDomains = &&&temp.AllDomains  &&&temp.Allnumber &temp.NoDomains=&&&temp.NoDomains;
     		%if &&&temp.Allnumber >1 %then %do;
     			ods pdf text = "^S={font=('Arial')}";
     			ods pdf text = "^S={font=('Arial')}";
     			ods pdf text = "^S={font=('Arial')}";
     			ods pdf text = "^S={font=('Arial') just=center} &temp Domain Variables Available for Studies";
     			ods pdf text = "^S={font=('Arial')}";
     		 	proc print width = min noobs label data = t&temp
     		 		style(bylabel) = [font_size = 7pt] style(obs) = [font_size = 7pt] style(obsheader) = [font_size = 7pt];
     		 		var Variable Domain &&&temp.AllDomains / style(data) = [font_size = 7pt] style(header) = [background=white font_size = 7pt];
     		 	run;
     		%end;
     		%if &&&temp.Nonumber >=1 %then %do;
				ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
            ods pdf text = "^S={font=('Arial') just=center} There is no &temp domains in";
            ods pdf text = "^S={font=('Arial') just=center} &&&temp.NoDomains";
            ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial')}";
     		%end;
     		%let j = %eval(&j+1);
     	%end;
   %end;   
	ods pdf close;
   ods listing;
%end;

%put "Finishing combing data sets ";
* use jsl to handle local files so this works in local and client-server mode;
* create JSL file;
%PrepOutFile(CombineStudiesScript,CombineStudiesScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &CombineStudiesScript;
%let FullAPName=%str(Combine Clinical Studies);

data _null_;
	file "&CombineStudiesScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "objRefNS = newNamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "jg:FromCombFlag=""&pdfFileFlag"";";
	put "jg:SDTMDom=""&SDTMDom"";";
	put "jg:ADaMDom=""&ADaMDom"";";
	put "jg:SDTMFFlag=""&SDTMFFlag"";";
	put "jg:ADaMFFlag=""&ADaMFFlag"";";
	put "objRefNS:FullAPName = ""&FullAPName"";";
	put "objRefNS:StudyName = ""&StudyName"";";
	put "sasmsgwindow=New Window(locale:get(""Results_Text"") || "": "" || locale:get(""CombineClinicalStudies_Text""),   Show Menu( 0 ), Show Toolbars( 0 ),";
	put "   VListBox(";
   put "      TextBox(locale:fill(locale:get(""ThenewstudyXiscreatedbycombiningstudiesX_Text""), {""&StudyName"",""&studyNameList"",""""})),"; 
	put "      TextBox("" ""),";
   %if &sdtmfobs ^= &sdtmfobs2 %then %do;
   put " TextBox(""After combining &studyNameList, the new study &StudyName contains subjects with duplicate USUBJID.""),";
	put "      TextBox("" ""),";
	%end;
	put "      TextBox(locale:get(""ItisreadywhentheAddStudyComplMsgAppears_Text"")),";	
	put "      TextBox("" ""),";
	put "   );";   
	put ");";
run; 
/*
%let tid = %sysfunc(open(Userlib.&metaData));

%let nObs = %sysfunc(attrn(&tid,NOBS));
%let nVars = %sysfunc(attrn(&tid,NVARS));
%put nObs =&nObs nVars=&nVars;
%do i = 1 %to &nVars;
	%let vname = %sysfunc(varname(&tid,&i)); 
	%do j = 1 %to &nObs;
	%end;
%end;

%let tid = %sysfunc(close(&tid));*/

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &CombineStudiesScriptName, , text/plain, yes);

%*include "&CLINICAL_HOME.\ProcessLibrary\AddStudyLocal.sas"; 

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, 1);

%mend CombineStudies;

* run macro;
%CombineStudies;
