/*
JMP Clinical Analytical Process

Name: Correlated Findings
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   This process calculates pairwise correlations between tests within each findings domain and identifies unusual results at
   study sites.
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName CorrFindScript CorrFind_OutData OutPath 
        ServerOutPath PopulationSet IgnoreSubFl IgnoreCovFl WhereStmt WhereForReport SiteCount ResType FindTests AllFind DQFByVars AlphaValue;

*ProcessBody;

%macro CorrelatedFindings;

/*
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and FINDINGS.USUBJID and FINDINGS.xxTEST and FINDINGS.xxTESTCD and (FINDINGS.xxSTRESN or FINDINGS.xxORRES)$ 
$CMP: CPV.{"Analyze:","Numeric results in standard units"}$ $UE: FINDINGS.xxSTRESN$ $NOTES: Only domains with these criteria are used$
$CMP: CPV.{"Analyze:","Results in original units"}$ $UE: FINDINGS.xxORRES$ $NOTES: Only domains with these criteria are used$
$CMP: OSYS$ $UE: FINDINGS.VISITNUM or FINDINGS.xxLOC or FINDINGS.xxMETHOD or FINDINGS.xxPOS or FINDINGS.xxSPEC or FINDINGS.xxTPT or FINDINGS.xxCAT or FINDINGS.xxSCAT$ 
            $NOTES: Used for transposing related records with one another for calculating correlation$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: OSYS$ $UE: FINDINGS.xxDTC $ $NOTES: Used to remove duplicates if usubjid, covariates and xxTEST does not make data unique for a transpose$
$CMP: OSYS$ $UE: FINDINGS.xxSEQ$ $NOTES: Used for uniqueness of rows or generated if not available.$  
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = CorrelatedFindings; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds = ;
%OUTNAMETRUNC(outdat = CorrFind_OutData, truncnum = 17);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);

%let trCorrelatedFindings = %nrbquote(%LocaleGet(CorrelatedFindings_Text));

%let availabledomains =;
%let domainlist = &findingsdomains;

%if ^%length(&findingsdomains) %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA(varlist = arm);
%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);

%GETSITELISTS(errflg = 1);
%if &exiterror %then %goto exit;

%DOMBYLISTS(inclvisit = 1, incllabs = 0);
%if %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &domainlist, flist = &FindTests);
%end;

%let flag = 1;
%let TestLen = 0;
%let i = 1;
%do %while(%length(%scan(&domainlist,&i,' ')));
   %let temp = %scan(&domainlist,&i,' ');
   %let tind = %sysfunc(left(%sysfunc(countw(%sysfunc(compbl("&&&var_&temp")), " "))));
   
   %CHECKPRES(domain = &temp, var = &temp.DTC);
   %CHECKPRES(domain = &temp, var = VISITNUM);
   
   %if %upcase(&AllFind) = NO %then %do;
      %let ifcond = and %length(&&&temp.findlist);
   %end;
   %else %do;
      %let ifcond =;
   %end;     

   %if %sysfunc(indexw(&&&temp.VARS, &&&var_&temp, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TESTCD, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TEST, ' ')) &ifcond %then %do;   
      %if %sysfunc(findw(%upcase(&&&temp.VARS), &temp.SEQ)) %then %do;
         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
         data &temp(keep = usubjid &temp.seq &&&temp.DTC &temp.test &temp.testcd &&&byvar_&temp &&&var_&temp);
            set &&in_&temp;
            %if %upcase(&AllFind) = NO %then %do;
               where &temp.testcd in ( %unquote(&&&temp.findlist) ) and not missing (&&&var_&temp);
            %end;
            %else %do;
               where not missing (&&&var_&temp);
            %end;                        
         run;
      %end;
      %else %do;
         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
         data &temp(keep = usubjid &temp.seq &&&temp.DTC &temp.test &temp.testcd &&&byvar_&temp &&&var_&temp);
            set &&in_&temp;
            %if %upcase(&AllFind) = NO %then %do;
               where &temp.testcd in ( %unquote(&&&temp.findlist) ) and not missing (&&&var_&temp);
            %end;
            %else %do;
               where not missing (&&&var_&temp);
            %end;     
         run;
         
         data &temp;
            set &temp;
            by usubjid;
            retain &temp.seq;
            if first.usubjid then &temp.seq = 1;
            else &temp.seq = &temp.seq + 1;           
         run;         
      %end;
      
      %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;                  
         %let ORRESLabel = &&&var_&temp;
         %CheckLabel(&temp,ORRESLabel);
         
         data &temp(where = (not missing (&&&var_&temp)));
            set &temp(rename = (&&&var_&temp = cval));
            &&&var_&temp = input(cval,best12.);
            drop cval;
            label &&&var_&temp = "&ORRESLabel";
         run;     
      %end;
            
      %let _tid_ = %sysfunc(open(&temp));
      %let vn = %sysfunc(varnum(&_tid_,&temp.test));
      %let TestLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let _tid_ = %sysfunc(close(&_tid_));
      %if %eval(&TestLentmp>&TestLen) %then %let TestLen = &TestLentmp; 

      data &temp;
         set &temp;
         if &temp.testcd = "" or &temp.test = "" then delete;     
      run;

      proc sort data = &temp out = &temp;
         by usubjid &&&byvar_&temp &temp.testcd;
      run;     

      %REMOVEDUPS;
      %nobsvars(&temp);
      
      %if &nobs %then %do;    
         %let codes_&temp = ;
         %let numcodes_&temp = 0;
         proc sql noprint;      
            select distinct &temp.testcd
            into :codes_&temp separated by ' '
            from &temp
            order by &temp.testcd;
            %let numcodes_&temp = &sqlobs;
         quit;
         *** cannot correlate only on one test code ***;
         %if &&numcodes_&temp = 1 %then %goto skipit;
         
         %let availabledomains = &availabledomains &temp;
         %let codes_&temp = %sysfunc(translate(&&&codes_&temp,_,/));

         %let ncodes_&temp=;
         %let j = 1;
         %do %while(%length(%scan(&&codes_&temp,&j,' ')));
            %let temptest = %scan(&&codes_&temp,&j,' ');
            %if &j = 1 %then %let ncodes_&temp = N&temptest;
            %else %let ncodes_&temp = &&ncodes_&temp N&temptest;
            %let j = %eval(&j+1);
         %end;

         proc sql noprint;
            create table testcd as        
            select distinct &temp.testcd, &temp.test
            from &temp
            order by upcase(&temp.test);
         quit;

         data testcd;
            set testcd;
            retain ORDER;
            if _n_ = 1 then order = 1;
            else order = order + 1;
         run;

         proc sql noprint;
            create table testcdpairs as
               select distinct aa.TESTCD, aa.TEST, aa.ORDER, bb.testcd2, bb.test2, bb.ORDER2 
               from testcd(rename = (&temp.TESTCD = TESTCD &temp.TEST = TEST)) as aa, testcd(rename = (&temp.TESTCD = TESTCD2 &temp.TEST = TEST2 ORDER = ORDER2)) as bb
               where upcase(aa.test) >= upcase(bb.test2)
               order by aa.testcd, bb.testcd2;
         quit;

         proc transpose data = &temp out = &temp(drop = _name_ _label_);
            by usubjid &&&byvar_&temp;
            var &&&var_&temp;
            id &temp.testcd;
            idlabel &temp.test;
         run;

         data &temp;
            merge subject(in=ina keep = usubjid siteid) &temp(in=inb);
            by usubjid;
            if ina and inb;        
         run;

         proc sort data = &temp out = &temp;
            by siteid;
         run;

         %let CorrFind_OutData_&temp =;      
         %PrepOutDS(CorrFind_OutData_&temp,OutDataName_&temp, &Outname.CorrF, _&temp);
         %let includeds = &includeds &&&&OutDataName_&temp...sas7bdat,;

         data &&&CorrFind_OutData_&temp;
            length domain $ 8;
            set &temp;
            Domain = "&temp";
            label Domain = "&trDOMAIN";
         run;

         %let firstflag = 0;
         %let j = 1;
         %do %while(%length(%scan(&SiteList,&j,' ')));
            %let tempsite = %scan(%upcase(&SiteList),&j," ");
            %let tempsitesize = %scan(%upcase(&SiteSamp),&j," ");
            %let trSiteTS = %nrbquote(%LocaleFill(&trSiteX, f1=%nrbquote(&tempsite)));
            %let trNotSiteTS = %nrbquote(%LocaleFill(&trNotSiteX, f1=%nrbquote(&tempsite)));

            data temp;
               length Label $ 30;
               set &temp;
               if siteid = "&tempsite" then Label = "&trSiteTS";
               else Label = "&trNotSiteTS";
               label Label = "&trLabel";
            run;

            proc sort data = temp;
               by label;
            run;

            proc corr outp = stats data = temp;
               ods exclude all;
               ods output pearsoncorr = pearsoncorr;
               ods output simplestats = simplestats(keep = label Variable NObs);
               var &&codes_&temp;
               by label;
            run;
            ods select all;

            proc sort data = pearsoncorr;
              by label Variable;
            run;
            
            proc sort data = simplestats;
              by label Variable;
            run;            

            data pearsoncorr;
               merge pearsoncorr simplestats;
               by label Variable;
            run;

            %let tempval =;         
            %let _tid_ = %sysfunc(open(pearsoncorr));
            %let k = 1;
            %do %while(%length(%scan(&&ncodes_&temp,&k,' ')));
               %let temptest = %scan(&&ncodes_&temp,&k,' ');
               %if %sysfunc(varnum(&_tid_,&temptest)) %then %let tempval = &tempval &temptest;
               %let k = %eval(&k+1);
            %end;       
            %let _tid_ = %sysfunc(close(&_tid_));        

            proc transpose data = pearsoncorr out = ptempcorr(rename = (col1 = RHO Variable = TESTCD _NAME_ = TESTCD2));
               by label Variable nobs;
               var &&codes_&temp;
            run;

            %if %length(&tempval) %then %do;
               proc transpose data = pearsoncorr out = ptempsamp(rename = (col1 = NPAIRS Variable = TESTCD _NAME_ = TESTCD2));
                  by label Variable nobs;
                  var &tempval;
               run;

               data ptempsamp;
                  set ptempsamp;
                  TESTCD2 = substr(TESTCD2,2);
               run;
               
               proc sort data = ptempcorr;
                  by label TESTCD nobs TESTCD2;
               run;
               
               proc sort data = ptempsamp;
                  by label TESTCD nobs TESTCD2;
               run;              

               data ptempcorr;
                  merge ptempcorr ptempsamp;
                  by label TESTCD nobs TESTCD2;
               run;
            %end;

            proc sort data = ptempcorr;
               by TESTCD TESTCD2;
            run;

            data ptempcorr(drop = nobs /* where = (NPAIRS > . and rho > .)*/);
               length siteid $ &SiteidLen domain $ 8;
               merge testcdpairs(in = ina) ptempcorr;
               by TESTCD TESTCD2;
               if ina;
               if missing(NPAIRS) then NPAIRS = nobs;
               if Label = "&trNotSiteTS" then label2 = "NotSite";
               else label2 = "Site";
               Domain = "&temp";
               SITEID = "&tempsite";
               SITESAMP = &tempsitesize;
               if testcd = testcd2 then npairs = .;
               if testcd = testcd2 then rho = .;
               label Domain = "&trDOMAIN" siteid = "&SITEIDlabel" sitesamp = "&trSiteSampleSize";                  
            run;

            proc sort data = ptempcorr;
               by TESTCD TEST TESTCD2 TEST2;
            run;

            proc datasets nolist;
               delete ptempsamp simplestats;
            quit;

            proc transpose data = ptempcorr prefix = rho_ out = ptempcorr_rho;
               by domain TESTCD TEST TESTCD2 TEST2;
               var RHO;               
               id label2;
            run;

            proc transpose data = ptempcorr prefix = npairs_ out = ptempcorr_npairs;
               by domain TESTCD TEST TESTCD2 TEST2;
               var NPAIRS;               
               id label2;
            run;

            data ptempcorrtests(drop = _name_);
               length siteid $ &SiteidLen Label $ 30;
               merge ptempcorr_rho ptempcorr_npairs;
               by domain TESTCD TEST TESTCD2 TEST2;
               CORRDIFF = rho_Site - rho_NotSite;
               TCORRDIFF = log((1+rho_Site)/(1-rho_Site))/2 - log((1+rho_NotSite)/(1-rho_NotSite))/2;
               if npairs_Site > 3 and npairs_NotSite > 3 then ZCORRDIFF = TCORRDIFF/sqrt(1/(npairs_Site-3)+1/(npairs_NotSite-3));
               if ZCORRDIFF > . then PCORRDIFF = 2*probnorm(-abs(ZCORRDIFF));
               if missing(PCORRDIFF) and (TESTCD ne TESTCD2) then PCORRDIFF = 1;
               Nlog10Pvalue = -log10(PCORRDIFF);            
               Label = "&trSiteTS";
               SITEID = "&tempsite";
               SITESAMP = &tempsitesize;
               label Label = "&trLabel" siteid = "&SITEIDlabel" Nlog10Pvalue = "&trNegLog10p" sitesamp = "&trSiteSampleSize";
            run;

            proc datasets nolist;
               delete ptempcorr_rho ptempcorr_npairs;
            quit;

            %if ^&firstflag %then %do;
               data volcano&temp;
                  set ptempcorrtests(drop = testcd testcd2);
               run;
               data heatmap&temp;
                  set ptempcorr(drop = testcd testcd2);
               run;
               %let firstflag = 1;
            %end;
            %else %do;   
               proc append force base = heatmap&temp data = ptempcorr(drop = testcd testcd2);
               run;
               proc append force base = volcano&temp data = ptempcorrtests(drop = testcd testcd2);
               run;     
            %end;

            proc datasets nolist;
               delete temp ptempcorr ptempcorrtests;
            quit;

            %let j = %eval(&j+1);
         %end;
         
         %skipit:;
      %end;
   %end;
   proc datasets nolist;
      delete &temp testcd testcdpairs;
   quit; 
   %let i = %eval(&i+1);   
%end;

%if &TestLen %then %do;
   data summary_volcano;
      length TEST TEST2 $ &testlen;
      set 
      %let i = 1;
      %do %while(%length(%scan(&availabledomains,&i,' ')));
         %let temp = %scan(&availabledomains,&i,' ');
         volcano&temp
         %let i = %eval(&i+1);
      %end;
      ;
   run;

   data testlist;
      length TEST $ &testlen;
      set summary_volcano(keep = TEST) summary_volcano(keep = TEST2 rename = (TEST2=TEST));
   run;

   proc sql noprint;      
      select distinct test
         into :SwitchList separated by ','
         from testlist
         order by upcase(test);     
   quit;

   proc datasets nolist;
      delete testlist;
   quit;

   /*
   %let QSwitchList = ;
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&SwitchList),&i,',')));
      %let temp = %kscan(%bquote(&SwitchList),&i,',');
      %if &i = 1 %then %let QSwitchList = %nrbquote(""&temp"");
      %else %let QSwitchList = &QSwitchList,%nrbquote(""&temp"");
      %let i = %eval(&i+1);
   %end; */

   proc datasets nolist;
      delete 
      %let i = 1;
      %do %while(%length(%scan(&availabledomains,&i,' ')));
         %let temp = %scan(&availabledomains,&i,' ');
         volcano&temp
         %let i = %eval(&i+1);
      %end;
      ;
   quit;

   data summary_heatmap;
      length TEST TEST2 $ &testlen;
      set 
      %let i = 1;
      %do %while(%length(%scan(&availabledomains,&i,' ')));
         %let temp = %scan(&availabledomains,&i,' ');
         heatmap&temp
         %let i = %eval(&i+1);
      %end;
      ;
   run;

   proc datasets nolist;
      delete 
      %let i = 1;
      %do %while(%length(%scan(&availabledomains,&i,' ')));
         %let temp = %scan(&availabledomains,&i,' ');
         heatmap&temp
         %let i = %eval(&i+1);
      %end;
      ;
   quit;

   %FDRPVALUES(inpv = PCORRDIFF, nlog10p = Nlog10Pvalue);

   %let CorrFind_OutData_v =;    
   %PrepOutDS(CorrFind_OutData_v,OutDataName_v, &Outname.CorrF, _vol);
   %let includeds = &includeds &OutDataName_v..sas7bdat,;

   proc sort data = summary_volcano;
      by siteid domain test test2;
   run;

   %nobsvars(summary_volcano);

   data &CorrFind_OutData_v;
      merge summary_volcano(in = ina) country(in = inb);
      by siteid;
      if ina;
      label Test = "&trTestName" Test2 = "&trTestName 2" rho_Site = "&trPearson &trSite" rho_NotSite = "&trPearson &trNotSite"
            npairs_Site = "&trNumberofPairs &trSite" npairs_NotSite = "&trNumberofPairs &trNotSite" corrdiff = "&trDiffCorr"
            zcorrdiff = "&trDiffCorrTest" pcorrdiff = "&trDiffCorrTestp" tcorrdiff = "&trDiffTrCorr";
   run;

   %let CorrFind_OutData_h =;    
   %PrepOutDS(CorrFind_OutData_h,OutDataName_h, &Outname.CorrF, _heat);
   %let includeds = &includeds &OutDataName_h..sas7bdat,;

   data &CorrFind_OutData_h;
      set summary_heatmap;
      label Test = "&trTestName" Test2 = "&trTestName 2" rho = "&trPearson" npairs = "&trNumberofPairs";
      drop label2;
   run;
%end;

* create JSL file;
%PrepOutFile(CorrFindScript,CorrFindScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &CorrFindScript;
%let FullAPName=%str(Correlated Findings);

* put JSL statements;
data _null_;
   file "&CorrFindScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:suffix = ""&_nThePro_"";";
   /* put "objRefNS:tmlist={";
   %let top = %sysfunc(countw(%bquote(&QSwitchList), '","'));
   %do i = 1 %to &top;
      %if &i = &top %then %do;
         put "%scan(%bquote(&QSwitchList),&i,',')";
      %end;
      %else %do;
         put "%scan(%bquote(&QSwitchList),&i,','),";
      %end;
   %end;
   put "};"; */
   put "objRefNS:mergelist = ""&SITEIDLabel~&trDOMAIN"";";
   put "objRefNS:SiteLab = ""&SITEIDLabel"";";
   put "objRefNS:TestLab = ""&trTestName"";";
   put "objRefNS:TestLab2 = ""&trTestName 2"";";
   put "objRefNS:DomainLab = ""&trDOMAIN"";";
   put "objRefNS:RhoLab = ""&trPearson"";";
   put "objRefNS:hist = ""&ClientOutPath.&OutDataName_h..sas7bdat"";";  
   put "objRefNS:clientoutpath = ""&ClientOutPath"";";
   %let i = 1;
   %do %while(%length(%scan(&availabledomains,&i,' ')));
      %let temp = %scan(&availabledomains,&i,' ');
         put "objRefNS:bylist_&temp = ""&&byvar_&temp"";";
      %let i = %eval(&i+1);
   %end; 
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   %if &TestLen %then %do;
      put "  objRefNS:dvolcano = open(""&ClientOutPath.&OutDataName_v..sas7bdat"",invisible);";
      put "  Column(""&SITEIDLabel"") << Label(1);";
      put "  Column(""&trTestName"") << Label(1);";
      put "  Column(""&trTestName 2"") << Label(1);";
      put "  Column(""&trNumberofPairs &trSite"") << Label(1);";
      put "  Column(""&trSiteSampleSize"") << Label(1);";         
   %end;
   put "objRefNS:volcano=Function({},{default local},";
   %if &TestLen %then %do;
      put "   VListBox(";
      %if %length(&RawCutOff) %then %do;
         put "   objRefNS:T1 = TextBox(locale:fill(locale:get(""XLineDrawnatMinusLogpX_Text""), {locale:get(""FDR_Text""), ""%sysfunc(round(&RawCutOff,0.0001))""})||"" (alpha=&AlphaValue)""),"; 
      %end;
      %else %do;
         put "   objRefNS:T1 = TextBox(locale:fill(locale:get(""NoPvalsSigAtAlphaXAfterAppXAdj_Text""), {""&AlphaValue"", locale:get(""FDR_Text"")}),<<Set Wrap(1000)),"; 
      %end;
      put "   HListBox(";
      put "   objRefNS:VolcPlot = Graph Builder(";
      put "    Auto Stretching(0),";
      put "    Automatic Recalc(1),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "    X( :&trDiffCorr ),";
      put "    Y( :Name( ""&trNegLog10p"" ) ),";
      put "    Color( :&SITEIDLabel )";
      put "   ),";
      put "   Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       OutlineBox,";
      put "       {Set Title( locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""CorrelatedFindings_Text"")}) )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trDiffCorr"",";
      put "       ScaleBox,";
      put "       {}";
      put "    ),";
      %if %length(&RawCutOff) %then %do;
         put "    Dispatch(";
         put "       {},";
         put "       ""&trNegLog10p"",";
         put "       ScaleBox,";
         put "       {Add Ref Line(";
         put "          &RawCutOff,";
         put "          Dashed,";
         put "          ""Red"",";
         put "          """",";
         put "          2";
         put "       )}";
         put "    ),";
      %end;
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 6 )} )";
      put "  ))";
      put ");";
      put ");";
   %end;
   %else %do;
      put "VListBox(";
      put "   objRefNS:T1 = TextBox(locale:get(""NowithindomainpairsoftestswithNumResStdUnsIdd_Text""),<<Set Wrap(1000), <<Font Color(5)),"; 
      put ");";   
   %end;
   put ");";

   %if &TestLen %then %do;
      put "    objRefNS:CustomPanel = Expr(objRefNS:Filter = objRefNS:dvolcano << Data Filter(<<Conditional(1),";
      put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&trDOMAIN"")),";
      put "       Mode( Show(1), Include(1) )";
      put "    ));";     
   %end;

   put "InsertInto(objRefNS:TabOpenList,1);";
   %if &TestLen %then %do;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dvolcano""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trCorrelatedFindings""}});";
   %end;
   %else %do;
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
   %end;
   put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""CorrelatedFindings_Text"")})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T1,objRefNS:VolcPlot}});";
   
   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";

   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsthenClick_Text""))});";  
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSites_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSite()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";    
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowCorrelationPlot_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:CorrelationPlot()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";  
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowFindings_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowFindings()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";     
   put "objRefNS:ActionSelectionCountList = {{1,1,1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.DataSetHighlightRows.gif"",""&IconPath.HeatMap.gif"",""&IconPath.ColumnsShow.gif""}};";
   put "objRefNS:ActionRequirementFlagList = {{0, 0, 0}};";
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CorrFindScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend CorrelatedFindings;
%CorrelatedFindings;

************************************;
************************************;
************************************;
