/*
JMP Clinical Analytical Process

Name: Create Cross Domain Data
Authors: Geoff Mann and Russ Wolfinger, SAS Institute Inc.

Description: 
	This process adds data for various domains to the adsl table corresponding to incidence of
	variables in interventions, events, and character findings domains and a summary statistic for 
	numeric findings domains. 
	The resulting output data set is suitable for 
	pattern discovery and predictive modeling. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2009 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*		Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"EventsO"},{}}$
$CMP: CPV.{"Interventions","Yes"}$ $UE: INTERVENTIONS.xxDECOD or INTERVENTIONS.xxTERM or INTERVENTIONS.xxTRT$ 
$CMP: CPV.{"Events","Yes"}$ $UE: EVENTS.xxDECOD or EVENTS.xxTERM or EVENTS.xxTRT$
$CMP: CPV.{"Binary Variables in ADSL","Yes"}$ $UE: DM.*FL$
$CMP: CPV.{"Character Findings","Yes"}$ $UE: FINDINGS.xxTESTCD or FINDINGS.xxSTRESN or FINDINGS.xxSTRESC or FINDINGS.xxSPEC$
$CMP: CPV.{"High and Low Labs","Yes"}$ $UE: LB.LBSTNRHI or LB.LBSTNRLO$
$CMP: CPV.{"Term Level","Reported Term for the Adverse Event"}$
	$UE: AE.AETERM$
	$NOTES: Also supplies Group Level Parameter.$	
$CMP: CPV.{"Term Level","Lowest Level Term"}$
	$UE: AE.AELLT$
	$NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Term Level","Dictionary-Derived Term"}$
	$UE: AE.AEDECOD$
	$NOTES: Also supplies Group Level Parameter. Default value for Term Level if found in data.$
$CMP: CPV.{"Term Level","Body System or Organ Class"}$
	$UE: AE.AEBODSYS$
	$NOTES: Also supplies Group Level Parameter. Default value for Group Level if found in data$
$CMP: CPV.{"Term Level","High Level Term"}$
	$UE: AE.AEHLT$
	$NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Term Level","High Level Group Term"}$
	$UE: AE.AEHLGT$
	$NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$	
$CMP: CPV.{"Term Level","Primary System Organ Class"}$
	$UE: AE.AESOC$
	$NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Create summary statistic variables for numeric findings","Yes"}$ $UE: FINDINGS.xxSTRESN$	
$CVS_STOP$
*/


*ProcessVariables;

%global StudyName InPath AInPath SubjectFilter WhereStmt TimeWindows TimeScale 
	Ind_Interventions Ind_Events Ind_AD Ind_Findings Ind_LB SS_Findings UNL_Mult LNL_Div
	IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL OutPath ServerOutPath 
	NormRef LogShft CDSTAT AETermLevel CDD_OutData CDDTrans_OutData CDDScript ClusterSubjects MacroPath OutData JSLFile MergeSuppDom;

*ProcessBody;

*options mlogic mprint symbolgen;

%macro CreateCrossDomainData;

*initialization and argument checking;
%let ProcessName = CreateCrossDomainData; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
%include "&MacroPath./CategoryIndicators.sas" / nosource;
%include "&MacroPath./FindingsMacros.sas"/nosource;

%if %UPCASE(&LogShft)=NONE %then %do;
	%if %UPCASE(&NormRef) = GEOMETRIC %then %do;
		%put [MS_S_0067];
		%put ERROR: The Geometric normalization method can only be used when you specify a log transformation of the lab measurements.;
		%let exiterror=1;
		%goto exit;
	%end;
	%let LogShft = ;
%end;
%if %UPCASE(&NormRef) = NONE %then %let NormRef = ;

%if &Ind_Interventions=No and &Ind_Events=No and &Ind_AD=No and &Ind_Findings=No and &Ind_LB=No and &SS_Findings=No %then %do;
	%put [MS_S_0068];
	%put ERROR: You must select at least one set of domains for which to compute indicator variables or summary statistics.;
	%let exiterror = 1;
	%goto exit;
%end;

%let varlist = ;
%let domindlist = ;
%let domindlistc = ;
%let domsslist = ;

* in adsl, convert all variables with values N and Y to have values 0 and 1, respectively;
%if &Ind_AD=Yes %then %do;
	data adsln;
		set &in_adsl;
		keep USUBJID _numeric_;
	run;

	data adslc;
		set &in_adsl;
		keep _character_;
	run;

	%NObsVars(adslc);
	%let ns = &nobs;

	proc transpose data=adslc out=c prefix=S_;
		var _character_;
		id USUBJID;
	run;

	data c1 c2;
		set c;
		array s{*} S_:;
		array z{*} z1-z&ns;
		ynvar = 1;
		do i=1 to &ns;
			if not missing(s[i]) and upcase(s[i]) not in ("Y","N") then do;
				ynvar = 0;
				i = &ns;
			end;
		end;
		if (ynvar) then output c1;
		else output c2;
		drop i;
	run;
	
	%NObsVars(c1);
	
	%if &nobs %then %do;
		data c1;
			length _label_ $ 32;
			set c1;
			array s{*} S_:;
			array z{*} z1-z&ns;
			do i=1 to &ns;
				if missing(s[i]) then z[i] = .;
				else if upcase(s[i]) = "Y" then z[i] = 1;
				else z[i] = 0;
			end;
			_label_ = cats(_name_,"01");
			drop i S_:;
		run;	

		proc transpose data=c1 out=c1t(drop=_name_) prefix=ADSLI_;
			var z1-z&ns;
			id _name_;
			idlabel _label_;
		run;

		proc transpose data=c2 out=c2t(drop=_name_);
			var S_:;
		run;

		data u;
			set adsln;
			keep USUBJID;
		run;

		data c1t;
			merge u c1t;
		run;

		%let c1t = c1t;
		%let c2t = c2t;
		%let adsln = adsln;

		proc sort data=c1t;
			by USUBJID;
		run;

		proc sort data=c2t;
			by USUBJID;
		run;

		proc sort data=adsln;
			by USUBJID;
		run;
		
		%let varlist = &varlist ADSLI_: ;
		%let domindlist = &domindlist ADSL;
		%if %length(&domindlistc) %then %do;
			%let domindlistc = &domindlistc,"ADSL";
		%end;
		%else %do;
			%let domindlistc = "ADSL";
		%end;
	%end;
	%else %do;
		%let Ind_AD = No;
		%let c1t = ;
		%let c2t = ;
		proc sort data=&in_adsl out=adsl;
			by USUBJID;
		run;
		%let adsln = adsl;
	%end;
%end;
%else %do;
	%let c1t = ;
	%let c2t = ;
	proc sort data=&in_adsl out=adsl;
		by USUBJID;
	run;
	%let adsln = adsl;
	%NObsVars(&adsln);
	%let ns = &nobs;
%end;

* create treatment indicators;
%do p = 1 %to &NTrtPeriods;
	%let curt = %scan(&TrtVarList,&p," ");
	%CategoryIndicators(ind=&in_adsl,cat=&curt,id=USUBJID,CIPrefix=TRTI&p,outd=adslti&p);
%end;

data adsl_sub;
	set &adsln;
	keep usubjid;
run;

* create interventions indicators;
%let intif = ;
%let inti = ;
%if &Ind_Interventions=Yes and %length(&interventionsdomains) %then %do;
	%do ii=1 %to %sysfunc(countw(&interventionsdomains));
		%let domain = %scan(&interventionsdomains,&ii,' ');
		%if &domain ne EX %then %do;
			proc sql;
				create table &domain._sub
					as select *
					from adsl_sub as d1 left join &&in_&domain %if %sysfunc(indexw(&&&domain.VARS, %upcase(&domain)OCCUR, ' ')) > 0 %then (where=(&domain.occur ne "N")); as d2 on d1.usubjid=d2.usubjid;
			quit;		
			%let CIPrefix = &domain.I;
			%let CatName = &domain.DECOD;
			%let _tid_ = %sysfunc(open(&domain._sub));
			%let vn = %sysfunc(varnum(&_tid_,&CatName));
			%if ^&vn %then %do;
				%let CatName = &domain.TRT;
				%let vn = %sysfunc(varnum(&_tid_,&CatName));
				%if ^&vn %then %do;
					%put [MS_S_0069];
					%put ERROR: The variables &domain.DECOD and &CatName do not exist in the &domain data set.;
					%let exiterror = 1;
				%end;
				%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
				%if ^%length(trim(&CatLabel)) %then %let CatLabel = &CatName;
			%end;
			%else %do;
				%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
				%if ^%length(trim(&CatLabel)) %then %let CatLabel = &CatName;
			%end;
			%let _tid_ = %sysfunc(close(&_tid_));
			%if &exiterror %then %goto exit;
			* incorporate time windows if specified;
			%AddTimeWindows(domain=&domain,indat=&domain._sub,outdat=&domain._sub,twb=&TimeWindows,errflag=0);
			%if &exiterror %then %goto exit;

			%CategoryIndicators(ind=&domain._sub,cat=&CatName,id=USUBJID,CIPrefix=&CIPrefix,outd=&domain.i);
			%if &exiterror = 22 %then %do;
			   %let exiterror = 0;
			   %goto skipidom;
			%end;			
			%if &syserr>4 %then %goto exit;
			%let intif = &intif &domain.if;
			%let inti = &inti &domain.i;
			%let varlist = &varlist &domain.I_: ;
			%let domindlist = &domindlist &domain;
			%if %length(&domindlistc) %then %do;
				%let domindlistc = &domindlistc,"&domain";
			%end;
			%else %do;
				%let domindlistc = "&domain";
			%end;
		%end;
		%skipidom:;
	%end;
%end;

* create events indicators;
%let evtif = ;
%let evti = ;
%let Ind_MedDRA = No;
%if &Ind_Events=Yes and %length(&eventsdomains) %then %do;
	%do ii=1 %to %sysfunc(countw(&eventsdomains));
		%let domain = %scan(&eventsdomains,&ii,' ');
		proc sql;
			create table &domain._sub
				as select *
				from adsl_sub as d1 left join &&in_&domain %if %sysfunc(indexw(&&&domain.VARS, %upcase(&domain)OCCUR, ' ')) > 0 %then (where=(&domain.occur ne "N"));  as d2 on d1.usubjid=d2.usubjid;
		quit;

		%if &domain=AE %then %do;
			%let CIPrefix = AEI;
			%GETMEDDRAVAR(label = &AETermLevel, column = CatName1);
			%let CatName = %upcase(&CatName1);
			%put CatName = &CatName;
			%if ^%length(&CatName) %then %do;
				%if %sysfunc(findw(&AEVARS, AEDECOD)) %then %let CatName = AEDECOD;
				%else %let CatName = AETERM;
			%end;
			
			%if ^%sysfunc(findw(&AEVARS, &CatName)) %then %do;
				%put [MS_S_0017];
				%put ERROR: Selected AE term variable has no non-missing values.  Please select an alternate variable.;
				%let exiterror = 1;
				%goto exit;
			%end;

			%let _tid_ = %sysfunc(open(&domain._sub));
			%let vn = %sysfunc(varnum(&_tid_,&CatName));
			%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
			%if ^%length(%trim(&CatLabel)) %then %let CatLabel = &CatName;
			%put CatLabel = &CatLabel;
			%let _tid_ = %sysfunc(close(&_tid_));

			* incorporate time windows if specified;
			%AddTimeWindows(domain=&domain,indat=&domain._sub,outdat=&domain._sub,twb=&TimeWindows,errflag=0);
			%if &exiterror %then %goto exit;
			%CategoryIndicators(ind=&domain._sub,cat=&CatName,id=USUBJID,CIPrefix=&CIPrefix,outd=&domain.i);
			%if &exiterror = 22 %then %do;
			   %let exiterror = 0;
			   %goto skipedom;
			%end;
			%let evtif = &evtif &domain.if;
			%let evti = &evti &domain.i;
			%let varlist = &varlist &domain.I_: ;
			%let domindlist = &domindlist &domain;
			%if %length(&domindlistc) %then %do;
				%let domindlistc = &domindlistc,"&domain";
			%end;
			%else %do;
				%let domindlistc = "&domain";
			%end;
		%end;
		%else %do;
			%let CIPrefix = &domain.I;
			%let CatName = &domain.DECOD;

			%let _tid_ = %sysfunc(open(&&in_&domain));
			%let vn = %sysfunc(varnum(&_tid_,&CatName));
			%if ^&vn %then %do;
				%let CatName = &domain.TERM;
				%let vn = %sysfunc(varnum(&_tid_,&CatName));
				%if ^&vn %then %do;
					%put [MS_S_0069];
					%put ERROR: The variables &domain.DECOD and &CatName do not exist in the &domain data set.;
					%let exiterror = 1;
				%end;
				%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
				%if ^%length(trim(&CatLabel)) %then %let CatLabel = &CatName;
			%end;
			%else %do;
				%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
				%if ^%length(trim(&CatLabel)) %then %let CatLabel = &CatName;
			%end;
			%let _tid_ = %sysfunc(close(&_tid_));
			%if &exiterror %then %goto exit;
			* incorporate time windows if specified;
			%AddTimeWindows(domain=&domain,indat=&domain._sub,outdat=&domain._sub,twb=&TimeWindows,errflag=0);
			%if &exiterror %then %goto exit;

			%CategoryIndicators(ind=&domain._sub,cat=&CatName,id=USUBJID,CIPrefix=&CIPrefix,outd=&domain.i);
			%if &exiterror = 22 %then %do;
			   %let exiterror = 0;
			   %goto skipedom;
			%end;
			%if &syserr>4 %then %goto exit;
			%let evtif = &evtif &domain.if;
			%let evti = &evti &domain.i;
			%let varlist = &varlist &domain.I_: ;
			%let domindlist = &domindlist &domain;
			%if %length(&domindlistc) %then %do;
				%let domindlistc = &domindlistc,"&domain";
			%end;
			%else %do;
				%let domindlistc = "&domain";
			%end;
		%end;
		%skipedom:;
	%end;
%end;

* create character findings indicators;
%let fndif = ;
%let fndi = ;
%if &Ind_Findings=Yes and %length(&findingsdomainschar) %then %do;
	%do ii=1 %to %sysfunc(countw(&findingsdomainschar));
		%let domain = %scan(&findingsdomainschar,&ii,' ');
		%let CIPrefix = &domain.I;
		%let CatName = &domain.CHARF;
		%let trDomCharFind = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XCharacterFinding_Text)), &domain));
		%let CatLabel = &trDomCharFind; 
		proc sql;
			create table &domain._sub
				as select *
				from adsl_sub as d1 left join &&in_&domain as d2 on d1.usubjid=d2.usubjid;
		quit;

		%let _tid_ = %sysfunc(open(&domain._sub));
		%let gotstresn = %sysfunc(varnum(&_tid_,&domain.STRESN));
		%let gotspec = %sysfunc(varnum(&_tid_,&domain.SPEC));
		%let _tid_ = %sysfunc(close(&_tid_));
		
		* build a single category variable from xxTESTCD, xxSPEC, and xxSTRESC by concatenating them;
		data &domain._sub;
			set &domain._sub;
			length &CatName $ 128;
			if missing(&domain.TESTCD) or missing(&domain.STRESC) then delete;
			* delete values with nonmissing numeric finding;
			%if &gotstresn %then %do;
				if not missing(&domain.STRESN) then delete;
			%end;
			%if &gotspec %then %do;
				&CatName = catx(" ", of &domain.TESTCD &domain.SPEC &domain.STRESC);
			%end;
			%else %do;
				&CatName = catx(" ", of &domain.TESTCD &domain.STRESC);			
			%end;
		run;	
		
		%NObsVars(&domain._sub);
		%if ^&nobs %then %do;
			%put WARNING: All character findings without a comparable numeric finding for domain &domain are missing.;
			%put This domain will not be included in the analysis.;
		%end;
		%else %do;		
			* incorporate time windows if specified;
			%AddTimeWindows(domain=&domain,indat=&domain._sub,outdat=&domain._sub,twb=&TimeWindows,errflag=0);
			%if &exiterror %then %goto exit;

			%CategoryIndicators(ind=&domain._sub,cat=&CatName,id=USUBJID,CIPrefix=&CIPrefix,outd=&domain.i);
			%if &syserr>4 %then %goto exit;
			%let fndif = &fndif &domain.if;
			%let fndi = &fndi &domain.i;
			%let varlist = &varlist &domain.I_: ;
			%let domindlist = &domindlist &domain;
			%if %length(&domindlistc) %then %do;
				%let domindlistc = &domindlistc,"&domain";
			%end;
			%else %do;
				%let domindlistc = "&domain";
			%end;
		%end;
	%end;
%end;

* create hi/lo lab indicators and numeric findings summary stats;
%let fnds = ;
%let trHigh = %nrbquote(%LocaleGet(High_Text));
%let trLow = %nrbquote(%LocaleGet(Low_Text));
%if (&Ind_LB=Yes or &SS_Findings=Yes) and %length(&findingsdomainsnum) %then %do;
	%do ii=1 %to %sysfunc(countw(&findingsdomainsnum));
		%let domain = %scan(&findingsdomainsnum,&ii,' ');

		%let &domain.pos_=0;
		%let _tid_ = %sysfunc(open(&&in_&domain));
		%let &domain.pos_ = %sysfunc(varnum(&_tid_,&domain.POS));
		%let _tid_ = %sysfunc(close(&_tid_));

		%if &domain=LB and &Ind_LB=Yes %then %do;
			proc sql;
				create table hllb_sub
					as select *
					from adsl_sub as d1 left join &&in_&domain as d2 on d1.usubjid=d2.usubjid;
			quit;
			data hllb_sub;
				set hllb_sub;
				length HL $ 4 HLLB $ 32 TRHL $ 4 TRHLLB $ 32;
				if missing(LBSTRESN) then delete;
				else if LBSTNRHI*&UNL_Mult <= LBSTRESN then do; HL = "High"; TRHL = symget('trHigh'); end;
				else if LBSTRESN <= LBSTNRLO/&LNL_Div then do; HL = "Low"; TRHL = symget('trLow'); end;
				else delete;
				HLLB = HL || " " || LBTESTCD;
				TRHLLB = TRHL || " " || LBTESTCD;
			run;
			%let CatName  = HLLB;
			%let CatLabel = TRHLLB;
			
			* incorporate time windows if specified;
			%AddTimeWindows(domain=&domain,indat=hllb_sub,outdat=hllb_sub,twb=&TimeWindows,errflag=0);
			%if &exiterror %then %goto exit;			

			%CategoryIndicators(ind=hllb_sub,cat=&CatName,id=USUBJID,CIPrefix=HLLBI,CatLabel=&CatLabel,outd=hllbi);
			%if &syserr>4 %then %goto exit;
			%let evtif = &evtif hllbif;
			%let evti = &evti hllbi;
			%let varlist = &varlist HLLBI_: ;
			%let domindlist = &domindlist HLLB;
			%if %length(&domindlistc) %then %do;
				%let domindlistc = &domindlistc,"HLLB";
			%end;
			%else %do;
				%let domindlistc = "HLLB";
			%end;
		%end;
			
		%if &SS_Findings=Yes %then %do;

			proc sql;
				create table &domain._sub
					as select *
					from adsl_sub as d1 left join &&in_&domain as d2 on d1.usubjid=d2.usubjid;
			quit;

			%let CatName = &domain.NUMFSS;
			%let trDomNumFindSS = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XNumericFindingSummaryStatistic_Text)), &domain));
			%let CatLabel = &trDomNumFindSS;
			%let trCDSTAT = %nrbquote(%LocaleEZX(%nrbquote(&CDSTAT), EIA=1));
			
			data &domain;
				length &CatName $128;
				set &domain._sub;	
				%if &domain = LB %then %do;
					%LabsNormalize(&domain.STRESN, &NormRef, &LogShft);
				%end;
				if missing(&domain.TESTCD) or missing(&domain.STRESN) then delete;
				%if &&&domain.pos_ > 0 %then %do;
					&CatName = trim(left(&domain.TESTCD))||"_"||trim(left(&domain.POS))||"_&trCDSTAT";
				%end;
				%else %do;
					&CatName = trim(left(&domain.TESTCD))||"_&trCDSTAT";
				%end;
			run;
			
			%NObsVars(&domain);
			%if ^&nobs %then %do;
				%put WARNING: All measurements for domain &domain are missing. This domain will not be included in the analysis.;
				%if &domain = LB %then %do;
					%put You may wish to change the Normalization of Lab Measurements parameter on the Options tab.;
				%end;
			%end;
			%else %do;

				* incorporate time windows if specified;
				%AddTimeWindows(domain=&domain,indat=&domain,outdat=&domain,twb=&TimeWindows,errflag=0);
				%if &exiterror %then %goto exit;			

				* compute summary statistic;
				%if (&CDStat = Last) %then %do;
					data &domain.ss;
						set &domain;
						if missing(&domain.STRESN) then delete;
						%DATE9(var=&&domain.dtc,return=&domain.dt);
					run;

					proc sort data=&domain.ss;
						by &CatName USUBJID &domain.dt;
					run;

					data &domain.ss;
						set &domain.ss;
						by &CatName USUBJID &domain.dt;
						if last.USUBJID;
					run;

					proc sort data=&domain.ss;
						by USUBJID &CatName;
					run;
				%end;	
				%else %do;
					proc sort data=&domain;
						by USUBJID &CatName;
					run;

					proc means data=&domain noprint;
						by USUBJID &CatName;
						var &domain.STRESN;
						output out=&domain.ss &CDSTAT=&domain.STRESN;
					run;
				%end;

				proc transpose data=&domain.ss out=&domain.st(drop=_name_ _label_) prefix=&domain.S_;
					by USUBJID;
					var &domain.STRESN;
					id &CatName;
					idlabel &CatName;
				run;

				%let fnds = &fnds &domain.st;
				%let varlist = &varlist &domain.S_: ;
				%let domsslist = &domsslist &domain;
			%end;
		%end;
	%end;
%end;

* macro to set missing indicators and frequencies to zeros;
%macro zeromiss(d);
	if missing(FREQ_&d.I) then FREQ_&d.I = 0;
	array ind&d {*} &d.I_:;
	do _i=1 to dim(ind&d);
		if missing(ind&d[_i]) then ind&d[_i] = 0;
	end;
%mend;

%put varlist = &varlist;

%if ^%length(&varlist) %then %do;
	%put [MS_S_0070];
	%put ERROR: No domains found in this study satisfying the requested output.  Please change the latter.;
	%let exiterror = 1;
	%goto exit;
%end;

%let CDD_OutData = &OutData;
%let CDDScript=&JSLFile;
%PrepOutDS(CDD_OutData,OutDataName,adsl,_cdd);
%PrepOutDS(CDDTrans_OutData,OutTransName,adsl,_cddt);

proc sort data=&in_adsl out=adsl;
	by USUBJID;
run;

* join with adsl and set missing indicators to zeros;;	
data OutLib.&OutDataName;
	merge adsl %do p = 1 %to &NTrtPeriods; adslti&p %end; &intif &evtif &fndif &inti &evti &fndi &c1t &fnds &c2t;
	by USUBJID;
	%do p = 1 %to &NTrtPeriods;
		%let curt = %scan(&TrtVarList,&p," ");
		if missing(&curt) then delete;
	%end;
	%if &Ind_Interventions=Yes and %length(&interventionsdomains) %then %do;
		%do ii=1 %to %sysfunc(countw(&interventionsdomains));
			%let domain = %scan(&interventionsdomains,&ii,' ');
			%if &domain ne EX %then %do;
				%zeromiss(&domain);
			%end;
		%end;
	%end;
	%if &Ind_Events=Yes and %length(&eventsdomains) %then %do;
		%do ii=1 %to %sysfunc(countw(&eventsdomains));
			%let domain = %scan(&eventsdomains,&ii,' ');
			%zeromiss(&domain);
		%end;
	%end;
	%if &Ind_Findings=Yes and %length(&findingsdomainschar) %then %do;
		%do ii=1 %to %sysfunc(countw(&findingsdomainschar));
			%let domain = %scan(&findingsdomainschar,&ii,' ');
			%zeromiss(&domain);
		%end;
	%end;
	%if &Ind_MedDRA=Yes %then %do;
		%zeromiss(MedDRA);
	%end;
	%if &Ind_LB=Yes %then %do;
		%zeromiss(HLLB);
	%end;
	%if &Ind_AD=Yes %then %do;
		%zeromiss(ADSL);
	%end;
	drop _i;
run;

%NobsVars(OutLib.&OutDataName);
%let ns = &nobs;

data cddn;
	set OutLib.&OutDataName;
	keep USUBJID &varlist;;	
run;

proc transpose data=cddn out=cddt prefix=S_;
	var &varlist;
	id USUBJID;
run;

%if ^%length(&domindlistc) %then %let domindlistc = "YODOG";


%let trNumMiss = %nrbquote(%LocaleGet(NumMissNS_Text));
%let trMean = %nrbquote(%LocaleGet(Mean_Text));
%let trDomain = %nrbquote(%LocaleGet(Domain_Text));
* assign domains and impute missing values with row mean so they are included in clustering;
data OutLib.&OutTransName;
	length NumMiss 8 Mean 8 Domain $8;
	set cddt;
	if length(_NAME_) > 4 then do;
		first4 = upcase(substr(_NAME_,1,4));
		%do ii=1 %to %sysfunc(countw(&varlist));
			%let vi = %substr(%qscan(&varlist,&ii,' '),1,4);
			if first4 = "&vi" then Domain = "%substr(&vi,1,2)";
			if Domain = "Me" then Domain = "MD";
		%end;
	end;
	else do;
		Domain = "ADSL";
	end;
	array s{*} S_:;
	if Domain in (&domindlistc) and first4 ne "LBS_" then do;
		z1var = 1;
		do i=1 to &ns;
			if s[i] not in (.,0,1) then do;
				z1var = 0;
				i = &ns;
			end;
		end;
		if (z1var) then do;
			NumMiss = nmiss(of S_:);
			Mean = mean(of S_:);
			if NumMiss < &ns then do;
				if NumMiss > 0 then do;
					do i=1 to &ns;
						if missing(s[i]) then s[i] = Mean;  
					end;
				end;
				output;
			end;
		end;
	end;
	else do;
		NumMiss = nmiss(of S_:);
		Mean = mean(of S_:);
		if NumMiss < &ns then do;
			if NumMiss > 0 then do;
				do i=1 to &ns;
					if missing(s[i]) then s[i] = Mean;  
				end;
			end;
			output;
		end;
	end;
	drop i z1var first4;
	label NumMiss = "&trNumMiss" Mean = "&trMean" Domain = "&trDomain";
run;

* Save individual tables of variable names for running Cluster Subjects;
%if %Upcase(&ClusterSubjects) = YES %then %do;
	%put DomIndList = &DomIndList;
	%let d=1;
	%do %while(%length(%scan(&DomIndList,&d)));
		%let domain = %scan(&DomIndList,&d);
		data dsix&d;
			set OutLib.&OutDataName(keep=&domain.i_:);
			if (0);
		run;
		%let _dsix_&d  = %sysfunc(open(dsix&d));
		%let nix&d  = %sysfunc(attrn(&&_dsix_&d, NVARS));
		%let d=%eval(&d+1);
	%end;
	%let NumDomInd = %eval(&d-1);
	%put NumDomInd = &NumDomInd;
	
	%put DomSSList = &DomSSList;
	%let d=1;
	%do %while(%length(%scan(&DomSSList,&d)));
		%let domain = %scan(&DomSSList,&d);
		%put d=&d domain=&domain;
		data d1;
			set OutLib.&OutDataName(obs=1 keep=&domain.s_:);
		run;

		* transpose to sort names for parallel plots;
		proc transpose data=d1 out=d1t;
		run;
		
		proc sort data=d1t;
			by _name_;
		run;
		
		proc transpose data=d1t out=d1tt;
		run;

		data dsis&d;
			set d1tt;
			if (0);
			drop _name_;
		run;

		%let _dsis_&d  = %sysfunc(open(dsis&d));
		%let nis&d  = %sysfunc(attrn(&&_dsis_&d, NVARS));
		%put nis&d = &&nis&d;
		
		/* for missing measurements */
		proc means data=OutLib.&OutDataName(keep=&domain.s_:) noprint;
			var &domain.s_:;
			output out=dmiss(keep =&domain.s_:) nmiss=/autoname;
		run;
				
		proc transpose data=dmiss out=dmisst;
		run;
		
		/*
		data outlib.dmisst&d;
			set dmisst;
		run;
		*/
		
		/*
		data dmisst; set dmisst;
			if (Col1/&ns) > 0.10 then delete;
		run;
		*/
		
		proc transpose data=dmisst out=dmiss&d(drop=_name_);
		run;
		
		/*
		data dmiss&d; set dmiss;
			%do i=1 %to &&nis&d;
				if (%sysfunc(varname(&&_dsis_&d,&i))_nmiss/&ns)>.30 then do;
					drop %sysfunc(varname(&&_dsis_&d,&i))_nmiss;
				end;
			%end;
		run;
		
		data outlib.dmiss&d; set dmiss&d; run;
		*/
		
		%let _dmiss_&d  = %sysfunc(open(dmiss&d));
		%let nmiss&d  = %sysfunc(attrn(&&_dmiss_&d, NVARS));
		%put nmiss&d = &&nmiss&d;
		%let d=%eval(&d+1);	
	%end;
	%let NumDomSS = %eval(&d-1);
	%put NumDomSS = &NumDomSS;	
%end;
%else %do;
	%let NumDomInd = 0;
	%let NumDomSS = 0;
%end;

/* Post-translation of "High XXX" and "Low XXX" strings in output data sets and JSL script (S1018608 (JC)) */ 
%if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) ^= EN %then %do;
	%let trHigh = %nrbquote(%LocaleGet(High_Text));
	%let trLow = %nrbquote(%LocaleGet(Low_Text));
	%macro PostTrans(_stringvar);
		%let &_stringvar = %nrbquote(%sysfunc(tranwrd(%nrbquote(&&&_stringvar), %str(High ), %nrbquote(&trHigh)%str( ))));
		%let &_stringvar = %nrbquote(%sysfunc(tranwrd(%nrbquote(&&&_stringvar), %str(Low ), %nrbquote(&trLow)%str( ))));
	%mend;
	%let dsid = %sysfunc(open(OutLib.&OutDataName));
	%let nVars = %sysfunc(attrn(&dsid,NVARS));
	%do i = 1 %to &nVars;
		%let origname = %sysfunc(varname(&dsid, &i));
		%let origlabel = %nrbquote(%sysfunc(varlabel(&dsid, &i)));
		%let newlabel = &origlabel;
		%if %eval(%length(&origname)) > 6 and %eval(%klength(&origlabel)) > 5 %then %do;
			%if %upcase(%substr(&origname,1,6)) = %str(HLLBI_) and
				(%nrbquote(%ksubstr(&origlabel,1,5)) = %str(High ) or
				%nrbquote(%ksubstr(&origlabel,1,4)) = %str(Low )) %then %do;
				%PostTrans(newlabel);
				%let dsid = %sysfunc(close(&dsid));
				data OutLib.&OutDataName; set OutLib.&OutDataName;
					label &origname = "&newlabel";
				run;
				%let dsid = %sysfunc(open(OutLib.&OutDataName));
				%do j = 0 %to 2;
					%if %eval(&j) = 0 %then %do;
						%let DOMXLIST = &DomIndList;
						%let DATASETX = dsix;
					%end;
					%else %if %eval(&j) = 1 %then %do;
						%let DOMXLIST = &DomSSList;
						%let DATASETX = dsis;
					%end;
					%else %if %eval(&j) = 2 %then %do;
						%let DOMXLIST = &DomSSList;
						%let DATASETX = dmiss;
					%end;
					%let domxlistHLLBindex = %sysfunc( findw(%upcase(&DOMXLIST) ,%str(HLLB)," ",E));
					%if &domxlistHLLBindex %then %do;
						%if %sysfunc(exist(&DATASETX&domxlistHLLBindex)) %then %do;
							%if %symexist(_&DATASETX._&domxlistHLLBindex) %then %do;
								%let _&DATASETX._&domxlistHLLBindex = %sysfunc(close(&&_&DATASETX._&domxlistHLLBindex));
							%end;
							data &DATASETX&domxlistHLLBindex; set &DATASETX&domxlistHLLBindex;
								label &origname = "&newlabel";
							run;
							%let _&DATASETX._&domxlistHLLBindex = %sysfunc(open(&DATASETX&domxlistHLLBindex));
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
	%let dsid = %sysfunc(close(&dsid));
	data OutLib.&OutTransName; set OutLib.&OutTransName;
		if substr(_NAME_,1,6) = "HLLBI_" then do;
			call symputx("newvalue", _LABEL_);
			call execute('%PostTrans(newvalue)');
			_LABEL_ = symget('newvalue');
		end;
	run;
%end;
/* End of Post-translation */

* create JSL file;
%PrepOutFile(CDDScript,CDDScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &CDDScript;
%let FullAPName=%str(Create Cross Domain Data);

* put JSL statements;
data _null_;
   file "&CDDScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
	put "objRefNS = newNamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:DataList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	put "objRefNS:CreateReportList={};";
	
	put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
		put "try(objRefNS:WhereState=""\[&WhereStmt]\"");";
		put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
		put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";	
	
	
	put "if(host is(Windows),";
	put "   include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows)),";
	put "   include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX))";
	put ");";
	/* kemicl remove hierarchical cluster of domains from clinical 6.0.  Consider bring back in 70 */
	/*
	put "objRefNS:cddbuttonswindow=Function({},{default local},";
	put "   VListBox(";
	put "      TextBox(locale:get(""ClickButtonToRunProcessUsingODS_Text"")),";
	put "      TextBox("" ""),";
	put "      ButtonBox(locale:get(""ClusterDomains_Text""),";
	put "         if(host is(Windows),";
	put "					Include(ConvertFilePath(""$HOME/JMPLS/ProcessLibrary/HierarchicalClustering.jsl"", absolute, windows)),";
	put "					Include( ConvertFilePath(""$HOME/JMPLS/ProcessLibrary/HierarchicalClustering.jsl"", absolute, POSIX))";
	put "         );";
	put "         selcollist = currentdatatable() << getselectedcolumns;";
	put "         vlist = {};";
	put "         for(i=1,i<=nitems(selcollist),i++,";
	put "            sname = headname(selcollist[i] << get property(""SAS NAME""));";
	put "            if (length(sname),insertinto(vlist,sname));";
	put "         );";
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""InData""),""&OutPath.&OutTransName..sas7bdat"");";
	put "         jg:HierarchicalClustering_updateDeps;";	
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""WhereStmt""),"""");";			
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""OrderingVar""),""Mean"");";		
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""LabelVar""),""_LABEL_"");";	
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""ListVars""),""%nrbquote(S_:)"");";	
	put "         if(length(vlist),";
	put "			     vlist1 = substr(char(vlist),2,length(char(vlist))-2);";
	put "            wherestmt = EvalInsert(""\[_name_ in (^vlist1^)]\"");";
	put "            show(vlist,vlist1,wherestmt);";
	put "            jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""WhereStmt""),wherestmt);";
	put "         );";
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""OutPath""),""&ClientOutPath"");";
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""CenterRows""),""No"");";
	put "         jg:setCurrentParmValue(1,""HierarchicalClustering"", jg:getParmNum(""HierarchicalClustering"",""Theme""),""White to Red"");";
	put "         jg:runProcess( 1,""HierarchicalClustering"" );";
	put "			)";		
	put "   )";
	put ");";
	put "objRefNS:LaunchCode = objRefNS:cddbuttonswindow();";
	*/
	
	%if %UPCASE(&ClusterSubjects)=YES %then %do;
		/* put "  if(jg:CreateHCTab==1,"; */

		put "      objRefNS:hcdata = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
		put "      Column(""&SubjectLabel"") << label(1);";

		%do d=1 %to &NumDomInd;
			%let domain = %scan(&DomIndList,&d);

			/* Code from Hierarchical Clustering */

			put "vb&d = Function({},{default local},";
			put "   If(IsEmpty(objRefNS:hcdata),";
				put "      objRefNS:hcdata = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
				put "      Column(""&SubjectLabel"") << label(1);";
			put "   );";
			put "   Current Data Table(objRefNS:hcdata);";
			/* S1321023 */
			%if &&nix&d <= 1 %then %do;
				put "write(""\!NNOTE: For &StudyName Create Cross Domain Data &domain Indicators, there are "");";
			%end;
			put "   HListBox(";
			%if &ns > 1 %then %do;
				put "   HCWin&d = Hierarchical Cluster(Y(";
				%do i=1 %to &&nix&d;
					/* %put i=&i; */
					%let v = %qsysfunc(varlabel(&&_dsix_&d,&i));
					/* %put v=&v; */
					%if ^%length(&v) %then %let v = %sysfunc(varname(&&_dsix_&d,&i));
					put " Name(""&v"")" @@;
					%if (&i<&&nix&d) %then %do;
						put "," @@;
					%end;
					%else %do;
						put "),";
					%end;
				%end;
				put "   Method(Fast Ward), Dendrogram Scale( Distance Scale ), " /
				"      Column Label Position( ""Below"" )," /
				"      Two way clustering," /
				"      Number of Clusters(5), Color Map(White to Red), Save Clusters(0)," /
				"      Legend(0), Save Display Order(0), Standardize(0), Missing value imputation(1)," /
				"      Color Clusters(1),"/
					" SendToReport("/
							"	Dispatch( {""Dendrogram""},"/ 
							"		"""", "/
							"		CellPlotBox( 2 ), "/
							"		{Frame Size( 460, 475 )} ),"/
							"	Dispatch("/
							"		{""Dendrogram""},"/
							"		""Clust Dendro"","/
							"		FrameBox( 2 ),"/
							"		{Frame Size( 236, 475 )}"/
							"	),"/
							"	Dispatch("/
							"		{""Dendrogram""},"/
							"		""Clust Dendro"","/
							"		FrameBox( 3 ),"/
							"		{Frame Size( 460, 142 )}"/
							"	)"/
							%if &&nix&d > 1 %then %do;
							"        ,"/
							"	Dispatch("/
							"		{""Dendrogram""},"/
							"		""Clust Dendro"","/
							"		FrameBox( 5 ),"/
							"		{Frame Size( 140,270 )}"/
							"	)"/
							%end;
					")"/

				");";
			%end;
			%else %do;
				put " Text Box(""              ""),";
				put " TextBox(locale:get(""ClusteringNotPerformedBcOnlyOneRowOfData_Text""));";
			%end;
			/*
			put "objRefNS:hcdata << Data Filter(<<Conditional(1),";
			put "   Add Filter(";
			put "      Columns(Column(""&SubjectLabel""))";
			put "   )";
			put ");";
			*/
			put "));";

			put "InsertInto(objRefNS:DataList,{{""objRefNS:hcdata""}});";
			put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XIndicators_Text""),{""&domain""})));";
			put "InsertInto(objRefNS:TabCodeList,expr(vb&d()));";
			put "InsertInto(objRefNS:TabAfterCodeList,"""");";		
			put "InsertInto(objRefNS:TabOpenList,1);";
		%end;

		%do d=1 %to &NumDomSS;
			%let domain = %scan(&DomSSList,&d);

			/* Code from Hierarchical Clustering */

			put "vbs&d = Function({},{default local},";
			put "   If(IsEmpty(objRefNS:hcdata),";
				put "      objRefNS:hcdata = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
				put "      Column(""&SubjectLabel"") << label(1);";
			put "   );";
			put "   Current Data Table(objRefNS:hcdata);";
			put "   HListBox(";
			%if &&nmiss&d > 1 %then %do;
				put "   PPWin&d = Parallel Plot(";
				put "		Scale Uniformly( 1 ),";
				put "		Y(";
				%do i=1 %to &&nis&d;
					%put i=&i;
					%let v = %qsysfunc(varlabel(&&_dsis_&d,&i));
					%put v=&v;
					%if ^%length(&v) %then %let v = %sysfunc(varname(&&_dsis_&d,&i));
					put " Name(""&v"")" @@;
					%if (&i<&&nis&d) %then %do;
						put "," @@;
					%end;
					%else %do;
						put "),";
					%end;
				%end;
				put " SendToReport("/
					 "  Dispatch("/
						"	{},"/
						"	""Parallel Coord"","/
						"	FrameBox,"/
						"	{Frame Size( 440, 290 ), Line Width Scale( 2 )}"/
					"	)"/
					" )"/
					"),";
				put "   HCWin&d = Hierarchical Cluster(Y(";
				%do i=1 %to &&nmiss&d;
					%let v = %qsysfunc(varlabel(&&_dmiss_&d,&i));
					%if ^%length(&v) %then %let v = %sysfunc(varname(&&_dmiss_&d,&i));
					put " Name(""&v"")" @@;
					%if (&i<&&nmiss&d) %then %do;
						put "," @@;
					%end;
					%else %do;
						put "),";
					%end;
				%end;
				put "   Method(Fast Ward), Dendrogram Scale( Distance Scale ), " /
				"      Column Label Position( ""Below"" )," /
				"      Two way clustering," /
				"      Number of Clusters(5), Color Map(Blue to Gray to Red), Save Clusters(0)," /
				"      Legend(1), Save Display Order(0)," /
				"      Color Clusters(1)," /
				"      Missing value imputation(1)," /
				%if %upcase(&domain) = LB %then %do;
					" Standardize(0),"/
				%end;
				%else %do;
					" Standardize(1),"/
				%end;
				
					" SendToReport("/
							"	Dispatch( {""Dendrogram""},"/ 
							"		"""", "/
							"		CellPlotBox( 2 ), "/
							"		{Frame Size( 100, 295 )} ),"/
							"	Dispatch("/
							"		{""Dendrogram""},"/
							"		""Clust Dendro"","/
							"		FrameBox(2),"/
							"		{Frame Size( 100, 295 )}"/
							"	),"/							
							"	Dispatch("/
							"		{""Dendrogram""},"/
							"		""Clust Dendro"","/
							"		FrameBox( 3 ),"/
							"		{Frame Size( 100, 295 )}"/
							"	),"/
					")"/
				
				");";
				/* put "HCWin&d<<Two way clustering;"; */
				/* put "HCrep&d = HCWin&d<<Report;"; */
				/* put "HCrep&d[framebox(4)]<<Frame Size( 390, 290 );"; */
			%end;
			%else %do;
				put " Text Box(""              ""),";
				put " TextBox(locale:get(""AnalysesNotPerformedBcOnlyOneColOfData_Text""));";
			%end;
			/*
			put "objRefNS:hcdata << Data Filter(<<Conditional(1),";
			put "   Add Filter(";
			put "      Columns(Column(""&SubjectLabel""))";
			put "   )";
			put ");";
			*/
			put "));";

			put "InsertInto(objRefNS:DataList,{{""objRefNS:hcdata""}});";
			put "InsertInto(objRefNS:TabButtonList, ""&domain &trCDSTAT"");";
			put "InsertInto(objRefNS:TabCodeList,expr(vbs&d()));";
			put "InsertInto(objRefNS:TabAfterCodeList,"""");";		
			put "InsertInto(objRefNS:TabOpenList,1);";
		%end;
		/*
		put "   pdcFunc =  Function({},{},";
		put "      outpath = ""&OutPath"";";
		put "      ::serveroutpath = ""&ServerOutPath"";";
		put "      ::clientoutpath = ""&ClientOutPath"";";
		put "      namevar = ""&NameVar"";";
		put "      labelvar = ""&LabelVar0"";";
		put "      hcvars = ""&Variables &ListVars"";";
		put "      include( ""$JMP_HOME/LifeSciences/Generator/DialogHelperFunctions.jsl"");";
		put "      include( ""$JMP_HOME/LifeSciences/JSLFiles/Hierarchical Clustering Drill Down Expressions.jsl"");";
		put "      HCSaveSubset;";
		put "      HCTranspose;";
		put "      LaunchPartialCorrelationDiagram;";
		put "      jg:runProcess(1,""PartialCorrelationDiagram"");";
		put "   );";	
		put "ActionButtonList= {{expr(locale:get(""PartialCorrelationDiagram_Text""))}};";
		put "ActionCodeList= {{pdcFunc()}};";
		put "ActionButtonDesc = {expr(locale:get(""SelectaSmallClusterthenClickButton_Text""))};";
		*/

		%SubjectDrillDowns;

		/* put "jg:CreateHCTab = 0;";

		put ")); //end ifs for creating HC tabs";
		*/
	%end;
	
run;	

%do d=1 %to &NumDomInd;
	%let _dsix_&d = %sysfunc(close(&&_dsix_&d));
%end;
%do d=1 %to &NumDomSS;
	%let _dsis_&d = %sysfunc(close(&&_dsis_&d));
	%let _dmiss_&d = %sysfunc(close(&&_dmiss_&d));
%end;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CDDScriptName, &OutDataName..sas7bdat, text/plain, yes);
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(TransposedOutputDataFromX_Text)),%nrbquote(%LocaleGet(&ProcessName._Label)))), OutLib, &OutTransName,_type=tall);
%AddDatatoStudy(&OutDataName,_type=wide);
	
%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend CreateCrossDomainData;


* run the macro;
%CreateCrossDomainData;




