/*
JMP Clinical Analytical Process

Name: Events Incidence Screen
Authors: Russ Wolfinger, SAS Institute Inc.
Support:  Russ Wolfinger and Kelci Miclaus

Description: 
	This process screens all medical history terms by performing a Fisher's exact test on
	all 2 x 2 tables constructed from term incidence and treatment arm.  Output is a table
	of multiplicity-adjusted p-values, an accompanying volcano plot of relative risk, and
	a SAS data set of indicator variables which can be used as input for other processes.
*/

/*
-----------------------------------------------------------
Copyright (C) 2009 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*		Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"EventsO","CrossoverPeriods","TreatmentEmergentEvents"}}$
$CMP: RSYS.Term Variable$ $UE: EVENTS_NOT_AE.xxDECOD or EVENTS_NOT_AE.xxTERM$ 
	$NOTES: At least one variable that provides the event term value must be present.$ 
$CMP: CPV.{"Term Level","Reported Name"}$ $UE: EVENTS.xxTERM$ $NOTES:  Counts and frequencies of values are summarized.$
$CMP: CPV.{"Term Level","Standardized Name"}$ $UE: EVENTS.xxDECOD$ $NOTES: Counts and frequencies of values are summarized.$ 
$CMP: OOUT$ $UE: EVENTS.xxBODSYS or EVENTS.xxCAT$ $NOTES: Used in Data Filter and for grouping.$ 
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName EventsDomain EventsTermLevel InPath AInPath TrtUse IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL 
TrtVar DesignByVars SubjectFilter WhereStmt WhereForReports EventsWhereStmt ScreenControlValue
VolcanoXCat DoubleFDR MultipleTestingMethod Alpha NegLog10pCutoff TimeWindows TimeScale StrataVars OutPath ServerOutPath 
ColorTheme EventsIncidenceScreen_OutData MacroPath EventsIncidenceScreenScript IncidenceScreenScript MergeSuppDom TEAEOnly TrtOffset IgnoreTEAEFlags;

*ProcessBody;

%macro EventsIncidenceScreen;

*initialization and argument checking;
%let ProcessName = EventsIncidenceScreen; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%LOADDATA(domlist=&EventsDomain,mergesupp=&MergeSuppDom,checkreview=1);
%let domain = %lowcase(&EventsDomain);
%if &exiterror %then %goto exit;

%global WhereState;
%let WhereState=&WhereStmt;
%let WhereForReports=%nrbquote(&WhereStmt);
* add numeric suffix;
%if ^%symexist(_nThePro_) %then %let _nThePro_ = 001;
%let suffix = &domain.i_&_nThePro_;

%include "&MacroPath./TEAEOTFUInt.sas";
%if &exiterror %then %goto exit;	

%if %length(&&in_&domain)=0 %then %do;
	%put [MS_S_0013];
	%put ERROR: The &domain or ad&domain data set does not exist for this study.;
	%let exiterror=1;
	%goto exit;
%end;

%let IncidenceScreenScript=&EventsIncidenceScreenScript;
%include "&MacroPath./TreatmentLevels.sas" / nosource;
%if &exiterror %then %goto exit;
* create category indicators;
%include "&MacroPath./CategoryIndicators.sas" / nosource;
%let CIPrefix = &domain.I;


%if %upcase(&EventsTermLevel) = STANDARDIZED NAME %then %let CatName = %upcase(&EventsDomain)DECOD;
%else %if %upcase(&EventsTermLevel) = REPORTED NAME %then %let CatName = %upcase(&EventsDomain)TERM;
%if %sysfunc(findw(&&&EventsDomain.VARS, &CatName)) = 0 %then %do;
	%put [MS_S_0086];
	%put ERROR:  The &CatName variable was not found in the &EventsDomain data set.; 
	%let exiterror=1;
	%goto exit;
%end;
	
%let _tid_ = %sysfunc(open(&&in_&domain));
%let vn = %sysfunc(varnum(&_tid_,&CatName));
%let CatLabel = %sysfunc(varlabel(&_tid_,&vn));
%if ^%klength(%ktrim(&CatLabel)) %then %let CatLabel = &CatName;
%let _tid_ = %sysfunc(close(&_tid_));

/*Check for bodysystem */
%if %sysfunc(findw(&&&EventsDomain.VARS, &EventsDomain.BODSYS)) > 0 %then %let GroupVar = &domain.BODSYS;
%else %if %sysfunc(findw(&&&EventsDomain.VARS, &EventsDomain.CAT)) > 0 %then %let GroupVar = &domain.CAT;
%else %let GroupVar = &CatName;

data &domain;
	set &&in_&domain;
	%PRESPECIFY(dom = &domain);
run;

%TEAEInt(dom = %upcase(&domain), type = events);

data &domain;
	set &domain;
	%if %length(&EventsWhereStmt) %then %do;
		where %unquote(&EventsWhereStmt);
	%end;
run;

%if &syserr > 4 %then %do;
	%put [MS_S_0087];
	%put ERROR: The specified filter &EventsWhereStmt is invalid.  Change the filter to be valid for the domain you are considering.;
	%let exiterror = 1;
	%goto exit;
%end;

%NObsVars(&domain);
%if ^&nobs %then %do;
	%put [MS_S_0088];
	%put ERROR: No observations satisfy &EventsWhereStmt..;
	%put Please relax the filter.;
	%let exiterror = 1;
	%goto exit;
%end;

* incorporate time windows if specified;
%AddTimeWindows(domain=&domain,indat=&domain,outdat=&domain,twb=&TimeWindows,tvar=&TimeScale);
%if &exiterror %then %goto exit;

* handle crossover design by adding treatment as an id in the incidence matrix and USUBJID as a strata variable in the CMH test;
%if &NTrtPeriods > 1 %then %do;
	proc sort data=&domain;
		by USUBJID;
	run;
	%AssignCOTreatment(&domain, &domain, &domain.STDTC);
	%let cidvar = USUBJID Period Trt;
	%let StrataVars = USUBJID &StrataVars;
%end;
%else %do;
	%let cidvar = USUBJID;
%end;

* create category indicators;
%CategoryIndicators(ind=&domain,cat=&CatName,id=&cidvar,CIPrefix=&CIPrefix,CatLabel=&CatLabel,outd=&suffix);
%if &syserr>4 %then %let exiterror = 1;

* join with adsl;	
data OutLib.adsl_&suffix;
	merge &suffix &suffix.f OutLib.adsl_&suffix ;
	by &cidvar;
	if missing(&TrtName) then delete;
	* set missings to zeros;
	if missing(FREQ_&CIPrefix) then FREQ_&CIPrefix = 0;
	array ind {*} &CIPrefix:;
	do _i=1 to dim(ind);
		if missing(ind[_i]) then ind[_i] = 0;
	end;
	drop _i;
run;

* create counts to be used in tree map;
%createCounts(&domain,KeepVars=&CatKeepVars);

* set up and call column enrichment process, which does the testing and p-value adjustment;
%global InData AnnoVar ASigVars BSigVars CSigVars Delimiter Delim2 IncidenceScreen_BOutData
WhereStmt NBins RankEm MultipleTestingMethods NSample PThresh NHits Seed 
IncidenceScreen_DesOutData;

%let InData=&OutPath.adsl_&suffix..sas7bdat;
%let AnnoVar= &CatName;
%let ASigVars= ;
%let CSigVars= ;
%let BSigVars = ;
%do i=1 %to %eval(&nlev-1);
	%let BSigVars = &BSigVars &TrtName&i;
%end;
%let Delimiter=%nrbquote(" / ");
%let Delim2=%nrbquote();
%let MultipleTestingMethods= &MultipleTestingMethod;
%let NSample= 10000;
%let Seed=%nrbquote(12345);
%let PThresh= 1;
%let NHits= 0;
%let NBins= 7;
%let RankEm= No;
%let WhereStmt=%nrbquote();
%let IncidenceScreen_DesOutData=adsl_&suffix;
%let IncidenceScreen_BOutData=&EventsIncidenceScreen_OutData;
%let Abbrev=%upcase(&domain);
%let CESuffix=s;
%let TreeData=&domain._counts;
%let TreeSize=Count;

%include "&CLINICAL_HOME.\ProcessLibrary\IncidenceScreen.sas"; 

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend EventsIncidenceScreen;

* run the macro;
%EventsIncidenceScreen;


