/*
JMP Clinical Analytical Process

Name: Exposure Summary 
Authors: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 15Oct10 initial coding
   kemicl 22Dec10 code restructure
   saslii Apr14   Add five analyses
      1) Percent of Subjects Left on Study Medication by Day of Study
      2) Distribution of Doses by Arm
      3) Dose Descriptive Statistics by Arm and Box Plot of Doses
      4) Planned Arm to Actual Treatment Comparison
      5) Dose Changes During the Study       

Description: 
   This process generates exposure plot to investigational product by dose and exposure time for the safety 
population by actual treatment arm.
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/
*******************************************************************************;
*     Clinical Variable Specification Block of ExposureSummary Variable Usage;
*******************************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$INCD: {"EX"}$
$CMP: RSYS$ $UE: DM.USUBJID$ $NOTES: To compute the percent of Subjects by treatment group.$
$CMP: RSYS$ $UE: DM.RFSTDTC$ $NOTES: To increase chronological number of days in the study.$
$CMP: OSYS$ $UE: DM.ACTARM or DM.ARM$ $NOTES: To compute the percent of Subjects by treatment group.$
$CMP: OSYS$ $UE: EX.VISITNUM$ $NOTES: To apply Cycles for Time Scale.$
$CMP: RSYS$ $UE: EX.EXSTDTC$ $NOTES: To compute the number of days in the study.$
$CMP: RSYS$ $UE: EX.EXENDTC$ $NOTES: To increase chronological number of days in the study.$
$CMP: OSYS$ $UE: EX.EXTRT$ $NOTES: To calculate count of subject in each actual treatment group.$
$CMP: OSYS$ $UE: EX.EXDOSE$ $NOTES: To calculate count of subject in each actual treatment group.$
$CMP: OSYS$ $UE: EX.EXDOSU$ $NOTES: To calculate count of subject in each actual treatment group.$ 
$CMP: OSYS$ $UE: EX.EXDOSFRQ $ $NOTES: To Determine the number of doses taken for each exposure event by multiplying the dosing frequency.$ 
$CMP: OSYS$ $UE: EX.EXDOSFRM$ $NOTES: To check if dose frequencies contains INJECTION, if so, assume one dose per exposure event.$ 
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName InPath AInPath TrtUse TrtVar DesignByVars SubjectFilter WhereStmt WhereForReports nDoseBin
   IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL MergeSuppDom
   OutPath ServerOutPath MacroPath 
   DurationTimeScale DurationWindowMethod NDWindows DurationWindows CycleVar
   Output_RTF Output_PDF Output_HTML Output_Text ExposureSummary_OutReport
   ExposureSummaryReport_Body ExposureSummaryReport_Contents ExposureSummaryReport_Frame
   ExposureSummary_OutData ExposureSummary_OutData2 ExposureSummary_OutData3 ExposureSummary_OutData4 ExposureSummaryScript
   ExposureSummary_OutData5 ExposureSummary_OutData6 ExposureSummary_OutData7;

%let WhereForReports=&WhereStmt;

%let Output_HTML = No;
%let Output_Text = No;

*ProcessBody;
%macro firstsection;
   /********************************First section of table ******************/   
   *run after computing time windows to get data removed;

	data table2;
		set table;
		if Total_Treatment_Duration <0 then delete;
	run;

   proc summary data=table2 mean median min max sum stderr;
      by &TrtVar;
      %if &TimeScale ne Cycles %then %do;
         var Total_Treatment_Duration;
      %end;
      %else %do;
         var _CycleNum_;
      %end;
      output out=_p1 mean=mean median=med stderr=stderr sum=sum min=min max=max;
   run; 

   data _p1;
      set _p1;
      %do i = 1 %to &_nArm_;
         if &TrtVar = "&&_armLabel_&i" then _trtNum = &i;
      %end;
   run;

   %do i = 1 %to &_nArm_;
      %global _max_&i;
   %end;

   data _null_;
      set _p1 end=_e_;
      if _FREQ_= 1 then stderr = 0;
      name = "_sum_"||trim(left(_trtNum));
      call symput(name,round(sum/365,.001)); *cumulative exprosure to treatment (patient years);
      name = "_num_"||trim(left(_trtNum));
      call symput(name,_FREQ_); *number of patients;
      name = "_mean_"||trim(left(_trtNum));
      call symput(name,trim(left(round(mean,.1)))||"("||trim(left(round(stderr,.1)))||")"); *mean treatment duration (days);
      name = "_med_"||trim(left(_trtNum));
      call symput(name,round(med,.1));
      name = "_max_"||trim(left(_trtNum));
      call symputx(name,max,"G");
      name = "_min_max_"||trim(left(_trtNum));
      call symput(name,trim(left(min))||":"||trim(left(max)));
      if _e_ then call symput('_nP1',n_);
   run;

   %let trcettsy = %nrbquote(%LocaleGet(CumulativeexposuretotreatmentSubjectYears_Text));
   %let trdost = %nrbquote(%LocaleGet(Durationofstudytreatment_Text));
   %if %Upcase(&TimeScale) eq DAYS %then %let trTimeScale = %nrbquote(%LocaleGet(Days_Text));
   %else %if %Upcase(&TimeScale) eq WEEKS %then %let trTimeScale = %nrbquote(%LocaleGet(Weeks_Text));
   %else %if %Upcase(&TimeScale) eq CYCLES %then %let trTimeScale = %nrbquote(%LocaleGet(Cycles_Text));
   %else %let trTimeScale = %nrbquote(&TimeScale);
   %let trnumsubjs = %nrbquote(%LocaleGet(NumberofSubjects_Text));
   %let trMean = %nrbquote(%LocaleGet(Mean_Text));
   %let trSD = %nrbquote(%LocaleGet(SD_Text));
   %let trMedian = %nrbquote(%LocaleGet(Median_Text));
   %let trMin = %nrbquote(%LocaleGet(Min_Text));
   %let trMax = %nrbquote(%LocaleGet(Max_Text));
   
   data _tp1;
      length label $70 
         %do i = 1 %to &_nArm_;
            &&_arm_&i $20
         %end;
      ;
      %if &TimeScale ne Cycles %then %do;
         Label = "&trcettsy";
         %do i = 1 %to &_nArm_;
            %if ^%symexist(_sum_&i) %then &&_arm_&i = "0";
            %else &&_arm_&i = trim(left("&&_sum_&i"));;
         %end;
         output;
         label = "";
         %do i = 1 %to &_nArm_;
            &&_arm_&i = "";
         %end;
         output;
      %end;
      label = "&trdost (&trTimeScale)";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "   &trnumsubjs";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_num_&i) %then &&_arm_&i = "0";
         %else &&_arm_&i = trim(left("&&_num_&i"));;
      %end;
      output;
      label = "   &trMean (&trSD)";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_mean_&i) %then&&_arm_&i = "0(0)";
         %else &&_arm_&i = trim(left("&&_mean_&i"));;
      %end;
      output;
      label = "   &trMedian";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_med_&i) %then &&_arm_&i = "0";
         %else &&_arm_&i = trim(left("&&_med_&i"));;
      %end;
      output;
      label = "   &trMin:&trMax";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_min_max_&i) %then %do; 
            &&_arm_&i = "0:0";
            %let _max_&i = 0;
         %end;
         %else %do;
            &&_arm_&i = trim(left("&&_min_max_&i"));
         %end;
      %end;
      output;
      label = "";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label %do i = 1 %to &_nArm_; &&_arm_&i = "&&_armLabel_&i" %end;;
   run;
   /**************************** end of first part of table *************/
%mend;

%macro EX_Report(_ExDs);

* time windows;
* second and third sections in the table;

%if %Upcase(&TimeScale) eq DAYS %then %let trTimeScale = %nrbquote(%LocaleGet(Days_Text));
%else %if %Upcase(&TimeScale) eq WEEKS %then %let trTimeScale = %nrbquote(%LocaleGet(Weeks_Text));
%else %if %Upcase(&TimeScale) eq CYCLES %then %let trTimeScale = %nrbquote(%LocaleGet(Cycles_Text));
%else %let trTimeScale = %nrbquote(&TimeScale);

%if &TimeScale ne Cycles %then %do;
   %put TimeScale = &TimeScale;
   
   %if %Upcase(&TimeScale) = DAYS %then %do;
      %let SingleTime = Day;
      %let trSingleTime = %nrbquote(%LocaleGet(Day_Text));
   %end;
   %else %do;
      %let SingleTime = Week;
      %let trSingleTime = %nrbquote(%LocaleGet(Week_Text));
   %end;
   %if %index(&TimeWindowMethod,Set) %then %do;
      %*firstsection;
      %put TimeWindowMethod = Set Number of Duration Time Windows;
      %let _tFlag = 1;
      %let _increment = %sysfunc(round(%sysevalf(&_MaxDuration_/&NWindows))); 
      * assign time frame group for each subject;
      data table;
         set &_ExDs;
         /* if Missing_End_Date then TimeWindowNum = 0; */
         TimeWindowNum = min( ceil(Total_Treatment_Duration / &_increment),&NWindows );
      run;
   %end;
   %else %do;
      %put TimeWindowMethod = Use Time Windows List;
      %let _tFlag = 2;
      %let STVar = Total_Treatment_Duration;
      %let ENVar = Total_Treatment_Duration;
      %AddTimeWindows(domain=ex,indat=&_ExDs,outdat=table1,twb=%nrbquote(&TimeWindows),tvar=&TimeScale,findingscall=1);
      %let nWindows = &ntw;
      %do i = 1 %to &nWindows;
         %let _TimeStart_&i = &&tws&i;
         %let _TimeEnd_&i = &&twe&i;
      %end;
      
      data table; 
         set &_ExDs(where = (Total_Treatment_Duration=0)) table1(in=_a_);
         if not _a_ then TimeWindowNum = 0;
      run;
      
      proc sort data = table;
         by &TrtVar;
      run;
   %end;

   %firstsection;

   proc sort data=table;
      by &TrtVar descending TimeWindowNum;
   run;

   proc summary data=table;
      by &TrtVar descending TimeWindowNum;
      output out=_p2;
   run;

   data _p2;
      set _p2;
      by &TrtVar;
      retain _accPatient;
      %do i = 1 %to &_nArm_;
         if &TrtVar = "&&_armLabel_&i" then _trtNum = &i;
      %end;
      if first.&TrtVar then do;
         _accPatient = 0;
      end;
      _accPatient = _accPatient + _FREQ_;
      name = "_nPatient_"||trim(left(_trtNum))||"_"||trim(left(TimeWindowNum));
      call symput(name,_FREQ_);
      name = "_nAccPatient_"||trim(left(_trtNum))||"_"||trim(left(TimeWindowNum));
      if TimeWindowNum = 0 then call symput(name,_FREQ_);
      else call symput(name,_accPatient);
      if last.&TrtVar then do;
         name = "_TotalPatient_" || trim(left(_trtNum));
         call symput(name,_accPatient);
      end;
   run;

   * Checking accumulated counts second time for patching time-window with 0 count;
   %do j = 1 %to %to &NWindows;
      %do i = 1 %to &_nArm_;
         %let k = %eval(&NWindows-&j+1);
         %if ^%symexist(_nAccPatient_&i._&k) %then %do;
            %if &k = &NWindows %then %do; 
               %let _nAccPatient_&i._&k = 0;
            %end;
            %else %do;
               %let l = %eval(&k+1);
               %let _nAccPatient_&i._&k = &&_nAccPatient_&i._&l;
            %end;
         %end;
      %end;
   %end;

   %let trdostbc = %nrbquote(%LocaleGet(DurationofstudytreatmentbycategoryNPCT_Text));
   %let trmissdur = %nrbquote(%LocaleGet(Missingduration_Text));
   %let trnoswdostbc = %nrbquote(%LocaleGet(NumSubjsWithDurOfStudyTrtByCatNPCT_Text));
   
   data _tp2;
      length label $70 
         %do i = 1 %to &_nArm_;
            &&_arm_&i $20
         %end;
      ;
      label = "&trdostbc";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "   &trmissdur";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_nPatient_&i._0) %then %do;
            &&_arm_&i = "0(0%)";
         %end;
         %else %do; 
            &&_arm_&i = trim(left("&&_nPatient_&i._0"||"("||
               "%sysfunc(round(%sysevalf(&&_nPatient_&i._0/&&_TotalPatient_&i)*100,.1))"||
               "%)"));
         %end;
      %end;
      output;
      %do j = 1 %to &NWindows;
         %if &_tFlag = 1 %then %do;
            %if %eval((&j-1)*&_increment+1) ne %eval(&j*&_increment) %then %do;
               label = "   " || trim(left("%eval((&j-1)*&_increment+1)")) || "-" ||
                  trim(left("%eval(&j*&_increment)")) || " &trTimeScale";
            %end;
            %else %do;
               %if %eval(&j*&_increment) = 1 %then %do;
                  label = "   " || trim(left("%eval(&j*&_increment)")) || " &trSingleTime";
               %end;
               %else %do;
                  label = "   " || trim(left("%eval(&j*&_increment)")) || " &trTimeScale";
               %end;
            %end;
         %end;
         %else %do;
            %if %eval(&&_TimeStart_&j) ne %eval(&&_TimeEnd_&j) %then %do; 
               label = "   " || trim(left("&&_TimeStart_&j")) || "-" ||
                  trim(left("&&_TimeEnd_&j")) || " &trTimeScale";
             %end; 
             %else %do;
               %if  %eval(&&_TimeStart_&j) = 1 %then %do;
                  label = "   " || trim(left("&&_TimeStart_&j")) || " &trSingleTime";
               %end;
               %else %do;
                  label = "   " || trim(left("&&_TimeStart_&j")) || " &trTimeScale";
               %end;
              %end;
         %end;
         %do i = 1 %to &_nArm_;
            %if ^%symexist(_nPatient_&i._&j) %then %do;
               &&_arm_&i = "0(0%)";
            %end;
            %else %do; 
               &&_arm_&i = trim(left("&&_nPatient_&i._&j"||"("||
                  "%sysfunc(round(%sysevalf(&&_nPatient_&i._&j/&&_TotalPatient_&i)*100,.1))"||
                  "%)"));
            %end;
         %end;
         output;
      %end;
      label = "";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output; * end of section 2;

      * section 3;
      label = "&trnoswdostbc";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "   &trmissdur";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_nPatient_&i._0) | ^%symexist(_TotalPatient_&i) %then %do;
            &&_arm_&i = "0(0%)";
         %end;
         %else %do; 
            &&_arm_&i = trim(left("&&_nPatient_&i._0"||"("||
               "%sysfunc(round(%sysevalf(&&_nPatient_&i._0/&&_TotalPatient_&i)*100,.1))"||
               "%)"));
         %end;
      %end;
      output;
      %do j = 1 %to &NWindows;
         %if &_tFlag = 1 %then %do;
            label = "   >= " || trim(left("%eval((&j-1)*&_increment+1)")) || " " ||"&trTimeScale";
         %end;
         %else %do;
            label = "   >= " || trim(left("&&_TimeStart_&j")) || " " ||"&trTimeScale";
         %end;
         %do i = 1 %to &_nArm_;
            %if ^%symexist(_TotalPatient_&i) | &&_nAccPatient_&i._&j = 0 %then %do;
               &&_arm_&i = "0(0%)";
            %end;
            %else %do; 
               &&_arm_&i = trim(left("&&_nAccPatient_&i._&j"||"("||
                  "%sysfunc(round(%sysevalf(&&_nAccPatient_&i._&j/&&_TotalPatient_&i)*100,.1))"||
                  "%)"));
            %end;
         %end;
         output;
      %end;
   run;

   data _FinalTable;
      set _tp1 _tp2;
   run;

%end;
%else %do;
   %put TimeScale = Cycles;
   %let SingleTime = Cycle;
   %let trSingleTime = %nrbquote(%LocaleGet(Cycle_Text));
   
   * handle time window methods;
   %if %index(&TimeWindowMethod,Set) %then %do;
      %put TimeWindowMethod = Set Number of Duration Time Windows;
      %let _tFlag = 1;
      data _null_;
         set &_ExDs end=_e_;
         retain _MaxCycle_ 0;
         _MaxCycle_ = Max(_MaxCycle_,_CycleNum_);
         if _e_ then call symput("_MaxCycle_",_MaxCycle_);
      run;

      %if &NWindows >= &_MaxCycle_ %then %do;
         %let NWindows = &_MaxCycle_; * reduce number of time window;
         %let _increment = 1;
         data table;
            set &_ExDs;
            TimeWindowNum = _CycleNum_;
         run;
      %end;
      %else %do;
         %let _increment = %sysfunc(round(%sysevalf(&_MaxCycle_/&NWindows))); 
         data table;
            set &_ExDs;
            TimeWindowNum = min( ceil(_CycleNum_ / &_increment),&NWindows );
         run;
      %end;
   %end;
   %else %do;
      %put TimeWindowMethod = Use Duration Time Windows List;
      %let _tFlag = 2;

      * get time window boundary levels; * code from %AddTimeWindows;
      %let i=1;
      %let maxtw = 0;
      %let twb=%nrbquote(&TimeWindows);
      %if %index(&twb,%str(])) %then %do;
         %do %while(%length(%scan(&twb,&i,']')));
            %global tw&i tws&i twe&i;
            %let tw&i = %qscan(&twb,&i,']')%str(]);
            %if %index(&&tw&i,%str(,)) %then %do;
               %let tws&i = %scan(&&tw&i,1,'[, ]');
               %let twe&i = %scan(&&tw&i,2,'[, ]');
               %let twv&i = 2;
            %end;
            %else %do;
               %let tws&i = %scan(&&tw&i,1,'[ ]');
               %let twe&i = %scan(&&tw&i,1,'[ ]');
               %let twv&i = 1;
            %end;
            %if %length(&&tw&i) > &maxtw %then %let maxtw = %length(&&tw&i);
            %let i=%eval(&i+1);
         %end;
      %end;
      %else %do;
         %do %while(%length(%scan(&twb,&i,' ')));
            %if %index(&twb,%str(,)) %then %do;
               %let exiterror=1;
               %put [MS_S_0089];
               %put ERROR: Commas must be used with brackets when specifying time windows.;
               %*goto exit;
            %end;
            %let tw&i = %qscan(&twb,&i,' ');
            %let tws&i =%scan(&twb,&i,' ');
            %let twe&i =%scan(&twb,&i,' ');
            %let twv&i = 1;
            %if %length(&&tw&i) > &maxtw %then %let maxtw = %length(&&tw&i);
            %let i=%eval(&i+1);
         %end;
      %end; 
      %let ntw = %eval(&i-1);
      %put ntw = &ntw;
      %let nWindows = &ntw;
      %do i = 1 %to &nWindows;
         %let _TimeStart_&i = &&tws&i;
         %let _TimeEnd_&i = &&twe&i;
      %end;

      * assign TimeWindowNum;
      data table;
         set &_ExDs;
         if _CycleNum_ = . then TimeWindowNum = 0;
         %do i =1 %to &ntw;
            if _CycleNum_ >= &&tws&i & _CycleNum_ < %sysevalf(&&twe&i+0.01) then TimeWindowNum = &i;
         %end;
         if _CycleNum_ = &&twe&ntw then TimeWindowNum = &ntw;
         if TimeWindowNum = . then delete;
      run;
   %end;

   %firstsection;
   * section 1.2 overall summary on cycles;
   proc summary data=table mean median min max sum stderr;
      by &TrtVar;
      var _CycleNum_;
      output out=_p12 mean=mean median=med stderr=stderr sum=sum min=min max=max;
   run; 

   data _null_;
      set _p12;
      %do i = 1 %to &_nArm_;
         if &TrtVar = "&&_armLabel_&i" then _trtNum = &i;
      %end;
      if _FREQ_ = 1 then stderr = 0;
      name = "_sum_"||trim(left(_trtNum));
      call symput(name,sum); *total numbe of cycles across all patients;
      name = "_num_"||trim(left(_trtNum));
      call symput(name,_FREQ_); *number of patients;
      name = "_mean_"||trim(left(_trtNum));
      call symput(name,trim(left(round(mean,.1)))||"("||trim(left(round(stderr,.1)))||")"); *mean treatment duration (days);
      name = "_med_"||trim(left(_trtNum));
      call symput(name,round(med,.1));
      name = "_max_"||trim(left(_trtNum));
      call symput(name,max);
      name = "_min_max_"||trim(left(_trtNum));
      call symput(name,trim(left(min))||":"||trim(left(max)));
   run;

   %let trtnocaap = %nrbquote(%LocaleGet(TotalNumCyclesAdminAllPatients_Text));
   %let trtnocappc = %nrbquote(%LocaleGet(TotalNumCyclesAdminPerPatientCycles_Text));
   %let trnumsubjs = %nrbquote(%LocaleGet(NumberofSubjects_Text));
   %let trMean = %nrbquote(%LocaleGet(Mean_Text));
   %let trSD = %nrbquote(%LocaleGet(SD_Text));
   %let trMedian = %nrbquote(%LocaleGet(Median_Text));
   %let trMin = %nrbquote(%LocaleGet(Min_Text));
   %let trMax = %nrbquote(%LocaleGet(Max_Text));
   
   data _tp12;
      length label $70 
         %do i = 1 %to &_nArm_;
            &&_arm_&i $20
         %end;
      ;
      Label = "&trtnocaap";
      %do i = 1 %to &_nArm_;
         %if %symexist(_sum_&i) %then &&_arm_&i = trim(left("&&_sum_&i"));
         %else &&_arm_&i = "0";;
      %end;
      output;
      label = "";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "&trtnocappc";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "   &trnumsubjs";
      %do i = 1 %to &_nArm_;
         %if %symexist(_num_&i) %then &&_arm_&i = trim(left("&&_num_&i"));
         %else &&_arm_&i = "0";;
      %end;
      output;
      label = "   &trMean (&trSD)";
      %do i = 1 %to &_nArm_;
         %if %symexist(_mean_&i) %then &&_arm_&i = trim(left("&&_mean_&i"));
         %else &&_arm_&i = "0(0)";;
      %end;
      output;
      label = "   &trMedian";
      %do i = 1 %to &_nArm_;
         %if %symexist(_med_&i) %then &&_arm_&i = trim(left("&&_med_&i"));
         %else &&_arm_&i = "0";;
      %end;
      output;
      label = "   &trMin:&trMax";
      %do i = 1 %to &_nArm_;
         %if %symexist(_min_max_&i) %then &&_arm_&i = trim(left("&&_min_max_&i"));
         %else &&_arm_&i = "0:0";;
      %end;
      output;
      label = "";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
   run;

   data _tp1;
      set _tp1 _tp12;
   run;

   * section 2;
   proc sort data=table;
      by &TrtVar descending TimeWindowNum;
   run;

   proc summary data=table;
      by &TrtVar descending TimeWindowNum;
      output out=_p2;
   run;

   data _p2;
      set _p2;
      by &TrtVar;
      retain _accPatient;
      %do i = 1 %to &_nArm_;
         if &TrtVar = "&&_armLabel_&i" then _trtNum = &i;
      %end;
      if first.&TrtVar then do;
         _accPatient = 0;
      end;
      _accPatient = _accPatient + _FREQ_;
      name = "_nPatient_"||trim(left(_trtNum))||"_"||trim(left(TimeWindowNum));
      call symput(name,_FREQ_);
      name = "_nAccPatient_"||trim(left(_trtNum))||"_"||trim(left(TimeWindowNum));
      if TimeWindowNum = 0 then call symput(name,_FREQ_);
      else call symput(name,_accPatient);
      if last.&TrtVar then do;
         name = "_TotalPatient_" || trim(left(_trtNum));
         call symput(name,_accPatient);
      end;
   run;

   * Checking accumulated counts second time for patching time-window with 0 count;
   %do j = 1 %to %to &NWindows;
      %do i = 1 %to &_nArm_;
         %let k = %eval(&NWindows-&j+1);
         %if ^%symexist(_nAccPatient_&i._&k) %then %do;
            %if &k = &NWindows %then %do; 
               %let _nAccPatient_&i._&k = 0;
            %end;
            %else %do;
               %let l = %eval(&k+1);
               %let _nAccPatient_&i._&k = &&_nAccPatient_&i._&l;
            %end;
         %end;
      %end;
   %end;
   
   %let trtnopapcg = %nrbquote(%LocaleGet(TotalNumPatientsAdminPerCycleGroupsNPCT_Text));
   %let trmissdur = %nrbquote(%LocaleGet(Missingduration_Text));
   %let trnopwtcbc = %nrbquote(%LocaleGet(NumPatientsWithTrtCyclesByCatNPCT_Text));

   data _tp2;
      length label $70 
         %do i = 1 %to &_nArm_;
            &&_arm_&i $20
         %end;
      ;
      label = "&trtnopapcg";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "   &trmissdur";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_nPatient_&i._0) %then %do;
            %let _nPatient_&i._0 = 0;
            &&_arm_&i = "0(0%)";
         %end;
         %else %do;
            &&_arm_&i = trim(left("&&_nPatient_&i._0"||"("||
               "%sysfunc(round(%sysevalf(&&_nPatient_&i._0/&&_TotalPatient_&i)*100,.1))"||
               "%)"));
         %end;
      %end;
      output;
      %do j = 1 %to &NWindows;
         %if &_tFlag = 1 %then %do;
            %if %eval((&j-1)*&_increment+1) ne %eval(&j*&_increment) %then %do;
               label = "   " || trim(left("%eval((&j-1)*&_increment+1)")) || "-" ||
                  trim(left("%eval(&j*&_increment)")) || " &trTimeScale";
            %end;
            %else %do;
               %if %eval(&j*&_increment) = 1 %then %do;
                  label = "   " || trim(left("%eval(&j*&_increment)")) || " &trSingleTime";
               %end;
               %else %do;
                  label = "   " || trim(left("%eval(&j*&_increment)")) || " &trTimeScale";
               %end;
            %end;
         %end;
         %else %do;
            %if %eval(&&_TimeStart_&j) ne %eval(&&_TimeEnd_&j) %then %do; 
               label = "   " || trim(left("&&_TimeStart_&j")) || "-" ||
                  trim(left("&&_TimeEnd_&j")) || " &trTimeScale";
             %end; 
             %else %do;
               %if  %eval(&&_TimeStart_&j) = 1 %then %do;
                  label = "   " || trim(left("&&_TimeStart_&j")) || " &trSingleTime";
               %end;
               %else %do;
                  label = "   " || trim(left("&&_TimeStart_&j")) || " &trTimeScale";
               %end;
              %end;
         %end;
         %do i = 1 %to &_nArm_;
            %if ^%symexist(_nPatient_&i._&j) %then %do;
               &&_arm_&i = "0(0%)";
            %end;
            %else %do; 
            %*put i=&i, j=&j, _nPatient_=&&_nPatient_&i._&j;
               &&_arm_&i = trim(left("&&_nPatient_&i._&j"||"("||
                  "%sysfunc(round(%sysevalf(&&_nPatient_&i._&j/&&_TotalPatient_&i)*100,.1))"||
                  "%)"));
            %end;
         %end;
         output;
      %end;
      label = "";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;

      * section 3;
      label = "&trnopwtcbc";
      %do i = 1 %to &_nArm_;
         &&_arm_&i = "";
      %end;
      output;
      label = "   &trmissdur";
      %do i = 1 %to &_nArm_;
         %if ^%symexist(_nPatient_&i._0) | ^%symexist(_TotalPatient_&i) %then %do;
            &&_arm_&i = "0(0%)";
         %end;
         %else %do; 
            &&_arm_&i = trim(left("&&_nPatient_&i._0"||"("||
               "%sysfunc(round(%sysevalf(&&_nPatient_&i._0/&&_TotalPatient_&i)*100,.1))"||
               "%)"));
         %end;
      %end;
      output;
      %do j = 1 %to &NWindows;
         %if &_tFlag = 1 %then %do;
            label = "   >= " || trim(left("%eval((&j-1)*&_increment+1)")) || " " ||"&trTimeScale";
         %end;
         %else %do;
            label = "   >= " || trim(left("&&_TimeStart_&j")) || " " ||"&trTimeScale";
         %end;
         %do i = 1 %to &_nArm_;
            %if ^%symexist(_TotalPatient_&i) | &&_nAccPatient_&i._&j = 0 %then %do;
               &&_arm_&i = "0(0%)";
            %end;
            %else %do; 
               &&_arm_&i = trim(left("&&_nAccPatient_&i._&j"||"("||
                  "%sysfunc(round(%sysevalf(&&_nAccPatient_&i._&j/&&_TotalPatient_&i)*100,.1))"||
                  "%)"));
            %end;
         %end;
         output;
      %end;
   run;

   data _FinalTable;
      set _tp1 _tp2;
   run;

%end;

data _FinalTable;
   set _FinalTable;
   if substr(label,1,1) = " " & length(label)>1 then do; 
      * handling indentation for each format;
      Label_rtf = %str('03'x) || " " ||%str('03'x) || Label;
      Label_pdf = %str('00'x) || Label;
      Label_html = %str('1E'x) || Label;
   end;
   else do;
      Label_rtf = Label;
      Label_pdf = Label;
      Label_html = Label;
   end;
run;

exit:;
%mend EX_Report; 

/*****************************************************************
/*    Macro function for the following analysis
/*    Percent of Subjects Left on Study Medication by Day of Study
/*    Distribution of Doses by Arm
/*    Dose Descriptive Statistics by Arm and Box Plot of Doses
/*    Planned Arm to Actual Treatment Comparison
/*    Dose Changes During the Study 
/*          EX: EXDOSFRQ                      */
/*              EXTRT                         */
/*              EXDOSE                        */
/*              EXENDTC and EXSTDTC           */
/*              USUBJID                       */
/*          DM: ARM or ACTARM                 */
/*              USUBJID                       */
/*              RFSTDTC                       
*******************************************************************/
%macro expoAnalyses_5;

      /****************************************/
      /* 2. ANALYSIS B: DISTRIBUTION OF DOSES */
      /*                                      */
      /****************************************/
   %if &Server = Local %then %do;   
      libname JMPCMeta "&LS_HOME\Sample Data\Nicardipine";
   %end;
   %else %do;
      libname JMPCMeta "&USER_HOME/Exposure";
   %end;
   data exdosfrq;
      set JMPCMeta.exdosfrq;
      index = index(frequency,'/');
      if not index then exdosfrqn = input(frequency,8.);
      else exdosfrqn = input(substr(frequency,1,index-1),8.) / input(substr(frequency,index+1),8.);
   run;
   
   %let trARM = %nrbquote(%LocaleGet(ARM_Text));
   %let trBasicStatistics = %nrbquote(%LocaleGet(BasicStatistics_Text));
   %let trChangeDate = %nrbquote(%LocaleGet(ChangeDate_Text));
   %let trDayofStudy = %nrbquote(%LocaleGet(DayofStudy_Text));
   %let trDoses = %nrbquote(%LocaleGet(Doses_Text));
   %let trListtruncatedatX = %nrbquote(%LocaleGet(ListtruncatedatX_Text));
   %let trListtruncatedat2500 = %nrbquote(%LocaleFill(%nrbquote(&trListtruncatedatX),%nrbquote(2500)));
   %let trMax = %nrbquote(%LocaleGet(Max_Text));
   %let trMaxDoses = %nrbquote(%LocaleGet(MaxDoses_Text));
   %let trMean = %nrbquote(%LocaleGet(Mean_Text));
   %let trMedian = %nrbquote(%LocaleGet(Median_Text));
   %let trMin = %nrbquote(%LocaleGet(Min_Text));
   %let trMode = %nrbquote(%LocaleGet(Mode_Text));
   %let trN10thpercentile = %nrbquote(%LocaleGet(N10thpercentile_Text));
   %let trN25thpercentile = %nrbquote(%LocaleGet(N25thpercentile_Text));
   %let trN75thpercentile = %nrbquote(%LocaleGet(N75thpercentile_Text));
   %let trN90thpercentile = %nrbquote(%LocaleGet(N90thpercentile_Text));
   %let trNewDose = %nrbquote(%LocaleGet(NewDose_Text));
   %let trNoReportedChangeinDose = %nrbquote(%LocaleGet(NoReportedChangeinDose_Text));
   %let trNotGiven = %nrbquote(%LocaleGet(NotGiven_Text));
   %let trNUMSIGNofObs = %nrbquote(%LocaleGet(NUMSIGNofObs_Text));
   %let trOriginalDose = %nrbquote(%LocaleGet(OriginalDose_Text));
   %let trOverXchangesindose = %nrbquote(%LocaleGet(OverXchangesindose_Text));
   %let trOver2500changesindose = %nrbquote(%LocaleFill(%nrbquote(&trOverXchangesindose),%nrbquote(2500)));
   %let trPercentOfMaximumDosesTaken = %nrbquote(%LocaleGet(PercentofMaximumDosesTaken_Text));
   %let trPercentOfSubjects = %nrbquote(%LocaleGet(PercentofSubjects_Text));
   %let trMaxDose = %nrbquote(%LocaleGet(MaxDoses_Text));
   %let trPercentOfSubjsRemStudyMed = %nrbquote(%LocaleGet(PercentofSubjectsRemainingonStudyMedication_Text));
   %let trSD = %nrbquote(%LocaleGet(SD_Text));
   %let trSubjectID = %nrbquote(%LocaleGet(SubjectID_Text));
   

   ** MERGE THE EXPOSURE AND PATIENT DATASETS AND DETERMINE STUDY DAYS AND NUMBER OF DOSES ****;
   proc sort data = ex;
      by USUBJID;
   run;
   proc sort data = dm;
       by USUBJID;
   run;
   
   * check if RFSTDTC available in dm;
   data _null_;
         set dm(keep=RFSTDTC) end=_e_;
         retain _c_ 0 max_len 0;
         if length(trim(left(RFSTDTC))) then do;
            _c_ = 1;
            if max_len < length(trim(left(RFSTDTC))) then max_len= length(trim(left(RFSTDTC)));
         end;
         if _e_ then do;
            call symputx('dateflag',_c_);
            call symputx('datevar',max_len);
         end;
   run;
   
   %if &dateflag & &datevar>1 %then %do;
      data ex_dm;
         merge ex(in=a) dm(in=b);
         format studydate treatdate enddate &IntDateFormat..;
         by USUBJID;
         if a and b;
   
         treatdate = input(substr(EXSTDTC,1,10), yymmdd10.);
         studydate = input(substr(RFSTDTC,1,10), yymmdd10.); 
         enddate = input(substr(EXENDTC,1,10), yymmdd10.);


         /* CALCULATE THE STUDY DAY OF THE EXPOSURE EVENT */
         if enddate = . then do;
           if treatdate >= studydate then studydays = (treatdate - studydate)+1;
             else studydays = (treatdate - studydate);
         end;
         else do;
         if enddate >= studydate then studydays = (enddate - studydate)+1;
           else studydays = (enddate - studydate);
         end;
         *if studydays > 0;


         /* CALCULATE THE PERIOD IN DAYS DURING WHICH THE SUBJECT WAS TAKING DOSES OF THE DRUG */
         studydays0 = lag(studydays);
         if first.usubjid then dose_days = studydays;
         else dose_days = studydays - studydays0;
         if not missing(dose_days) and dose_days <= 0 then dose_days = 1;
      run;
   %end;
   %else %do;
      proc sort data=ex out=ex_chkEndDate(keep =USUBJID EXENDTC) nodupkey;
         by USUBJID EXENDTC;
      run;

      data ex_chkEndDate;
         set ex_chkEndDate;
         if EXENDTC= "" then delete;
      run;
      %Nobsvars(ex_chkEndDate);
      %let chkEndobs = &nobs;
      %Nobsvars(ex);
      %let exObs = &nobs;

      %if &exObs > &chkEndobs %then %do;
         data ex;
            merge ex(drop=EXENDTC) ex_chkEndDate;
            by USUBJID;
         run;
      %end;
      data ex_dm;
         merge ex(in=a) dm(in=b);
         format treatdate enddate &IntDateFormat..;
         by USUBJID;
         if a and b;
         treatdate = input(substr(EXSTDTC,1,10), yymmdd10.);
         enddate = input(substr(EXENDTC,1,10), yymmdd10.);
         if enddate >= treatdate then studydays = (enddate - treatdate)+1;
         /* CALCULATE THE PERIOD IN DAYS DURING WHICH THE SUBJECT WAS TAKING DOSES OF THE DRUG */
         studydays0 = lag(studydays);
         if first.usubjid then dose_days = studydays;
         else dose_days = studydays - studydays0;
         if not missing(dose_days) and dose_days <= 0 then dose_days = 1;
      run;
      data ex_dm;
         set ex_dm;
         if studydays= . then delete;
      run;
   %end;

   data ex_dm;
      length  exdosfrq $20 exdosfrqn 8. unit $1;
      set ex_dm;
      
      /* LOOK UP THE FREQUENCY OF DOSES PER INTERVAL FROM THE REFERENCE TABLE */
      if _n_ = 1 then do;
         declare hash h(dataset:'exdosfrq');
         h.definekey('exdosfrq');
         h.definedata('exdosfrqn','unit');
         h.definedone();
      end;

      label exdosfrqn='Dosing Frequency Per Interval' unit='Dosing Frequency Unit';
      call missing(exdosfrqn,unit);
      rc = h.find();
      drop rc;

      /* if no dose frequency was found, assume one dose per study day */
      if exdosfrqn = . then do;
         exdosfrqn = 1;
         unit = 'd';
      end;

      /* special case to deal with injections */
      /* only one injection per exposure event */
      if index(upcase(exdosfrm),'INJECTION') then do;
         exdosfrqn = 1;
         unit = 'o';
      end;

      select (unit);
         when ('h') dose_days_fu = dose_days * 24;
         when ('d') dose_days_fu = dose_days;
         when ('w') dose_days_fu = dose_days / 7;
         when ('m') dose_days_fu = dose_days / 30.4375;
         when ('o') dose_days_fu = 1;
         when ('H') dose_days_fu = dose_days * 24;
         when ('D') dose_days_fu = dose_days;
         when ('W') dose_days_fu = dose_days / 7;
         when ('M') dose_days_fu = dose_days / 30.4375;
         when ('O') dose_days_fu = 1;
         otherwise;
      end;

      doses = dose_days_fu * exdosfrqn;
   run;  
   ** NOW DO THE DOSE BUCKET ANALYSIS ***;
   ** SUMMARIZE BY ID TO GET THE CUMMULATIVE NUMBER OF DOSES ****;
   proc summary data = ex_dm nway;
        class usubjid;
        id &TrtVar &DesignByVars;
        var doses;
        output out = by_usubjid(drop = _type_ _freq_)  sum=;
   run;
   ** NOW GET THE MAX DOSES FOR EACH ARM **;
   proc sort data = by_usubjid;
     by &TrtVar decending doses;
   run;
   data max_doses;
     set by_usubjid;
     by &TrtVar;
      if first.&TrtVar;
      max_doses = doses;
      drop usubjid doses;
   run;
   ** MERGE ON ***;
   proc sql;
     create table by_usubjid2
     as select b.*, m.*
     from by_usubjid b, max_doses m
     where b.&TrtVar = m.&TrtVar and 
     b.doses > 0
     ;
   quit; 
   ** DETERMINE THE PECENTS FOR EACH PERSON ***;
   data by_usubjid2;
     set by_usubjid2;

      percent = round(doses/max_doses,.01);
   run;
   ** NUMBER WITH EACH AMOUNT OF DOSE percents ***;
   proc summary data = by_usubjid2 nway;
     class &TrtVar percent;
     output out = by_doses(drop=_type_ rename=_freq_=subjects);
   run;
   ** NOW GO AND MAKE SURE ALL PERCENTS FROM 0 to 100 ARE IN THE DATASET FOR EACH ARM ***;
   data by_doses2(rename=(subjects2=subjects percent2=percent));
        set by_doses;
        by &TrtVar;
      percent0 = lag(percent);
      if first.&TrtVar then do;
      if percent > 0 then do;
        do J = 0 to percent-.01 by .01;
          percent2 = J;
         subjects2 = 0;
         output;
        end;
       percent2 =percent;
       subjects2 = subjects;
       output;
      end;
      else do;
       percent2 = percent;
       subjects2 = subjects;
       output;
      end;
      end;
      else do;
       if round((percent - percent0),.01) = .01 then do;
         percent2 = percent;
         subjects2 = subjects;
         output;
       end;
       else do;
        do I = percent0 to percent-.01 by .01;
          percent2 = I;
         subjects2 = 0;
         output;
        end;
        percent2 = percent;
        subjects2 = subjects;
        output;
       end;
      end;
      drop I j percent0 percent subjects;
   run;

   proc summary data = by_doses2 nway;
     class &TrtVar percent;
     var subjects;
     output out = by_doses2(drop=_type_ _freq_) sum=;
   run;
   ** NUMBER BY EACH ARM ***;
   proc summary data = by_usubjid2 nway;
     class &TrtVar;
     output out = by_treatment(drop = _type_ rename=(_freq_=num_treatment));
   run;

   data by_treatment;
     set by_treatment;
   arm_n = _n_;
   run;

   ** MERGE TO GET THE NUMBERATOR AND DENOMINATOR ON SAME DATASET ***;

   data by_doses_percs;
     merge by_doses2 by_treatment;
     by &TrtVar;
     n_percent = subjects / num_treatment;
   run;


   ** GET CUMMULATIVE PERCENT ***;
   data by_doses_percs;
        set by_doses_percs;
        by &TrtVar;
        retain cum_percent;

      if first.&TrtVar then cum_percent = 1;
      cum_percent = sum(cum_percent, - n_percent);
   run;
   ** GET THE LAST OBS BY ID ***;
   data ex_dm2;
      set ex_dm;
      by usubjid;
      /* get last nonmissing exposure event */
      where studydays ne .;
      if last.usubjid then output ex_dm2;
   run;
   ** NUMBER OF PEOPLE EACH DAY ***;
   proc summary data = ex_dm2;
     class studydays &TrtVar;
     output out = num_days;
   run;
   ** KEEP THE RIGHT OBS ***;

   proc sort data = num_days (where=(_type_ in (1,3)));
     by &TrtVar studydays;
   run;
   ** FIGURE OUT THE NUMBER OF TREATMENTS **;

   proc sort data = num_days nodupkey out = num_treatments;
     by &TrtVar;
   run;
   ** MAKE THE NUMBER OF TREATMENTS A MACRO **;
   %global num_treatments;
   proc sql noprint;
     select count(*) into: num_treatments
     from num_treatments;
   quit;
/*options mprint; */
   ** LOOP TO FIGURE OUT NUMBER OF PERCENTS BY ARM ***;
   %do n = 1 %to &num_treatments;

      data treatment;
        set num_treatments;
      if _N_ = &N;
      run;


      proc sql noprint;
        select &TrtVar into: treatment
        from treatment;
      quit;

      data _null_;
        set max_doses;
      if &TrtVar = "&treatment.";
      call symput('MD',max_doses);
      run;

      data _null_;
      treatment2 = trim(compress("&treatment.",'.'))||" (&trMaxDoses = "||compress("&MD.")||')';
      call symput('treatment2',treatment2);
      run;

      %global cumPercLabel percentLabel maxdoseLabel;
      %let cumPercLabel=%nrbquote(&trPercentOfSubjects);
      %let percentLabel=%nrbquote(&trPercentOfMaximumDosesTaken);
      %let maxdoseLabel=%nrbquote(&trMaxDose);
      %put cumPercLabel=&cumPercLabel percentLabel=&percentLabel;
      %put &treatment2.;

      data by_doses_percs&N.;
        set by_doses_percs;

      if &TrtVar = "&treatment.";
      cum_percent&N. = cum_percent*100;
      percent=percent*100;
      *percent= round(percent,.01);
      MaxDose = "&treatment2";
      label cum_percent&N = "&trPercentOfSubjects" percent ="&trPercentOfMaximumDosesTaken" MaxDose="&trMaxDose";
      arm&N = &TrtVar;

      keep percent cum_percent&N. MaxDose;
      run;
      ** MERGE BACK ALL THE DATASETS ***;
   %end; 
   %do S = 1 %to &num_treatments;
      data by_doses_percs&S;
         set by_doses_percs&S;
         rename cum_percent&S =PercentofSubjects;
         label PercentofSubjects ="Percent of Subjects";
      run;
   %end;
   data outlib.&OutDataName5;
      set
      %do S = 1 %to &num_treatments;
        by_doses_percs&S.
      %end;
      ;
   run;
   /*****************************************/
   /* 3. ANALYSIS C: BOXPLOT OF DOSES      
   /***************************************/
   proc univariate data = by_usubjid (where=(doses >= 0)) noprint;
     class &TrtVar;
     var doses;
     output out = final_stats n=n_subjects std=sd_doses mean=mean_doses min=min_doses max=max_doses
                 median=median_doses q1=q1_doses q3=q3_doses mode=mode_doses 
                 p10=p10_doses p90=p90_doses;
   run;
   data outlib.final_stats;
   set final_stats;
   run;

   ** DETERMINE THE NUMBER OF ARMS ***;
   %global num_arms armLevCol;
   %let armLevCol=;
   proc sql noprint;
     select count(*) into: num_arms
     from final_stats;
   quit;
   %do I = 1 %to &num_arms.;
      %global armlevel&I;
      data _null_;
        set final_stats;
        if _N_ = &I;
        call symput('arm',&TrtVar);
        call symput('arm_2',compress(&TrtVar,'.'));
      run;
      %let armlevel&I=&arm;
      %if I = &num_arms %then %do;
         %let armLevCol=&armLevCol :&arm;
      %end;
      %else %do;
         %let armLevCol=&armLevCol :&arm,;
      %end;
      ** TRANSFORM THE DATA TO VERTICAL **;
      data arm&I.(keep = sort_order arm&I.);
           set final_stats;
           format sort_order $20.;
         if &TrtVar = "&arm.";
         label arm&I. = "&arm.";

         sort_order = "&trNUMSIGNofObs";
         arm&I. = n_subjects;
         output;
         sort_order = "&trMean";
         arm&I. = round(mean_doses,0.1);
         output;
         sort_order = "&trSD";
         arm&I. = round(sd_doses,0.1);
         output;
         sort_order = "&trMedian";
         arm&I. = median_doses;
         output;
         sort_order = "&trMode";
         arm&I. = mode_doses;
         output;
         sort_order = "&trN10thpercentile";
         arm&I. = p10_doses;
         output;
         sort_order = "&trN25thpercentile";
         arm&I. = q1_doses;
         output;
         sort_order = "&trN75thpercentile";
         arm&I. = q3_doses;
         output;
         sort_order = "&trN90thpercentile";
         arm&I. = p90_doses;
         output;
         sort_order = "&trMin";
         arm&I. = min_doses;
         output;
         sort_order = "&trMax";
         arm&I. = max_doses;
         output;
   /*    sort_order = 'Median-P10';
         arm&I. = median_doses-p10_doses;
         output;
         sort_order = 'Q1';
         arm&I. = q1_doses;
         output;
         sort_order = 'Median - Q1';
         arm&I. = median_doses-q1_doses;
         output;
         sort_order = 'Q3 - Median';
         arm&I. = q3_doses-median_doses;
         output;
         sort_order = 'P90 - Median';
         arm&I. = p90_doses-median_doses;
         output;
         sort_order = 'Q1 - Min';
         arm&I. = q1_doses-min_doses;
         output;
         sort_order = 'Max - Q3';
         arm&I. = max_doses-q3_doses;
         output;*/

      run;

      proc sort data = arm&I.;
        by sort_order;
      run;
   %end;
   data final;
     merge arm:;
     by sort_order;
   run;
   data for_macro;
      format a $5.;
      do I = 1 to &num_arms.;
      a = compress("arm"||compress(I));
      output;
      end;
   run;

   proc sql noprint;
     select a into: arms separated by ','
     from for_macro;
   quit;

   proc sql;
     create table final_exposureC
     as select sort_order, &arms.
     from   final;
   quit; 
   data outlib.&OutDataName4;
      set final_exposureC;
      Label sort_order="&trBasicStatistics";
   run;
   data outlib.&OutDataName6;
      set by_usubjid;
      label &TrtVar="&trARM" doses="&trDoses";
   run;
   /********************************************************/
   /* 4. ANALYSIS D: PLANNED ARM VS ACTUAL TREATMENT TABLE 
   /********************************************************/  

   * add check for TrtVar;
   %let trtflag = 0;
   %global TrtARM;
   %let _tid_ = %sysfunc(open(dm));
   %if %sysfunc(varnum(&_tid_,&TrtVar)) %then %let trtflag = 1;
   %let _tid_ = %sysfunc(close(&_tid_));  
   %if &trtflag=0 %then %let TrtARM=ARM;
   %else %let TrtARM=&TrtVar;

   proc sql noprint;
      /* find out how many subjects there were per planned arm */
      create table ex_d_planned as
      select &TrtARM, count(distinct usubjid)   as arm_count %if %length(&DesignByVars) %then &DesignByVars2;
      from (select a.usubjid, 
     /*       %if &dm_actarm. %then plannedarm;
       %else arm; as arm*/
      a.&TrtARM as &TrtARM %if %length(&DesignByVars) %then &DesignByVars_a;
            from dm a,
                &in_ex b
           where a.usubjid = b.usubjid)
      group by &TrtARM %if %length(&DesignByVars) %then &DesignByVars2;;

      /* find the number of subjects per treatment and dose */
      create table ex_d_actual as
      select &TrtARM, extrt, exdose, exdosu, count(distinct usubjid) as extrt_count %if %length(&DesignByVars) %then &DesignByVars2;
      from (select a.usubjid, 
                        /*%if &dm_actarm. %then plannedarm;
                       %else arm; as arm, */
                b.&TrtARM as &TrtARM,
                        extrt, exdose, exdosu %if %length(&DesignByVars) %then &DesignByVars_a;
            from &in_ex a,
                dm b
             where a.usubjid = b.usubjid)
      group by &TrtARM, extrt, exdose, exdosu %if %length(&DesignByVars) %then &DesignByVars2;;
      /* combined the planned and actual counts */
      create table ex_d_pva as
      select a.&TrtARM, arm_count, extrt, exdose, exdosu, extrt_count %if %length(&DesignByVars) %then &DesignByVars_a;
      from ex_d_planned a,
           ex_d_actual b 
      where a.&TrtARM = b.&TrtARM
      order by a.&TrtARM, extrt, exdose, exdosu %if %length(&DesignByVars) %then &DesignByVars2;;
   quit;

   /* format the planned vs. actual table for output */
   data final_exposureD;
      retain &TrtARM arm_count actual actual_dose actual_count;
      set ex_d_pva;
      by &TrtARM extrt exdose;
      if not first.&TrtARM then do;
         &TrtARM = '';
         arm_count = .;
      end;

      if not first.extrt then extrt = '';
      extrt = propcase(extrt);

      length actual $100 actual_dose $100; 
      actual = extrt;
      actual_dose = left(ifc(not missing(exdose),compress(exdose),'')||' '||
                        ifc(not missing(exdosu),exdosu,''));
      actual_count = extrt_count;

      keep &TrtARM arm_count actual actual_dose actual_count %if %length(&DesignByVars) %then &DesignByVars;;
   run;  

   data final_exposureD;
      length arm_count2 $9;
      set final_exposureD;
      if arm_count =. then arm_count2="";
      else arm_count2=arm_count;
      drop arm_count;
      rename arm_count2=arm_count;
   run;
   %if %length(&DesignByVars) %then %do;
      proc sort data=final_exposureD;
         by &DesignByVars;
      run;
   %end;
   /********************************************************/
   /*             5. ANALYSIS E: DOSE CHANGES 
   /********************************************************/
   ** OUTPUT IF THERE IS A CHANGE IN TREATMENT OR DOSE FROM ONE RECORD TO THE NEXT ****;
   %global changes oldDoseLen newDoseLen;
   data ex2;
      set ex_dm;
      format change_date mmddyy10. exadj $50.;
      usubjid0 = lag(usubjid);
      extrt0 = lag(extrt);
      exdose0 = lag(exdose);
      exdosu0 = lag(exdosu);
      if exendtc ne "" then change_date = input(exendtc, yymmdd10.);
        else change_date = treatdate ;
      if usubjid0 = usubjid;
      if (extrt0 ne extrt) or (exdose0 ne exdose);
      old_dose = compress(trim(left(extrt0))|| ' '||compress(trim(left(exdose0)))||' '||compress(trim(left(exdosu0))));
      new_dose = compress(trim(left(extrt))|| ' '||compress(trim(left(exdose)))||' '||compress(trim(left(exdosu))));
      if exadj = '' then exadj = "&trNotGiven";
      else exadj = propcase(exadj);
      label old_dose = "&trOriginalDose"
            new_dose = "&trNewDose"
           usubjid = "&trSubjectID"
           change_date = "&trChangeDate"
           studydays = "&trDayofStudy"
           ;
   run;
   ** SEE IF THERE WAS A CHANGE IN COUNT FOR OUTPUT **;
   proc sql noprint;
     select count(*) into: changes
     from ex2;
   quit;
   %if %eval(&changes.) = 0 %then %do;

      data final_exposureE;
         usubjid = "&trNoReportedChangeinDose";
      run;
   %end;
   %else %do;
      proc sql;
        create table final_exposureE
        as select usubjid, old_dose, new_dose, change_date, studydays, exadj %if %length(&DesignByVars) %then &DesignByVars2;
        from ex2
        order by usubjid, change_date %if %length(&DesignByVars) %then &DesignByVars2;;
      ;
      quit;
      data final_exposureE;
         set final_exposureE(obs=2500) end=eof;
         by usubjid change_date;
         if not first.usubjid then do;
            usubjid = '';
         end;
         output;
         if _n_ = 2500 and eof then do;
            usubjid = "&trOver2500changesindose";
            old_dose = "&trListtruncatedat2500";
            new_dose = '';
            studydays = .;
            exadj = '';
            output;
         end;
      run;
      %let tid = %sysfunc(open(final_exposureE));
      %let nVars = %sysfunc(attrn(&tid,NVARS));
      %do i = 1 %to &nVars;
         %let olddname=%sysfunc(VARNAME(&tid,&i));
         %if &olddname = old_dose %then %do;
            %let oldDoseLen = %sysfunc(varlen(&tid,&i));
         %end;
         %if &olddname = new_dose %then %do;
            %let newDoseLen = %sysfunc(varlen(&tid,&i));
         %end;       
      %end;    
      %let tid = %sysfunc(close(&tid));
      %*put oldDoseLen=&oldDoseLen newDoseLen=&newDoseLen;
      
      %if %length(&DesignByVars) %then %do;
         proc sort data=final_exposureE;
            by &DesignByVars;
         run;
      %end;
   %end;
   
%mend expoAnalyses_5;

%macro ExposureSummary;

*initialization and argument checking;
%let ProcessName = ExposureSummary; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%if ^%sysfunc(findw(&domains, EX)) %then %do;
   %put [MS_S_0090];
   %put ERROR: The EX or ADEX data set is required for exposure information.;
   %let exiterror = 1;
   %goto exit;
%end;

%LoadData(domlist = EX,mergesupp = &MergeSuppDom,checkreview=1);

%let trBasicStatistics = %nrbquote(%LocaleGet(BasicStatistics_Text));
%let trDoseGroups = %nrbquote(%LocaleGet(DoseGroups_Text));
%let trPercentOfSubjsRemStudyMed = %nrbquote(%LocaleGet(PercentofSubjectsRemainingonStudyMedication_Text));

%if ^%sysfunc(indexw(&EXVARS, EXSTDTC, ' ')) %then %do;
   %put [MS_S_0091];
   %put ERROR: EXSTDTC is not available in either EX or ADEX.;
   %let exiterror = 1;
   %goto exit;
%end;
%if ^%sysfunc(indexw(&EXVARS, EXENDTC, ' ')) %then %do;
   %put [MS_S_0092];
   %put ERROR: EXENDTC is not available in either EX or ADEX.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %sysfunc(findw(&ADSLVARS, RFSTDTC)) %then %let adslst = RFSTDTC;
%else %let adslst =;
%if %sysfunc(findw(&ADSLVARS, RFENDTC)) %then %let adslen = RFENDTC;
%else %let adslen =;

%if (^%length(&adslst) or ^%length(&adslen)) %then %do;
   %if ^%length(&adslst) %then %let adslst = RFSTDTC;
   %if ^%length(&adslen) %then %let adslen = RFENDTC;
      %put [MS_S_0021];
   %put ERROR: ADSL, ADDM, or DM is missing the following required variables: &adslst  &adslen;
   %let exiterror = 1;
   %goto exit;
%end;

*for newly added 5 analyses, the following variables are required;
%if ^%sysfunc(indexw(&EXVARS, EXTRT, ' ')) or ^%sysfunc(indexw(&EXVARS, EXDOSE, ' ')) or ^%sysfunc(indexw(&EXVARS, EXDOSU, ' ')) %then %do;
   %put [MS_S_0093];
   %put ERROR: Either EXTRT, EXDOSE or EXDOSU is not available in either EX or ADEX.;
   %put ERROR: EXTRT, EXDOSE, and EXDOSU must exist to run this analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&TrtVar) %then %let TrtVar = &TrtName;
%global TimeScale  TimeWindowMethod NWindows TimeWindows;
%let TimeScale = &DurationTimeScale;
%let TimeWindowMethod = &DurationWindowMethod;
%let NWindows = &NDWindows;
%let TimeWindows = &DurationWindows;
   
%PrepOutDS(ExposureSummary_OutData,OutDataName,ex,_esp);
%PrepOutDS(ExposureSummary_OutData2,OutDataName2,ex,_es2);
%PrepOutDS(ExposureSummary_OutData3,OutDataName3,ex,_dur);
%PrepOutDS(ExposureSummary_OutData4,OutDataName4,ex,_st);
%PrepOutDS(ExposureSummary_OutData5,OutDataName5,ex,_es5);
%PrepOutDS(ExposureSummary_OutData6,OutDataName6,ex,_es6);
%PrepOutDS(ExposureSummary_OutReport,OutReport,ex,_Summary);

%global DesignByVars2 DesignByVars_a;

%if %length(&DesignByVars) %then %do;
   %let i = 1;
   %do %while(%length(%scan(&DesignByVars,&i)));
      %if &i=1 %then %do;
         %let DesignByVars2 =,%scan(&DesignByVars,&i);
         %let DesignByVars_a =,a.%scan(&DesignByVars,&i);
      %end;
      %else %do;
         %let DesignByVars2 =&DesignByVars2,%scan(&DesignByVars,&i);
         %let DesignByVars_a =&DesignByVars_a,a.%scan(&DesignByVars,&i);
      %end;
      %let i = %eval(&i+1);
   %end;
   %let _nDBVar = %eval(&i-1);
%end;

%if &Output_Text = No %then %do;
   ods listing close;
%end;
%if &syserr > 4 %then %goto exit;

* check if dose variable (EXDOSE) exist;
%if ^%sysfunc(indexw(&EXVARS, EXDOSE, ' ')) %then %do;
   %let _DoseFlag = 0;
%end;
%else %do;
   %if &nDoseBin < 2 %then %do;
      %let _DoseFlag = 0;
      %put The specified Number of Bins for Dose Group is lass than 2. No Dose grouping will be applied.;
   %end;
   %else %do;
      %let _DoseFlag = 1;
      * update dose binning group;
      proc sort data=&in_ex(keep=EXDOSE) out=_tmp nodupkey;
         by EXDOSE;
      run;
      data _tmp;
         set _tmp;
         if _n_ = 1 then do;
            if EXDOSE = . then call symput("_missingDose",1);
            else call symput("_missingDose",0);
         end;
         if EXDOSE ne .;
      run;
      data _null_;
         set _tmp end=_e_;
         if _n_ = 1 then call symput("_dLeast",EXDOSE);
         if _e_ then do;
            call symput("_nDose",_n_);
            call symput("_dLargest",EXDOSE);
         end;
      run;
      %if &_missingDose %then %let _nDoseGroups = %eval(&_nDose + 1);
      %else %let _nDoseGroups = &_nDose;
   
      * create dummy dose groups;
      data _ex_2;
         set &in_ex end=_e_;
         Length _EXDOSEG $30;
         if missing(EXDOSE) then do;
            _EXDOSEG = "NA";
         end;
         else do;
            %if &_nDose <= &nDoseBin %then %do;
               _EXDOSEG = trim(left(EXDOSE)) || " " || EXDOSU;
            %end;
            %else %do;
               * reassign dose group;
               %let _dIncrement = %sysfunc(round(%sysevalf((&_dLargest-&_dLeast)/&nDoseBin),.1));
               %do i = 1 %to %eval(&nDoseBin-1);
                  %let _low_ = %sysevalf(&_dLeast+&_dIncrement*(&i-1));
                  %let _up_ = %sysevalf(&_dLeast+&_dIncrement*&i);
                  if EXDOSE >= &_low_ & EXDOSE < &_up_ then 
                     _EXDOSEG = "D" || trim(left(&i)) ||": " || trim(left(&_low_)) || " - " || trim(left(&_up_)) || " " || EXDOSU;
               %end;
               * last group;
               %let _low_ = %sysevalf(&_dLeast+&_dIncrement*(&i-1));
               if EXDOSE >= &_low_ then _EXDOSEG = "D" || trim(left(&i)) ||": >= " || trim(left(&_low_)) || " " || EXDOSU;
            %end;
         end;
         label _EXDOSEG = "&trDoseGroups";
      run;

      %let in_ex = _ex_2;
   %end;
%end;

******** Hard Coded Macro **************;
%let CycleVar = VISITNUM;
******** Hard Code **************;
%if %substr(%upcase(%nrbquote(&TimeScale)),1,6)=CYCLES %then %let TimeScale = Cycles;
%if &TimeScale = Cycles %then %do;
   %CheckVarExist(&CycleVar,&in_ex,,_ShowError=0);
   %if &exiterror %then %do;
      %put [MS_S_0094];
      %put ERROR: Variable VISITNUM was not in the EX dataset. This variable is required for applying Cycles for Time Scale.;  
      %goto exit;
   %end;
%end;

%let EXDURFlag = 1;
%CheckVarExist(EXDUR,&in_ex,,_ShowError=0);
%if &exiterror %then %do;
   %let EXDURFlag = 0;
   %let exiterror = 0;
%end;

* 1. clean up exposure dataset;
proc sort data=&in_ex out=ex(%if &EXDURFlag %then drop=EXDUR;);
   by USUBJID EXSTDTC;
run;
proc sort data=&in_adsl out=adsl;
   by USUBJID;
run;

%if ^%sysfunc(findw(&domains, DM)) %then %do;
   %put [MS_S_0015];
   %put ERROR: The DM or ADDM data set does not exist for this study.;
   %let exiterror = 1;
   %goto exit;
%end; 
proc sort data=&in_dm out=dm;
   by USUBJID;
run;

*calculate the total count per treatment group;
proc freq data=&in_adsl noprint;
   tables &TrtName / out=pc;
   where not missing(&TrtName);
run;

data _null_;
   set pc;
   call symput ("trtCount"||compress(_n_),trim(left(count)));
   call symput ("trtLevel"||compress(_n_),trim(left(&TrtVar)));
   call symput ("trtObs",trim(left(_n_)));
run;
      
/* Hash Merge;
data es;
   if 0 then set &in_adsl;
   if _N_ = 1 then do;
      declare hash h(dataset: "&in_adsl", hashexp: 6);
      h.defineKey('USUBJID');
      h.defineDone();
   end;
   set es;
   if (h.find() = 0) then output; 
run;
*/

data ex;
   merge ex (in=_e_) adsl(keep=USUBJID %if %sysfunc(findw(&ADSLVARS, ETHNIC)) %then %do; ETHNIC %end; &DesignByVars &TrtVar in=_a_) dm(in=_d_);
   by USUBJID;
   if _a_ & _e_ & _d_;
   if &TrtVar  = " " then delete;
   %calculateStudyDays(ex);
   /*
   Ref_End_Date = ENDY;
   if input(EXSTDTC,is8601da.) = . then delete;
   * if end of trement date is missing, then set it to starting treatment date + 1;
   if input(EXENDTC,is8601da.) = . then do;
      Ref_End_Date = put(input(EXSTDTC,is8601da.)+1,e8601da.);
      Missing_End_Date = 1;
   end;
   else do;
      Ref_End_Date = EXENDTC;
      Missing_End_Date = 0;
   end;
   */
   if missing(ENDY) then ENDY = STDY;
   if STDY=ENDY then EXDUR =1;
run;

%if &syserr > 6 %then %do;
	%goto exit;
%end;
/*
%CheckVarExist(EXDUR,ex,_ShowError=0);
%if &exiterror %then %do;
   %let exiterror = 0; 
   %CheckVarExist(EXDURN,ex,_ShowError=0);
   %if &exiterror %then %do;
      %let EXDUR_Flag = 0;
      %let exiterror = 0; 
   %end;
   %else %do;
      %let EXDUR_Flag = 2; * with numeric values;
   %end;
%end;
%else %do;
   %let EXDUR_Flag = 1; * with char values (ISO 8601);
%end;

data ex; 
   set ex;
   by USUBJID;
   retain Ref_Start_Date;
   if first.USUBJID then do;
      Ref_Start_Date = STDY;
   end;
   Trt_Duration = EXDUR;
run;
*/

/* need to calculate non-overlapping intervals */
/* use for report (summarized over dose groups) and if doseflag is 0 */
proc sort data = ex;
by USUBJID STDY EXDUR;
run;

/* make consistent intervals of exposure, use for report and when no dose groups*/
data exuni; 
   set ex;
   by USUBJID;
   retain Start_Duration End_Duration Trt_Duration ;
   if first.USUBJID then do;
      Start_Duration = STDY;
      End_Duration = ENDY;
      Trt_Duration = EXDUR;
   end;
   if (not first.USUBJID and  STDY = End_Duration and ENDY=STDY) then delete;
   if (not first.USUBJID and STDY = End_Duration) then STDY = STDY+1;
   if STDY>End_Duration then do;
      Start_Duration = STDY;
      End_Duration = ENDY;
      Trt_Duration = EXDUR;
   end;
   if (STDY = End_Duration and ENDY>End_Duration) then do;
      End_Duration = ENDY;
      Trt_Duration = Trt_Duration + EXDUR;
   end;
run; 

/* get non duplicated non overlapping intervals of duration for exposure */
/* use this data set for reporting */
proc sort data =exuni out=exuni nodupkey;
   by USUBJID Start_Duration End_Duration;
run;

* to calculate Percent of Subjects Left on Study Medication by Day of Study;
data exPcount;
   set exuni;
   if Start_Duration ne . and End_Duration ne . then do;
      do Study_Duration = Start_Duration to End_Duration;
         Count = 1;
         output;
      end;
   end;
   keep &TrtVar USUBJID &DesignByVars Study_Duration Count;
run;
proc sort data = exPcount;
   by &TrtVar &DesignByVars Study_Duration;
run;

proc means data = exPcount noprint;
   by &TrtVar &DesignByVars Study_Duration;
   var count;
   output out = trtcp(drop= _Type_ _Freq_) n=nSubjects;
run;
/* end of calculate subject left on each day of medication .*/

%if ^&_DoseFlag %then %do;
   /* expand out if on trtmnt for each subject to get counts for */
   data excount; 
      set exuni;
      if Start_Duration ne . and End_Duration ne . then do;
         do Study_Duration = Start_Duration to End_Duration;
            Count = 1;
            output;
         end;
      end;
      keep &TrtVar USUBJID &DesignByVars Study_Duration Count;
   run;

   proc sort data = excount;
      by &TrtVar &DesignByVars Study_Duration;
   run;
   /* data set now has summarized counts for JMP plot */
   proc means data = excount noprint;
      by &TrtVar &DesignByVars Study_Duration;
      var count;
      output out = exm(drop= _Type_ _Freq_) n=nSubjects;
   run;
%end;
%if &_DoseFlag %then %do;
   proc sort data = ex;
      by _EXDOSEG USUBJID STDY EXDUR;
   run;
   
   /* make consistent intervals of exposure, use for report and when no dose groups*/
   data exunid; 
      set ex;
      by _EXDOSEG USUBJID;
      retain Start_Duration End_Duration Trt_Duration ;
      if first.USUBJID then do;
         Start_Duration = STDY;
         End_Duration = ENDY;
         Trt_Duration = EXDUR;
      end;
      if (not first.USUBJID and  STDY = End_Duration and ENDY=STDY) then delete;
      if (not first.USUBJID and STDY = End_Duration) then STDY = STDY+1;
      if STDY>End_Duration then do;
         Start_Duration = STDY;
         End_Duration = ENDY;
         Trt_Duration = EXDUR;
      end;
      if (STDY = End_Duration and ENDY>End_Duration) then do;
         End_Duration = ENDY;
         Trt_Duration = Trt_Duration + EXDUR;
      end;
   run; 
   
   /* get non duplicated non overlapping intervals of duration for exposure for each dose group*/
   /* use this data set for  jmp graphic */
   proc sort data =exunid out=exunid nodupkey;
      by USUBJID _EXDOSEG Start_Duration End_Duration;
   run;
   
   /* expand out if on trtmnt for each subject to get counts for */
      data excount; 
         set exunid;
         if Start_Duration ne . and End_Duration ne . then do;
            do Study_Duration = Start_Duration to End_Duration;
               Count = 1;
               output;
            end;
         end;
         keep &TrtVar _EXDOSEG USUBJID &DesignByVars Study_Duration Count;
      run;
   
      proc sort data = excount;
         by &TrtVar _EXDOSEG &DesignByVars Study_Duration;
      run;
      /* data set now has summarized counts for JMP plot */
      proc means data = excount noprint;
         by &TrtVar _EXDOSEG &DesignByVars Study_Duration;
         var count;
         output out = exm(drop= _Type_ _Freq_) n=nSubjects;
      run;
%end;

%if &TimeScale ne Cycles %then %do;
   * calculate total duration;
   proc sort data=exuni out=exr;
      by &TrtVar &DesignByVars USUBJID;
   run;

   data exuni_date;
      set exr;
      by &TrtVar &DesignByVars USUBJID;
      retain Total_Treatment_Duration ;
      if first.USUBJID then do;
         Total_Treatment_Duration = Trt_Duration;
      end;
      else do;
         Total_Treatment_Duration = Total_Treatment_Duration + Trt_Duration;
      end;
      if last.USUBJID then do;
         %if %Upcase(&TimeScale)=WEEKS %then %do;
            Total_Treatment_Duration = ceil(Total_Treatment_Duration/7);
         %end;
         output;     
      end;
      keep USUBJID &TrtVar 
           %if %sysfunc(findw(&ADSLVARS, AGE)) %then %do; AGE %end; 
           %if %sysfunc(findw(&ADSLVARS, SEX)) %then %do; SEX %end;
           %if %sysfunc(findw(&ADSLVARS, RACE)) %then %do; RACE %end;
           %if %sysfunc(findw(&ADSLVARS, SITEID)) %then %do; SITEID %end;
           %if %sysfunc(findw(&ADSLVARS, STUDYID)) %then %do; STUDYID %end;
           %if %sysfunc(findw(&ADSLVARS, COUNTRY)) %then %do; COUNTRY %end;
           %if %sysfunc(findw(&ADSLVARS, ETHNIC)) %then %do; ETHNIC %end;
           &DesignByVars Total_Treatment_Duration %if &TimeScale=Cycles %then &CycleVar; ;
   run;

   data _null_;
      set exuni_date end=_e_;
      retain _MaxDuration_ 0;
      _MaxDuration_ = Max(_MaxDuration_,Total_Treatment_Duration);
      if _e_ then call symput("_MaxDuration_",_MaxDuration_);
   run;
   %let _Rep_Inds = exuni_date;
   %if &_MaxDuration_ < &nWindows %then %let nWindows=&_MaxDuration_;
%end;
%else %do;
   * update exuni dataseet (the input dataset for reports) based on cycle information;
   proc sort data=exuni out=exuni2 nodupkey;
      by &TrtVar &DesignByVars USUBJID &CycleVar;
   run;

   proc summary data=exuni2;
      by &TrtVar &DesignByVars USUBJID;
      id 
      %if %sysfunc(findw(&ADSLVARS, AGE)) %then %do; AGE %end; 
      %if %sysfunc(findw(&ADSLVARS, SEX)) %then %do; SEX %end;
      %if %sysfunc(findw(&ADSLVARS, RACE)) %then %do; RACE %end;
      %if %sysfunc(findw(&ADSLVARS, SITEID)) %then %do; SITEID %end; 
      %if %sysfunc(findw(&ADSLVARS, STUDYID)) %then %do; STUDYID %end;
      %if %sysfunc(findw(&ADSLVARS, COUNTRY)) %then %do; COUNTRY %end;
      %if %sysfunc(findw(&ADSLVARS, ETHNIC)) %then %do; ETHNIC %end;
      &CycleVar;
      output out=exuni_cycle(drop=_TYPE_ rename=(_FREQ_=_CycleNum_));
   run;

   data _null_;
      set exuni_cycle end=_e_;
      retain _MaxCycle_ 0;
      _MaxCycle_ = Max(_MaxCycle_,_CycleNum_);
      if _e_ then call symput("_MaxCycle_",_MaxCycle_);
   run;
   %let _Rep_Inds = exuni_cycle;
%end;

* extract ARM info;
proc sort data=exuni(keep=&TrtVar) out=Trt nodupkey;
   by &TrtVar;
run;

data _null_;
   set Trt end=_e_;
   name = "_armLabel_"||trim(left(_n_));
   call symput(name,&TrtVar);
   if _e_ then call symput("_nArm_",_n_);
   * set temperary var name for ARM to avoid SAS variable naming confliction when preparing report data sets;
   name = "_arm_" || trim(left(_n_));
   call symput(name,"ARM"||trim(left(_n_)));
run;

* 2. summarize ex dataset;
%if ^%length(&DesignByVars) %then %do;
   %EX_Report(&_Rep_Inds);
   %let _finalTable = _finalTable;
   %let durtable = table;
%end;
%else %do;
   proc sort data=&_Rep_Inds out=ex_dbv;
      by &DesignByVars;
   run;

   proc sort data=ex_dbv(keep=&DesignByVars) out=ex_short nodupkey;
      by &DesignByVars;
   run;

   data ex_short;
      set ex_short end=_e_;
      _db_level = _n_;
      if _e_ then call symput('ndb_level',_n_);
   run;

   data %do i=1 %to &ndb_level; db_Level_&i %end;;
      merge ex_dbv ex_short;
      by &DesignByVars;
      %do i = 1 %to &ndb_level;
         if _db_level = &i then output db_Level_&i; 
      %end;
   run;

   %do _i_ = 1 %to &ndb_level;
      %put Current working dataset: db_Level_&_i_;
      %EX_Report(db_Level_&_i_);
      %*goto exit;
      data _null_;
         set _finaltable end=_e_;
         if _e_ then call symput("n_row",_n_);
      run;
      data dbVars;
         set ex_short;
         if _n_ = &_i_;
         %do i = 1 %to &n_row;
            output;
         %end;
      run; 

      %if &_i_ = 1 %then %do;
         data _finalTable_all; 
            merge _finaltable dbVars;
         run;
         
         data _durtable_all;
            set table;
         run;
      %end;
      %else %do;
         data _finaltable;
            merge _finaltable dbVars;
         run;
         data _finalTable_all; 
            set _finalTable_all _finaltable;
         run;
         
         proc append base=_durtable_all data=table;
         run;
      %end;
      %removeData(_finalTable);
   %end;
   %let _finalTable = _finalTable_all;
   %let durtable = _durtable_all;
%end;
%if &syserr > 4 %then %goto exit;

* add Exposure 5 Analysis Panel;
%expoAnalyses_5;

* display table;
proc format;
   picture pct low-high='09)'( prefix='(');
run;

%let domain = Exposure;
%let trtedps = %nrbquote(%LocaleGet(Treatmentexposuredurationpersubject_Text));
%let trswtdw = %nrbquote(%LocaleGet(Subjectswithtotaldurationwithin_Text));
%let trtnocatedpssp = %nrbquote(%LocaleGet(TotalNumCyclesAndTrtExpDurPerSubjSafetyPop_Text));
%if &TimeScale ne Cycles %then %do;
   %let title = &trtedps;
   %if %index(&TimeWindowMethod,Use) %then %do;
      %let subtitle = &trswtdw &TimeWindows;
   %end;
%end;
%else %do;
   %let title = &trtnocatedpssp;
%end;
%let title4=%nrbquote(%LocaleGet(PlannedARMtoActualTreatmentComparison_Text));
%let title5=%nrbquote(%LocaleGet(DoseChangesDuringtheStudy_Text));
%let ARMLabel=&TrtARM;
%CheckLabel(final_exposureD,ArmLabel);

%let trActual = %nrbquote(%LocaleGet(Actual_Text));
%let trActualCount = %nrbquote(%LocaleGet(ActualCount_Text));
%let trActualDose = %nrbquote(%LocaleGet(ActualDose_Text));
%let trChangedDose = %nrbquote(%LocaleGet(ChangedDose_Text));
%let trDayofStudy = %nrbquote(%LocaleGet(DayofStudy_Text));
%let trexpsum = %nrbquote(%LocaleGet(ExposureSummary_Text));
%let trNewDose = %nrbquote(%LocaleGet(NewDose_Text));
%let trOriginalDose = %nrbquote(%LocaleGet(OriginalDose_Text));
%let trReason = %nrbquote(%LocaleGet(Reason_Text));
%let trSubjectCount = %nrbquote(%LocaleGet(SubjectCount_Text));
%let trSubjectID = %nrbquote(%LocaleGet(SubjectID_Text));
%let trCounts = %nrbquote(%LocaleGet(Counts_Text));
%let trPercents = %nrbquote(%LocaleGet(Percents_Text));
%let trDoses = %nrbquote(%LocaleGet(Doses_Text));

%if &Output_RTF = Yes %then %do;
   ods rtf file="&OutPath.\&OutReport..rtf" bodytitle;
   PROC REPORT DATA=WORK.&_finaltable LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title";
      column label_rtf ("&TrtLabel" %do i=1 %to &_nArm_; &&_arm_&i %end;);

      DEFINE  label_rtf / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data;
      %do i=1 %to &_nArm_; 
         DEFINE  &&_arm_&i / display FORMAT= $30. WIDTH=20 SPACING=2 CENTER "&&_armLabel_&i" ;
      %end;
   %if %length(&DesignByVars) %then %do;
      by &DesignByVars;
   %end;
      title height=17pt "&trexpsum";
      title2 j=center height=11pt color=black "&title";
      %if %index(&TimeWindowMethod,Use) and &TimeScale ne Cycles %then %do;
         title3 j=center height=10pt color=black "&subtitle";
      %end;
      ods proclabel = "&domain"; 
   RUN;
   
   * Output reports for analysis 4: Planned ARM to Actual Treatment Comparison;
   proc report DATA=Work.final_exposureD LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title4";;
      column &TrtARM arm_count actual actual_dose actual_count;
      DEFINE  &TrtARM / display FORMAT= $60. WIDTH=70 SPACING=2 LEFT "&ARMLabel" order=data;
      DEFINE  arm_count / display FORMAT= $9. WIDTH=20 SPACING=2 RIGHT "&trSubjectCount";
      DEFINE  actual / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trActual" ;
   *  DEFINE  change_date / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "Change Date" ;
      DEFINE  actual_dose / display FORMAT= $20. WIDTH=9 SPACING=2 CENTER "&trActualDose" ;
      DEFINE  actual_count / display FORMAT= BEST9. WIDTH=20 SPACING=2 RIGHT "&trActualCount" ;
      %if %length(&DesignByVars) %then %do;
            by &DesignByVars;
      %end;
      title height=17pt "&title4";
   *  title2 j=center height=11pt color=black "&title4";
      ods proclabel = "&domain"; 
   RUN;
   * Output reports for analysis 5:dose changes during the study;
   %if %eval(&changes.) ne 0 %then %do;
      proc report DATA=Work.final_exposureE LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title5";;
         column usubjid old_dose new_dose change_date studydays exadj;
         DEFINE  usubjid / display FORMAT= $30. WIDTH=70 SPACING=2 LEFT "&trSubjectID" order=data;
         %if &oldDoseLen >20 %then %do;
            DEFINE  old_dose / display FORMAT= $30. WIDTH=30 SPACING=2 CENTER "&trOriginalDose";
         %end;
         %else %do;
            DEFINE  old_dose / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trOriginalDose";
         %end;
         %if &newDoseLen > 20 %then %do;
            DEFINE  new_dose / display FORMAT= $30. WIDTH=30 SPACING=2 CENTER "&trChangedDose" ;
         %end;
         %else %do;
            DEFINE  new_dose / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trChangedDose" ;
         %end;
      *  DEFINE  change_date / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "Change Date" ;
         DEFINE  studydays / display FORMAT= BEST9. WIDTH=9 SPACING=2 RIGHT "&trDayofStudy" ;
         DEFINE  exadj / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trReason" ;
         %if %length(&DesignByVars) %then %do;
               by &DesignByVars;
         %end;
         title height=17pt "&title5";
      *  title2 j=center height=11pt color=black "&title5";
         ods proclabel = "&domain"; 
      RUN;
   %end;
   ods rtf close;
   %if &syserr > 4 %then %goto exit;
%end;

%if &Output_PDF = Yes %then %do;
   ods pdf file="&OutPath.\&OutReport..pdf";
   PROC REPORT DATA=WORK.&_finaltable LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title";
      column label_pdf  ("&TrtLabel" %do i=1 %to &_nArm_; &&_arm_&i %end;);
      DEFINE  label_pdf / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data;
      %do i=1 %to &_nArm_; 
         DEFINE  &&_arm_&i / display FORMAT= $30. WIDTH=20 SPACING=2 CENTER "&&_armLabel_&i" ;
      %end;
   %if %length(&DesignByVars) %then %do;
      by &DesignByVars;
   %end;
      title height=17pt "&trexpsum";
      title2 j=center height=11pt color=black "&title";
      ods proclabel = "&domain"; 
   RUN;
   * Output reports for analysis 4: Planned ARM to Actual Treatment Comparison;
   proc report DATA=Work.final_exposureD LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title4";;
      column &TrtARM arm_count actual actual_dose actual_count;
      DEFINE  &TrtARM / display FORMAT= $60. WIDTH=70 SPACING=2 LEFT "&ARMLabel" order=data;
      DEFINE  arm_count / display FORMAT= $9. WIDTH=20 SPACING=2 RIGHT "&trSubjectCount";
      DEFINE  actual / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trActual" ;
   *  DEFINE  change_date / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "Change Date" ;
      DEFINE  actual_dose / display FORMAT= $20. WIDTH=9 SPACING=2 CENTER "&trActualDose" ;
      DEFINE  actual_count / display FORMAT= BEST9. WIDTH=20 SPACING=2 RIGHT "&trActualCount" ;
      %if %length(&DesignByVars) %then %do;
         by &DesignByVars;
      %end;
      title height=17pt "&title4";
   *  title2 j=center height=11pt color=black "&title4";
      ods proclabel = "&domain"; 
   RUN;
   * Output reports for analysis 5:dose changes during the study;
   %if %eval(&changes.) ne 0 %then %do;
      proc report DATA=Work.final_exposureE LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title5";;
         column usubjid old_dose new_dose change_date studydays exadj;
         DEFINE  usubjid / display FORMAT= $30. WIDTH=70 SPACING=2 LEFT "&trSubjectID" order=data;
         %if &oldDoseLen >20 %then %do;
            DEFINE  old_dose / display FORMAT= $30. WIDTH=30 SPACING=2 CENTER "&trOriginalDose";
         %end;
         %else %do;
            DEFINE  old_dose / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trOriginalDose";
         %end;
         %if &newDoseLen > 20 %then %do;
            DEFINE  new_dose / display FORMAT= $30. WIDTH=30 SPACING=2 CENTER "&trChangedDose" ;
         %end;
         %else %do;
            DEFINE  new_dose / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trNewDose" ;
         %end;
      *  DEFINE  change_date / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "Change Date" ;
         DEFINE  studydays / display FORMAT= BEST9. WIDTH=9 SPACING=2 RIGHT "&trDayofStudy" ;
         DEFINE  exadj / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trReason" ;
         %if %length(&DesignByVars) %then %do;
            by &DesignByVars;
         %end;
         title height=17pt "&title5";
      *  title2 j=center height=11pt color=black "&title5";
         ods proclabel = "&domain"; 
      RUN;
   %end;
   ods pdf close;
   %if &syserr > 4 %then %goto exit;
%end;

%if &Output_HTML = Yes %then %do;
   %PrepHTML(
      ExposureSummaryReport_Body, 
      ExposureSummaryReport_Frame, 
      ExposureSummaryReport_Contents, 
      &ExposureSummary_OutReport._body.html, 
      &ExposureSummary_OutReport._frame.html, 
      &ExposureSummary_OutReport._contents.html
   );
   ods html 
      body = "&ExposureSummaryReport_Body" (url="&bodyfile")
      frame = "&ExposureSummaryReport_Frame"  (url="&framefile")
      contents = "&ExposureSummaryReport_Contents" (url="&contentsfile")
   ;
   PROC REPORT DATA=WORK.&_finaltable LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title";
      column label_html ("&TrtLabel" %do i=1 %to &_nArm_; &&_arm_&i %end;);
      DEFINE  label_html / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data;
      %do i=1 %to &_nArm_; 
         DEFINE  &&_arm_&i / display FORMAT= $30. WIDTH=20 SPACING=2 CENTER "&&_armLabel_&i" ;
      %end;
   %if %length(&DesignByVars) %then %do;
      by &DesignByVars;
   %end;
      title height=17pt "&trexpsum";
      title2 j=center height=11pt color=black "&title";
      ods proclabel = "&domain"; 
   RUN;
   * Output reports for analysis 4: Planned ARM to Actual Treatment Comparison;
   proc report DATA=Work.final_exposureD LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title4";;
      column &TrtARM arm_count actual actual_dose actual_count;
      DEFINE  &TrtARM / display FORMAT= $60. WIDTH=70 SPACING=2 LEFT "&ARMLabel" order=data;
      DEFINE  arm_count / display FORMAT= $9. WIDTH=20 SPACING=2 RIGHT "&trSubjectCount";
      DEFINE  actual / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trActual" ;
   *  DEFINE  change_date / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "Change Date" ;
      DEFINE  actual_dose / display FORMAT= $20. WIDTH=9 SPACING=2 CENTER "&trActualDose" ;
      DEFINE  actual_count / display FORMAT= BEST9. WIDTH=20 SPACING=2 RIGHT "&trActualCount" ;
      %if %length(&DesignByVars) %then %do;
         by &DesignByVars;
      %end;
      title height=17pt "&title4";
   *  title2 j=center height=11pt color=black "&title4";
      ods proclabel = "&domain"; 
   RUN;
   * Output reports for analysis 5:dose changes during the study;
   %if %eval(&changes.) ne 0 %then %do;
      proc report DATA=Work.final_exposureE LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title5";;
         column usubjid old_dose new_dose change_date studydays exadj;
         DEFINE  usubjid / display FORMAT= $20. WIDTH=70 SPACING=2 LEFT "&trSubjectID" order=data;
         %if &oldDoseLen >20 %then %do;
            DEFINE  old_dose / display FORMAT= $30. WIDTH=30 SPACING=2 CENTER "&trOriginalDose";
         %end;
         %else %do;
            DEFINE  old_dose / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trOriginalDose";
         %end;
         %if &newDoseLen > 20 %then %do;
            DEFINE  new_dose / display FORMAT= $30. WIDTH=30 SPACING=2 CENTER "&trChangedDose" ;
         %end;
         %else %do;
            DEFINE  new_dose / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trNewDose" ;
         %end;
      *  DEFINE  change_date / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "Change Date" ;
         DEFINE  studydays / display FORMAT= BEST9. WIDTH=9 SPACING=2 RIGHT "&trDayofStudy" ;
         DEFINE  exadj / display FORMAT= $20. WIDTH=20 SPACING=2 CENTER "&trReason" ;
         %if %length(&DesignByVars) %then %do;
            by &DesignByVars;
         %end;
         title height=17pt "&title5";
      *  title2 j=center height=11pt color=black "&title5";
         ods proclabel = "&domain"; 
      RUN;
   %end;
   ods html close;
   %if &syserr > 4 %then %goto exit;
%end;


   
/***********************************************************************/
* section of JMP summary graph;

%let trnosot = %nrbquote(%LocaleGet(NumberofSubjectsRemainingonStudyMedication_Text));
%let trDnosot = %nrbquote(%LocaleGet(DisplayNumberofSubjectsRemainingonStudyMedication_Text));
%let trposot = %nrbquote(%LocaleGet(DisplayPercentofSubjectsRemainingonStudyMedication_Text));
%let trstudydays = %nrbquote(%LocaleGet(StudyDay_Text));

data OutLib.&OutDataName; 
   set exm;
   label nSubjects = "&trnosot";
   label Study_Duration = "&trstudydays";
run;
data trtcp;
   set trtcp;
   if study_duration=1;
run;

data _null_;
   set trtcp;
   call symput ("trtPCount"||compress(_n_),trim(left(nSubjects)));
   call symput ("trtPLevel"||compress(_n_),trim(left(&TrtVar)));
   call symput ("trtPObs",trim(left(_n_)));
run;
*calculate the percentage of subjects on each day;
data OutLib.&OutDataName;
   set OutLib.&OutDataName;
   %do i=1 %to &trtPObs;
      if &TrtVar="&&trtPLevel&i" then do;
         percSub=round((nSubjects/&&trtPCount&i)*100,0.1);
         label percSub="&trPercentOfSubjsRemStudyMed";
      end;
   %end;
   label nSubjects = "&trnosot";
   label Study_Duration = "&trstudydays";
run;
data OutLib.&OutDataName2; 
   set excount;
   label Study_Duration = "&trstudydays";
   drop count;
run;

%let trtdoe = %nrbquote(%LocaleGet(TotalDurationofExposure_Text));
%let trweeks = %nrbquote(%LocaleGet(Weeks_Text));
%let trdays = %nrbquote(%LocaleGet(Days_Text));
%let trcycles = %nrbquote(%LocaleGet(Cycles_Text));

data OutLib.&OutDataName3;
   set &durtable;
   %if %Upcase(&TimeScale) = WEEKS %then %do;
      %let DurLabel = %nrbquote(&trtdoe (&trweeks));  
      label Total_Treatment_Duration="&DurLabel";  
   %end;
   %else %if %Upcase(&TimeScale) = DAYS %then %do;
      %let DurLabel = %nrbquote(&trtdoe (&trdays));
      label Total_Treatment_Duration="&DurLabel";
   %end;   
   %else %do;
      %let DurLabel = %nrbquote(&trtdoe (&trcycles));
      label _CycleNum_="&DurLabel";
   %end;     
   label Total_Treatment_Duration="&DurLabel";
   drop gotoutput domain;
run;

%DMFlags;
%DMLabels(OutLib.&OutDataName3);
%CheckStudyLevels;
/*
proc sort data=ex out=ex;
   by &TrtVar &DesignByVars %if &_doseFlag %then _EXDOSEG; USUBJID EXSTDTC;
run;

data Outlib.&outDataName2; ***** need to clean up the data set especially for (starting) date columns;
   set ex;
   by &TrtVar &DesignByVars %if &_doseFlag %then _EXDOSEG; USUBJID;
   retain Study_Duration;
   if last.USUBJID then do;
      Study_Duration = input(Ref_End_Date,is8601da.) - input(Ref_Start_Date,is8601da.) + 1;
   end;
   if last.USUBJID;
   EXSTDTC = input(Ref_Start_Date,is8601da.);
   *label Study_Duration = "Time Since Receiving Treatment";
   label Study_Duration = "Time Since Randomization (Days)";
run;

* 3. Apply Proc Lifetest for KM estimation & convert survival rate to actural counts;
proc lifetest data=Outlib.&outDataName2 
   outsurv=es2(keep=&TrtVar &DesignByVars  %if &_doseFlag %then _EXDOSEG; Study_Duration SURVIVAL)
   method=km noprint;
   by &TrtVar &DesignByVars %if &_doseFlag %then _EXDOSEG;;
   time Study_Duration;
run;


proc summary data=Outlib.&outDataName2;
   by &TrtVar &DesignByVars %if &_doseFlag %then _EXDOSEG;;
   output out=ses(drop=_type_);
run;

data outlib.&OutDataName;
   merge es2 ses;
   by &TrtVar &DesignByVars %if &_doseFlag %then _EXDOSEG;;
   _tmp1_ = round(SURVIVAL*_FREQ_);
   _tmp2_ = lag(_tmp1_);
   %if &_doseFlag %then %do;
      if first._EXDOSEG then nSubjects = _tmp1_;
      else nSubjects = _tmp2_;
   %end;
   %else %do;
      if first.&TrtVar then nSubjects = _tmp1_;
   else nSubjects = _tmp2_;
   %end;
   drop _tmp1_ _tmp2_;
   rename _FREQ_ = Total_Subjects SURVIVAL = Proportion;
   label nSubjects = "Number of Subjects on Treatment"
      SURVIVAL = "Proportion of Total Subjects on Treatment";
run;
*/

%if &_doseFlag %then %do;
   %if &_nDose > &nDoseBin %then %do;
      proc sort data=Outlib.&outDataName2;
         by &TrtVar &DesignByVars _EXDOSEG;
      run;
   %end;
%end;
* end of section of handling summary graph;

%let ArmLabel = &TrtVar;
%CheckLabel(OutLib.&OutDataName,ArmLabel);
%let TimeLabel = Study_Duration;
%CheckLabel(OutLib.&OutDataName,TimeLabel);
%if %length(&DesignByVars) %then %do;
   %do i = 1 %to &_nDBVar;
      %let DBVarLabel_&i = %scan(&DesignByVars,&i);
      %CheckLabel(OutLib.&OutDataName,DBVarLabel_&i);
   %end;
%end;
%if &_doseFlag %then %do;
   %let DoseLabel = _EXDOSEG;
   %CheckLabel(OutLib.&OutDataName,DoseLabel);
%end;
%let CountsLabel = nSubjects;
%CheckLabel(OutLib.&OutDataName,CountsLabel);

%let percSubLabel=percSub;
%CheckLabel(OutLib.&OutDataName,percSubLabel);
%let ARMLabel2=&TrtVar;
%CheckLabel(OutLib.&OutDataName6,ArmLabel2);
/* 
%let RateLabel = Proportion;
%CheckLabel(OutLib.&OutDataName,RateLabel);
*/


* create JSL file;
%PrepOutFile(ExposureSummaryScript,ExposureSummaryScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ExposureSummaryScript;
%let FullAPName=%str(Exposure Summary);

* put JSL statements;
data _null_;
   file "&ExposureSummaryScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
    put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:NoteDomain = ""EX"";";

   %let datalist = ""objRefNS:esd2"";
   put "objRefNS:esd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   
   %if &_doseFlag %then %do;
      put "Column(""&DoseLabel"") << Label(1);";
   %end;
   put "Column(""&TimeLabel"") << Label(1);";
   put "Column(""&CountsLabel"") << Label(1);";
   
   put "objRefNS:ex_sum=Function({},{default local},";
   put "   If(IsEmpty(objRefNS:esd),";
   put "      objRefNS:esd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   put "   );";   
   put " Current Data Table(objRefNS:esd);"; 
   put "   hListBox(";
   put "      objRefNS:gb = Graph Builder(";
   put "        AutoStretching(0), Show Control Panel( 0 ),";
   put "         Variables(";
   put "            X( :Name( ""&TimeLabel"" ) ),";
   put "            Y( :Name( ""&CountsLabel"" ) ),";
   %if %length(&DesignByVars) %then %do;
      %do i = 1 %to &_nDBVar;
         put "            Group Y( :Name( ""&&DBVarLabel_&i"" ) ),";
      %end;
   %end;
   %if &_doseFlag %then %do;
      put "            Wrap( :Name( ""&ArmLabel"") ),";
      put "            Overlay( :Name(""&DoseLabel"") )";
   %end;
   %else %do;
      put "            Overlay(:Name(""&ArmLabel""))";
   %end;
   put "         ),";
   put "         Elements( Points( X, Y, Color, Legend( 1 ) ) ),";
   put '         SendToReport( ';
   put "             Dispatch({},""Graph Builder"",OutlineBox, {Set Title(locale:get(""TotalNumberofDaysofDrug_Text""))}),";
   put '             Dispatch( {}, "Graph Builder", FrameBox, {Marker Size( 3 )} ) ),';
   put "         SendToReport(";
   %if &_doseFlag %then %do;
      %if &_nDoseGroups > 1 %then %do;
         * apply different marker for diferent dose group;
         put "            Dispatch(";
         put "               {},";
         put '               "400",';
         put "               ScaleBox,";
         put "               {Legend Model(";
         put "                  1,";
         %do i = 1 %to %eval(&_nDoseGroups-1);
            put "                  Properties( %eval(&i-1), {Marker( %sysfunc(mod(&i,32)) )} ),";
         %end;
         put "                  Properties( %eval(&i-1), {Marker( %sysfunc(mod(&i,32)) )} )";
         put "               )}";
         put "           ),";
      %end;
   %end;
   /*
   put "           Dispatch(";
   put "               {},";
   put '               "graph title",';
   put "               TextEditBox,";
   %if &_doseFlag %then %do;
      put "               {Set Text( ""Exposure Counts by Dose and Study Days"" )}";
   %end;
   %else %do;
      put "               {Set Text( ""Exposure Counts by Study Days"" )}";
   %end;
   put "            )";
   */
   put "         )";
   put "      );";
   put "   )"; * end of hListBox;
   put ");"; * end of ex_sum function;

   put "objRefNS:ppdd = evalexpr(Function({},{default local},";
   put "   this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   
   put "objRefNS:spdd =  Function({},{},";
   * include ShowSubjects.jsl;
   put "   Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl"");";
   put ");";
   
   put "objRefNS:cpdd =  evalexpr(Function({}, {default local},";
   put "	 this = namespace(expr(objRefNS<<getName)); clusterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""); clusterFunc(this:getParent());));";
   
   put "objRefNS:fpdd =  evalexpr(Function({}, {default local},";
   put "   objRefNS = namespace(expr(objRefNS<<getName));";
   put "   reportRefNS = namespace(objRefNS:getParent());";
   put "   reviewRefNS = namespace(reportRefNS:getParent());";
   put "   createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"", <<newContext);";
   put "   createFilterFunc(reviewRefNS << getName);";
   put "));";

   /********************************************************************************************  
      Analysis 1: Percent of Subjects Left on Study Medication by Day of Study.
      Provides a chart showing the percentage of subjects remaining in the study by treatment arm. 
      Graph type: Graph Builder
   *********************************************************************************************/
   put "objRefNS:ex_perc=Function({},{default local},";
   put "   If(IsEmpty(objRefNS:esd),";
   put "      objRefNS:esp=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   put "   );";
   put "   If(IsEmpty(objRefNS:esp2),";
   put "      objRefNS:esp2=open(""&ClientOutPath.&OutDataName5..sas7bdat"",invisible);";
   put "   );";
   put "   If(IsEmpty(objRefNS:esp3),";
   put "      objRefNS:esp3=open(""&ClientOutPath.&OutDataName6..sas7bdat"",invisible);";
   put "   );";
   put "   If(IsEmpty(objRefNS:esp4),";
   put "      objRefNS:esp4=open(""&ClientOutPath.&OutDataName4..sas7bdat"",invisible);";
   put "   );";
   %let datalist = &datalist,""objRefNS:esp3"";
   put "   Current Data Table(objRefNS:esd);";
   put "   VlistBox(";
   put "       VListBox(";
   %if %upcase(&Output_RTF) = YES or %upcase(&Output_PDF) = YES %then %do;
      %let trOpen = %nrbquote(%LocaleGet(Open_Text));
      put "TextBox(""""),";
      %if &Output_RTF = Yes %then %do;
         put "      objRefNS:Button1=ButtonBox(""&trOpen &trexpsum..rtf"",";
         put "          Open(""&OutPath.\&OutReport..rtf"");";
         put "      ),";
         put "      objRefNS:Button1 << Underline Style(1),";   
      %end;
      %if &Output_PDF = Yes %then %do;
         put "      objRefNS:Button2=ButtonBox(""&trOpen &trexpsum..pdf"",";
         put "          Open(""&OutPath.\&OutReport..pdf"");";
         put "      ),";
         put "      objRefNS:Button2 << Underline Style(1),";   
      %end;
      put "TextBox(""""),";      
   %end;   
   put "   HListBox(";
   put "   objRefNS:EXBB = ButtonBox(locale:get(""&trDnosot""),<<UnderlineStyle(0),";
   put "             <<Set Function(";
   put "                Function({this},";
   put "                   Try(gbb1 = report(objRefNS:gb2)[Graph Builder Box(1)]);";
   put "                   objRefNS:CurrBName = this << Get Button Name();";
   put "                   If(objRefNS:CurrBName == locale:get(""&trDnosot""),";
   put "                      Try(gbb1 << Add Variable({:Name(""&CountsLabel""), Role(""Y"")}));";
   put "                      Try(gbb1 << Remove Variable({:Name(""&percSubLabel""), Role(""Y"")}));";
   put "                      this << setButtonName(locale:get(""&trposot""));";
   put "                   ,";
   put "                         Try(gbb1 << Add Variable({:Name(""&percSubLabel""), Role(""Y"")}));";
   put "                         Try(gbb1 << Remove Variable({:Name(""&CountsLabel""), Role(""Y"")}));";
   put "                         this << setButtonName(locale:get(""&trDnosot""))";
   put "                   );";
   put "                );";
   put "             )";
   put "    ),";
   put "    TextBox("" ""),";
   put "   ),";
   put "   HlistBox(";
   put "      objRefNS:gb2 = Graph Builder(";
   put "        AutoStretching(0), Show Control Panel( 0 ),";
   put "         Variables(";
   put "            X( :Name( ""&TimeLabel"" ) ),";
   put "            Y( :Name( ""&percSubLabel"" ) ),";
   %if %length(&DesignByVars) %then %do;
      %do i = 1 %to &_nDBVar;
         put "            Group Y( :Name( ""&&DBVarLabel_&i"" ) ),";
      %end;
   %end;
   %if &_doseFlag %then %do;
      put "            Wrap( :Name( ""&ArmLabel"") ),";
      put "            Overlay(:Name(""&DoseLabel""))";
   %end;
   %else %do;
      put "            Overlay(:Name(""&ArmLabel""))";
   %end;
   put "         ),";
   put "         Elements( Points( X, Y, Color, Legend( 1 ) ) ),";
   put '         SendToReport( ';
   put "             Dispatch({},""Graph Builder"",OutlineBox, {Set Title(locale:get(""TotalNumberofDaysofDrug_Text""))}),";
   put 'Dispatch( {}, "Graph Builder", FrameBox, {Marker Size( 3 )} ) ),';
   
   put "         SendToReport(";
   %if &_doseFlag %then %do;
      %if &_nDoseGroups > 1 %then %do;
         * apply different marker for diferent dose group;
         put "            Dispatch(";
         put "               {},";
         put '               "400",';
         put "               ScaleBox,";
         put "               {Legend Model(";
         put "                  1,";
         %do i = 1 %to %eval(&_nDoseGroups-1);
            put "                  Properties( %eval(&i-1), {Marker( %sysfunc(mod(&i,32)) )} ),";
         %end;
         put "                  Properties( %eval(&i-1), {Marker( %sysfunc(mod(&i,32)) )} )";
         put "               )}";
         put "           ),";
      %end;
   %end;
   put "         )";
   
   put "      ),"; *end of graph builder;
   /*********************************************
         Analysis 2: Distribution of Doses by ARM
   **********************************************/ 
   put "   Current Data Table(objRefNS:esp2);";
   put "    objRefNS:gb3 = Graph Builder(";
   put "        AutoStretching(0), Show Control Panel( 0 ),";
   put " Variables(";
   put "    X( :Name(""&percentLabel"") ),";
   put "    Y( :Name( ""&cumPercLabel"" ) ),";
   put "    Color( :Name(""&maxdoseLabel"") ) ";
/* %do n = 1 %to &num_treatments;
      %if &n=1 %then %do;
         put "    Y( :Name( ""&cumPercLabel"" ) ),";
      %end;
      %else %do;
         put "    Y( :Name( ""&&cumPercLabel&N"" ), Position( 1 ) ),";
      %end;    
   %end;*/
   put " ),";
   put " Elements( Points( X, ";
   put " Y,";
   put " Legend( 5 ), Jitter( 1 ) ) ),";
   put " SendToReport(";
   put "    Dispatch({},""Graph Builder"",OutlineBox, {Set Title(locale:get(""CumulativeDose_Text""))}),";
   put "    Dispatch({},""Graph Title"",TextEditBox,{Set Text( locale:get(""DistributionofDosesbyArm_Text"") )}),";
   put "    Dispatch( {}, ""Y title"", TextEditBox, {Set Text( locale:get(""PercentofSubjects_Text"") )} )";
   put " )";
   put ")),";
   put "),";
   put "       VListBox(";
   put "       HListBox(";

   /******************************************
         Analysis 3: Dose by ARM
   *******************************************/ 
   put "   Current Data Table(objRefNS:esp3);";
   put "    objRefNS:ow = Oneway(";
   put "       Y( :Name(""&trDoses"")),";
   put "       X( :Name(""&ArmLabel2"")),";
   %if %length(&DesignByVars) %then %do;
      put "       By(";
      %do i = 1 %to &_nDBVar;
         %if &i=&_nDBVar %then %do;
            put "            :Name( ""&&DBVarLabel_&i"" )),";
         %end;
         %else %do;
            put "            :Name( ""&&DBVarLabel_&i"" ) ,";
         %end;
      %end;
   %end;
   put "         Means( 1 ),";
   put "         Box Plots( 1 ),";
   put "       Mean Diamonds( 1 ),";
   put "       Grand Mean( 0 ),";
   put "       SendToReport(";
   put "          Dispatch( {},";
   put "             ""Oneway Plot"",";
   put "             FrameBox,";
   put "             {DispatchSeg(";
   put "                Box Plot Seg( 1 ),";
   put "                {Box Type( ""Outlier"" ),  Confidence Diamond( 1 ),Line Color( ""Red"" )}";
   put "                ), DispatchSeg(";
   put "                Box Plot Seg( 2 ),";
   put "                {Box Type( ""Outlier"" ),  Confidence Diamond( 1 ),Line Color( ""Red"" )}";
   put "             )}";
   put "          ),";
   put "	Dispatch( {}, ""Oneway Anova"", OutlineBox, {Close( 1 )} )";
   put "       )";
   put "       ),";
   /******************************************
         Analysis 4: Basic Statistics of ARM
   *******************************************/ 
   put "Current Data Table(objRefNS:esp4);";
   put "column(""&trBasicStatistics"")<<set property(""Value Ordering"",
      {locale:get(""NUMSIGNofObs_Text""),locale:get(""Mean_Text""),locale:get(""Mode_Text""),locale:get(""Median_Text""),
      locale:get(""SD_Text""),locale:get(""Min_Text""),locale:get(""N10thpercentile_Text""),locale:get(""N25thpercentile_Text""),
      locale:get(""N75thpercentile_Text""),locale:get(""N90thpercentile_Text""),locale:get(""Max_Text"")});";
   put "objRefNS:gb4 =Tabulate(";
   put " Change Item Label( Statistics( Sum, ""  "" ) ),";
   put " Show Control Panel( 0 ),";
   put " Add Table(";
   %do I = 1 %to &num_arms.;
      put " Column Table( Analysis Columns(:Name(""&&armlevel&I"")) ),";
   %end;
   put " Row Table( Grouping Columns( :&trBasicStatistics ) )";
   put "),";
   put " SendToReport(";
   put " Dispatch(";
   put "       {},";
   put "       ""Tabulate"",";
   put "       OutlineBox,";
   put "       {Set Title( locale:get(""BasicStatisticsforARM_Text"") )}";
   put "    )  )";
   put "),";
   
   put "      ));";
   put "      );"; /* end of VListbox;*/
   put "      );"; /* end of function ex_perc;*/
   put "InsertInto(objRefNS:DataList,{{""objRefNS:esd"",""objRefNS:esp2"",""objRefNS:esp3"",""objRefNS:esp4""}});";
   put "InsertInto(objRefNS:DataNameList,{{locale:get(""DaysofDrug_Text""),locale:get(""CumulativeDose_Text""),locale:get(""DoseAnalysis_Text""),""&trBasicStatistics""}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""ExposureSummary_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ex_perc()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:gb2,objRefNS:gb3,objRefNS:ow}});";
   
   /*
   put "objRefNS:set_CDT = Function({a},CurrentDataTable(objRefNS:dur));";
   put "objRefNS:set_RSH = Function({},rs = objRefNS:dur << make row state handler(objRefNS:set_CDT));";
   */
   /* new output - distribution of duration to match report */
   put "objRefNS:dur_dist = Function({},{default local},";
   put "   If(IsEmpty(objRefNS:dur),";
   put "      objRefNS:dur=open(""&ClientOutPath.&OutDataName3..sas7bdat"",invisible);";
   put "   );";
   put "   Current Data Table(objRefNS:dur);";
   put "   Column(""&DurLabel"") << Label(1);";
   put "   HlistBox(";
   put "objRefNS:Filter=   objRefNS:dur << Data Filter(<<Conditional(1),";
   put "         Add Filter(";
   put "            Columns(";
   %if &doStudy %then %do;
		put " 		:Name(""&StudyLabel""),";
	%end;
	%if &doAge %then %do;
		put " :Name(""&AgeLabel""),";
	%end;
	%if &doSex %then %do;
		put " :Name(""&SexLabel""),";
	%end;
	%if &doRace %then %do;
		put " :Name(""&RaceLabel""), ";
	%end;
	%if &doSite %then %do;
		put "       :Name(""&SiteLabel""),";
	%end;
	put" :Name(""&TrtLabel""))";	
   put "         ),";
   put "         Mode( Show(1), Include(1) )";
   put "      ),";   
   put "       VListBox(";
   put "          objRefNS:dd = Distribution(";
   put "            Automatic Recalc(1),";
   %let datalist = &datalist,""objRefNS:dur"";
   
   %if %length(&DesignByVars) %then %do;
      put "            By(";
      %do i = 1 %to &_nDBVar;
         put "    :Name(""&&DBVarLabel_&i"")"@@;
         %if &i < &_nDBVar %then %do;
            put " ,";
         %end;
      %end;
      put "       ),";
   %end;
   put "          Continuous Distribution(Column(:Name(""&DurLabel"")), Show Counts(1)),";
   put "          Nominal Distribution(Column(:Name(""&ArmLabel"")), Show Counts(1)),";
   put "         SendToReport(";
   put "            Dispatch( {""&DurLabel""}, ""Quantiles"", OutlineBox, Close( 1 ) )";
   put "          )";
   put "       ),";  /* close distribution */
   put "    )";   /* close vlist */
   put "    );";     /* close hlist */
   put " );"; /* close function */
   put "InsertInto(objRefNS:DataList,{{""objRefNS:dur""}});";
   put "InsertInto(objRefNS:DataNameList,{{locale:get(""DurationDistribution_Text"")}});";   
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""DurationDistribution_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:dur_dist()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:dd}});";
   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReports]\"");";
   put "try(objRefNS:PopulationSet=""&PopulationSet"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 

   /*
   put "jg:setCDT = Function({a},Try(CurrentDataTable(jg:durdt)));";
   put "setCDT = Function({a},print(a));";
   put "rs = jg:durdt << make row state handler( setCDT );
   */
   put "    objRefNS:ActionButtonList= {{Expr(locale:get(""ProfileSubjects_Text"")),Expr(locale:get(""ShowSubjects_Text"")), Expr(locale:get(""ClusterSubjects_Text"")),Expr(locale:get(""CreateSubjectFilter_Text""))}};";
   put "    objRefNS:ActionIconList = {{""/install/lifesciences/documentation/icons/clinical/ProfileSubjects.gif"", ""/install/lifesciences/documentation/icons/clinical/DataSetUser.gif"", ""/install/lifesciences/documentation/icons/clinical/HierarchicalClustering.gif"", ""/install/lifesciences/documentation/icons/clinical/AddFilter.gif""}};";
   put "    objRefNS:ActionSelectionCountList = {{1, 1, 3, 1}};";
   put "    objRefNS:ActionRequirementFlagList = {{3, 0, 3, 2}};";      
   put "    objRefNS:ActionCodeList= {{objRefNS:ppdd(),objRefNS:spdd(),objRefNS:cpdd(),objRefNS:fpdd}};";
   *Can not call this macro without subsetting on the second dataset (objRefNS:esd2) which was recorded by patient; 
   put "    objRefNS:ActionDataTableList = {{}};";
   put "    insertInto(objRefNS:ActionDataTableList, {{}});";
   put "    insertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:esp3"",""objRefNS:dur""}});";
   put "    insertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:esp3"",""objRefNS:dur""}});";
   put "    insertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:esp3"",""objRefNS:dur""}});";
   put "    insertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:esp3"",""objRefNS:dur""}});";
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ExposureSummaryScriptName, "&OutDataName..sas7bdat,&OutDataName3..sas7bdat,&OutDataName4..sas7bdat,&OutDataName5..sas7bdat,&OutDataName6..sas7bdat", yes);
%if &Output_RTF = Yes %then %do;
   %PackageInsertFile(RTF file from &ProcessName, &OutPath, &OutReport..rtf, , rtf, yes, no);
%end;

%if &Output_PDF = Yes %then %do;
   %PackageInsertFile(PDF file from &ProcessName, &OutPath, &OutReport..pdf, , pdf, yes, no);
%end;

%if &Output_HTML = Yes %then %do;
   %PackageInsertHtml(HTML from &ProcessName, &OutPath, &bodyfile, &framefile, &contentsfile, yes, no);
%end;
%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ExposureSummary;

* run the macro;
%ExposureSummary;



