
/*
JMP Clinical Analytical Process

Name: Findings ANOVA
Authors: Kelci Miclaus and Russ Wolfinger, SAS Institute Inc.

Description: 
	This process screens all findings measurements one at a time by performing a 
	repeated-measures analysis of variance.  Trial epoch, 
	treatment arm, and their interaction are the fixed effects, and a compound symmetry covariance structure
	is assumed within each subject.  A separate model is fit for each measurement.  A volcano plot of the 
	interaction effect and other output enable efficient
	screening of vital sign scores that differ between treatment groups.
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/
*******************************************************************;
*		Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"Findings","CrossoverPeriods"},{"LabNormalization"}}$
$CMP: RSYS.Required Analysis Variables$ $UE: FINDINGS.xxTESTCD and FINDINGS.xxSTRESN$
	$NOTES: Repeated measures model fit for each value of Findings Test based on xxSTRESN measurements.$
$CMP: RSYS.Required Timing Variables$ $UE: LB.LBBLFL or LB.LBDY or (LB.LBDTC and DM.RFSTDTC)$
	$NOTES: For LB.LBBLFL, Value of "Y" categorizes baseline records. For LB.LBDY and LB.LBDTC, Records with Study Day <= 1 categorize baseline. STDY calculations required for using Time Windows.$
$CMP: OSYS$ $UE: FINDINGS.xxPOS$ $NOTES: Position variable will be appended to xxTEST and/or xxTESTCD for analysis.$	
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName InPath AInPath FDomain  FindDTests TrtUse TrtVar 
	IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL 
	BaselineUse DesignByVars SubjectFilter WhereStmt  MERGESUPPDOM  WhereForReports WhereTests BaselineTimeWindow BaselineCalc TimeWindows TimeScale OutPath ServerOutPath MacroPath
	ANOVAModel ClassVars FixedEffects RandomEffects 
	UseXaxisFilter XaxisCutoff XaxisDirection
	LSMeansDiffSet LSMeansControlValues MultipleTestingMethod Alpha NegLog10pCutoff TStats PValues PlotResiduals ListAll 
	NormRef OutData OutDataRes JSLFile
	FindingsANOVABody FindingsANOVAContents FindingsANOVAFrame FindingsANOVAScript
;
%global &FDomain.BaselineANOVABody &FDomain.BaselineANOVAContents &FDomain.BaselineANOVAFrame &FDomain.BaselineANOVAScript FDeltaDrop;
*ProcessBody;

%macro FindingsANOVA;

*initialization and argument checking;
%let ProcessName = FindingsANOVA; 
%global InData ExpInData FullAPName;
%let FullAPName = %str(Findings ANOVA);
%let ListAll = No;

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";

%if &exiterror %then %goto exit;

* add numeric suffix;
%if ^%symexist(_nThePro_) %then %let _nThePro_ = 001;
%let suffix = %lowcase(&FDomain)a_&_nThePro_;
%let TrtVar = &TrtName;
%let TestVar = &FDomain.TESTCD;
%let ResponseVar = &FDomain.STRESN;
%let TEAEText =;

%if %index(%upcase(&LSMeansDiffSet),TREATMENT) %then %do;
	%let LSMeansDiffSet = All Pairwise Differences;
%end;

%include "&MacroPath./MixedModelMacros.sas" / nosource;
%let WhereForReports=%nrbquote(&WhereStmt);
%let AnnoInData= ;
%let AnnoMergeVars= ;
%let AnnoAccession= ;
%let AnnoLabel= ;
%let AnnoGeneSymbol= ;
%let AnnoDescription= ;
%let AnnoGeneId= ;
%let AnnoSnpId= ;
%let AnnoOrganism= ;
%let BaseMergeVars2= ;
%let ToKeepVars= ;
%let LSMeanStdMethod= STD;
%let VolcanoUnifiedY= yes;
%let EBShrink= no;
%let Speak= no;
%let ChrVar = ;
%let PosVar = ;
%let AnnoChrVar = ;
%let AnnoPosVar = ;
%let AnnoWhereStmt = ;
%let AnnoOnly = no;
%let TrackFolder = ;
%let TrackFiles = ;
%let StdErrs = no;
%let JournalChrom = no;
%let LabelVar= ;
%let ChrVar= ;
%let PosVar= ;
%let InClassVars= ;
%let AdjPValues = no;
%let MultipleTestingFixedEffects=%nrbquote();
%let LComponents= no;
%let OutDataRes= ;
%let AnnoLabResid= ;
%let FilterType= None;
%let ResidualFalsePositive= 0.05;
%let ResidualCutoff= ;
%let MaxLoop= 3;
%let LSMeanStdMethod= STD;
%let ClusterLSM= yes;
%let VolcanoUnifiedY= no;
%let Speak= no;
%let ChunkSize= 1000;
%let OutDataComp=%nrbquote();
%let MMEIData=%nrbquote();
%let AnnoChrVar= ;
%let AnnoPosVar= ;
%let AnnoWhereStmt=%nrbquote();
%let AnnoOnly= no;
%let AnnoAccession= ;
%let AnnoGeneSymbol= ;
%let AnnoDescription= ;
%let AnnoGeneId= ;
%let AnnoSnpId= ;
%let AnnoKeepVars= ;
%let AnnoOrganism= ;
%let Anno2InData=%nrbquote();
%let Anno2MergeVars= ;
%let Anno2Label= ;
%let Anno2ChrVar= ;
%let Anno2PosVar= ;
%let Anno2WhereStmt= ;
%let Anno2Only= no;
%let UDiffs = no;
%let ProcMixedOptions = ;
%let TrackPath=%nrbquote();
%let TrackAdd= 0;
%let OrigInPath = &InPath;
%let ClientDownload = Yes;
%if %length(%trim(&DesignByVars)) %then %do;
	%let SplitResults = yes;
%end;
%else %do;
	%let SplitResults = no;
%end;

%let BaselineTimeWindow0 = &BaselineTimeWindow;
%let TimeWindows0 = &TimeWindows;

%include "&MacroPath./FindingsMacros.sas";

%CheckBLTWsyntax(&BaselineTimeWindow);
%if &exiterror %then %goto exit;

libname InLib "&InPath";

%CheckFindingsData(&FDomain);

%if &exiterror %then %goto exit;

%put FDomain = &FDomain;

%if ^%length(&&in_&FDomain) %then %do;
	%put [MS_S_0095];
	%put ERROR: Neither the &FDomain nor the ad&FDomain data set exist in the specified SDTM or ADAM folder.;
	%let exiterror=1;
%end;

%if %sysfunc(indexw(&&&FDomain.VARS, %upcase(&FDomain)STRESN, ' ')) = 0 %then %do;
	%put [MS_S_0096];
	%put ERROR: Analysis cannot be performed due to missing required variable %upcase(&FDomain)STRESN.;
	%let exiterror=1;
	%goto exit;
%end;
		
/* kemicl check for date variables */
%if &stdy_ = 0 and (&fdtc_ = 0 or ^%sysfunc(findw(%upcase(&DMVARS), RFSTDTC))) %then %do;
	%put ERROR: Analysis cannot be performed due to missing required date variables to calculate study day.;
	%let exiterror=1;
	%goto exit;
%end;

%if &fpool %then %do;	
	%let IDVar = POOLID;
%end;
%else %do;
	%let IDVar = USUBJID;
%end;
%put IDVar = &IDVar;

%if &fpos_ %then %let TestPosVar = &FDomain.POS;
%else %let TestPosVar = ;
%let TimeVar = &TimeScale;
%let BaseMergeVars = Domain &IDVar TimeWindowNum &TestVar &TestPosVar &TimeVar;

* determine min and max time values;
%if &fpool & %length(&InPath) and %sysfunc(fileexist(&InPath./pooldef.sas7bdat)) %then %do;
	proc sort data=sdtm.pooldef out=pooldef;
		by USUBJID;
	run;
	
	proc sort data=&in_dm out=dm;
		by USUBJID;
	run;

	data dm;
		merge dm pooldef;
		by USUBJID;
		keep POOLID rfstdtc;
	run;

	* use the demographics from the first subject in each pool;
	proc sort data=dm nodupkey;
		by POOLID;
	run;
	
	data &FDomain;
		merge &FDomain dm;
		by &IDVar;
	run;
%end;

data &FDomain;
	set &FDomain;
	where not missing(&FDomain.STRESN) ;
	if missing(&TestVar) then delete;
	if missing(ENDY) then ENDY = STDY;
	%if %Upcase(&TimeVar) = WEEKS %then %do;
		STWK = ceil(STDY/7);
		ENWK = ceil(ENDY/7);
		%let STvar = STWK;
		%let ENvar = ENWK;
	%end;
	%else %do;	
		%let STvar = STDY;
		%let ENvar = ENDY;
	%end;
	* compute standardized lab scores;
	%if %Upcase(&FDomain) = LB %then %do;
		%LabsNormalize(&ResponseVar,&NormRef,Log2); 
	%end;
run;

%NObsVars(&FDomain);
%if &nobs = 0 %then %do;
	%put [MS_S_0097];
	%put ERROR: There are no observations in the analysis data set after deleting missing values of &FDomain.STRESN and &TestVar.;
	%let exiterror=1;
	%goto exit;
%end;

%if &NTrtPeriods > 1 %then %do;
	%AssignCOTreatment(&FDomain,&FDomain,&Fdomain.DTC);
	%let COVars = &TrtName Period;
%end;
%else %do;
	%let COVars = ;
%end;

proc means data=&FDomain noprint;
	var &STvar;
	output out=mm min=MinTime max=MaxTime;
run;

data _null_;
	set mm;
	call symput('MinTime',compress(MinTime));
	call symput('MaxTime',compress(MaxTime));
run;
%put MinTime = &MinTime;
%put MaxTime = &MaxTime;


/* create a baseline time window if BLFL doesnt exist;*/
%if ^&blflag %then %do;
	%if ^%length(&BaselineTimeWindow) %then %do;
		%if &MinTime > 1 %then %let i1 = &MinTime;
		%else %let i1 = 1;
		%let i2 = %eval(&i1+1);
		%let BaselineTimeWindow = %str([&MinTime,&i1]);
		%if %length(&TimeWindows) and ^%index(&TimeWindows,%str(])) %then %do;
			%put [MS_S_0098];
			%put ERROR: You must use brackets to specify the Trial Time Windows, e.g. [2,5];
			%let exiterror = 1;
			%goto exit;
		%end;
	%end;
	%else %if %length(&TimeWindows) %then %do;
		%let ibw = %index(&BaselineTimeWindow,%str(]);
		%let itw = %index(&TimeWindows,%str(]);
		%if (&itw and ^&ibw) or (^&itw and &ibw) %then %do;
			%put [MS_S_0099];
			%put ERROR: Baseline Time Window and the Trial Time Windows must be specified with similar syntax.;
			%let exiterror = 1;
			%goto exit;
		%end;	
	%end;
%end;

/* time windows */
%if ^%length(&TimeWindows) %then %do;
	%if %length(&BaselineTimeWindow) %then %do;
	/* if Time Windows not specified then make  window to cover the rest of the trial that starts where BLTW left off; */
		%let ib = %index(&BaselineTimeWindow,%str(]));
		%if &ib = 0 %then %do;
			%put [MS_S_0100];
			%put ERROR: You must use brackets to specify the Baseline Time Window, e.g. [-7,1];
			%let exiterror = 1;
			%goto exit;
		%end;
		%let ic = %index(&BaselineTimeWindow,%str(,));
		%if &ic = 0 %then %do;
			%let ic = %index(&BaselineTimeWindow,%str([)); 
		%end;
		%let i2 = %eval(%substr(&BaselineTimeWindow,%eval(&ic+1),%eval(&ib-&ic-1))+1);
		%let TimeWindows = %str([&i2,&MaxTime]);
	%end;
	%else %do;
		%global tw1;
		%let TimeVar = &BlFlagVar;
		%let tw1 =Y;
		%let BaseMergeVars= &IDVar &TestVar &TimeVar &TestPosVar ;
	%end;
%end;
		
%let TimeWindows = %str(&BaselineTimeWindow.&TimeWindows);

%put TimeWindows = &TimeWindows;

%if %length(&TimeWindows) %then %do;
	%if &blflag %then %do;
		%AddTimeWindows(domain=&FDomain,indat=&FDomain,outdat=&FDomain,twb=&TimeWindows,tvar=&TimeVar,findingscall=1,blflag=1);
	%end;
	%else %do;
		%AddTimeWindows(domain=&FDomain,indat=&FDomain,outdat=&FDomain,twb=&TimeWindows,tvar=&TimeVar,findingscall=1);		
	%end;
%end;
%if &exiterror %then %goto exit;

/*
%if &ntw <= 2 and (%upcase(&BaselineUse) = COVARIATE) and %index(&ANOVAModel,Time) %then %do;
	%put ERROR: For models with a baseline covariate and a time effect, you must specify at least two time windows.;
	%let exiterror = 1;
	%goto exit;
%end;
*/

%if %length(&TimeWindows0) %then %do;
	%let ByVars = &TestVar &TimeVar;
%end;
%else %do;
	%let ByVars = &TestVar;
%end;

/*
%if (%upcase(&BaselineUse) = COVARIATE) %then %do;
	%let BaseByVars = &TestVar;
	%let BaseSubjectVar = USUBJNUM;
	%if %length(&TimeWindows) and &blflag %then %do;
		%let BaselineVar = &BlFlagVar;
		%let BaselineValue = Y;
		%let BaseMergeVars = &BaseMergeVars &BlFlagVar;
	%end;
	%else %do;
		%let BaselineVar = &TimeVar;
		%let BaselineValue = &tw1;
	%end;
	%let BaseCov = BaseCov;
	%let BaselineTimeVar = &FDomain.DY;
%end;
%else %do;
	%let BaseSubjectVar = ;
	%let BaselineVar = ;
	%let BaselineValue = ;
	%let BaseCov = ;
	%let BaselineTimeVar = ;
%end;

%put BaselineVar = &BaselineVar;
%put BaselineValue = &BaselineValue;
%put TimeVar = &TimeVar;
*/
%let BaseSubjectVar = ;
%let BaselineVar = ;
%let BaselineValue = ;
%let BaselineTimeVar = ;
%let BaseCov = Baseline;

data &FDomain;
	set &FDomain;
	%if %length(&TimeWindows) and &blflag and ^%length(&BaselineTimeWindow) %then %do;
		if &BlFlagVar = 'Y' then &TimeVar = -999;
	%end;
	keep &ByVars &BaseMergeVars &ResponseVar &FDomain.DY &COVars &BlFlagVar;
run;

* extract numeric covariates from &FixedEffects and &RandomEffects;
%let KeepVars = ;
%if %length(&FixedEffects) %then %do;
	%let i=1;
	%do %while(%length(%scan(&FixedEffects,&i,' *|()')));
		%let KeepVars = &KeepVars %scan(&FixedEffects,&i,' *|()');
		%let i = %eval(&i+1);
	%end;
	%put KeepVars including Fixed Effects = &KeepVars;
%end;

%if %length(&RandomEffects) %then %do;
	%let i=1;
	%do %while(%length(%scan(&RandomEffects,&i,' *|()')));
		%let KeepVars = &KeepVars %scan(&RandomEffects,&i,' *|()');
		%let i = %eval(&i+1);
	%end;
	%put KeepVars including Random Effects = &KeepVars;
%end;

* merge in modeling variables from adsl;
data asub;
	set &in_adsl;
	keep USUBJID &DesignByVars &ClassVars &KeepVars &TrtVar;
run;

* check for POOLID;
%if %length(&InPath) & &fpool and %sysfunc(fileexist(&InPath./pooldef.sas7bdat)) %then %do;
	proc sort data=sdtm.pooldef out=pooldef;
		by USUBJID;
	run;
	
	proc sort data=asub;
		by USUBJID;
	run;

	data asub;
		merge asub pooldef;
		by USUBJID;
	run;

	* use the demographics from the first subject in each pool;
	proc sort data=asub nodupkey;
		by POOLID;
	run;
%end;

proc sql;
	create table &FDomain as
		select *
		from &FDomain d1, asub d2
		where d1.&IDVar=d2.&IDVar;
quit;

%nobsvars(&FDomain);
%if &nobs = 0 %then %do;
	%put [MS_S_0101];
	%put ERROR:  The merge of &in_lb and &in_adsl data sets resulted in 0 observations.  No subject identifiers match between the two data sets found in the defined Study folders.;
	%put ERROR:  Please check that the data sets in the Study folders are accurate.;
	%let exiterror = 1;
	%goto exit;
%end;

proc sort data=&FDomain;
	by &TestVar &TestPosVar &DesignByVars &IDVar &FDomain.DY;
run;

* create numeric subject counter for speed;
data &FDomain;
	set &FDomain;
	Response = &ResponseVar;
	retain USUBJNUM 0;
	if &IDVar ne lag(&IDVar) then USUBJNUM = USUBJNUM+1;
run;

* delete values with only one measurement per subject;
proc freq data=&FDomain noprint;
	tables &TestVar*&IDVar / out=sa;
	where not missing(Response);
run;

proc sort data=sa;
	by &TestVar &IDVar;
run;

proc means data=sa noprint;
	var COUNT;
	by &TestVar ;
	output out=sm mean=_mc;
run;

data sm;
	set sm;
	if _mc <= 1 then delete;
	keep &TestVar  _mc;
run;

data OutLib.&FDomain.d;
	merge &FDomain sm;
	by &TestVar;
	Domain = "&FDomain";
	if missing(_mc) then delete;
	drop _mc;
run;

%nobsvars(OutLib.&FDomain.d);
%if &nobs = 0 %then %do;
	%put [MS_S_0102];
	%put ERROR:  There is not enough valid data to complete analysis.;
	%let exiterror = 1;
	%goto exit;
%end;

* split out baseline;
%let GroupSortVars = &ByVars;
%let MeasurementValues = ;
%SplitBaselineTrial(&Fdomain, dset=OutLib.&FDomain.d, TimeV = &TimeVar, doblchg = 1);
/*
data OutLib._std;
	set _std;
run;
*/

data OutLib.&FDomain.d;
	set _std;
run;

%put TestVar = &TestVar;
%if &NTrtPeriods > 1 %then %do;
	%put Crossover TrtVar = &TrtName;
%end;
%else %do;
	%put TrtVar = &TrtVar;
%end;
%put TimeVar = &TimeVar;

%let InData = &OutPath%lowcase(&FDomain)d.sas7bdat;
%let AlreadyStacked = 1;
%let ResponseVar0 = &ResponseVar;
%let TestVarLabel = &TestVar;
%CheckLabel(OutLib.&FDomain.d,ResponseVar);
%CheckLabel(OutLib.&FDomain.d,TestVarLabel);

* determine number of treatment levels;
%include "&MacroPath./TreatmentLevels.sas" / nosource;

* create ExpInData from adsl;
data OutLib.adsl_&suffix;
	set OutLib.adsl_&suffix;
	length ColumnName $ 32;
	ColumnName = "S_" || trim(left(_n_));
	* delete subjects with missing by variables;
	%let i = 1;
	%do %while(%length(%scan(&DesignByVars,&i)));
		if missing(%scan(&DesignByVars,&i)) then delete;
		%let i=%eval(&i+1);
	%end;
	Array = _n_;
	keep ColumnName Array &DesignByVars;
run;

%let ExpInData = &OutPath.adsl_&suffix..sas7bdat;

%GetOrigFileName(InData,OrigInData);

* check name for OutData;
%if %length(&OrigInData) < 29 %then %let OID = &OrigInData;
%else %let OID = %substr(&OrigInData,1,28);

%PrepOutDS(OutData,OutDataName,&OID,_&suffix);
%PrepOutDS(OutDataRes,OutDataResName,&OID,_&suffix.r);
%PrepHTML(FindingsANOVABody, FindingsANOVAFrame, FindingsANOVAContents);

* create PROC MIXED statements;
%if &NTrtPeriods > 1 %then %do;
	%let ClassStmt = %str(class &ClassVars &TrtName Period &FDomain.DY;);
	%let ModelStmt = %str(model Response = &BaseCov &FixedEffects &TrtName Period / ddfm=kr solution;);
	%let LSMeansEffect = &TrtName ;
%end;
%else %do;
	%let ClassStmt = %str(class &ClassVars &TrtVar &FDomain.DY;);
	%let ModelStmt = %str(model Response = &BaseCov &FixedEffects &TrtVar / ddfm=kr solution;);
	%let LSMeansEffect = &TrtVar ;
%end;

%if %length(&RandomEffects) %then %do;
	%let RandomStmt = %str(random &RandomEffects;);
%end;
%else %do;
	%let RandomStmt = ;
%end;

%let RepeatedStmt = %str(repeated &FDomain.DY / type=cs subject=USUBJNUM;);

%if (&LSMeansDiffSet = None) %then %let lsmdiffstring = ;
%else %do;
	%if %index(&LSMeansDiffSet,Control) %then %do;
		%if %length(&LSMeansControlValues) %then %do;
			%let LSMeansControlValues = %AddQuotes(&LSMeansControlValues);
			%let lsmdiffstring = diff=control(&LSMeansControlValues);
		%end;
		%else %let lsmdiffstring = diff=control;
	%end;
	%else %let lsmdiffstring = diff;
%end;
%let LSMeansStmt = lsmeans &LSMeansEffect / &lsmdiffstring %str(;);

%let ProcMixedStatements = %str(&ClassStmt &ModelStmt &RandomStmt &RepeatedStmt &LSMeansStmt);
%put ProcMixedStatements = &ProcMixedStatements;

%CheckInData(_ExpFlag=1,_CheckDesign=0);
%if (&syserr | &exiterror) %then %goto exit;

%MixedParmCheck;
%if (&syserr | &exiterror) %then %goto exit;

%MixedFit(_htmlbody=FindingsANOVABody,_htmlframe=FindingsANOVAFrame,_htmlcontents=FindingsANOVAContents);
%if (&syserr | &exiterror) %then %goto exit;

data &OutData;
	set &OutData;
	drop &_MergeNonByVars;
run;

%MixedCreateJSL;
%MixedPackage;
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(StackedDataFromX_Text)),
	%nrbquote(%LocaleGet(&ProcessName._Label)))), outlib, &FDomain.d);
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(ExperimentalDesignDataFromX_Text)),
	%nrbquote(%LocaleGet(&ProcessName._Label)))), outlib, adsl_&suffix);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend FindingsANOVA;

* run the macro;
%FindingsANOVA;


