/*
JMP Clinical Analytical Process

Name: Findings Shift Plots
Authors: Kelci Miclaus, SAS Institute Inc.


Description: 
   	This process displays shift plots to compare findings measurements at baseline versus
   	on-therapy values and performs a matched pairs analysis by treatment groups on
   	average score during baseline and average score during the trial.  
   	A separate analysis is done for each findings measurement. 
   	Results also include rotated shift plots comparing baseline and trial averages
   	by treatment groups. Shift tables displaying counts of ele
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/
*/
*******************************************************************;
*		Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags"},{"GeneralOptions","BaselineMeasurements","LabNormalization"}}$
$CMP: RSYS.Required Analysis Variables$ $UE: FINDINGS.xxSTRESN and (FINDINGS.xxTESTCD or FINDINGS.xxTEST)$
$CMP: RSYS.Required Baseline Calculation Variables$ $UE: FINDINGS.xxBLFL or FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$
   $NOTES: For FINDINGS.xxBLFL, Value of "Y" categorizes baseline records. For FINDINGS.xxDY and FINDINGS.xxDTC, Records with Study Day <= 1 categorize baseline.$
$CMP: OOUT$ $UE: FINDINGS.xxCAT$ $NOTES: Plots will be organized by values of category in outline boxes.$
$CMP: OOUT$ $UE: FINDINGS.xxSCAT$ $NOTES: Plots will be organized by values of sub category within category outline boxes.$
$CVS_STOP$
*/
*ProcessVariables;

%global StudyName InPath AInPath FDomain  FindDTests TrtUse TrtVar WhereTests WhereForReports
	IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL Statistic NormRef 
	DesignByVars SubjectFilter WhereStmt  MERGESUPPDOM TimeScale BaselineTimeWindow BaselineCalc LogShft NormRef
	DisplayCrossTabs SameAxes FDeltaDrop MatchedPairs
	OutPath ServerOutPath MacroPath FindingsShiftPlots_OutData FindingsShiftPlotsScript objReferenceName;

*ProcessBody;

%macro FindingsShiftPlots;
%let exerr = 0;
*initialization and argument checking;
%let ProcessName = FindingsShiftPlots; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./FindingsMacros.sas" /nosource;
%let WhereForReports=%nrbquote(&WhereStmt);

%if &exiterror %then %goto exit;

%global OutName;
%let OutName = &StudyName._&Fdomain;

/* Set up variables for use in code */
%Global _cd_;
%let _cd_ =  %sysfunc(indexw(&&&FDomain.Vars, %upcase(&FDomain)TESTCD, ' '));
/* set up variables for Shiftplot macro use short name instead of lab name*/
%if &_cd_ > 0 %then %do;
	%let _TestVar = TESTCD;
	%let KeepVar = &FDomain.TEST;
%end;
%else %do;
	%let _TestVar = TEST; 
	%let KeepVar = ;
%end;
%let ResponseVar = &FDomain.STRESN;
%let TestVar = &Fdomain.&_TestVar;

%if %UPCASE(&FDomain) ne LB %then %do;
	%let LogShft = ;
	%let NormRef = ;
	%let DisplayCrossTabs = No;
%end;

%Global BL_Label T_Label;
%PrepOutDS(FindingsShiftPlots_OutData,OutDataName,&FDomain,_sp);

%CheckBLTWsyntax(&BaselineTimeWindow);
%if &exiterror %then %goto exit;

/* load in data*/
%CheckFindingsData(&FDomain,_test=&_TestVar);
%if &exiterror %then %goto exit;

%let trBaseline = %nrbquote(%LocaleGet(Baseline_Text));
%let trTrialStatistic = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(TrialXns_Text)), %nrbquote(%LocaleEZX(%nrbquote(&Statistic), EIA=1))));

/* if a baseline window is specified, use it with BLFL if it exists to partition trial from bl data */
data d;
	set &&in_&Fdomain;
	if missing(&TestVar) then delete;
	%if %UPCASE(&TimeScale)=DAYS %then %do;
		%let STvar = STDY;
		%let ENvar = STDY;
	%end;
	%else %do;
		%let STvar = STWK;
		%let ENvar = STWK;
	%end;
	%if %upcase(&TimeScale)=WEEKS %then %do;
		STWK = ceil(STDY/7);
	%end;
	%if %UpCase(&Fdomain) = LB %then %do;
		%LabsNormalize(&ResponseVar, &NormRef, &LogShft);
	%end;
run;

%if %length(&BaselineTimeWindow) %then %do;
	proc means data=d(keep=&STvar) noprint;
		var &STvar;
		output out=mm min=MinTime max=MaxTime;
	run;

	data _null_;
		set mm;
		call symput('MinTime',compress(MinTime));
		call symput('MaxTime',compress(MaxTime));
	run;
	%put MinTime = &MinTime;
	%put MaxTime = &MaxTime;

	%let TimeWindows = ;
	/* call macro to check and create Baseline Time window and prep for creating Time window*/
	%CreateBaselineTW;
	%if &exerr %then %goto exit;

	%let TimeWindows = %str(&BaselineTimeWindow.[&MinTime,&MaxTime]);
	%let TimeVar = TimeWindow;
	%put TimeWindows = &TimeWindows;
	%put BaselineFlagVar = &BLFlagVar;

	%if &blflag %then %do;
			%AddTimeWindows(domain=&Fdomain,indat=d,outdat=d,twb=&TimeWindows,tvar=&TimeVar,findingscall=1,blflag=1);
		%end;
		%else %do;
			%AddTimeWindows(domain=&Fdomain,indat=d,outdat=d,twb=&TimeWindows,tvar=&TimeVar,findingscall=1);		
	%end;
%end;

/*Creates data sets d1m and d2 containing baseline and trial measurements respectively (baseline change table _std not created)*/
%SplitBaselineTrial(&Fdomain, dset=d, TimeV = &TimeVar, Stat=&Statistic)

/* rename the baseline variable coming from basline macro to do correct shiftplot merge */
/* close initial findings datasets to preserver memory */
proc datasets nolist;
	modify d1m;
	rename baseline = &ResponseVar;
	delete &Fdomain d;
quit;run;

* trial data;
%if (&Statistic = Last) %then %do;
	data d2;
		set d2;
		if missing(&ResponseVar) then delete;
	run;

	proc sort data=d2;
		by &GroupSortVars &TestVar &KeepVar USUBJID &Fdomain.DY;
	run;

	data d2m;
		set d2;
		by &GroupSortVars &TestVar &KeepVar USUBJID &Fdomain.DY;
		if last.USUBJID;
	run;
%end;
%else %do;
	proc sort data=d2;
		by &GroupSortVars &TestVar &KeepVar USUBJID MFLAG;
	run;

	proc means data=d2 noprint;
		by &GroupSortVars &TestVar &KeepVar USUBJID MFLAG;
		var &ResponseVar ;
		output out=d2m &Statistic=&ResponseVar;
	run;
%end;

/* create macro variables for text for elevation categories */
%Global Mtext C1text C2text C3text C4text C5text;
%let trLogShft = %nrbquote(%LocaleEZX(%nrbquote(&LogShft), EIA=1));
%let trNormRef = %nrbquote(%LocaleEZX(%nrbquote(&NormRef), EIA=1));
%let Mtext = %nrbquote(%LocaleGet(Missing_Text));
%let C1text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(LessthanXxX_Text)), 2, &trNormRef));
%let C2text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XxandXxX_Text)), 2, 5, &trNormRef));
%let C3text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XxandXxX_Text)), 5, 10, &trNormRef));
%let C4text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XxandXxX_Text)), 10, 20, &trNormRef));
%let C5text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XxXorGreater_Text)), 20, &trNormRef));

* combine the results and transpose;
/* kemicl added conditional logic to create elevation categories for baseline and trial cross tabs */
data fdata;
	set d1m d2m;
	%if %upcase(&DisplayCrossTabs)=YES %then length Test_Elevation $25;;
	where not missing(&TestVar);
	 if &ResponseVar = . then delete; 
	%if %length(&NormRef) and %length(&LogShft) %then %do;
		_TestLab = trim(left(&TestVar))||" &trLogShft.("||trim(left(MFLAG))||"/&trNormRef.)";
	%end;
	%else %if %length(&NormRef) %then %do;
		_TestLab = trim(left(&TestVar))||trim(left(MFLAG))||"/&trNormRef";
	%end;
	%else %if %length(&LogShft) %then %do;
		_TestLab = trim(left(&TestVar))||" &trLogShft.("||trim(left(MFLAG))||")";
	%end;
	%else %do;
		_TestLab = trim(left(&TestVar))||" "||trim(left(MFLAG));
	%end;
	%if %upcase(&DisplayCrossTabs)=YES %then %do;
		_TestCLab = trim(left(&TestVar))||" "||trim(left(MFLAG));
		/* kemicl add new categories of elevation for tables */
		%if %upcase(&NormRef) = ULN %then %do;	
			if missing(&ResponseVar) then Test_Elevation = " ";
			else if &ResponseVar < &LogShft(2) then Test_Elevation = "&C1text";
			else if &LogShft(2) <= &ResponseVar < &LogShft(5) then Test_Elevation = "&C2text";
			else if &LogShft(5) <= &ResponseVar < &LogShft(10) then Test_Elevation = "&C3text";
			else if &LogShft(10) <= &ResponseVar < &LogShft(20) then Test_Elevation = "&C4text";
			else Test_Elevation = "&C5text";
		%end;
	%end;
run;

proc sort data=fdata out=ftmp nodupkey;
	by &TestVar MFLAG;
run;

proc freq data=ftmp noprint;
	tables &TestVar / out=ftmp;
run;

data ftmp; set ftmp;
	if COUNT <=1 then delete;
run;

proc sort data=fdata;
	by &TestVar;
run;

data fdata; merge fdata ftmp;
	by &TestVar;
	if missing(COUNT) then delete;
run;

proc sort data=fdata;
	by USUBJID _TestLab &GroupSortVars ;
run;

proc means data=fdata noprint;
	by USUBJID _TestLab;
	var &ResponseVar;
	id &GroupSortVars &TestVar &KeepVar;
	%if %upcase(&DisplayCrossTabs)=YES %then %do;
		id Test_Elevation _TestCLab;
	%end;
	output out=fdata mean=&ResponseVar;
run;

%if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH & &Statistic=Median %then %do;
	proc sort data=fdata;
		by USUBJID &GroupSortVars DESCENDING _TestLab ;
	run;
%end;
%else %do;
	proc sort data=fdata;
		by USUBJID &GroupSortVars _TestLab ;
	run;
%end;

proc transpose data=fdata out=fdatat(drop=_name_);
	by USUBJID;
	var &ResponseVar;
	id _TestLab;
	idlabel _TestLab;
run;

%if %upcase(&DisplayCrossTabs)=YES %then %do;
	proc transpose data=fdata out=fdatat2(drop=_name_);
		by USUBJID;
		var Test_Elevation;
		id _TestCLab;
		idlabel _TestCLab;
	run;
%end;

%DMFlags;

%if %upcase(&DisplayCrossTabs)=YES %then %do;
	proc sql;
		create table OutLib.&OutDataName as
			select *
			from fdatat _ds1, fdatat2 _ds2, &in_adsl _ds3
			where _ds1.USUBJID=_ds2.USUBJID=_ds3.USUBJID;
	quit;
%end;
%else %do;
	proc sql;
		create table OutLib.&OutDataName as
			select *
			from fdatat _ds1, &in_adsl _ds3
			where _ds1.USUBJID=_ds3.USUBJID;
	quit;
%end;	

%nobsvars(OutLib.&OutDataName);
%if &nobs = 0 %then %do;
	%put [MS_S_0109];
	%put ERROR:  The merge of &&in_&Fdomain and &in_adsl data sets resulted in 0 observations.  No subject identifiers match between the two data sets found in the defined Study folders.;
	%put ERROR:  Please check that the data sets in the Study folders are accurate.;
	%let exiterror = 1;
	%goto exit;
%end;

%let SubjectLabel = USUBJID;
%CheckLabel(OutLib.&OutDataName,SubjectLabel);

%let odata = OutLib.&OutDataName;
%include "&MacroPath./DesignByVars.sas";

%if %upcase(&SplitResults)=YES %then %do;
	%global TTotLvls;
	proc sort data=&in_adsl(where = (not missing(&TrtName))) out=bvdat;
		by &DesignByVars &TrtName;
	run;

	%NObsVars(bvdat);
	%let TTotLvls = &nobs;
%end;

* determine number of treatment levels.;
proc freq data=&in_adsl noprint;
	tables &TrtName / out=fc;
	where not missing(&TrtName);
run;

%NObsVars(fc);
%let nlev = &nobs;

 %if &syserr > 4 or &exerr=1 %then %do;
	%let exiterror = 1;
 %end; 
%if &exiterror %then %goto exit;

* create JSL file;
%PrepOutFile(FindingsShiftPlotsScript,FindingsShiftPlotsScriptName,&StudyName &ProcessName Script for &FDomain..jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &FindingsShiftPlotsScript;
%let FullAPName=%str(Findings Shift Plots (&FDomain));

%if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH & &Statistic=Median %then %do;
	proc sort data=fdata nodupkey out=vals(keep=_TestLab %if %upcase(&DisplayCrossTabs)=YES %then _TestCLab;);
		by &GroupSortVars DESCENDING _TestLab %if %upcase(&DisplayCrossTabs)=YES %then DESCENDING _TestCLab;;
	run;
%end;
%else %do;
	proc sort data=fdata nodupkey out=vals(keep=_TestLab %if %upcase(&DisplayCrossTabs)=YES %then _TestCLab;);
		by &GroupSortVars _TestLab %if %upcase(&DisplayCrossTabs)=YES %then _TestCLab;;
	run;
%end;

data vals;
	set vals end=_last;
	call symput("_test_"||trim(left(_n_)),trim(left(_TestLab)));
	%if %upcase(&DisplayCrossTabs)=YES %then call symput("_testc_"||trim(left(_n_)),trim(left(_TestCLab)));;
	if _last then call symput("ntest", _n_/2);
run;

/* Get Test Names for Plot Title Labels */
%if &_cd_ > 0 %then %let TLabelVar = &FDomain.TEST;
%else %let TLabelVar = &FDomain.TESTCD;

%if &funits or &fcat or &_cd_ %then %do;
	/* get units of measurement */
	/* Get order of GroupVars */
	/* Get xxTEST or CD variable to label titles with */
	proc sort data = fdata(keep=&GroupSortVars &TLabelVar where=(not missing(&TLabelVar))) out=gunits nodupkey;
		by &GroupSortVars &TLabelVar;
	run;

	/*kemicl:S1049452 - add check for unique values of test code across categories and group sort variables */
	%nobsvars(gunits);
	%let ntottest = &nobs;

	proc sort data = gunits out=uttests nodupkey;
		by &Fdomain.TEST;
	run;
	%nobsvars(uttests);

	%if &nobs < &ntottest %then %do;
		%put [MS_S_0110];
		%put ERROR: Analysis cannot be completed because the &TestVar values are not unique across values of test category and/or subcategory.;
		%let exiterror = 1;
		%goto mexit;
	%end;

	data gunits; 
		set gunits end=_last;
		%if &fcat %then %do;
			by &GroupSortVars;
		%end;
		%if &fcat %then %do;
			if missing(&Fdomain.Cat) then &Fdomain.CAT = "NONE";
			if first.&FDomain.CAT then call symput("Cat"||trim(left(_n_)),trim(left(&FDomain.CAT)));
			%if &fscat %then %do;
				if first.&FDomain.SCAT then call symput("SCat"||trim(left(_n_)),trim(left(&FDomain.SCAT)));
			%end;
		%end;
		call symput("testnamelev"||trim(left(_n_)),trim(left(&TLabelVar)));
	run;
%end;		
			

%DMLabels(OutLib.&OutDataName);
%CheckStudyLevels;

* put JSL statements;
data _null_;
	file "&FindingsShiftPlotsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "objRefNS = newNamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:DataList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	put "objRefNS:CreateReportList = {};";
	put "objRefNS:CreateReportListone={};";
	put "objRefNS:CreateReportListtwo= {};";
	put "objRefNS:NoteDomain = ""&FDomain"";";

	put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
	put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";

	put "objRefNS:includestring=""Findings Tests: "";";
	%let DomainWhereStmt = &WhereTests;
	put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";			

	%if %length(&WhereForReports) %then %do;
		put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
		put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
		%if %length(&DomainWhereStmt) %then %do;
		put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
		%end;
	%end;
	%else %if %length(&DomainWhereStmt) %then %do;
		put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
	%end;

	put "objRefNS:OpendtExpr = Expr(";
	put "   If(IsEmpty(objRefNS:spmp),";
	put "      objRefNS:spmp=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
	put "      Column(""&SubjectLabel"") << label(1);";
	put "      objRefNS:spmp << Color By Column(Column(""&TrtLabel""));";
	put "		objRefNS:dbv = {};";
	%do i=1 %to &ndbv;
		put "		InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
	%end;	
	%if %upcase(&DisplayCrossTabs)=YES %then %do;
		%do i=1 %to &ntest;
			%let j=%eval(2*&i-1);
			%let k = %eval(2*&i);
			put "	Column(""&&_testc_&j"")<<Set Property(""Value Ordering""," @;
			put "{""&Mtext"",""&C1text"",""&C2text"",""&C3text"",""&C4text"",""&C5text""} " @;
			put ");";
			put "	Column(""&&_testc_&k"")<<Set Property(""Value Ordering""," @;
			put "{""&Mtext"",""&C1text"",""&C2text"",""&C3text"",""&C4text"",""&C5text""} " @;
			put ");";	
		%end;
	%end;

	put "  	,"; /*else*/
	put "     Current Data Table(objRefNS:spmp);";
	put "  	);";
	put ");";

	put "	 objRefNS:OpendtExpr;";

	%if %symexist(mlchklow) or %symexist(mlchkhi) %then %do;
		%if &mlchklow or &mlchkhi %then %do;
			put "objRefNS:MLfunc=Function({},{default local},";
			put "		VListBox(";
			put "    objRefNS:mlwarn= TextBox(locale:get(""WarningResultsNotGenDueToMissValsGenByNorm_Text"")||"; 
			%if &mlchklow %then %do;
				put "    ""\!N\!N "" || locale:fill(locale:get(""AllLBSTNRLOMissOrZeroForLBTESTCDX_Text""), {""%trim(&chklln)""}) || ""\!N ""||";     
			%end;
			%if &mlchkhi %then %do;
				put "    ""\!N\!N "" || locale:fill(locale:get(""AllLBSTNRHIMissOrZeroForLBTESTCDX_Text""), {""%trim(&chkuln)""}) || ""\!N""||";      
			%end;    
			put "       "" "",";
			put "          <<Set Wrap(600), Set Font Size(11)";
			put "    );";  
			put "));";
			
			put 'InsertInto(objRefNS:DataList,{{""}});';
			put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XWarningMessage_Text""), {locale:get(""Findings_Text"")})));";
			put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:MLfunc()));';
			put 'InsertInto(objRefNS:TabAfterCodeList," ");';
			put 'InsertInto(objRefNS:TabOpenList,1);';
			put "InsertInto(objRefNS:CreateReportList,{{objRefNS:mlwarn}});";			
		%end;
	%end;	
	put "objRefNS:SPfunc=Function({},{default local},";
	put "	 objRefNS:OpendtExpr;";
	put "   objRefNS:fsize = 400;";
	put "   objRefNS:msize = 2;";
	put "   VListBox(";
	put '       HListBox(TextBox(" " || locale:get("FrameSize_Text")),';
	put '          SliderBox(num(100),num(700),objRefNS:fsize,';
	%do i = 1 %to &ntest;
		put "          	objRefNS:spbivrep&i = (objRefNS:spbiv&i<<report);";
		%if %upcase(&SplitResults)=YES %then %do;
			%do j=1 %to %eval(&TotLvls);
				put "             try(objRefNS:spbivrep&i.[&j][framebox(3)]<<FrameSize(objRefNS:fsize,0.85*objRefNS:fsize));";
			%end;		
		%end;
		%else %do;
			put "             try(objRefNS:spbivrep&i.[framebox(3)]<<FrameSize(objRefNS:fsize,0.85*objRefNS:fsize));";
		%end;
	%end;
	put " 		 	),";
	put "  		 ),";
	put "objRefNS:bivlb = VListBox(";
	%do i=1 %to &ntest;
		%let j=%eval(2*&i-1);
		%let k = %eval(2*&i);
		%if &fcat %then %do;
			%if %symexist(Cat&i) %then %do;
				put "	OutlineBox(locale:fill(locale:get(""XTestCategory_Text""), {""&FDomain""}) || "" = &&Cat&i"",";
			%end;
			%if %symexist(SCat&i) %then %do;
				%if %length(&&SCat&i) %then %do;
					put "	TextBox(locale:fill(locale:get(""XTestSubcategory_Text""), {""&FDomain""}) || "" = &&Scat&i""),";
				%end;
			%end;
		%end;		
		%if %upcase(&SplitResults)=YES %then %do;
			put "   	LineupBox(Ncol(&TotLvls),";
		%end;		
		put "   	 	objRefNS:spbiv&i = Bivariate(";
		%if %upcase(&SplitResults)=YES %then %do;
			put "         By(eval(objRefNS:dbv)),";
		%end;
		put "   			X(Column(""&&_test_&j"")),";
		put "				Y(Column(""&&_test_&k"")),";
		put "				Histogram Borders(1),";
		put "				Automatic Recalc( 1 ),";
		/* %if %length(&_Label2) %then %do;
			put " 			By( &JSLByVar, :&_Label2),";
		%end;
		%else %do;
		put " 			By( &JSLByVar),";
		*/
		put " 			SendToReport(";
		put '            Dispatch({}, "Bivar Plot",';
		put "			      FrameBox,{Frame Size(objRefNS:fsize, .85*objRefNS:fsize), Marker Size(objRefNS:msize),";
		put '             Add Graphics Script( 1, Description( "RefLine" ), Y Function( x, x ) ),';
		put "			      Row Legend(";
		put "              Column(""&TrtLabel""), Color(1), Marker(1),";
		put '             Marker Theme("Solid"),';
		put "				   Continuous Scale(0),";
		put "           	)}";
		put "         	  )";
		put "       	)";
		put "			),";		
		%if %upcase(&SplitResults)=YES %then %do;
			put "),";   /* close lineup, */
		%end;		
		* end for each bivariate plot;
		%if &i < &ntest and &fcat %then %do;
			%let ni = %eval(&i+1);
			%if %symexist(Cat&ni) %then %do;
				put "   ),";  /* end each outline box for test category*/
			%end;
		%end;
		%if &i = &ntest and &fcat %then %do;
			put "    ),";  /* close last outline box if xxCat used */
		%end;		
	%end;	
	%put ")";  
	put ")));";  /* close function*/
	
	%let TabCodeListFunc = "" "";
	
	%if %upcase(&SameAxes) = YES %then %do;
		put "objRefNS:SPAfterfunc=Function({},{default local},";
		%do ii = 1 %to &ntest;
			put "bivr&ii = objRefNS:spbiv&ii<<Report;";
			%if %upcase(&SplitResults)=YES %then %do;
				put "for(byl = 1, byl <= nitems(bivr&ii), byl++,"; 
				put "		combmax = max(bivr&ii[byl][AxisBox(1)]<<Get max,bivr&ii[byl][AxisBox(2)]<<Get max);";
				put "		combmin = min( bivr&ii[byl][AxisBox(1)]<<Get min,bivr&ii[byl][AxisBox(2)]<<Get min);";
				put "		bivr&ii[byl][AxisBox(1)]<<Max(combmax);";
				put "		bivr&ii[byl][AxisBox(2)]<<Max(combmax);";
				put "		bivr&ii[byl][AxisBox(1)]<<Min(combmin);";
				put "		bivr&ii[byl][AxisBox(2)]<<Min(combmin);";
				put ");";
			%end;
			%else %do;
				put "combmax = max(bivr&ii[AxisBox(1)]<<Get max,bivr&ii[AxisBox(2)]<<Get max);";
				put "combmin = min( bivr&ii[AxisBox(1)]<<Get min,bivr&ii[AxisBox(2)]<<Get min);";
				put "bivr&ii[AxisBox(1)]<<Max(combmax);";
				put "bivr&ii[AxisBox(2)]<<Max(combmax);";
				put "bivr&ii[AxisBox(1)]<<Min(combmin);";
				put "bivr&ii[AxisBox(2)]<<Min(combmin);";
			%end;
		%end;
		put ");";
		%let TabCodeListFunc = expr(objRefNS:SPAfterFunc());
	%end;
	
	put "objRefNS:CustomPanel = Expr(objRefNS:Filter= objRefNS:spmp <<Data Filter(<<Conditional(1),";
	put "	Add Filter(";
	put "       Columns(";
	%if &doStudy %then %do;
		put " :Name(""&StudyLabel""),";
	%end;
	%if &doAge %then %do;
		put " :Name(""&AgeLabel""),";
	%end;
	%if &doSex %then %do;
		put " :Name(""&SexLabel""),";
	%end;
	%if &doRace %then %do;
		put " :Name(""&RaceLabel""), ";
	%end;
	%if &doSite %then %do;
		put "       :Name(""&SiteLabel""),";
	%end;      
	put" :Name(""&TrtLabel"")),";	
	put "	),";
	put "	Mode(Show(1), Include(1))";
	put "));";

	put 'InsertInto(objRefNS:DataList,{{"objRefNS:spmp"}});';
	put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XShiftPlots_Text""),{""&FDomain""})));";
	put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:SPfunc()));';
	put "InsertInto(objRefNS:TabAfterCodeList,&TabCodeListFunc);";
	put 'InsertInto(objRefNS:TabOpenList,1);';
	put "InsertInto(objRefNS:CreateReportList,{{objRefNS:bivlb}});";

	/*Kemicl added new tables of lab elevations for shift plots */
	%if %upcase(&DisplayCrossTabs)=YES %then %do;
		put "objRefNS:TBfunc=Eval(Parse(EvalInsert(""\[Function({},{default local},";
		put "	 objRefNS:OpendtExpr;";
		put "	 objRefNS:VLB = VListBox(";
		%do i=1 %to &ntest;
			%let j=%eval(2*&i-1);
			%let k = %eval(2*&i);
			%if &fcat %then %do;
				%if %symexist(Cat&i) %then %do;
					put "	OutlineBox(locale:fill(locale:get(""XTestCategory_Text""), {""&FDomain""}) || "" = &&Cat&i"",";
				%end;
				%if %symexist(SCat&i) %then %do;
					%if %length(&&SCat&i) %then %do;
						put "	TextBox(locale:fill(locale:get(""XTestSubcategory_Text""), {""&FDomain""}) || "" = &&Scat&i""),";
					%end;
				%end;
			%end;					
			put "				objRefNS:tb&i = Tabulate(";
			put "					Change Item Label(Statistics(N,^locale:get(""Count_Text"")^),Statistics(""% of Total"", ""%    "")),";
			put "					Set Format(""% of Total""(9,1)),";
			put "					Show Control Panel(0),";
			put "					Include missing for grouping columns( 1 ),";
			put "					Add Table(";
			put "						Column Table(Grouping Columns(:Name(""&&_testc_&j"")), Add Aggregate Statistics(:Name(""&&_testc_&j"")),Statistics(N, ""% of Total"")),";		
			put "						Row Table(Grouping Columns(:Name(""&&_testc_&k"")), Add Aggregate Statistics(:Name(""&&_testc_&k"")))";
			put "					),";
			/* kemicl comment out  */
			put "				//By(:Name(""&TrtLabel""))";
			put "				SendToReport(Dispatch({},""Tabulate"",OutlineBox,{Set Title( locale:fill(locale:get(""XVsXElevations_Text""), {""&&_testc_&j"", ""&&_testc_&k""}))}))";			
			put "				),";
			%if &i < &ntest and &fcat %then %do;
				%let ni = %eval(&i+1);
				%if %symexist(Cat&ni) %then %do;
					put "   ),";  /* end each outline box for test category*/
				%end;
			%end;
			%if &i = &ntest and &fcat %then %do;
				put "    )";  /* close last outline box if xxCat used */
			%end;					
		%end;
		put "))]\""))); //end hlist datafcontext vlist and function";

		put 'InsertInto(objRefNS:DataList,{{"objRefNS:spmp"}});';
		put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XShiftTables_Text""),{""&FDomain""})));";
		put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:Tbfunc()));';
		put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
		put 'InsertInto(objRefNS:TabOpenList,1);';
		put"InsertInto(objRefNS:CreateReportList,{{objRefNS:VLB}});"; 
	%end;
		/* end new tables */
	%if %upcase(&MatchedPairs) = YES %then %do;
		put "objRefNS:MPfunc=Function({},{default local},";
		put "	 objRefNS:OpendtExpr;";
		put "  objRefNS:fsizemp = 300;";

		put " 	VListBox(";
		put '       HListBox(TextBox(" " || locale:get("FrameSize_Text")),';
		put '          frameSizeSlidermp = SliderBox(num(100),num(700),objRefNS:fsizemp,';
		%do i = 1 %to &ntest;
			put "          	try(objRefNS:mprep&i = (objRefNS:mp&i<<report));";
			%if %upcase(&SplitResults)=YES %then %do;
				%do j=1 %to %eval(&TTotLvls);
					put "             try(objRefNS:mprep&i.[&j][framebox(1)]<<FrameSize(objRefNS:fsizemp,objRefNS:fsizemp));";
				%end;		
			%end;
			%else %do;
				%do j=1 %to &nlev;
					put "             try(objRefNS:mprep&i.[&j][framebox(1)]<<FrameSize(objRefNS:fsizemp,objRefNS:fsizemp));";
				%end;
			%end;
		%end;
		put " 		 	),";
		put "  		 ),";
		put "   objRefNS:mp = LineupBox(NCol(&nlev),";
		%do i=1 %to &ntest;
			%let j=%eval(2*&i-1);
			%let k = %eval(2*&i);		
			put "     objRefNS:mp&i = Matched Pairs(";
			put "         Y(Column(""&&_test_&j""), Column(""&&_test_&k"")),";
			put "         Reference Frame(1),";
			/* %if %length(&_Label2) %then %do;
				put "         By(&JSLByVar, &_Label2, Column(""&TrtLabel"")),";
			%end;
			%else %do;
			*/
			%if %upcase(&SplitResults)=YES %then %do;
				put "         By(eval(objRefNS:dbv),Column(""&TrtLabel"")),";
			%end;
			%else %do;
				put "         By(Column(""&TrtLabel"")),";
			%end;
			put " 			SendToReport(";
			put '          Dispatch({}, "Paired Plot Dif by Mean",';
			put "			      FrameBox,{Frame Size(288,288)}";
			put "				))";
			put "      ),";
		%end;
		put ")));";
		put 'InsertInto(objRefNS:DataList,{{"objRefNS:spmp"}});';
		put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XMatchedPairs_Text""),{""&FDomain""})));";
		put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:MPfunc()));';
		put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
		put 'InsertInto(objRefNS:TabOpenList,1);';
		put"InsertInto(objRefNS:CreateReportList,{{objRefNS:mp}});";		
	%end;
	%SubjectDrillDowns;
	put 'objRefNS:ActionDataTableList = {{{"objRefNS:spmp"},{"objRefNS:spmp"},{"objRefNS:spmp"},{"objRefNS:spmp"},{"objRefNS:spmp"}}};';


run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &FindingsShiftPlotsScriptName, &OutDataName..sas7bdat, yes);


%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend FindingsShiftPlots;

* run the macro;
%FindingsShiftPlots;


