/*
JMP Clinical Analytical Process

Name: Findings Waterfall Plots
Authors: Kelci Miclaus, SAS Institute Inc.
History:
	kemicl 14Jul16 Initial Coding

Description: 
   	This process creates waterfall plots to show the distribution of changes in test measurements for a given Findings domain
   	across subjects (ordered by their magnitude of change). The Findings measurements are summarized based on a specified summary 
   	statistic and a waterfall plot is created for each Findings test.
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*/
*******************************************************************;
*		Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"FindingsO"},{"GeneralOptions","BaselineMeasurements","LabNormalization"}}$
$CMP: RSYS.Required Analysis Variables$ $UE: FINDINGS.xxSTRESN and (FINDINGS.xxTEST or FINDINGS.xxTESTCD)$
$CMP: OOUT$ $UE: FINDINGS.xxSTRESU$ $NOTES: Annotate plots with Standardized units on Y-axis measurement value.$
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName InPath AInPath FDomain  FindDTests TrtUse TrtVar WhereTests WhereForReports 
	IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL MeasurementValues Statistic  NormRef 
	DesignByVars SubjectFilter WhereStmt  MERGESUPPDOM TimeScale BaselineTimeWindow BaselineCalc LogShft NormRef
	DisplaySubjectAxis OverlayTrtGroups ShowStat OutPath ServerOutPath MacroPath FindingsWPlots_OutData FindingsWPlotsScript FDeltaDrop objReferenceName;

*ProcessBody;

%macro FindingsWaterfallPlots;
%let exerr = 0;
*initialization and argument checking;
%let ProcessName = FindingsWaterfallPlots; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./FindingsMacros.sas" /nosource;
%let WhereForReports=%nrbquote(&WhereStmt);
%if &exiterror %then %goto exit;

%global OutName;
%let OutName = &StudyName._&Fdomain;
%let LogShft = ;
%if %UPCASE(&FDomain) ne LB %then %do;
	%let NormRef = ;
%end;
%let ResponseVar = &Fdomain.STRESN;

%global HasBoth TestCodeVar;
%let HasBoth = 0;
	
/* kemicl look for xxTEST first then xxTESTCD */
%if %sysfunc(indexw(&&&FDomain.Vars, %upcase(&FDomain)TEST, ' ')) %then %do;
	%let tvar = TEST;
	%let TestVar = &Fdomain.TEST;
	%if %sysfunc(indexw(&&&FDomain.Vars, %upcase(&FDomain)TESTCD, ' ')) %then %do;
		%let HasBoth = 1;
		%let TestCodeVar = &Fdomain.TESTCD;
	%end;
%end;
%else %do;
	%let tvar = TESTCD;
	%let TestVar = &Fdomain.TESTCD;
%end;

%if %UPCASE(&TimeScale)=DAYS %then %do;
	%let TimeVar = STDY;
	%let STvar = STDY;
	%let ENvar = STDY;
%end;
%else %do;
	%let TimeVar = STWK;
	%let STvar = STWK;
	%let ENvar = STWK;
%end;

%if %Upcase(&OverlayTrtGroups) = NO and %Upcase(&TrtUse) ne NONE %then %do;
	%let TrtUse = None;
	%let DesignByVars = &TrtName &DesignByVars;
%end;

%PrepOutDS(FindingsWPlots_OutData,OutDataName,&FDomain,_wp);
%CheckBLTWsyntax(&BaselineTimeWindow);
%if &exiterror %then %goto exit;

/* load in data*/
%CheckFindingsData(&FDomain,_test=&tvar);
%if &exiterror %then %goto exit;

/* reset units back to 0 since transformation makes findings unitless*/
%if %index(%upcase(&MeasurementValues),PERCENT) or %length(&NormRef) %then %let funits = 0;
%let ObsRVarLab = %nrbquote(%LocaleGet(ObservedResult_Text));
%if %length(&NormRef) %then %do;
	%let ObsRVarLab = %nrbquote(&ObsRVarLab./&NormRef);
%end;

data _std;
	set &&in_&Fdomain end=_last;
	label &ResponseVar = "&ObsRVarLab";
	where not missing(&TestVar) and not missing(&ResponseVar);
	%if %upcase(&TimeScale)=WEEKS %then %do;
		STWK = ceil(STDY/7);
	%end;
	%if %UpCase(&Fdomain) = LB %then %do;
		%LabsNormalize(&ResponseVar, &NormRef, &LogShft);
	%end;
	retain _mintime 99999999 _maxtime -999999999;
	if not missing(&TimeVar) then do;
		_mintime = min(_mintime,&TimeVar);
		_maxtime = max(_maxtime,&TimeVar);
	end;
	if _last then do;
			/* need this for baseline calculations whether they are subsetting plots or not */
		call symput('MinTime',trim(left(_mintime)));
		call symput('MaxTime',trim(left(_maxtime)));
	end;
	drop _mintime _maxtime;
run;

%if &blchg_ and ^%length(&BaselineTimeWindow) and %upcase(&MeasurementValues) = CHANGE FROM BASELINE %then %do;
	%let ResponseVar = &FDomain.BLCHG;
%end;
%else %if %index(%upcase(&MeasurementValues),CHANGE) or %index(%upcase(&MeasurementValues),PERCENT) %then %do;
	%let TimeWVar = ;
	%if %length(&BaselineTimeWindow) %then %do;
		%let TimeWindows = ;
		%CreateBaselineTW;
		%if &exerr %then %goto exit;
		%let TimeWindows = %str(&BaselineTimeWindow.[&MinTime,&MaxTime]);
		%let TimeWVar = TimeWindow;
		%put TimeWindows = &TimeWindows;
		%put BaselineFlagVar = &BLFlagVar;

		%if &blflag %then %do;
				%AddTimeWindows(domain=&Fdomain,indat=_std,outdat=_std,twb=&TimeWindows,tvar=&TimeWVar,findingscall=1,blflag=1);
		%end;
		%else %do;
				%AddTimeWindows(domain=&Fdomain,indat=_std,outdat=_std,twb=&TimeWindows,tvar=&TimeWVar,findingscall=1);		
		%end;	
	%end;
	%SplitBaselineTrial(&Fdomain, dset=_std, TimeV = &TimeWVar, doblchg = 1);
	%if &exerr %then %goto exit;
%end;

proc sort data=_std;
	by &GroupSortVars &TestVar USUBJID STDY;
run;

%if (&Statistic = Last) %then %do;
	data stdm;
		set _std;
		by &GroupSortVars &TestVar USUBJID STDY;
		if last.USUBJID;
		keep &TestVar &Fdomain.TESTCD &Fdomain.POS &Fdomain.STRESN &Fdomain.STRESU &Fdomain.BLCHG &Fdomain.PBLCHG &Fdomain.PofBL &Fdomain.BL
			STDY VISIT VISITNUM &GroupSortVars &Fdomain.BLFL &Fdomain.STRNLO &Fdomain.STRNHI
			Baseline &Fdomain.RFlg USUBJID STUDYID;
	run;
%end;
%else %do;
	proc means data=_std noprint;
		by &GroupSortVars &TestVar USUBJID;
		var &ResponseVar ;
		output out=stdm(drop=_type_ _freq_) &Statistic=&ResponseVar;
	run;
%end;

/* create macro vars for tests for graphs;*/
proc sort data=_std(keep=&GroupSortVars &TestVar %if &funits %then &Fdomain.STRESU; %if &hasboth %then &TestCodeVar;) nodupkey out=vals;
	by &GroupSortVars &TestVar;
run;

data vals;
	set vals end=_last;
	where not missing(&TestVar);
	call symput("testlev"||trim(left(_n_)),trim(left(&TestVar)));
	%if &HasBoth %then %do;
		call symput("testcdlev"||trim(left(_n_)),trim(left(&TestCodeVar)));
	%end;
	%if &funits %then call symput("testunits"||trim(left(_n_)),trim(left(&Fdomain.STRESU)));;
	if _last then call symput("nvals", _n_);
run;

/* Check for non-unique values of Test across test category or subcategory */
%if &fcat %then %do;
	proc sort data = vals out=uttests nodupkey;
		by &TestVar;
	run;
	%nobsvars(uttests);

	%if &nobs < &nvals %then %do;
		%put [MS_S_0110];
		%put ERROR: Analysis cannot be completed because the &TestVar values are not unique across values of test category and/or subcategory.;
		%let exiterror = 1;
		%goto exit;
	%end;
%end;

/*create list to send to value order for JSL to be ordered by category */
%if %length(&GroupSortVars) %then %do;
	%let _tList_ = ""%nrbquote(%trim(%nrbquote(&testlev1)))"";
	%do i = 2 %to %eval(&nvals);
		%let  _tList_ = &_tList_ , ""%nrbquote(%trim(%nrbquote(&&testlev&i)))"";
	%end;	
%end;

proc sql;
	create table &FindingsWPlots_OutData(compress=yes) as
	select *
	from stdm d1, &in_adsl d2
	where d1.USUBJID=d2.USUBJID;
quit;

%nobsvars(&FindingsWPlots_OutData);
%if &nobs = 0 %then %do;
	%put [MS_S_0101];
	%put ERROR:  The merge of &&in_&Fdomain and &in_adsl data sets resulted in 0 observations.  No subject identifiers match between the two data sets found in the defined Study folders.;
	%put ERROR:  Please check that the data sets in the Study folders are accurate.;
	%let exiterror = 1;
	%goto exit;
%end;	

/* create flags and get labels*/
%DMFlags;
%DMLabels(&FindingsWPlots_OutData);
%CheckStudyLevels;

%let ResponseLabel = &ResponseVar;
%CheckLabel(&FindingsWPlots_OutData, ResponseLabel);
%let TestLabel = &TestVar;
%CheckLabel(&FindingsWPlots_OutData, TestLabel);
%if &fcat %then %do;
	%let CatLabel = &Fdomain.CAT;
	%CheckLabel(&FindingsWPlots_Outdata,CatLabel);

	%if &fscat %then %do;
		%let SCatLabel = &Fdomain.SCAT;
		%CheckLabel(&FindingsWPlots_Outdata,SCatLabel);
	%end;
%end;

%let odata = &FindingsWPlots_OutData;
%include "&MacroPath./DesignByVars.sas";

 %if &syserr > 4 or &exerr=1 %then %do;
	%let exiterror = 1;
 %end; 
%if &exiterror %then %goto exit;

* create JSL file;
%PrepOutFile(FindingsWPlotsScript,FindingsWPlotsScriptName,&StudyName &ProcessName Script for &FDomain..jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &FindingsWPlotsScript;
%let FullAPName=%str(Findings Waterfall Plots (&FDomain));

* put JSL statements;
data _null_;
	file "&FindingsWPlotsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "objRefNS = newNamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:DataList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	put "objRefNS:CreateReportList = {};";
	put "objRefNS:CreateReportListone={};";
	put "objRefNS:CreateReportListtwo= {};";
	put "objRefNS:NoteDomain = ""&FDomain"";";
	
	%if %upcase(&SplitResults)=YES %then %do;
		%if &ndbv > 1 %then %do;
			put "objRefNS:fvsize = 80;";
		%end;
		%else %do;
			put "objRefNS:fvsize = 175;";
		%end;
	%end;
	%else %do;
		put "objRefNS:fvsize = 325;";
	%end;
	put "objRefNS:fhsize = 650;";
	put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
	put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";

	put "objRefNS:includestring=""Findings Tests: "";";
	%let DomainWhereStmt = &WhereTests;
	put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";			

	%if %length(&WhereForReports) %then %do;
		put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
		put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
		%if %length(&DomainWhereStmt) %then %do;
		put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
		%end;
	%end;
	%else %if %length(&DomainWhereStmt) %then %do;
		put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
	%end;

	put "objRefNS:openDTExpr = Expr(";
	put "   If(IsEmpty(objRefNS:wfdt),";
	put "      objRefNS:wfdt=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
	%if %length(&GroupSortVars) %then %do;
		put "		Column(""&TestLabel"")<<Set Property(""Value Ordering"",{&_tlist_});";
	%end;
	put "      Column(""&SubjectLabel"") << label(1);";
	put "      objRefNS:wfdt << Color By Column(Column(""&TrtLabel""));";
	put "		objRefNS:dbv = {};";
	%do i=1 %to &ndbv;
		put "		InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
	%end;	
	put "  	,"; /*else*/
	put "     Current Data Table(objRefNS:wfdt);";
	put "  	);";
	put ");";
	put "	 objRefNS:openDTExpr;";

	put "objRefNS:WFfunc=Function({},{default local},";
	put "	 objRefNS:openDTExpr;";
	put "   objRefNS:GBvlb = VListBox(";  
	put " 	objRefNS:GB = GraphBuilder(ShowControlPanel(0),";
	put "			By(:Name(""&TestLabel"")),";
	put "			Variables(";
	put "				X(:Name(""&SubjectLabel""), OrderBy(:Name(""&ResponseLabel""), Ascending, OrderStatistic(""Mean""))),";
	put "				Y(:Name(""&ResponseLabel""))";
	%if %upcase(&SplitResults)=YES %then %do;
		%do j=1 %to &ndbv;
			put "       ,Group Y(Column(""&&dbv&j""))";
		%end;
	%end;		
	%if %UPCase(&TrtUse) ne NONE %then %do;
		put "			,Overlay(:Name(""&TrtLabel""))";
	%end;
	put "			),";
	put "			Elements(";
	put "				Bar(X,Y, Bar Style( ""Stacked"" ))";
	%if %Upcase(&ShowStat) ne NONE %then %do;
		put "			,Caption Box(X,Y, Summary Statistic(""&ShowStat""), X Position(""Right""), Y Position(""Top""))";
	%end;
	put "			),";
	/* Remove axis ticks and labels for cleaner X axis unless requested by dialog*/
	%if %upcase(&DisplaySubjectAxis) = NO %then %do;
		put "			SendToReport(Dispatch({},""&SubjectLabel"",Scalebox,";
		put "				{Label Row({Automatic Tick Marks( 0 ),Show Major Ticks(0), Show Major Labels(0)})}";
		put "			))";
	%end;
	put "		);";
	%do j = 1 %to &nvals;
		put " 	try((objRefNS:GB<<report)[&j][OutlineBox(1)]<<Set Title(""&&TestLev&j (&&testcdlev&j)""));";
		put "		try((objRefNS:GB<<report)[&j][FrameBox(1)]<<FrameSize(objRefNS:fhsize,objRefNS:fvsize));";
		%if &funits %then %do;
			put "		try((objRefNS:GB<<report)[&j]<<Dispatch({},""Y title"", TextEditBox,{Set Text(""&ResponseLabel (&&testunits&j)"")}));";
		%end;
	%end;
	put "	) //Close VListBox of Graphs;";
	put "); //Close Function";
	

	put "	objRefNS:CustomPanel = Expr(objRefNS:Filter=objRefNS:wfdt << Data Filter(<<Conditional(1),";
	put "  Add Filter(Columns("; 
	%if &doStudy %then %do;
		put " :Name(""&StudyLabel""),";
	%end;
	%if &doAge %then %do;
		put " :Name(""&AgeLabel""),";
	%end;
	%if &doSex %then %do;
		put " :Name(""&SexLabel""),";
	%end;
	%if &doRace %then %do;
		put " :Name(""&RaceLabel""), ";
	%end;
	%if &doSite %then %do;
		put "       :Name(""&SiteLabel""),";
	%end;      
	%if &doTrt & %UPCase(&TrtUse) ne NONE  %then %do;
		put "	:Name(""&TrtLabel""),";
	%end;
	put "	 )), ";
	put "	 Mode( Show(1), Include(1))";
	put "	);";
	put "	); //expression";	
	
	put 'InsertInto(objRefNS:DataList,{{"objRefNS:wfdt"}});';
	put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XWaterFallPlots_Text""),{""&FDomain""})));";
	put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:WFfunc()));';
	put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
	put 'InsertInto(objRefNS:TabOpenList,1);';
	put "InsertInto(objRefNS:CreateReportList,{{objRefNS:GBvlb}});";	
	%SubjectDrillDowns;

run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &FindingsWPlotsScriptName, &OutDataName..sas7bdat, yes);


%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend FindingsWaterfallPlots;

* run the macro;
%FindingsWaterfallPlots;


