/*
JMP Clinical Analytical Process

Name: Hy's Law Screening
Authors: Kelci Miclaus, SAS Institute Inc.
History: 
   kemicl 10Oct18 Initial Coding
   kemicl 11Jun10 Add tab of missing lab test summaries
   kemicl 11Jul11 Support time specific classification of Hys Law
   kemicl 12Aug21 Further functionality similar to eDISH
   kemicl 12Sep13 Graph Time Trends drill down with exposure summary information
   kemicl 13Jun06 Add Shift tables, Tests by Study day, other major features for FDA
   
Description: 
      This process visualizes peak values for lab measurements pertaining to Hy's Law
      for detecting potential liver toxicity for all subjects across treatment arms.   
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/
*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"Findings"},{"GeneralOptions"}}$
$INCD: {{"LB"},{"LB"}}$
$CMP: RSYS.Required Analysis Variables$ $UE: LB.LBSTRESN and LB.LBTESTCD$
   $NOTES: LBTESTCD must contain values for Bilirubin, Alanine Aminotransferase, 
      Asparatate Aminotransferase, and Alkaline Phosphatasase$
$CMP: CPV.{"Normalize laboratory values by:","Baseline"}$ $UE: LB.LBBLFL or LB.LBDY or (LB.LBDTC and DM.RFSTDTC)$
   $NOTES: For LB.LBBLFL, Value of "Y" categorizes baseline records. For LB.LBDY and LB.LBDTC, Records with Study Day <= 1 categorize baseline.$
$CMP: CPV.{"Normalize laboratory values by:","Upper Limit of Normal"}$ $UE: LB.LBSTNRHI$ $NOTES: Variable must contain values for relevant liver tests$
$CVS_STOP$
*/


*ProcessVariables;

%global StudyName InPath AInPath TrtUse DesignByVars TrtVar SubjectFilter WhereStmt MERGESUPPDOM WhereForReport
ReferenceVar FDomain BaselineCalc TBLTest ALTTest ASTTest ALPTest
HyLag ThreeDPlot  eDishPlot DisplayLogShftH LogShft HyLabel DoRefLines BRefLine ARefLine 
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL OutPath ServerOutPath MacroPath OutName
HysLawScreening_OutData HysLawScreening_OutMissDat HysLawScreeningScript FDeltaDrop WhereTests objReferenceName;

*ProcessBody;

%macro HysLawScreening;

* initialization and argument checking;
%let ProcessName = HysLawScreening; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./FindingsMacros.sas" /nosource;
%let WhereForReport=%nrbquote(&WhereStmt);

%let TrtVar = &TrtName;
%if %Upcase(&ReferenceVar) = BASELINE %then %do;
   %let FDomain = LB;
   %let ResponseVar = LBSTRESN;
   %let MeasurementValues = BASELINE RATIO;
%end;
%else %do;
   %let ResponseVar = LBSULN;
%end;
%let LogShft=;
%let DisplayLogShft = &DisplayLogShftH;
%let eDishPlot = Yes;

%CheckFindingsData(LB);
%let GroupSortVars = ;
data lb; 
   set lb;
   %if %Upcase(&ReferenceVar) = BASELINE %then %do;
      LBTEST = "Peak "||trimn(LBTESTCD)||"/Baseline";
   %end;
   %else %do;
      LBSULN = LBSTRESN/LBSTNRHI;
      LBTEST = "Peak "||trimn(LBTESTCD)||"/ULN";
      LBSULN = round(LBSULN,0.01);
   %end;  
run;

proc freq data=lb noprint;
   tables LBTESTCD*LBTEST / out=fl;
   where not missing(LBTESTCD);
run;

/*Create Macros for each of the lab tests pertaining to Hy's */
%let TBLLab = ;
%let AltTBLLab = ;
%let Alt2TBLLab = ;
%let ALTLab = ;
%let ASTLab = ;
%let ALPLab = ;
%let AltALPLab = ;
data _null_;
   set fl;
   %if %length(&TBLTest) %then %do;
      if trim(LBTESTCD) = "&TBLTEST" then call symput("TBLLab",trim(left(LBTEST)));
   %end;
   %else %do;
      if trim(upcase(LBTESTCD))="BILI" or trim(upcase(LBTESTCD))="TBIL" or trim(upcase(LBTESTCD))="TBILI"  then call symput("TBLLab",trim(left(LBTEST)));
      if trim(upcase(LBTESTCD))="TBL" or trim(upcase(LBTESTCD))="BILT" or trim(upcase(LBTESTCD))="BILTOT" then call symput("AltTBLLab",trim(left(LBTEST)));
      if index(upcase(LBTESTCD),"BIL") then call symput("Alt2TBLLab",trim(left(LBTEST)));
   %end;
   %if %length(&ALTTest) %then %do;    
      if trim(LBTESTCD) = "&ALTTEST" then call symput("ALTLab",trim(left(LBTEST)));
   %end;
   %else %do;
      if trim(upcase(LBTESTCD))="ALT" or trim(upcase(LBTESTCD))="SGPT" or trim(upcase(LBTESTCD))="ALAT" or trim(upcase(LBTESTCD))="SGPT" then call symput("ALTLab",trim(left(LBTEST)));
   %end;
   %if %length(&ASTTest) %then %do;
      if trim(LBTESTCD) = "&ASTTEST" then call symput("ASTLab",trim(left(LBTEST)));
   %end;
   %else %do;  
      if trim(upcase(LBTESTCD))="AST" or trim(upcase(LBTESTCD))="SGOT" or trim(upcase(LBTESTCD))="ASAT" then call symput("ASTLab",trim(left(LBTEST)));
   %end;
   %if %length(&ALPTest) %then %do;
      if trim(LBTESTCD) = "&ALPTEST" then call symput("ALPLab",trim(left(LBTEST)));
   %end;
   %else %do;  
      if trim(upcase(LBTESTCD))="ALP" or trim(upcase(LBTESTCD))="ALKP" then call symput("ALPLab",trim(left(LBTEST)));
      if trim(upcase(LBTESTCD))="ALK" or trim(upcase(LBTESTCD))="APH" then call symput("AltALPLab",trim(left(LBTEST)));
   %end;
run;


%if ^%length(&TBLLab) %then %do;
   %let TBLLab = &AltTBLLab;
   %if ^%length(&TBLLab) %then %let TBLLab = &Alt2TBLLab;
   %if ^%length(&TBLLab) %then %do;
      %put [MS_S_0117];
      %put ERROR: There are no measurements for the Bilirubin Lab Test.  This process cannot be run.;
      %put ERROR: The LBTESTCD variable for Total Bilirubin must be either specified correctly on the dialog or coded as one of these forms: BILI, TBIL, BILTOT, TBL, BILT, or BIL.;
      %let exiterror=1;
   %end;
%end;
%if ^%length(&ALTLab) %then %do;
   %put [MS_S_0118];
   %put ERROR: There are no measurements for the Alanine trasaminase Lab Test.  This process cannot be run.;
   %put ERROR: The LBTESTCD variable for Alanine trasaminase must be either specified correctly on the dialog or coded as one of these forms: ALT, SGPT, or ALAT.;
   %let exiterror=1;
%end;
%if ^%length(&ASTLab) %then %do;
   %put [MS_S_0119];
   %put ERROR: There are no measurements for the Aspartate transaminase Lab Test.  This process cannot be run.;
   %put ERROR: The LBTESTCD variable for Aspartate transaminase must be either specified correctly on the dialog or coded as one of these forms: AST, SGOT, or ASAT.;
   %let exiterror=1;
%end;
%if ^%length(&ALPLab) %then %do;
   %let ALPLab = &AltALPLab;
   %if ^%length(&ALPLab) %then %do;
      %put [MS_S_0120];
      %put ERROR: There are no measurements for the Alkaline phosphatase Lab Test.  This process cannot be run.;
      %put ERROR: The LBTESTCD variable for Alkaline phosphatase must be either specified correctly on the dialog or coded as one of these forms: ALP, ALKP, ALK, APH.;
      %let exiterror=1;
   %end;
%end;
%if &exiterror %then %goto exit;

%put LBTESTCD values used for Hys Law Screening Report;
%put TBLLab = &TBLLab;
%put ALTLab = &ALTLab;
%put ASTLab = &ASTLab;
%put ALPLab = &ALPLab;

%if %Upcase(&DoRefLines) = NO %then %do;
   %let BRefLine = 2;
   %let ARefLine = 3;
%end;

%let PRefLine = &BRefLine;
   
%if %length(&LogShft) %then %do;
   %let HyLaw_TBL = %sysevalf(%sysfunc(&LogShft(&BRefLine)));
   %let HyLaw_ALT = %sysevalf(%sysfunc(&LogShft(&ARefLine)));
   %let HyLaw_ALP = %sysevalf(%sysfunc(&LogShft(&PRefLine)));
%end; 
%else %do;
   %let HyLaw_TBL =&BRefLine;
   %let HyLaw_ALT =&ARefLine;
   %let HyLaw_ALP =&PRefLine;
%end;

%let Outname = &HysLawScreening_OutData;
%let HysLawScreening_OutData = ;
%if ^%length(&Outname) %then %let Outname = lb;

%PrepOutDS(HysLawScreening_OutData,OutDataName,&Outname._hls);
%PrepOutDS(HysLawScreening_OutMissDat,OutDataName2,&Outname._mhl);
 

data _std; set lb;
   where trim(left(LBTEST)) = "&TBLLab" or trim(left(LBTEST)) = "&ALTLab" or 
         trim(left(LBTEST)) = "&ASTLab" or trim(left(LBTEST)) = "&ALPLab"; 
run;

proc datasets nolist;
   delete lb;
quit; run;

%if %Upcase(&ReferenceVar) = BASELINE %then %do;
   %let TimeWVar = ;
   %SplitBaselineTrial(LB, dset=_std, TimeV = &TimeWVar, doblchg = 1);
%end;
%else %do;
/* kemicl add check  ULN missing or 0 in labs */
   %global chklln chkuln mlchklow mlchkhi;
   %let chklln = ;
   %let chkuln = ;
   %let mlchklow = 0;
   %let mlchkhi = 0;
   %CHKULN(_std);
%end;
%if &exiterror %then %goto exit;
   
proc sort data=_std;
   by USUBJID LBTEST LBDY;
run;

* precompute data for graph time trend drill down;
* compute max for each unique time point;
proc means data=_std noprint;
   by USUBJID LBTEST LBDY;
   var &ResponseVar;
   output out=_stdm max=&ResponseVar;
run;

proc sort data=_stdm;
   by USUBJID LBDY;
run;

/***********************************************/
/* Code to produce Graph Time Trends data set */
* transpose to get into form required for graph builder;
proc transpose data=_stdm out= stdmt(drop=_Name_);
   by USUBJID LBDY;
   var &ResponseVar;
   id LBTEST;
   idlabel LBTEST;
run;

/* kemicl added to create hy law indicators */
%let TBLName = &TBLLab;
%let ALTName = &ALTLab;
%let ASTName = &ASTLab;
%let ALPName = &ALPLab;
%Naming(_SingleVar=TBLName);
%Naming(_SingleVar=ALTName);
%Naming(_SingleVar=ASTName);
%Naming(_SingleVar=ALPName);
%let TBLName = %trim(%left(&TBLName));
%let ALTName = %trim(%left(&ALTName));
%let ASTName = %trim(%left(&ASTName));
%let ALPName = %trim(%left(&ALPName));

%let trHLC = %nrbquote(%LocaleGet(HysLawCase_Text));
%let trEATT = %nrbquote(%LocaleGet(ElevatedATTests_Text));
%let trFDATE = %nrbquote(%LocaleGet(FirstDayofATElevation_Text));
%let trFDHLE = %nrbquote(%LocaleGet(FirstDayofHysLawEvent_Text));
data stdmt; set stdmt;
   label HLC_Indicator = &trHLC;
   label AT_Indicator = &trEATT;
   label ATEDay = &trFDATE;
   label HLCDay = &trFDHLE;
   %if &HyLag = 0 %then %do;
      if (&ALTName >= &HyLaw_ALT or &ASTName >=&HyLaw_ALT) then do;
         AT_Indicator = 1;
         ATEDay = LBDY;
      end;
      else AT_Indicator = 0;
      if (&ALTName >= &HyLaw_ALT or &ASTName >=&HyLaw_ALT) and &TBLName >= &HyLaw_TBL then do;
         HLC_Indicator = 1;
         HLCDay = LBDY;
      end;
      else HLC_Indicator = 0;
   %end;
   %else %do;
      by USUBJID;
      retain Aday AT_Indicator;
      HLC_Indicator = 0;
      if first.USUBJID then do;
         AT_Indicator = 0;
         Aday = LBDY;
      end;
      if (&ALTName >= &HyLaw_ALT or &ASTName >=&HyLaw_ALT) then do;
         AT_Indicator = 1;
         Aday = LBDY;
         ATEDay = LBDY;
      end;
      if (LBDY - Aday) > &HyLag then do;
         AT_Indicator = 0;
      end;
      else if AT_Indicator = 1 then do;
         if &TBLName >= &HyLaw_TBL then do;
            HLC_Indicator = 1;
            HLCDay = LBDY;
         end;
      end;
   %end;
   drop Aday;
run;

proc means data=stdmt noprint;
   by USUBJID;
   var HLC_Indicator AT_Indicator ATEDay HLCDay;
   output out=hyind(keep = USUBJID HL_Days AT_Days FATEDY FHLCDY) 
      sum(HLC_Indicator)=HL_Days sum(AT_Indicator)=AT_Days min(ATEDay) = FATEDY min(HLCDay)=FHLCDY;
run;

 * merge adsl;
 proc sql;
   create table outlib.&OutDataName.gtt as
      select *
      from stdmt d1, &in_adsl d2
      where d1.USUBJID=d2.USUBJID;
quit;

/* Get label for LBDY */
%let DYLabel = LBDY;
%CheckLabel(OutLib.&OutDataName.gtt,DYLabel);


/* Create Exposure data set for drawing start stop lines */
%let EXuse = 1;
%if ^%sysfunc(findw(&domains, EX)) %then %do;
   %let EXuse = 0;
%end;

/*kemicl - again surfacing variables that should be captured in CDISCPARMS calculatestudydays algorithm.  Doing here first for transparency */
/*Need to determine how to reference the conditional variables within a macro - multiple code blocks for each macro? */
/*    
$CVS_START$
$CMP: OOUT$ $UE: EX.USUBJID$
$CMP: OOUT$ $UE: EX.EXSTDY or (DM.RFSTDTC and EX.EXSTDTC)$ $NOTES: Draw exposure relative study day start reference lines in Graph Time Trends Profiles.$
$CMP: OOUT$ $UE: EX.EXENDY or (DM.RFSTDTC and EX.EXENDTC)$ $NOTES: Draw exposure relative study day end reference lines in Graph Time Trends Profiles.$
$CVS_STOP$
*/
%if &EXuse %then %do;
   /* Only show from first exposure start to last ENDY */
   %LOADDATA(domlist=EX,mergesupp= 0, checkreview = 0);
   
   data ex;
      set &in_ex;
      if STDY=ENDY then EXDUR =1;
   run;
   
   proc sql;
      create table ex1 as
      select d1.USUBJID, STDY, ENDY  
      from ex d1, &in_adsl d2
      where d1.USUBJID=d2.USUBJID; 

      create table exuni as
      select USUBJID, min(STDY) as StartDose, max(ENDY) as EndDose 
      from ex1 
      group by USUBJID;
      
      create table Outlib.&OutDataName.gtt as
      select *
      from Outlib.&OutDataName.gtt d1, exuni d2
      where d1.USUBJID=d2.USUBJID;
   quit;
   
   proc datasets nolist;
      delete ex;
   quit; run;
%end;


/******************************************/
* compute peak values for each subject;
proc means data=_std noprint;
   by USUBJID LBTEST;
   var &ResponseVar;
   output out=_stdm max=&ResponseVar;
run;

* run means again to get max val for plots; 
proc sort data=_stdm out=maxchk;
   by LBTEST;
run;

proc means data=maxchk noprint;
   by LBTEST;
   var &ResponseVar;
   output out=maxm max=maxval min=minval;
run;

data _null_; set maxm;
   if minval > .2 then minval = .1;
   if minval < .1 then minval = .03;
   call symput("LabMax"||trim(left(_n_)),trim(left(maxval)));
   call symput("LabMin"||trim(left(_n_)),trim(left(minval)));
run;

%put ALP Max = &LabMax1, ALT Max = &LabMax2, AST Max = &LabMax3, TBILI Max = &LabMax4;

/* get some padding for graphics if lower */
%if %UPCASE(&DisplayLogShft) = YES %then %do;
   %let TBL_max = %sysevalf(&HyLaw_TBL+125);
   %let ALT_max = %sysevalf(&HyLaw_ALT+150);
   %let ALP_max = %sysevalf(&HyLaw_ALP+125);
%end;
%else %do;
   %let TBL_max = %sysevalf(&HyLaw_TBL+1);
   %let ALT_max = %sysevalf(&HyLaw_ALT+2.25);
   %let ALP_max = %sysevalf(&HyLaw_ALT+1);
%end;
   
* transpose to get into form;
proc transpose data=_stdm out=lb_hls;
   by USUBJID;
   var &ResponseVar;
   id LBTEST;
   idlabel LBTEST;
run;

 %if &syserr > 4 %then %do;
   %let exiterror = 1;
   %goto exit;
 %end;

 * merge adsl;
 %let trDHL = %nrbquote(%LocaleGet(DaysinHysLaw_Text));
 %let trDATE = %nrbquote(%LocaleGet(DayswithATElevation_Text));
 proc sql;
   create table OutLib.&OutDataName as
      select *
      from lb_hls d1, hyind d3, &in_adsl d2 
      where d1.USUBJID=d3.USUBJID=d2.USUBJID;
   alter table OutLib.&OutDataName
      modify HL_Days label="&trDHL"
      modify AT_Days label="&trDATE";
quit;

%nobsvars(OutLib.&OutDataName);
%if &nobs = 0 %then %do;
   %put [MS_0101];
   %put ERROR:  The merge of &in_lb and &in_adsl data sets resulted in 0 observations.  No subject identifiers match between the two data sets found in the defined Study folders.;
   %put ERROR:  Please check that the data sets in the Study folders are accurate.;
   %let exiterror = 1;
   %goto exit;
%end;    

%let dsid = %sysfunc(open(OutLib.&OutDataName));
%let MType = %sysfunc(vartype(&dsid,%sysfunc(varnum(&dsid,&TrtVar))));
%let dsid = %sysfunc(close(&dsid));
%put Treatment variable type is &MType;

/* Kemicl get just name of lab for table report */
%let ALTCD = %scan(&ALTLab,2," /");
%let ASTCD = %scan(&ASTLab,2," /");
%let ALPCD = %scan(&ALPLab,2," /");
%let TBLCD = %scan(&TBLLab,2," /");

/* Translate ReferenceVar */
%let ReferenceVar = %nrbquote(%LocaleEZX(&ReferenceVar, EIA=1));

/* create macro variables for text for elevation categories */
%let Mtext = %nrbquote(%LocaleGet(MissingTestResult_Text));
%let C1text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(LessthanXxX_Text)),2,%nrbquote(&ReferenceVar)));
%let C2text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(BetweenXxandXxX_Text)),2,5,%nrbquote(&ReferenceVar)));
%let C3text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(BetweenXxandXxX_Text)),5,10,%nrbquote(&ReferenceVar)));
%let C4text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(BetweenXxandXxX_Text)),10,20,%nrbquote(&ReferenceVar)));
%let C5text = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XxXorGreater_Text)),20,%nrbquote(&ReferenceVar)));

%let trALTCDE = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XElevations_Text)), %nrbquote(&ALTCD)));
%let trASTCDE = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XElevations_Text)), %nrbquote(&ASTCD)));
%let trALPCDE = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XElevations_Text)), %nrbquote(&ALPCD)));
%let trTBLCDE = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XElevations_Text)), %nrbquote(&TBLCD)));

%let trYes = %nrbquote(%LocaleGet(Yes_Text));
%let trNo = %nrbquote(%LocaleGet(No_Text));

data OutLib.&OutDataName; set OutLib.&OutDataName;
   label HLC_Indicator = "&trHLC" 
      ALT_Elevation = "&trALTCDE" AST_Elevation = "&trASTCDE" ALP_Elevation = "&trALPCDE" TBL_Elevation = "&trTBLCDE";
   length ALT_Elevation $40 AST_Elevation $40 ALP_Elevation $40 TBL_Elevation $40;
   if HL_Days > 0 then HLC_Indicator = symget('trYes');
   else do;
      HLC_Indicator = symget('trNo');
      HL_Days = .;
   end;
   label AT_Indicator = "&trEATT";
   if AT_Days > 0 then AT_Indicator = symget('trYes');
   else do;
      AT_Indicator = symget('trNo');
      AT_Days = .;
   end;
   %if %Upcase(&MType) = C %then %do;
      if missing(&TrtVar) then &TrtVar = "Missing";
   %end;
   %else %do;
      if missing(&TrtVar) then &TrtVar = -99;
   %end;
   /* kemicl add new categories of elevation for tables */
   if missing(&ALTName) then ALT_Elevation = "&Mtext";
   else if &ALTName < 2 then ALT_Elevation = "&C1text";
   else if 2 <= &ALTName < 5 then ALT_Elevation = "&C2text";
   else if 5 <= &ALTName <10 then ALT_Elevation = "&C3text";
   else if 10 <= &ALTName <20 then ALT_Elevation = "&C4text";
   else ALT_Elevation = "&C5text";

   if missing(&ASTName) then AST_Elevation = "&Mtext";
   else if &ASTName < 2 then AST_Elevation = "&C1Text";
   else if 2 <= &ASTName < 5 then AST_Elevation = "&C2text";
   else if 5 <= &ASTName <10 then AST_Elevation = "&C3text";
   else if 10 <= &ASTName <20 then AST_Elevation = "&C4text";
   else AST_Elevation = "&C5text";
   
   if missing(&ALPName) then ALP_Elevation = "&Mtext";
   else if &ALPName < 2 then ALP_Elevation = "&C1text";
   else if 2 <= &ALPName < 5 then ALP_Elevation = "&C2text";
   else if 5 <= &ALPName <10 then ALP_Elevation = "&C3text";
   else if 10 <= &ALPName <20 then ALP_Elevation = "&C4text";
   else ALP_Elevation = "&C5text";
   
   if missing(&TBLName) then TBL_Elevation = "&Mtext";
   else if &TBLName < 2 then TBL_Elevation = "&C1text";
   else if 2 <= &TBLName < 5 then TBL_Elevation = "&C2text";
   else if 5 <= &TBLName <10 then TBL_Elevation = "&C3text";
   else if 10 <= &TBLName <20 then TBL_Elevation = "&C4text";
   else TBL_Elevation = "&C5text";  
run;

data checkhys; set OutLib.&OutDataName(keep=HLC_Indicator);
   where HLC_Indicator = symget('trYes');
run;

%nobsvars(checkhys);
%let numhys = &nobs;

data checkhys; set OutLib.&OutDataName(keep=AT_Indicator);
   where AT_Indicator = symget('trYes');
run;

%nobsvars(checkhys);
%let numat = &nobs;

/* Get label for LBDY */
%let HLDLabel = HL_Days;
%CheckLabel(OutLib.&OutDataName,HLDLabel);
%let HLILabel = HLC_Indicator;
%CheckLabel(OutLib.&OutDataName,HLILabel);
%let ATELabel = AT_Indicator;
%CheckLabel(OutLib.&OutDataName,ATELabel);
%let FATELabel = FATEDY;
%CheckLabel(OutLib.&OutDataName,FATELabel);
%let FHLCLabel = FHLCDY;
%CheckLabel(OutLib.&OutDataName,FHLCLabel);
%let SubjectLabel = USUBJID;
%CheckLabel(OutLib.&OutDataName,SubjectLabel);


%DMFlags;
%DMLabels(OutLib.&OutDataName);
%CheckStudyLevels;

%let trMLT = %nrbquote(%LocaleGet(MissingLabTest_Text));
/* %if %Upcase(&SumMissing)= YES %then %do; */
   /* ALT */
   data tmpl; set _stdm(keep = USUBJID LBTEST);
      where trim(left(LBTEST)) = "&ALTLab";
      rename USUBJID = usubtmp;
   run;
   proc sql;
      create table alt as
         select *
         from &in_adsl d1
            left join
            tmpl d2
         on d1.USUBJID=d2.usubtmp
         where missing(d2.LBTEST);
   quit;
   data alt; length Missing_Test $25;
      set alt;
      keep USUBJID &DesignByVars &TrtVar &DMKeepVars Missing_Test;
      Missing_Test = "&ALTLab";
   run;
   /* ALP */
   data tmpl; set _stdm(keep = USUBJID LBTEST);
      where trim(left(LBTEST)) = "&ALPLab";
      rename USUBJID = usubtmp;
   run;
   proc sql;
      create table alp as
         select *
         from &in_adsl d1
            left join
            tmpl d2
         on d1.USUBJID=d2.usubtmp
         where missing(d2.LBTEST);
   quit;
   data alp; length Missing_Test $25;
      set alp;
      keep USUBJID &DesignByVars &TrtVar &DMKeepVars Missing_Test;
      Missing_Test = "&ALPLab";
   run;
   /* AST */
   data tmpl; set _stdm(keep = USUBJID LBTEST);
      where trim(left(LBTEST)) = "&ASTLab";
      rename USUBJID = usubtmp;
   run;
   proc sql;
      create table ast as
         select *
         from &in_adsl d1
            left join
            tmpl d2
         on d1.USUBJID=d2.usubtmp
         where missing(d2.LBTEST);
   quit;
   
   data ast; length Missing_Test $25;
      set ast;
      keep USUBJID &DesignByVars &TrtVar &DMKeepVars Missing_Test;
      Missing_Test = "&ASTLab";
   run;  
   /* TBL */
   data tmpl; set _stdm(keep = USUBJID LBTEST);
      where trim(left(LBTEST)) = "&TBLLab";
      rename USUBJID = usubtmp;
   run;
   proc sql;
      create table tbl as
         select *
         from &in_adsl d1
            left join
            tmpl d2
         on d1.USUBJID=d2.usubtmp
         where missing(d2.LBTEST);
   quit;
   data tbl; length Missing_Test $25;
      set tbl;
      keep USUBJID &DesignByVars &TrtVar &DMKeepVars Missing_Test;
      Missing_Test = "&TBLLab";
   run;     
   
   data Outlib.&OutDataName2; 
      label Missing_Test = "&trMLT";
      set alt alp ast tbl;
      %if %Upcase(&MType) = C %then %do;
         if missing(&TrtVar) then &TrtVar = "Missing";
      %end;
      %else %do;
         if missing(&TrtVar) then &TrtVar = -99;
      %end;
   run;
   
   %nobsvars(Outlib.&OutDataName2);
   %let misslabs = 0;
   %let Nallmiss = 0;
   %if &nobs > 0 %then %do;
      %let misslabs = 1;
      proc freq noprint data = Outlib.&OutDataName2;
         tables USUBJID / out = mall(where=(Count=4));
      run;
      
      %nobsvars(mall);
      %let Nallmiss = &nobs;
   %end;
   
   
%let odata = OutLib.&OutDataName;
%include "&MacroPath./DesignByVars.sas";

/* Post-translation of "Peak XXX/YYY" strings in output data sets and JSL script (S0994766 (JC)) */
%if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) ^= EN %then %do;
   %let trPeakX = %nrbquote(%LocaleGet(PeakX_Text));
   %let trULN = %nrbquote(%LocaleGet(ULN_Text));
   %let trBaseline = %nrbquote(%LocaleGet(Baseline_Text));
   %macro PostTrans(_stringvar);
      %let &_stringvar = %nrbquote(%sysfunc(prxchange(s/Peak //, 1, %nrbquote(&&&_stringvar))));
      %let &_stringvar = %nrbquote(%sysfunc(tranwrd(%nrbquote(&&&_stringvar), %str(/ULN), %str(/)%nrbquote(&trULN))));
      %let &_stringvar = %nrbquote(%sysfunc(tranwrd(%nrbquote(&&&_stringvar), %str(/Baseline), %str(/)%nrbquote(&trBaseline))));
      %let &_stringvar = %nrbquote(%LocaleFill(%nrbquote(&trPeakX), f1=%nrbquote(&&&_stringvar)));
   %mend;
   %let Done = 0;
   %do %while (%eval(&Done) < 2);
      %if %eval(&Done) = 0 %then %let ODN = &OutDataName;
      %else %if %eval(&Done) = 1 %then %let ODN = &OutDataName.gtt;
      %let dsid = %sysfunc(open(OutLib.&ODN));
      %let nVars = %sysfunc(attrn(&dsid,NVARS));
      %do i = 1 %to &nVars;
         %let origname = %sysfunc(varname(&dsid, &i));
         %let origlabel = %nrbquote(%sysfunc(varlabel(&dsid, &i)));
         %let newlabel = &origlabel;
         %if %eval(%length(&origname)) > 5 and %eval(%klength(&origlabel)) > 5 %then %do;
            %if %upcase(%substr(&origname,1,5)) = %str(PEAK_) and %nrbquote(%ksubstr(&origlabel,1,5)) = %str(Peak ) %then %do;
               %PostTrans(newlabel);
               %let dsid = %sysfunc(close(&dsid));
               data OutLib.&ODN; set OutLib.&ODN;
                  label &origname = "&newlabel";
               run;
               %let dsid = %sysfunc(open(OutLib.&ODN));
            %end;
         %end;
      %end;
      %let dsid = %sysfunc(close(&dsid));
      %let Done = %eval(&Done + 1);
   %end;
   data OutLib.&OutDataName2; set OutLib.&OutDataName2;
      call symputx("newvalue", Missing_Test);
      call execute('%PostTrans(newvalue)');
      Missing_Test = symget('newvalue');
   run;
   %PostTrans(TBLLab);
   %PostTrans(ALTLab);
   %PostTrans(ASTLab);
   %PostTrans(ALPLab);
%end;
/* End of Post-translation */


/* translations for table names */
%let trHyLawScreeningTable = %nrbquote(%LocaleGet(HysLawScreeningTable_Text));
%let trStudyDayTable = %nrbquote(%LocaleGet(TestsbyStudyDayTable_Text));
%let trMissingTestsTable = %nrbquote(%LocaleGet(MissingTestsTable_Text));

* create JSL file;
%PrepOutFile(HysLawScreeningScript,HysLawScreeningScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &HysLawScreeningScript;
%let FullAPName=%str(Hy%'s Law Screening);

* JSL statements to produce JMP report;
data _null_;
   file "&HysLawScreeningScript" lrecl=32767 %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newNamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:ActionDataTableList = {};";
   put "objRefNS:NoteDomain = ""LB"";";   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   
   
   put "objRefNS:fsize = 200;";
   put "if(hostis(windows), _slash_ = ""\"", _slash_ = ""/"");";
   put "if(Host Is(Windows),";
   put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME\Clinical\"", absolute, windows),";
   put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME/Clinical/"", absolute, POSIX)";
   put ");";

   put "objRefNS:sdtmFolder = ""&InPath"";"; 
   put "objRefNS:refvar = ""&ReferenceVar"";";
   put " objRefNS:openhld = Expr(";
   put " If(IsEmpty(objRefNS:hld),";
   put "    objRefNS:hld = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   put "    Column(""&SubjectLabel"") << Label(1);";
   put "    Column(""&trALTCDE"")<<Set Property(""Value Ordering""," @;
   put "{""&Mtext"",""&C1text"",""&C2text"",""&C2text"",""&C3text"",""&C4text"",""&C5text""} " @;
   put ");";
   put "    Column(""&trASTCDE"")<<Set Property(""Value Ordering""," @;
   put "{""&Mtext"",""&C1text"",""&C2text"",""&C2text"",""&C3text"",""&C4text"",""&C5text""} " @;
   put ");";
   put "    Column(""&trALPCDE"")<<Set Property(""Value Ordering""," @;
   put "{""&Mtext"",""&C1text"",""&C2text"",""&C2text"",""&C3text"",""&C4text"",""&C5text""} " @;
   put ");";
   put "    Column(""&trTBLCDE"")<<Set Property(""Value Ordering""," @;
   put "{""&Mtext"",""&C1text"",""&C2text"",""&C2text"",""&C3text"",""&C4text"",""&C5text""} " @;
   put ");";   
   put "    objRefNS:dbv = {};";
   %do i=1 %to &ndbv;
      put "    InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
   %end; 
   put "  ,";  /*else for isempty*/;
   put "  Current Data Table(objRefNS:hld);";
   put " );";
   put " );";
   %let datalist = ""objRefNS:hld"";
   put " objRefNS:openhld;";
   
	put "objRefNS:edfunc = Function({}, {default local},";
	put " objRefNS:openhld;";
	put "		HListBox(";
	put "		objRefNS:Filter= objRefNS:hld << Data Filter(<<Conditional(1),";
	put "           Add Filter(";
	put "            Columns(";
	%if &numhys %then %do;
		put "          :Name(""&HLILabel""),";
		put " //       :Name(""&FHLCLabel""),";
	%end;
	%if &numat %then %do;
		put "          :Name(""&ATELabel""),";
		put "//           :Name(""&FATELabel""),";
	%end;
	put "       :Name( ""&TBLLab"" ),";
	put "       :Name( ""&ALTLab"" ),";
	put "       :Name( ""&ASTLab"" ),";
	put "       :Name( ""&ALPLab"" ),"; 
	%if &doStudy %then %do;
			put ":Name(""&StudyLabel""),";
   %end;  
	%if &doAge %then %do;
		put " :Name(""&AgeLabel""),";
	%end;
	%if &doSex %then %do;
		put " :Name(""&SexLabel""),";
	%end;
	%if &doRace %then %do;
		put " :Name(""&RaceLabel""), ";
	%end;
	%if &doSite %then %do;
		put "       :Name(""&SiteLabel""),";
	%end;
	put "       :Name(""&TrtLabel""),"; 	
	put "            )";
	put "         ),";
	put "          Mode( Show(1), Include(0) )";
	put "    ),";
	put "    VListBox(";
	put "    OutlineBox(locale:get(""LiverTestElevationsPlot_Text""),";
	%if &numhys %then %do;
		put "    objRefNS:tbm =TextBox(""     "" || locale:fill(locale:get(""HysLawCFlgifALTASTgeXxXandBILIgeXxXwiXdALTASTpk_Text""), {""&ARefLine"", ""&ReferenceVar"", ""&BRefLine"", ""&ReferenceVar"", ""&HyLag""}), <<Set Wrap(625),<<Set Font Size(9)),";
	%end;
	%else %do;
		put "    objRefNS:tbm =TextBox(""     "" || locale:fill(locale:get(""NoHysLawCFndALTASTgeXxXandBILIgeXxXwiXdALTASTpk_Text""), {""&ARefLine"", ""&ReferenceVar"", ""&BRefLine"", ""&ReferenceVar"", ""&HyLag""}), <<Set Wrap(625),<<Set Font Size(9)),";
	%end;    
	put "       objRefNS:op = Overlay Plot(";
	put "          X(:Name(""&ALTLab"")),";
	put "          Y(:Name(""&TBLLab"")),";
	%if %upcase(&SplitResults)=YES %then %do;
		put "         By(eval(objRefNS:dbv)),";
	%end;    
	put "          Sort X(0),";
	%if %upcase(&DisplayLogShft) = YES %then %do;
		put "          Left Y Log Scale(1), X Log Scale(1),";
	%end;
	put "          SendToReport(";
	put "             Dispatch({},""106"",ScaleBox,{Minor Ticks(1), Inc(1), Add Ref Line( &HyLaw_TBL, Dashed, ""Medium Dark Gray"" )}),";
	put "             Dispatch({}, ""101"", ScaleBox, {Minor Ticks(1), Inc(1), Add Ref Line( &HyLaw_ALT, Dashed, ""Medium Dark Gray""  )} ),";
	put "             Dispatch({}, ""Overlay Plot Graph"", FrameBox,";
	put "                {Frame Size(460,350), Marker Drawing Mode(Normal), Marker Size(2),";
	put "                Row Legend( Column(""&HLILabel""), ";
	put "                   Color(1), Marker(0), Continuous Scale(0)";
	put "                ),";
	put "                Row Legend( Column(""&TrtLabel""), ";
	put "                   Color(0), Marker(1), Marker Theme(""Solid"")";
	put "                )";
	%if %Upcase(&HyLabel) = YES %then %do;
			put "          ,{Add Text Annotation(Text( locale:get(""Cholestasis_Text"") ),";
			put "             Text Box( {13, 10, 120, 35}  ),";
			put "             Filled( 0 ),Editable(0),Text Color( ""Medium Dark Gray"" ),Font( ""Arial"", 10, ""Plain"" )),";
			put "          Add Text Annotation(Text( locale:get(""HysLaw_Text"") ),";
			put "          Text Box( {365, 10, 500, 35} ),";
			put "          Filled( 0 ),Editable(0),Text Color( ""Medium Dark Gray"" ),Font( ""Arial"", 10, ""Plain"" )),";
			put "       Add Text Annotation(Text( locale:get(""TemplesCorollary_Text"") ),";
			put "          Text Box( {320, 330, 455, 345} ),";
			put "          Filled( 0 ),Editable(0),Text Color( ""Medium Dark Gray"" ),Font( ""Arial"", 10, ""Plain"" )";
			put "       )}";
	%end;    
	put "             })";
	put "          )";
	put "    )),";  /* Close objRefNS:op and outlinebox*/   
	put "	OutlineBox(locale:get(""ScatterplotsandContingencyAnalysis_Text""),";
	put " HListBox(";
	put " VListBox(";
	put "    objRefNS:sp = Scatterplot Matrix(";
	put "      Y( ";
	put "       :Name( ""&ALTLab"" ),";
	put "       :Name( ""&ASTLab"" ),";
	put "       :Name( ""&ALPLab"" ),";
	put "       :Name( ""&TBLLab"" ),";
	put "      ),";
	%if %upcase(&SplitResults)=YES %then %do;
		put "         By(eval(objRefNS:dbv)),";
	%end;
	put "      Matrix Format(""Lower Triangular""),";
	put "      SendToReport(";
	put "       Dispatch({}, ""103"", ScaleBox,";
	put "          {";
	%if %UPCASE(&DisplayLogShft) = YES %then %do;
		put "       Scale(""Log""),Minor Ticks(1), Inc(1), Min(&LabMin4),";
	%end;
	%if %sysevalf(&LabMax4 < &TBL_max) or %UPCASE(&DisplayLogShft) = YES %then %do;
		put "         Max(&TBL_max), ";
	%end;
	put "             Add Ref Line( &HyLaw_TBL, Dashed, ""Medium Dark Gray"" )}";
	put "       ),";
	put "       Dispatch({}, ""102"", ScaleBox,";
	put "          {Format( ""Best"", 10 ),";
	%if %UPCASE(&DisplayLogShft) = YES %then %do;
		put "       Scale(""Log""),Minor Ticks(1), Inc(1),  Min(&LabMin1),";
	%end; 
	%if %sysevalf(&LabMax1 < &ALP_max) or %UPCASE(&DisplayLogShft) = YES %then %do;
			put "         Max(&ALP_max), ";
	%end;
	put "             Add Ref Line( &HyLaw_ALP, Dashed, ""Medium Dark Gray"" )}";
	put "       ),";
	put "       Dispatch({}, ""101"", ScaleBox,";
	put "          {";
	%if %UPCASE(&DisplayLogShft) = YES %then %do;
			put "       Scale(""Log""), Minor Ticks(1), Inc(1),  Min(&LabMin3),";
	%end;
	%if %sysevalf(&LabMax3 < &ALT_max) or %UPCASE(&DisplayLogShft) = YES %then %do;
			put "         Max(&ALT_max), ";
	%end;
	put "             Add Ref Line( &HyLaw_ALT, Dashed, ""Medium Dark Gray"" )}";
	put "       ),";  
	put "       Dispatch({}, ""100"", ScaleBox,";
	put "          {";
	%if %UPCASE(&DisplayLogShft) = YES %then %do;
		put "       Scale(""Log""),Minor Ticks(1), Inc(1),  Min(&LabMin2),";
	%end; 
	%if %sysevalf(&LabMax2 < &ALT_max) or %UPCASE(&DisplayLogShft) = YES %then %do;
		put "         Max(&ALT_max), ";
	%end;
	put"           Add Ref Line( &HyLaw_ALT, Dashed, ""Medium Dark Gray"" )}";
	put "       ),";  
	%do j=1 %to 6; 
		put "       Dispatch({}, ""Scatterplot Matrix Plot"",";
		put "          FrameBox(&j),";
		put "          {Frame Size(200,200), Marker Drawing Mode(Normal), Marker Size(2)";
		%if &j=3 %then %do;
			put "          , Row Legend( Column(""&HLILabel""), ";
			put "             Color(1), Marker(0), Continuous Scale(0)";
			put "            )";
			put "          , Row Legend( Column(""&TrtLabel""), ";
			put "             Color(0), Marker(1), Marker Theme(""Solid"")";
			put "            )";       
		%end;
		%if %Upcase(&HyLabel) = YES %then %do;
			%if &j=4 %then %do;
				put "          ,{Add Text Annotation(Text( locale:get(""Cholestasis_Text"") ),";
				put "             Text Box( {1, 1, 112, 26}  ),";
				put "             Filled( 0 ),Editable(0),Text Color( ""Medium Dark Gray"" ),Font( ""Arial"", 9, ""Plain"" )),";
				put "          Add Text Annotation(Text( locale:get(""HysLaw_Text"") ),";
				put "          Text Box( {148, 0, 257, 25} ),";
				put "          Filled( 0 ),Editable(0),Text Color( ""Medium Dark Gray"" ),Font( ""Arial"", 9, ""Plain"" )),";
				put "       Add Text Annotation(Text( locale:get(""TemplesCorollary_Text"") ),";
				put "          Text Box( {118, 207, 249, 225} ),";
				put "          Filled( 0 ),Editable(0),Text Color( ""Medium Dark Gray"" ),Font( ""Arial"", 9, ""Plain"" )";
				put "       )}";
			%end;
		%end;
		put"           }";
		put "       ),";
	%end;
	put "      )";
	put "    );";
	%if %symexist(mlchkhi) %then %do;
		%if  &mlchkhi %then %do;
			put "    objRefNS:mlnw = New Window(locale:fill(locale:get(""XWarningMessage_Text""), {locale:get(""HysLaw_Text"")}),showmenu(0),showtoolbars(0),"; 
			put "       TextBox(locale:fill(locale:get(""WarningMissValsHysLawLBSTNRHIMissZeroLBTEXTCDX_Text""), {""%trim(&chkuln)""}),";      
			put "          <<Set Wrap(600), << Font Color( red ), Set Font Size(11)";
			put "    ));"; 
			put " InsertInto(objRefNS:TabWinList,0); ";
			put " objRefNS:TabWinList[objRefNS:nwcounter]=objRefNS:mlnw; ";
			put " objRefNS:nwcounter++; ";         
		%end;
	%end; 
	put "),"; /* Close VListBox */
	%if &numat %then %do;
		put " VListBox(";
		put "    objRefNS:atc = Contingency(";
		put "       Y( :Name(""&ATELabel"")),";
		put "       X( :Name(""&TrtLabel"")),";
		%if %upcase(&SplitResults)=YES %then %do;
				put "         By(eval(objRefNS:dbv)),";
		%end;
		put "       Odds Ratio(1),";
		put "       SendToReport( Dispatch( {}, ""Tests"", OutlineBox, {Close( 1 )} ) )";
		put " )),";
	%end; 
	%if &numhys %then %do;
		put " VListBox(";
		put "    objRefNS:hdc = Contingency(";
		put "       Y( :Name(""&HLDLabel"")),";
		put "       X( :Name(""&TrtLabel"")),";
		%if %upcase(&SplitResults)=YES %then %do;
				put "         By(eval(objRefNS:dbv)),";
		%end;
		put "       Tests( 0 ), Contingency Table( 0 ), ";
		put " );";
		put " //(objRefNS:hdc<<report)<<Close;";
		put ")";
	%end; 
   put "))"; /* close HlistBox and OutlineBox ScatterPlots*/
   %if &numat %then %do;
   	put " ,HListBox(";
      put " objRefNS:dd = Distribution(";
      %if &numhys %then %do;
         put "    Nominal Distribution( Column( :Name(""&HLILabel"") ) ),";
      %end;
      put "    Nominal Distribution( Column( :Name(""&ATELabel"") ) ),";
      %if &numhys %then %do;
         put "    Continuous Distribution( Column( :Name(""&FHLCLabel"") ), Quantiles( 0 ) ),";
      %end;
      put "    Continuous Distribution( Column( :Name(""&FATELabel"") ), Quantiles( 0 ) ),";
      put "    Nominal Distribution(Column( :Name(""&TrtLabel"") )),";
      %if &doAge %then %do;
         put "    Continuous Distribution( Column( :Name(""&AgeLabel"") ), Quantiles( 0 ) ),";
      %end;
      %if &doSex %then %do;
         put "    Nominal Distribution( Column( :Name(""&SexLabel"") )),";
      %end;
      %if &doRace %then %do;
         put "    Nominal Distribution( Column( :Name(""&RaceLabel"") ) ),";
      %end;
      %if &doSite %then %do;
         put "    Nominal Distribution( Column( :Name(""&SiteLabel"") ), Frequencies( 0 )),";
      %end;
      put " ))";   
    %end;
   %if  %upcase(&ThreeDPlot) = YES %then %do;
		put "    ,objRefNS:sp3 = Scatterplot 3D(";
		put "      Y( ";
		put "       :Name( ""&TBLLab"" ),";
		put "       :Name( ""&ALTLab"" ),";
		put "       :Name( ""&ASTLab"" ),";
		put "       :Name( ""&ALPLab"" ),";
		put "      ),";
		%if %upcase(&SplitResults)=YES %then %do;
			put "         By(eval(objRefNS:dbv)),";
		%end;
		put "       Frame3D(";
		put "          Set Graph Size(650,575),";
		put "          Set Marker Transparency(0.95),";
		put "          Set Marker Quality(1),";
		put "          Set Marker Scale(2.75)";
		put "       )";
		put "    );";  
   %end;   
	put ")));"; /* close vlistbox hlistbox and function*/

   put "objRefNS:spafterfunc=Function({},{default local},"; 
      put "  objRefNS:sprep = objRefNS:sp<<report;";
      %let LabelList = "&ASTLab ** &ALPLab ** &TBLLab ** &ALTLab ** &ASTLab ** &ALPLab";
      %if %upcase(&SplitResults)=YES %then %do;
            %do i = 1 %to &TotLvls;
               %do j=1 %to 6;
                  %let Clabel = %scan("&LabelList",&j,"**");
                  put "  objRefNS:sprep[&i][AxisBox(&j)]<<RemoveAxisLabel()<<RemoveAxisLabel();";
                  put "  objRefNS:sprep[&i][AxisBox(&j)]<<AddAxisLabel(""&Clabel"");";
               %end;
            %end;
      %end;
      %else %do;
         %do j=1 %to 6;
            %let Clabel = %scan("&LabelList",&j,"**");
            put "  objRefNS:sprep[1][AxisBox(&j)]<<RemoveAxisLabel()<<RemoveAxisLabel();";
            put "  objRefNS:sprep[1][AxisBox(&j)]<<AddAxisLabel(""&Clabel"");";
         %end;    
      %end; 
      put "    try(objRefNS:sprep <<Sib Prepend(";
      put "       HListBox(TextBox("" "" || locale:get(""FrameSize_Text"")),";
         put "          SliderBox(num(100),num(450),objRefNS:fsize,";
         %if %upcase(&SplitResults)=YES %then %do;
            %do j=1 %to %eval(&TotLvls);
               put "          try(objRefNS:sprep[&j][framebox(1)]<<FrameSize(objRefNS:fsize,objRefNS:fsize));";
            %end;
         %end;
         %else %do;
            put "          objRefNS:sprep[1][framebox(1)]<<FrameSize(objRefNS:fsize,objRefNS:fsize);";
         %end;
      put "       ),";
      %if &numhys %then %do;
         put "    objRefNS:tbm2 =TextBox(""     "" || locale:fill(locale:get(""HysLawCFlgifALTASTgeXxXandBILIgeXxXwiXdALTASTpk_Text""), {""&ARefLine"", ""&ReferenceVar"", ""&BRefLine"", ""&ReferenceVar"", ""&HyLag""}), <<Set Wrap(600),<<Set Font Size(9)),";
      %end;
      %else %do;
         put "    objRefNS:tbm2 =TextBox(""     "" || locale:fill(locale:get(""NoHysLawCFndALTASTgeXxXandBILIgeXxXwiXdALTASTpk_Text""), {""&ARefLine"", ""&ReferenceVar"", ""&BRefLine"", ""&ReferenceVar"", ""&HyLag""}), <<Set Wrap(650),<<Set Font Size(9)),";
      %end;
      put " )));";
   put ");";
   
	put 'InsertInto(objRefNS:DataList,{{"objRefNS:hld"}});';
	put "InsertInto(objRefNS:DataNameList,{{locale:get(""HysLawScreening_Text"")}});";
	put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""HysLawScreening_Text"")));";
	put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:edfunc()));';
	put 'InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:spafterfunc()));';
	put 'InsertInto(objRefNS:TabOpenList,1);';
	put "InsertInto(objRefNS:CreateReportList,{{objRefNS:tbm,objRefNS:op,objRefNS:tbm,objRefNS:sp ";
   %if &numat %then %do;
      put ",objRefNS:atc,objRefNS:dd";
   %end;
   
   %if %upcase(&ThreeDPlot) = YES %then %do;
   	put ",objRefNS:sp3";
   %end;
   put "}});";

   /* kemicl added tables of elevations v4.1 */
   %let TestNames = &ALTCD &ASTCD &TBLCD &ALPCD;
   %let CTRlistexpr = %nrbquote();
   put "objRefNS:tbfunc = Function({}, {default local},";
   put " objRefNS:openhld;";
   put "  VListBox(";
   put "  DataFilterContextBox(";
   put "    HListBox(";
   put " Current Data Table()<< Data Filter(<<Conditional(1),Local,";
   put "    Add Filter(columns(";
   put "       :Name(""&HLILabel""),:Name(""&ATELabel""),:Name(""&ALTLab""),:Name(""&ASTLab""),:Name(""&TBLLab""),:Name(""&ALPLab""),";
   put "       :Name(""&AgeLabel""), :Name(""&SexLabel""), :Name(""&RaceLabel""), :Name(""&SiteLabel"")"; 
   put "    )),Mode( Select( 0 ), Show( 1 ), Include( 1 ) )";
   put "  );";
   put " TextBox(""     ""),  //spacer between tables and data filter ";
   put " VListBox(";
   %if &numat|&numhys %then %do;
   put "       Platform(Current Data Table(),";
   put "          objRefNS:tbhys = Eval(Parse(EvalInsert(""\[Tabulate(";
   put "             Change Item Label(Statistics(N, ^locale:get(""SubjectCount_Text"")^),Statistics(Column %, ""^locale:get(""PCTofSubjects_Text"")^"")),";
   put "             Set Format(Column %(9,1)),";
   put "             Show Control Panel(0),";
   put "             Add Table(";
   put "                Column Table(Grouping Columns(:Name(""&TrtLabel"")), Statistics(N, Column %)),";
   put "                Row Table(Grouping Columns(:Name(""&ATELabel""),:Name(""&HLILabel"")))";
   put "             ),";
   put "          SendToReport(Dispatch({},""Tabulate"",OutlineBox,{Set Title( locale:get(""ATTestElevationandPossibleHysLawCases_Text"") )}))";      
   put "          )]\"")));";
   put "       ),";
   put "       (objRefNS:tbhys<<report)[OutlineBox(1)]<<Append(TextBox(""   "" || locale:fill(locale:get(""XdefinedasALTorASTgeXxX_Text""), {""&ATELabel"", ""&ARefLine"", ""&ReferenceVar""}),<<Set Wrap(800),<<Set Font Size(9)));"; 
   put "       (objRefNS:tbhys<<report)[OutlineBox(1)]<<Append(TextBox(""   "" || locale:fill(locale:get(""XdefinedasALTorASTgeXxXandBILIgeXxXwiXdALTASTpk_Text""),";
   put "          {""&HLILabel"", ""&ARefLine"", ""&ReferenceVar"", ""&BRefLine"", ""&ReferenceVar"", ""&HyLag""}) || ""\!N\!N"",<<Set Wrap(800),<<Set Font Size(9)));";
   %let CTRlistexpr = %nrbquote(objRefNS:tbhys);
   %end;
   %let i=1;
   %do %while(%length(%scan(&TestNames,&i,' ')));
      %let curLT = %scan(&TestNames,&i,' ');
      %let trcurLTE = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XElevations_Text)), &curLT));
      put "       Platform(Current Data Table(),";
      put "          objRefNS:tb&i = Eval(Parse(EvalInsert(""\[Tabulate(";
      put "             Change Item Label(Statistics(N, ^locale:get(""SubjectCount_Text"")^),Statistics(Column %, ""^locale:get(""PCTofSubjects_Text"")^"")),";
      put "             Set Format(Column %(9,1)),";
      put "             Show Control Panel(0),";
      put "             Add Table(";
      put "                Column Table(Grouping Columns(:Name(""&TrtLabel"")), Statistics(N, Column %)),";    
      put "                Row Table(Grouping Columns(:Name(""&trcurLTE"")), Add Aggregate Statistics(:Name(""&trcurLTE"")))";
      put "             ),";
      put "          SendToReport(Dispatch({},""Tabulate"",OutlineBox,{Set Title( locale:fill(locale:get(""XCountsandPercents_Text""), {""&curLT""}) )}))";    
      put "          )]\"")));";
      put "       ),";
      %let CTRlistexpr = &CTRlistexpr.%nrbquote(,objRefNS:tb&i);
      %let i = %eval(&i+1);
   %end;
   put " ) //vlistbox of platforms ";
   put ")))); //end hlist datafcontext vlist and function";
   
   put 'InsertInto(objRefNS:DataList,{{"objRefNS:hld"}});';
   put "InsertInto(objRefNS:DataNameList,{{locale:get(""HysLawScreening_Text"")}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""Tables_Text"")));";
   put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:tbfunc()));';
   put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
   put 'InsertInto(objRefNS:TabOpenList,1);';
   put "InsertInto(objRefNS:CreateReportList,{{&CTRlistexpr}});";
   
   %let datalist = &datalist,""objRefNS:hltt"";
   /* kemicl added tests by study day plot for v4.1 */
   put "objRefNS:pbdfunc = Function({}, {default local},";
   put " If(IsEmpty(objRefNS:hltt),";
   put "    objRefNS:hltt = open(""&ClientOutPath.&OutDataName.gtt.sas7bdat"",invisible);";
   put "    Column(""&SubjectLabel"") << Label(1);";
   put "    Column(""&HLILabel"")<< Set Property(""Value Ordering"",{""&trYes"",""&trNo""});";
   %if &numat %then %do;
      put "    Column(""&ATELabel"")<< Set Property(""Value Ordering"",{""&trYes"",""&trNo""});";
   %end;
   put "    Column(""&HLILabel"")<<Set Property(""Value Colors"",{""&trYes""=1,""&trNo""=21});";
   put "    objRefNS:dbv = {};";
   %do i=1 %to &ndbv;
      put "    InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
   %end; 
   put "  ,";  /*else for isempty*/;
   put "  Current Data Table(objRefNS:hltt);";
   put " );";
   put " HListBox(";
   put "   VListBox(";
   put "    objRefNS:Filter2=objRefNS:hltt << Data Filter(<<Conditional(1),";
   put "           Add Filter(";
   put "            Columns(";
   put "             :Name( ""&DYLabel"" ),";
   put "             :Name(""&TrtLabel""), :Name(""&AgeLabel""), :Name(""&SexLabel""), :Name(""&RaceLabel"")";
   put "            )";
   put "         ),";
   put "          Mode( Show(1), Include(1) )";
   put "       )";
   put "    ),";         
   put "    objRefNS:gbps = Graph Builder(Size(675,575), Show Control Panel(0),";
   /* put "       Lock Scales(1),"; */
   put "       Variables(";
   put "          X(:Name(""&DYLabel"")),";
   put "          Y(:Name(""&ALPLab"")),";
   put "          Y(:Name(""&ASTLab"")),";
   put "          Y(:Name(""&ALTLab"")),";
   put "          Y(:Name(""&TBLLab"")),";
   put "          Color(:Name(""&TrtLabel"")),";
   put "       ),";
   put "       Elements(Position(1,1), Points(X,Y)),Elements(Position(1,2), Points(X,Y)),Elements(Position(1,3), Points(X,Y)),Elements(Position(1,4), Points(X,Y)),";
   put "       SendToReport(";
   put "          Dispatch({},""Graph Builder"",OutlineBox,{Set Title(locale:get(""PeakLiverLabTestsperSubjectbyStudyDay_Text""))}),";
   put "          Dispatch({},""graph title"", TextEditBox,{Set Text("" "")}),";
   put "          Dispatch({},""&ALPLab"", ScaleBox,{Add Ref Line(&HyLaw_ALT, Dashed, ""Medium Dark Gray"")}),";
   put "          Dispatch({},""&ALTLab"", ScaleBox,{Add Ref Line(&HyLaw_ALT, Dashed, ""Medium Dark Gray"")}),";
   put "          Dispatch({},""&ASTLab"", ScaleBox,{Add Ref Line(&HyLaw_ALT, Dashed, ""Medium Dark Gray"")}),";
   put "          Dispatch({},""&TBLLab"", ScaleBox,{Add Ref Line(&HyLaw_TBL, Dashed, ""Medium Dark Gray"")})";
   put "       )";
   put "    );";
   put "    gbr = objRefNS:gbps<<report;";
   put "    try(gbr[axisbox(2)]<<Show Minor Ticks(0); gbr[axisbox(3)]<<Show Minor Ticks(0); gbr[axisbox(4)]<<Show Minor Ticks(0); gbr[axisbox(5)]<<Show Minor Ticks(0););";
   put "));";

   put 'InsertInto(objRefNS:DataList,{{"objRefNS:hltt"}});';
	put "InsertInto(objRefNS:DataNameList,{{locale:get(""TestsbyStudyDay_Text"")}});";  
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""TestsbyStudyDay_Text"")));";
   put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:pbdfunc()));';
   put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
   put 'InsertInto(objRefNS:TabOpenList,1);';
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:gbps}});";   
   
   %if &misslabs %then %do;
      %let datalist = &datalist,""objRefNS:hldm"";
      put "objRefNS:mlfunc=Function({},{default local},";
         put " If(IsEmpty(objRefNS:hldm),";
         put "    objRefNS:hldm= open(""&ClientOutPath.&OutDataName2..sas7bdat"",invisible);";
         put "    Column(""&SubjectLabel"") << Label(1);";  
         put "    objRefNS:dbv = {:Name(""&TrtLabel"")};";
         %do i=1 %to &ndbv;
            put "    InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
         %end; 
         put "  ,";  /*else for isempty*/;
         put "  Current Data Table(objRefNS:hldm);";
         put " );";
      put " VListBox(";
      put "    TextBox(locale:get(""CountsOfSubjsNoRecsOrMissMeasLLTDisplayedBelow_Text""),";
      put "        <<Font Color(5), <<Set Font Size(11), <<Set Wrap( 550 )";
      put "    ),";
      put "    TextBox("" ""),";
      put "    objRefNS:MLT = Tabulate(";
      put "       Show Control Panel(0),";
      put "       Add Table(";
      put "          Column Table(Grouping Columns(eval(objRefNS:dbv))),";
      put "          Row Table( Grouping Columns(:Name(""&trMLT"")))";
      put "       ),";
      put "    SendToReport(Dispatch({},""Tabulate"",OutlineBox,{Set Title( locale:get(""CountsofSubjectsMissingLiverLabRecords_Text"") )}))";     
      put "    )";
      %if Nallmiss > 0 %then %do;
         put "       ,(objRefNS:MLT<<report)[OutlineBox(1)]<<Append(TextBox(""   "" || locale:fill(locale:get(""NoteXSubjectsHaveNoRecOrMissMeasOfAllFourLLT_Text""), {""&Nallmiss""}),<<Set Wrap(800),<<Set Font Size(9)));";         
      %end;
      put " ));";
      put 'InsertInto(objRefNS:DataList,{{"objRefNS:hldm"}});';
      put "InsertInto(objRefNS:DataNameList,{{locale:get(""MissingLabTestSummary_Text"")}});";  
   %end;
   %else %do;
      put "objRefNS:mlfunc=Function({},{default local},";
      put " VListBox(";
      put "    objRefNS:MLT = TextBox(locale:get(""AllSubjsHaveAtLeastOneRecMeasHysLawLabTests_Text""),";
      put "        <<Font Color(5), <<Set Font Size(12), <<Set Wrap( 550 )";
      put "    ),";
      put " ));";
      put 'InsertInto(objRefNS:DataList,{{""}});';
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
   %end;
   
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""MissingLabTestSummary_Text"")));";
   put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:mlfunc()));';
   put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
   put 'InsertInto(objRefNS:TabOpenList,1);';
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:MLT}});";
   %SubjectDrillDowns;
   

   put "Include(""$CLINICAL_HOME\JSLFiles\Findings Drill Down Expressions.jsl"");";
   put "Include(""$LS_HOME\JSLFiles\DrilldownReport.jsl"");";
   put "objRefNS:gttdd =  Function({},{},";
   put "   objRefNS:SelectSubjects;";
   put "   if(nsel,";
   put "    objRefNS:GraphTimeTrendsHys(""&ClientOutPath.&OutDataName.gtt.sas7bdat"",""&DYLabel"",""&TBLLab"",""&ALTLab"",""&ASTLab"",""&ALPLab"",""&TrtLabel"", &ExUse,""&DisplayLogShft"",""&HyLaw_TBL"", ""&HyLaw_ALT""));";
   put ");";
   put "objRefNS:tdd =  Function({},{},";
   put "   objRefNS:SelectSubjects;";
   put "   if(nsel,";
   put "    objRefNS:GraphTrellisHys(""&ClientOutPath.&OutDataName.gtt.sas7bdat"",""&DYLabel"",""&TBLLab"",""&ALTLab"",""&ASTLab"",""&ALPLab""));";
   put ");";   
   
   put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""GraphTimeProfiles_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:gttdd()));";
   put "InsertInto(objRefNS:ActionSelectionCountList[1],1);";
   put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.LinesPlot.gif"");";
   
   /*
   put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""GraphTrellisPlot_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:tdd()));";  
   put "InsertInto(objRefNS:ActionSelectionCountList[1],1);";      
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.Trellis.gif"");";
   */
	put "objRefNS:ActionDataTableList = {{{&datalist},{&datalist},{&datalist},{&datalist},{""objRefNS:hld"",""objRefNS:hltt""}}};";

	put "InsertInto(objRefNS:ActionButtonDesc , {Expr(locale:get(""ClicktoView_Text""))});";;
	put "objRefNS:ActionButtonList[2]={};";
	put "objRefNS:ActionCodeList[2] ={};";
	put "objRefNS:ActionIconList[2] = {};";
	put "objRefNS:ActionRequirementFlagList[2] = {};";
	put "objRefNS:ActionSelectionCountList[2] = {};";
	put "objRefNS:ActionDataTableList[2] = {{&datalist}};";
	
   %let HysWhere = LBTESTCD in ("&ALTCD","&ASTCD","&ALPCD","&TBLCD");
   %put HysWhere = &HysWhere;
   put "objRefNS:splotsDDFunc=evalexpr(Function({},{default local},";
  	put "		this = namespace(expr(objRefNS<<getName)); ";
  	put "		reportRefNS = namespace(this:getParent());";
	put "		reviewRefNS = namespace(reportRefNS:getParent());";
	put "		parameterOverrides = [=>.];";
	put "		parameterOverrides << insertItem(""FDomain"", ""LB"");";
	put "		parameterOverrides << insertItem(""NormRef"", ""ULN"");";
	put "		parameterOverrides << insertItem(""Statistic"", ""Max"");";
	put "		parameterOverrides << insertItem(""WhereTests"", ""\[%nrbquote(&HysWhere)]\"");";
	put "		parameterOverrides << insertItem(""DislayCrossTabs"", ""Yes"");";
	put " 	reviewRefNS:addDrillDownReportTab(this:getParent(), ReportDefinitions:drilldownListMap[""FindingsShiftPlots""],parameterOverrides,""Liver Shift Plots"",0);";
   put "););";

   put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""ShiftPlots_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:splotsDDFunc()));";
   put "InsertInto(objRefNS:ActionSelectionCountList[2],0);";
   put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
   put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.LinearRegression.gif"");";      
run;


%let includeds = &OutDataName..sas7bdat , &OutDataName.gtt.sas7bdat;
%if &misslabs %then %do;
   %let includeds = &includeds , &OutDataName2..sas7bdat;
%end;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &HysLawScreeningScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend HysLawScreening;

* run the macro;
%HysLawScreening;


