/*
JMP Clinical Analytical Process

Name: IncidenceScreen
Author: Russ Wolfinger

History:
   sasrdw 22May06 initial coding 
   sasrdw 30Apr08 maxlen fix from Tim Myers, NIH

Description: Incidence Screen peforms an enrichment analysis by
      comparing a binary significance column
      with a column that defines a set of annotation categories.
      
      The process constructs the complete set of unique categories,
      computes a Fisher's exact test for each, and then adjusts
      the final set of p-values for multiplicity using a variety
      of methods.
*/

/*
-----------------------------------------------------------
Copyright (C) 2006 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global InData AnnoVar ASigVars BSigVars CSigVars Delimiter Delim2
SubjectFilter WhereStmt WhereForReports WhereState AltWhereStmt NBins RankEm MultipleTestingMethods NSample PThresh NHits Seed 
IncidenceScreen_AOutData IncidenceScreen_BOutData IncidenceScreen_COutData 
IncidenceScreen_CatOutData IncidenceScreen_DesOutData IncidenceScreen_GOutData
IncidenceScreenScript OutPath ServerOutPath MacroPath;

*ProcessBody;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;

%macro IncidenceScreen;

*initialization;
%if ^%symexist(ProcessName) %then %let ProcessName = IncidenceScreen;

%if ^%symexist(exiterror) %then %do;
   %global exiterror;
   %let exiterror = 1;
%end;
%if &exiterror %then %goto exit;
   
%GetOrigFileName(InData,OrigInData);
%let OrigInData = %lowcase(&OrigInData);
%PathName(&InData);
%if &exiterror %then %do;
   %put [MS_S_0071];
   %put ERROR: Input dataset specification is invalid.;
   %goto exit;
%end;
%let InData = &TmpName;

%put InData = &InData;
libname InLib "&TmpPath";

%if ^%length(&ASigVars) and ^%length(&BSigVars) and ^%length(&CSigVars) %then %do;
   %put [MS_S_0121];
   %put ERROR:  At least one Binary or Continuous Significance Variable must be specified;
   %let exiterror = 1;
   %goto exit;
%end;

%if %index(&MultipleTestingMethods,Bootstrap) and %index(&MultipleTestingMethods,Permutation) %then %do;
   %put [MS_S_0122];
   %put ERROR: Bootstrap and Permutation cannot both be specified.  Deselect one or the other.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %length(&CSigVars) and (%index(&MultipleTestingMethods,Bootstrap) or %index(&MultipleTestingMethods,Permutation) or
   %index(&MultipleTestingMethods,FDRBoot) or %index(&MultipleTestingMethods,FDRPerm)) %then %do;
   %put [MS_S_0123];
   %put ERROR:  Bootstrap and Permutation multiple testing adjustments are not available for PAGE tests.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%symexist(CIPrefix) %then %let CIPrefix = ci;
%if ^%symexist(CatName) %then %let CatName = &AnnoVar._Category;
%if ^%symexist(CatLabel) %then %let CatLabel = &AnnoVar Category;
%if ^%symexist(Abbrev) %then %let Abbrev = &AnnoVar;
%if ^%symexist(TimeWindows) %then %let TimeWindows = ;
%if ^%symexist(AlreadyStacked) %then %let AlreadyStacked = 0;
%if ^%symexist(ResponseVar) %then %let ResponseVar = ;
%if ^%symexist(ScreenType) %then %let ScreenType = Incidence;
%if ^%symexist(suffix) %then %let suffix = %lowcase(&domain)i;
%if ^%symexist(DoubleFDR) %then %let DoubleFDR = no;
%if ^%symexist(GroupVar) %then %let GroupVar = ;
%if ^%symexist(TreeData) %then %let TreeData = ;

%if ^%length(&groupvar) %then %let DoubleFDR = no;
%if ^%length(&MultipleTestingMethod) %then %let DoubleFDR = no;

%put CatName = &CatName;
%put GroupVar = &GroupVar;

* set up names for OutData;
%if %length(&OrigInData) < 29 %then %let OID = &OrigInData;
%else %let OID = %substr(&OrigInData,1,28);
%PrepOutDS(IncidenceScreen_AOutData,IncidenceScreen_AOutDataName,&OID,_cec);
%PrepOutDS(IncidenceScreen_BOutData,IncidenceScreen_BOutDataName,&OID,&CESuffix);
%if &DoubleFDR = yes %then %do;
   %PrepOutDS(IncidenceScreen_GOutData,IncidenceScreen_GOutDataName,&OID,&CGSuffix);
%end;

%PrepOutDS(IncidenceScreen_COutData,IncidenceScreen_COutDataName,&OID,_cep);
%if %length(&IncidenceScreen_CatOutData) %then %do;
   %PrepOutDS(IncidenceScreen_CatOutData,IncidenceScreen_CatOutDataName,&OID,_cat);
%end;
%if %length(&IncidenceScreen_DesOutData) %then %do;
   %PrepOutDS(IncidenceScreen_DesOutData,IncidenceScreen_DesOutDataName,&OID,_des);
%end;
%put InData = &InData;

* check strata variables;
%if ^%symexist(StrataVars) %then %let StrataVars = ;
%let StrataStar = ;
%if %length(&StrataVars) %then %do;

   %let StrataStar = %scan(&StrataVars,1,' ');
   %do i=2 %to %sysfunc(countw(&StrataVars));
      %let StrataStar = &StrataStar * %scan(&StrataVars,&i,' ');
   %end;

   proc freq data=InLib.&InData noprint;
      tables &StrataStar / out=fc;
      %do i=1 %to %sysfunc(countw(&StrataVars));
         where not missing(%scan(&StrataVars,&i,' '));
      %end;
   run;
   %NObsVars(fc);
   %if &nobs = 0 %then %do;
      %put [MS_S_0124];
      %put ERROR: Stratification variables have no valid levels.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %else %if &nobs = 1 %then %do;
      %put Stratification variables &StrataVars have only one unique level.  CMH tests will not be stratified.;
      %let StrataVars = ;
      %let StrataStar = ;
   %end;
   %else %do;
      %put Stratification variables &StrataVars have &nobs unique levels and CMH tests will be stratified.;
      %let StrataStar = &StrataStar *;
      %put StrataStar = &StrataStar;
   %end;
%end;
%else %do;
   %let StrataStar = ;
%end;

data cen;
   set InLib.&InData;
   obs = _n_;
   %if &AlreadyStacked %then %do;
      keep &StrataVars &DesignByVars &CatName &ResponseVar &ASigVars &BSigVars &CSigVars;
   %end;
   %else %do;
      keep &StrataVars &DesignByVars &ASigVars &BSigVars &CSigVars &CIPrefix._:;
   %end;
run;

/*
data outlib.cen;
   set cen;
run;
*/

/*
* merge category data with adsl;
data adsl;
   set &in_adsl;
   %if %length(&WhereStmt) %then %do;
      where &WhereStmt;
   %end;
   %if %length(&StrataVars) %then %do;
      if missing(&StrataVars) then delete;
   %end;
   keep USUBJID &StrataVars &ASigVars &BSigVars &CSigVars;
run;

proc sort data=adsl;
   by USUBJID;
run;

data cen;
   set &&in_&domain;
   keep USUBJID &CatName &GroupVar;
run;

proc sort data=cen;
   by USUBJID;
run;

data cen;
   merge cen adsl;
   by USUBJID;
run;

data OutLib.&OrigInData;
   set cen;
run;
*/

%if &syserr>4 %then %goto exit;


%if %length(&ASigVars) %then %do;
   * create new variables to avoid potential conflicts with overlapping CSigVars;
   data cen;
      set cen;
      %let i=1;
      %let ANew = ;
      %do %while(%length(%scan(&ASigVars,&i,' ')));
         %let sv = %scan(&ASigVars,&i,' ');
         %if %length(&sv) <= 28 %then %do;
            %let nv = &sv._b&nBins;
         %end;
         %else %do;
            %let nv = %substr(&sv,1,27)_&i;
         %end;
         %let ANew = &ANew &nv;
         &nv = &sv;
         %let i=%eval(&i+1);
      %end;
   run;
   
   %let ASigVars = &ANew;

   proc rank data=cen out=cen groups=&nBins;
      var &ASigVars;
   run;

   data cen;
      set cen;
      _obs_ = _n_;
   run;
%end;

%if %length(&CSigVars) and (&RankEm=Yes) %then %do;
   proc rank data=cen out=cen;
      var &CSigVars;
   run;
   %let RankLab = Rank;
   data cen;
      set cen;
      _obs_ = _n_;
   run;
%end;
%else %do;
   %let RankLab = ;
%end;



/*---create unique category entries using a delimiter if specified---*/
%if ^%symexist(CatIndicators) %then %let CatIndicators = 0;
%global makecatind;
%if ^&CatIndicators %then %do;
   %let Delimiter = %unquote(&Delimiter);
   %let ldim = %eval(%length(&Delimiter)-2);
   %put ldim = &ldim;

   %if %length(&Delim2) %then %do;
      %let Delim2 = %unquote(&Delim2);
      %let ldim2 = %eval(%length(&Delim2)-2);
      %put ldim2 = &ldim2;
   %end;
   %else %let ldim2 = 0;

   data cencat;
      set cen;
      retain maxlen 0;

      %if (&ldim > 0) %then %do;
         _av = strip(&AnnoVar);
         _idx = 1;
         do while(_idx);
            _idx = index(_av,&Delimiter);
            if _idx = 0 then cat = strip(_av);
            else do;
               if (_idx > 1) then cat = strip(substr(_av,1,_idx-1));
               else cat = "";
               _av = strip(substr(_av,_idx+&ldim));
               if not lengthn(_av) then _idx = 0;
            end;
            lenc = lengthn(cat);
            if (lenc) then do;
               %if (&ldim2 > 0) %then %do;
                  _av2 = strip(cat);
                  _idx2 = 1;
                  do while(_idx2);
                     _idx2 = index(_av2,&Delim2);
                     if _idx2 = 0 then cat = strip(_av2);
                     else do;
                        if (_idx2 > 1) then cat = strip(substr(_av2,1,_idx2-1));
                        else cat = "";
                        _av2 = strip(substr(_av2,_idx2+&ldim2));
                        if not lengthn(_av2) then _idx2 = 0;
                     end;
                     lenc = lengthn(cat);
                     if (lenc) then do;
                        maxlen = max(maxlen,lenc);
                        call symput("maxlen",trim(left(maxlen)));
                        output;
                     end;
                  end;
               %end;
               %else %do;
                  maxlen = max(maxlen,lenc);
                  call symput("maxlen",trim(left(maxlen)));
                  output;
               %end;
            end;
         end;
      %end;
      %else %do;
         cat = strip(&AnnoVar);
         * fix to prevent "maxlen not defined" error when ldim=0;
         lenc = lengthn(cat);
         if (lenc) then do;
            maxlen = max(maxlen,lenc);
            call symput("maxlen",trim(left(maxlen)));
            output;
         end;
      %end;
      * drop _av _idx &AnnoVar lenc;
   run;

   %put maxlen = &maxlen;

   %if %length(&IncidenceScreen_CatOutData) %then %do;
      proc sort data=cencat out=OutLib.&IncidenceScreen_CatOutDataName;
         by cat &StrataVars;
         where cat ne "---";
      run;
   %end;

   /*
   title "cencat1";
   proc print data=OutLib.cencat1(obs=1000);
   run;
   */
%end;
%else %do;
   data censum0;
      set InLib.&InData;
   run;
   /*
   data cendest;
      set &CIPrefix.l;
   run;
   */
%end;

* process binary significance variables;
%if %length(&BSigVars) %then %do;

   %let tests = Cochran Mantel Haenszel tests;
   %put Computing &tests;
   
   /*
   %if %length(&StrataVars) %then %do;
      %put Computing &tests;
   %end;
   %else %do;
      %put Computing Fisher exact tests;
   %end;
   */

   * if not doing bootstrap or permutation, perform calculations in proc freq for speed;
   %if ^%index(&MultipleTestingMethods,Bootstrap) and 
      ^%index(&MultipleTestingMethods,Permutation) and
      ^%index(&MultipleTestingMethods,FDRBoot) and 
      ^%index(&MultipleTestingMethods,FDRPerm) %then %do;
      
      /*
      data outlib.cen;
         set cen;
      run;
      */

      %let _tid_ = %sysfunc(open(cen));
      %let s=1;
      %let maxlensv = 0;
      %let maxlabsv = 0;
      %do %while(%length(%scan(&BSigVars,&s,' ')));
         %let sv&s = %scan(&BSigVars,&s,' ');
         %let lensvs = %length(&&sv&s);
         %if (&lensvs > &maxlensv) %then %let maxlensv = &lensvs;
         %let svvn = %sysfunc(varnum(&_tid_,&&sv&s));
         %let svlab&s = %qtrim(%qleft(%qsysfunc(varlabel(&_tid_,&svvn))));
         %let lensvs = %length(&&svlab&s);
         %if (&lensvs > &maxlabsv) %then %let maxlabsv = &lensvs;
         %let s = %eval(&s+1);
      %end;
      %let nsv = %eval(&s-1);
      %let _tid_ = %sysfunc(close(&_tid_));
      %let trover = %nrbquote(%LocaleGet(over_Text));
      %let j=1;
      %do i=1 %to &nlev;
         %if &i ^= &plev %then %do;
            %put sv&j = &&sv&j;
            %let svlab&j = %nrbquote(&&tlev&i &trover &&tlev&plev);
            %put svlab&j = &&svlab&j;
            %let j=%eval(&j+1);
         %end;
      %end;
      
      %put TrtName = &TrtName;

      /*---create by groups for each sigvar---*/
      data censum;
         length &TrtName.I $32 Ratio $128;
         set cen;
         %do i=1 %to &nsv;
            &TrtName.I = "&&sv&i";
            &TrtName.IV = 1 - &&sv&i;
            /*
            %if (&nsv = 1) %then %do;
               &TrtName.IV = &&sv&i;
            %end;
            %else %do;
               &TrtName.IV = 1 - &&sv&i;
            %end;
            */
            Ratio = "&&svlab&i";
            if not missing(&TrtName.I) and not missing(&TrtName.IV) then output;
         %end;
      run;

      /*
      * delete subjects with missing design-level by variables;
      %if %length(&DesignByVars) %then %do;
         data censum;
            set sensum;
            %let i = 1;
            %do %while(%length(%scan(&DesignByVars,&i)));
               if missing(%scan(&DesignByVars,&i)) then delete;
               %let i = %eval(&i+1);
            %end;
         run;
      %end;
      */

      %if &AlreadyStacked %then %do;
         %let CatBy = &CatName;
         %let RVars = &ResponseVar;
      %end;
      %else %do;
         %let CatBy = ;
         %let RVars = &CIPrefix._:;
      %end;

      proc sort data=censum;
         by Ratio &TrtName.I &DesignByVars &CatBy;
      run;

      /*
      proc contents data=censum;
      run;
      */
      
      data cmh;
         if (0);
      run;

      /*---perform cmh tests and adjust for multiplicity---*/
      ods exclude all;
      ods noresults;
      
      /*
      data outlib.censum;
         set censum;
      run;
      */

      proc freq data=censum;
         by Ratio &TrtName.I &DesignByVars &CatBy;
         tables &StrataStar &TrtName.IV * (&RVars) / cmh1 relrisk riskdiff(equal);
         ods output cmh=cmh crosstabfreqs=ctf(drop=&CIPrefix._:)
            /* relativerisks=rr pdifftest=pdt riskdiffcol1=rdc1 riskdiffcol2=rdc2 */
            %if %length(&StrataVars) %then %do;
               breslowdaytest=bdt commonrelrisks=crr
            %end;
         ;
      run;
      
      ods exclude none;
      ods results;
      
      /*
      data outlib.cmh;
         set cmh;
      run;
      data outlib.ctf;
         set ctf;
      run;
      */
      
      /*
      * results in these tables should agree with additional stats computed below;
      data outlib.rr;
         set rr;
      run;
      data outlib.pdt;
         set pdt;
      run;
      data outlib.rdc1;
         set rdc1;
      run;
      data outlib.rdc2;
         set rdc2;
      run;
      
      * extra tables when stratifying;
      %if %length(&StrataVars) %then %do;
         data outlib.bdt;
            set bdt;
         run;
         data outlib.crr;
            set crr;
         run;
      %end;
      */    
      
      data cmh0;
         set cmh;
         if missing(prob) then delete;
      run;
      
      %NObsVars(cmh0);
      %if ^&nobs %then %do;
         %put [MS_S_0125];
         %put ERROR: Insufficient data to perform Cochran Mantel Haenztel tests.;
         %put It may help to relax any filters and/or use more complete data.;
         %let exiterror = 1;
         %goto exit;
      %end;

      /* S0887992 kemicl added sort to avoid error when missing data issues */ 
      proc sort data=ctf;
         by Ratio &TrtName.I &DesignByVars &CatBy table;
      run;
            
      * collapse strata counts and table names;
      %if %length(&StrataVars) %then %do;
         proc sort data=ctf;
            by Ratio &TrtName.I &DesignByVars &CatBy table;
         run;
         
         %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH %then %do;
            *there is no "of" in the chinese dataset and the parentheses uses DBCS;
            data _null_;
               set ctf;
               if _n_=1 then do;
                  ip=ksubstr(table, 4,1);
                  call symput ("_pTmp_", trim(ip));
               end;
               stop;
            run;              
            data ctf;
               set ctf;
               by Ratio &TrtName.I &DesignByVars &CatBy table;
               retain idx 1;
               if first.table then idx = 1;
               else idx+1;
               io = kindex(table,"%nrbquote(&_pTmp_)");
               table1 = ksubstr(table,io+1);
               table1 = ksubstr(table1,1,klength(table1)-1);
               type = _type_;
               drop _type_;
            run;
         %end;
         %else %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = JA %then %do;
            data _null_;
               set ctf;
               if _n_=1 then do;
                  ip=ksubstr(table, 5,1);
                  call symput ("_pTmp_", trim(ip));
               end;
               stop;
            run;
            data ctf;
               set ctf;
               by Ratio &TrtName.I &DesignByVars &CatBy table;
               retain idx 1;
               if first.table then idx = 1;
               else idx+1;
               io = kindex(table,"%nrbquote(&_pTmp_)");
               table1 = ksubstr(table,io+2);
               type = _type_;
               drop _type_;
            run;
         %end;
         %else %do;
            data ctf;
               set ctf;
               by Ratio &TrtName.I &DesignByVars &CatBy table;
               retain idx 1;
               if first.table then idx = 1;
               else idx+1;
               io = index(table,"of");
               table1 = substr(table,io+3);
               type = _type_;
               drop _type_;
            run;
         %end;
         proc sort data=ctf;
            by Ratio &TrtName.I &DesignByVars &CatBy table1 idx type &TrtName.IV;
         run;
         ods exclude all;
         ods noresults;
         proc means data=ctf;
            by Ratio &TrtName.I &DesignByVars &CatBy table1 idx type &TrtName.IV;
            var frequency;
            output out=ctfs(drop=_TYPE_) sum=Frequency;
         run;
         ods exclude none;
         ods results;
         data ctf;
            set ctfs;
            rename table1=Table type=_TYPE_;
            drop idx;
         run;
         /*
         data outlib.ctf;
            set ctf;
         run;
         */
         
         /*
         data outlib.cmh;
            set cmh;
         run;
         */
         
         * parse out table name in a way that works across languages;
         %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH %then %do;
         data cmh;
            set cmh;
            length e1 e2 $ 50;
            starloc = kindex(Table,"*");
            e1 = kscan(ksubstr(Table,1,starloc-2),-1);
            e1=ksubstr(e1,2);
            e2 = ksubstr(Table,starloc+1);
            e2 = ksubstr(e2,1,klength(e2)-4);
            Table = kcompress(e1) || " * " || kcompress(e2);
            * drop starloc e1 e2;
         run;
         %end;
         %else %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = JA %then %do;
            data cmh;
               set cmh;
               length e1 e2 $ 50;
               starloc = kindex(Table,"*");
               e1 = kscan(ksubstr(Table,1,starloc-2),-1);
               e2 = ksubstr(Table,starloc+2);
               e2 = ksubstr(e2,1,klength(e2)-4);
               Table = kcompress(e1) || " * " || kcompress(e2);
               * drop starloc e1 e2;
            run;
         %end;
         %else %do;
            data cmh;
               set cmh;
               length e1 e2 $ 50;
               starloc = index(Table,"*");
               e1 = scan(substr(Table,1,starloc-2),-1);
               e2 = scan(substr(Table,starloc+1),1);
               Table = compress(e1) || " * " || compress(e2);
               * drop starloc e1 e2;
            run;
         %end;
         
         %if %sysfunc(exist(bdt)) %then %do;
            %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH %then %do;
               data bdt;
                  set bdt;
                  io = 1;
                  Table = ksubstr(table,io+1,klength(table)-5);
                  P_BDCHI = nValue1;
                  where Name1 = "P_BDCHI";
                  label P_BDCHI = "Breslow-Day Pr > ChiSq";
                  drop io Name1 Label1 cValue1 nValue1;
               run;
            %end;
            %else %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = JA %then %do;
               data bdt;
                  set bdt;
                  io = 1;
                  Table = ksubstr(table,io,klength(table)-4);
                  P_BDCHI = nValue1;
                  where Name1 = "P_BDCHI";
                  label P_BDCHI = "Breslow-Day Pr > ChiSq";
                  drop io Name1 Label1 cValue1 nValue1;
               run;
            %end;
            %else %do;
               data bdt;
                  set bdt;
                  io = index(table,"for");
                  Table = substr(table,io+4);
                  P_BDCHI = nValue1;
                  where Name1 = "P_BDCHI";
                  label P_BDCHI = "Breslow-Day Pr > ChiSq";
                  drop io Name1 Label1 cValue1 nValue1;
               run;
            %end;
         %end;
      %end;
      
      * extract stats;
      %let _levs1 = 0;
      %let _levs2 = 1;
      %let _qlevs1 = 0;
      %let _qlevs2 = 1;
      /*
      %if &nsv > 1 %then %do;
         %let _levs1 = 0;
         %let _levs2 = 1;
         %let _qlevs1 = 0;
         %let _qlevs2 = 1;
      %end;
      %else %do;
         %let _levs1 = &tlev1;
         %let _levs2 = &tlev2;
         %let _qlevs1 = "&tlev1";
         %let _qlevs2 = "&tlev2";
      %end;
      */
      
      data ctf;
         set ctf;
         retain idx 1;
         by Ratio &TrtName.I &DesignByVars &CatBy table;
         lt = length(trim(left(_TYPE_)));
         if lt > 2 then ttype = substr(_TYPE_,lt-1,2);
         else ttype = _TYPE_;
         if first.table then idx = 1;
         else idx+1;
         m9 = mod(idx,9);
         
         if idx=1 or idx=4 then delete;
         else if ttype = "11" and &TrtName.IV = &_qlevs1 then idt = "_xval_";
         else if ttype = "11" and &TrtName.IV = &_qlevs2 then idt = "_yval_";
         else if ttype = "10" and &TrtName.IV = &_qlevs1 then idt = "_mval_";
         else if ttype = "10" and &TrtName.IV = &_qlevs2 then idt = "_nval_";
         else delete;
         
         /* previous logic when order of trtname and rvars is switched in tables statement
         if m9 <= 2 then delete;
         else if ttype = "11" and &TrtName.IV = &_qlevs1 then idt = "_xval_";
         else if ttype = "11" and &TrtName.IV = &_qlevs2 then idt = "_yval_";
         else if ttype = "01" and &TrtName.IV = &_qlevs1 then idt = "_mval_";
         else if ttype = "01" and &TrtName.IV = &_qlevs2 then idt = "_nval_";
         else delete;
         */
      run;

      /*
      data outlib.ctf;
         set ctf;
      run;
      */

      proc transpose data=ctf out=b(drop=_name_ _label_);
         by Ratio &TrtName.I &DesignByVars &CatBy table;
         var frequency;
         id idt;
      run;
      
      data cmh;
         set cmh;
         rename prob=Raw_p;
         drop statistic althypothesis df;
      run;

      data b;
         merge b cmh %if %length(&StrataVars) and %sysfunc(exist(bdt)) %then %do; bdt %end; ;
         by Ratio &TrtName.I &DesignByVars &CatBy table;
      run;

      %if ^&AlreadyStacked %then %do;     
         %let lci = %length(&CIPrefix);

         data b;
            length _NAME_ $ 32;
            set b;
            firstunder = index(table,'_');
            _NAME_ = trim(left(scan(substr(table,firstunder+1),1,' *')));
            * drop table;
            drop firstunder;
         run;

         * merge in category names;
         data c1;
            set censum(obs=1);
            keep &CIPrefix._: ;
         run;

         proc transpose data=c1 out=c1t;
            var &CIPrefix._:;
         run;

         %let lci = %eval(%length(&CIPrefix)+2);

         data c1t;
            set c1t;
            drop COL1;
            _NAME_ = substr(_NAME_,&lci);
            &CatName = _LABEL_;
            label &CatName="&CatLabel";
            drop _LABEL_;
         run;

         /*
         data outlib.b;
            set b;
         run;

         data outlib.c1t;
            set c1t;
         run;

         %put ERROR: Not!;
         %let exiterror = 1;
         %goto exit;
         */
         
         proc sql;
            create table b as
               select *
               from b d1, c1t d2
               where d1._NAME_=d2._NAME_;
         quit;
      %end;

      * merge grouping data;
      %put merging TreeData = &TreeData;
      %if %length(&TreeData) %then %do;
         proc sql;
            create table b as
               select *
               from b d1, OutLib.&TreeData d2
               where d1.&CatName=d2.&CatName;
         quit;
      %end; 
      
      /*
      data outlib.b;
         set b;
      run;
      */

      * apply multiple testing adjustments;
      %include "&MacroPath./MultipleTestingAdjustment.sas";
      
      %if &DoubleFDR = yes %then %do;
         %let bvar = &GroupVar;
         proc sort data=b;
            by &bvar;
         run;
      %end;
      %else %let bvar = ;
      
      data b;
         set b;
         if missing(raw_p) then delete;
      run;

      %let m=1;
      %do %while(%length(%scan(&MultipleTestingMethods,&m,' ')));
         %let MultipleTestingMethod = %scan(&MultipleTestingMethods,&m,' ');
         %put Computing p-value adjustment &m: &MultipleTestingMethod;
         %AdjPValueName(method=&MultipleTestingMethod);
         %MultipleTestingAdjustment(data=b,method=&MultipleTestingMethod,out=b,byvar=&bvar);
         %let m = %eval(&m+1);
      %end;
      
      %let trcount = %nrbquote(%LocaleGet(Count_Text));
      
      /*
      * loop over BSigVars, collect names, labels, and maximum lengths of each;
      %let _tid_ = %sysfunc(open(cen));
      %let s=1;
      %let maxlensv = 0;
      %let maxlabsv = 0;
      %do %while(%length(%scan(&BSigVars,&s,' ')));
         %let sv&s = %scan(&BSigVars,&s,' ');
         %let lensvs = %length(&&sv&s);
         %if (&lensvs > &maxlensv) %then %let maxlensv = &lensvs;
         %let svvn = %sysfunc(varnum(&_tid_,&&sv&s));
         %let svlab&s = %trim(%left(%sysfunc(varlabel(&_tid_,&svvn))));
         %let lensvs = %length(&&svlab&s);
         %if (&lensvs > &maxlabsv) %then %let maxlabsv = &lensvs;
         %let s = %eval(&s+1);
      %end;
      %let nsv = %eval(&s-1);
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %do s=1 %to &nsv;

         %let sv = &&sv&s;
         %put s=&s  sv=&sv;

         proc freq data=cen noprint;
            tables &sv / out=fc;
            where not missing(&sv);
         run;

         %put ERROR: Not!;
         %let exiterror = 1;
         %goto exit;

         %NObsVars(fc);
         %if &nobs ^= 2 %then %do;
            %put ERROR: Binary significance variable &&sv&s has &nobs levels instead of 2.;
            %let exiterror = 1;
            %goto exit;
         %end;

         data _null_;
            set fc;
            call symput("_levs"||trim(left(_n_)),trim(left(&sv)));
            call symput("_cnts"||trim(left(_n_)),trim(left(count)));
         run;

         %do i=1 %to 2;
            %put _levs&i = &&_levs&i;
         %end;
         
         * tally frequencies;
         proc freq data=cen noprint;
            tables &CatName * &StrataStar &sv / out=fq;
            where not missing(&sv);
         run;
         
         data fq;
            set fq;
            if trim(left(&sv)) = "&_levs1" then i01 = 0;
            else i01 = 1;
         run;

         proc transpose data=fq out=fqt(drop=_name_ _label_) prefix=count;
            by &CatName &StrataVars;
            var count;
            label count = "&trcount";
            id i01;
         run;

         data b1;
            length %if &maxlabsv %then %do; SigVarLabel $ &maxlabsv %end;
               &TrtName.I $ &maxlensv; 
            set fqt;
            %if &maxlabsv %then %do; 
               SigVarLabel = "&&svlab&s";
            %end;
            &TrtName.I = "&sv";
            _xval_ = count0;
            if _xval_ = . then _xval_ = 0;
            _mval_ = &_cnts1;
            _yval_ = count1;
            if _yval_ = . then _yval_ = 0;
            _nval_ = &_cnts2;
            label %if &maxlabsv %then %do; SigVarLabel = "Binary Significance Label" %end;
               &TrtName.I = "Binary Significance Variable" &CatName = "&CatLabel";
            keep %if &maxlabsv %then %do; SigVarLabel %end; &TrtName.I 
               &CatName _xval_ _mval_ _yval_ _nval_;
         run;

         * set up for proc freq;
         data f;
            set b1;
            cat01 = 0;
            sig01 = 0;
            w = _mval_ - _xval_;
            output;
            cat01 = 0;
            sig01 = 1;
            w = _nval_ - _yval_;
            output;
            cat01 = 1;
            sig01 = 0;
            w = _xval_;
            output;
            cat01 = 1;
            sig01 = 1;
            w = _yval_;
            output;
            keep &CatName &StrataVars cat01 sig01 w;
         run;

         * compute tests;
         %if %length(&StrataVars) %then %do;
            proc freq data=f noprint;
               by &CatName;
               tables &StrataStar cat01*sig01 / cmh1;
               weight w / zeros;
               output out=f1 cmh1;
            run;
            
            data outlib.f1;
               set f1;
            run;
            
            %put ERROR: Not!;
            %let exiterror = 1;
            %goto exit;

            data f1;
               set cmh1;
               Raw_p = xp2_fish;
               keep &CatName raw_p;
            run;
         %end;
         %else %do;
            proc freq data=f noprint;
               by &CatName;
               tables cat01*sig01 / fisher;
               weight w / zeros;
               output out=f1 fisher;
            run;

            data f1;
               set f1;
               Raw_p = xp2_fish;
               keep &CatName raw_p;
            run;
         %end;

         data b1;
            merge b1 f1;
            by &CatName;
         run;

         * apply multiple testing adjustments;
         %include "&MacroPath./MultipleTestingAdjustment.sas";

         %let m=1;
         %do %while(%length(%scan(&MultipleTestingMethods,&m,' ')));
            %let MultipleTestingMethod = %scan(&MultipleTestingMethods,&m,' ');
            %put Computing p-value adjustment &m: &MultipleTestingMethod;
            %AdjPValueName(method=&MultipleTestingMethod);
            %MultipleTestingAdjustment(data=b1,method=&MultipleTestingMethod,out=b1);
            %let m = %eval(&m+1);
         %end;

         %if (&s=1) %then %do;
            data b;
               set b1;
            run;
         %end;
         %else %do;
            proc append base=b data=b1;
            run;
         %end;
      %end;
      */
            
   %end;
   %else %do;

      %if ^&CatIndicators %then %do;
         %MakeCatIndicators;
         %if &exiterror %then %goto exit;
      %end;

      %let i=1;
      %do %while(%length(%scan(&BSigVars,&i,' ')));
         %let sv&i = %scan(&BSigVars,&i,' ');
         %let i=%eval(&i+1);
      %end;
      %let nsv = %eval(&i-1);

      /*---create by groups for each sigvar---*/
      data censum;
         length &TrtName.I $32;
         set censum0;
         %do i=1 %to &nsv;
            %let sv&i = %scan(&BSigVars,&i,' ');
            &TrtName.I = "&&sv&i";
            %if (&nsv = 1) %then %do;
               &TrtName.IV = &&sv&i;
            %end;
            %else %do;
               &TrtName.IV = 1 - &&sv&i;
            %end;
            if not missing(&TrtName.I) and not missing(&TrtName.IV) then output;
         %end;
         label &TrtName.I = "Binary Significance Variable";
      run;

      /*
      * delete subjects with missing design-level by variables;
      %if %length(&DesignByVars) %then %do;
         data censum;
            set sensum;
            %let i = 1;
            %do %while(%length(%scan(&DesignByVars,&i)));
               if missing(%scan(&DesignByVars,&i)) then delete;
               %let i = %eval(&i+1);
            %end;
         run;
      %end;
      */
      
      proc sort data=censum;
         by Ratio &TrtName.I &DesignByVars;
      run;
            
      /*
      proc contents data=censum;
      run;
      */

      /*---fishers exact tests and adjust for multiplicity---*/
      ods exclude all;
      ods noresults;

      proc multtest data=censum &MultipleTestingMethods
            %if %length(&NSample) %then %do; nsample=&NSample %end;
            %if %length(&Seed) %then %do; seed=&Seed %end;
            out=esp;
         by Ratio &TrtName.I &DesignByVars;
         class &TrtName.IV;
         %if %length(&StrataVars) %then %do;
            strata &StrataVars;
            test ca(&CIPrefix._:);
            Ratio "1 vs 0" 1 -1;
         %end;
         %else %do;
            test fisher(&CIPrefix._:);
         %end;
      run;

      ods exclude none;
      ods results;

      %if &syserr>4 %then %goto exit;
      
      data esp;
         set esp;
         _var_ = lowcase(_var_);
      run;

      proc sort data=esp;
         by _var_;
      run;

      proc sort data=cendest;
         by _var_;
      run;

      data b;
         merge cendest esp;
         by _var_;
      run;
      
      /*
      data outlib.b;
         set b;
      run;
      
      %put ERROR: Not!;
      %let exiterror = 1;
      %goto exit;
      */
      
      data b;
         set b;
         drop _var_ _test_;
         %if &nsv = 1 %then %do;
            drop _Ratio_;
         %end;
      run;  

      * when there is only one sig var, use its values as labels in the results table;
      %if &nsv = 1 %then %do;
         proc freq data=cen noprint;
            tables &sv1 / out=fc;
            where not missing(&sv1);
         run;

         data _null_;
            set fc;
            call symput("_levs"||trim(left(_n_)),trim(left(&sv1)));
            call symput("_cnts"||trim(left(_n_)),trim(left(count)));
         run;

         %do i=1 %to 2;
            %put _levs&i = &&_levs&i;
         %end;
      %end;
      %else %do;
         %let _levs1 = 0;
         %let _levs2 = 1;
      %end;    
      
   %end;

   %let trcl1p = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(ClassXPercent_Text)), &_levs1));
   %let trcl2p = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(ClassXPercent_Text)), &_levs2));
   %let trcount = %nrbquote(%LocaleGet(Count_Text));
   %let trriskdiff = %nrbquote(%LocaleGet(RiskDifference_Text));
   %let trrelrisk = %nrbquote(%LocaleGet(RelativeRisk_Text));
   %let trlog2relrisk = %nrbquote(%LocaleGet(Log2RelativeRisk_Text));
   %let troddsratio = %nrbquote(%LocaleGet(OddsRatio_Text));
   %let trlog2oddsratio = %nrbquote(%LocaleGet(Log2OddsRatio_Text));
   %let trslog10p = %nrbquote(%LocaleGet(Signedlog10p_Text));
   %let trneglog10rawpv = %nrbquote(%LocaleGet(MinusLog10Rawpvalue_Text));
   %let trratio = %nrbquote(%LocaleGet(Ratio_Text));
   %let lengthtrratiopluseq = %eval(%klength(&trratio) + 1);
   * compute additional statistics;
   data OutLib.&IncidenceScreen_BOutDataName;
      length _x_over_m_ 8 _y_over_n_ 8;
      set b end=_last_;
      retain maxpercent 0;
      label _xval_="Class &_levs1 Count" _mval_="Class &_levs1 Total"  
            _yval_="Class &_levs2 Count" _nval_="Class &_levs2 Total"
            ;
      if (Raw_p > &PThresh) then delete;
      if _xval_=. then _xval_=0;
      if _yval_=. then _yval_=0;
      
      if (_xval_ < &NHits) and (_yval_ < &NHits) then delete;
      if missing(Raw_p) then NegLog10_Raw_p = 6 - 0.25*ranuni(214870);
      else if (Raw_p > 1e-32) then NegLog10_Raw_p = -log10(Raw_p);
      else NegLog10_Raw_p = 32;
      _x_over_m_ = _xval_/_mval_;
      _y_over_n_ = _yval_/_nval_;
      _x_over_m_Percent = 100 * _x_over_m_;
      _y_over_n_Percent = 100 * _y_over_n_;     
      if _x_over_m_Percent > maxpercent then maxpercent = _x_over_m_Percent;
      if _y_over_n_Percent > maxpercent then maxpercent = _y_over_n_Percent;
      
      if (_x_over_m_ <  _y_over_n_) then c0_less_than_c1 = 1;
      else c0_less_than_c1 = 0;

      if (_mval_ > 0 and _nval_ > 0) then RiskDiff = _x_over_m_ - _y_over_n_;
      else if _mval_ > 0 then RiskDiff = _x_over_m_;
      else if _nval_ > 0 then RiskDiff = -(_y_over_n_);
      else RiskDiff = 0;
      
      if missing(_y_over_n_) then do;
         RR = 2**3;
      end;
      else if (_y_over_n_ > 0) then do;
         RR = _x_over_m_/_y_over_n_;
      end;
      else do;
         RR = _x_over_m_/(1/_nval_);
      end;
      
      if (RR > 0) then Log2_RR = log2(RR);
      else Log2_RR = -3;
      
      /*RCZ changed calculation of Odds ratio */
      if _yval_ > 0 and _xval_ > 0 and _mval_ > _xval_ and _nval_ > _yval_ then OddsR = (_xval_*(_nval_ - _yval_)) / (_yval_*(_mval_ - _xval_));
      else OddsR = ((_xval_ + 0.5)*(_nval_ - _yval_ + 0.5)) / ((_yval_ + 0.5)*(_mval_ - _xval_ + 0.5));  *Woolf-Haldane continuity correction for any 0 cell;
      
      if (OddsR > 0) then Log2_OddsR = log2(OddsR);
      else Log2_OddsR = -3;
      /* kemicl added SE and odds ratio CIs */
      SE_Log2_OddsR = sqrt((1/max(_yval_,1))+(1/max(_xval_,1))+(1/_mval_)+(1/_nval_))/log(2);
      L95_Log2_OddsR = Log2_OddsR - probit(1-(1-0.95)/2)*SE_Log2_OddsR;
      U95_Log2_OddsR = Log2_OddsR + probit(1-(1-0.95)/2)*SE_Log2_OddsR;
      L95_OddsR = 2**(L95_Log2_OddsR);
      U95_OddsR = 2**(U95_Log2_OddsR);
      SE_Log2_RR = sqrt(((_mval_-_xval_)/max(_xval_,1)/_mval_)+
         ((_nval_-_yval_)/max(_yval_,1)/_nval_))/log(2);
      L95_Log2_RR = Log2_RR - probit(1-(1-0.95)/2)*SE_Log2_RR;
      U95_Log2_RR = Log2_RR + probit(1-(1-0.95)/2)*SE_Log2_RR;
      L95_RR = 2**(L95_Log2_RR);
      U95_RR = 2**(U95_Log2_RR);
      if (abs(Log2_RR) < 1e-6) then Signed_Log10_p = 0;
      else Signed_Log10_p = min(6.001,NegLog10_Raw_p)*abs(Log2_RR)/Log2_RR;
      * if (Signed_Log10_p < -6) then Signed_Log10_p = 5.999;
      /*kemicl added character columns for creating tabular report */
      x_count_percent = "   "||trim(left(_xval_))||"   ("||trim(left(round(_x_over_m_Percent,.1)))||"%)";
      y_count_percent = "   "||trim(left(_yval_))||"   ("||trim(left(round(_y_over_n_Percent,.1)))||"%)";
      RR_CI = "    "||trim(left(round(RR,.1)))||"   ("||trim(left(round(L95_RR,.1)))||", "||trim(left(round(U95_RR,.1)))||")";
      oddsR_CI = "   "||trim(left(round(OddsR,.1)))||"   ("||trim(left(round(L95_RR,.1)))||", "||trim(left(round(U95_RR,.1)))||")";
      label 
         _x_over_m_="Class &_levs1 Proportion" 
         _y_over_n_="Class &_levs2 Proportion" 
         _x_over_m_Percent="&trcl1p" 
         _y_over_n_Percent="&trcl2p"         
         c0_less_than_c1="Class &_levs1 Prop < Class &_levs2 Prop"
         RR = "&trrelrisk" 
         Log2_RR="&trlog2relrisk" 
         SE_Log2_RR="Standard Error log2(RR)"
         L95_Log2_RR="Lower 95% log2(RR)"
         U95_Log2_RR="Upper 95% log2(RR)"
         L95_RR="Lower 95% RR"
         U95_RR="Upper 95% RR"
         NegLog10_Raw_p="&trneglog10rawpv"
         Signed_Log10_p="&trslog10p"   
         OddsR="&troddsratio"
         Log2_OddsR = "&trlog2oddsratio"
         SE_Log2_OddsR = "Standard Error log2(Odds Ratio)"
         L95_Log2_OddsR = "Lower 95% log2(Odds Ratio)"
         U95_Log2_OddsR = "Upper 95% log2(Odds Ratio)"
         L95_OddsR = "Lower 95% Odds Ratio"
         U95_OddsR = "Upper 95% Odds Ratio"
         x_count_percent = "&_levs1 Count (%)"
         y_count_percent = "&_levs2 Count (%)"
         oddsR_CI = "Odds Ratio (95% CI)"
         RR_CI = "Relative Risk (95% CI)"
         RiskDiff="&trriskdiff"
         Ratio = "&trratio"
      ;
      call symputx("maxpercent",ceil(maxpercent));
   run;
   
   %NObsVars(OutLib.&IncidenceScreen_BOutDataName);

   %if (&nobs = 0) %then %do;
      %put [MS_S_0126];
      %put ERROR: No valid results were produced for &tests..  There may be too much;
      %put missing data or each by group may be too small.;
      %let exiterror = 1;
      %goto exit;
   %end;

   * rearrange columns;
   data a;
      set OutLib.&IncidenceScreen_BOutDataName;
      drop _xval_ _mval_ _yval_ _nval_ _x_over_m_ _y_over_n_ c0_less_than_c1;
   run;
   data b;
      set OutLib.&IncidenceScreen_BOutDataName;
      keep _xval_ _mval_ _yval_ _nval_ _x_over_m_ _y_over_n_ c0_less_than_c1;
   run;

   data OutLib.&IncidenceScreen_BOutDataName;
      merge a b;
   run;
   
   * Double FDR calculations;
   %if &DoubleFDR = yes %then %do;
      proc sort data=OutLib.&IncidenceScreen_BOutDataName;
         by &GroupVar Ratio &mtpvar;
      run;

      * group-level representative is the minimum adjusted p from the term level;
      data g;
         set OutLib.&IncidenceScreen_BOutDataName;
         by &GroupVar Ratio &mtpvar;
         if first.Ratio;
         Raw_p = &mtpvar;
         if missing(Raw_p) then NegLog10_Raw_p = 6 - 0.25*ranuni(214870);
         else if (Raw_p > 1e-32) then NegLog10_Raw_p = -log10(Raw_p);
         else NegLog10_Raw_p = 32;
         if (abs(Log2_RR) < 1e-6) then Signed_Log10_p = 0;
         else Signed_Log10_p = min(6.001,NegLog10_Raw_p)*abs(Log2_RR)/Log2_RR;
         drop &mtpvar;
      run;

      * adjust p-values at the group level;
      %MultipleTestingAdjustment(data=g,method=&MultipleTestingMethod,out=g);
      
      data OutLib.&IncidenceScreen_GOutDataName;
         set g;
      run;

      * determine group-level FDR cutoff for group-level adjusted p-values, use Bonferroni as upper bound;
      %NObsVars(OutLib.&IncidenceScreen_GOutDataName);
      %let ntests = &nobs;

      %put Alpha = &Alpha;
      %put ntests = &ntests;
      %put mtpvar = &mtpvar;

      data _null_;
         cutoff = -log10(&Alpha/&ntests);
         call symput("NegLog10pGroupCutoff",trim(left(round(cutoff,0.01))));
      run;

      %put Initial -log10 group cutoff (Bonferroni) = &NegLog10pGroupCutoff;

      proc sort data=OutLib.&IncidenceScreen_GOutDataName;
         by Raw_p;
      run;     

      data g;
         set OutLib.&IncidenceScreen_GOutDataName;
         if not missing(&mtpvar) and (&mtpvar <= &Alpha) then do;
            cutoff = -log10(raw_p);
            call symput("NegLog10pGroupCutoff",trim(left(round(cutoff-0.01,0.01)))); 
         end;
         else if (&mtpvar > &Alpha) then stop;
         keep &GroupVar;
      run;

      %put NegLog10pGroupCutoff = &NegLog10pGroupCutoff;

      * if necessary, add dummy rows so color scale always has the same range;
      proc means noprint data=OutLib.&IncidenceScreen_GOutDataName;
         var Signed_Log10_p;
         output out=check min=minslp max=maxslp;
      run;

      data dummy;
         set check;
         if minslp <= -6 and maxslp >= 6 then delete;
         else do;
            if minslp > -6 then do;
               &GroupVar = "ZZ_Low ";
               Ratio = "&svlab1";
               Signed_Log10_p = -6;
               output;
            end;
            if maxslp < 6 then do;
               &GroupVar = "ZZ_High";
               Ratio = "&svlab1";
               Signed_Log10_p = 6;
               output;
            end;
         end;
         drop _type_ _freq_ minslp maxslp;
      run;

      %NObsVars(dummy);
      %if &nobs %then %do;
         proc append base=OutLib.&IncidenceScreen_GOutDataName data=dummy force;
         run;
      %end;
         
      * subset term-level data to only significant groups;
      proc sort data=g;
         by &GroupVar;
      run;
      
      data OutLib.&IncidenceScreen_BOutDataName;
         merge OutLib.&IncidenceScreen_BOutDataName g(in=_ing_);
         by &GroupVar;
         if _ing_;
      run;

   %end;

   * determine cutoff for raw p-values empirically from the adjusted p-values, use Bonferroni as upper bound;
   %NObsVars(OutLib.&IncidenceScreen_BOutDataName);
   %let ntests = &nobs;
   
   %if %length(&MultipleTestingMethod) %then %do;
      %AdjPValueName(method=&MultipleTestingMethod);

      %put Alpha = &Alpha;
      %put ntests = &ntests;
      %put mtpvar = &mtpvar;

      data _null_;
         cutoff = -log10(&Alpha/&ntests);
         call symput("NegLog10pCutoff",trim(left(round(cutoff,0.01))));
      run;

      %put Initial -log10 cutoff (Bonferroni) = &NegLog10pCutoff;

      *** RCZ: Add AEBODSYS if AEDECOD or AETERM requested and not doubleFDR;

      %if %upcase(&domain) = AE and &DoubleFDR = no and (%upcase(&CatName) = AEDECOD or %upcase(&CatName) = AETERM) %then %do;
      
         proc sort data=OutLib.&IncidenceScreen_BOutDataName;
            by &CatName;
         run;
         
         proc sort nodupkey data = ae out = aelist(keep = &CatName AEBODSYS);
            by &CatName AEBODSYS;
         run;
         
         data OutLib.&IncidenceScreen_BOutDataName;
            merge OutLib.&IncidenceScreen_BOutDataName(in = ina) aelist;
            by &CatName;
            if ina;
         run;
      %end;
      
      proc sort data=OutLib.&IncidenceScreen_BOutDataName;
         by Raw_p;
      run;
      
      data _null_;
         set OutLib.&IncidenceScreen_BOutDataName;
         if not missing(&mtpvar) and (&mtpvar <= &Alpha) then do;
            cutoff = -log10(raw_p);
            call symput("NegLog10pCutoff",trim(left(round(cutoff-0.01,0.01)))); 
         end;
         else stop;
      run;
   %end;
   
   %put NegLog10pCutoff = &NegLog10pCutoff;

   * set up labels for jsl;
   %let nbsv = 1;
   %let svi = &TrtName;
   %CheckLabel(&IncidenceScreen_DesOutData,svi);
   %let bsv1 = &svi;
   
   /*
   %let nbsv = &nsv;
   %do i=1 %to &nbsv;
      %let svi = &&sv&i;
      %CheckLabel(&IncidenceScreen_DesOutData,svi);
      %let bsv&i = &svi;
   %end;
   */
   
   %let odata = OutLib.&IncidenceScreen_BOutDataName;
   %include "&MacroPath./DesignByVars.sas";
   
   * if necessary, add dummy rows so color scale always has the same range;
   proc means noprint data=&dbdata;
      %if %upcase(&SplitResults)=YES %then %do;
         by &DesignByVars;
      %end;
      var Signed_Log10_p;
      output out=check min=minslp max=maxslp;
   run;
   
   %Global trNone;
   %let trNone = %nrbquote(%LocaleGet(None_Text));
   
   data dummy;
      set check;
      if minslp <= -6 and maxslp >= 6 then delete;
      else do;
         if minslp > -6 then do;
            &CatName = "ZZ_Low ";
            Ratio = "&svlab1";
            %if %length(&TreeData) %then %do;
               &TreeSize = 1;
               %if %length(&GroupVar) %then %do;
                  &GroupVar = "&trNone";
               %end;
            %end;
            Signed_Log10_p = -6;
            Count = .;
            label Count = "&trcount";
            output;
         end;
         if maxslp < 6 then do;
            &CatName = "ZZ_High";
            Ratio = "&svlab1";
            %if %length(&TreeData) %then %do;
               &TreeSize = 1;
               %if %length(&GroupVar) %then %do;
                  &GroupVar = "&trNone";
               %end;
            %end;
            Signed_Log10_p = 6;
            Count = .;
            label Count = "&trcount";
            output;
         end;
      end;
      drop _type_ _freq_ minslp maxslp;
   run;

   %NObsVars(dummy);
   %if &nobs %then %do;
      proc append base=OutLib.&IncidenceScreen_BOutDataName data=dummy force;
      run;
   %end;
   
   /* kemicl - added to replace overall count with trt pair specific count total */
   data OutLib.&IncidenceScreen_BOutDataName;
      set OutLib.&IncidenceScreen_BOutDataName;
      Total_Count = Count;
      Count = _xval_ + _yval_;
      label Count = "&trcount";
   run;
   
   /* kemicl - added to get right counts for body system and max number of FBs */
   %if &DoubleFDR = yes %then %do;
      data OutLib.&IncidenceScreen_GOutDataName;
         set OutLib.&IncidenceScreen_GOutDataName;
         Total_Count = Count;
         Count = _xval_ + _yval_;
         label Count = "&trcount";
      run;
      %let datforFB = OutLib.&IncidenceScreen_GOutDataName;
   %end;
   %else %do;
      %let datforFB = OutLib.&IncidenceScreen_BOutDataName;
   %end;
   
   /* kemicl - check if Bout table is empty so don't display preferred term plots */
   %nobsvars(OutLib.&IncidenceScreen_BOutDataName);
   %let _BObs_ = &nobs;
   
   /* kemicl - added to calc TotFBs as total of nonmissing levels of by groups */
   proc sort data=&datforFB out=bvnum nodupkey;
      by Ratio &DesignByVars;
   run;
   %NobsVars(bvnum);
   %let TotFBs = &nobs;
   %put TotFBs = &TotFBs;
%end;


* Cochran-Armitage tests for continuous significance variables;
%if %length(&ASigVars) %then %do;

   %put Computing Cochran-Armitage tests.;

   %if ^&CatIndicators %then %do;
      %MakeCatIndicators;
      %if &exiterror %then %goto exit;
   %end;

   /*---create by groups for each sigvar---*/
   data censum;
      length &TrtName.I $32;
      set censum0;
      %let i=1;
      %do %while(%length(%scan(&ASigVars,&i,' ')));
         %let sv = %scan(&ASigVars,&i,' ');
         &TrtName.I = "&sv";
         &TrtName.IV = &sv;
         output;
         %let i=%eval(&i+1);
      %end;
      label &TrtName.I = "Continuous Significance Variable";
   run;

   proc sort data=censum;
      by Ratio &TrtName.I;
   run;

   /*
   proc contents data=censum;
   run;
   */

   /*---ca tests and adjust for multiplicity---*/
   ods exclude all;
   ods noresults;

   proc multtest data=censum &MultipleTestingMethods
         %if %length(&NSample) %then %do; nsample=&NSample %end;
         %if %length(&Seed) %then %do; seed=&Seed %end;
         out=esp;
      by Ratio &TrtName.I;
      class &TrtName.IV;
      test ca(&CIPrefix._:);
   run;

   ods exclude none;
   ods results;

   %if &syserr>4 %then %goto exit;

   proc sort data=esp;
      by _var_;
   run;

   proc sort data=cendest;
      by _var_;
   run;
   
   data a;
      merge cendest esp;
      by _var_;
   run;

   data OutLib.&IncidenceScreen_AOutDataName;
      length &TrtName.I $32;
      set a;
      if missing(raw_p) or (raw_p > &PThresh) then delete;
      drop _var_ _test_ _Ratio_;
      /*
      label _xval_="Class 0 Count" _mval_="Class 0 Total"  _yval_="Class 1 Count" _nval_="Class 1 Total";
      if (_xval_ < &NHits) and (_yval_ < &NHits) then delete;
      _x_over_m_ = _xval_/_mval_;
      _y_over_n_ = _yval_/_nval_;
      if (_x_over_m_ <  _y_over_n_) then c0_less_than_c1 = 1;
      else c0_less_than_c1 = 0;
      label _x_over_m_="Class 0 Proportion" _y_over_n_="Class 1 Proportion" c0_less_than_c1="Class 0 Prop < Class 1 Prop";
      */
   run;

   %NObsVars(OutLib.&IncidenceScreen_AOutDataName);

   %if (&nobs = 0) %then %do;
      %put [MS_S_0127];
      %put ERROR: No categories passed the specified enrichment thresholds for Cochran-Armitage tests.;
      %let exiterror = 1;
      %goto exit;
   %end;

   proc sort data=OutLib.&IncidenceScreen_AOutDataName;
      by Ratio &TrtName.I raw_p;
   run;

%end;

* PAGE tests on continuous significance variables;
* refer to Kim and Volsky BMC Bioinformatics. 2005, 6: 144;
%if %length(&CSigVars) %then %do;

   %put Computing PAGE tests.;

   * load sigvars into macro variables;
   %let s=1;
   %let maxlensv = 0;
   %do %while(%length(%scan(&CSigVars,&s,' ')));
      %let sv&s = %scan(&CSigVars,&s,' ');
      %let lensvs = %length(&&sv&s);
      %if (&lensvs > &maxlensv) %then %let maxlensv = &lensvs;
      %let s = %eval(&s+1);
   %end;
   %let nsv = %eval(&s-1);

   * compute n and mean for each sigvar and category combination;
   proc sort data=cencat;
      by cat;
   run;
   
   proc means data=cencat n mean noprint;
      by cat;
      var &CSigVars;
      output out=cm mean=&CSigVars;
   run;

   %if &syserr>4 %then %goto exit;
   
   data cm;
      set cm;
      if _freq_ < &NHits then delete;
      drop _type_;
   run;

   /*
   title "cm";
   proc print data=cm(obs=100);
   run;
   */
   
   * overall mean and std for each sigvar;
   proc means data=cen mean std noprint;
      var &CSigVars;
      output out=oms 
         mean=%do s=1 %to &nsv; Mean&s %end;
         std=%do s=1 %to &nsv; Std&s %end;
      ;
   run;
   
   data _null_;
      set oms;
      %do s=1 %to &nsv;
         call symput("Mean&s",trim(left(Mean&s)));
         call symput("Std&s",trim(left(Std&s)));
      %end;
   run;  
   
   data c;
      length &TrtName.I $ &maxlensv Category $ &maxlen; 
      set cm;
      %do s=1 %to &nsv;
         &TrtName.I = "&&sv&s";
         Category = cat;
         Category_Mean = &&sv&s;
         Overall_Mean = &&Mean&s;
         Overall_Std = &&Std&s;
         Z_Score = (Category_Mean - Overall_Mean)*sqrt(_freq_)/Overall_Std;
         Raw_p = min(1,2*(1 - probnorm(abs(Z_Score))));
         output;
      %end;
      label &TrtName.I = "Continuous Significance Variable" Category ="&CatLabel"
         Category_Mean = "Category Mean &RankLab" Overall_Mean = "Overall Mean &RankLab" 
         Overall_Std = "Overall Std &RankLab"  
         Z_Score = "Z Score" 
      ;
      drop cat %do s=1 %to &nsv; &&sv&s %end; ;
      rename _freq_ = Frequency;
   run;

   * apply multiple testing adjustments;
   %if %length(&MultipleTestingMethods) %then %do;
   
      %include "&MacroPath./MultipleTestingAdjustment.sas";

      data c0;
         set c;
      run;

      %do s=1 %to &nsv;
         data c1;
            set c0;
            where &TrtName.I = "&&sv&s";
         run;
         %let m=1;
         %do %while(%length(%scan(&MultipleTestingMethods,&m,' ')));
            %let MultipleTestingMethod = %scan(&MultipleTestingMethods,&m,' ');
            %AdjPValueName(method=&MultipleTestingMethod);
            %MultipleTestingAdjustment(data=c1,method=&MultipleTestingMethod,out=c1);
            %let m = %eval(&m+1);
         %end;
         %if (&s=1) %then %do;
            data c;
               set c1;
            run;
         %end;
         %else %do;
            proc append base=c data=c1;
            run;
         %end;
      %end;
   %end;
      
   data OutLib.&IncidenceScreen_COutDataName;
      set c;
      if (Raw_p > &PThresh) then delete;
   run;

   %NObsVars(OutLib.&IncidenceScreen_COutDataName);

   %if (&nobs = 0) %then %do;
      %put [MS_S_0128];
      %put ERROR: No categories passed the specified enrichment thresholds for PAGE tests.;
      %let exiterror = 1;
      %goto exit;
   %end;

   proc sort data=OutLib.&IncidenceScreen_COutDataName;
      by Ratio &TrtName.I Raw_p;
   run;

%end;


%PrepOutFile(IncidenceScreenScript,IncidenceScreenScriptName,&StudyName &ProcessName Script for &domain..jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &IncidenceScreenScript;
%let FullAPName=%upcase(&domain) &ScreenType Screen;
%if %upcase(&domain) = AE %then %let FullAPName = &FullAPName (&AETermLevel);
%if %upcase(&domain) = MEDDRA %then %do;
   %if %upcase(&MedDRATermLevel) =STANDARD MEDDRA QUERY %then %let MedDRATermLevel2=SMQ;
   %if %upcase(&MedDRATermLevel) =HIGHER LEVEL TERM %then %let MedDRATermLevel2=HLT;
   %if %upcase(&MedDRATermLevel) =HIGHER LEVEL GROUP TERM %then %let MedDRATermLevel2=HLGT;
   %if %symexist(MedDRATermLevel2) %then %let FullAPName = AE Incidence Screen (&MedDRATermLevel2);
%end;
%if ^%symexist(TotLvls) or ^%length(&TotLvls) %then %let TotLvls = 1;

%CheckLabel(InLib.&InData,TrtName);
%put TrtName = &TrtName;
%let treemap = %length(&TreeData);
%let datafilter = %length(&GroupVar.&TimeWindows);
%if &domain = SMQ %then %do;
   %let datafilter = 1;
%end;
%put DataFilter = &datafilter;
%if %length(&GroupVar) %then %do;
   %CheckLabel(OutLib.&IncidenceScreen_BOutDataName,GroupVar);
%end;

* create JSL code;
data _null_;
   file "&IncidenceScreenScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put '//!';
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:clientoutpath = ""&ClientOutPath"";";
   put "objRefNS:clientoutpath2 = ""&ClientOutPath"";";
   put "objRefNS:serveroutpath = ""&ServerOutPath"";";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";   
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   %if %upcase(&domain) = SMQ %then %do;
      put "objRefNS:NoteDomain = ""%upcase(AE)"";";   
   %end;
   %else %do;
      put "objRefNS:NoteDomain = ""%upcase(&domain)"";"; 
   %end;
   put "try(objRefNS:WhereFilters="""");";
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&AltWhereStmt)]\"");";
   
   
   %if %upcase(&ProcessName) = AEINCIDENCESCREEN %then %do;
      put " objRefNS:includestring= ""Adverse Events: "";"; 
      %let DomainWhereStmt=&AEWhereStmt;
   %end;
   %else %if %upcase(&ProcessName) = AERESOLUTIONSCREEN %then %do;
      put "objRefNS:includestring=""Adverse Events: "";";
      %let DomainWhereStmt = &AEWhereStmt;
      put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&AEWhereStmt)]\"");";
   %end;
   %else %if %upcase(&ProcessName) = EVENTSINCIDENCESCREEN %then %do;
      put " objRefNS:includestring=""Events: "";";
      put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&EventsWhereStmt)]\"");";
      %let DomainWhereStmt=&EventsWhereStmt;
   %end;
   %else %do;
      put " objRefNS:includestring=""Interventions: "";";
   %end;
   
         
   %if %length(&WhereForReports) %then %do;
      put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
      put "try(objRefNS:WhereFilters=""AdditionalFilter to include Subjects: "" || objRefNS:WhereState);";
      %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;
   %end;
   %else %if %length(&DomainWhereStmt) %then %do;
      put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
   %end; 
   
   
   %if %length(&BSigVars) %then %do;
      put "objRefNS:macropath = ""&MacroPath"";";
      put "objRefNS:domain = ""&domain"";";
      put "objRefNS:idata = ""&OrigInData"";";
      put "objRefNS:istacked = 1;";
      put "objRefNS:ciprefix = ""&CIPrefix"";";
      put "objRefNS:catname = ""&CatName"";";
      put "objRefNS:catlabel = ""&CatLabel"";";
      put "objRefNS:abbrev = ""&Abbrev"";";
      put "objRefNS:alreadystacked = &AlreadyStacked;";
      put "objRefNS:responsevar = ""&ResponseVar"";";
      put "objRefNS:suffix = ""&suffix"";";
      put "objRefNS:ls_product = ""&LS_Product"";";
      put "objRefNS:nbsv = &nbsv;";
      put "objRefNS:bsv = {};";
      %do i=1 %to &nbsv;
         put "InsertInto(objRefNS:bsv,""&&bsv&i"");";
      %end;
      put "objRefNS:trtname = ""&TrtName"";";
      put "if(IsEmpty(jg:LSIndex),jg:LSIndex = 3);";
      put "include(""$JMP_HOME\LifeSciences\JSLFiles\Category Drill Down Expressions.jsl"");";
      put "include(""$LS_HOME\JSLFILES\DrillDownReport.JSL"");";
      %if &_BObs_ > 0 %then %do;
         put "objRefNS:cenbdata = Open(""&ClientOutPath.&IncidenceScreen_BOutDataName..sas7bdat"",invisible);";
         put "objRefNS:cenbdata << Color By Column(:Name(""&trslog10p""),ColorTheme(""&ColorTheme""));";
        	put "PadColorRows = objRefNS:cenbdata << SelectWhere(OR(:Name(""&CatLabel"") ==""&trNone"",:Name(""&CatLabel"") ==""ZZ_Low"", :Name(""&CatLabel"") == ""ZZ_High""));";
         put "if (NItems(As List(objRefNS:cenbdata << Get Selected Rows())) > 0,";
         put "    PadColorRows << Delete Rows;";
         put ");";
         %if %length(&TimeWindows) %then %do;
            put "timecol = Try(tcol = Column(""&TimeScale"");1,0);";
            put "if(timecol, tcol << Set Property(Value Ordering, {";
            %do t = 1 %to &ntw-1;
               put " ""&&tw&t"", ";
            %end;
            put " ""&&tw&ntw""});";
            put ");";
         %end;
         put "Column(""&CatLabel"") << Label(1);";
         put "cutoff = &NegLog10pCutoff;";
         put "currentdatatable(objRefNS:cenbdata);";
         put "for each row(";
         put "   if (:Name(""&trneglog10rawpv"")>=cutoff,labeled(rowstate())=1)";
         put ");";
      %end;
      %if &DoubleFDR = yes %then %do;
         put "objRefNS:cengdata = Open(""&ClientOutPath.&IncidenceScreen_GOutDataName..sas7bdat"",invisible);";
         put "objRefNS:cengdata << Color By Column(:Name(""&trslog10p""),ColorTheme(""&ColorTheme""));";
         put "Column(""&GroupVar"") << Label(1);";
         %if %length(&TimeWindows) %then %do;
            put "timecol = Try(tcol = Column(""&TimeScale"");1,0);";
            put "if(timecol, tcol << Set Property(Value Ordering, {";
            %do t = 1 %to &ntw-1;
               put " ""&&tw&t"", ";
            %end;
            put " ""&&tw&ntw""});";
            put ");";
         %end;       
         put "groupcutoff = &NegLog10pGroupCutoff;";
         put "currentdatatable(objRefNS:cengdata);";
         put "for each row(";
         put "   if (:Name(""&trneglog10rawpv"")>=groupcutoff,labeled(rowstate())=1;)";
         put ");";
      %end;
      put "objRefNS:fsize = 552;";
      put 'objRefNS:msize = 4;';
      put "objRefNS:dbv = {};";
      put "objRefNS:tvratio = 1.392;";
      put "objRefNS:thratio = 1.366;";
      put "objRefNS:lhratio = 1.14;";
      %do i=1 %to &ndbv;
         put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
      %end;    
      
      /* kemicl add loop over function creation to create separate functions with unique report names for sliders */
      %if &DoubleFDR = yes %then %let nbptabs = 2;
      %else %let nbptabs = 1;
      %if %length(&TimeWindows) and &ntw > 1 %then %let nbptabs = %eval(2*&nbptabs);
      
      %do f=1 %to &nbptabs;
         * functions to draw bubble plots;
         put "objRefNS:vp&f = Function({animate,grplev, dt,tm,df,cutoff},{default local},";
         put "   CurrentDataTable(dt);";
         /*kemicl added for animated bbplots with Time windows */
         put "if(animate,";
         put "    objRefNS:BPTimeVar = ""&TimeScale"";";
         put "    if(grplev, objRefNS:BPIDVar = ""&GroupLabel0""; , objRefNS:BPIDvar = ""&CatLabel0"";);";
         put " ,";
         put "    objRefNS:BPTimeVar =Empty();";
         put "    objRefNS:BPIDvar =Empty();";
         put ");";
         put "   VListBox(";
         %if %length(&TEAEText) %then %do;
            put "objRefNS:TEAEText = TextBox(""      "" ||""&TEAEText"", <<Set Wrap(1000)),";
         %end;                  
         %if %upcase(&domain) = SMQ %then %do;
            put "      objRefNS:SMQTB = TextBox(""      "" ||locale:get(""Analysisperformed_Text"") || "": &SMQAnalysis"",),";
         %end;
         put "      HListBox(";
         put "         TextBox(""      "" || locale:fill(locale:get(""XlinedrawnatMinusLog10pX_Text""),{""&MultipleTestingMethod"", char(cutoff)})),";
         put "            HListBox(";
         /* remove marker size slider for bubble plots */
         /* 
         put '               HListBox(TextBox("     Marker Size"),';
         put '                  markerSizeSlider = SliderBox(num(-1),num(10.5),objRefNS:msize,';
         put "                        try(objRefNS:cbivrep = (objRefNS:cbiv&f<<report));";
         %if &TotFBs > 1  %then %do;
            %do i = 1 %to &TotFBs;
               put "                     try(objRefNS:cbivrep[&i][framebox(1)]<<MarkerSize(objRefNS:msize));";
            %end;
         %end;
         %else %do;
            put "                      try(objRefNS:cbivrep[framebox(1)]<<MarkerSize(objRefNS:msize));";
         %end;
         put '                  ),';
         put '               ),';
         */
         put '               HListBox(TextBox("     " || locale:get("FrameSize_Text")),';
         put '                  frameSizeSlider = SliderBox(num(150),num(750),objRefNS:fsize,';
         put "                     try(objRefNS:cbivrep = (objRefNS:cbiv&f<<report));";      
         %if &TotFBs > 1  %then %do;
            %do i = 1 %to &TotFBs;
               put "                     try(objRefNS:cbivrep[&i][framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize));";
               put "                     try(objRefNS:cbivrep[&i][framebox(2)]<<FrameSize(objRefNS:fsize*objRefNS:lhratio,0.083*objRefNS:fsize));"; 
            %end;
         %end;
         %else %do;
            put "                     try(objRefNS:cbivrep[framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize));";
               put "                     try(objRefNS:cbivrep[framebox(2)]<<FrameSize(objRefNS:fsize*objRefNS:lhratio,0.083*objRefNS:fsize));"; 
         %end;
         /*
         put "                     try(objRefNS:ctmaprep = (objRefNS:ctmap<<report));";      
         %if &TotFBs > 1  %then %do;
            %do i = 1 %to &TotFBs;
               put "                     try(objRefNS:ctmaprep[&i][treemapbox(1)]<<FrameSize(objRefNS:thratio*objRefNS:fsize,objRefNS:tvratio*0.75*objRefNS:fsize));";
            %end;
         %end;
         %else %do;
            put "                     try(objRefNS:ctmaprep[treemapbox(1)]<<FrameSize(objRefNS:thratio*objRefNS:fsize,objRefNS:tvratio*0.75*objRefNS:fsize));";
         %end;    
         */
         put "                  )";
         put "               )";
         put '            )';
         put "      ),";
         put "      HListBox(";
         put "       if(df,";
         put "     objRefNS:Filter= dt << Data Filter(<<Conditional(1),";
         put "                  Add Filter(";
         %if &domain = SMQ %then %do;
                  put "                     Columns(:Name(""&CatLabel"")),";
                  put "                     Display(:Name(""&CatLabel""), Size( 226, 400 ), List Display )";         
         %end;
         %else %do;
            %if %length(&TimeWindows) %then %do;
               %if %length(&GroupVar) %then %do;
                  put "                     Columns(:Name(""&GroupLabel0""),:Name(""&TimeScale""),:Name(""&CatLabel0"")),";
                  put "                     Display(:Name(""&GroupLabel0""), Size( 226, 250 ), List Display ),";
               %end;
               %else %do;
                  put "                     Columns(:Name(""&TimeScale""),:Name(""&CatLabel0"")),";
               %end;
               put "                     Display(:Name(""&CatLabel0""), Size( 226, 250 ), List Display ),";
               put "                     Display(:Name(""&TimeScale""), Size( 226, 100 ), List Display )";  
            %end; 
            %else %do;
               %if %length(&GroupVar) %then %do;
                  put "                     Columns(:Name(""&GroupVar""),:Name(""&CatLabel"")),";
                  put "                     Display(:Name(""&CatLabel""), Size( 226, 250 ), List Display ),";
                  put "                     Display(:Name(""&GroupVar""), Size( 226, 250 ), List Display ),";
               %end;
               %else %do;
                  put "                     Columns(:Name(""&CatLabel"")),";
                  put "                     Display(:Name(""&CatLabel""), Size( 226, 400 ), List Display )";
               %end;
            %end;
         %end;
         put "                  ),";
         put "                  Mode( Show( 1 ), Include( 1 ) )";
         put "               ),";
         put "    );";
         put "         VListBox(";
         put "           Eval(Substitute(Expr(";
         put "            objRefNS:cbiv&f = Bubble Plot(";
         put "               Y(:Name(""&trneglog10rawpv"")),";
         /* Re-translate input dialog "x-Axis for Volcano Plot" options */
         %if %nrbquote(&VolcanoXCat) eq %nrstr(Risk Difference) %then %do;
            %let VolcanoXCat = &trriskdiff;
            %let xaxislabel = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(RiskDifferenceforX_Text)), %nrbquote(&TrtLabel)));
         %end;
         %else %if %nrbquote(&VolcanoXCat) eq %nrstr(log2%(Relative Risk%)) %then %do;
            %let VolcanoXCat = &trlog2relrisk;
            %let xaxislabel = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(Log2RelativeRiskforX_Text)), %nrbquote(&TrtLabel)));
         %end;
         %else %if %nrbquote(&VolcanoXCat) eq %nrstr(log2%(Odds Ratio%)) %then %do;
            %let VolcanoXCat = &trlog2oddsratio;
            %let xaxislabel = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(Log2OddsRatioforX_Text)), %nrbquote(&TrtLabel)));
         %end;
         %else %do;
            %let xaxislabel = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XforX_Text)), %nrbquote(&VolcanoXCat), %nrbquote(&TrtLabel)));
         %end;
         /* End of re-translate */
         put "               X(:Name(""&VolcanoXCat"")),";
         put "               Sizes( :&trcount ),";
         put "               Time(BPTime),";
         put "               ID(BPID),";
         /* put "               Coloring( :Name(""Signed log10(p)"")),"; */
         put "               All Labels( 0 ),";
         put "               No Labels( 0 ),";
         put "               Trail Lines( ""All"" ),";
         %if %upcase(&SplitResults)=YES %then %do;
            put "               By(:Ratio,eval(objRefNS:dbv)),";
         %end;
         %else %do;
            put "               By(:Name(""&trratio"")),";
         %end;
         put '               SendToReport(';
         put '                  Dispatch(';
         put '                     {},';
         put '                     "2",';
         put '                     ScaleBox,';
         put '                     {Scale( Linear ), Format( "Best" ),';
         put '                     Add Ref Line( cutoff, Dashed, Red )}';
         put '                  ),';
               put "    Dispatch({},""&VolcanoXCat"",TextEditBox,{Set Text( ""&xaxislabel"" )}),";
         put '               )';
         put "            )), Expr(BPTime), objRefNS:BPTimeVar, Expr(BPID), objRefNS:BPIDVar));";
         put '         ),';
         put "         if(animate, objRefNS:cbiv&f<< All Labels(1););";
         %if &TotFBs > 1 %then %do;
         %let RepExpr&f=%nrbquote(objRefNS:TEAEText,objRefNS:Title&f.1,objRefNS:cbiv2&f.1,objRefNS:colorgb&f.1);
            %do i = 1 %to &TotFBs;
               put "         try((objRefNS:cbiv&f[&i]<<report)[framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize));";
               /* kemicl - added for legend */
               put "       try(";
               put "         objRefNS:cbivrep = objRefNS:cbiv&f[&i]<<report;";
               put "         objRefNS:bivtext = objRefNS:cbivrep[OutlineBox(1)]<<Get Title;";
               put "       objRefNS:Title&f&i=textbox(objRefNS:bivtext, <<set font size(12), <<set wrap (850));";
               put "         redvstart = munger(objRefNS:bivtext,1,""&trratio="")+&lengthtrratiopluseq;";
               put "         bluevstart = munger(objRefNS:bivtext,redvstart, """"||locale:get(""over_Text"")||"""");";
               put "         endredvar = bluevstart - redvstart - 1;";
               put "         objRefNS:RedVar&i = Trim(Substr(objRefNS:bivtext,redvstart, endredvar));";
               %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) ^= EN %then %do;
               put "         objRefNS:trtlevwhole&i = As Column(locale:get(""Ratio_Text""))[1];";
               put "         objRefNS:regOverStrng = ""(.*) ""||locale:get(""over_Text"")||"" (.*)"";";
               put "         objRefNS:trtlev&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\1"");";
               put "         objRefNS:trtlev2&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\2"");";
               put "         objRefNS:RedVar&i = objRefNS:trtlev&i;";
               put "         objRefNS:BlueVar&i = objRefNS:trtlev2&i;";
               %end;
               %else %do;
               put "         objRefNS:BlueVar&i = Trim(Word(1,Substr(objRefNS:bivtext,bluevstart +4),"",""));";
               %end;
               put "         objRefNS:colorgb&f.&i = Substitute(Expr(VListBox(Graph Box(";
               put "           Frame Size(objRefNS:fsize*objRefNS:lhratio, 0.083*objRefNS:fsize),";
               put "           X Scale(0,120),";
               put "           Y Scale(0,3),";
               put "           SuppressAxes,";
               put "           Text Color(""black""); Text Size(8);";
               /* put "           Rect(3,2.3,7,1.5,1);"; */
               put "           Text(Right Justified, {56,1.5}, locale:fill(locale:get(""RiskinX_Text""), {_bv1_}));  ";
               put "           For(_inc_=0,_inc_ < 1,_inc_+=.05,";
               put "                      xinc = 47.5+25*_inc_;";
               put "                      Fill Color(";
               put "                            cc=Heat Color(";
               put "                               _inc_,";
               put "                               <<""&ColorTheme"" ";
               put "                            )";
               put "                      );";

               put "                      Rect(xinc -0.75 , 1.3,xinc + 0.75, 0.2,1);";
               put "           );";
               put "           Text Color(""black""); Text Size(8);";
               put "           Text({64,1.5},  locale:fill(locale:get(""RiskinX_Text""), {_bv2_}));";
               /* put "           Rect(3,1,7,0.2,1);"; */
               put "         ))), Expr(_bv1_), objRefNS:BlueVar&i, Expr(_bv2_), objRefNS:RedVar&i);";
               put "         Ochild = (objRefNS:cbiv&f[&i]<<report)<<Find(LineUpBox(1));";
               put "         Ochild << Sib Prepend(objRefNS:colorgb&f.&i, Vertical);";
               put "         objRefNS:cbiv2&f.&i=(objRefNS:cbiv&f[&i]<<Report)[PictureBox(1)];";
               put "       );";  /* close try for legend  */
               %if &i>1 %then %do;
               %let RepExpr&f=&&RepExpr&f.%nrbquote(,objRefNS:Title&f.&i,objRefNS:cbiv2&f.&i,objRefNS:colorgb&f.&i);
               %end;
            %end; 
         %end;
         %else %do;
            put "         try((objRefNS:cbiv&f<<report)[framebox(1)]<<FrameSize(objRefNS:fsize,0.75*objRefNS:fsize));";
            put "         try((objRefNS:cbiv&f<<report)[AxisBox(1)]<<RemoveAxisLabel()<<RemoveAxisLabel());";
            put "         try((objRefNS:cbiv&f<<report)[AxisBox(1)]<<AddAxisLabel(locale:get(""MinusLog10Rawpvalue_Text"")));";
            /* kemicl - added for legend */
            put "       //try(";
            put "         objRefNS:cbivrep = objRefNS:cbiv&f<<report;";
            put "         objRefNS:bivtext = objRefNS:cbivrep[OutlineBox(1)]<<Get Title;";
            put "       objRefNS:Title&f=textbox(objRefNS:bivtext, <<set font size(12), <<set wrap (850));";
            put "         redvstart = munger(objRefNS:bivtext,1,""&trratio="")+&lengthtrratiopluseq;";
            put "         bluevstart = munger(objRefNS:bivtext,redvstart, """"||locale:get(""over_Text"")||"""");";
            put "         endredvar = bluevstart - redvstart - 1;";
            put "         objRefNS:RedVar = Trim(Substr(objRefNS:bivtext,redvstart, endredvar));";
            %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) ^= EN %then %do;
            put "         objRefNS:trtlevwhole = As Column(locale:get(""Ratio_Text""))[1];";
            put "         objRefNS:regOverStrng = ""(.*) ""||locale:get(""over_Text"")||"" (.*)"";";
            put "         objRefNS:trtlev = Regex(objRefNS:trtlevwhole,objRefNS:regOverStrng, ""\1"");";
            put "         objRefNS:trtlev2 = Regex(objRefNS:trtlevwhole,objRefNS:regOverStrng, ""\2"");";
            put "         objRefNS:RedVar = objRefNS:trtlev;";
            put "         objRefNS:BlueVar = objRefNS:trtlev2;";
            %end;
            %else %do;
            put "         objRefNS:BlueVar = Trim(Word(1,Substr(objRefNS:bivtext,bluevstart +4),"",""));";
            %end;
            put "        objRefNS:colorgb&f = Substitute(Expr(VListBox(Graph Box(";
            put "           Frame Size(objRefNS:fsize*objRefNS:lhratio, 0.083*objRefNS:fsize),";
            put "           X Scale(0,120),";
            put "           Y Scale(0,3),";
            put "           SuppressAxes,";
            put "           Text Color(""black""); Text Size(8);";
            /* put "           Rect(3,2.3,7,1.5,1);"; */
            put "           Text(Right Justified, {56,1.5}, locale:fill(locale:get(""RiskinX_Text""), {_bv1_}));  ";
            put "           For(_inc_=0,_inc_ < 1,_inc_+=.05,";
            put "                      xinc = 47.5+25*_inc_;";
            put "                      Fill Color(";
            put "                            cc=Heat Color(";
            put "                               _inc_,";
            put "                               <<""&ColorTheme"" ";
            put "                            )";
            put "                      );";        
            put "                      Rect(xinc -0.75 , 1.3,xinc +0.75, 0.2,1);";
            put "           );";
            put "           Text Color(""black""); Text Size(8);";
            put "           Text({64,1.5},  locale:fill(locale:get(""RiskinX_Text""), {_bv2_}));";
            put "         ))), Expr(_bv1_), objRefNS:BlueVar, Expr(_bv2_), objRefNS:RedVar);";
            put "         Ochild = (objRefNS:cbiv&f<<report)<<Find(LineUpBox(1));";
            put "         Ochild << Sib Prepend(objRefNS:colorgb&f, Vertical);";
            put "         objRefNS:cbiv2&f=(objRefNS:cbiv&f<<Report)[PictureBox(1)];";
            put "       //);"; /* close try for legend*/
            %let RepExpr&f=%nrbquote(objRefNS:Title&f,objRefNS:cbiv2&f,objRefNS:colorgb&f);
         %end;
         put "               TextBox(""\!N \!N""),";        
         put '      )';
         put '   ),';   
         put ');';

      %end;
   %end;
   %if %length(&CSigVars) %then %do;
      put "cencdata = Open("".\&IncidenceScreen_COutDataName..sas7bdat"");";
   %end;
   %if %length(&ASigVars) %then %do;
      put "cenadata = Open("".\&IncidenceScreen_AOutDataName..sas7bdat"");";
   %end;
   
   %if %length(&TimeWindows) %then %do;
      %let TabGLvar = &GroupLabel0;
      %let TabCLvar = &CatLabel0;
   %end;
   %else %do;
      %let TabGLvar = &GroupVar;
      %let TabCLvar = &CatLabel;
   %end;
   
   %let usef = 1;
   
   
      %if &DoubleFDR = yes %then %do;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:cengdata""}});";
      put "InsertInto(objRefNS:DataNameList,{{""%upcase(&domain) &TabGLvar""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XResults_Text""),{""&TabGLvar""})));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:vp&usef.(0,1,objRefNS:cengdata,0,&datafilter,&NegLog10pGroupCutoff)));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      %if %upcase(&domain) = SMQ %then %let RepExpr&usef = objRefNS:SMQTB, &&RepExpr&usef;
      put "InsertInto(objRefNS:CreateReportList,{{&&RepExpr&usef}});";     
      %let usef = %eval(&usef +1);
   %end;

   %if &_BObs_ > 0 %then %do;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:cenbdata""}});";
      %if %upcase(&domain) ne SMQ %then %do;
         put "InsertInto(objRefNS:DataNameList,{{""%upcase(&domain) &TabCLvar""}});";
      %end;
      %else %do;
         put "InsertInto(objRefNS:DataNameList,{{""&TabCLvar""}});";
      %end;
      put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XResults_Text""),{""&TabCLvar""})));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:vp&usef.(0,0,objRefNS:cenbdata,1,&datafilter,&NegLog10pCutoff)));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      %if %upcase(&domain) = SMQ %then %let RepExpr&usef = objRefNS:SMQTB, &&RepExpr&usef;
      put "InsertInto(objRefNS:CreateReportList,{{&&RepExpr&usef}});";
      %let usef = %eval(&usef +1);
   %end;
   
   %if %length(&TimeWindows) and &ntw > 1 %then %do;
      %if &DoubleFDR = yes %then %do;
         put "InsertInto(objRefNS:DataList,{{""objRefNS:cengdata""}});";
         put "InsertInto(objRefNS:DataNameList,{{""%upcase(&domain) &TabGLvar""}});";
         put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XExplodingVolcano_Text""),{""&TabGLvar""})));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:vp&usef.(1,1,objRefNS:cengdata,0,0,&NegLog10pGroupCutoff)));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         %if %upcase(&domain) = SMQ %then %let RepExpr&usef = objRefNS:SMQTB, &&RepExpr&usef;
         put "InsertInto(objRefNS:CreateReportList,{{&&RepExpr&usef}});";
         %let usef = %eval(&usef +1);
      %end;
      %if &_BObs_ > 0 %then %do;
         put "InsertInto(objRefNS:DataList,{{""objRefNS:cenbdata""}});";
         %if %upcase(&domain) ne SMQ %then %do;
            put "InsertInto(objRefNS:DataNameList,{{""%upcase(&domain) &TabCLvar""}});";
         %end;
         %else %do;
            put "InsertInto(objRefNS:DataNameList,{{""&TabCLvar""}});";
         %end;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XExplodingVolcano_Text""),{""&TabCLvar""})));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:vp&usef.(1,0,objRefNS:cenbdata,0,0,&NegLog10pCutoff)));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         %if %upcase(&domain) = SMQ %then %let RepExpr&usef = objRefNS:SMQTB, &&RepExpr&usef;
         put "InsertInto(objRefNS:CreateReportList,{{&&RepExpr&usef}});";
      %end;
   %end;
   
   * Action Buttons;
   put "objRefNS:ActionButtonDesc = {expr(locale:get(""SelectPointsinPlotsthenClick_Text""))};";
   put "objRefNS:ActionButtonList = {{}};";
   put "objRefNS:ActionCodeList = {{}};";
   put "objRefNS:ActionDataTableList = {{}};";
   put "objRefNS:ActionIconList = {{}};";
   put "objRefNS:ActionRequirementFlagList = {{}};";
   put "objRefNS:ActionButtonsperRow = {};";
   
   * Dot Plot;
   %do i=1 %to &TotFBs;
      put "objRefNS:GBDPfunc&i = EvalExpr(Function({},{objRefNS= Namespace(expr(objRefNS<<GetName))},";
      put "   objRefNS:dpvlb&i<<prepend(Vlistbox(objRefNS:dpgb&i = Graph Builder(";
      put "   Size( 735, 630 ),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "      X( :Name(""&trcl1p"") ),";
      put "      X( :Name(""&trcl2p""), Position( 1 ) ),";
      put "      X( :&trriskdiff ),";
      put "      X( :&trrelrisk ),";
      put "      X( :&troddsratio ),";
      put "      Y( Column(""&CatLabel""), Order By( Column(objRefNS:sdp&i<<Get Selected),Order Statistic( ""Mean"" )), Size( 220 ) ),";
      %if &ndbv %then %do;
         put "      Wrap(Column(""&dbv1"")),";
         %do j=2 %to &ndbv;
            put "      GroupY(Column(""&&dbv&j"")),";
         %end;
      %end;
      put "   ),";
      put "   Elements( Position( 1, 1 ), Points( X( 1 ), X( 2 ), Y, Legend( 2 ), Jitter( 0 ) ) ),";
      put "   Elements( Position( 2, 1 ), Points( X, Y, Legend( 4 ), Jitter( 0 ) ) ),";
      put "   Elements( Position( 3, 1 ), Points( X, Y, Legend( 5 ), Jitter( 0 ) ) ),";
      put "   Elements( Position( 4, 1 ), Points( X, Y, Legend( 7 ), Jitter( 0 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch({},:Name(""&trcl1p""),ScaleBox,";
      put "       {Min( -3 ), Max( %eval(&maxpercent + 3) ), Inc( 10 ), Minor Ticks( 0 ),";
      put "       Rotated Labels( ""Automatic"" )}";
      put "    ),";
      put "    Dispatch(";
      put "        {},";
      put "        ""&CatLabel"",";
      put "        ScaleBox,";
      put "        {Show Major Grid( 1 ), Rotated Labels( ""Automatic"" )}";
      put "    ),";
      put "    Dispatch(";
      put "     {},";
      put "     ""400"",";
      put "     ScaleBox,";
      put "     {Legend Model(";
      put "       2,";
      put "       Level Name( 0, objRefNS:trtlev&i ),";
      put "       Level Name( 1, objRefNS:trtlev2&i),";
      put "       Properties( 0, {Line Color( Heat Color(1,""&ColorTheme"")), Marker( ""Triangle"" )} ),";
      put "       Properties( 1, {Line Color( Heat Color(0,""&ColorTheme"")), Marker( ""Circle"" )} )";
      put "     )}";
      put "    ),";
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( locale:fill(locale:get(""XDotPlotforX_Text""), {""%upcase(&domain)"", objRefNS:trtlevwhole&i}) )} ),";
      put "    Dispatch( {}, ""400"", LegendBox, {Set Title( ""&TrtLabel"" )} ),";
      put "    Dispatch( {}, ""X title"", TextEditBox, {Set Text( locale:fill(locale:get(""PercentX_Text""), {""%upcase(&Domain)""}) )} ),";
      put "  Dispatch( {}, ""X 1 title"", TextEditBox, {Set Text( locale:get(""RiskDifference_Text""))} ),";
      put "  Dispatch( {}, ""X 2 title"", TextEditBox, {Set Text( locale:get(""RelativeRisk_Text""))} ),";
      put "  Dispatch( {}, ""X 3 title"", TextEditBox, {Set Text( locale:get(""OddsRatio_Text""))} ),";
      put "    Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&CatLabel"" )}     ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(1), {Marker Size( 2 )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(2), {Marker Size( 2 )} ),";   
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(3), {Marker Size( 2 )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(4), {Marker Size( 2 )} ),";      
      put "   ) //end send to report";
      put " )));";
      put "objRefNS:dpgb&i << On Close(local({objRefNS=namespace(expr(objRefNS<<getName))}, try(objRefNS:dpdata&i<<Close Window)));";
      put "));";
   %end;
   
   put "objRefNS:firstdp = 1;";
   put "objRefNS:dotplotfunc=EvalExpr(Function({},{default local},objrefNS = Namespace(expr(objrefns<<getName));";
   put "  if(!isempty(objRefNS:cengdata),";
   put " if(Current Data Table() == objRefNS:cengdata,";
   put "    objRefNS:glist = {};";
   put "    objRefNS:GVar = Column(""&TabGLvar"");";
   put "    for each row(if(selected(rowstate())==1,";
   put "       InsertInto(objRefNS:glist, objRefNS:GVar()[]);";
   put "    ));";
   put "    if(nItems(objRefNS:glist),";
   put "       Current Data Table(objRefNS:cenbdata);";
   put "       objRefNS:cenbdata<<SelectWhere(contains(objRefNS:glist, :Name(""&TabGLvar"")));";
   put "       objRefNS:nsel = 0;";
   put "       For Each Row(if(selected(rowState()),objRefNS:nsel++;));";
   put "       if(objRefNS:nsel ==0, Throw(locale:get(""NoSignificantTermLevelTestsForSelectedGroupLevels_Text"")));";
   put "    );";
   put " ));"; 
   put " objRefNS:cenbdata << Clear Column Selection();";
   put " CurrentDataTable(objRefNS:cenbdata);";
   put " Summarize(objRefNS:ratiolist = By(:Name(""&trratio"")));";
   put " objRefNS:cenbdatasub = objRefNS:cenbdata << Subset(Selected rows(1),Output table name(""&IncidenceScreen_BOutDataName._dp""),invisible);";
   put " Try(CurrentDataTable(objRefNS:cenbdatasub);,";
   put " objRefNS:errwin = New Window(locale:fill(locale:get(""XMessage_Text""), {locale:get(""DrillDown_Text"")}), <<modal,";
   put "    TextBox(locale:fill(locale:get(""ErrorNoRowsSelectedMakeSureTableWithXActive_Text""), {""&CatLabel""})),";
   put "    jg:messageHelp(""ErrorNoRowsSelectedMakeSureTableWithXActive_Text""),";
   put "    widgetHandle = ButtonBox(locale:get(""OK_Text""))";
   put " );";
   put " Throw(););"; 
   %do i=1 %to &TotFBs;
      put "  Try(objRefNS:DotPlotDDWin&i<<Close Window);";
      put " try(";
      put " objRefNS:cenbdatasub<<ClearSelect;";
      put " objRefNS:cenbdatasub<<SelectWhere(:&trratio == objRefNS:ratiolist[&i]);";
      put " objRefNS:nsel = NRow(objRefNS:cenbdatasub<<GetSelectedRows);";
      put " if(objRefNS:nsel,";
      put "  objRefNS:dpdata&i = objRefNS:cenbdatasub<<Subset(Selected rows(1),Output table name(""&IncidenceScreen_BOutDataName._dp &i""),invisible);";
      put "  if(!isEmpty(objRefNS:dpdata&i),";
      put "   Current Data Table(objRefNS:dpdata&i);";
      put "   objRefNS:trtlevwhole&i = As Column(locale:get(""Ratio_Text""))[1];";
      /*kemicl fix for S1141352 */
      put " objRefNS:regOverStrng = ""(.*) ""||locale:get(""over_Text"")||"" (.*)"";";
      put " objRefNS:trtlev&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\1"");";
      put " objRefNS:trtlev2&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\2"");";    
      put " objRefNS:DPTitle&i= locale:fill(locale:get(""XDotPlotforX_Text""), {""%upcase(&domain)"", objRefNS:trtlevwhole&i});";
      put "   //show(objRefNS:DPcreateReport);";
      put "   objRefNS:DotPlotDDWin&i = New Window(objRefNS:DPTitle&i,";
      put "    eval(parse(objRefNS:DrillDownReport(objRefNS:DPTitle&i,""objRefNS:dpvlb&i""))),";
      put "    objRefNS:dpvlb&i=VListBox(";     
      put "     Hlistbox(";
      put "       TextBox(""                 ""),";
      put "       PanelBox(locale:get(""SelectColumnToOrderTermsInPlot_Text""),";
      put "        objRefNS:RBL = Eval List({""&CatLabel"",""&trcount"",""&trriskdiff"",""&trrelrisk"",""&troddsratio"",""&trslog10p""});";
      put "        objRefNS:sdp&i=RadioBox(objRefNS:RBL,<<Set(4),local({objrefNS = Namespace(expr(objrefns<<getName))}, if(objRefNS:firstdp&i==0,objRefNS:dpgb&i << Delete; objRefNS:GBDPFunc&i())));";
      put "       )";
      put "     );";
      put "    );";
      put "   );";
      put " If(Is Empty(objRefNS:DrillDownWindowList),";
      put "   objRefNS:DrillDownWindowList = {};";
      put " );";   
      put " InsertInto(objRefNS:DrillDownWindowList,objRefNS:DotPlotDDWin&i);";
      put "   currwin = objRefNS:DotPlotDDWin&i;";
      put "   objRefNS:dpdata&i << Clear Row States ;";
      put "   objRefNS:dpdata&i<<Select Where(is missing(:&trrelrisk));";
      put "   objRefNS:anysel = objRefNS:dpdata&i << Get Selected Rows;";
      put "   if(NRow(objRefNS:anysel) > 0,objRefNS:dpdata&i<<Delete Rows);"; 
      put "   if(NRow(objRefNS:dpdata&i)==0,Close(objRefNS:dpdata&i,NoSave); currwin<<Close Window; throw(locale:fill(locale:get(""UnableToConstructDotPlotXDueToZeroMissing_Text""), {""&i""})));";
      put "   objRefNS:GBDPFunc&i();";
      put "   objRefNS:firstdp&i = 0;";
      put "   Insert Into(objRefNS:TabWinList,objRefNS:dpdata&i);";  
      put "   objRefNS:nwcounter++;";
      put "   objRefNS:dpdata&i << Show Window(0);";     
      put "  ); //end data exists";
      put " );";
      put ");";
   %end;
   put " Try(Close(objRefNS:cenbdatasub,NoSave));";
   put "));";         
   %if &_BObs_ > 0 %then %do;
      put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""DotPlot_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[1],expr(objRefNS:dotplotfunc()));";
      put "InsertInto(objRefNS:ActionDataTableList[1],{""objRefNS:cenbdata""});";
      put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";
      put "InsertInto(objRefNS:ActionButtonsperRow,1);";
      put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.GridDot.gif"");";      
   %end;
   
   * Relative Risk Plot;
   %do i=1 %to &TotFBs;
      put "objRefNS:GBRRfunc&i = EvalExpr(Function({dt&i},{default local},";
      put "   objRefNS= Namespace(expr(objRefNS<<GetName));";
      put "   CurrentDataTable(dt&i);";
      put "   objRefNS:rrvlb&i<<prepend(objRefNS:rrgb&i = Graph Builder(";
      put "   Size( 735, 630 ),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "      X( :Name(""&trcl1p"") ),";
      put "      X( :Name(""&trcl2p""), Position( 1 ) ),";
      put "      X( Column(""&trlog2relrisk"") ),";
      put "      X( Column(""Lower 95% log2(RR)""), Position( 2 ) ),";
      put "      X( Column(""Upper 95% log2(RR)""), Position( 2 ) ),";
      put "      Y( Column(""&CatLabel""), Order By( Column(objRefNS:srr&i<<Get Selected),  Order Statistic( ""Mean"" )), Size( 220 ) ),";
      put "   ),";
      put "   Elements( Position( 1, 1 ), Points( X( 1 ), X( 2 ), Y, Legend( 2 ), Jitter( 0 ) ) ),";
      put "   Elements( Position( 2, 1 ), Points( X( 1 ), X( 2 ), X( 3 ), Y, Legend( 4 ), Jitter( 0 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch({},:Name(""&trcl1p""),ScaleBox,";
      put "       {Min( -3 ), Max( %eval(&maxpercent + 3) ), Inc( 10 ), Minor Ticks( 0 ),";
      put "       Rotated Labels( ""Automatic"" )}";
      put "    ),";  
      put "    Dispatch( {}, ""&trlog2relrisk"", ScaleBox, {Add Ref Line( 0, Dotted, ""Black"" ), Rotated Labels( ""Automatic"" )} ),";

      put "    Dispatch(";
      put "        {},";
      put "        ""&CatLabel"",";
      put "        ScaleBox,";
      put "        {Show Major Grid( 1 ), Rotated Labels( ""Automatic"" )}";
      put "    ),";
      put "    Dispatch(";
      put "     {},";
      put "     ""400"",";
      put "     ScaleBox,";
      put "     {Legend Model(";
      put "       2,";
      put "       Level Name( 0, objRefNS:trtlev&i ),";
      put "       Level Name( 1, objRefNS:trtlev2&i ),"; 
      put "       Properties( 0, {Line Color( Heat Color(1,""&ColorTheme"") ), Marker( ""Triangle"" )} ),";
      put "       Properties( 1, {Line Color( Heat Color(0,""&ColorTheme"") ), Marker( ""Circle"" )} )";
      put "      ),";
      put "      Legend Model(";
      put "       4,";
      put "       Properties( 0, {Line Color( 0 ), Marker( ""FilledCircle"" )} ),";
      put "       Properties( 1, {Line Color( 0 ), Marker( ""Pipe"" )} ),";
      put "       Properties( 2, {Line Color( 0 ), Marker( ""Pipe"" )} )"; 
      put "     )}"; 
      put "    ),";
      put "    Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( locale:get(""Log2RelativeRiskPlot_Text"") )} ),";     
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""400"", LegendBox, {Set Title( ""&TrtLabel"" ), Position( {0, 1, -1, -1, -1}  )}),";
      put "    Dispatch( {}, ""X title"", TextEditBox, {Set Text( locale:fill(locale:get(""PercentX_Text""), {""%upcase(&Domain)""}))} ),";
      put "    Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&CatLabel"" )}     ),";
      put "    Dispatch( {}, ""X 1 title"", TextEditBox, {Set Text( locale:get(""Log2RelativeRiskWith95PercentCI_Text""))} ),";  
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(1), {Marker Size( 2 )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(2), {Marker Size( 2 )} ),";

      put "   ) //end send to report";
      put " )); //end graph builder and prepend";
      put " objRefNS:rrhlb&i<<append(objRefNS:tb&i = Tabulate(Show Control Panel(0),";
      put "    Include missing for grouping columns( 1 ),";
      put "    Add Table(Row Table(Grouping Columns(";
      %if %length(&GroupVar) %then %do;
         %if %length(&TimeWindows) %then %do;
            put " :Name(""&GroupLabel0""),";
         %end;
         %else %do;
            put " :Name(""&GroupVar""),";
         %end;
      %end;
      put "       :Name(""&CatLabel""),Column(objRefNS:NColNameRR1&i), Column(objRefNS:NColNameRR2&i), Column(objRefNS:NColNameRR3&i)";
      put "    ))),";
      put "    SendToReport(Dispatch({},""Tabulate"",OutlineBox,{Set Title(locale:get(""RelativeRiskTable_Text""))}))";
      put " );";
      put "); //end append ";
      put " try(objRefNS:tbr = objRefNS:tb&i<<Report);";
      put " try(objRefNS:tbr[MultiTblNumColBox(1)]<<delete;);";
      %if %length(&GroupVar) %then %do;
         put " try(objRefNS:tbr[MultiTblStrColBox(1)]<<Set Width(240););";
         %if "&GroupVar" ne "&CatLabel" %then %do;
            put " //try(objRefNS:tbr[MultiTblStrColBox(2)]<<Set Width(220););";  
         %end;
      %end;
      %else %do;
         put " try(objRefNS:tbr[MultiTblStrColBox(1)]<<Set Width(210););";
      %end;
      put " objRefNS:rrgbr&i = Report(objRefNS:rrgb&i);";
      put " objRefNS:yheight&i = NRows(dt&i) + 1;";
      put " objRefNS:rrfuncdt&i = dt&i;";
      put " try(objRefNS:rrgbr&i[framebox(2)] << AddGraphicsScript(";
      put "  local({objRefNS= Namespace(expr(objRefNS<<GetName))},";
      put "  CurrentDataTable(objRefNS:rrfuncdt&i);";
      put "  for each row(";
      put "   Pen Color( ""Black"" );";
      put "   Line({Column(""Lower 95% log2(RR)"")[],row()-1},{Column(""Upper 95% log2(RR)"")[],row()-1});";
      put "  );";
      put "  Transparency( 0.075 );";
      put "  Fill Color( Heat Color(0,""&ColorTheme"")  );";
      put "  Rect( {-50, -5}, {0, objRefNS:yheight&i}, Fill = 1 );";
      put "  Fill Color( Heat Color(1,""&ColorTheme"")  );";
      put "  Rect( {0, objRefNS:yheight&i}, {50, -5}, Fill = 1 );";
      put " )));";
      put " objRefNS:rrgb&i << On Close(local({objRefNS= Namespace(expr(objRefNS<<GetName))}, try(objRefNS:rrfuncdt&i<<Close Window)));";
      put "));";   
   %end;
   /* End relative Risk function generation */
   
   put "objRefNS:rrplotfunc=EvalExpr(Function({},{default local},";
   put " objRefNS= Namespace(expr(objRefNS<<GetName));";
   put " if(!isempty(objRefNS:cengdata),";
   put " if(Current Data Table() == objRefNS:cengdata,";
   put "    objRefNS:glist = {};";
   put "    objRefNS:GVar = Column(""&TabGLvar"");";
   put "    for each row(if(selected(rowstate())==1,";
   put "       InsertInto(objRefNS:glist, objRefNS:GVar()[]);";
   put "    ));";
   put "    if(nItems(objRefNS:glist),";
   put "       Current Data Table(objRefNS:cenbdata);";
   put "       objRefNS:cenbdata<<SelectWhere(contains(objRefNS:glist, :Name(""&TabGLvar"")));";
   put "       objRefNS:nsel = 0;";
   put "       For Each Row(if(selected(rowState()),objRefNS:nsel++;));";
   put "       if(objRefNS:nsel ==0, Throw(locale:get(""NoSignificantTermLevelTestsForSelectedGroupLevels_Text"")));";
   put "    );";
   put " ));"; 
   put " objRefNS:cenbdata << Clear Column Selection();";
   put " CurrentDataTable(objRefNS:cenbdata);";
   %if %upcase(&SplitResults)=YES %then %do;
      put "objRefNS:blist1 = ""Column(\!""Ratio\!"")"";";
      put " for(i=1,i<=NItems(objRefNS:dbv),i++,";
      put "    objRefNS:blist1 = objRefNS:blist1 || "","" || char(objRefNS:dbv[i]);";
      put " );";
      put " objRefNS:jslcode = EvalInsert(""\[Summarize( objRefNS:ratiolist = By(^objRefNS:blist1^) );]\"");";
      put " eval(parse(objRefNS:jslcode));";
   %end;
   %else %do;  
      put " Summarize(objRefNS:ratiolist = By(:Name(""&trratio"")));";
   %end;
   /*
   put " show(objRefNS:ratiolist);";
   */
   put " objRefNS:cenbdatasub = objRefNS:cenbdata << Subset(Selected rows(1),Output table name(""&IncidenceScreen_BOutDataName._rr""),invisible);";
   put " Try(CurrentDataTable(objRefNS:cenbdatasub);,";
   put " objRefNS:errwin = New Window(locale:fill(locale:get(""XMessage_Text""), {locale:get(""DrillDown_Text"")}), <<modal,";
   put "    TextBox(locale:fill(locale:get(""ErrorNoRowsSelectedMakeSureTableWithXActive_Text""), {""&CatLabel""})),";
   put "    jg:messageHelp(""ErrorNoRowsSelectedMakeSureTableWithXActive_Text""),";
   put "    widgetHandle = ButtonBox(locale:get(""OK_Text""))";
   put " );";  
   put " Throw(););";
   %do i=1 %to &TotFBs;
      put "  Try(objRefNS:RelativeRiskDDWin&i<<Close Window);";
      put " objRefNS:cenbdatasub<<ClearSelect;";
      %if %upcase(&SplitResults)=YES %then %do;
         put " objRefNS:whereclause = "":&trratio == objRefNS:ratiolist[1][&i]"";";
         put " objRefNS:title = ""&trratio="" || eval(objRefNS:ratiolist[1][&i]);";
         %do j=1 %to &ndbv;
            put "objRefNS:cenbdatasub:&&dbv&j << Data Type( Character);";
            put " objRefNS:whereclause ||= "" & :&&dbv&j == objRefNS:ratiolist[%eval(&j+1)][&i]"";";
            put " objRefNS:title = objRefNS:title || "", &&dbv&j="" || eval(objRefNS:ratiolist[%eval(&j+1)][&i]);";
         %end;
         put " objRefNS:cenbdatasub<<SelectWhere(eval(parse(objRefNS:whereclause)));"; 
         /*
         put " show(""by group &i"");";
         put " show(whereclause);";
         */
      %end;
      %else %do;  
         put " objRefNS:cenbdatasub<<SelectWhere(:&trratio == objRefNS:ratiolist[&i]);";
         put " objRefNS:title = eval(objRefNS:ratiolist[&i]);";
      %end;
      put " objRefNS:nsel = NRow(objRefNS:cenbdatasub<<GetSelectedRows);";
      put " if(objRefNS:nsel,";
      put "  objRefNS:cenbdatasub<<Subset(Selected rows(1),Output table name(""&IncidenceScreen_BOutDataName._rr &i""),invisible);";
      put " try(objRefNS:rrdata&i = Data Table(""&IncidenceScreen_BOutDataName._rr &i""));";
      /*
      put " show(objRefNS:rrdata&i);";
      */
      put " if(!isEmpty(objRefNS:rrdata&i),";   
      put "  objRefNS:firstrr&i = 1;";

      /* Remove multiple rows with the same term */
      put " objRefNS:rrdt&i=objRefNS:rrdata&i << Join(";
      put "    With( objRefNS:rrdata&i ),";
      put "    By Matching Columns(";
      put "    :Name(""&CatLabel"") = :Name(""&CatLabel""), ";
      put "    ),";
      put "    Drop multiples( 1, 1 ),";
      put "    Name( ""Include non-matches"" )(0, 0),";
      put "    Output table name(""&IncidenceScreen_BOutDataName._rrj &i""),";
      put "    Invisible";
      put " );";  
      put " objRefNS:rrdt1_&i = objRefNS:rrdt&i << Sort(By(Column(""&CatLabel"")),Invisible);";
      put " Close(objRefNS:rrdt&i,NoSave);";
      put " objRefNS:rrdt&i = objRefNS:rrdt1_&i;";    
      
      put " Current Data Table(objRefNS:rrdt&i);";
      put " objRefNS:trtlevwhole&i = As Column(locale:get(""Ratio_Text""))[1];";  
      put " objRefNS:ByVarList = "" "";";
      %if &TotLvls > 1 %then %do;
         %do b=1 %to &ndbv;
            put " objRefNS:ByVarList = objRefNS:ByVarList || "" &&dbv&b="" || (objRefNS:rrdata&i << Get Table Variable(""&&dbv&b"")) || "" "";";
         %end;
      %end;

      /*kemicl fix for S1141352 */
      put " objRefNS:regOverStrng = ""(.*) ""||locale:get(""over_Text"")||"" (.*)"";";
      put " objRefNS:trtlev&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\1"");";
      put " objRefNS:trtlev2&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\2"");";    
      put " objRefNS:RRTitle&i = locale:fill(locale:get(""XRelativeRiskforX_Text""), {""%upcase(&domain)"", objRefNS:trtlevwhole&i});";
      put "objRefNS:NColNameRR1&i = locale:fill(locale:get(""XCountPercent_Text""),{objRefNS:trtlev&i});"; 
      put "objRefNS:NColNameRR2&i = locale:fill(locale:get(""XCountPercent_Text""),{objRefNS:trtlev2&i});"; 
      put "objRefNS:NColNameRR3&i = locale:get(""RelativeRisk95PercentCI_Text"");";
      put "Column(""&_levs1 Count (%)"")<<Set Name(objRefNS:NColNameRR1&i);";
      put "Column(""&_levs2 Count (%)"")<<Set Name(objRefNS:NColNameRR2&i);";
      put "Column(""Relative Risk (95% CI)"")<<Set Name(objRefNS:NColNameRR3&i);";
      put " objRefNS:RelativeRiskDDWin&i = New Window(objRefNS:RRTitle&i,";
      put " eval(parse(objRefNS:DrillDownReport(objRefNS:RRTitle&i,""objRefNS:rrhlb&i""))),";
      put " objRefNS:rrhlb&i = HListBox(";
      put " objRefNS:rrvlb&i=VListBox(";  
      put "   Hlistbox(";
      put "     TextBox(""                 ""),";
      put "     PanelBox(locale:get(""SelectColumnToOrderTermsInPlot_Text""),";
      put "    objRefNS:RBL = Eval List({""&CatLabel"",""&trcount"",""&trrelrisk"",""&trslog10p""});";
      put "      objRefNS:srr&i=RadioBox(objRefNS:RBL,<<Set(3),";
      put "       local({objRefNS= Namespace(expr(objRefNS<<GetName))},";
      put "        if(objRefNS:firstrr&i==0, Try(objRefNS:rrgbr&i<<Delete);";
      put "        objRefNS:rrdt&i << Show Window(1);";
      put "        objRefNS:rrdt1_&i = objRefNS:rrdt&i << Sort(By(Column(objRefNS:srr&i<<Get Selected),Column(""&CatLabel"")),Invisible);";
      put "        Close(objRefNS:rrdt&i,NoSave);";
      put "        objRefNS:rrdt&i = objRefNS:rrdt1_&i;";
      put "        objRefNS:GBRRFunc&i(objRefNS:rrdt&i);";
      put "       )";
      put "      ));";
      put "     )";
      put "   );";   
      put "  ));";  /* Close vlist and hlist box for forest plots */
      put " );";
      put " If(Is Empty(objRefNS:DrillDownWindowList),";
      put "   objRefNS:DrillDownWindowList = {};";
      put " );";   
      put " InsertInto(objRefNS:DrillDownWindowList,objRefNS:RelativeRiskDDWin&i);";      
      put " currwin = objRefNS:RelativeRiskDDWin&i;";

      put " objRefNS:rrdt&i << Clear Row States ;";
      put " objRefNS:rrdt&i<<Select Where(is missing(:&trrelrisk));";
      put " objRefNS:anysel = objRefNS:rrdt&i << Get Selected Rows;";
      put " if(NRow(objRefNS:anysel) > 0,objRefNS:rrdt&i<<Delete Rows);";
      put " if(NRow(objRefNS:rrdt&i)==0,";
      put "   Try(Close(objRefNS:rrdt&i,NoSave));";
      put "   Try(Close(objRefNS:rrdata&i,NoSave));";
      put "   currwin << Close Window;";
      put "   show(""Unable to construct relative risk plot &i due to zero or missing counts."")";
      put "  ,";  /* kemicl added to have an else if the data has rows */
      put "   objRefNS:rrdt&i << Show Window(1);";
      put "   objRefNS:rrdt1_&i = objRefNS:rrdt&i << Sort(By(Column(objRefNS:srr&i<<Get Selected),Column(""&CatLabel"")),Invisible);";
      put "   Try(Close(objRefNS:rrdt&i,NoSave));";
      put "   objRefNS:rrdt&i = objRefNS:rrdt1_&i;";

      put "   objRefNS:GBRRFunc&i(objRefNS:rrdt&i);";
      put "   objRefNS:firstrr&i = 0;";
      put "   Insert Into(objRefNS:TabWinList,objRefNS:rrdt&i);"; 
      put "   objRefNS:nwcounter++;";  

      put "   objRefNS:rrdt&i << Show Window(0);";    
      put "   objRefNS:rrdata&i << Close Window;";
      put " );";
      put " )); //end data exists and nsel";

   %end;
   put " Try(Close(objRefNS:cenbdatasub,NoSave));";
   put "));";         
   %if &_BObs_ > 0 %then %do;
      put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""RelativeRiskPlot_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[1],expr(objRefNS:rrplotfunc()));";
      put "InsertInto(objRefNS:ActionDataTableList[1],{""objRefNS:cenbdata""});";
      put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";
      put "InsertInto(objRefNS:ActionButtonsperRow,1);";
      put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.RelativeRiskPlot.gif"");";            
   %end; 

   * Odds Ratio Plot;
   %do i=1 %to &TotFBs;
      put "objRefNS:GBORfunc&i = EvalExpr(Function({dt&i},{default local},";
      put "   objRefNS= Namespace(expr(objRefNS<<GetName));";
      put "   CurrentDataTable(dt&i);";
      put "   objRefNS:orvlb&i<<prepend(objRefNS:orgb&i = Graph Builder(";
      put "   Size( 735, 630 ),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "      X( :Name(""&trcl1p"") ),";
      put "      X( :Name(""&trcl2p""), Position( 1 ) ),";
      put "      X( Column(""&trlog2oddsratio"") ),";
      put "      X( Column(""Lower 95% log2(Odds Ratio)""), Position( 2 ) ),";
      put "      X( Column(""Upper 95% log2(Odds Ratio)""), Position( 2 ) ),";
      put "      Y( Column(""&CatLabel""), Order By( Column(objRefNS:sor&i<<Get Selected), Order Statistic( ""Mean"" )), Size( 220 ) ),";
      put "   ),";
      put "   Elements( Position( 1, 1 ), Points( X( 1 ), X( 2 ), Y, Legend( 2 ), Jitter( 0 ) ) ),";
      put "   Elements( Position( 2, 1 ), Points( X( 1 ), X( 2 ), X( 3 ), Y, Legend( 4 ), Jitter( 0 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch({},:Name(""&trcl1p""),ScaleBox,";
      put "       {Min( -3 ), Max( %eval(&maxpercent + 3) ), Inc( 10 ), Minor Ticks( 0 ),";
      put "       Rotated Labels( ""Automatic"" )}";
      put "    ),";  
      put "    Dispatch( {}, ""&trlog2oddsratio"", ScaleBox, {Add Ref Line( 0, Dotted, ""Black"" ), Rotated Labels( ""Automatic"" )} ),";

      put "    Dispatch(";
      put "        {},";
      put "        ""&CatLabel"",";
      put "        ScaleBox,";
      put "        {Show Major Grid( 1 ), Rotated Labels( ""Automatic"" )}";
      put "    ),";
      put "    Dispatch(";
      put "     {},";
      put "     ""400"",";
      put "     ScaleBox,";
      put "     {Legend Model(";
      put "       2,";
      put "       Level Name( 0, objRefNS:trtlev&i ),";
      put "       Level Name( 1, objRefNS:trtlev2&i  ),";   
      put "       Properties( 0, {Line Color( Heat Color(1,""&ColorTheme"") ), Marker( ""Triangle"" )} ),";
      put "       Properties( 1, {Line Color( Heat Color(0,""&ColorTheme"") ), Marker( ""Circle"" )} )";
      put "      ),";
      put "      Legend Model(";
      put "       4,";
      put "       Properties( 0, {Line Color( 0 ), Marker( ""FilledCircle"" )} ),";
      put "       Properties( 1, {Line Color( 0 ), Marker( ""Pipe"" )} ),";
      put "       Properties( 2, {Line Color( 0 ), Marker( ""Pipe"" )} )"; 
      put "     )}"; 
      put "    ),";
      put "    Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( locale:get(""Log2OddsRatioPlot_Text"") )} ),";     
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""400"", LegendBox, {Set Title( ""&TrtLabel"" ), Position( {0, 1, -1, -1, -1}  )}),";
      put "    Dispatch( {}, ""X title"", TextEditBox, {Set Text( locale:fill(locale:get(""PercentX_Text""), {""%upcase(&Domain)""}))} ),";
      put "    Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&CatLabel"" )}     ),";
      put "    Dispatch( {}, ""X 1 title"", TextEditBox, {Set Text( locale:get(""Log2OddsRatioWith95PercentCI_Text""))} ),";  
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(1), {Marker Size( 2 )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox(2), {Marker Size( 2 )} ),";

      put "   ) //end send to report";
      put " )); //end graph builder and prepend";
      put " objRefNS:orhlb&i<<append(objRefNS:tb&i = Tabulate(Show Control Panel(0),";
      put "    Include missing for grouping columns( 1 ),";
      put "    Add Table(Row Table(Grouping Columns(";
      %if %length(&GroupVar) %then %do;
         %if %length(&TimeWindows) %then %do;
            put " :Name(""&GroupLabel0""),";
         %end;
         %else %do;
            put " :Name(""&GroupVar""),";
         %end;
      %end;
      put "       :Name(""&CatLabel""),Column(objRefNS:NColName1&i), Column(objRefNS:NColName2&i), Column(objRefNS:NColName3&i)";
      put "    ))),";
      put "    SendToReport(Dispatch({},""Tabulate"",OutlineBox,{Set Title(locale:get(""OddsRatioTable_Text""))}))";
      put " );";
      put "); //end append ";
      put " try(objRefNS:tbr = objRefNS:tb&i<<Report);";
      put " try(objRefNS:tbr[MultiTblNumColBox(1)]<<delete;);";
      %if %length(&GroupVar) %then %do;
         put " try(objRefNS:tbr[MultiTblStrColBox(1)]<<Set Width(240););";
         %if "&GroupVar" ne "&CatLabel" %then %do;
            put " //try(objRefNS:tbr[MultiTblStrColBox(2)]<<Set Width(220););";  
         %end;
      %end;
      %else %do;
         put " try(objRefNS:tbr[MultiTblStrColBox(1)]<<Set Width(210););";
      %end;
      put " objRefNS:orgbr&i = Report(objRefNS:orgb&i);";
      put " objRefNS:yheight&i = NRows(dt&i) + 1;";
      put " objRefNS:ORFuncdt&i = dt&i;";
      put " try(objRefNS:orgbr&i[framebox(2)] << AddGraphicsScript(";
      put "  local({objRefNS= Namespace(expr(objRefNS<<GetName))},";
      put "  CurrentDataTable(objRefNS:ORFuncdt&i);";
      put "  for each row(";
      put "   Pen Color( ""Black"" );";
      put "   Line({Column(""Lower 95% log2(Odds Ratio)"")[],row()-1},{Column(""Upper 95% log2(Odds Ratio)"")[],row()-1});";
      put "  );";
      put "  Transparency( 0.075 );";
      put "  Fill Color( Heat Color(0,""&ColorTheme"")  );";
      put "  Rect( {-50, -5}, {0, objRefNS:yheight&i}, Fill = 1 );";
      put "  Fill Color( Heat Color(1,""&ColorTheme"")  );";
      put "  Rect( {0, objRefNS:yheight&i}, {50, -5}, Fill = 1 );";
      put " )));";
      put " objRefNS:orgb&i << On Close(local({objRefNS= Namespace(expr(objRefNS<<GetName))}, try(objRefNS:ORFuncdt&i<<Close Window)));";
      put "));";   
   %end;
   /* End odds ratio plot function generation */
   
   put "objRefNS:orplotfunc=EvalExpr(Function({},{default local},";
   put " objRefNS= Namespace(expr(objRefNS<<GetName));";
   put " if(!isempty(objRefNS:cengdata),";
   put " if(Current Data Table() == objRefNS:cengdata,";
   put "    objRefNS:glist = {};";
   put "    objRefNS:GVar = Column(""&TabGLvar"");";
   put "    for each row(if(selected(rowstate())==1,";
   put "       InsertInto(objRefNS:glist, objRefNS:GVar()[]);";
   put "    ));";
   put "    if(nItems(objRefNS:glist),";
   put "       Current Data Table(objRefNS:cenbdata);";
   put "       objRefNS:cenbdata<<SelectWhere(contains(objRefNS:glist, :Name(""&TabGLvar"")));";
   put "       objRefNS:nsel = 0;";
   put "       For Each Row(if(selected(rowState()),objRefNS:nsel++;));";
   put "       if(objRefNS:nsel ==0, Throw(locale:get(""NoSignificantTermLevelTestsForSelectedGroupLevels_Text"")));";
   put "    );";
   put " ));"; 
   put " objRefNS:cenbdata << Clear Column Selection();";
   put " CurrentDataTable(objRefNS:cenbdata);";
   %if %upcase(&SplitResults)=YES %then %do;
      put " objRefNS:blist1 = ""Column(\!""Ratio\!"")"";";
      put " for(i=1,i<=NItems(objRefNS:dbv),i++,";
      put "    objRefNS:blist1 = objRefNS:blist1 || "","" || char(objRefNS:dbv[i]);";
      put " );";
      put " objRefNS:jslcode = EvalInsert(""\[Summarize( objRefNS:ratiolist = By(^objRefNS:blist1^) );]\"");";
      put " eval(parse(objRefNS:jslcode));";
   %end;
   %else %do;  
      put " Summarize(objRefNS:ratiolist = By(:Name(""&trratio"")));";
   %end;

   put " objRefNS:cenbdatasub = objRefNS:cenbdata << Subset(Selected rows(1),Output table name(""&IncidenceScreen_BOutDataName._or""),invisible);";
   put " Try(CurrentDataTable(objRefNS:cenbdatasub);,";
   put " objRefNS:errwin = New Window(locale:fill(locale:get(""XMessage_Text""), {locale:get(""DrillDown_Text"")}), <<modal,";
   put "    TextBox(locale:fill(locale:get(""ErrorNoRowsSelectedMakeSureTableWithXActive_Text""), {""&CatLabel""})),";
   put "    jg:messageHelp(""ErrorNoRowsSelectedMakeSureTableWithXActive_Text""),";
   put "    widgetHandle = ButtonBox(locale:get(""OK_Text""))";
   put " );";  
   put " Throw(););";
   %do i=1 %to &TotFBs;
      put "  Try(objRefNS:OddsRatioDDWin&i<<Close Window);";   
      put " objRefNS:cenbdatasub<<ClearSelect;";
      %if %upcase(&SplitResults)=YES %then %do;
         put " objRefNS:whereclause = "":&trratio == objRefNS:ratiolist[1][&i]"";";
         put " objRefNS:title = ""&trratio="" || eval(objRefNS:ratiolist[1][&i]);";
         %do j=1 %to &ndbv;
            put "objRefNS:cenbdatasub:&&dbv&j << Data Type( Character);";
            put " objRefNS:whereclause ||= "" & :&&dbv&j == objRefNS:ratiolist[%eval(&j+1)][&i]"";";
            put " objRefNS:title = objRefNS:title || "", &&dbv&j="" || eval(objRefNS:ratiolist[%eval(&j+1)][&i]);";
         %end;
         put " objRefNS:cenbdatasub<<SelectWhere(eval(parse(objRefNS:whereclause)));"; 
      %end;
      %else %do;  
         put " objRefNS:cenbdatasub<<SelectWhere(:&trratio == objRefNS:ratiolist[&i]);";
         put " objRefNS:title = eval(objRefNS:ratiolist[&i]);";
      %end;
      put " objRefNS:nsel = NRow(objRefNS:cenbdatasub<<GetSelectedRows);";

      put " if(objRefNS:nsel,";
      put "  objRefNS:cenbdatasub<<Subset(Selected rows(1),Output table name(""&IncidenceScreen_BOutDataName._or &i""),invisible);";

      put " try(objRefNS:ordata&i = Data Table(""&IncidenceScreen_BOutDataName._or &i""));";
      put " if(!isEmpty(objRefNS:ordata&i),";   
      put "  objRefNS:firstor&i = 1;";

      /* Remove multiple rows with the same term */
      put " objRefNS:ordt&i=objRefNS:ordata&i << Join(";
      put "    With( objRefNS:ordata&i ),";
      put "    By Matching Columns(";
      put "    :Name(""&CatLabel"") = :Name(""&CatLabel""), ";
      put "    ),";
      put "    Drop multiples( 1, 1 ),";
      put "    Name( ""Include non-matches"" )(0, 0),";
      put "    Output table name(""&IncidenceScreen_BOutDataName._orj &i""),";
      put "    Invisible";
      put " );";  
      put " objRefNS:ordt1_&i = objRefNS:ordt&i << Sort(By(Column(""&CatLabel"")),Invisible);";
      put " Close(objRefNS:ordt&i,NoSave);";
      put " objRefNS:ordt&i = objRefNS:ordt1_&i;";    
      
      put " Current Data Table(objRefNS:ordt&i);";
      put " objRefNS:trtlevwhole&i = :Ratio[1];";  
      put " objRefNS:ByVarList = "" "";";
      %if &TotLvls > 1 %then %do;
         %do b=1 %to &ndbv;
            put " objRefNS:ByVarList = objRefNS:ByVarList || "" &&dbv&b="" || (objRefNS:ordata&i << Get Table Variable(""&&dbv&b"")) || "" "";";
         %end;
      %end;
      /*kemicl fix for S1141352 */
      put " objRefNS:regOverStrng = ""(.*) ""||locale:get(""over_Text"")||"" (.*)"";";
      put " objRefNS:trtlev&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\1"");";
      put " objRefNS:trtlev2&i = Regex(objRefNS:trtlevwhole&i,objRefNS:regOverStrng, ""\2"");";    
      put " objRefNS:ORTitle&i = locale:fill(locale:get(""XOddsRatioforX_Text""), {""%upcase(&domain)"", objRefNS:trtlevwhole&i});";
      put "objRefNS:NColName1&i = locale:fill(locale:get(""XCountPercent_Text""),{objRefNS:trtlev&i});"; 
      put "objRefNS:NColName2&i = locale:fill(locale:get(""XCountPercent_Text""),{objRefNS:trtlev2&i});"; 
      put "objRefNS:NColName3&i = locale:get(""OddsRatio95PercentCI_Text"");";
      put "Column(""&_levs1 Count (%)"")<<Set Name(objRefNS:NColName1&i);";
      put "Column(""&_levs2 Count (%)"")<<Set Name(objRefNS:NColName2&i);";
      put "Column(""Odds Ratio (95% CI)"")<<Set Name(objRefNS:NColName3&i);";
      put " objRefNS:OddsRatioDDWin&i = New Window(objRefNS:ORTitle&i,";
      put " eval(parse(objRefNS:DrillDownReport(objRefNS:ORTitle&i,""objRefNS:orhlb&i""))),";
      put " objRefNS:orhlb&i = HListBox(";
      put " objRefNS:orvlb&i=VListBox(";  
      put "   Hlistbox(";
      put "     TextBox(""                 ""),";
      put "     PanelBox(locale:get(""SelectColumnToOrderTermsInPlot_Text""),";
      put "    objRefNS:RBL = Eval List({""&CatLabel"",""&trcount"",""&troddsratio"",""&trslog10p""});";
      put "      objRefNS:sor&i=RadioBox(objRefNS:RBL,<<Set(3),";
      put "       local({objRefNS= Namespace(expr(objRefNS<<GetName))},";
      put "       if(objRefNS:firstor&i==0, Try(objRefNS:orgbr&i<<Delete);";
      put "        objRefNS:ordt&i << Show Window(1);";
      put "        objRefNS:ordt1_&i = objRefNS:ordt&i << Sort(By(Column(objRefNS:sor&i<<Get Selected),Column(""&CatLabel"")),Invisible);";
      put "        Close(objRefNS:ordt&i,NoSave);";
      put "        objRefNS:ordt&i = objRefNS:ordt1_&i;";
      put "        objRefNS:GBORFunc&i(objRefNS:ordt&i);";
      put "       )";
      put "      ));";
      put "     )";
      put "   );";   
      put "  ));";  /* Close vlist and hlist box for forest plots */
      put " );";
      put " If(Is Empty(objRefNS:DrillDownWindowList),";
      put "   objRefNS:DrillDownWindowList = {};";
      put " );";   
      put " InsertInto(objRefNS:DrillDownWindowList,objRefNS:OddsRatioDDWin&i);";      
      put " currwin = objRefNS:OddsRatioDDWin&i;";

      put " objRefNS:ordt&i << Clear Row States ;";
      put " objRefNS:ordt&i<<Select Where(is missing(:&troddsratio));";
      put " objRefNS:anysel = objRefNS:ordt&i << Get Selected Rows;";
      put " if(NRow(objRefNS:anysel) > 0,objRefNS:ordt&i<<Delete Rows);";
      put " if(NRow(objRefNS:ordt&i)==0,";
      put "   Try(Close(objRefNS:ordt&i,NoSave));";
      put "   Try(Close(objRefNS:ordata&i,NoSave));";
      put "   currwin << Close Window;";
      put "   show(""Unable to construct Odds Ratio plot &i due to zero or missing counts."")";
      put "  ,"; 
      put "   objRefNS:ordt&i << Show Window(1);";
      put "   objRefNS:ordt1_&i = objRefNS:ordt&i << Sort(By(Column(objRefNS:sor&i<<Get Selected),Column(""&CatLabel"")),Invisible);";
      put "   Try(Close(objRefNS:ordt&i,NoSave));";
      put "   objRefNS:ordt&i = objRefNS:ordt1_&i;";

      put "   objRefNS:GBORFunc&i(objRefNS:ordt&i);";
      put "   objRefNS:firstor&i = 0;";
      put "   Insert Into(objRefNS:TabWinList,objRefNS:ordt&i);"; 
      put "   objRefNS:nwcounter++;";  

      put "   objRefNS:ordt&i << Show Window(0);";    
      put "   objRefNS:ordata&i << Close Window;";
      put " );";
      put " )); //end data exists and nsel";

   %end;
   put " Try(Close(objRefNS:cenbdatasub,NoSave));";
   put "));";  /*end OR function*/       
   %if &_BObs_ > 0 %then %do;
      put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""OddsRatioPlot_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[1],expr(objRefNS:orplotfunc()));";
      put "InsertInto(objRefNS:ActionDataTableList[1],{""objRefNS:cenbdata""});";
      put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";      
      put "InsertInto(objRefNS:ActionButtonsperRow,1);";
      put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.OddsRatioPlot.gif"");";                  
   %end; 
   
   * Contingency Analysis;
   put "objRefNS:CatContExpr = Expr(";
   put "   objRefNS:CatSaveSelected;";
   put "   if(objRefNS:alreadystacked,objRefNS:CatOpenSubsetStacked,objRefNS:CatOpenSubset);";
   put "   objRefNS:Conttable = objRefNS:subset_dt;";
   put "   objRefNS:CatContingency;";
   put ");";
   put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""ContingencyAnalysis_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],expr(objRefNS:CatContExpr));";
   put "InsertInto(objRefNS:ActionDataTableList[1],{""objRefNS:cenbdata""});";
   put "InsertInto(objRefNS:ActionRequirementFlagList[1],1);";   
   put "InsertInto(objRefNS:ActionButtonsperRow,1);";
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.MosaicPlot.gif"");";            
   

   * Venn Diagram;
   put "objRefNS:CatVennExpr = Expr(";
   put "   objRefNS:CatSaveSelected;";
   put "   if(objRefNS:alreadystacked,objRefNS:CatOpenSubsetStacked,objRefNS:CatOpenSubset);";
   put "   objRefNS:CatVenn;";
   put ");";
   put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""VennDiagram_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],expr(objRefNS:CatVennExpr));";
   put "InsertInto(objRefNS:ActionDataTableList[1],{""objRefNS:cenbdata""});";
   put "InsertInto(objRefNS:ActionRequirementFlagList[1],3);";   
   put "InsertInto(objRefNS:ActionButtonsperRow,1);";
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.VennDiagramAnd.gif"");";            

   * Tabulate;
   put "objRefNS:CatTabulateExpr = Expr(";
   put "   objRefNS:CatSaveSelected;";
   put "   if(objRefNS:alreadystacked,objRefNS:CatOpenSubsetStacked,objRefNS:CatOpenSubset);";
   put "   objRefNS:tabtable = objRefNS:subset_dt;";
   put "   objRefNS:CatTabulate;";
   put ");";
   put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""Tabulate_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],expr(objRefNS:CatTabulateExpr));";
   put "InsertInto(objRefNS:ActionDataTableList[1],{""objRefNS:cenbdata""});";
   put "InsertInto(objRefNS:ActionRequirementFlagList[1],1);";      
   put "InsertInto(objRefNS:ActionButtonsperRow,1);";
   put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.TabulateIcon.gif"");";      
run;

%let includeds = adsl_&suffix..sas7bdat;
%if %length(&BSigVars) %then %do;
   %let includeds = &includeds , &IncidenceScreen_BOutDataName..sas7bdat;
   %if &DoubleFDR = yes %then %do;
      %let includeds = &includeds , &IncidenceScreen_GOutDataName..sas7bdat;
   %end;
%end;
%if %length(&ASigVars) %then %do;
   %let includeds = &includeds , &IncidenceScreen_AOutDataName..sas7bdat;
%end;
%if %length(&CSigVars) %then %do;
   %let includeds = &includeds , &IncidenceScreen_COutDataName..sas7bdat;
%end;
%if %length(&IncidenceScreen_CatOutData) %then %do;
   %let includeds = &includeds , &IncidenceScreen_CatOutDataName..sas7bdat;
%end;
%if %length(&IncidenceScreen_DesOutData) %then %do;
   %let includeds = &includeds , &IncidenceScreen_DesOutDataName..sas7bdat;
%end;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &IncidenceScreenScriptName, 
   "&includeds", text/plain);
   
%exit:;

%mend IncidenceScreen;





* create indicator variables for each category for use in proc multtest;
%macro MakeCatIndicators;

   %put Entering MakeCatIndicators;

   * check total number of levels;
   data ocp1;
      set cencat;
      if not missing(cat);
      * cat = lowcase(cat);
      cat = trim(left(cat));
      keep cat;
   run;

   proc sort data=ocp1 nodupkey;
      by cat;
   run;

   %NObsVars(ocp1);
   %let ncat = &nobs;
   %put ncat = &ncat;

   %if (&ncat > 32767) %then %do;
      %put [MS_S_0129];
      %put ERROR:  Unable to process more than 32767 distinct categories.;
      %put You may want to try reducing the number of categories or the number of rows in the Input Data Set.;
      %let exiterror = 1;
      %goto mexit;
   %end;

   data cencat;
      set cencat;
      _row_ = _n_;
   run;

   /*
   proc print data=cencat(obs=100);
   run;
   */

   /*
   * get length of cat variable in order to size quoted value;
   * quoting is used to handle special characters like single quotes;
   %let _cid_ = %sysfunc(open(cencat));
   %let _cnum_ = %sysfunc(varnum(&_cid_,cat));
   %let _clen_ = %sysfunc(varlen(&_cid_,&_cnum_));
   %let _rc_ = %sysfunc(close(&_cid_));

   %put clen = &_clen_;
   */

   data d;
      length &CIPrefix $ %eval(&maxlen+4);
      set cencat;
      &CIPrefix = cat;
      drop cat;
   run;

   %if &syserr>4 %then %goto mexit;

   %let ClassVars = &CIPrefix;
   %let ContinuousVars = ;
   %let PredictorVars = ;
   %let ClassPredVars = ;

   %include "&MacroPath./ClassVars.sas" / nosource;

   proc datasets library=work;
      change d=cendes;
   quit;

   %let ncat = &nic1;

   /*---use glmmod to create indicator variables for each category---*/
   /*
   ods exclude all;
   ods noresults;

   proc glmmod data=cencat outdesign=cendes;
      class cat;
      model _row_ = cat / noint;
   run;

   %NObsVars(cendes);
   %let ncat = %eval(&nvars-1);
   */

   %put ncat = &ncat;
   %if &exiterror or &syserr>4 %then %goto mexit;

   /*---transpose to get labels---*/
   data cendes1;
      set cendes(obs=1);
   run;

   /*
   proc print;
   run;
   */

   proc transpose data=cendes1 out=cendest;
      var &CIPrefix._:;
   run;
      
   %let lci = %eval(%length(&CIPrefix)+2);
   %put lci = &lci;
   
   %put CatName = &CatName;

   data cendest;
      set cendest;
      if lengthn(_label_) then do;
         * &CatName = dequote(substr(_label_,&lci));
         &CatName = _label_;
      end;
      else do;
         &CatName = substr(_name_,&lci);     
      end;
      _var_ = lowcase(_name_);
      label &CatName="&CatLabel";
      keep &CatName _var_;
   run;

   data cendes;
      merge cencat cendes;
      by _row_;
   run;

   proc sort data=cendes;
      by _obs_;
   run;

   /*---sum up for each obs---*/
   proc means data=cendes sum noprint;
      by _obs_;
      var %do i=1 %to &ncat; &&_c&i %end; ;
      output out=censum0 sum=%do i=1 %to &ncat; &&_c&i %end; ;
      id &ASigVars &BSigVars &CSigVars;
   run;

   %if &syserr>4 %then %goto mexit;
   
   /*---remove labels---*/
   /*
   data censum0;
      set censum0;
      label %do i=1 %to &ncat; &&_c&i=' ' %end; ;
   run;
   */

   %if %length(&IncidenceScreen_DesOutData) %then %do;
      data cen1;
         set InLib.&InData;
         %if %length(&WhereStmt) %then %do;
            where &WhereStmt;
         %end;
         len = lengthn(&AnnoVar);
         if len > 0;
      run;
      data cen1;
         set cen1;
         _obs_ = _n_;
      run;
      data OutLib.&IncidenceScreen_DesOutDataName;
         merge cen1 censum0;
         by _obs_;
      run;
   %end;

   %let CatIndicators = 1;

   %mexit:;
   
   %put Exiting MakeCatIndicators;
%mend;


%IncidenceScreen;




