/*
JMP Clinical Analytical Process

Name: Missing Findings
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
Based on data available at each visit number and time point number, determines whether individuals are
missing data for all test codes across all Findings domains

*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath TrtUse TrtVar DesignByVars SubjectFilter WhereStmt OutPath ServerOutPath 
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL  WhereForReport SiteCount UnSchVisit
MissingFindings_OutData MacroPath MissingFindingsScript RemoveUnsch PropSubsCut OutName AlphaValue FindTests AllFind
MergeSuppDom;
 
*ProcessBody;

%macro MissingFindings;

/*    
$CVS_START$
$ID: MissingFindings$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags","CalcStudyDay_FindingsO"}}$
$CMP: CPV.{"Remove unscheduled visits","Yes"}$ $UE: FINDINGS.VISIT$ $NOTES: If available, records will be removed if contains the word UNSCHEDULED.$
$CMP: RSYS$ $UE: DM.SITEID and FINDINGS.xxTEST and FINDINGS.xxTESTCD and FINDINGS.VISITNUM and (FINDINGS.xxSTRESN or FINDINGS.xxSTRESC or FINDINGS.xxORRES) and SV.VISITNUM$
$CMP: OSYS$ $UE: DM.SEX$ $NOTES: Included in summary table if available.$
$CMP: OSYS$ $UE: FINDINGS.VISIT or FINDINGS.xxTEST or FINDINGS.xxTPTNUM $ $NOTES: Included in summary if available and for determining missing records.$
$CMP: OSYS$ $UE: FINDINGS.xxDTC or FINDINGS.xxSTAT or FINDINGS.xxREASND $ $NOTES: Used if available to provide details for included missing records.$
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$ 
$CVS_STOP$
*/

*initialization and argument checking;
%let ProcessName = MissingFindings; 
%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds =;
%OUTNAMETRUNC(outdat = MissingFindings_OutData, truncnum = 15);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);

%let trXVolcanoPlot = %nrbquote(%LocaleGet(XVolcanoPlot_Text));
%let trXDistribution = %nrbquote(%LocaleGet(XDistribution_Text));
%let blank =;
%let trVolcanoPlot = %nrbquote(%LocaleFill(&trXVolcanoPlot, &blank));
%let trDistribution = %nrbquote(%LocaleFill(&trXDistribution, &blank));
%let trVolcanoPlot = %sysfunc(strip(&trVolcanoPlot));
%let trDistribution = %sysfunc(strip(&trDistribution));


%let availabledomains =;
%let domainlist = &findingsdomains;

%if ^%length(&findingsdomains) %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%CHECKPRES(domain = ADSL, var = SITEID);
%CHECKPRES(domain = ADSL, var = SEX);

%ADDCOUNTRYTODATA(varlist = sex);

%if ^%sysfunc(indexw(&domains, SV, ' ')) %then %do;
   %put [MS_S_0141];
   %put ERROR: The SV domain is not available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%GETSITELISTS(errflg = 0);
%if &exiterror %then %goto exit;

%if %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &domainlist, flist = &FindTests);
%end;

*** to determine which visits attended ***;
%LOADDATA(domlist = SV,mergesupp = 0,checkreview=0);
proc sort data = sv out = sv;
   by usubjid visitnum;
run;

%let SVMissFlag = 0;
%if %upcase(&RemoveUnsch) = YES %then %do;
   *** Delete unscheduled records ***;
   %if %sysfunc(indexw(&SVVARS, VISIT, ' ')) %then %do;
      data sv svmiss(keep = usubjid visitnum);
         set sv;
         if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then output svmiss;
         else output sv;
      run;
   
      %NObsVars(svmiss);
      %if &nobs %then %let SVMissFlag = 1;
   %end;
%end;

%if %length(&UnSchVisit) %then %do;
   data sv;
      set sv;
      if visitnum in (%sysfunc(transtrn(%str(&UnSchVisit), %str( ), %str(,)))) then delete;
   run;
%end;

data sv(rename = (visitnum = maxvisitnum));
   set sv(keep = usubjid visitnum);
   by usubjid visitnum;
   if last.usubjid;
run;

%let TestLen = 0;
%let VisitLen = 0;
%let MissDom =;
%let i = 1;
%do %while(%length(%scan(&domainlist,&i,' ')));
   %let temp = %scan(&domainlist,&i,' ');
   
   %if %upcase(&AllFind) = NO %then %do;
      %let ifcond = and %length(&&&temp.findlist);
   %end;
   %else %do;
      %let ifcond =;
   %end;   

   %if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TEST, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TESTCD, ' ')) and (%sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) or %sysfunc(indexw(&&&temp.VARS, &temp.STRESC, ' ')) 
      or %sysfunc(indexw(&&&temp.VARS, &temp.ORRES, ' '))) &ifcond %then %do;
      
      %if %sysfunc(indexw(&&&temp.VARS, &temp.TPTNUM, ' ')) %then %let TPTNUM = &temp.tptnum;
      %else %let TPTNUM =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.TEST, ' ')) %then %let TESTY = &temp.test;
      %else %let TESTY =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.DTC, ' ')) %then %let DATEY = &temp.DTC;
      %else %let DATEY =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.STAT, ' ')) %then %let STAT = &temp.stat;
      %else %let STAT =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.REASND, ' ')) %then %let REASND = &temp.reasnd;
      %else %let REASND =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %let STRESN = &temp.stresn;
      %else %let STRESN =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.STRESC, ' ')) %then %let STRESC = &temp.stresc;
      %else %let STRESC =;
      %if %sysfunc(indexw(&&&temp.VARS, &temp.ORRES, ' ')) %then %let ORRES = &temp.orres;
      %else %let ORRES =;     
      %if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %let VISIT = visit;
      %else %let VISIT =;

      %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
      
      %if %upcase(&RemoveUnsch) = YES %then %do;
         *** Delete unscheduled records ***;
         %if &SVMissFlag %then %do;         
            proc sort data = &temp;
               by usubjid visitnum;
            run;
         
            data &temp;
               merge &temp(in = ina) svmiss(in = inb);
               by usubjid visitnum;
               if ina and not inb;
            run;            
         %end;
      %end;      
      
      %let VISITNUML&temp = VISITNUM;
      %CheckLabel(&temp,VISITNUML&temp);
      %let byvar&temp = ;
      %if %length(&VISIT) %then %do;
         %let byvar&temp = VISIT;
         %let VISITL&temp = VISIT;
         %CheckLabel(&temp,VISITL&temp);
      %end;
      %let TPTNUML&temp =;
      %if %length(&TPTNUM) %then %do;
         %let byvar&temp = &&byvar&temp TPTNUM;
         %let TPTNUML&temp = &trPlannedTime;
      %end;    

      %let _tid_ = %sysfunc(open(&temp));
      %let vn = %sysfunc(varnum(&_tid_,&temp.test));
      %let TestLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,visit));
      %let VisitLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %if %eval(&TestLentmp>&TestLen) %then %let TestLen = &TestLentmp; 
      %if %eval(&VisitLentmp>&VisitLen) %then %let VisitLen = &VisitLentmp; 
      
      proc sort data = &&in_&temp out = &temp(keep = usubjid &TESTY &temp.testcd visitnum &VISIT &TPTNUM &STAT &REASND &STRESN &STRESC &ORRES &DATEY);
         by &TESTY visitnum &VISIT &TPTNUM;
      run;
      
      %if %upcase(&AllFind) = NO %then %do;
         data &temp;
            set &temp;       
            where &temp.testcd in ( %unquote(&&&temp.findlist) );
         run;      
      %end;

      %if %upcase(&RemoveUnsch) = YES %then %do;
         *** Delete unscheduled records ***;
         %if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
            data &temp;
               set &temp;
               if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
            run;
         %end;
      %end;
      
      %NObsVars(&temp); 
      %if &nobs %then %let availabledomains = &availabledomains &temp;

      %if &nobs %then %do;
         proc sql noprint;
            create table &temp.skel as
               select distinct &TESTY, visitnum %if %length(&VISIT) %then ,&VISIT; %if %length(&TPTNUM) %then ,&TPTNUM;
               from &temp
               order by &TESTY, visitnum %if %length(&VISIT) %then ,&VISIT; %if %length(&TPTNUM) %then ,&TPTNUM;;
            create table &temp.skel2 as
               select * 
               from subject, &temp.skel
               order by usubjid, &TESTY, visitnum %if %length(&VISIT) %then ,&VISIT; %if %length(&TPTNUM) %then ,&TPTNUM;;
         quit;    

         proc datasets nolist;
            delete &temp.skel;
            change &temp.skel2 = &temp.skel;
         quit; 

         *** removes visits that were not attended ***;
         data &temp.skel(drop = maxvisitnum);
            merge SV(in = ina) &temp.skel(in = inb);
            by usubjid;
            if ina and inb and (visitnum <= maxvisitnum %if %length(&UnSchVisit) %then %do; or visitnum in (%sysfunc(transtrn(%str(&UnSchVisit), %str( ), %str(,)))) %end;);
         run;
         
         *** determine visits missing all records ***;
         data &temp(drop = &STRESN &STRESC &ORRES);
            set &temp;
            int = 1;
            %if %length(&STRESN) %then %do;
               if missing(&STRESN)
               %if %length(&STRESC) %then %do;
                  and missing(&STRESC)
               %end;
               %if %length(&ORRES) %then %do;
                  and missing(&ORRES)
               %end;
               then MISSALL = 1;
               else MISSALL = 0;
            %end;
            %else %if %length(&STRESC) %then %do;
               if missing(&STRESC)
               %if %length(&ORRES) %then %do;
                  and missing(&ORRES)
               %end;
               then MISSALL = 1;
               else MISSALL = 0;
            %end;         
            %else %if %length(&ORRES) %then %do;
               if missing(&ORRES) then MISSALL = 1;
               else MISSALL = 0;
            %end;         
         run;

         data &temp(drop = &STAT &REASND &DATEY) &temp.missall;
            set &temp;
            if MISSALL = 0 then output &temp;
            else output &temp.missall;
         run; 

         proc means sum nway noprint missing data = &temp;
            class usubjid &TESTY visitnum &VISIT &TPTNUM;
            var int;
            output out = stats(drop = _TYPE_ _FREQ_) sum = FREQ;
         run;

         proc sort data = &temp;
            by usubjid &TESTY visitnum &VISIT &TPTNUM;
         run;

         data &temp;
            merge &temp.skel(in = ina) stats(in = inb);
            by usubjid &TESTY visitnum &VISIT &TPTNUM;       
            if ina;
            if ina and inb then flag = 1;
            else flag = 0;
         run;

         proc datasets nolist;
            delete &temp.skel stats;
         quit; 

         proc sql noprint;
            create table &temp.2 as
               select *, mean(flag) as PROPSUBS 
               from &temp
               group by &TESTY, visitnum %if %length(&VISIT) %then ,&VISIT; %if %length(&TPTNUM) %then ,&TPTNUM;
               order by usubjid, &TESTY, visitnum %if %length(&VISIT) %then ,&VISIT; %if %length(&TPTNUM) %then ,&TPTNUM;;
         quit;

         proc datasets nolist;
            delete &temp;
            change &temp.2 = &temp;
         quit;

         data &temp(drop = flag);
            set &temp;
            propsubs = round(propsubs*100, 0.1);
            if missing(freq) then freq = 0;
            if freq = 0 then MISS = 1;
            else MISS = 0;
            if . < propsubs <= &PropSubsCut then delete;
         run;

         proc means sum nway noprint missing data = &temp.missall;
            class usubjid &TESTY visitnum &VISIT &TPTNUM;
            var int;
            output out = stats(drop = _TYPE_ _FREQ_) sum = FREQMISS;
         run;     

         data &temp;
            merge &temp(in = ina) stats(in = inb);
            by usubjid &TESTY visitnum &VISIT &TPTNUM;
            if ina;
            if missing(FREQMISS) then FREQMISS = 0; 
         run;

         %let MissingFindings_OutData_&temp =;     
         %PrepOutDS(MissingFindings_OutData_&temp,OutDataName_&temp, &Outname.MissF, _&temp);
         %let includeds = &includeds &&OutDataName_&temp...sas7bdat,;

         data &&MissingFindings_OutData_&temp;
            length domain $ 8;
            set &temp(rename = (&TESTY = TEST %if %length(&TPTNUM) %then %do; &TPTNUM = TPTNUM %end;));
            domain = "&temp";       
            label Test = "&trTestName" Domain = "&trDOMAIN" Miss = "&trMissingStatus" freq = "&trFrequencyofRecordswithAvail" 
                  freqmiss = "&trFrequencyofRecordswithMissing" propsubs = "&trPropSubs" %if %length(&TPTNUM) %then %do; TPTNUM = "&trPlannedTime" %end;;
         run;           
         %NObsVars(&temp);

         %if ^&nobs %then %let MissDom = &MissDom &temp;

         proc sort data = &temp.missall;
            by usubjid &TESTY visitnum &VISIT &TPTNUM;
         run;

         *** check that missing records are part of the selected set based on percentages ***;
         data &temp.missall;
            merge &temp(keep = usubjid &TESTY visitnum &VISIT &TPTNUM in = ina)
                  &temp.missall(in = inb);
            by usubjid &TESTY visitnum &VISIT &TPTNUM;
            if ina and inb;
         run;

         %NObsVars(&temp.missall);
         %if &nObs %then %do;
            %let MissingFindings_OutMiss_&temp =;     
            %PrepOutDS(MissingFindings_OutMiss_&temp,OutMiss_&temp, &Outname.MissF, _&temp._m);
            %let includeds = &includeds &&OutMiss_&temp...sas7bdat,;          

            data &&MissingFindings_OutMiss_&temp(drop = missall int);
               length domain $ 8;
               set &temp.missall(rename = (&TESTY = TEST %if %length(&TPTNUM) %then %do; &TPTNUM = TPTNUM %end;));
               domain = "&temp"; 
               label Test = "&trTestName" Domain = "&trDOMAIN" Miss = "&trMissingStatus" freq = "&trFrequencyofRecordswithAvail" 
                     freqmiss = "&trFrequencyofRecordswithMissing" propsubs = "&trPropSubs" %if %length(&TPTNUM) %then %do; TPTNUM = "&trPlannedTime" %end;;
            run;        
         %end;
      %end;
      
      proc datasets nolist;
         delete stat &temp &temp.missall;
      quit;      
   %end;
   %let i = %eval(&i+1);
%end;

data alldata;
   length TEST $ &testlen %if %length(&VISIT) %then %do; &VISIT $ &visitlen %end;;
   set 
   %let i = 1;
   %do %while(%length(%scan(&availabledomains,&i,' ')));
      %let temp = %scan(&availabledomains,&i,' ');
      &&MissingFindings_OutData_&temp(keep = usubjid siteid TEST visitnum domain %if %length(&VISIT) %then &VISIT; %if %length(&&TPTNUML&temp) %then TPTNUM; miss)
      %let i = %eval(&i+1);
   %end;
   ;
run;

%let TESTlabel = TEST;
%CheckLabel(alldata,TESTlabel);

%if &numsites > 1 %then %do;
   proc sort data = alldata;
      by domain test;
   run;

   %let firstflag = 0;
   %let i = 1;
   %do %while(%length(%scan(&SiteList,&i,' ')));
      %let tempsite = %scan(%upcase(&SiteList),&i," ");
      %let tempsitesize = %scan(%upcase(&SiteSamp),&i," ");
      %let trSiteTS = %nrbquote(%LocaleFill(&trSiteX, f1=%nrbquote(&tempsite)));
      %let trNotSiteTS = %nrbquote(%LocaleFill(&trNotSiteX, f1=%nrbquote(&tempsite)));

      data temp;
         length Label $ 30;
         set alldata;
         if siteid = "&tempsite" then Label = "&trSiteTS";
         else Label = "&trNotSiteTS";
         label Label = "&trLabel";
      run;

      proc freq noprint data = temp;
         tables label * Miss / sparse chisq out = stats(drop = PERCENT PCT_COL) outpct;
         output out = rowmean(keep = domain test XP2_FISH) fisher;      
         by domain test; 
      run;   

      %NObsVars(rowmean);
      %if &nobs %then %do;
         data stats;
            length siteid $ &SiteidLen;
            set stats(rename = (PCT_ROW = Percent));
            SITEID = "&tempsite";
            SITESAMP = &tempsitesize;
            label percent = "&trPercentofMeasurements" siteid = "&SITEIDlabel" sitesamp = "&trSiteSampleSize";
         run;

         proc sort data = stats out = stats2;
            by domain test label MISS;
         run;      

         data stats2;
            set stats2;
            by domain test label MISS;
            if last.Label;
         run;

         data stats2(drop = count percent);
            set stats2;
            by domain test label;
            retain diff;
            if first.test then diff = .;
            if first.test and ^last.test and kindex(label, "&trNot") then diff = -percent;
            else if last.test and ^first.test and ^kindex(label, "&trNot") then diff = diff + percent;
            if last.test and diff > .;
            label diff = "&trDiffMissingRates";
         run;

         data stats2(drop = MISS);    
            merge stats2(in = ina) rowmean(rename = (XP2_FISH = Fisher));
            by domain test;
            if ina;
            if missing(Fisher) then Fisher = 1;
            Nlog10Fisher = -log10(Fisher);
            label Fisher = "&trFisherExact" Nlog10Fisher = "&trNegLog10(&trFisherExact)";
         run;

         %if ^&firstflag %then %do;
            data summary_volcano;
               set stats2;
            run;
            data summary_miss;
               set stats;
            run;
            %let firstflag = 1;
         %end;
         %else %do;   
            proc append force base = summary_volcano data = stats2;
            run;
            proc append force base = summary_miss data = stats;
            run;     
         %end;
      %end;
      
      proc datasets nolist;
         delete temp stats stats2 rowmean;
      quit;

      %let i = %eval(&i+1);
   %end;

   proc datasets nolist;
      delete alldata;
   quit;

   %FDRPVALUES(inpv = Fisher, nlog10p = Nlog10Fisher);

   %let MissingFindings_OutData_v =;      
   %PrepOutDS(MissingFindings_OutData_v,OutDataName_v, &Outname.MissF, _vol);
   %let includeds = &includeds &OutDataName_v..sas7bdat,;

   data &MissingFindings_OutData_v;
      merge summary_volcano(in = ina) country(in = inb);
      by siteid;
      if ina;
   run;

   %let MissingFindings_OutData_m = ;     
   %PrepOutDS(MissingFindings_OutData_m,OutDataName_m, &Outname.MissF, _hist);
   %let includeds = &includeds &OutDataName_m..sas7bdat,;

   data &MissingFindings_OutData_m;
      set summary_miss;
      label Miss = "&trMissingStatus";
   run;
%end;
proc datasets nolist;
   delete summary_miss summary_volcano country subject;
quit;

* create JSL file;
%PrepOutFile(MissingFindingsScript,MissingFindingsScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &MissingFindingsScript;
%let FullAPName=%str(Missing Findings);

* put JSL statements;
data _null_;
   file "&MissingFindingsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:SiteLab = ""&SITEIDLabel"";";
   put "objRefNS:TestLab = ""&TESTLabel"";";
   put "objRefNS:DomainLab = ""&trDOMAIN"";";
   put "objRefNS:MissingStatus = ""&trMissingStatus"";";
   put "objRefNS:clientoutpath = ""&ClientOutPath"";";
   put "objRefNS:hist = ""&ClientOutPath.&OutDataName_m..sas7bdat"";";
   put "objRefNS:suffix = ""&_nThePro_"";";
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   %if &numsites > 1 %then %do;
      put "objRefNS:volcano=Function({},{default local},";
      put "   objRefNS:dvolcano = open(""&ClientOutPath.&OutDataName_v..sas7bdat"",invisible);";
      put "  Column(""&SITEIDLabel"") << Label(1);";
      put "  Column(""&TESTLabel"") << Label(1);";
      put "  Column(""&trSiteSampleSize"") << Label(1);";
      put "   VListBox(";
      put "   objRefNS:T1 = TextBox(locale:fill(locale:get(""IncludesvisitsandtimeswhereatleastX_Text""), {""&PropSubsCut""}), <<Set Wrap(1000),<< Font Color(5)),";
      %if %length(&RawCutOff) %then %do;
         put "   objRefNS:T2 = TextBox(locale:fill(locale:get(""XLineDrawnatMinusLogpX_Text""), {locale:get(""FDR_Text""), ""%sysfunc(round(&RawCutOff,0.0001))""})||"" (alpha=&AlphaValue)""),"; 
      %end;
      %else %do;
         put "   objRefNS:T2 = TextBox(locale:fill(locale:get(""NoPvalsSigAtAlphaXAfterAppXAdj_Text""), {""&AlphaValue"", locale:get(""FDR_Text"")}),<<Set Wrap(1000)),"; 
      %end;
      put "   HListBox(";
      put "    objRefNS:Filter = objRefNS:dvolcano << Data Filter(<<Conditional(1),";
      put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&trDOMAIN"",""&TESTLabel"")),";
      put "       Mode( Show(1), Include(1) )";
      put "    ),";      
      put "   objRefNS:VolcPlot = Graph Builder(";
      put "    Auto Stretching(0),";
      put "    Automatic Recalc(1),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "    X( :&trDiffMissingRates ),";
      put "    Y( :Name( ""&trNegLog10(&trFisherExact)"" ) ),";
      put "    Color( :&SITEIDLabel )";
      put "   ),";
      put "   Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       OutlineBox,";
      put "       {Set Title( locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""MissingFindings_Text"")}) )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trDiffMissingRates"",";
      put "       ScaleBox,";
      put "       {}";
      put "    ),";
      %if %length(&RawCutOff) %then %do;
         put "    Dispatch(";
         put "       {},";
         put "       ""&trNegLog10(&trFisherExact)"",";
         put "       ScaleBox,";
         put "       {Add Ref Line(";
         put "          &RawCutOff,";
         put "          Dashed,";
         put "          ""Red"",";
         put "          """",";
         put "          2";
         put "       )}";
         put "    ),";
      %end;
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 6 )} )";
      put "   )";
      put "   )";
      put "));";
      put ");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dvolcano""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trVolcanoPlot""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""MissingFindings_Text"")})));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T1,objRefNS:T2,objRefNS:VolcPlot}});";
   %end;
   %else %do;
      put "objRefNS:volcano=Function({},{default local},";
      put "   VListBox(";
      put "   objRefNS:T1 = TextBox(locale:get(""Testscannotbeperformedwithonlyonesitepresent_Text""), <<Set Wrap(1000),<< Font Color(5))";
      put "));";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""MissingFindings_Text"")})));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T1}});";     
   %end;
   
   %let domdatlist = ;
   %let firstdomdat = 0;   
   %let i = 1;
   %do %while(%length(%scan(&findingsdomains,&i,' ')));
      %let temp = %scan(&findingsdomains,&i,' ');
      %let templab = %bquote(%scan(&findingsdomainslabel,&i,~));
      %if %sysfunc(indexw(&availabledomains, &temp, ' ')) %then %do; 
         %if ^&firstdomdat %then %do;
            %let domdatlist = ""objRefNS:df&temp"";
            %let firstdomdat = 1;
         %end;
         %else %let domdatlist = &domdatlist,""objRefNS:df&temp"";      
         put "objRefNS:f&temp=Function({},{default local},";
         put "   objRefNS:df&temp = open(""&ClientOutPath.&&OutDataName_&temp...sas7bdat"",invisible);";
         put "   column(""&&VISITNUML&temp"") << Set Modeling Type( Ordinal );";
         put "   objRefNS:mergelist&temp = ""&SUBJECTLabel~&TESTlabel~&&VISITNUML&temp"";";
         %if %sysfunc(indexw(&&byvar&temp, VISIT, ' ')) %then %do;
            put "   column(""&&VISITL&temp"") << Set Modeling Type( Nominal );";
            put "   objRefNS:mergelist&temp = objRefNS:mergelist&temp||""~""||""&&VISITL&temp"";";
         %end;
         %if %sysfunc(indexw(&&byvar&temp, TPTNUM, ' ')) %then %do;
            put "   column(""&&TPTNUML&temp"") << Set Modeling Type( Nominal );";
            put "   objRefNS:mergelist&temp = objRefNS:mergelist&temp||""~""||""&&TPTNUML&temp"";";
         %end;
         put "   column(""&trFrequencyofRecordswithAvail"") << Set Modeling Type( Ordinal );";        
         put "   column(""&trFrequencyofRecordswithMissing"") << Set Modeling Type( Ordinal );";         
         put "   column(""&trMissingStatus"") << Set Modeling Type( Ordinal );";
         put "   VListBox(";
         %if &i = 1 %then %do;
            put "   objRefNS:TD1 = TextBox(locale:get(""Recordswithmissingdataare_Text""), <<Set Wrap(1000),<< Font Color(5)),"; 
            put "   objRefNS:TD1 = TextBox(locale:get(""Otherwisefrequencyofrecordswithavailabledata_Text""), <<Set Wrap(1000),<< Font Color(5)),"; 
            put "   TextBox(""""),"; 
         %end;
         put "   If(nrows(objRefNS:df&temp) > 0,";
         put "   HListBox(";
         put "   objRefNS:Filter = objRefNS:df&temp << Data Filter(<<Conditional(1),";
         put "        Add Filter(";
         put "           Columns(:Name(""&COUNTRYLabel""), :Name(""&SITEIDlabel""), :Name(""&trFrequencyofRecordswithAvail""), :Name(""&trPropSubs""), :Name(""&trMissingStatus""))"; 
         put "        ),";
         put "        Mode( Show(1), Include(1) )";
         put "   ),";       
         put "   objRefNS:Dist&temp = Distribution(";
         put "      Automatic Recalc(1),";
         put "      Nominal Distribution( Column( :&SITEIDlabel ), Show Counts( 1 )  ),";
         put "      Nominal Distribution( Column( :&TESTlabel ), Show Counts( 1 )  ),";
         put "      Nominal Distribution( Column( :Name(""&&VISITNUML&temp"") ), Show Counts( 1 )  ),";
         %if %sysfunc(indexw(&&byvar&temp, VISIT, ' ')) %then %do;
            put "      Nominal Distribution( Column( :&&VISITL&temp ), Show Counts( 1 )  ),";         
         %end;
         %if %sysfunc(indexw(&&byvar&temp, TPTNUM, ' ')) %then %do;
            put "      Nominal Distribution( Column( :&&TPTNUML&temp ), Show Counts( 1 )  ),";
         %end;
         put "      Nominal Distribution( Column( :&trFrequencyofRecordswithAvail ), Show Counts( 1 )  ),";
         /* put "      Nominal Distribution( Column( :&trFrequencyofRecordswithMissing ), Show Counts( 1 )  ),"; */
         put "      SendToReport(";
         put "         Dispatch(";
         put "          {""Distributions"", ""&SITEIDlabel""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";
         put "         Dispatch(";
         put "          {""Distributions"", ""&TESTlabel""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";
         put "       Dispatch(";
         put "          {""Distributions"", ""&&VISITNUML&temp""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";
         %if %sysfunc(indexw(&&byvar&temp, VISIT, ' ')) %then %do;
            put "       Dispatch(";
            put "          {""Distributions"", ""&&VISITL&temp""},";
            put "          ""Frequencies"",";
            put "          OutlineBox,";
            put "          {Close( 1 )}";
            put "       ),";        
         %end;
         %if %sysfunc(indexw(&&byvar&temp, TPTNUM, ' ')) %then %do;
            put "       Dispatch(";
            put "          {""Distributions"", ""&&TPTNUML&temp""},";
            put "          ""Frequencies"",";
            put "          OutlineBox,";
            put "          {Close( 1 )}";
            put "       ),";           
         %end;
         put "       Dispatch(";
         put "          {""Distributions"", ""&trFrequencyofRecordswithAvail""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";
         /* put "       Dispatch(";
         put "          {""Distributions"", ""&trFrequencyofRecordswithMissing""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       )"; */
         put "      )";
         put "   )";
         put "   );";
         put "   , //else";
         put "   objRefNS:Dist&temp = TextBox(locale:fill(locale:get(""NodataexpectedwhereatleastX_Text""), {""&PropSubsCut""}), <<Set Wrap(1000),<< Font Color(5));";
         put "   );";
         put "   );";
         %if %sysfunc(indexw(&MissDom, &temp, ' ')) %then %do;
            put "Close(objRefNS:df&temp,nosave);";
         %end;
         put ");";
         put "TabToDisplay = ""&templab"";";
         put "if (TabToDisplay == ""ECG Test Results"", TabToDisplay = locale:get(""ECGTestResults_Text""););"; 
         put "if (TabToDisplay == ""Laboratory Test Results"", TabToDisplay = locale:get(""LaboratoryTestResults_Text""););"; 
         put "if (TabToDisplay == ""Vital Signs"", TabToDisplay = locale:get(""VitalSigns_Text""););";         
         put "InsertInto(objRefNS:TabOpenList,1);";
         %if %sysfunc(indexw(&MissDom, &temp, ' ')) %then %do;
            put "InsertInto(objRefNS:DataList,{{""""}});";
            put "InsertInto(objRefNS:DataNameList,{{""""}});";
         %end;
         %else %do;
            put "InsertInto(objRefNS:DataList,{{""objRefNS:df&temp""}});";
            put "InsertInto(objRefNS:DataNameList,{{""&temp &trDistribution""}});";
         %end;
         put "InsertInto(objRefNS:TabButtonList, TabToDisplay||"" (&temp)"");";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:f&temp()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:Dist&temp}});";       
      %end;
      %let i = %eval(&i+1);      
   %end;    
   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
   put "objRefNS:ppdd = evalexpr(Function({},{default local},";
   put "this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   put "objRefNS:spdd =  Function({}, {},";
   put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";

   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{},{}};";
   put "objRefNS:ActionCodeList ={{},{}};";
   put "objRefNS:ActionDataTableList = {{},{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsfromVolcanothenClick_Text""))});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSites_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSite()));";      
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowMissingBar_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:MissingBar()));";
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
   
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsfromDomainTabsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:ppdd()));";    
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&domdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&domdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ShowRecordswithMissingData_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:MissingRecord()));";    
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&domdatlist}});";

   put "objRefNS:ActionSelectionCountList = {{1,1},{1,1,1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.DataSetHighlightRows.gif"",""&IconPath.BarGraph.gif""},";
   put "{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"",""&IconPath.ShowRecordsWithMissingResults.gif""}};";
   put "objRefNS:ActionRequirementFlagList = {{0, 0},{3, 0, 0}};";
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &MissingFindingsScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend MissingFindings;

* run the macro;
%MissingFindings;