/*
JMP Clinical Analytical Process

Name: Multivariate Inlier
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

This process calculates Mahalanobis Distance to identify multivariate inliers and outliers
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName MultivariateInlierScript MultivariateInlier_OutData OutPath 
        ServerOutPath PopulationSet MissMax FindDistFl InterDistFl EventDistFl DelMissFl WhereStmt WhereForReport AgeSexFl SiteCount ResType FindTests AllFind EventIntInc;

*ProcessBody;

%macro MultivariateInlier;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and ((FINDINGS.USUBJID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTESTCD and FINDINGS.xxTEST and FINDINGS.VISITNUM) or
          (EVENTS.USUBJID and EVENTS.xxDECOD) or (INTERVENTIONS.USUBJID and INTERVENTIONS.xxDECOD))$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: CPV.{"Age and Sex","Yes"}$ $UE: DM.AGE and DM.SEX $
$CVS_STOP$

$CVS_START$
$CMP: CPV.{"Findings domains","Yes"}$ $UE: FINDINGS.USUBJID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTESTCD and FINDINGS.xxTEST and FINDINGS.VISITNUM$ $NOTES: Only domains with these criteria are used$
$CMP: OSYS$ $UE: FINDINGS.VISIT$ $NOTES: To remove UNSCHEDULED visits, if available$ 
$CMP: OSYS$ $UE: FINDINGS.xxTPTNUM$ $NOTES: Will be used to get findings by time point also, if available.$ 
$CMP: CPV.{"Analyze findings using:","Numeric results in standard units"}$ $UE: FINDINGS.xxSTRESN$ $NOTES: Only domains with these criteria are used$
$CMP: CPV.{"Analyze findings using:","Results in original units"}$ $UE: FINDINGS.xxORRES$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$

$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"EventOccur"}}$
$CMP: CPV.{"Event domains","Yes"}$ $UE: EVENTS.USUBJID and EVENTS.xxDECOD$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$

$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"InterventionOccur"}}$
$CMP: CPV.{"Intervention domains","Yes"}$ $UE: INTERVENTIONS.USUBJID and INTERVENTIONS.xxDECOD$ $NOTES: Only domains with these criteria are used$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = MultivariateInlier; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%OUTNAMETRUNC(outdat = MultivariateInlier_OutData, truncnum = 19);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);

%if &AgeSexFl = No and &EventDistFl = No and &InterDistFl = No and &FindDistFl = No %then %do;
   %put [MS_S_0061];
   %put ERROR: At least one domain type must be selected for analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

%CHECKPRES(domain = ADSL, var = AGE);
%CHECKPRES(domain = ADSL, var = SEX);

%let trMahalanobisDistance = %nrbquote(%LocaleGet(MahalanobisDistance_Text));
%let trMissingData = %nrbquote(%LocaleGet(MissingData_Text));

%let _tid_ = %sysfunc(open(&in_adsl));
%let vn = %sysfunc(varnum(&_tid_,USUBJID));
%let USubLen = %sysfunc(varlength(&_tid_,&vn));
%let _tid_ = %sysfunc(close(&_tid_));

%if ^%length(&findingsdomains) %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA(varlist = arm &AGE &sex);
%GETSITELISTS;
%if &exiterror %then %goto exit;

%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);

proc sort data = subject;
   by siteid;
run;

data subject;
   merge subject(in = ina) siteids(in = inb);
   by siteid;
   if ina and inb;
run;

proc sort data = subject;
   by usubjid;
run;

proc sql noprint;
   create table subpairs as
      select aa.usubjid, bb.usubjid2, aa.siteid
      from subject as aa, subject(rename = (usubjid = USUBJID2 siteid = siteid2)) as bb   
      where aa.usubjid < bb.usubjid2 and siteid = siteid2   
      order by aa.usubjid, bb.usubjid2;
quit;

%let availabledomains = ;
%if &FindDistFl = Yes and %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &findingsdomains, flist = &FindTests);
%end;
%GETFINDINGS(predose = 0);
%if &EventDistFl = Yes %then %do;
   %INTEVENT(list = &eventsdomains, predose = 0);
%end;
%if &InterDistFl = Yes %then %do;
   %INTEVENT(list = &interventionsdomains, predose = 0); 
%end;

%if ^%length(&availabledomains) and ((^%length(&AGE) and ^%length(&SEX) and &AgeSexFl = Yes) or &AgeSexFl = No) %then %do;
   %put ERROR: No domains meet the necessary criteria for the analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

data all;
   merge subject(in = ina) &availabledomains end = eof;
   by usubjid;
   if ina;
   %if %length(&sex) %then %do;
      if sex = "M" or sex="&trM" then MALE = 1;
      else if sex = "F" or sex ="&trF" then MALE = 0;
      drop sex;
      label male = "&trMaleGender";
   %end;
   %if &AgeSexFl = No %then %do;
      %if %length(&AGE) %then %do;
         drop age;
      %end;
      %if %length(&sex) %then %do;
         drop male;
      %end;
   %end;
   if eof then call symput("TotalSubjects", trim(left(_n_)));
run;

proc datasets nolist;
   delete &availabledomains;
quit;

proc means std noprint data = all;
   var _numeric_;
   output out = stddev(drop = _type_ _freq_) std = / autoname;
run;

proc transpose data = stddev out = stddev;
run;

data stddev;
   set stddev(rename = (col1 = stddev)) end = eof;
   _name_ = substr(_name_,1,length(_name_)-%length(StdDev)-1);
   where stddev = 0;
run;

%let NoVariance =;
proc sql noprint;
   select _Name_ 
      into :NoVariance separated by ' '
      from stddev(where = (upcase(_Name_) ne "SITESAMP"))
      order by _Name_;
quit;

%if %length(&NoVariance) %then %do;
   data all;
      set all;
      drop &NoVariance;
   run;
%end;

%NObsVars(all);
%let TotalSubs = &nobs;

proc means nmiss noprint data = all;
   output out = nmiss(drop = _type_ _freq_) nmiss = / autoname;
run;

proc transpose data = nmiss out = nmiss;
run;

data nmiss;
   set nmiss(rename = (col1 = nummiss) where = (_name_ ne "SiteSamp_NMiss")) end = eof;
   _name_ = substr(_name_, 1, length(_name_)-%length(&trNMiss)-1);
   pctmiss = round(nummiss / &TotalSubs * 100, 0.01);
   %if %upcase(&DelMissFl) = YES %then %do; 
      if pctmiss > 0 then missflag = 2;
   %end;
   %else %do;
      if pctmiss >= &MissMax then missflag = 2;
   %end;
   else if pctmiss ne 0 then missflag = 1;
   else if pctmiss = 0 then missflag = 0;
   if eof then call symput("TotalVars", trim(left(_n_)));
   label _name_ = "&trVariable" _label_ = "&trLabel" nummiss = "&trNOSMV" pctmiss = "&trPOSMV" missflag = "&trMissFlag";
run;

*** degree of freedom is number of variables contributing to maha scores;
%let DegFree = 0;
data _null_;
   set nmiss(where = (missflag in (0,1))) end = eof;
   if eof then call symput("DegFree", trim(left(_n_)));
run;

%let MissyVar = 0;
data _null_;
   set nmiss(where = (missflag in (1))) end = eof;
   if eof then call symput("MissyVar", trim(left(_n_)));
run;

%let RemovedVars = 0;
data _null_;
   set nmiss(where = (missflag = 2)) end = eof;
   if eof then call symput("RemovedVars", trim(left(_n_)));
run;

%if ^%length(&RemovedVars) %then %let RemovedVars = 0;
%if ^%eval(&TotalVars - &RemovedVars) %then %do;
   %put [MS_S_0060];
   %put ERROR: No variables remain for analysis after deleting variables with at least &MissMax percent missing data.;
   %let exiterror = 1;
   %goto exit;
%end;
%let OtherVars = %eval(&MissyVar);

%let MissMaxVars = ;
%let MissNoMaxVars =;
%let NoMissVars =;
proc sql noprint;
   select _name_ 
      into :MissMaxVars separated by ' '
      from nmiss
      where missflag = 2
      order by _name_;
   select _name_ 
      into :MissNoMaxVars separated by ' '
      from nmiss
      where missflag = 1
      order by _name_;
   select _name_ 
      into :NoMissVars separated by ' '
      from nmiss
      where missflag = 0
      order by _name_;
quit;

data drops;
   set all(drop = &MissMaxVars);
   nmissing = nmiss(of %sysfunc(transtrn(&MissNoMaxVars, %str( ), %str(,))));
   if nmissing > 0 then output drops;
run;

%let SubMissMaha = 0;
data _null_;
   set drops end = eof;
   if eof then call symput("SubMissMaha", trim(left(_n_)));
run;

proc datasets nolist;
   delete drops;
quit;

%let Ref = %sysfunc(sqrt(&DegFree));

%let MultivariateInlier_OutData_1 =;
%PrepOutDS(MultivariateInlier_OutData_1,OutDataName_1,&Outname.mvio);   
%let MultivariateInlier_OutData_2 =;
%PrepOutDS(MultivariateInlier_OutData_2,OutDataName_2,&Outname.mviomiss);

data &MultivariateInlier_OutData_1;
   set all;
   drop &MissMaxVars;
   label usubjid = "&SUBJECTLabel";
run;

data &MultivariateInlier_OutData_2;
   set nmiss(where = (missflag > 0));
   label usubjid = "&SUBJECTLabel";
run;

%NObsVars(&MultivariateInlier_OutData_2);
%let MissFlag = &nobs;

%let includeds = &OutDataName_0..sas7bdat,&OutDataName_1..sas7bdat,&OutDataName_2..sas7bdat;

proc datasets nolist;
   delete all nmiss;
quit;    

* create JSL file;
%PrepOutFile(MultivariateInlierScript,MultivariateInlierScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &MultivariateInlierScript;
%let FullAPName=%str(Multivariate Inliers and Outliers);

* put JSL statements;
data _null_;
   file "&MultivariateInlierScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 

   %let domdatlist = ""objRefNS:dmaha"";
   put "objRefNS:fbox=Function({},{default local},";
   put "objRefNS:dmaha=open(""&ClientOutPath.&OutDataName_1..sas7bdat"",invisible);";
   put "objRefNS:dmaha << Color By Column(:Name(""&SITEIDlabel""),ColorTheme(""&trJMPDark""));";
   put "Column(""&SubjectLabel"") << Label(1);";
   put "  Column(""&trSiteSampleSize"") << Label(1);";      
   put "if(""&SITEIDlabel"" == ""Study Site Identifier"", trSITEIDlabel = locale:get(""StudySiteIdentifier_Text""),";
   put " trSITEIDlabel = ""&SITEIDlabel"");";
   put "trMahDist = ""&trMahDist"";";
   put "DistanceVars = objRefNS:dmaha << Get Column Names(Numeric, string);";
   put "Remove From(DistanceVars,loc(DistanceVars, ""&trSiteSampleSize"")[1]);";   
   put "VListBox(";
   %if %upcase(&DelMissFl) = YES %then %do;
      put "objRefNS:B=textbox(locale:fill(locale:get(""MahDistCompFromXofXPossVarXVarRemSlidXAll_Text""), {""&DegFree"", ""&TotalVars"", ""&RemovedVars""}),<<Set Wrap(1000),<< Font Color(5)),";  
   %end;
   %else %do;  
      put "objRefNS:B=textbox(locale:fill(locale:get(""MahDistCompFromXofXPossVarXVarRemSlidXPerc_Text""), {""&DegFree"", ""&TotalVars"", ""&RemovedVars"", ""&MissMax""}),<<Set Wrap(1000),<< Font Color(5)),"; 
   %end;
   %if &OtherVars %then %do;
      put "objRefNS:C=textbox(locale:fill(locale:get(""XVarsMissDatRatesltXPercBecXVarsXSubjNoMahDist_Text""), {""&OtherVars"", ""&MissMax"", ""&OtherVars"", ""&SubMissMaha""}),<<Set Wrap(900),<< Font Color(5)),"; 
   %end;
   %else %do;
      put "objRefNS:C=textbox("" "",<<Set Wrap(900),<< Font Color(5)),";
   %end; 
   put "textbox(""""),"; 
   put "objRefNS:F=textbox(locale:get(""ClickingIncludeFilterrecomputesMaha_Text""),<<Set Wrap(900),<< Font Color(5)),";
   put "textbox(""""),"; 
   put " HListBox(";
   put " objRefNS:Filter = objRefNS:dmaha << Data Filter(<<Conditional(1),";
   put "       Add Filter(";
   put "          Columns(:Name(""&COUNTRYLabel""), :Name(""&SITEIDlabel""))"; 
   put "       ),";
   put "       Mode( Show(1), Include(0) )";
   put "    ),";
   put " VListBox(objRefNS:M = objRefNS:dmaha << Multivariate(";
   put "       Automatic Recalc(1),";  
   put "       Y(Eval(Parse(EvalInsert(""\[^DistanceVars^]\"")))),";
   put "    Estimation Method( ""Pairwise"" ),";
   put "    Scatterplot Matrix( 0 ),";
   put "    Correlations Multivariate( 0 ),";
   put "    Mahalanobis Distances( 1, Save Outlier Distances),";
   put "    SendToReport(";
   put "       Dispatch(";
   put "          {""Outlier Analysis"", ""Mahalanobis Distances""},";
   put "          ""Multiv Outlier"",";
   put "          FrameBox,";
   put "          {Frame Size( 940, 236)}";
   put "       ),";
   put "       Dispatch(";
   put "          {""Outlier Analysis"", ""Mahalanobis Distances""},";
   put "          ""Multiv Outlier"",";
   put "          FrameBox( 2 ),";
   put "          {Frame Size( 56, 236)}";
   put "       )";
   put "    ) ";
   put " ),";
   put "textbox(""""),"; 
   put "Column(objRefNS:dmaha,N Cols(objRefNS:dmaha)) << Set Name( ""&trDistance"" ),";
   put "HListBox(";
   put "    objRefNS:MD=graph Builder(";
   put "       Automatic Recalc(1),";
   put "       Auto Stretching( 0 ),";
   put "       Show Legend( 0 ),";
   put "       Show Control Panel( 0 ),";
   put "       Variables( Y( :Name(""&trDistance"") ) ),";
   put "       Elements(";
   put "          Box Plot(";
   put "             X,";
   put "             Y,";
   put "             Legend( 3 ),";
   put "             Jitter( 1 ),";
   put "             Outliers( 0 )";
   put "          ),";
   put "          Points( Y, Legend( 4 ), Jitter( 1 ) )";
   put "       ),";
   put "       SendToReport(";
   put "          Dispatch({},""Graph Builder"",OutlineBox, {Set Title( locale:fill(locale:get(""XforallSites_Text""), {trMahDist}) )}),";
   put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
   put "          Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
   put "          Dispatch({},""&trDistance"",ScaleBox, {Add Ref Line( &Ref, Dashed, ""Red"", ""E(&trDistance)"", 2)})";
   put "       )";
   put "    ),";        
   put "    objRefNS:MDS = objRefNS:dmaha << Graph Builder(";
   put "       Automatic Recalc(1),";
   put "       Auto Stretching( 0 ),"; 
   put "       Show Legend( 0 ),";
   put "       Show Control Panel( 0 ),";
   put "       Variables( X( :Name(""&SITEIDlabel"") ), Y( :Name(""&trDistance"") ) ),";
   put "       Elements(";
   put "          Box Plot(";
   put "             X,";
   put "             Y,";
   put "             Legend( 3 ),";
   put "             Jitter( 1 ),";
   put "             Outliers( 1 )";
   put "          )";
   put "       ),";
   put "       SendToReport(";
   put "          Dispatch({},""Graph Builder"",OutlineBox, {Set Title( locale:fill(locale:get(""XvsX_Text""), {trMahDist, trSITEIDlabel}) )}),";
   put "          Dispatch( {}, ""&SITEIDlabel"", ScaleBox, {Show Major Ticks( 0 )} ),";
   put "          Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
   put "          Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 3 )} ),";
   put "          Dispatch({},""&trDistance"",ScaleBox, {Add Ref Line( &Ref, Dashed, ""Red"", ""E(&trDistance)"", 2)})";
   put "       )";
   put "    ),";
   put "),"; 
   put "),"; *end of Vlist;
   put ")));"; 


   put "InsertInto(objRefNS:DataList,{{""objRefNS:dmaha""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trMahalanobisDistance""}});";
   put "TabToDisplay = ""&trMahDist"";";
   put "if(TabToDisplay == ""Mahalanobis Distance"", TabToDisplay = locale:get(""MahalanobisDistance_Text""););";
   put "InsertInto(objRefNS:TabButtonList, TabToDisplay);";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:B,objRefNS:C,objRefNS:F,objRefNS:M,objRefNS:MD,objRefNS:MDS}});";
   %if &MissFlag %then %do;
      %let domdatlist = &domdatlist,""objRefNS:dmahamiss"";
      put "objRefNS:fbox2=Function({},{default local},";
      put "objRefNS:dmahamiss=open(""&ClientOutPath.&OutDataName_2..sas7bdat"",invisible);";
      put "   column(""&trMissFlag"") << Set Modeling Type( Ordinal );";
      put "VListBox(";
      put "textbox(locale:fill(locale:get(""MissFlag1ReflVarsMissDatNotRemDialOptSlidXPerc_Text""), {""&MissMax""}),<<Set Wrap(1000),<< Font Color(5)),"; 
      put "textbox(locale:get(""SubjWithMissValDoNotHaveCompMahDist_Text""),<<Set Wrap(700),<< Font Color(5)),"; 
      put "textbox(locale:get(""MissFlag2ReflVarsMissDatRemPriorCompMahDist_Text""),<<Set Wrap(700),<< Font Color(5)),"; 
      put "textbox(""""),"; 
      put "HListBox(";
      put "    objRefNS:Filter = objRefNS:dmahamiss << Data Filter(<<Conditional(1),";
      put "       Add Filter(";
      put "          Columns(:&trMissFlag)"; 
      put "       ),";
      put "       Mode( Show(1), Include(1) )";
      put "    ),";      
      put "objRefNS:FDist=Distribution(";
      put "         Automatic Recalc(1),";
      put " Freq ( :&trNOSMV ),";
      put " Nominal Distribution( Column( :&trLabel ), Show Counts( 1 ) ),";
      put " SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trLabel"",";
      put "       OutlineBox,";
      put "       {Set Title( locale:get(""VariableswithMissingDataFreq_Text"") )}";
      put "    ),";
      put "    Dispatch( {}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
      put " )";
      put "),";
      put "objRefNS:PDist=Distribution(";
      put "         Automatic Recalc(1),";
      put " Freq ( :&trPOSMV ),";
      put " Nominal Distribution( Column( :&trLabel ), Show Counts( 1 ) ),";
      put " SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trLabel"",";
      put "       OutlineBox,";
      put "       {Set Title( locale:get(""VariableswithMissingDataPercent_Text"") )}";
      put "    ),";
      put "    Dispatch( {}, ""Frequencies"", OutlineBox, {Close( 1 )} )";
      put " )";
      put ")";
      put ")";
      put ");"; 
      put ");"; 
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dmahamiss""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trMissingData""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""MissingData_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox2()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:FDist,objRefNS:PDist}});";
   %end;
   %else %do;  
   %end; 
   
   put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   put "objRefNS:cpdd =  evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); clusterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""); clusterFunc(this:getParent());));";
   put "objRefNS:fpdd = evalexpr(Function({},{default local},  this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); reviewRefNS = namespace(reportRefNS:getParent()); createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< newContext); createFilterFunc(reviewRefNS << getName);));";  
   put "objRefNS:spdd =  Function({}, {}, Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";

   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectSubjectsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ppdd()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&domdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&domdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ClusterSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:cpdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&domdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""CreateSubjectFilter_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:fpdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&domdatlist}});";

   put "objRefNS:ActionSelectionCountList = {{1,1,3,1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"",""&IconPath.HierarchicalClustering.gif"",""&IconPath.AddFilter.gif""}};";
   put "objRefNS:ActionRequirementFlagList = {{3, 0, 3, 2}};";
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &MultivariateInlierScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend MultivariateInlier;
%MultivariateInlier;

************************************;
************************************;
************************************;