/*
JMP Clinical Analytical Process

Name: Outliers
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   his process identifies outliers and identifies sites with excessive outliers
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName OutliersScript Outliers_OutData OutPath ServerOutPath PopulationSet 
        ResType WhereStmt WhereForReport SiteCount ResType FindTests AllFind DQFByVars AlphaValue;

*ProcessBody;

%macro Outliers;

/*
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTEST and FINDINGS.xxTESTCD and FINDINGS.USUBJID$ $NOTES: At least two sites are needed.$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: OSYS$ $UE: FINDINGS.xxLOC $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxMETHOD $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxPOS $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxSPEC $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxTPT $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxCAT $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxSCAT $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: CPV.{"Analyze:","Numeric results in standard units"}$ $UE: FINDINGS.xxSTRESN$ $NOTES: Only domains with these criteria are used$
$CMP: CPV.{"Analyze:","Results in original units"}$ $UE: FINDINGS.xxORRES$ $NOTES: Only domains with these criteria are used$
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$ 
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = Outliers; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds = ;
%OUTNAMETRUNC(outdat = Outliers_OutData, truncnum = 17);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);

%let trXVolcanoPlot = %nrbquote(%LocaleGet(XVolcanoPlot_Text));
%let trXDistribution = %nrbquote(%LocaleGet(XDistribution_Text));
%let blank =;
%let trVolcanoPlot = %nrbquote(%LocaleFill(&trXVolcanoPlot, &blank));
%let trDistribution = %nrbquote(%LocaleFill(&trXDistribution, &blank));
%let trVolcanoPlot = %sysfunc(strip(&trVolcanoPlot));
%let trDistribution = %sysfunc(strip(&trDistribution));

%let domainlist = &findingsdomains;

%if ^%length(&findingsdomains) %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA(varlist = arm);
%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);

%GETSITELISTS(errflg = 1);
%if &exiterror %then %goto exit;

%let got1 = 0;
%DOMBYLISTS(inclvisit = 0, incllabs = 0);
%if %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &domainlist, flist = &FindTests);
%end;

%if ^&got1 and %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %do;
   %put [MS_S_0077];
   %put ERROR: No findings domains have the xxSTRESN variable. This variable is required to compute outliers.;
   %let exiterror = 1;
   %goto exit;
%end;
%else %if ^&got1 and %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;
   %put ERROR: No findings domains have the xxORRES variable. This variable is required to compute outliers.;
   %let exiterror = 1;
   %goto exit;
%end;

%let flag = 1;
%let TestLen = 0;
%let CatLen = 0;
%let ScatLen = 0;
%let LocLen = 0;
%let MethodLen = 0;
%let PosLen = 0;
%let SpecLen = 0;
%let TptLen = 0;
%let i = 1;
%let OverallByVar =;
%let OverallByVarLabs =;
%let nOverallByvar = 0;
%let AvailDomains = ;
%let first = 0;
%put domainlist = &domainlist;
%do %while(%length(%scan(&domainlist,&i,' ')));
   %let temp = %scan(&domainlist,&i,' ');
   %let tind = %sysfunc(left(%sysfunc(countw(%sysfunc(compbl("&&&var_&temp")), " "))));

   %if %upcase(&AllFind) = NO %then %do;
      %let ifcond = and %length(&&&temp.findlist);
   %end;
   %else %do;
      %let ifcond =;
   %end;   
   
   %if %length(&&&var_&temp) &ifcond %then %do;
      %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
      
      %CHECKPRES(domain = &temp, var = VISITNUM);     
      
      data &temp(keep = usubjid &temp.test &VISITNUM &&&byvar_&temp &&&var_&temp);
         set &&in_&temp;
         %if %upcase(&AllFind) = NO %then %do;
            where &temp.testcd in ( %unquote(&&&temp.findlist) ) and not missing (&&&var_&temp);
         %end;
         %else %do;
            where not missing (&&&var_&temp);
         %end;            
      run;      
      
      proc sort data = &temp out = &temp;
         by usubjid &&&byvar_&temp &temp.test &&&var_&temp;
      run;
      
      %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;                  
         %let ORRESLabel = &&&var_&temp;
         %CheckLabel(&temp,ORRESLabel);
         
         data &temp(where = (not missing (&&&var_&temp)));
            set &temp(rename = (&&&var_&temp = cval));
            &&&var_&temp = input(cval,best12.);
            drop cval;
            label &&&var_&temp = "&ORRESLabel";
         run;     
      %end;
      
      %NObsVars(&temp);                  
      %if &nobs %then %let AvailDomains = &AvailDomains &temp;

      %let _tid_ = %sysfunc(open(&temp));
      %let vn = %sysfunc(varnum(&_tid_,&temp.test));
      %let TestLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.cat));
      %let CatLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.scat));
      %let ScatLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.loc));
      %let LocLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.method));
      %let MethodLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.pos));
      %let PosLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.spec));
      %let SpecLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.tpt));
      %let TptLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %if %eval(&TestLentmp>&TestLen) %then %let TestLen = &TestLentmp; 
      %if %eval(&CatLentmp>&CatLen) %then %let CatLen = &CatLentmp; 
      %if %eval(&ScatLentmp>&ScatLen) %then %let ScatLen = &ScatLentmp; 
      %if %eval(&LocLentmp>&LocLen) %then %let LocLen = &LocLentmp; 
      %if %eval(&MethodLentmp>&MethodLen) %then %let MethodLen = &MethodLentmp;
      %if %eval(&PosLentmp>&PosLen) %then %let PosLen = &PosLentmp; 
      %if %eval(&SpecLentmp>&SpecLen) %then %let SpecLen = &SpecLentmp;
      %if %eval(&TptLentmp>&TptLen) %then %let TptLen = &TptLentmp;           
      
      data &temp;
         merge subject(in = ina keep = usubjid siteid) &temp(in = inb);
         by usubjid;
         if ina and inb;
      run;
      
      proc sort data = &temp;
        by &&&byvar_&temp &temp.test;
      run;
      
      proc means noprint missing nway data = &temp;
         class &&&byvar_&temp &temp.test;
         var &&&var_&temp;
         output out = stats(drop = _TYPE_ _FREQ_) Q1 = Q1 Q3 = Q3 QRANGE = QRANGE;
      run;     
      
      data &temp(drop = Q1 Q3 QRANGE);
         merge &temp(in = ina) stats;
         by &&&byvar_&temp &temp.test;
         LOutlier = Q1 - 1.5*QRANGE;
         UOutlier = Q3 + 1.5*QRANGE;
         if not (LOutlier <= &&&var_&temp <= UOutlier) then Outlier = 1;
         else Outlier = 0;
         int = 1;
         label Outlier = "&trOutlier";
      run;
      
      %let Outliers_OutData_&temp =;      
      %PrepOutDS(Outliers_OutData_&temp,OutDataName_&temp, &Outname.Outlier, _&temp);
      %let includeds = &includeds &&OutDataName_&temp...sas7bdat,;
      
      data &&Outliers_OutData_&temp;
         length domain $ 8 OutlierType $ 15;
         set &temp;
         domain = "&temp";
         drop int;
         %let j = 1;
         %do %while(%length(%scan(&&&byvar_&temp,&j,' ')));
            %let tempvar = %scan(&&&byvar_&temp,&j,' ');                
            %if ^%index(&OverallByVar,%substr(&tempvar,%eval(%length(&temp)+1))) %then %do;
               %let OverallByVar = &OverallByVar %substr(&tempvar,%eval(%length(&temp)+1));
               %let nOverallByvar = %eval(&nOverallByvar+1);
            %end;
            %LABELRENAME;
            %let j = %eval(&j+1);            
         %end;
         %if %length(&VISITNUM) %then %do;
            label VISITNUM = "&trVisitNum";
         %end;
         if &&&var_&temp > UOutlier then OutlierType = "&trOutlierAbove";
         else if &&&var_&temp < LOutlier then OutlierType = "&trOutlierBelow";
         else OutlierType = "&trNonOutlier";
         label &&&var_&temp = "&varsufflabel";      
         rename &&&var_&temp = &varsuff;
         label &temp.Test = "&trTestName";
         rename &temp.Test = test;
         label Domain = "&trDOMAIN" LOutlier = "&trLowerOutlier" UOutlier = "&trUpperOutlier" OutlierType = "&trOutlierType";
      run;           

      data &temp(drop = LOutlier UOutlier);
         set &temp;
      run;
      
      %GETFREQS(variable = Outlier);
      
      data &temp.2;
         length domain $ 8;
         set &temp.2(drop = numvals total rename = (&temp.test = Test));
         Domain = "&temp";
         label Test = "&trTestName" Domain = "&trDOMAIN";
         if missing(count) then count = 0;
      run;
               
      %if %length(&&&byvar_&temp) %then %do;
         data &temp.2;
            set &temp.2;
            %let j = 1;
            %do %while(%length(%scan(&&&byvar_&temp,&j,' ')));
               %let tempvar = %scan(&&&byvar_&temp,&j,' ');                
               %if ^%index(&OverallByVar,%substr(&tempvar,%eval(%length(&temp)+1))) %then %do;
                  %let OverallByVar = &OverallByVar %substr(&tempvar,%eval(%length(&temp)+1));
                  %let nOverallByvar = %eval(&nOverallByvar+1);
               %end;
               %LABELRENAME;
               %let j = %eval(&j+1);
            %end;
         run;
      %end;
   %end;
   %let i = %eval(&i+1);
%end;

%if ^%length(&AvailDomains) %then %do;
   %put ERROR: No Findings domains meet the necessary criteria for the analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

data alldata;
   length test $ &TestLen      
      %do i = 1 %to &nOverallByvar; 
         %let tempvar = %scan(&OverallByvar,&i,' ');
         &tempvar $ &&&tempvar.Len 
      %end;
   ;     
   set
   %let i = 1;
   %do %while(%length(%scan(&AvailDomains,&i,' ')));
      %let temp = %scan(&AvailDomains,&i,' '); 
      &temp.2
      %let i = %eval(&i+1);
   %end;
   ;
run;

%let i = 1;
%do %while(%length(%scan(&OverallByVar,&i,' ')));
   %let temp = %scan(&OverallByVar,&i,' '); 
   %let &temp.Label = &temp;
   %CheckLabel(alldata,&temp.Label);
   %if &i = 1 %then %let OverallByVarLabs = &&&temp.Label;
   %else %let OverallByVarLabs = &OverallByVarLabs~&&&temp.Label;
   %let i = %eval(&i+1);
%end;

proc sort data = alldata;
   by domain test &OverallByVar;
run;

proc datasets nolist;
   delete 
   %let i = 1;
   %do %while(%length(%scan(&AvailDomains,&i,' ')));
      %let temp = %scan(&domainlist,&i,' '); 
      &temp.2
      %let i = %eval(&i+1);
   %end;
   ;
quit;

%let firstflag = 0;
%let i = 1;
%do %while(%length(%scan(&SiteList,&i,' ')));
   %let tempsite = %scan(%upcase(&SiteList),&i," ");
   %let tempsitesize = %scan(%upcase(&SiteSamp),&i," ");
   %let trSiteTS = %nrbquote(%LocaleFill(&trSiteX, f1=%nrbquote(&tempsite)));
   %let trNotSiteTS = %nrbquote(%LocaleFill(&trNotSiteX, f1=%nrbquote(&tempsite)));
   data temp;
      length Label $ 30;
      set alldata;
      if siteid = "&tempsite" then Label = "&trSiteTS";
      else Label = "&trNotSiteTS";
     label Label = "&trLabel";
   run;

   proc freq noprint data = temp;
      tables label * Outlier / chisq sparse out = stats(drop = PERCENT PCT_COL) outpct;
      weight count;
      output out = rowmean(keep = domain test &OverallByVar XP2_FISH) fisher;
      by domain test &OverallByVar; 
   run;   

   %NObsVars(rowmean);
   %if &nobs %then %do;   
      data stats;
         length siteid $ &SiteidLen;
         set stats(rename = (PCT_ROW = Percent));
         SITEID = "&tempsite";
         SITESAMP = &tempsitesize;
         label percent = "&trPercentofMeasurements" siteid = "&SITEIDlabel" sitesamp = "&trSiteSampleSize";
      run;

      proc sort data = stats out = stats2;
         by domain test &OverallByVar Outlier label;
      run;

      data stats2;
         set stats2;
         by domain test &OverallByVar Outlier label;
         retain diff;
         if first.Outlier then diff = .;
         if first.Outlier and ^last.Outlier and kindex(label, "&trNot") then diff = -percent;
         else if last.Outlier and ^first.Outlier and ^kindex(label, "&trNot") then diff = diff + percent;
         if last.Outlier and diff > .;
         label diff = "&trDiffProps";
      run;

      data stats2;
         set stats2;
         diff2 = abs(diff);
      run;

      *** Take largest absolute difference, but in direction of single site if tied;
      proc sort data = stats2;
         by domain test &OverallByVar diff2 diff; 
      run;

      data stats2(drop = diff2 count percent);
         set stats2;
         by domain test &OverallByVar;
         %if %length(&OverallByVar) %then %do;
            if last.%scan(test &OverallByVar, -1, ' ');
         %end;
         %else %do;
            if last.test;
         %end;
      run;

      %nobsvars(rowmean);
      %if &nobs %then %do;
         data stats2(drop = Outlier);    
            merge stats2(in = ina) rowmean(rename = (XP2_FISH = Fisher));
            by domain test &OverallByVar;
            if ina;
            if missing(Fisher) then Fisher = 1;
            Nlog10Fisher = -log10(Fisher);
            label Fisher = "&trFisherExact" Nlog10Fisher = "&trNegLog10(&trFisherExact)";
         run;
      %end;

      %if ^&firstflag %then %do;
         data summary_volcano;
            length test $ &TestLen      
               %do j = 1 %to &nOverallByvar; 
                  %let tempvar = %scan(&OverallByvar,&j,' ');
                  &tempvar $ &&&tempvar.Len 
               %end;
            ;  
            set stats2;
         run;
         data summary_Outlier;
            length test $ &TestLen      
               %do j = 1 %to &nOverallByvar; 
                  %let tempvar = %scan(&OverallByvar,&j,' ');
                  &tempvar $ &&&tempvar.Len 
               %end;
            ;  
            set stats;
         run;
         %let firstflag = 1;
      %end;
      %else %do;   
         proc append force base = summary_volcano data = stats2;
         run;
         proc append force base = summary_Outlier data = stats;
         run;     
      %end;
   %end;
   
   proc datasets nolist;
      delete temp stats stats2 rowmean;
   quit;

   %let i = %eval(&i+1);
%end;

proc datasets nolist;
   delete alldata;
quit;

%FDRPVALUES(inpv = Fisher, nlog10p = Nlog10Fisher);

%let Outliers_OutData_v =;    
%PrepOutDS(Outliers_OutData_v,OutDataName_v, &Outname.Outlier, _vol);
%let includeds = &includeds &OutDataName_v..sas7bdat,;

data &Outliers_OutData_v;
   merge summary_volcano(in = ina) country(in = inb);
   by siteid;
   if ina and inb;
   %do i = 1 %to &nOverallByvar; 
      if missing(%scan(&OverallByvar,&i,' ')) then %scan(&OverallByvar,&i,' ') = "&trNA";
   %end;    
run;

%let Outliers_OutData_d = ;      
%PrepOutDS(Outliers_OutData_d,OutDataName_d, &Outname.Outlier, _hist);
%let includeds = &includeds &OutDataName_d..sas7bdat,;

data &Outliers_OutData_d;
   set summary_Outlier;
   label Outlier = "&trOutlier";
   %do i = 1 %to &nOverallByvar; 
      if missing(%scan(&OverallByvar,&i,' ')) then %scan(&OverallByvar,&i,' ') = "&trNA";
   %end; 
run;  

%let TESTlabel = TEST;
%CheckLabel(summary_Outlier,TESTlabel);

proc datasets nolist;
   delete summary_Outlier summary_volcano country subject;
quit;

* create JSL file;
%PrepOutFile(OutliersScript,OutliersScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &OutliersScript;
%let FullAPName=%str(Outliers);

* put JSL statements;
data _null_;
   file "&OutliersScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:OverallByVar = ""&OverallByVar"";";
   put "objRefNS:OutlierStatus = ""&trOutlier"";";
   put "objRefNS:SiteLab = ""&SITEIDLabel"";";
   put "objRefNS:TestLab = ""&TESTLabel"";";
   put "objRefNS:DomainLab = ""&trDOMAIN"";";
   put "objRefNS:OverallByVarLabs = ""&OverallByVarLabs"";";
   put "objRefNS:nOverallByvar = &nOverallByvar;";
   put "objRefNS:hist = ""&ClientOutPath.&OutDataName_d..sas7bdat"";";
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   put "objRefNS:volcano=Function({},{default local},";
   put "   objRefNS:dvolcano = open(""&ClientOutPath.&OutDataName_v..sas7bdat"",invisible);";
   put "  Column(""&SITEIDLabel"") << Label(1);";
   put "  Column(""&TESTLabel"") << Label(1);";
   put "  Column(""&trSiteSampleSize"") << Label(1);";
   put "   VListBox(";
   put "   objRefNS:T1 = TextBox(""&trAnalysisPerformedUsing %sysfunc(lowcase(&TrOutlier)).""),";
   %if %length(&RawCutOff) %then %do;
      put "   objRefNS:T2 = TextBox(locale:fill(locale:get(""XLineDrawnatMinusLogpX_Text""), {locale:get(""FDR_Text""), ""%sysfunc(round(&RawCutOff,0.0001))""})||"" (alpha=&AlphaValue)""),"; 
   %end;
   %else %do;
      put "   objRefNS:T2 = TextBox(locale:fill(locale:get(""NoPvalsSigAtAlphaXAfterAppXAdj_Text""), {""&AlphaValue"", locale:get(""FDR_Text"")}),<<Set Wrap(1000)),"; 
   %end;
   put "   HListBox(";
   put "    objRefNS:Filter = objRefNS:dvolcano << Data Filter(<<Conditional(1),";
   put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&trDOMAIN"",""&TESTLabel"")),";
   put "       Mode( Show(1), Include(1) )";
   put "    ),";      
   put "   objRefNS:VolcPlot = Graph Builder(";
   put "    Auto Stretching(0),";
   put "    Automatic Recalc(1),";
   put "   Show Control Panel( 0 ),";
   put "   Variables(";
   put "    X( :&trDiffProps ),";
   put "    Y( :Name( ""&trNegLog10(&trFisherExact)"" ) ),";
   put "    Color( :&SITEIDLabel )";
   put "   ),";
   put "   Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
   put "   SendToReport(";
   put "    Dispatch(";
   put "       {},";
   put "       ""Graph Builder"",";
   put "       OutlineBox,";
   put "       {Set Title( locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""Outliers_Text"")}) )}";
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""&trDiffProps"",";
   put "       ScaleBox,";
   put "       {}";
   put "    ),";
   %if %length(&RawCutOff) %then %do;
      put "    Dispatch(";
      put "       {},";
      put "       ""&trNegLog10(&trFisherExact)"",";
      put "       ScaleBox,";
      put "       {Add Ref Line(";
      put "          &RawCutOff,";
      put "          Dashed,";
      put "          ""Red"",";
      put "          """",";
      put "          2";
      put "       )}";
      put "    ),";
   %end;
   put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
   put "    Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 6 )} )";
   put "   )";
   put "   )";
   put "));";
   put ");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "InsertInto(objRefNS:DataList,{{""objRefNS:dvolcano""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trVolcanoPlot""}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""Outliers_Text"")})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T1,objRefNS:T2,objRefNS:VolcPlot}});";

   %let domdatlist = ;
   %let firstdomdat = 0;
   %let i = 1;
   %do %while(%length(%scan(&findingsdomains,&i,' ')));
      %let temp = %scan(&findingsdomains,&i,' ');
      %let templab = %bquote(%scan(&findingsdomainslabel,&i,~));
      %if %sysfunc(indexw(&AvailDomains, &temp, ' ')) %then %do;
         %if ^&firstdomdat %then %do;
            %let domdatlist = ""objRefNS:df&temp"";
            %let firstdomdat = 1;
         %end;
         %else %let domdatlist = &domdatlist,""objRefNS:df&temp"";      
         put "objRefNS:f&temp=Function({},{default local},";
         put "   objRefNS:df&temp = open(""&ClientOutPath.&&OutDataName_&temp...sas7bdat"",invisible);";
         put "   column(""&trOutlier"") << Set Modeling Type( Ordinal );";
         %if %sysfunc(findw(%upcase(&&&temp.VARS), VISITNUM)) %then %do;
            put "   column(""&trVisitNum"") << Set Modeling Type( Ordinal );";
         %end;                
         put "   VListBox(";
         put "   If(nrows(objRefNS:df&temp) > 0,";
         put "   HListBox(";
         put "   objRefNS:Filter = objRefNS:df&temp << Data Filter(<<Conditional(1),";
         put "        Add Filter(";
         put "           Columns(:Name(""&COUNTRYLabel""), :Name(""&SITEIDlabel""), :Name(""&trTestName""), :Name(""&trOutlier""))"; 
         put "        ),";
         put "        Mode( Show(1), Include(1) )";
         put "   ),";       
         put "   objRefNS:Dist&temp = Distribution(";
         put "      Automatic Recalc(1),";
         put "      Nominal Distribution( Column( :&SUBJECTlabel ), Show Counts( 1 )  ),";
         put "      Nominal Distribution( Column( :&SITEIDlabel ), Show Counts( 1 )  ),";
         %if %sysfunc(findw(%upcase(&&&temp.VARS), VISITNUM)) %then %do;
            put "      Nominal Distribution( Column( :&trVisitNum), Show Counts( 1 )  ),";
         %end;       
         %let j = 1;
         %do %while(%length(%scan(&OverallByVarLabs,&j,~)));
            %let tempby = %scan(&OverallByVarLabs,&j,~);
            %let tempby2 = %scan(&OverallByVar,&j);
            %if %sysfunc(findw(%upcase(&&&temp.VARS), &temp.&tempby2)) %then %do;            
               put "      Nominal Distribution( Column( :&tempby ), Show Counts( 1 )  ),";
            %end;
            %let j = %eval(&j+1);
         %end;
         put "      Nominal Distribution( Column( :&trTestName ), Show Counts( 1 )  ),";
         put "      Nominal Distribution( Column( :&trOutlier ), Show Counts( 1 )  ),";
         put "      Nominal Distribution( Column( :&trOutlierType ), Show Counts( 1 )  ),";
         put "      SendToReport(";
         put "         Dispatch(";
         put "          {""Distributions"", ""&SUBJECTlabel""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";
         put "         Dispatch(";
         put "          {""Distributions"", ""&SITEIDlabel""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";
         %if %sysfunc(findw(%upcase(&&&temp.VARS), VISITNUM)) %then %do;
            put "         Dispatch(";
            put "          {""Distributions"", ""&trVisitNum""},";
            put "          ""Frequencies"",";
            put "          OutlineBox,";
            put "          {Close( 1 )}";
            put "       ),";
         %end;       
         %let j = 1;
         %do %while(%length(%scan(&OverallByVarLabs,&j,~)));
            %let tempby = %scan(&OverallByVarLabs,&j,~); 
            %let tempby2 = %scan(&OverallByVar,&j);
            %if %sysfunc(findw(%upcase(&&&temp.VARS), &temp.&tempby2)) %then %do;
               put "         Dispatch(";
               put "          {""Distributions"", ""&tempby""},";
               put "          ""Frequencies"",";
               put "          OutlineBox,";
               put "          {Close( 1 )}";
               put "       ),";
            %end;
            %let j = %eval(&j+1);
         %end;       
         put "         Dispatch(";
         put "          {""Distributions"", ""&trTestName""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";        
         put "         Dispatch(";
         put "          {""Distributions"", ""&trOutlier""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";        
         put "         Dispatch(";
         put "          {""Distributions"", ""&trOutlierType""},";
         put "          ""Frequencies"",";
         put "          OutlineBox,";
         put "          {Close( 1 )}";
         put "       ),";        
         put "      )";
         put "   ),";
         put "   );";
         put "   );";
         put "   );";
         put ");";
         put "TabToDisplay = ""&templab"";";
         put "if (TabToDisplay == ""ECG Test Results"", TabToDisplay = locale:get(""ECGTestResults_Text""););"; 
         put "if (TabToDisplay == ""Laboratory Test Results"", TabToDisplay = locale:get(""LaboratoryTestResults_Text""););"; 
         put "if (TabToDisplay == ""Vital Signs"", TabToDisplay = locale:get(""VitalSigns_Text""););";         
         put "InsertInto(objRefNS:TabOpenList,1);";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:df&temp""}});";
         put "InsertInto(objRefNS:DataNameList,{{""&temp &trDistribution""}});";
         put "InsertInto(objRefNS:TabButtonList, TabToDisplay||"" (&temp)"");";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:f&temp()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:Dist&temp}});";       
      %end;
      %let i = %eval(&i+1);      
   %end;       
   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
   put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   put "objRefNS:spdd =  Function({}, {},";
   put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";

   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{},{}};";
   put "objRefNS:ActionCodeList ={{},{}};";
   put "objRefNS:ActionDataTableList = {{},{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsfromVolcanothenClick_Text""))});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSites_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSite()));";      
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""OutlierBarChart_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:OutlierBar()));";
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
   
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsfromDomainTabsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:ppdd()));";    
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&domdatlist}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&domdatlist}});";   
   put "objRefNS:ActionSelectionCountList = {{1, 1},{1, 1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.DataSetHighlightRows.gif"",""&IconPath.BarGraph.gif""},{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif""}};";   
   put "objRefNS:ActionRequirementFlagList = {{0, 0},{3, 0}};";

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &OutliersScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend Outliers;
%Outliers;

************************************;
************************************;
************************************; 