/*
JMP Clinical Analytical Process

Name: Perfect Schedule
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   This process identifies dates on Weekends and Holidays.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName PerfectScheduleScript PerfectSchedule_OutData OutPath 
        ServerOutPath PopulationSet WhereStmt WhereForReport PopulationSet SiteCount SiteXact NMCSamps AlphaValue;

*ProcessBody;

%macro PerfectSchedule;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags","CalcStudyDay_SV"}}$
$INCD: {"SV"}$
$CMP: RSYS$ $UE: DM.SITEID and SV.VISITNUM$ 
$CMP: OSYS$ $UE: SV.VISIT$  $NOTES: To remove UNSCHEDULED visits, if available$ 
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$
$CMP: OOUT$ $UE: SV.VISIT$  $NOTES: Added to filter, if available.$
$CVS_STOP$
*/
*initialization and argument checking;

%let ProcessName = PerfectSchedule; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds =;
%OUTNAMETRUNC(outdat = PerfectSchedule_OutData, truncnum = 16);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);
%let SiteXact = %upcase(&SiteXact);
%let trPerfectScheduledAttendance = %nrbquote(%LocaleGet(PerfectScheduledAttendance_Text));

%if ^%sysfunc(findw(&domains, SV)) %then %do;
   %put [MS_S_0141];
   %put ERROR: The SV domain is not available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%CHECKPRES(domain = SV, var = VISIT);
%CHECKPRES(domain = SV, var = SVSTDY);
%CHECKPRES(domain = SV, var = SVSTDTC);

%if ^%length(&SVSTDY) %then %do;
   %if ^%sysfunc(findw(&SVVARS, SVSTDTC)) %then %do;
      %put [MS_S_0149];
      %put ERROR: The variable SVSTDTC is not available in the SV domain.;
      %let exiterror = 1;
      %goto exit;
   %end;

   %if ^%sysfunc(findw(%upcase(&DMVARS), RFSTDTC)) %then %do;
      %put [MS_S_0150];
      %put ERROR: RFSTDTC is not available in either DM or ADDM.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end;

%if ^%sysfunc(findw(&SVVARS, VISITNUM)) %then %do;
   %put [MS_S_0151];
   %put ERROR: The variable VISITNUM is not available in the SV domain.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA;

%GETSITELISTS(errflg = 1);
%if &exiterror %then %goto exit;

%LOADDATA(domlist = SV,mergesupp = 0,checkreview=0);

proc sort data = &in_SV out = sv(keep = usubjid visitnum stdy &visit);
   by usubjid visitnum;
run;

%if %length(&Visit) %then %do;
   %let _tid_ = %sysfunc(open(sv));
   %let vn = %sysfunc(varnum(&_tid_,visit));
   %let VisitLen = %sysfunc(varlength(&_tid_,&vn));
   %let _tid_ = %sysfunc(close(&_tid_));
   %let VISITLabel = Visit;
   %CheckLabel(sv,VISITLabel);
%end;
%let VISITNUMLabel = VISITNUM;
%CheckLabel(sv,VISITNUMLabel);

%let utrUnscheduled = %qupcase(%nrbquote(%LocaleGet(Unscheduled_Text)));

data sv;
   merge subject(in = ina) sv;
   by usubjid;
   if ina;
   drop SVSTDY errdtstdy endy errdtendy svdur svendtc log2_svdur svendy;
   if missing(stdy) then delete;
   %if %length(&visit) %then %do;
      if index(upcase(&visit), "UNSCHEDULED") or kindex(kupcase(&visit), "&utrUnscheduled") then delete;
   %end;
run;

proc sort data = sv;
   by visitnum &visit;
run;

proc sql noprint;
   select distinct visitnum 
      into :VisitNums separated by ' '
      from sv
      order by visitnum;
   %if %length(&visit) %then %do;
      select distinct Visit 
         into :Visits separated by '~'
         from sv
         order by visitnum;
   %end;
quit;

%if ^%length(&visit) %then %do;
   %let Visits = ;
   %let j = 1;
   %do %while(%length(%scan(&VisitNums,&j,' ')));
      %let tempvisnum = %scan(%upcase(&VisitNums,&j," "));
      %if &j = 1 %then %do;
         %let Visits = Visit &tempvisnum; 
      %end;
      %else %do;
         %let Visits = &Visits~Visit &tempvisnum;  
      %end;
      %let j = %eval(&j+1);
   %end;    
%end;

%let firstflag = 0;
%let i = 1;
%do %while(%length(%scan(&SiteList,&i,' ')));
   %let tempsite = %scan(%upcase(&SiteList),&i," ");
   %let tempsitesize = %scan(%upcase(&SiteSamp),&i," ");
   %let trSiteTS = %nrbquote(%LocaleFill(&trSiteX, &tempsite));
   %let trNotSiteTS = %nrbquote(%LocaleFill(&trNotSiteX, &tempsite));
   data temp;
      length Label $ 30;
      set sv;
      if siteid = "&tempsite" then Label = "&trSiteTS";
      else Label = "&trNotSiteTS";
     label Label = "&trLabel";
   run;

   proc freq data = temp;
      ods exclude all;
      %if &SiteXact = NO %then %do;
         tables label * stdy / cmh sparse scores = modridit out = stats(drop = PERCENT PCT_COL) outpct;
         output out = testvals(keep = visitnum &visit P_CMHRMS) CMHRMS;   
         %let pval = P_CMHRMS;
         %let pnew = PRowMean;
         %let plab0 = &trRawRowMeanScorePVal;
         %let plab = &trNegLog10(&trRawRowMeanScorePVal);
      %end;
      %else %do;     
         tables label * stdy / sparse scores = modridit out = stats(drop = PERCENT PCT_COL) outpct;
         exact MHCHI / MC n = &NMCSamps;
         ods output MHChiSqMC = testvals;
         %let pval = MCP_MHCHI;
         %let pnew = PXMHChi;
         %let plab0 = &trREMCMHPV;
         %let plab = &trNegLog10(&trREMCMHPV);
      %end;
      by visitnum &visit; 
   run;
   
   ods select all;
   
   %if &SiteXact = YES %then %do;
      data testvals(drop = name1 rename = (nvalue1 = MCP_MHCHI));
         set testvals(where = (Name1 = "MCP_MHCHI") keep = visitnum &visit Name1 nValue1);
      run;
   %end;
 
   %NObsVars(testvals);
 
   data stats;
      length siteid $ &SiteidLen;
      set stats(rename = (PCT_ROW = Percent));
      SITEID = "&tempsite";
      SITESAMP = &tempsitesize;
      label percent = "&trPercentofTotalVisits" stdy = "&trStudyDayofStartofVisit" siteid = "&SITEIDlabel" sitesamp = "&trSiteSampleSize";
   run;
 
   proc sort data = stats out = stats2;
      by visitnum &visit stdy label;
   run;
   
   data stats2;
      set stats2;
      by visitnum &visit stdy label;
      retain diff;
      if first.stdy then diff = .;
      if first.stdy and ^last.stdy and kindex(label, "&trNot") then diff = -percent;
      else if last.stdy and ^first.stdy and ^kindex(label, "&trNot") then diff = diff + percent;
      if last.stdy and diff > .;
      label diff = "&trMaxPercentDifference";
   run;
   
   data stats2;
      set stats2;
      diff2 = abs(diff);
   run;
   
   *** Take largest absolute difference, but in direction of single site if tied;
   proc sort data = stats2;
      by visitnum &visit diff2 diff; 
   run;
   
   data stats2(drop = diff2 count percent);
      set stats2;
      by visitnum &visit;
      if last.visitnum;
   run;

  %if &nobs %then %do;
      data stats2;
         merge stats2(in = ina) testvals(rename = (&pval = &pnew));
         by visitnum;
         if ina;
         if missing(&pnew) then &pnew = 1;
         Nlog10&pnew = -log10(&pnew);
         label &pnew = "&plab0" Nlog10&pnew = "&plab";
      run;

      data stats;
         merge stats2(keep = visitnum in = ina) stats;
         by visitnum;
         if ina;
      run;

     %if ^&firstflag %then %do;
         data summary_volcano;
            %if %length(&Visit) %then %do;
               length Visit $ &VisitLen;  
            %end;
            set stats2;
         run;
         data summary_visit;
            %if %length(&Visit) %then %do;
               length Visit $ &VisitLen;  
            %end;
            set stats;
         run;
         %let firstflag = 1;
      %end;
      %else %do;   
         proc append force base = summary_volcano data = stats2;
         run;
         proc append force base = summary_visit data = stats;
         run;     
      %end;
   %end;

   proc datasets nolist;
      delete temp stats stats2 testvals;
   quit;
   %let i = %eval(&i+1);
%end;

proc datasets nolist;
   delete sv;
quit; 

%let NoDiffFlag = 1;
%if %sysfunc(exist(summary_volcano)) %then %do;   
   %let NoDiffFlag = 0;  

   %FDRPVALUES(inpv = &pnew, nlog10p = Nlog10&pnew);

   %let PerfectSchedule_OutData_v =;      
   %PrepOutDS(PerfectSchedule_OutData_v,OutDataName_v, &Outname.PSched, _vol);
   %let includeds = &includeds &OutDataName_v..sas7bdat,;

   data &PerfectSchedule_OutData_v;
      merge summary_volcano(in = ina) country(in = inb);
      by siteid;
      if ina and inb;
   run;

   %let PerfectSchedule_OutData_d = ;     
   %PrepOutDS(PerfectSchedule_OutData_d,OutDataName_d, &Outname.PSched, _hist);
   %let includeds = &includeds &OutDataName_d..sas7bdat,;

   data &PerfectSchedule_OutData_d;
      set summary_visit;
   run;  

   %let SVSTDYLabel = STDY;
   %CheckLabel(summary_visit,SVSTDYLabel);

   proc datasets nolist;
      delete summary_visit summary_volcano country subject;
   quit;
%end;

* create JSL file;
%PrepOutFile(PerfectScheduleScript,PerfectScheduleScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &PerfectScheduleScript;
%let FullAPName=%str(Perfect Scheduled Attendance);

* put JSL statements;
data _null_;
   file "&PerfectScheduleScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:NoteDomain = ""SV"";";      

   put "objRefNS:OverallByVar = ""&OverallByVar"";";
   put "objRefNS:SiteLab = ""&SITEIDLabel"";";
   put "objRefNS:VisitNumLab = ""&VISITNUMLabel"";";
   put "objRefNS:SVSTDYLab = ""&SVSTDYLabel"";";
   %if %length(&visit) %then %do;
      put "objRefNS:VisitLab = ""&VISITLabel"";";
   %end;
   %else %do;
      put "objRefNS:VisitLab = """";";
   %end;
   %if ^&NoDiffFlag %then %do;
      put "objRefNS:hist = ""&ClientOutPath.&OutDataName_d..sas7bdat"";";
   %end;
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   %if ^&NoDiffFlag  %then %do;
      put "objRefNS:dvolcano = open(""&ClientOutPath.&OutDataName_v..sas7bdat"",invisible);";
   %end;
   put "objRefNS:volcano=Function({},{default local},";
   %if ^&NoDiffFlag  %then %do;
      put "  Column(""&SITEIDLabel"") << Label(1);";
      put "  Column(""&VISITNUMLabel"") << Label(1);";
      put "  Column(""&trSiteSampleSize"") << Label(1);";
      %if %length(&visit) %then %do;
         put "  Column(""&VISITLabel"") << Label(1);";
      %end;
      put "   VListBox(";
      %if %length(&RawCutOff) %then %do;
         put "   TextBox(locale:fill(locale:get(""XLineDrawnatMinusLogpX_Text""), {locale:get(""FDR_Text""), ""%sysfunc(round(&RawCutOff,0.0001))""})||"" (alpha=&AlphaValue)""),";
      %end;
      %else %do;
         put "   TextBox(locale:fill(locale:get(""NoPvalsSigAtAlphaXAfterAppXAdj_Text""), {""&AlphaValue"", locale:get(""FDR_Text"")}),<<Set Wrap(1000)),"; 
      %end;   
      %if &SiteXact = YES %then %do;
         put "   TextBox(locale:fill(locale:get(""MontecarlopvaluesarebasedonXsamples_Text""), {""&NMCSamps""})),";
         put "   TextBox(""""),";
      %end;   
      put "   HListBox(";
      put "   objRefNS:VolcPlot = Graph Builder(";
      put "    Auto Stretching(0),";
      put "    Automatic Recalc(1),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "    X( :&trMaxPercentDifference ),";
      put "    Y( :Name( ""&plab"" ) ),";
      put "    Color( :&SITEIDLabel )";
      put "   ),";
      put "   Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       OutlineBox,";
      put "       {Set Title( locale:fill(locale:get(""XVolcanoPlot_Text""), {locale:get(""&ProcessName._Label"")}) )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trMaxPercentDifference"",";
      put "       ScaleBox,";
      put "       {}";
      put "    ),";
      %if %length(&RawCutOff) %then %do;
         put "    Dispatch(";
         put "       {},";
         put "       ""&plab"",";
         put "       ScaleBox,";
         put "       {Add Ref Line(";
         put "          &RawCutOff,";
         put "          Dashed,";
         put "          ""Red"",";
         put "          """",";
         put "          2";
         put "       )}";
         put "    ),";
      %end;
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 6 )} )";
      put "   )";
      put "   ),";
      put "));";
   %end;
   %else %do;
      put "   VListBox(";
      put "       TextBox(locale:get(""NoDifferencesWereIdentifiedBetweenSuspect_Text""),<<Set Wrap(1000),<< Font Color(5));";
      put "   );";
   %end;
   put ");";

   %if ^&NoDiffFlag  %then %do;
      put "objRefNS:CustomPanel = Expr(objRefNS:Filter = objRefNS:dvolcano << Data Filter(<<Conditional(1),";
      %if %length(&visit) %then %do;
         put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&VISITLabel"")),";
      %end;
      %else %do;
         put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&VISITNUMLabel"")),";
      %end;
      put "       Mode( Show(1), Include(1) )";
      put "    ));";
   %end;

   put "InsertInto(objRefNS:TabOpenList,1);";
   %if ^&NoDiffFlag  %then %do;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dvolcano""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trPerfectScheduledAttendance""}});";
   %end;
   %else %do;
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
   %end;
   put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""PerfectScheduledAttendance_Text"")})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:VolcPlot}});";

   %if ^&NoDiffFlag  %then %do;
      put "objRefNS:ActionButtonDesc = {};";
      put "objRefNS:ActionButtonList={{}};";
      put "objRefNS:ActionCodeList ={{}};";
      put "objRefNS:ActionDataTableList = {{}};";
   
      put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
      put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsthenClick_Text""))});";  
      put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSites_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSite()));";    
      put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";        
      put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowVisitBar_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:VisitBar()));";    
      put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";  
      put "objRefNS:ActionSelectionCountList = {{1,1}};";
      put "objRefNS:ActionIconList = {{""&IconPath.DataSetHighlightRows.gif"", ""&IconPath.BarGraph.gif""}};";
      put "objRefNS:ActionRequirementFlagList = {{0, 0}};";
   %end;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &PerfectScheduleScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend PerfectSchedule;
%PerfectSchedule;

************************************;
************************************;
************************************; 