/*
JMP Clinical Analytical Process

Name: Required Variable Check
Authors: Geoff Mann and Russ Wolfinger, SAS Institute Inc.

Description: 
	This process checks SAS data sets in specified SDTM and ADaM folders for all variables required
	in various JMP Clinical analytical processes.  The required variables are a subset of those from
	complete CDISC standards. Output is a table of results indicating which 
	variables are not present or have mismatching types or labels.  Note labels are not required 
	but are recommended for consistency with CDISC standards.
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath OutPath ServerOutPath MacroPath OutName
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL
RequiredVariable_OutData RequiredVariableScript chkdom_in_adsl PDFRTF;

*ProcessBody;

%macro RequiredVariableCheck;

*initialization and argument checking;
%let ProcessName = RequiredVariableCheck; 

%let PDFRTF = %LOWCASE(&PDFRTF);

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%let trYes = %nrbquote(%LocaleGet(Yes_Text));
%let trNoneAvailable = %nrbquote(%LocaleGet(NoneAvailable_Text));

%let reqvarsC = %jg_getPath(/user/clinical/metadata);
%let reqvars = %jg_getPath(/user/clinical/metadata);

libname reqvars "&reqvars";

*** This bit of code to add domains to the list that are required for many processes;
%let pastdomains = AE CM CO DM DS EG LB EX MH VS SV;
%let leftovers = ;
%let i = 1;
%do %while(%length(%scan(&pastdomains,&i,' ')));
	%let temp = %scan(%upcase(&pastdomains),&i," ");
	%if ^%sysfunc(findw(&domains, &temp)) %then %do;
	   %let domains = &domains &temp;
	   %let leftovers = &leftovers &temp;
	   %if %index(CM EX, &temp) %then %let interventionsdomains = &interventionsdomains &temp;
	   %if %index(AE DS MH, &temp) %then %let eventsdomains = &eventsdomains &temp;
	   %if %index(EG LB VS, &temp) %then %let findingsdomains = &findingsdomains &temp;
	%end;
  %let i = %eval(&i+1);
%end; 

%ALPH(varlist = domains);
%ALPH(varlist = interventionsdomains);
%ALPH(varlist = eventsdomains);
%ALPH(varlist = findingsdomains);

%let includeds =;
%if %length(&RequiredVariable_OutData)>17 %then %let RequiredVariable_OutData = %substr(&RequiredVariable_OutData,1,17);
%if %length(&RequiredVariable_OutData) %then %let Outname = &RequiredVariable_OutData._;
%else %let Outname = &RequiredVariable_OutData;
%let RequiredVariable_OutData =;
%let RequiredVariable_OutData_d =;	   
%PrepOutDS(RequiredVariable_OutData_d,OutDataName_d, &Outname.ReqVar, _availdat);
%let includeds = &includeds &OutDataName_d..sas7bdat,;

data &RequiredVariable_OutData_d;
   length Dataset $ 15 Available $ 30;
	%let i = 1;
	%do %while(%length(%scan(&domains, &i, " ")));
      %let domaintmp = %scan(&domains, &i);
      %if ^%sysfunc(findw(&leftovers, &domaintmp)) %then %do;
      	Dataset = "%upcase(%scan(&domains, &i))"; Available = "%upcase(%scan(&&in_&domaintmp,2,.))"; output;   
      %end;
      %else %do;
      	Dataset = "%upcase(%scan(&domains, &i))"; Available = symget('trNoneAvailable'); output;   
      %end;
      %let i = %eval(&i + 1);   
   %end;
   
   %if %length(&adslpresent) %then %do;
      Dataset = "ADSL"; Available = "%upcase(%scan(&in_adsl,2,.))"; output;
   %end;
   %else %do;
      Dataset = "ADSL"; Available = "%upcase(%scan(&in_dm,2,.))"; output;
   %end;
run;

%let RequiredVariable_OutData_r =;	   
%PrepOutDS(RequiredVariable_OutData_r,OutDataName_r, &Outname.ReqVar, _reqvar);
%let includeds = &includeds &OutDataName_r..sas7bdat,;

%if %sysfunc(fileexist(%jg_getPath(/user/clinical/metadata, study_report_vars.sas7bdat))) %then %do;
    data &RequiredVariable_OutData_r;
       set reqvars.study_report_vars;
       where upcase(component_required) = "%upcase(Yes)" and UE_result = 0;
    run;
%end;
   
%if &syserr > 4 %then %goto exit;

ods listing close;
ods escapechar = "^";

%let trRequiredDomainReport = %nrbquote(%LocaleGet(RequiredDomainReportNS_Text));

%if &PDFRTF = pdf %then %do;
	%if &Server = Local %then %do;
		ods &PDFRTF file = "&ClientOutPath.&trRequiredDomainReport..&PDFRTF" notoc style = statdoc;
	%end;
	%else %if &Server ne Local %then %do;
		ods &PDFRTF file = "&ServerOutPath.&trRequiredDomainReport..&PDFRTF" notoc style = statdoc;
	%end;
%end;
%else %do;
	%if &Server = Local %then %do;
		ods &PDFRTF file = "&ClientOutPath.&trRequiredDomainReport..&PDFRTF" style = statdoc;
	%end;
	%else %if &Server ne Local %then %do;
		ods &PDFRTF file = "&ServerOutPath.&trRequiredDomainReport..&PDFRTF" style = statdoc;
	%end;
%end;

options nodate nonumber orientation = landscape;

%let trReqVarReport = %nrbquote(%LocaleGet(RequiredDomainReport_Text));
%let trDSJMPCUtil = %nrbquote(%LocaleGet(SDTMslashADaMDataSetsJMPCUtil_Text));
title1 "&studyname" justify = right;
title2 "&trReqVarReport";
title3 "&trDSJMPCUtil";

%let trDomSF = %nrbquote(%LocaleGet(DomainsJMPClinicalSearchesFor_Text));
%let trDataUtil = %nrbquote(%LocaleGet(DataSetJMPClinicalWillUtilize_Text));
proc report nowindows missing nofs colwidth = 15 style = {frame = void rules=none} data = &RequiredVariable_OutData_d;
	column dataset available;
	define dataset / left width = 15 "&trDomSF";
	define available / left width = 15 "&trDataUtil";
run;

%let trNote1 = %nrbquote(%LocaleGet(NoteADaMOfSDTMDomTakePrecJMPC_Text));
%let trNote2 = %nrbquote(%LocaleGet(NoteShouldADSLUnavailDMDomUsed_Text)); 
ods &PDFRTF text = "^S={font=('Arial')}";
ods &PDFRTF text = "^S={font=('Arial')} &trNote1";
ods &PDFRTF text = "^S={font=('Arial')} &trNote2";
%let trNote3 = %nrbquote(%LocaleGet(NoteTheFollowUnavailDomsSummReport_Text));
%if %length(&leftovers) %then %do;
	ods &PDFRTF text = "^S={font=('Arial')} &trNote3 ^S={FOREGROUND=blue font=('Arial')}&leftovers.";
%end;

%let trPMNRC = %nrbquote(%LocaleGet(ProcessorOptionsUnavailable_Text));
%let trObs = %nrbquote(%LocaleGet(Obs_Text));
%let trReportName = %nrbquote(%LocaleGet(ReportName_Text));
%let trDomain = %nrbquote(%LocaleGet(Domain_Text));
%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trComponentIndex = %nrbquote(%LocaleGet(ComponentIndex_Text));
%let trUsageExpressionMissing = %nrbquote(%LocaleGet(UsageExpressionMissing_Text));
%let trNotes = %nrbquote(%LocaleGet(Notes_Text));
%let trand_ss = %nrstr( )%nrbquote(%LocaleGet(and_Text))%nrstr( );
%let tror_ss = %nrstr( )%nrbquote(%LocaleGet(or_Text))%nrstr( );
title3 "&trPMNRC";
footnote;

%if %sysfunc(fileexist(%jg_getPath(/user/clinical/metadata, study_report_vars.sas7bdat))) %then %do;
    %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) ^= EN %then %do;
	data &RequiredVariable_OutData_r;
		set &RequiredVariable_OutData_r;
		call symput('TranslateThis', Report_Name);
		Report_Name = resolve('%LocaleEZX(&TranslateThis, EIA=1, CI=0)');
		UE_Miss = tranwrd(UE_Miss, " and ", "&trand_ss");
		UE_Miss = tranwrd(UE_Miss, " or ", "&tror_ss");
		label var = "&trObs" Report_Name = "&trReportName" Domain = "&trDomain" Variable = "&trVariable"
		  Component_Index = "&trComponentIndex" UE_Miss = "&trUsageExpressionMissing" notes = "&trNotes";
	run;
    %end;
%end;

%if %sysfunc(fileexist(%jg_getPath(/user/clinical/metadata, study_report_vars.sas7bdat))) %then %do;
    proc print label data = &RequiredVariable_OutData_r;
	var Report_Name Domain Variable Component_Index UE_Miss notes;
    run;
%end;

%let trNote4 = %nrbquote(%LocaleGet(NoteTheFollowUnavailDomsSummReport_Text));
ods &PDFRTF text = "^S={font=('Arial')}";
%if %length(&leftovers) %then %do;
	ods &PDFRTF text = "^S={font=('Arial')} &trNote4 ^S={FOREGROUND=blue font=('Arial')} &leftovers";
%end;
%let trNote7 = %nrbquote(%LocaleGet(NoteDatasetofavailableSDTMADaMDataSets_Text));
ods &PDFRTF text = "^S={font=('Arial')} &trNote7 ^S={FOREGROUND=blue} (&ClientOutPath&OutDataName_d..sas7bdat)";

ods &PDFRTF close;

%if &PDFRTF = rtf %then %do;
	%PackageInsertFile(RTF file from &ProcessName, &OutPath, &trRequiredDomainReport..rtf, , rtf, yes);
%end;
%else %do;
	%PackageInsertFile(PDF file from &ProcessName, &OutPath, &trRequiredDomainReport..pdf, , pdf, yes);
%end;

* create JSL file;
%PrepOutFile(RequiredVariableScript,RequiredVariableScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &RequiredVariableScript;
%let FullAPName = %str(Check Required Variables);

data _null_;
   file "&RequiredVariableScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "objRefNS = newnamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:FullAPName = ""&FullAPName"";";
	put "objRefNS:StudyName = ""&StudyName"";";
/*	%if &Server ne Local %then %do;
		put "rc = jg:SASGetFile2(""&ServerOutPath.RequiredVariablesReport.pdf"",""&ClientOutPath.RequiredVariablesReport.pdf"");";
		put "rc = jg:SASGetFile2(""&ServerOutPath.available_datasets.sas7bdat"",""&ClientOutPath.available_datasets.sas7bdat"");";
		put "rc = jg:SASGetFile2(""&ServerOutPath.process_variables.sas7bdat"",""&ClientOutPath.process_variables.sas7bdat"");";
   %end; 

	put "Try(open(""&ClientOutPath.RequiredVariablesReport.&PDFRTF"")";
	put "   , //else";
	put "   New Window(locale:fill(locale:get(""XMessage_Text""), {locale:get(""JMPClinical_Text"")}), Show Menu( 0 ), Show Toolbars( 0 ),";
	put "	     tb = TextBox(locale:fill(locale:get(""RequiredVariablesReportdotXCannotBeOpenedMsg_Text""), {""&PDFRTF""})),";
	put "	     tb << Font Color(3),";
	put "        jg:messageHelp(""RequiredVariablesReportdotXCannotBeOpenedMsg_Text"")";
	put "   );";	
	put ");";
   */
	put "Names Default to Here(1);";
	put "objRefNS:DataList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	put "objRefNS:CreateReportList = {};";
        put "objRefNS:ShowDataTables = 0;";
        put "objRefNS:ShowCreateReport = 0;";
        put "objRefNS:ShowNotes = 0;";	
	/*put "Include(""$CLINICAL_HOME\JSLFiles\Required Variables Expressions.jsl"");";
	put "jg:CVSInit;";
	put "objRefNS:reqvars = open(CVSLocalizeVarDS(""&reqvarsC.study_report_vars.sas7bdat""),invisible);";	
	put "Eval(Parse(EvalInsert(""\[objRefNS:reqvars << New Column(locale:get(""ComponentMet_Text""), Character,";
	put "   Formula(If(:^locale:get(""UsageExpressionResult_Text"")^ == 1,";
	put "     ""^locale:get(""Yes_Text"")^"", ""^locale:get(""No_Text"")^"")));]\"")));";
	put "   Column(locale:get(""UsageExpressionResult_Text"")) << Set Modeling Type( Ordinal );";
	
	put "objRefNS:DistVFunc=Function({},{default local},";
	put "   VlistBox(";
	put "      objRefNS:VariablesDistExpr;";
	put "   );";
	put ");";
	
	put "InsertInto(objRefNS:TabOpenList,1);";
	put "InsertInto(objRefNS:DataList,{{""objRefNS:reqvars""}});";
	put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""VariableSummary_Text"")));";
	put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:DistVFunc()));";
	put "InsertInto(objRefNS:TabAfterCodeList, "" "");";

	
	put "objRefNS:DistCFunc=Function({},{default local},";
	put "   VlistBox(";
	put "      objRefNS:ComponentsDistExpr;";
	put "   );";
	put ");";

   put "InsertInto(objRefNS:TabOpenList,1);";
	put "InsertInto(objRefNS:DataList,{{""objRefNS:reqvars""}});";
	put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""UsageComponents_Text"")));";
	put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:DistCFunc()));";
	put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
	
	put "objRefNS:OpenReq = Function({}, {},";
	put "	 Names Default to Here(1);";
	put "  jg:CVSInit;";
	put "  rc = GenerateStudyStarterFilter( ""&StudyName"", 0, 1 );";
	put "  Try(";
	put "     Open(""&reqvarsC.StudyVariableReport.pdf"");";
	put "     , //else";
	put "     Try(";
	put "         Open(""&reqvarsC.StudyVariableReport.txt"");";
	put "     );";
	put "  );";
	put ");";*/
	/* Kemicl  use new variable detected in TabbedReport to append Filter/Custom (nonbutton) code to DrillDowns outline box */
	/*put "Eval(Parse(EvalInsert(""\[";
	put "objRefNS:CustomPanel = Expr(";
	put "	objRefNS:Filter = objRefNS:reqvars << Data Filter(	Conditional,";
	put "		Mode( Select( 0 ), Show( 1 ), Include( 1 ) ),";
	put "			Add Filter(";
	put "				columns(";
	put "					:^locale:get(""ComponentType_Text"")^,";
	put "					:^locale:get(""ComponentRequired_Text"")^,";	
	put "					:^locale:get(""ComponentMet_Text"")^,";
	put "					:^locale:get(""VariableRequired_Text"")^,";
	put "					:^locale:get(""VariableExists_Text"")^,";
	put "					:^locale:get(""VariableUsed_Text"")^";
	put "				),";
	put "				Display( :^locale:get(""VariableRequired_Text"")^, Size( 201, 80 ), List Display ),";
	put "				Display( :^locale:get(""ComponentType_Text"")^, Size( 200, 61 ), List Display )";
	put "			)";
	put "		);";
	put ");";
	put "]\"")));";
	
	put "objRefNS:ActionButtonDesc = {expr(locale:get(""ClicktoView_Text""))};";
	put "objRefNS:ActionButtonList= {{expr(locale:get(""StudyVariableReport_Text""))}};";
	put "objRefNS:ActionCodeList= {{objRefNS:OpenReq()}};";*/	
	put "objRefNS:fbox=Function({},{default local},";
	put "   Vlistbox(";
	put "      objRefNS:Button=ButtonBox(locale:fill(locale:get(""OpenX_Text""), {""&trRequiredDomainReport..&PDFRTF""}),";
	put "          Open(""&ClientOutPath.&trRequiredDomainReport..&PDFRTF"");";
	put "      ),";
	put "      objRefNS:Button << Underline Style(1);";
	put "      if (!jg:CVSOptions[""MASTER""],";
	put "          if (!File Exists(jg:getPath(""/user/clinical/metadata"", ""study_report_vars.sas7bdat"")),";
	put "              TextBox(""%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(NoteCVSDisabledSoXPortionRepCNBC_Text)), %nrbquote(&trPMNRC)))"",<<Set Wrap(900),<< Font Color(6))";
	put "              ,";
	put "              TextBox(""%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(WarningCVSDisabledSoXPortionRepPrevAccNotG_Text)), %nrbquote(&trPMNRC)))"",<<Set Wrap(900),<< Font Color(6))";
	put "          )";
	put "      ),";	
	put "   )";
        put ");";
        put "InsertInto(objRefNS:DataList,{{""  ""}});";
        put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""RequiredDomainReport_Text"")));";
        put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
        put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
        put "InsertInto(objRefNS:TabOpenList,1);";        
	
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &RequiredVariableScriptName,"&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend RequiredVariableCheck;

* run the macro;
%RequiredVariableCheck;