/*
JMP Clinical Analytical Process

Name: Summary Statistics
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   This process calculates summary statistics within each findings domain and identifies unusual results at
   study sites.
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName SumStatScript SumStat_OutData OutPath 
        ServerOutPath PopulationSet WhereStmt WhereForReport SiteCount ResType FindTests SumStatDQF ByVisit RemoveUnsch ResType AllFind DQFByVars AlphaValue;

*ProcessBody;

%macro SummaryStatistics;

/*
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and (FINDINGS.xxSTRESN or FINDINGS.xxORRES) and FINDINGS.xxTEST and FINDINGS.xxTESTCD and FINDINGS.USUBJID$ $NOTES: At least two sites are needed.$
$CMP: CPV.{"Generate summary statistics by visit","Yes"}$ $UE: FINDINGS.VISITNUM$
$CMP: CPV.{"Remove unscheduled visits","Yes"}$ $UE: FINDINGS.VISIT$
$CMP: OSYS$ $UE: DM.ARM$ $NOTES: If ARM is missing, it will be created and filled as Blinded Treatment.$
$CMP: OSYS$ $UE: FINDINGS.xxLOC $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxMETHOD $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxPOS $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxSPEC $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxTPT $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxCAT $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: OSYS$ $UE: FINDINGS.xxSCAT $ $NOTES: Results generated within patient by subgroups using this variable, if available$
$CMP: CPV.{"Analyze:","Numeric results in standard units"}$ $UE: FINDINGS.xxSTRESN$ $NOTES: Only domains with these criteria are used$
$CMP: CPV.{"Analyze:","Results in original units"}$ $UE: FINDINGS.xxORRES$ $NOTES: Only domains with these criteria are used$
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$ 
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = SummaryStatistics; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds = ;
%OUTNAMETRUNC(outdat = SumStat_OutData, truncnum = 17);
%DQFTRANS;

%let WhereForReport=%nrbquote(&WhereStmt);

%let trSummaryStatistics = %nrbquote(%LocaleGet(SummaryStatistics_Text));

%let availabledomains =;
%let domainlist = &findingsdomains;

%if ^%length(&findingsdomains) %then %do;
   %put [MS_S_0057];
   %put ERROR: No Findings domains are available for the current study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
   %put [MS_S_0052];
   %put ERROR: ARM is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA(varlist = arm);
%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);

%GETSITELISTS(errflg = 1);
%if &exiterror %then %goto exit;

%let got1 = 0;
%if %upcase(&ByVisit) = YES %then %do;
   %DOMBYLISTS(inclvisit = 1, incllabs = 0);
%end;
%else %do;
   %DOMBYLISTS(inclvisit = 0, incllabs = 0);
%end;
%if ^&got1 and %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %do;
   %put [MS_S_0077];
   %put ERROR: No findings domains have the xxSTRESN variable. This variable is required to compute summary statistics.;
   %let exiterror = 1;
   %goto exit;
%end;
%else %if ^&got1 and %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;
   %put ERROR: No findings domains have the xxORRES variable. This variable is required to compute summary statistics.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %upcase(&AllFind) = NO %then %do;
   %PARSEFINDINGS(dlist = &domainlist, flist = &FindTests);
%end;

%let TestLen = 0;
%let CatLen = 0;
%let ScatLen = 0;
%let LocLen = 0;
%let MethodLen = 0;
%let PosLen = 0;
%let SpecLen = 0;
%let TptLen = 0;
%let i = 1;
%let OverallByVar =;
%let OverallByVarLabs =;
%let nOverallByvar = 0;

%if %upcase(&SumStatDQF) = MEAN %then %do;
   %let trXAxis = &trStdDiffMean;
%end;
%else %if %upcase(&SumStatDQF) = VARIANCE %then %do;
   %let trXAxis = &trlog2(&trVarRatio);
%end;
%else %if %upcase(&SumStatDQF) = SKEWNESS %then %do;
   %let trXAxis = &trSkewRatio;
%end;
%else %if %upcase(&SumStatDQF) = KURTOSIS %then %do;
   %let trXAxis = &trKurtRatio;
%end; 

%let i = 1;
%do %while(%length(%scan(&domainlist,&i,' ')));
   %let temp = %scan(&domainlist,&i,' ');
   %let tind = %sysfunc(left(%sysfunc(countw(%sysfunc(compbl("&&&var_&temp")), " "))));
   
   %CHECKPRES(domain = &temp, var = &temp.DTC);
   %CHECKPRES(domain = &temp, var = VISITNUM);
   %CHECKPRES(domain = &temp, var = VISIT);
   
   %if %upcase(&AllFind) = NO %then %do;
      %let ifcond = and %length(&&&temp.findlist);
   %end;
   %else %do;
      %let ifcond =;
   %end;  
   
   %if %sysfunc(indexw(&&&temp.VARS, &&&var_&temp, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TESTCD, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TEST, ' ')) &ifcond %then %do;   
      %if %sysfunc(findw(%upcase(&&&temp.VARS), &temp.SEQ)) %then %do;
         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
         data &temp(keep = usubjid &temp.seq &&&temp.DTC &VISIT &VISITNUM &temp.test &temp.testcd &&&byvar_&temp &&&var_&temp);
            set &&in_&temp;
            %if %upcase(&AllFind) = NO %then %do;
               where &temp.testcd in ( %unquote(&&&temp.findlist) ) and not missing (&&&var_&temp);
            %end;
            %else %do;
               where not missing (&&&var_&temp);
            %end;                          
         run;
      %end;
      %else %do;
         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
         data &temp(keep = usubjid &&&temp.DTC &VISIT &VISITNUM &temp.test &temp.testcd &&&byvar_&temp &&&var_&temp);
            set &&in_&temp;
            %if %upcase(&AllFind) = NO %then %do;
               where &temp.testcd in ( %unquote(&&&temp.findlist) ) and not missing (&&&var_&temp);
            %end;
            %else %do;
               where not missing (&&&var_&temp);
            %end;                          
         run;
         
         data &temp;
            set &temp;
            by usubjid;
            retain &temp.seq;
            if first.usubjid then &temp.seq = 1;
            else &temp.seq = &temp.seq + 1;           
         run;
      %end;
      
      %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;                  
         %let ORRESLabel = &&&var_&temp;
         %CheckLabel(&temp,ORRESLabel);
         
         data &temp(where = (not missing (&&&var_&temp)));
            set &temp(rename = (&&&var_&temp = cval));
            &&&var_&temp = input(cval,best12.);
            drop cval;
            label &&&var_&temp = "&ORRESLabel";
         run;     
      %end;      
      
      data &temp;
         set &temp;
         if &temp.testcd = "" or &temp.test = "" then delete;     
         %if %upcase(&RemoveUnsch) = YES %then %do;
            *** Delete unscheduled records ***;
            %if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
               data &temp;
                  set &temp;
                  if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
               run;
            %end;
         %end;       
      run;
      
      %NObsVars(&temp);
      %if &nobs %then %let availabledomains = &availabledomains &temp;

      %let _tid_ = %sysfunc(open(&temp));
      %let vn = %sysfunc(varnum(&_tid_,&temp.test));
      %let TestLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.cat));
      %let CatLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.scat));
      %let ScatLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.loc));
      %let LocLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.method));
      %let MethodLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.pos));
      %let PosLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.spec));
      %let SpecLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,&temp.tpt));
      %let TptLentmp = %sysfunc(varlength(&_tid_,&vn));
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %if %eval(&TestLentmp>&TestLen) %then %let TestLen = &TestLentmp; 
      %if %eval(&CatLentmp>&CatLen) %then %let CatLen = &CatLentmp; 
      %if %eval(&ScatLentmp>&ScatLen) %then %let ScatLen = &ScatLentmp; 
      %if %eval(&LocLentmp>&LocLen) %then %let LocLen = &LocLentmp; 
      %if %eval(&MethodLentmp>&MethodLen) %then %let MethodLen = &MethodLentmp;
      %if %eval(&PosLentmp>&PosLen) %then %let PosLen = &PosLentmp; 
      %if %eval(&SpecLentmp>&SpecLen) %then %let SpecLen = &SpecLentmp;
      %if %eval(&TptLentmp>&TptLen) %then %let TptLen = &TptLentmp;    
      
      data &temp;
         merge subject(in=ina keep = usubjid siteid) &temp(in=inb);
         by usubjid;
         if ina and inb;        
      run;

      proc sort data = &temp out = &temp;
         by siteid;
      run;

      %let firstflag = 0;
      %let j = 1;
      %do %while(%length(%scan(&SiteList,&j,' ')));
         %let tempsite = %scan(%upcase(&SiteList),&j," ");       
         %let trSiteTS = %nrbquote(%LocaleFill(&trSiteX, f1=%nrbquote(&tempsite)));
         %let trNotSiteTS = %nrbquote(%LocaleFill(&trNotSiteX, f1=%nrbquote(&tempsite)));

         data temp;
            length siteid $ &SiteidLen Label $ 30 cLabel $ 2;
            set &temp;
            if siteid = "&tempsite" then do;
               Label = "&trSiteTS";
               cLabel = "S";
            end;
            else do;
               Label = "&trNotSiteTS";
               cLabel = "NS";
            end;
            siteid = "&tempsite";
            label Label = "&trLabel";
         run;

         proc sort data = temp;
            by label;
         run;
         
         proc means noprint nway missing data = temp;
            class &&&byvar_&temp &temp.testcd &temp.test label clabel siteid;
            var &&&var_&temp;
            %if %upcase(&SumStatDQF) = MEAN or %upcase(&SumStatDQF) = VARIANCE %then %do;
               output out = stats(drop = _TYPE_ _FREQ_) n = n mean = mean std = std;
            %end;
            %else %do;
               output out = stats(drop = _TYPE_ _FREQ_) n = n &SumStatDQF = &SumStatDQF;
            %end;
         run;
         
         data stats(drop = clabel label &temp.testcd);
            %if %upcase(&SumStatDQF) = MEAN or %upcase(&SumStatDQF) = VARIANCE %then %do;
               merge stats(where = (clabel = "S"))
                     stats(where = (clabel = "NS") rename = (n = n_ns mean = mean_ns std = std_ns));
               by &&&byvar_&temp &temp.testcd &temp.test siteid;
               label n = "&trN &trSite" mean = "&trMean &trSite" std = "&trSTD &trSite";
               label n_ns = "&trN &trNotSite" mean_ns = "&trMean &trNotSite" std_ns = "&trSTD &trNotSite";
            %end;
            %else %do;
               merge stats(where = (clabel = "S"))
                     stats(where = (clabel = "NS") rename = (n = n_ns &SumStatDQF = &SumStatDQF._ns));
               by &&&byvar_&temp &temp.testcd &temp.test siteid;
               label n = "&trN &trSite" &SumStatDQF = "&&tr&SumStatDQF &trSite";
               label n_ns = "&trN &trNotSite" &SumStatDQF._ns = "&&tr&SumStatDQF &trNotSite";   
            %end;
            %if %upcase(&SumStatDQF) = MEAN %then %do;
               t = (mean - mean_ns) / sqrt(std**2/n + std_ns**2/n_ns);
               df = (std**2/n + std_ns**2/n_ns)**2 / ((std**2/n)**2/(n-1) + (std_ns**2/n_ns)**2/(n_ns-1));
               pval = 2*probt(-abs(t),df);
            %end;
            %else %if %upcase(&SumStatDQF) = VARIANCE %then %do;
               f = max(std**2,std_ns**2) / min(std**2,std_ns**2);
               if std**2 >= std_ns**2 and f > . then do;
                  df1 = n-1;
                  df2 = n_ns-1;
               end;
               else if  std**2 < std_ns**2 and f > . then do;
                  df1 = n_ns-1;
                  df2 = n-1;
               end;
               pval = 2*(1 - probf(f,df1,df2));
            %end;
         run;
         
         data stats;
            length domain $ 8;
            set stats(rename = (&temp.test = test));
            Domain = "&temp";
            label Test = "&trTestName" Domain = "&trDOMAIN";
         run;        

         %if %length(&&&byvar_&temp) %then %do;
            data stats;
               set stats;
               %let k = 1;             
               %do %while(%length(%scan(&&&byvar_&temp,&k,' ')));
                  %let tempvar = %scan(&&&byvar_&temp,&k,' ');
                  %if &tempvar ne VISITNUM %then %do;
                     %if ^%index(&OverallByVar,%substr(&tempvar,%eval(%length(&temp)+1))) %then %do;
                        %let OverallByVar = &OverallByVar %substr(&tempvar,%eval(%length(&temp)+1));
                        %let nOverallByvar = %eval(&nOverallByvar+1);
                     %end;
                  %end;
                  %else %do;
                     %if ^%index(&OverallByVar,&tempvar) %then %do;
                        %let OverallByVar = &OverallByVar &tempvar;
                        %let nOverallByvar = %eval(&nOverallByvar+1);
                     %end;                   
                  %end;                   
                  %LABELRENAME;                    
                  %let k = %eval(&k+1);
               %end;
            run;
         %end;         

         %if ^&firstflag %then %do;
            data stats&temp;
               set stats;
            run;
            %let firstflag = 1;
         %end;
         %else %do;   
            proc append force base = stats&temp data = stats;
            run;
         %end;

         proc datasets nolist;
            delete temp stats;
         quit;

         %let j = %eval(&j+1);
      %end;       
   %end;
   proc datasets nolist;
      delete &temp;
   quit; 
   %let i = %eval(&i+1);   
%end;

%if ^%length(&availabledomains) %then %do;
   %put ERROR: No Findings domains meet the necessary criteria for the analysis.;
   %let exiterror = 1;
   %goto exit;
%end;

data summary_volcano;
   length test $ &TestLen      
      %do i = 1 %to &nOverallByvar; 
         %let tempvar = %scan(&OverallByvar,&i,' ');
         %if &tempvar ne VISITNUM %then %do; &tempvar $ &&&tempvar.Len %end;
      %end;
   ;     
   set
   %let i = 1;
   %do %while(%length(%scan(&availabledomains,&i,' ')));
      %let temp = %scan(&availabledomains,&i,' '); 
      stats&temp
      %let i = %eval(&i+1);
   %end;
   ;
   %if %upcase(&SumStatDQF) = MEAN %then %do;
      diffmean = mean - mean_ns;
      xaxis = t;
      label diffmean = "&trDiffMean";
   %end;
   %else %if %upcase(&SumStatDQF) = VARIANCE %then %do;
      xaxis = log2(std**2 / std_ns**2);
   %end; 
   %else %do;
      xaxis = &SumStatDQF / &SumStatDQF._ns;
   %end;
   %if %upcase(&SumStatDQF) = MEAN or %upcase(&SumStatDQF) = VARIANCE %then %do;
      Nlog10p = -log10(pval); 
      label Nlog10P = "&trNegLog10p" XAxis = "&trXAxis" pval = "&trPValue" Test = "&trTestName";
   %end;
   %else %if %upcase(&SumStatDQF) = SKEWNESS %then %do;
      label XAxis = "&trSkewRatio" Test = "&trTestName";
   %end;
   %else %if %upcase(&SumStatDQF) = KURTOSIS %then %do;
      label XAxis = "&trKurtRatio" Test = "&trTestName";
   %end; 
run;

%let TESTlabel = TEST;
%CheckLabel(summary_volcano,TESTlabel);

%let i = 1;
%do %while(%length(%scan(&OverallByVar,&i,' ')));
   %let temp = %scan(&OverallByVar,&i,' '); 
   %let &temp.Label = &temp;
   %CheckLabel(summary_volcano,&temp.Label);
   %if &i = 1 %then %let OverallByVarLabs = &&&temp.Label;
   %else %let OverallByVarLabs = &OverallByVarLabs~&&&temp.Label;
   %let i = %eval(&i+1);
%end;

proc sort data = summary_volcano;
   by domain test &OverallByVar;
run;

proc datasets nolist;
   delete 
   %let i = 1;
   %do %while(%length(%scan(&availabledomains,&i,' ')));
      %let temp = %scan(&domainlist,&i,' '); 
      stats&temp
      %let i = %eval(&i+1);
   %end;
   ;
quit;

%if %upcase(&SumStatDQF) = MEAN or %upcase(&SumStatDQF) = VARIANCE %then %do;
   %FDRPVALUES(inpv = pval, nlog10p = Nlog10P);
%end;

%let SumStat_OutData_v =;     
%PrepOutDS(SumStat_OutData_v,OutDataName_v, &Outname.SumStat, _vol);
%let includeds = &includeds &OutDataName_v..sas7bdat,;

proc sort data = summary_volcano;
   by siteid domain test;
run;

data &SumStat_OutData_v;
   merge summary_volcano(in = ina) country(in = inb) siteids;
   by siteid;
   if ina;
   %do i = 1 %to &nOverallByvar; 
      if missing(%scan(&OverallByvar,&i,' ')) then %scan(&OverallByvar,&i,' ') = "&trNA";
   %end;    
run;

* create JSL file;
%PrepOutFile(SumStatScript,SumStatScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &SumStatScript;
%let FullAPName=%str(Summary Statistics);

* put JSL statements;
data _null_;
   file "&SumStatScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:OverallByVar = ""&OverallByVar"";";
   put "objRefNS:SiteLab = ""&SITEIDLabel"";";
   put "objRefNS:TestLab = ""&TESTLabel"";";
   put "objRefNS:DomainLab = ""&trDOMAIN"";";
   put "objRefNS:OverallByVarLabs = ""&OverallByVarLabs"";";
   put "objRefNS:nOverallByvar = &nOverallByvar;";
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReport]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 
   put "  objRefNS:dvolcano = open(""&ClientOutPath.&OutDataName_v..sas7bdat"",invisible);";
   put "  Column(""&SITEIDLabel"") << Label(1);";
   put "  Column(""&TESTLabel"") << Label(1);";
   put "  Column(""&trSiteSampleSize"") << Label(1);";      
   %if %upcase(&SumStatDQF) = SKEWNESS or %upcase(&SumStatDQF) = KURTOSIS %then %do;
      %let j = 1;
      %do %while(%length(%scan(&OverallByVarLabs,&j,~)));
         %let tempby = %scan(&OverallByVarLabs,&j,~);
         put "  Column(""&tempby"") << Label(1);";
         %let j = %eval(&j+1);
      %end;       
   %end;
   %if %upcase(&SumStatDQF) = MEAN %then %do;
      put "  Column(""&trDiffMean"") << Label(1);";
   %end;
   put "objRefNS:volcano=Function({},{default local},";
   put "   VListBox(";
   %if %upcase(&SumStatDQF) = MEAN or %upcase(&SumStatDQF) = VARIANCE %then %do;
      %if %length(&RawCutOff) %then %do;
         put "   objRefNS:T1 = TextBox(locale:fill(locale:get(""XLineDrawnatMinusLogpX_Text""), {locale:get(""FDR_Text""), ""%sysfunc(round(&RawCutOff,0.0001))""})||"" (alpha=&AlphaValue)""),"; 
      %end;
      %else %do;
         put "   objRefNS:T1 = TextBox(locale:fill(locale:get(""NoPvalsSigAtAlphaXAfterAppXAdj_Text""), {""&AlphaValue"", locale:get(""FDR_Text"")}),<<Set Wrap(1000)),"; 
      %end;
      put "   HListBox(";
      put "   objRefNS:VolcPlot = Graph Builder(";
      put "    Auto Stretching(0),";
      put "    Automatic Recalc(1),";
      put "   Show Control Panel( 0 ),";
      put "   Variables(";
      put "    X( :Name(""&trXAxis"") ),";
      put "    Y( :Name( ""&trNegLog10p"" ) ),";
      put "    Color( :&SITEIDLabel )";
      put "   ),";
      put "   Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
      put "   SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       OutlineBox,";
      put "       {Set Title( locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""SummaryStatistics_Text"")}) )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trXAxis"",";
      put "       ScaleBox,";
      put "       {}";
      put "    ),";
      %if %length(&RawCutOff) %then %do;
         put "    Dispatch(";
         put "       {},";
         put "       ""&trNegLog10p"",";
         put "       ScaleBox,";
         put "       {Add Ref Line(";
         put "          &RawCutOff,";
         put "          Dashed,";
         put "          ""Red"",";
         put "          """",";
         put "          2";
         put "       )}";
         put "    ),";
      %end;
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 6 )} )";
      put "   )";
      put "   ),";
   %end;
   %else %do;
      put "   HListBox(";
      put "   objRefNS:VolcPlot = Graph Builder(";
      put "    Auto Stretching(0),";
      put "    Automatic Recalc(1),";
      put "    Show Control Panel( 0 ),";    
      put "    Variables(";
      put "       X( :&SITEIDLabel ),";
      put "       Y( :Name(""&trXAxis"") ),";
      put "       Color( :&SITEIDLabel ),";
      put "       Size( :&trN &trSite)";     
      put "    ),";
      put "    Elements( Points( X, Y, Legend( 6 ) ) ),";
      put "    SendToReport(";
      put "       Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( ""&trXAxis by &SITEIDLabel "" )} ),";
      put "       Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
      put "    );";
      put "   ),";
   %end;
   put "));";
   put ");";
   put "    objRefNS:CustomPanel = Expr(objRefNS:Filter = objRefNS:dvolcano << Data Filter(<<Conditional(1),";
   put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&trDOMAIN"",""&TESTLabel"")),";
   put "       Mode( Show(1), Include(1) )";
   put "    ));";      
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "InsertInto(objRefNS:DataList,{{""objRefNS:dvolcano""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trSummaryStatistics""}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""SummaryStatistics_Text"")})));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T1,objRefNS:VolcPlot}});";

   put "objRefNS:ActionButtonDesc = {};"; 
   put "objRefNS:ActionButtonList={{}};";
   put "objRefNS:ActionCodeList ={{}};";
   put "objRefNS:ActionDataTableList = {{}};";

   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsthenClick_Text""))});";  
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSummaryStatistics_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSite()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
   put "objRefNS:ActionSelectionCountList = {{1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.GenericSummaryReportingTable.gif""}};";
   put "objRefNS:ActionRequirementFlagList = {{0}};";
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SumStatScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend SummaryStatistics;
%SummaryStatistics;

************************************;
************************************;
************************************;
