/*
JMP Clinical Macros

Name: CDISCParms.sas
Description: CDISCParms contains a collection
of useful SAS macros that can be called from
Clinical analytical processes.

Detailed:
   1)  PRESPECIFY:         This macro will delete pre-specified records for events or interventions that do not occur.
   2)  MERGEDELTA:         This macro will merge delta when the review flag is not new and there is updated record.
   3)  ALPH:               Get domain information when SDTM is not specified and then use ADaM folder.
   4)  CUTWORD:            This macro will search specified word from a list and trim the list based on the word.
   5)  MERGEDMSUPP:        This macro will merge SUPPDM. If ADSL is represented by SDTM version of DM, it will look for SUPPDM 
                           and merge it if available;
   6)  CHKBLIND:           This macro will check if ARM is null and will impute values of Blinded Treatment.
   7)  INITIALIZE:         This macro is used to initialize data in JMPC. The majority of JMPC processes will run 
                           this macro in the beginning.   
   8)  LOADDATA:           This macro will apply subject filter and check for snap shot comparison and supplemental domains, 
                           and then merge in on the fly. It also calculates study day for each domain.
   9)  TREATMENTEMERGENT:  Calculate TEAE if TEAE flag Treatment Emergent Flag.
   10) GROUPASSIGN:        This macro assigns domain grouping data set macro variable.
   11) CDISCParms:         Assign CDISC parameters as macro variables. Run this macro since all processes need it.
   12) AssignCOTreatment:  This macro is used in data step that will create a new treatment variable to incorporate crossover.
   13) GroupVars:          Define new variables &domain.GRP, &domain.GRPN (numeric group index) and &domain.SGRPN (numeric group subindex) in order to meaningfully sort values.
   14) CHKDATE:            This examines the validity of ISO8601 Dates.
   15) CalculateStudyDays  This macro will calculate study day.
   16) calcStudyDay:       Calculate Study Day Directly from the SDTM IG.
   17) DATE9:              Create numeric days variable from iso 8601 variable.
   18) TIME5:              Create numeric seconds variable from iso 8601 variable.
   19) AddTimeWindows:     Add time window string to CatName and CatLabel if specified and/or create time window variable.
   20) createCounts:       Create counts to be used in tree map.
   21) SubjectDrillDowns:  macro for including drill-down buttons for Patient Profiler and Compare Subjects in tabbed report.
   22) DMFlags:            Set flags for demographic variables.
   23) DMLabels:           Set labels for demographic variables.
   24) COMPAREDATES:       Compare start and stop date, month and year.
   25) PROFILEDATEIMPUTE:  Impute profile date.
   26) GETMEDDRAVAR:       Get Variable Based on Drop Down Label.
   27) COMPAREMEDDRA:      Ensure that the Grouping Variable is at least the level of Term Variable.
   27) AESerOnlyToAEWhereStmt:   check AESerOnly and modify AEWhereStmt accordingly, accounting for translation of "Y".
   28) NONNULLVARS         For a dataset, determines the non-null variables
   29) OUTNAMETRUNC        Truncates the name of a specified output data set name to keep name, suffixes and number within 32 characters
   30) PARSEFINDINGS       Used with findings JSL widget to separate domain and test codes and create macro variables by domain 
   31) CHECKOVERLAP        Used with findings JSL widget to check overlap of test codes between categorical and continuous lists 
   32) CHECKPRES           Checks presence of variable in macro list and creates an indicator variable
   33) SHRINKCHARS         Shrink length of character variable to maximum observed length
   34) REPLACEWORD         Removes a word from a text string and replaces with new word. Accounts for some arithmetic symbols
   35) DELTAWHERE          Subsets data based on delta flag
   36) TEAETEXTFLAG        Create footnote to communicate TEAE preferences
*/

*************************************************************************************;
*     Comment Blocks to parse and track variable specification in clinical system;
*     Create general Events, Interventions, Findings blocks for process inclusion;
*     Choose block to use based on requirements for calculating Study Day;
*************************************************************************************;
/*    
$CVS_START$
$ID: Events$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur","TreatmentUse","PopulationFlags","CalcStudyDay_Events"}}$
$CVS_STOP$

$CVS_START$
$ID: EventsO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur","TreatmentUse","PopulationFlags","CalcStudyDay_EventsO"}}$
$CVS_STOP$

$CVS_START$
$ID: Interventions$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","InterventionOccur","TreatmentUse","PopulationFlags","CalcStudyDay_Interventions"}}$
$CVS_STOP$

$CVS_START$
$ID: InterventionsO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","InterventionOccur","TreatmentUse","PopulationFlags","CalcStudyDay_InterventionsO"}}$
$CVS_STOP$


$CVS_START$
$ID: Findings$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_Findings"}}$
$CVS_STOP$

$CVS_START$
$ID: FindingsV$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\FindingsMacros.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_FindingsV"},{"Visits"}}$
$CVS_STOP$

$CVS_START$
$ID: FindingsO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags","CalcStudyDay_FindingsO"}}$
$CVS_STOP$
*/

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro SHRINKCHARS(dsn = , vars = , flag = 0);
   %NObsVars(&dsn);
   %if &nobs %then %do; 
      %if &flag %then %do;
         %global siz;
      %end;
      
      data _null_;                                                
        set &dsn;                                                 
        array qqq(*) &vars;                                 
        call symput('siz',put(dim(qqq),5.-L));                    
        stop;                                                     
      run;
      
      %if &flag %then %do;
         %do i = 1 %to &siz.; 
            %global lll&i;
         %end;          
      %end;      
      
      data _null_;                                                
        set &dsn end=done;                                        
        array qqq(&siz) &vars;                              
        array www(&siz.);                                         
        if _n_=1 then do i= 1 to dim(www);                        
          www(i)=0;                                               
        end;                                                      
        do i = 1 to &siz.;                                        
          www(i)=max(www(i),length(qqq(i)));                      
        end;                                                      
        retain _all_;                                             
        if done then do;                                          
          do i = 1 to &siz.;                                      
            length vvv $50;                                       
            vvv=catx(' ','length',vname(qqq(i)),'$',www(i),';');  
            call symput('lll'||put(i,3.-L),vvv) ;                 
          end;                                                    
        end;                                                      
      run;                                                        

      %if ^&flag %then %do;         
         data &dsn;
            %do i = 1 %to &siz.;                                      
              &&lll&i                                                 
            %end;                                                     
            set &dsn;                                                 
         run;
      %end;      
   %end;
%mend SHRINKCHARS;    

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*** Delete events or interventions that did not occur;
*** AE Domain contains all events that actually occur;
*** Modified to accept these flags for any domain;
*** kemicl: Modified to remove xxPRESP check, unconditionally remove if xxOCCUR = N;

%macro PRESPECIFY(dom = &domain); 
 %if %sysfunc(indexw(&&&dom.VARS, %upcase(&dom)OCCUR, ' ')) > 0 %then %do;      
   if &dom.occur = "N" then delete;
 %end;   
%mend PRESPECIFY;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro MERGEDELTA(domlist = DM CO SE SV &findingsdomains &eventsdomains &interventionsdomains &adslpresent);
   %put Entering MERGEDELTA;
   %put DeltaDir = &DeltaDir;
   %let domlist = %sysfunc(compbl(&domlist));
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));

   %LocaleInit();
   
   %let trDropped = %nrbquote(%LocaleGet(Dropped_Text));
   
   %let i = 1;
   %do %while(%length(%scan(&domlist,&i,' ')));
      %let temp = %scan(&domlist,&i,' '); 
      %put domain = &temp;
      %if %sysfunc(findw(%upcase(&domains), &temp)) or &temp = ADSL %then %do;
         %if %sysfunc(fileexist(&DeltaDir.%lowcase(&temp).sas7bdat)) %then %do;
            %put &&temp.keys = &&&temp.keys;
            %if %length(&&&temp.keys) %then %do;
               proc sort data = &&in_&temp out = &temp;
                  by &&&temp.keys;
               run;

               proc sort data = DeltaLib.&temp out = keys(where = (&temp.RFlg ne "&trDropped"));
                  by &&&temp.keys;
               run;           

               data &temp;
                  merge &temp keys(keep = &&&temp.keys &temp.RFlg);
                  by &&&temp.keys;
                  %PRESPECIFY(dom = &temp);
               run;              

               proc datasets nolist;
                  delete keys;
               quit;

               %let in_&temp = work.&temp;
               %put updated in_&temp = &&in_&temp;

               *** if DM used for ADSL ***;
               %if ^%length(&adslpresent) and &temp = DM %then %do;
                  %let in_ADSL = work.&temp;
                  %put updated in_ADSL = &in_ADSL;          
               %end;
            %end;
         %end;
      %end;
      %let i = %eval(&i+1);
   %end;
   %put Exiting MERGEDELTA;
%mend MERGEDELTA;

*************************************************************************************;
*      This function should always be used to load domains for every report;
*      This function may be called in %macro INITIALIZE or directly from each report;
*      It will check for snap shot comparison and supplemental domains
*      and merge in on the fly. 
*      Merge with delta folder only if ReviewNumber is greater than 0.
*      Calculate Study Day for each domain;
*      filter data set can be a file with path or just a file name;
*************************************************************************************;

/*
$CVS_START$
$ID: LoadData$
$CMP: RSYS.Load Data$ $UE: DM.USUBJID and xx.USUBJID$ 
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: xx.STUDYID and xx.DOMAIN$
   $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$
$CVS_STOP$

$CVS_START$
$ID: LoadDataO$
$CMP: OSYS.Load Data$ $UE: DM.USUBJID and xx.USUBJID$ 
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: xx.STUDYID and xx.DOMAIN$
   $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$
$CVS_STOP$

$CVS_START$
$ID: InterventionOccur$
$CMP: OSYS.Interventions Occurrence Check$ $UE: INTERVENTIONS.xxOCCUR$ 
   $NOTES: Removes records with xxOCCUR = N$
$CVS_STOP$

$CVS_START$
$ID: EventOccur$
$CMP: OSYS.Events Occurrence Check$ $UE: EVENTS.xxOCCUR$ 
   $NOTES: Removes records with xxOCCUR = N$
$CVS_STOP$
*/
%Global sfname; 
%macro LOADDATA(domlist = ,mergesupp = &mergesuppdom,checkreview=1,filterds=,checkForZeroObs=0);
   %put Entering LOADDATA for the following domains: &domlist;
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _)); 
   %if ^%length(&domlist) %then %do;
      %put [MS_S_0188];
      %put ERROR: There is no domain to analyze with this report.;
      %let exiterror=1;
      %goto cexit;
   %end;
   %let domlist = %sysfunc(compbl(&domlist));
   %let trNew = %nrbquote(%LocaleGet(New_Text));

   %LocaleInit();
   
   %let first = 0;
   %let trXReviewFlag = %nrbquote(%LocaleGet(XReviewFlag_Text));

   %if %symexist(SubjectFilter)=0 %then %let SubjectFilter=;
   * check if WhereStmt is defined, if so, get the subset of the Subjects;
   %if ^%symexist(WhereStmt) %then %let WhereStmt=;
   
   %if %length(&filterds) %then %do;
      * check if filter file has a path;
      %let lp = %length(&filterds);
      %let x1 = 0;
      %let x2 = 0;
       %do _i_=1 %to &lp;
         %let ri = %qsubstr(&filterds,&_i_,1);
         %if &ri = %quote(/) or &ri = %quote(\) %then %let x1 = &_i_;
         %if &ri = %quote(.) %then %let x2 = &_i_;
      %end;
      %if &x2 = 0 | &x1 > &x2 %then %let x2 = %eval(&lp+1);
      %if &x1 > 0 %then %do;
         %let sfpath = %qsubstr(&_ref,1,&x1);
         %let sfName = %qsubstr(&_ref,%eval(&x1+1),%eval(&x2-&x1-1));
         libname sf "&sfpath";
         %let sfLib = sf;
      %end;
      %else %do;
         %let sfName = &filterds;
         %let sfLib = work;
      %end;
   %end;
   %else %do;
      * check if SubjectFilter is defined, if so, get the subset based on subject Filter;
      %if %length(&SubjectFilter) %then %do;
         %if &Server = Local %then %do;
            %let sfpath = %jg_getPath(/user/clinical/filters,,0,0,1,&studyname);
         %end;
         %else %do;
            %let sfpath = %jg_getPath(/user/clinical);
         %end;
         libname sf "&sfpath";
         %let sfLib = sf;
         * get filterdataset name;
         data _null_;
            set sf.subjectfilters;
            if FilterName="&SubjectFilter" then call symput ("sfName",FilterDataSet);
         run;
      %end;
      %else %let sfName=&SubjectFilter;
   %end; 
   %put sfName = &sfName;
   
   %let _i_ = 1;
   %do %while(%length(%scan(&domlist,&_i_,' ')));
      %let temp = %scan(&domlist,&_i_,' ');  
      %let data = %scan(&&in_&temp, 2, ".");
      %let lib = %scan(&&in_&temp, 1, ".");  
      %put temp = &temp;
      %let trxrf = %nrbquote(%LocaleFill(%nrbquote(&trXReviewFlag), f1=%nrbquote(&temp)));

      %if %sysfunc(findw(%upcase(&domains), %upcase(&temp))) or %upcase(&temp) = ADSL %then %do;
         
         *Filter subjcts and merge data with that in delta directory;
         %if %length(&sfName) %then %do;
            data &temp(compress=yes);
                  if 0 then set &sfLib..&sfName;
                  if _n_=1 then do;
                     declare hash h(dataset:"&sfLib..&sfName");
                     h.defineKey("USUBJID");
                     h.defineData(ALL:"YES");
                     h.defineDone();
                  end;
                  set &&in_&temp end=eof_e;                          
                  if h.find() = 0;
                  %PRESPECIFY(dom = &temp);
                  %if &ReviewNumber=0 & &ReviewsEnabled %then %do;
                     &temp.RFlg = "&trNew"; 
                     label &temp.RFlg = "&trxrf";
                  %end;
               run;           

               * merge in delta review flag if needed;
               %if &ReviewNumber>0 and &checkreview and %symexist(&&temp.keys) %then %do;
                  %let DeltaDir=%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder);
                  libname DeltaLib "&DeltaDir";
                  %if %sysfunc(fileexist(&DeltaDir.%lowcase(&temp).sas7bdat)) and %length(&&&temp.keys) %then %do;
                     data d&temp(compress=yes);
                        if 0 then set &sfLib..&sfName;
                        if _n_=1 then do;
                           declare hash h(dataset:"&sfLib..&sfName");
                           h.defineKey("USUBJID");
                           h.defineData(ALL:"YES");
                           h.defineDone();
                        end;
                        set DeltaLib.&temp end=eof_list;
                        if h.find() = 0;
                        keep &&&temp.keys &temp.RFlg;
                     run;  
                     * consider one way to use proc sql for merging;
                     proc sort data =&temp;
                        by &&&temp.keys;
                     run;
                     proc sort data =d&temp;
                        by &&&temp.keys;
                     run;
                     data &temp(compress=yes);
                        merge &temp d&temp;
                        by &&&temp.keys;
                     run;
                  %end;
               %end;
         %end; 
         %else %do;
         * if no subject filter is selected, merge domain with delta directory;
         * for cases snapshot comparison being disabled or ReviewNumber is zero;
         * there is no need to merge with that data in delta domain;
            %if &ReviewNumber>0 & &checkreview  %then %do;
               proc sort data = &&in_&temp out = &temp;
                  by &&&temp.keys;
               run;
               proc sort data = DeltaLib.&temp out = keys(where = (&temp.RFlg ne "Dropped"));
                  by &&&temp.keys;
               run;
               data &temp(compress=yes);
                  merge &temp keys(keep = &&&temp.keys &temp.RFlg);
                  by &&&temp.keys;
                  %PRESPECIFY(dom = &temp);
               run;  

               proc datasets nolist;
                  delete keys;
               quit; 
            %end;
            %else %do;
               data &temp(compress=yes);
                  set &&in_&temp;
                  %if &ReviewsEnabled %then %do;
                     length &temp.RFlg $ 16;
                     &temp.RFlg = "&trNew"; 
                     label &temp.RFlg = "&trxrf";
                  %end;
                  %PRESPECIFY(dom = &temp);
               run;              
            %end;
         %end;

         %let in_&temp = work.&temp;
         %put updated in_&temp = &&in_&temp;
       
         *if there is supp domain, we will need to merge it with main domain;
         %if %upcase(&lib) = SDTM and &mergesupp = 1 and %sysfunc(findw(&suppdomains, %upcase(SUPP&temp))) %then %do;
            *** Removes duplicate qnams based on STUDYID, DOMAIN, USUBJID, IDVAR, IDVARVAL;  
            %let IDVAR =; 
            %let IDVARVAL =;
            %let IDVARTYPE =;
            %let _tid_ = %sysfunc(open(&lib..SUPP&temp));
            %if %sysfunc(varnum(&_tid_,IDVAR)) %then %let IDVAR = IDVAR;
            %if %sysfunc(varnum(&_tid_,IDVARVAL)) %then %let IDVARVAL = IDVARVAL;
            %let _tid_ = %sysfunc(close(&_tid_)); 

            %if %length(&IDVAR) %then %do;
               %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) eq %nrbquote(zh) %then %do;
                  proc freq noprint data = sdtm.SUPP&temp(encoding=ANY);
                     tables &IDVAR / missing out = temp1(rename = (&IDVAR = valuec));
                  run;
               %end;
               %else %do;
                  proc freq noprint data = sdtm.SUPP&temp;
                     tables &IDVAR / missing out = temp1(rename = (&IDVAR = valuec));
                  run;
               %end;

               data _null_;
                  set temp1;
                  if missing(valuec) and round(percent) = 100 then do;
                     call symput("IDVAR", "");
                  end;
               run;  
            %end;
            %if %length(&IDVARVAL) %then %do;
               %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) eq %nrbquote(zh) %then %do;
                  proc freq noprint data = sdtm.SUPP&temp(encoding=ANY);
                     tables &IDVARVAL) / missing out = temp1(rename = (&IDVARVAL = valuec));
                  run;
               %end;
               %else %do;
                  proc freq noprint data = sdtm.SUPP&temp;
                  tables &IDVARVAL / missing out = temp1(rename = (&IDVARVAL = valuec));
                  run;
               %end;
               data _null_;
                  set temp1;
                  if missing(valuec) and round(percent) = 100 then do;
                        call symput("IDVARVAL", "");
                     end;
                  run;
            
                  proc datasets nolist;
                        delete temp1;
                  quit;
            %end;

            * if both IDvar and IDVarval are present;
            %put IDVAR=&IDVAR IDVARVAL=&IDVARVAL;
            %if (%length(&IDVAR) and %length(&IDVARVAL)) or (^%length(&IDVAR) and ^%length(&IDVARVAL)) %then %do;
               %let IDVARACTUAL =;
               %if %length(&IDVAR) %then %do;
                  data _null_;
                     set sdtm.supp&temp(obs=1 keep = IDVAR where = (IDVAR ne ""));
                     if length(IDVAR) then call symput("IDVARACTUAL",IDVAR);
                  run;
               %end;
               %if %length(&IDVARACTUAL) %then %do;
                  %let _tid_ = %sysfunc(open(&lib..&temp));
                  %let _vn_ = %sysfunc(varnum(&_tid_,&IDVARACTUAL));
                  %let idvartype = %sysfunc(vartype(&_tid_,&_vn_)); 
                  %let _tid_ = %sysfunc(close(&_tid_));  
               %end;
               *** divert errors on merge with supp info ***;
               %if ^&first %then %do; 
                  proc printto log = "&OutPath.SUPPMerge.log" new;
                  run;
               %end;
               %else %do;
                  proc printto log = "&OutPath.SUPPMerge.log";
                  run;           
               %end;
               *** Removes duplicate qnams based on STUDYID, DOMAIN, USUBJID, IDVAR, IDVARVAL;  
               proc sql noprint;
                  create table supp&temp(rename = (rdomain = DOMAIN) where = (IDVAR = "&IDVARACTUAL")) as
                  select *, FREQ(qnam) AS numqnam
                  from sdtm.supp&temp
                  %if (%length(&IDVAR) and %length(&IDVARVAL)) %then %do;
                     group by studyid, rdomain, usubjid, idvar, idvarval, qnam
                  %end;
                  %else %do;
                     group by studyid, rdomain, usubjid, qnam
                  %end;
                  having numqnam = 1
                  %if (%length(&IDVAR) and %length(&IDVARVAL)) %then %do;
                     order by studyid, rdomain, usubjid, idvar, idvarval, qnam;
                  %end;
                  %else %do;
                     order by studyid, rdomain, usubjid, qnam;
                  %end;
               quit;
               * if IDVAR and IDVARVAL are available;
               %if (%length(&IDVAR) and %length(&IDVARVAL)) %then %do; 
               proc transpose data = supp&temp out = supp&temp(drop = _name_ _label_ idvar idvarval);
                  by studyid domain usubjid idvar idvarval qnam;
                  var idvarval;
                  id idvar;
                  copy qval qlabel;
               run;
               %end;
               proc transpose data = supp&temp out = supp&temp(drop = _name_ _label_);
                  %if %length(&IDVARACTUAL) %then %do;
                     by studyid domain usubjid &IDVARACTUAL;
                  %end;
                  %else %do;
                     by studyid domain usubjid;
                  %end;
                  var qval;
                  id qnam;
                  idlabel qlabel;
               run;
               
               %SHRINKCHARS(dsn = supp&temp, vars = &&&temp.suppvars, flag = 1);
                              
               %if &idvartype = N %then %do;
                  data supp&temp(rename = (N&IDVARACTUAL = &IDVARACTUAL));
                     set supp&temp;
                     N&IDVARACTUAL = &IDVARACTUAL + 0;
                     drop &IDVARACTUAL;
                  run;
               %end;

               proc contents data = &temp;
                  ods exclude all;
                  ods output variables = vars(keep = num variable);
               run;
               ods select all;               
               
               proc sql noprint;
                  select distinct variable
                     into :tempvars separated by ' '
                     from vars
                     order by num;
               quit;
               
               proc datasets nolist;
                  delete variables;
               quit;              

               proc sql noprint;
                  create table &temp as
                  select * from &temp as aa left join supp&temp(drop = domain) as bb
                  %if %length(&IDVARACTUAL) %then %do;
                     on aa.studyid = bb.studyid and aa.usubjid = bb.usubjid and aa.&IDVARACTUAL = bb.&IDVARACTUAL
                     order by %sysfunc(transtrn(&&&temp.keys, %str( ), %str(,)));
                  %end;
                  %else %do;
                     on aa.studyid = bb.studyid and aa.usubjid = bb.usubjid
                     order by %sysfunc(transtrn(&&&temp.keys, %str( ), %str(,)));
                  %end; 
               quit;
               %if %eval(&syserr > 4) %then %do;
                  %put NOTE: SUPP Merges Failure for supp&temp;;
               %end;
               %else %do;
                  %put Supplemental domain &temp has been successfully merged with supp&temp;
               %end;
               proc printto;
               run;
               %let first = 1;
               
               proc datasets nolist;
                  delete supp&temp;
               quit;
                              
               %if &temp = DM %then %do;
                  data &temp;
                     retain &tempvars;
                     %do i = 1 %to &siz.; &&lll&i %end;                          
                     set &temp;
                  run;
                  %do i = 1 %to &siz.; 
                     %symdel lll&i;
                  %end; 
                  %symdel siz tempvars;                  
               %end;

            %end;
            %let &temp.vars = &&&temp.VARS &&&temp.SUPPVARS;
         %end;
      %end;
      %else %do;
         %put [MS_S_0189];
         %put ERROR: Domain &temp does not exist in the specified SDTM or ADAM folder.;
         %let exiterror=1;
      %end;

      *** rename variables to accommodate BDS ***;
      %if %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
         %let _tid_ = %sysfunc(open(&temp));
         %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
            %let Datevarnum = %sysfunc(varnum(&_tid_,ADTM));
            %let DatevarLab = %sysfunc(varlabel(&_tid_,&Datevarnum));
         %end;
         %else %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADT)) %then %do;
            %let Datevarnum = %sysfunc(varnum(&_tid_,ADT));
         %end;
         %else %do;
            %let Datevarnum = 0;
         %end;
         %let DatevarLab = %sysfunc(varlabel(&_tid_,&Datevarnum));
         %let _tid_ = %sysfunc(close(&_tid_)); 

         data &temp(compress=yes);
            %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
               length %sysfunc(tranwrd(xxDTC, xx, &temp)) $ 19;
            %end;
            %else %if %sysfunc(findw(upcase(&&&temp.BDSVARS), ADT)) %then %do;
               length %sysfunc(tranwrd(xxDTC, xx, &temp)) $ 10;
            %end;
            set &temp;
         %let k = 1;
         %do %while(%length(%scan(&_fromlist,&k,' ')));
            %let tempf = %scan(&_fromlist,&k,' ');
            %let tempt = %scan(&_tolist,&k,' ');
            %let VarName = %sysfunc(tranwrd(&tempt, xx, &temp));
            %if &tempf ne ADT %then %do;
               %if %sysfunc(findw(upcase(&&&temp.BDSVARS), &tempf)) %then %do;
                  %if &tempf = ADTM %then %do;        
                     &VarName = put(&tempf,E8601DT19.); 
                     drop &tempf;
                     label &VarName = "&DatevarLab";
                  %end;
                  %else %do;
                     rename &tempf = &VarName;
                  %end;                           
               %end;             
            %end;
            %else %if &tempf = ADT %then %do;
               %if ^%sysfunc(findw(upcase(&&&temp.BDSVARS), ADTM)) %then %do;
                  &VarName = put(&tempf,E8601DA.);
                  drop &tempf;                         
                  label &VarName = "&DatevarLab";
               %end;             
            %end;                                        
            %let k = %eval(&k+1);      
         %end;
         run;
      %end;
         
      *** Add Reference Date and calculate Study Day for each domain;  
      %if &temp ne DM and &temp ne ADSL %then %do;
         *check if rfstdtc and rfendtc are available;
         %let rfflag = 0;
         %let rfeflag = 0;
         %let unsubflag=0;
         %if %sysfunc(findw(upcase(&ADSLVARS), RFSTDTC)) %then %let rfflag = 1;
         %if %sysfunc(findw(upcase(&ADSLVARS), RFENDTC)) %then %let rfeflag = 1;
       
         %if %length(&WhereStmt) %then %do;
            data adsl;
               set &in_adsl;
               where &WhereStmt;
            run;
            %let in_adsl1 = adsl;
         %end;
         %else %do;
            %let in_adsl1 = &in_adsl;
         %end;

         * calculate study day when there is usubjid;
            proc sql noprint;
            create table &temp.2(drop = usubjid1 usubjid2) as
               select coalesce(aa.usubjid, bb.usubjid2) as usubjid1, aa.*, bb.*
                  from &temp as aa left join &in_adsl1(keep = usubjid %if &rfflag %then rfstdtc; %if &rfeflag %then rfendtc; rename=(usubjid= usubjid2)) as bb
               on aa.usubjid = bb.usubjid2
                  order by aa.usubjid;
            quit;

            data &temp.2(compress=yes);               
               %if %upcase(&lib) = SDTM and &mergesupp = 1 and %sysfunc(findw(&suppdomains, %upcase(SUPP&temp))) %then %do;
                  retain &tempvars;
                  %do i = 1 %to &siz.; &&lll&i %end;      
               %end; 
               set &temp.2;
               %calculateStudyDays(&temp);
            run;
                                     
            %if %upcase(&lib) = SDTM and &mergesupp = 1 and %sysfunc(findw(&suppdomains, %upcase(SUPP&temp))) %then %do;
               %do i = 1 %to &siz.; 
                  %symdel lll&i;
               %end; 
               %symdel siz tempvars;
            %end;

            proc datasets nolist;
               delete &temp;
               change &temp.2 = &temp;
            quit; 
         
         /* kemicl: error check if loaded data has now valid rows for analysis */
         /* sasajf: make zero obs conditional (see argument list) */
         %if &checkForZeroObs %then %do;
         %nobsvars(&temp);
         %if &nobs = 0 %then %do;
            %put [MS_S_0190];
            %put ERROR: There are no valid observations in the &temp domain for analysis.  Please check that your study data sets or any applied subject filters are correct.;
            %let exiterror = 1;
         %end;
      %end;       
      %end;       
      %let _i_ = %eval(&_i_+1);     
   %end;
   %put Exiting LOADDATA;
   %cexit:;
%mend LOADDATA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

   %macro ALPH(varlist = );      
      %if %length(&&&varlist) %then %do;
         data alph;
            length domain $ 8;
            %let i = 1;
            %do %while(%length(%scan(&&&varlist,&i,' ')));
               %let temp = %scan(%upcase(&&&varlist),&i," ");
               domain = "&temp";
               output;
              %let i = %eval(&i+1);
            %end; 
         run;

         proc sort data = alph;
            by domain;
         run;

         proc sql noprint;
            select domain
               into :&varlist separated by ' '
               from alph;
         quit;
         
         proc datasets nolist;
            delete alph;
         quit;
      %end;
   %mend ALPH;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CUTWORD(list = , word = TRT01P);
   %let len = %length(&word);
   %let llen = %length(&&&list);
   %let first = %sysfunc(indexw(&&&list,&word, %str( )));
   %let last = %eval(&first + &len - 1);
   %if &first = 1 %then %let &list = %substr(&&&list, %eval(&last+2));
   %else %if &last = &llen %then %let &list = %substr(&&&list, 1, %eval(&first-2));
   %else %let &list = %substr(&&&list, 1, %eval(&first-2)) %substr(&&&list, %eval(&last+2));
%mend CUTWORD;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro REPLACEWORD(where = , word = , newword = );
   %let len = %length(&word);
   %let llen = %length(&&&where);
   %let first = %sysfunc(indexw(%upcase(&&&where), &word, %str( )+/-%str(*)=));
   %let last = %eval(&first + &len - 1);

   %if &first = 1 %then %let &where = &newword%substr(&&&where, %eval(&last+1));
   %else %if &last = &llen %then %let &where = %substr(&&&where, 1, %eval(&first-1))&newword;
   %else %let &where = %substr(&&&where, 1, %eval(&first-1))&newword%substr(&&&where, %eval(&last+1)); 
%mend REPLACEWORD;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro MERGEDMSUPP; 
      %let _tid_ = %sysfunc(open(&in_adsl));
      %let usubjvn = %sysfunc(varnum(&_tid_,USUBJID));
      %let studyvn = %sysfunc(varnum(&_tid_,STUDYID));
      %let SubLab = %sysfunc(varlabel(&_tid_,&usubjvn));
      %let StudyLab = %sysfunc(varlabel(&_tid_,&studyvn));
      %let _tid_ = %sysfunc(close(&_tid_));  

      %let _tid_ = %sysfunc(open(SDTM.suppdm));
         %let idvarnum = %sysfunc(varnum(&_tid_,idvar));
      %let _tid_ = %sysfunc(close(&_tid_));

      *** Removes duplicate qnams based on STUDYID, DOMAIN, USUBJID, IDVAR, IDVARVAL;   
      proc sql noprint;
         create table suppdm(rename = (rdomain = domain)) as
            select *, FREQ(qnam) AS numqnam
            from sdtm.suppdm
            group by studyid, rdomain, usubjid, %if &idvarnum %then %do; idvar, idvarval, %end; qnam
            having numqnam = 1
            order by studyid, rdomain, usubjid, %if &idvarnum %then %do; idvar, idvarval, %end; qnam;
      quit;

      *** This works if IDVAR and IDVARVAL are NULL; 
      %if &idvarnum %then %do;
         proc transpose data = suppdm out = suppdm(drop = _name_ _label_ idvar);
            by studyid domain usubjid idvar idvarval qnam;
            var idvarval;
            id idvar;
            copy qval qlabel qorig;
         run;
      %end;

      proc transpose data = suppdm out = suppdm(drop = _name_ _label_ idvarval);
         by studyid domain usubjid %if &idvarnum %then idvarval;;
         var qval;
         id qnam;
         idlabel qlabel;
      run;
      
      %SHRINKCHARS(dsn = suppdm, vars = &dmsuppvars, flag = 1);
      
      %let in_ADSL = work.DM;
      %let in_DM = work.DM;
      %put updated in_adsl = &in_adsl;
      %put updated in_dm = &in_dm;

      proc contents data = sdtm.dm;
         ods exclude all;
         ods output variables = vars(keep = num variable);
      run;
      ods select all;

      proc sql noprint;
         select distinct variable
            into :tempvars separated by ' '
            from vars
            order by num;
      quit;

      proc datasets nolist;
         delete variables;
      quit;
      
      proc sql noprint;
         create table suppdm2 as
         select * from sdtm.dm natural left join suppdm
         order by usubjid;
      quit;
            
      data &in_DM;
         retain &tempvars;
         %do i = 1 %to &siz.;                                      
            &&lll&i                     
         %end;                                     
         set suppdm2;
         label usubjid = "&SubLab" STUDYID="&StudyLab";
      run;     

      %do i = 1 %to &siz.; 
         %symdel lll&i;
      %end; 
      %symdel siz tempvars;               

      proc datasets nolist;
         delete suppdm suppdm2;
      quit;      
%mend MERGEDMSUPP;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* This macro will check to see if ARM is null and will impute values
   of Blinded Treatment */

%macro CHKBLIND(domain = );
   %if &domain = DM or &domain = ADSL %then %do;   
      *** Need to check if ARM in data and if so if null ***;
      %let rschk = %sysfunc(open(&&in_&domain));
      %let armchk = %sysfunc(varnum(&rschk,ARM));
      %let pchk = %sysfunc(varnum(&rschk,TRT01P));
      %let achk = %sysfunc(varnum(&rschk,TRT01A));
      %let actchk = %sysfunc(varnum(&rschk,ACTARM));
      %let rschk = %sysfunc(close(&rschk));

      %if ^&armchk or (&armchk and ^%sysfunc(indexw(&&&domain.VARS, ARM, ' '))) %then %do;
         proc sort data = &&in_&domain out = &domain;
            by usubjid;
         run;

         %let in_&domain = work.&domain;
         %put updated in_&domain = &&in_&domain;
         
         %if &domain = DM and %sysfunc(indexw(%upcase(&in_adsl), DM, '.')) %then %do;
            %let in_adsl = work.&domain;
            %put updated in_adsl = &&in_&domain;
            %let ADSLVARS = &ADSLVARS ARM;
         %end;   
         %let &domain.VARS = &&&domain.VARS ARM;

         data &&in_&domain;
            length ARM $ 50;
            set &domain;
            ARM = "Blinded Treatment";
            %if &pchk %then %do;
               drop TRT01P;
               %CUTWORD(list = &&domain.VARS, word = TRT01P);
               %if &domain = DM and %sysfunc(indexw(%upcase(&in_adsl), DM, '.')) %then %do;
                  %CUTWORD(list = ADSLVARS, word = TRT01P);
               %end;   
            %end;
            %if &achk %then %do;
               drop TRT01A;
               %CUTWORD(list = &&domain.VARS, word = TRT01A);
               %if &domain = DM and %sysfunc(indexw(%upcase(&in_adsl), DM, '.')) %then %do;
                  %CUTWORD(list = ADSLVARS, word = TRT01A);
               %end;   
            %end;
            %if &actchk %then %do;
               drop ACTARM;
               %CUTWORD(list = &&domain.VARS, word = ACTARM);
               %if &domain = DM and %sysfunc(indexw(%upcase(&in_adsl), DM, '.')) %then %do;
                  %CUTWORD(list = ADSLVARS, word = ACTARM);
               %end;                
            %end;               
            %if ^&armchk %then %do;
               label arm = "Description of Planned Arm";
            %end;
         run;   
      %end;
   %end;
   %else %if &domain = EX %then %do;
      %let rschk = %sysfunc(open(&&in_&domain));
      %let doschk = %sysfunc(varnum(&rschk,EXDOSE));
      %let trtchk = %sysfunc(varnum(&rschk,EXTRT));
      %let rschk = %sysfunc(close(&rschk));

      %if ^&doschk or (&doschk and ^%sysfunc(indexw(&&&domain.VARS, EXDOSE, ' '))) or 
          ^&trtchk or (&trtchk and ^%sysfunc(indexw(&&&domain.VARS, EXTRT, ' '))) %then %do;

         data &domain;
            set &&in_&domain;
         run;
         
         %let in_&domain = work.&domain;
         %put updated in_&domain = &&in_&domain;               
                  
         %if ^&trtchk or (&trtchk and ^%sysfunc(indexw(&&&domain.VARS, EXTRT, ' '))) %then %do;
            %let &domain.VARS = &&&domain.VARS EXTRT;
            data &domain;
               length EXTRT $ 17;
               set &domain;
               EXTRT = "Blinded Treatment";
               %if ^&trtchk %then %do;
                  label EXTRT = "Name of Actual Treatment";
               %end;
            run;
         %end;

         %if ^&doschk or (&doschk and ^%sysfunc(indexw(&&&domain.VARS, EXDOSE, ' '))) %then %do;
            %let &domain.VARS = &&&domain.VARS EXDOSE;
            data &domain;     
               set &domain;
               EXDOSE = 1;
               %if ^&doschk %then %do;
                  label EXDOSE = "Dose per Administration";
               %end;
            run;                 
         %end;          
         
         data &&in_&domain;
            set &domain;
         run;        
      %end;
   %end; 
%mend CHKBLIND;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro INITIALIZE;
   %if &ProcessName ne DefineRiskThresholdDataSet %then %do;
      %put Entering INITIALIZE;
      %global _fromlist _tolist;
      options nofmterr;
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %if &sysscp = WIN %then %do;
         %let _slash_ = \;
      %end;
      %else %do;
         %let _slash_ = /;
      %end;

      %LocaleInit();

      %if ^%symexist(Server) %then %let Server = Local;
      %include "&MacroPath./UtilityPathMacros.sas" / nosource;
      %let mbc = %jg_getPath(/user/clinical/metadata/base, CSTMacroVars.sas, 0, 0, 3, &studynameunder);
      %include "&mbc" / nosource;

      *** translate variable lists to accommodate BDS ***;
      %if %symexist(bdsdomains)=0 %then %let bdsdomains=;   
      %if %length(&bdsdomains) %then %do;
         %let i = 1;
         %let _fromlist = PARAM PARAMCD AVISIT ADT ADTM ADY AVISITN ATPT ATPTN AVAL AVALC ANRLO ANRHI ABLFL;
         %let _tolist = xxTEST xxTESTCD VISIT xxDTC xxDTC xxDY VISITNUM xxTPT xxTPTNUM xxSTRESN xxSTRESC xxSTNRLO xxSTNRHI xxBLFL; 
         %do %while(%length(%scan(&bdsdomains,&i,' ')));
            %let temp = %scan(&bdsdomains,&i,' ');
            %global &temp.BDSVARS;
            %let &temp.BDSVARS = &&&temp.VARS;
               %let j = 1;
               %do %while(%length(%scan(&_fromlist,&j,' ')));
                  %let tempf = %scan(&_fromlist,&j,' ');
                  %let tempt = %scan(&_tolist,&j,' ');
                     %if &tempf ne ADT %then %do;
                        %if %sysfunc(findw(upcase(&&&temp.VARS), &tempf)) %then %do;                     
                           %CUTWORD(list = &&temp.VARS, word = &tempf);                      
                           %let &temp.VARS = &&&temp.VARS %sysfunc(tranwrd(&tempt, xx, &temp));             
                        %end;             
                     %end;
                     %else %if &tempf = ADT %then %do;
                        %if ^%sysfunc(findw(upcase(&&&temp.VARS), ADTM)) and %sysfunc(findw(upcase(&&&temp.VARS), &tempf)) %then %do;
                           %CUTWORD(list = &&temp.VARS, word = &tempf);
                           %let &temp.VARS = &&&temp.VARS %sysfunc(tranwrd(&tempt, xx, &temp));             
                        %end;             
                     %end;                                  
                  %let j = %eval(&j+1);      
               %end;
            %let i = %eval(&i+1);
         %end;

         *** modify where statements ***;
         %let wherestatelist = WhereTests FWhereStmtA FWhereStmtB FWhereStmtC;
         %let wheredomlist = FDomain FFDomA FFDomB FFDomC;

         %let i = 1;
         %do %while(%length(%scan(&wherestatelist,&i,' ')));
            %let tempwstate = %scan(&wherestatelist,&i,' '); 
            %let tempwdom = %scan(&wheredomlist,&i,' '); 

            %if %symexist(&tempwstate) %then %do;
               %if %length(&&&tempwstate) and %sysfunc(findw(upcase(&bdsdomains), &&&tempwdom))%then %do;
                  %put NOTE: Original &tempwstate &&&tempwstate;
                  *** add spaces around parentheses ***;
                  %let &tempwstate = %sysfunc(tranwrd(%nrbquote(&&&tempwstate), %str(%(), %str(%( )));
                  %let &tempwstate = %sysfunc(tranwrd(%nrbquote(&&&tempwstate), %str(%)), %str( %))));

                  %let j = 1;
                  %do %while(%length(%scan(&_fromlist,&j,' ')));
                     %let tempf = %scan(&_fromlist,&j,' ');
                      %let tempt = %scan(&_tolist,&j,' ');
                      %do %while(%sysfunc(indexw(%upcase(&&&tempwstate), &tempf,%str( )+/-%str(*)=)));
                         %let newvar = %sysfunc(tranwrd(&tempt, xx, &&&tempwdom)); 
                         %REPLACEWORD(where = &tempwstate, word = &tempf, newword = &newvar);                       
                      %end;             
                      %let j = %eval(&j+1);      
                  %end;
               %end;
               %put NOTE: Modified &tempwstate &&&tempwstate;
            %end;   
            %let i = %eval(&i+1);
         %end;      
      %end;   

      %if %sysfunc(findw(&domains, DM)) = 0 and %length(&adslpresent) %then %do;
         %let domains = &domains DM;
         %ALPH(varlist = domains);
         %global in_DM;
         %let in_DM = &in_ADSL;
         %global DMVARS DMKEYS;
         %let DMVARS = &ADSLVARS;
         %let DMKEYS = ;
         /*
         %let DMKEYS = &ADSLKEYS;
         data deltalib.dm;
            set deltalib.adsl;
            rename ADSLRFlg = DMRFlg;
         run;
         */
      %end;

      %put NOTE: Available Domains: &domains;
      %put NOTE: Domains with Supplemental Qualifiers: &suppdomains;
      %put NOTE: Events Domains: &eventsdomains;
      %put NOTE: Findings Domains: &findingsdomains;
      %put NOTE: Interventions Domains: &interventionsdomains;
      %put NOTE: ADaM BDS Domains: &bdsdomains;
      %if %length(&adslpresent) %then %put NOTE: ADSL Available;
      %else %put NOTE: ADSL NOT Available;

      *** Unquote domain macros;
      %if %symexist(FDomain) %then %do;
         %let FDomain = %unquote(&FDomain);
      %end;
      %if %symexist(EventsDomain) %then %do;
         %let EventsDomain = %unquote(&EventsDomain);
      %end;
      %if %symexist(InterventionsDomain) %then %do;
         %let InterventionsDomain = %unquote(&InterventionsDomain);
      %end;

      *** If ADSL is represented by SDTM version of DM, look for SUPPDM and merge in if available;
      %if %upcase(&in_adsl) = SDTM.DM and %sysfunc(findw(%upcase(&suppdomains), SUPPDM)) > 0 %then %do;
         %MERGEDMSUPP;
         %let ADSLVARS = &ADSLVARS &ADSLSUPPVARS;
      %end;   

      %let i = 1;
      %do %while(%length(%scan(&domains,&i,' ')));
         %let temp = %scan(&domains,&i,' ');
         %put NOTE: in_&temp = &&in_&temp;
         %let i = %eval(&i+1);
      %end;
      %put NOTE: in_adsl = &in_adsl;

      *** Merge in Deltah Revue Flags ***;
      %if ^%symexist(ReviewsEnabled) %then %do;
         %global ReviewsEnabled;
         %let ReviewsEnabled=1;
      %end;

      *merge DM table with ADSL if variables is present in DM, but not in ADSL;
      %if %length(&SDTMMETA) & %length(&adslpresent) %then %do;
         proc sql noprint;
            create table ADSL as
            select * from &in_ADSL a1 left join &in_DM a2
            on a1.USUBJID=a2.USUBJID;
         quit;
         %let in_adsl = work.adsl;
      %end;
      %put updated in_adsl = &in_adsl;

      *** Check if ARM are null and fill in with "Blinded" ***;
      %if %length(&adslpresent) %then %do;
         %CHKBLIND(domain = ADSL);
      %end;
      %if %sysfunc(findw(&domains, DM)) %then %do;
         %CHKBLIND(domain = DM);
      %end;
      *** Check if EXTRT and EXDOSE are null ***;
      %if %sysfunc(findw(&domains, EX)) %then %do;
         %CHKBLIND(domain = EX);
      %end;

      title;
      footnote;
      options orientation = portrait;

      %global IconPath;
      %let IconPath = /install/lifesciences/documentation/icons/clinical/;
      
      %global IntDateFormat;
      %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(ja) or
         %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then
            %let IntDateFormat = DDMMYY10;
      %else %let IntDateFormat = DATE9;

      %put Exiting INITIALIZE;
   %end;
%mend INITIALIZE;
%macro RUNINITIALIZE;
   %if ^%symexist(runCDISCParmsFlag) %then %let runCDISCParmsFlag=0;
   %if ^&runCDISCParmsFlag %then %do;
      %INITIALIZE;
   %end;
%mend RUNINITIALIZE;
%RUNINITIALIZE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/*
$CVS_START$
$ID: TreatmentEmergentEvents$
$CMP: OPV.{"Event Type","Pre treatment events"}$
   $UE: EVENTS.xxTRTEM or EVENTS.TRTEMFL or (EVENTS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$
$CMP: CPV.{"Event Type","Treatment emergent events"}$
   $UE: EVENTS.xxTRTEM or EVENTS.TRTEMFL or (EVENTS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value of Y or Yes for flag variables. See parameter documentation for details of algorithm$
$CMP: CPV.{"Event Type","On treatment events"}$
   $UE: (EVENTS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Event start date and Treatment start date determine on treatment events, optionally use end dates.$
$CMP: OPV.{"Event Type","On treatment events"}$
   $UE: DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC$
   $NOTES: End dates used if available, otherwise everything after start date is assumed on treatment.$  
$CMP: CPV.{"Event Type","Off treatment follow up events"}$
   $UE: (EVENTS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC)
        and (DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC))$
   $NOTES: Treatment has to both start and stop to calculate off treatment events$
$CVS_STOP$


$CVS_START$
$ID: TreatmentEmergentInterventions$
$CMP: OPV.{"Intervention Type","Pre treatment interventions"}$
   $UE: INTERVENTIONS.xxTRTEM or INTERVENTIONS.TRTEMFL or (INTERVENTIONS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All interventions pre-treated if patient not treated$
$CMP: CPV.{"Intervention Type","Treatment emergent interventions"}$
   $UE: INTERVENTIONS.xxTRTEM or INTERVENTIONS.TRTEMFL or (INTERVENTIONS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value of Y or Yes for flag variables. See parameter documentation for details of algorithm$
$CMP: CPV.{"Intervention Type","On treatment interventions"}$
   $UE: (INTERVENTIONS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Intervention start date and Treatment start date determine on treatment events, optionally use end dates.$
$CMP: OPV.{"Intervention Type","On treatment interventions"}$
   $UE: DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC$
   $NOTES: End dates used if available, otherwise everything after start date is assumed on treatment.$  
$CMP: OPV.{"Intervention Type","Off treatment follow up interventions"}$
   $UE: (INTERVENTIONS.xxSTDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC)
        and (DM.TRTEDTM or DM.TRTEDT or DM.RFXENDTC or EX.EXENDTC or DM.RFENDTC))$
   $NOTES: Treatment has to both start and stop to calculate off treatment interventions$
$CVS_STOP$

$CVS_START$
$ID: OnTreatmentFindingsO$
$CMP: OSYS$ $UE: (FINDINGS.xxDTC and (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Findings start date and Treatment start date determine on treatment events, optionally use end dates.$
$CVS_STOP$

$CVS_START$
$ID: PreTreatmentFindingsO$
$CMP: OSYS$ $UE: FINDINGS.xxTRTEM or FINDINGS.TRTEMFL or (FINDINGS.xxDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$
$CVS_STOP$

$CVS_START$
$ID: PreTreatmentEventsO$
$CMP: OSYS$ $UE: EVENTS.xxTRTEM or EVENTS.TRTEMFL or (EVENTS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$$CVS_STOP$

$CVS_START$
$ID: PreTreatmentInterventionsO$
$CMP: OSYS$ $UE: INTERVENTIONS.xxTRTEM or INTERVENTIONS.TRTEMFL or (INTERVENTIONS.xxSTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value NOT Y or Yes for flag variables. All events pre-treated if treatment variables not found$$CVS_STOP$
*/

%macro TREATMENTEMERGENT(dataset = AE, domain = AE, date = , offset = 0, IgnoreFlag = NO);
   %let DateCalcVarSt = ;
   %let DateCalcDataSt =;
   %let DateCalcVarEn =;
   %let TEAEFlag = 0;   
   %global TEAEVARFLAG TEAEVARDOM TEAEVARFLAGs TEAEVARDOMs TEAEVARFLAGe TEAEVARDOMe;
   
   %let TEAEVARFLAG =;
   %let TEAEVARDOM =;
   %let TEAEVARFLAGs =;
   %let TEAEVARDOMs =;
   %let TEAEVARFLAGe =;
   %let TEAEVARDOMe =;
   
   %if &IgnoreFlag = NO %then %do;
      %if %sysfunc(findw(&&&domain.VARS, &domain.TRTEM)) %then %do;
         %let TEAEVARFLAG = &domain.TRTEM;
         %let TEAEVARDOM = &domain;
         data &dataset;
            set &dataset;
            if upcase(&domain.TRTEM) in ("Y","YES") then _TRTEMRG = 1;
            else _TRTEMRG = 0;          
         run;
         %let TEAEFlag = 1;
      %end;
      %else %if %sysfunc(findw(&&&domain.VARS, TRTEMFL)) %then %do;
         %let TEAEVARFLAG = TRTEMFL;
         %let TEAEVARDOM = &domain;
         data &dataset;
            set &dataset;
            if upcase(TRTEMFL) in ("Y","YES") then _TRTEMRG = 1;
            else _TRTEMRG = 0;          
         run;
         %let TEAEFlag = 1;
      %end;
   %end;
   %put TEAEFlag = &TEAEFlag;
   
   *** Start Date ***;
   %if %symexist(ADSLVARS) %then %do;
      %if %sysfunc(findw(&ADSLVARS, TRTSDTM)) %then %do;
         %let DateCalcVarSt = TRTSDTM;
         %let DateCalcDataSt = ADSL;
      %end;
      %else %if %sysfunc(findw(&ADSLVARS, TRTSDT)) %then %do;
         %let DateCalcVarSt = TRTSDT;
         %let DateCalcDataSt = ADSL;
      %end;
   %end;
   %if ^%length(&DateCalcVarSt) %then %do;
      %if %symexist(DMVARS) %then %do;
         %if %sysfunc(findw(&DMVARS, RFXSTDTC)) %then %do;
            %let DateCalcVarSt = RFXSTDTC;
            %let DateCalcDataSt = DM;
         %end;
      %end;
   %end;
   %if ^%length(&DateCalcVarSt) %then %do;
      %if %symexist(EXVARS) %then %do;
         %if %sysfunc(findw(&EXVARS, EXSTDTC)) %then %do;
            %let DateCalcVarSt = EXSTDTC;
            %let DateCalcDataSt = EX;
         %end;
      %end; 
   %end; 
   %if ^%length(&DateCalcVarSt) %then %do;
      %if %symexist(DMVARS) %then %do;
         %if %sysfunc(findw(&DMVARS, RFSTDTC)) %then %do;
            %let DateCalcVarSt = RFSTDTC;
            %let DateCalcDataSt = DM;
         %end;
      %end;
   %end;
   
   %let TEAEVARFLAGs = &DateCalcVarSt;
   %let TEAEVARDOMs = &DateCalcDataSt;

   %if ^&TEAEFlag %then %do;
      %let TEAEVARFLAG = &DateCalcVarSt;
      %let TEAEVARDOM = &DateCalcDataSt;
   %end;

   %if ^&TEAEFlag and ^%length(&DateCalcVarSt) %then %do;
      %put [MS_S_0191];
      %put ERROR: Treatment emergent flags or date variables missing for calculating event type.;
      %put ERROR: &domain.TRTEM, TRTEMFL, TRTSDTM, TRTSDT, RFXSTDTC, EXSTDTC, RFSTDTC are not available.;
      %let exiterror = 1;
   %end;
   
   %if %length(&DateCalcVarSt) %then %do;

      %put DateCalcVarSt = &DateCalcVarSt;
      %put DateCalcDataSt = &DateCalcDataSt;
   
      %if &DateCalcVarSt = EXSTDTC %then %do;
         proc sort data = &in_ex out = exst(keep = usubjid EXSTDTC);
            by usubjid EXSTDTC;
         run;

         data exst;
            set exst;
            by usubjid EXSTDTC;
            if first.usubjid;
         run;        
      %end;

      proc sql noprint;
         create table &dataset.2(drop = usubjid1 usubjid2) as
            select coalesce(aa.usubjid1, bb.usubjid2) as usubjid, aa.*, bb.*
            %if &DateCalcVarSt = EXSTDTC %then %do;
               from &dataset(rename=(usubjid=usubjid1)) as aa left join exst(keep = usubjid &DateCalcVarSt rename=(usubjid=usubjid2)) as bb
            %end;
            %else %do;
               from &dataset(rename=(usubjid=usubjid1)) as aa left join &&in_&DateCalcDataSt(keep = usubjid &DateCalcVarSt rename=(usubjid=usubjid2)) as bb
            %end;
            on aa.usubjid1 = bb.usubjid2 
            order by usubjid, &date;
      quit;

      proc datasets nolist;
         delete &dataset;
         change &dataset.2 = &dataset;
      quit;

      *** End Date ***;
      %if %sysfunc(findw(&ADSLVARS, TRTEDTM)) %then %do;
         %let DateCalcVarEn = TRTEDTM;
         %let DateCalcDataEn = ADSL;
      %end;
      %else %if %sysfunc(findw(&ADSLVARS, TRTEDT)) %then %do;
         %let DateCalcVarEn = TRTEDT;
         %let DateCalcDataEn = ADSL;
      %end;
      %else %if %sysfunc(findw(&DMVARS, RFXENDTC)) %then %do;
         %let DateCalcVarEn = RFXENDTC;
         %let DateCalcDataEn = DM;
      %end;
      %else %if %sysfunc(findw(&EXVARS, EXENDTC)) %then %do;
         %let DateCalcVarEn = EXENDTC;
         %let DateCalcDataEn = EX;
      %end;    
      %else %if %sysfunc(findw(&DMVARS, RFENDTC)) %then %do;
         %let DateCalcVarEn = RFENDTC;
         %let DateCalcDataEn = DM;
      %end;

      %put DateCalcVarEn = &DateCalcVarEn;
      %put DateCalcDataEn = &DateCalcDataEn;

      %let TEAEVARFLAGe = &DateCalcVarEn;
      %let TEAEVARDOMe = &DateCalcDataEn;

      %if %length(&DateCalcVarEn) %then %do;
         %if &DateCalcVarEn = EXENDTC %then %do;
            proc sort data = &in_ex out = exen(keep = usubjid EXENDTC);
               by usubjid EXENDTC;
            run;

            data exen;
               set exen;
               by usubjid EXENDTC;
               if last.usubjid;
            run;        
         %end; 

         proc sql noprint;
            create table &dataset.2(drop = usubjid1 usubjid2) as
               select coalesce(aa.usubjid1, bb.usubjid2) as usubjid, aa.*, bb.*
               %if &DateCalcVarEn = EXENDTC %then %do;
                  from &dataset(rename=(usubjid=usubjid1)) as aa left join exen(keep = usubjid &DateCalcVarEn rename=(usubjid=usubjid2)) as bb
               %end;
               %else %do;
                  from &dataset(rename=(usubjid=usubjid1)) as aa left join &&in_&DateCalcDataEn(keep = usubjid &DateCalcVarEn rename=(usubjid=usubjid2)) as bb
               %end;
               on aa.usubjid1 = bb.usubjid2 
               order by usubjid, &date;
         quit;

         proc datasets nolist;
            delete &dataset;
            change &dataset.2 = &dataset;
         quit;
      %end;

      *** TEAE ***;
      *** only calculate TEAE if TEAE flag was not present ***;
      %if ^&TEAEFlag %then %do;
         data &dataset;
            set &dataset;  
            reftimefl = 0;
            datetimefl = 0;
            _TRTEMRG = 0;
            %if %length(&DateCalcVarSt) %then %do;
               *** if both dates do not have time component use only date part for treatment emergence ***;
               %if &DateCalcVarSt = TRTSDTM %then %do;
                  reftimefl = 1;
               %end;
               %else %if &DateCalcVarSt ne TRTSDT %then %do;
                  if index(&DateCalcVarSt,"T") then reftimefl = 1;
               %end;
               if index(&date,"T") then datetimefl = 1;

               if reftimefl = 1 and datetimefl = 1 then do;
                  %if &DateCalcVarSt = TRTSDTM %then %do;
                     refdate = TRTSDTM;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarSt) = 19 then 
   refdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),substr(&DateCalcVarSt,18,2));
                     else if lengthn(&DateCalcVarSt) = 16 then 
   refdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),"00");
                     else if lengthn(&DateCalcVarSt) = 13 then 
   refdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),"00","00");
                  %end;
                  if lengthn(&Date) = 19 then valdate = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),substr(&Date,18,2));
                  else if lengthn(&Date) = 16 then valdate = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),"00");
                  else if lengthn(&Date) = 13 then valdate = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),"00","00");         
               end;
               else do;
                  %if &DateCalcVarSt = TRTSDTM %then %do;
                     refdate = datepart(TRTSDTM);
                  %end;
                  %else %if &DateCalcVarSt = TRTSDT %then %do;
                     refdate = TRTSDT;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarSt) >= 10 then refdate = mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4));
                     else if lengthn(&DateCalcVarSt) = 7 then refdate = mdy(substr(&DateCalcVarSt, 6, 2), "01", substr(&DateCalcVarSt, 1, 4));
                     else if lengthn(&DateCalcVarSt) = 4 then refdate = mdy("01", "01", substr(&DateCalcVarSt, 1, 4));
                  %end;

                  if lengthn(&Date) >= 10 then valdate = mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4));
                  else if lengthn(&Date) = 7 then valdate = mdy(substr(&Date, 6, 2), "01", substr(&Date, 1, 4));
                  else if lengthn(&Date) = 4 then valdate = mdy("01", "01", substr(&Date, 1, 4));         
               end;

               if refdate > . then do;
                  *** AE and CE will be TE if equal to dose datetime or if date is missing for conservativeness ***;
                  %if &domain = AE or &domain = CE %then %do;
                     if (valdate >= refdate) or missing(valdate) then _TRTEMRG = 1;
                  %end;
                  %else %if &domain = CM %then %do;
                     if (valdate >= refdate) then _TRTEMRG = 1;
                  %end;
                  %else %do;
                     *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                     if (valdate > refdate) then _TRTEMRG = 1;
                  %end;
               end;
               drop refdate valdate;      
            %end;
            drop reftimefl datetimefl;
         run;
      %end;   

      *** OT and OTFU ***;
      data &dataset;
         set &dataset;  
         sttimefl = 0;
         entimefl = 0;
         datetimefl = 0;
         endate = .;
         valdateen = .;
         _OTFLAG = 0;
         _OTFUFLAG = 0;
         *** calculate if start date present ***;
         %if %length(&DateCalcVarSt) %then %do;
            *** will take most informative date for start and for stop separately incase time is present for stop or start ***;
            %if &DateCalcVarSt = TRTSDTM %then %do;
               sttimefl = 1;
            %end;
            %else %if &DateCalcVarSt ne TRTSDT %then %do;
               if index(&DateCalcVarSt,"T") then sttimefl = 1;
            %end;
            %if &DateCalcVarEn = TRTEDTM %then %do;
               entimefl = 1;
            %end;
            %else %if &DateCalcVarEn ne TRTEDT %then %do;
               if index(&DateCalcVarEn,"T") then entimefl = 1;
            %end;       
            if index(&date,"T") then datetimefl = 1;

            *** Start ***;
            if sttimefl = 1 and datetimefl = 1 then do;
               %if &DateCalcVarSt = TRTSDTM %then %do;
                  stdate = TRTSDTM;
               %end;
               %else %do;
                  if lengthn(&DateCalcVarSt) = 19 then 
   stdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),substr(&DateCalcVarSt,18,2));
                  else if lengthn(&DateCalcVarSt) = 16 then 
   stdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),substr(&DateCalcVarSt,15,2),"00");
                  else if lengthn(&DateCalcVarSt) = 13 then 
   stdate = dhms(mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4)),substr(&DateCalcVarSt,12,2),"00","00");
               %end;
               if lengthn(&Date) = 19 then valdatest = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),substr(&Date,18,2));
               else if lengthn(&Date) = 16 then valdatest = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),"00");
               else if lengthn(&Date) = 13 then valdatest = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),"00","00");         
            end;
            else do;
               %if &DateCalcVarSt = TRTSDTM %then %do;
                  stdate = datepart(TRTSDTM);
               %end;
               %else %if &DateCalcVarSt = TRTSDT %then %do;
                  stdate = TRTSDT;
               %end;
               %else %do;
                  if lengthn(&DateCalcVarSt) >= 10 then stdate = mdy(substr(&DateCalcVarSt, 6, 2), substr(&DateCalcVarSt, 9, 2), substr(&DateCalcVarSt, 1, 4));
                  else if lengthn(&DateCalcVarSt) = 7 then stdate = mdy(substr(&DateCalcVarSt, 6, 2), "01", substr(&DateCalcVarSt, 1, 4));
                  else if lengthn(&DateCalcVarSt) = 4 then stdate = mdy("01", "01", substr(&DateCalcVarSt, 1, 4));
               %end;
               if lengthn(&Date) >= 10 then valdatest = mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4));
               else if lengthn(&Date) = 7 then valdatest = mdy(substr(&Date, 6, 2), "01", substr(&Date, 1, 4));
               else if lengthn(&Date) = 4 then valdatest = mdy("01", "01", substr(&Date, 1, 4));         
            end;

            %if %length(&DateCalcVarEn) %then %do;
               *** End ***;
               if entimefl = 1 and datetimefl = 1 then do;
                  %if &DateCalcVarEn = TRTEDTM %then %do;
                     endate = TRTEDTM;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarEn) = 19 then 
   endate = dhms(mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4)),substr(&DateCalcVarEn,12,2),substr(&DateCalcVarEn,15,2),substr(&DateCalcVarEn,18,2));
                     else if lengthn(&DateCalcVarEn) = 16 then 
   endate = dhms(mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4)),substr(&DateCalcVarEn,12,2),substr(&DateCalcVarEn,15,2),"00");
                     else if lengthn(&DateCalcVarEn) = 13 then 
   endate = dhms(mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4)),substr(&DateCalcVarEn,12,2),"00","00");
                  %end;
                  if lengthn(&Date) = 19 then valdateen = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),substr(&Date,18,2));
                  else if lengthn(&Date) = 16 then valdateen = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),substr(&Date,15,2),"00");
                  else if lengthn(&Date) = 13 then valdateen = dhms(mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4)),substr(&Date,12,2),"00","00");         
               end;
               else do;
                  %if &DateCalcVarEn = TRTEDTM %then %do;
                     endate = datepart(TRTEDTM);
                  %end;
                  %else %if &DateCalcVarEn = TRTEDT %then %do;
                     endate = TRTEDT;
                  %end;
                  %else %do;
                     if lengthn(&DateCalcVarEn) >= 10 then endate = mdy(substr(&DateCalcVarEn, 6, 2), substr(&DateCalcVarEn, 9, 2), substr(&DateCalcVarEn, 1, 4));
                     else if lengthn(&DateCalcVarEn) = 7 then endate = mdy(substr(&DateCalcVarEn, 6, 2), "01", substr(&DateCalcVarEn, 1, 4));
                     else if lengthn(&DateCalcVarEn) = 4 then endate = mdy("01", "01", substr(&DateCalcVarEn, 1, 4));
                  %end;
                  if lengthn(&Date) >= 10 then valdateen = mdy(substr(&Date, 6, 2), substr(&Date, 9, 2), substr(&Date, 1, 4));
                  else if lengthn(&Date) = 7 then valdateen = mdy(substr(&Date, 6, 2), "01", substr(&Date, 1, 4));
                  else if lengthn(&Date) = 4 then valdateen = mdy("01", "01", substr(&Date, 1, 4));         
               end;
            %end;

            if stdate > . and missing(endate) then do;
               *** AE and CE will be TE if equal to dose datetime or if date is missing for conservativeness ***;
               %if &domain = AE or &domain = CE %then %do;
                  if (valdatest >= stdate) or missing(valdatest) then _OTFLAG = 1;
               %end;
               %else %if &domain = CM %then %do;
                  if (valdatest >= stdate) then _OTFLAG = 1;
               %end;
               %else %do;
                  *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                  if (valdatest > stdate) then _OTFLAG = 1;
               %end;
            end;
            else if stdate > . and endate > . then do;
               *** if enddate has time then need to convert offset ***; 
               if entimefl = 1 and datetimefl = 1 then do;
                  *** AE and CE will be TE if equal to dose datetime or if date is missing for conservativeness ***;
                  %if &domain = AE or &domain = CE %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+%eval(&offset*86400))) or missing(valdatest) then _OTFLAG = 1;
                     if (valdateen > endate+%eval(&offset*86400)) then _OTFUFLAG = 1;
                  %end;
                  %else %if &domain = CM %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+%eval(&offset*86400))) then _OTFLAG = 1;
                     if (valdateen > endate+%eval(&offset*86400)) then _OTFUFLAG = 1;
                  %end;
                  %else %do;
                     *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                     if ((valdatest > stdate) and (valdateen <= endate+%eval(&offset*86400))) then _OTFLAG = 1;
                     if (valdateen > endate+%eval(&offset*86400)) then _OTFUFLAG = 1;
                  %end;
               end;
               else do;
                  *** AE and CE will be TE if equal to dose datetime or if date is missing for conservativeness ***;
                  %if &domain = AE or &domain = CE %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+&offset)) or missing(valdatest) then _OTFLAG = 1;
                     if (valdateen > endate+&offset) then _OTFUFLAG = 1;
                  %end;
                  %else %if &domain = CM %then %do;
                     if ((valdatest >= stdate) and (valdateen <= endate+&offset)) then _OTFLAG = 1;
                     if (valdateen > endate+&offset) then _OTFUFLAG = 1;
                  %end;
                  %else %do;
                     *** Dosing usually last thing to happen so other things like findings will be assumed measured prior to dosing ***; 
                     if ((valdatest > stdate) and (valdateen <= endate+&offset)) then _OTFLAG = 1;
                     if (valdateen > endate+&offset) then _OTFUFLAG = 1;
                  %end;          
               end;           
            end;
            drop stdate valdatest endate valdateen;
         %end;
         drop entimefl sttimefl datetimefl;
      run;     
   %end;
%mend TREATMENTEMERGENT;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* Assign domain grouping data set macro variable;
%macro GroupAssign(domain);
   %let domain = %lowcase(&domain);
   %global ing_&domain;
   %let ing_&domain = ;
   %if %length(&AInPath) %then %do;
      %if %sysfunc(fileexist("&AInPath./ad&domain.grp.sas7bdat")) %then %do;
         %let ing_&domain = adam.ad&domain.grp;
      %end;
   %end;
   %if ^%length(&&ing_&domain) and %length(&InPath) %then %do;
      %if %sysfunc(fileexist("&InPath./&domain.grp.sas7bdat")) %then %do;
         %let ing_&domain = sdtm.&domain.grp;
      %end;
   %end;
   %put ing_&domain = &&ing_&domain;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/*
$CVS_START$
$ID: TreatmentUse$
$CMP: CPV.{"Treatment or Comparison Variable to Use","Actual"}$
   $UE: DM.TRT01A or DM.ACTARM or DM.TRT01P or DM.ARM$
   $NOTES: Will attempt to use Planned treatment variables if Actual treatment variables not found.
      If ARM is missing, it will be created and set to "Blinded Treatment" so that certain reports
      can still be run.$
$CMP: CPV.{"Treatment or Comparison Variable to Use","Planned"}$
   $UE:DM.TRT01P or DM.ARM$
   $NOTES: If ARM is missing, it will be created and set to "Blinded Treatment" so that certain reports
      can still be run.$
$CVS_STOP$

$CVS_START$
$ID: PopulationFlags$
$CMP: CPV.{"Select the population to include in the analysis","Safety"}$
   $UE: DM.SAFFL or DM.SAFETY$
   $NOTES: SAFETY may be used in DM Supp.$
$CMP: OPV.{"Select the population to include in the analysis","All Subjects Excluding Screen Failures"}$
   $UE: DM.ARM$
   $NOTES: Records with Screen Failure or SCRNFAIL values dropped.  All subjects used if ARM not available$
$CMP: CPV.{"Select the population to include in the analysis","Full Analysis Set"}$
   $UE: DM.FASFL or DM.FULLSET$
   $NOTES: FULLSET may be used in DM Supp.$
$CMP: CPV.{"Select the population to include in the analysis","Intent-to-Treat"}$
   $UE: ADSL.ITTFL or ADSL.ITT$   
$CMP: CPV.{"Select the population to include in the analysis","Per-Protocol"}$
   $UE: DM.PPROTFL or DM.PPROT$
   $NOTES: PPROT may be used in DM Supp.$
$CMP: CPV.{"Select the population to include in the analysis","Completers"}$
   $UE: DM.COMPFL or DM.COMPLETED or DM.COMPLT$
   $NOTES: COMPLETED or COMPLT may be used in DM Supp.$
$CMP: CPV.{"Select the population to include in the analysis","Randomized"}$
   $UE: DM.RANDFL or DM.RANDOMIZED or DM.RANDOM$
   $NOTES: RANDOMIZED or RANDOM may be used in DM Supp.$
$CMP: CPV.{"Select the population to include in the analysis","Enrolled"}$
   $UE: DM.ENRLFL or DM.ENROLLED or DM.ENRL$
   $NOTES: ENROLLED or ENRL may be used in DM Supp.$
$CVS_STOP$
*/
* Assign CDISC parameters as macro variables;
%macro CDISCParms;
   %if &ProcessName ne DefineRiskThresholdDataSet %then %do;
      %global studynameunder;
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));

      %LocaleInit();

      * assign group data sets, to be used later for sorting;
      %let i = 1;
      %do %while(%length(%scan(&findingsdomains,&i,' ')));
         %let temp = %scan(&findingsdomains,&i,' ');
         %GroupAssign(&temp);
         %let i = %eval(&i+1);
      %end;

      * subject label;
      %let _tid_ = %sysfunc(open(&in_adsl));
      %if ^&_tid_ %then %do;
         %put [MS_S_0192];
         %put ERROR: Subject data not found. The study may not exist on the active server.;
         %let exiterror = 1;
         %goto cexit;
      %end;
      %else %do;
         %let usubjvn = %sysfunc(varnum(&_tid_,USUBJID));
         %let armchk = %sysfunc(varnum(&_tid_,ARM));
         %if ^&usubjvn %then %do;
            %put [MS_S_0193];
            %put ERROR: Variable USUBJID not found in adsl data set.;
            %let exiterror = 1;
         %end;
         %else %do;
            %global SubjectLabel;
            %let SubjectLabel = %sysfunc(varlabel(&_tid_,&usubjvn));
            %if ^%length(%trim(&SubjectLabel)) %then %let SubjectLabel = USUBJID;
            %put SubjectLabel = &SubjectLabel;
         %end;

         * determine treatment variable to use;
         %global TrtName TrtLabel;
         %let TrtName = ;
         %let TrtLabel = ;
         %let tvn = 0;
         %if ^%symexist(TrtVar) %then %let TrtVar = ;
         %if %length(&TrtVar) %then %do;
            %let TrtName = &TrtVar;
            %let tvn = %sysfunc(varnum(&_tid_,&TrtName));
         %end;
         %else %do;
            %if ^%symexist(TrtUse) %then %do;
               %global TrtUse;
               %let TrtUse = Actual;
            %end;
            %if (&TrtUse = Actual) %then %do;
               %let TrtName = TRT01A;
               %let tvn = %sysfunc(varnum(&_tid_,&TrtName));
               %if ^&tvn %then %do;
                  %let TrtName = ACTARM;              
                  %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), &TrtName));
               %end;
            %end;
            %if ^&tvn %then %do;
               %if (&TrtUse = Actual) %then %do;
                  %put WARNING: Actual treatment variable not found, so using planned treatment variable.;
               %end;
               %let TrtName = TRT01P;
               %let tvn = %sysfunc(varnum(&_tid_,&TrtName));
               %if ^&tvn %then %do;
                  %let TrtName = ARM;
                  %let tvn = %sysfunc(varnum(&_tid_,&TrtName));
               %end;
            %end;
         %end;

         %let TrtLabel = &TrtName;
         %CheckLabel(&in_adsl,TrtLabel);
         %put TrtName = &TrtName;
         %put TrtLabel = &TrtLabel;

         %let tflag = ;
         %if &TrtName = TRT01A %then %do;
            %let tflag = A;
         %end;
         %else %if &TrtName = TRT01P %then %do;
            %let tflag = P;
         %end;
   /*
   $CVS_START$
   $ID: CrossoverPeriods$
   $CMP: OSYS.Treatment Crossover Detection$
      $UE: (DM.TRT02P or DM.TRT02A) and (DM.TRT02SDTM or DM.TRT02SDT)$
      $NOTES: Creates Treatment and Period variables to use in analysis to support crossover trial. 
      Handles more than 2 periods by presence of TRTxxP or TRTxxA and corresponding period date variables.$
   $CVS_STOP$
   */

         /* Macros to check for Crossover treatment periods */
         %global NTrtPeriods TrtVarList TrtStartDateList MaxTrtVarLen;
         %let NTrtPeriods = 1;
         %if %length(&tflag) %then %do;
            %let tvn = %sysfunc(varnum(&_tid_,&TrtName));
            %let TrtVarList = ;
            %do %while(&tvn);
               %let TrtVarList = &TrtVarList TRT0&NTrtPeriods.&tflag;
               %let NTrtPeriods = %eval(&NTrtPeriods +1);
               %let tvn = %sysfunc(varnum(&_tid_,TRT0&NTrtPeriods.&tflag));
            %end;
            %let NTrtPeriods = %eval(&NtrtPeriods-1);
         %end;
         %else %do;
            %let TrtVarList = &TrtName;
         %end;
         %put Number of Treatment Period Variables = &NTrtPeriods;
         %put Treatment Variables = &TrtVarList;

         %if &NTrtPeriods > 1 %then %do;
            %let NTrtPeriods = 1;

            %Global dtsuffix;
            /* try TR01SDTM */
            %let dtsuffix = ;
            %let trtsdtmnum = %sysfunc(varnum(&_tid_,tr01sdtm));
            %if &trtsdtmnum=0 %then %do;
               /* try TR01SDT */
               %let trtsdtnum = %sysfunc(varnum(&_tid_,tr01sdt));
               %if &trtsdtnum>0 %then %do;
                  %let dtsuffix = sdt;
               %end;
            %end;
            %else %do;
               %let dtsuffix = sdtm;
            %end;

            %if %length(dtsuffix) %then %do;
               %let tvn = %sysfunc(varnum(&_tid_,Tr01&dtsuffix));
               %let TrtStartDateList = ;
               %do %while(&tvn);
                  %let TrtStartDateList = &TrtStartDateList TR0&NTrtPeriods.&dtsuffix;
                  %let NTrtPeriods = %eval(&NTrtPeriods +1);
                  %let tvn = %sysfunc(findw(%upcase(&ADSLVARS), %upcase(TR0&NTrtPeriods.&dtsuffix)));
               %end;
               %let NTrtPeriods = %eval(&NtrtPeriods-1);
            %end;
            %put Number of CrossOver Start Dates = &NTrtPeriods;
            %put Treatment Start Date Variables = &TrtStartDateList;

         %end;
         %put Number of Treatment Period Variables to be Used = &NTrtPeriods;
         /**********************
         If NTrtPeriods is greater than 1 above, we detect a crossover trial and use the 
         start date list and treatment list to create a new conditional treatment variable in 
         stacked time data.  See the %AssignCOTreatment macro below.
         ***********************/

         %let MaxTrtVarlen = 0;
         %do m = 1 %to &NTrtPeriods;
            %let curt = %scan(&TrtVarList,&m," ");
            %let tnum = %sysfunc(varnum(&_tid_,&curt));
            %let curlen = %sysfunc(varlen(&_tid_,&tnum));
            %if &curlen > &MaxTrtVarlen %then %let MaxTrtVarlen = &curlen;
         %end;
         %let _tid_ = %sysfunc(close(&_tid_));
      %end;

      * remove screen failures and patients not in selected populations;

      %let Indicators = FAS SAF ITT PPROT COMPL RAND ENRL;

      /*initialize all flags to no */
      %let IncSF = No;
      %do i = 1 %to 7;
         %let currind = %scan(&Indicators,&i);
         %let Inc&currind = No;
      %end;

      %let trFull = %nrbquote(%LocaleGet(Full_Text));
      %let trFullAnalysisSet = %nrbquote(%LocaleGet(FullAnalysisSet_Text));
      %let trSafety = %nrbquote(%LocaleGet(Safety_Text));
      %let trIntent = %nrbquote(%LocaleGet(Intent_Text));
      %let trIntenttoTreat = %nrbquote(%LocaleGet(IntenttoTreat_Text));
      %let trProtocol = %nrbquote(%LocaleGet(Protocol_Text));
      %let trPerProtocol = %nrbquote(%LocaleGet(PerProtocol_Text));
      %let trComplete = %nrbquote(%LocaleGet(Complete_Text));
      %let trCompleters = %nrbquote(%LocaleGet(Completers_Text));
      %let trRandom = %nrbquote(%LocaleGet(Random_Text));
      %let trRandomized = %nrbquote(%LocaleGet(Randomized_Text));
      %let trEnroll = %nrbquote(%LocaleGet(Enroll_Text));
      %let trEnrolled = %nrbquote(%LocaleGet(Enrolled_Text));
      %let trAllSubjects = %nrbquote(%LocaleGet(AllSubjects_Text));

      %if %kindex(%quote(%kupcase(&PopulationSet)),FULL) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trFull))) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trFullAnalysisSet))) %then %do;
         %let IncFAS = Yes;
      %end;
      %else %if %kindex(%quote(%kupcase(&PopulationSet)),SAFETY) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trSafety))) %then %do;
         %let IncSAF = Yes;
      %end;
      %else %if %kindex(%quote(%kupcase(&PopulationSet)),INTENT) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trIntent))) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trIntenttoTreat))) %then %do;
         %let IncITT = Yes;
      %end;
      %else %if %kindex(%quote(%kupcase(&PopulationSet)),PROTOCOL) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trProtocol))) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trPerProtocol))) %then %do;
         %let IncPPROT = Yes;
      %end;
      %else %if %kindex(%quote(%kupcase(&PopulationSet)),COMPLETE) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trComplete))) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trCompleters))) %then %do;
         %let IncCOMPL = Yes;
      %end; 
      %else %if %kindex(%quote(%kupcase(&PopulationSet)),RANDOM) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trRandom))) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trRandomized))) %then %do;
         %let IncRAND = Yes;
      %end; 
      %else %if %kindex(%quote(%kupcase(&PopulationSet)),ENROLL) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trEnroll))) or
         %kindex(%quote(%kupcase(&PopulationSet)),%quote(%kupcase(&trEnrolled))) %then %do;
         %let IncENRL = Yes;
      %end; 
      %else %if %quote(%kupcase(&PopulationSet)) = %quote(%kupcase(All Subjects)) or
         %quote(%kupcase(&PopulationSet)) = %quote(%kupcase(&trAllSubjects)) %then %do;
         %let IncSF = Yes;
      %end;

      %let i=1;
      %let YesIndicators=;
      %let yeses = 0;
      %let adslid = %sysfunc(open(&in_adsl));
      %do i=1 %to 7;
         %let currind = %scan(&Indicators,&i);
         %if &&Inc&currind = Yes %then %do;
            %let currindvar = %sysfunc(varnum(&adslid,&currind.FL));
            /* May encounter FULLSET in place of FASFL.  Might also have to check DM for FULLSET (Supplemental Qualifiers) */
            %if &currindvar=0 and &currind=FAS %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,FULLSET));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators FULLSET;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %put WARNING: The variables FASFL and FULLSET were not found in the adsl data set.;    
            %end;
            /* May encounter ENRL or ENROLLED in place of ENRLFL.  Might also have to check DM for ENRL (Supplemental Qualifiers) */
            %else %if &currindvar=0 and &currind=ENRL %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,ENROLLED));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators ENROLLED;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %do;
                  %let currindvar = %sysfunc(varnum(&adslid,ENRL));
                  %if &currindvar %then %do;
                     %let YesIndicators = &YesIndicators ENRL;
                     %let yeses = %eval(&yeses + 1);
                  %end;    
                  %else %put WARNING: The variables ENRLFL, ENROLLED, and ENRL were not found in the adsl data set.;    
               %end;
            %end;
            /* May encounter PPROT in place of PPROTFL.  Might also have to check DM for PPROT (Supplemental Qualifiers) */
            %else %if &currindvar=0 and &currind=PPROT %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,PPROT));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators PPROT;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %put WARNING: The variables PPROTFL and PPROT were not found in the adsl data set.;    
            %end;
            /* May encounter SAFETY in place of SAFFL.  Might also have to check DM for SAFETY (Supplemental Qualifiers) */
            %else %if &currindvar=0 and &currind=SAF %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,SAFETY));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators SAFETY;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %put WARNING: The variables SAFFL and SAFETY were not found in the adsl data set.;     
            %end;
            /* May encounter ITT in place of ITTFL.  */
            %else %if &currindvar=0 and &currind=ITT %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,ITT));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators ITT;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %put WARNING: The variables ITTFL and ITT were not found in the adsl data set.;     
            %end;    
            /* May encounter RANDOM or RANDOMIZED in place of RANDFL.  */
            %else %if &currindvar=0 and &currind=RAND %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,RANDOMIZED));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators RANDOMIZED;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %do;
                  %let currindvar = %sysfunc(varnum(&adslid,RANDOM));
                  %if &currindvar %then %do;
                     %let YesIndicators = &YesIndicators RANDOM;
                     %let yeses = %eval(&yeses + 1);
                  %end;    
                  %else %put WARNING: The variables RANDFL, RANDOM, and RANDOMIZED were not found in the adsl data set.;   
               %end;
            %end;    
            /* May encounter COMPLETED or COMPLT in place of COMPLFL.  */
            %else %if &currindvar=0 and &currind=COMPL %then %do;
               %let currindvar = %sysfunc(varnum(&adslid,COMPLETED));
               %if &currindvar %then %do;
                  %let YesIndicators = &YesIndicators COMPLETED;
                  %let yeses = %eval(&yeses + 1);
               %end;    
               %else %do;
                  %let currindvar = %sysfunc(varnum(&adslid,COMPLT));
                  %if &currindvar %then %do;
                     %let YesIndicators = &YesIndicators COMPLT;
                     %let yeses = %eval(&yeses + 1);
                  %end;    
                  %else %put WARNING: The variables COMPLFL, COMPLETED, and COMPLT were not found in the adsl data set.;      
               %end;
            %end;                
            %else %if &currindvar %then %do;
               %let YesIndicators = &YesIndicators &currind.FL;
               %let yeses = %eval(&yeses + 1);
            %end;
            %else %put WARNING: The variable &currind.FL was not found in the adsl data set.;
         %end;
      %end; 

      %let popval =;
      %do i=1 %to &yeses;
         %if  %sysfunc(vartype(&adslid,%sysfunc(varnum(&adslid,%scan(&YesIndicators,&i)))))=N %then %let popval = %bquote(&popval 1);
         %else %Let popval=%bquote(&popval "Y");
      %end; 

      %let adslid = %sysfunc(close(&adslid));
      %put adslid = &adslid;

      %if %symexist(WhereStmt)=0 %then %let WhereStmt=;
      %if %symexist(SubjectFilter)=0 %then %let SubjectFilter=;
      %put IncSF=&IncSF YesIndicators=&YesIndicators WhereStmt=&WhereStmt SubjectFilter=&SubjectFilter;
      %if (&IncSF = No) or %length(&YesIndicators) or %length(&WhereStmt) or %length(&SubjectFilter) %then %do;

         %if %length(&SubjectFilter) %then %do;

            %if &Server = Local %then %do;
               %let sfpath = %jg_getPath(/user/clinical/filters,,0,0,1,&studynameunder);
            %end;
            %else %do;
               %let sfpath = %jg_getPath(/user/clinical,,0,0,2,&studynameunder);
            %end;

            * look up filter data set name;
            %if %sysfunc(fileexist(&sfpath./subjectfilters.sas7bdat)) %then %do;

               libname sf "&sfpath";

               %let filterds = ;
               data _null_;
                  set sf.subjectfilters;
                  if FilterName = "&SubjectFilter" then call symputx("filterds",FilterDataSet);
               run;

               * apply the filter;
               %if %length(&filterds) and %sysfunc(fileexist(&sfpath./&filterds..sas7bdat)) %then %do;

                  %if ^%symexist(sfLib) %then %do;
                     %let sfLib = sf;
                  %end;
                  proc sql;
                     create table adslfilt as
                     select coalesce(firstdat.usubjid, seconddat.usubjid) as usubjid, firstdat.*, seconddat.*
                     from &sfLib..&filterds(keep = usubjid) as firstdat left join &in_adsl as seconddat
                     on firstdat.usubjid = seconddat.usubjid 
                     order by usubjid;
                  quit;                

                  %let in_adsl = work.adsl;
                  data &in_adsl;
                     set adslfilt;
                     label usubjid="&SubjectLabel";
                  run;

                  proc datasets nolist;
                     delete adslfilt;
                  quit;
                  %put updated in_adsl = &in_adsl;
               %end;
               %else %do;
                  %put [MS_S_0194];
                  %put ERROR: Subject filter &SubjectFilter does not exist for study &StudyName.. Either recreate it or apply a different filter.;
                  %let exiterror=1;
               %end;
            %end;
            %else %do;
               %put [MS_S_0195];
               %put ERROR: Study &StudyName does not have any subject filters defined. Use the Create Subject Filter drill-down button to create them.;
               %let exiterror=1;
            %end;       
         %end;

         %put Removing screen failures by subsetting adsl;

         %let trScreenFailure = %nrbquote(%LocaleGet(ScreenFailure_Text));

         data adsl;
            set &in_adsl;
            %if %length(&WhereStmt) %then %do;
               where &WhereStmt;
            %end;
            %if (&IncSF = No) %then %do;
               %if &armchk %then %do;
                  if upcase(arm) = "SCREEN FAILURE" or upcase(arm) = "SCRNFAIL" or upcase(arm) = upcase("&trScreenFailure") or missing(arm) then delete;
               %end;
               %if %length(&TrtName) %then %do;
                  if upcase(&TrtName) = "SCREEN FAILURE" or upcase(&TrtName) = "SCRNFAIL" or upcase(arm) = upcase("&trScreenFailure") or missing(&TrtName) then delete;
               %end;   
            %end;
            %do i=1 %to &yeses;
               if %scan(&YesIndicators,&i) ne %scan(&popval,&i) then delete;
            %end; 
         run;
         %if &syserr>4 %then %let exiterror=1;

         %Nobsvars(adsl);
         %if &nobs=0 %then %do;
            %put [MS_S_0196];
            %put ERROR: No subjects satisfy the Filter to Include Subjects or are in all populations selected on the Population tab.;
            %let exiterror=1;
         %end;

         %let in_adsl = work.adsl;
      %end; 
      %cexit:;
   %end;
%mend;
* Run this macro since all processes need it;
%macro RunCDISCParms;
   %if ^%symexist(runCDISCParmsFlag) %then %let runCDISCParmsFlag=0;
   %if ^&runCDISCParmsFlag %then %do;
      %CDISCParms;
   %end;
%mend RunCDISCParms;
%RunCDISCParms;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;
/* Macro to use in data step that will create a new treatment variable to incorporate crossover */
%macro AssignCOTreatment(Indat, OutDat, DTvar);
   %put Assigning crossover treatment;
   %put Indat=&Indat Outdat=&Outdat DTvar=&DTvar TrtStartDateList=&TrtStartDateList TrtVarList=&TrtVarList dtsuffix=&dtsuffix;
   %Global TrtNameP TrtLabelP;
   %let TrtNameP = ;
   %let TrtLabelP = ;
   data &OutDat;
      merge &Indat(in=a) &in_adsl(keep=USUBJID &TrtVarList &TrtStartDateList);
      by USUBJID;
      %CHKDATE(date = &DTVar,suffix = CO);
      array trtdtc{*} &TrtStartDateList;
      array trts{*} &TrtVarList;
      length Trt $ &MaxTrtVarLen TrtPeriod $ %eval(&MaxTrtVarLen + 5);
      label Trt = "Treatment";
      label TrtPeriod = "Treatment (Period)";
      *Trt = "None";
      if not missing(&DTVar) and (missing(errdtCO) or lengthn(trim(left(errdtCO))) = 0) then do;
         if lengthn(&DTVar) = 19 then do;
            DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),substr(&DTvar,12,2),substr(&DTvar,15,2),substr(&DTvar,18,2));
         end;
         else if lengthn(&DTVar) = 16 then do;
            DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),substr(&DTvar,12,2),substr(&DTvar,15,2),"00");
         end;
         else if lengthn(&DTVar) = 13 then do;
            DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),substr(&DTvar,12,2),"00","00");
         end;
         else if lengthn(&DTVar) = 10 then do;
            DTCvar = dhms(mdy(substr(&DTvar, 6, 2), substr(&DTvar, 9, 2), substr(&DTvar, 1, 4)),"00","00","00");
         end;
         else if lengthn(&DTVar) = 7 then do;
            DTCvar = dhms(mdy(substr(&DTvar, 6, 2), "01", substr(&DTvar, 1, 4)),"00","00","00");
         end;  
         else if lengthn(&DTVar) = 4 then do;
            DTCvar = dhms(mdy("01", "01", substr(&DTvar, 1, 4)),"00","00","00");
         end;        
         do i = &NTrtPeriods to 1 by -1;
            %if &dtsuffix = sdtm %then %do;
               if not missing(trtdtc[i]) and DTCvar >= trtdtc[i] then do;
            %end;
            %else %do;
               if DTCvar >= dhms(trtdtc[i],"00","00","00") then do;
            %end;
               Trt = trts[i];
               Period = i;
               TrtPeriod = trim(left(Trt))||" ("||trim(left(Period))||")";
               i = 0;
            end;
         end;
      end;
      if a;
      drop i;
   run;
   
   /*
   data outlib.co;
      set &outdat;
   run;
   */
   
   %let TrtName = Trt;
   %let TrtLabel = Treatment;
   %let TrtNameP = TrtPeriod;
   %let TrtLabelP = Treatment (Period);
   %put TrtName = &TrtName;
   %put TrtLabel = &TrtLabel;
%mend AssignCOTreatment;
   

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* Define new variables &domain.GRP, &domain.GRPN (numeric group index) and &domain.SGRPN (numeric group subindex) in order to meaningfully sort values;
* Useful for Findings Domains, such as LB, VS, ECG;
%macro GroupVars(domain,indat,outdat);

   %if %length(&&ing_&domain) %then %do;

      %put Creating group variables for ing_&domain = &&ing_&domain;
   
      * max number of tags;
      %NObsVars(&&ing_&domain);
      %let nsg = %eval(&nvars-1);
      %let ng = &nobs;
      
      * check for vspos;
      %if %upcase(&domain)=VS %then %do;
         %let gposv = &domain.POS;
         %let _ds = %sysfunc(open(&indat));
         %let gpos = %sysfunc(varnum(&_ds,&gposv));
         %let _ds =%sysfunc(close(&_ds));
         %let nsg = %eval(&nsg/2);
         %if ^&gpos %then %let gposv = ;
      %end;
      %else %do;
         %let gposv = ;
         %let gpos = 0;
      %end;
      
      * max number of subgroups;
      %if ^%symexist(maxnsg) %then %do;
         %global maxnsg;
         %let maxnsg = 0;
      %end;

      * assign macros from the group definition table;
      data _null_;
         set &&ing_&domain end=_last;
         retain maxglen 0;
         call symput("g"||compress(_n_),%upcase(&domain.GRP));
         maxglen = max(maxglen,length(&domain.GRP));
         %do i=1 %to &nsg;
            if lengthn(&domain.TESTC&i) then do;
               call symput("g"||compress(_n_)||"_&i",%upcase(&domain.TESTC&i));
               call symput("ng"||compress(_n_),compress(&i));
               %if &gpos %then %do;
                  call symput("gp"||compress(_n_)||"_&i",%upcase(&domain.POS&i));
               %end;
               if &i > &maxnsg then call symput("maxnsg",compress(&i));
            end;
         %end;
         if _last then do;
            call symput("maxglen",compress(maxglen));
         end;
      run;
      
      * assign nonmatching values to their own group;
      data temp;
         set &indat;
         keep &domain.TESTCD &gposv;
      run;
      proc sort data=temp nodupkey;
         by &domain.TESTCD &gposv;
      run;
      
      data _null_;
         set temp end=_last;
         retain gx &ng;
         * determine if the current value of &domain.TESTCD matches any of the group tags;
         match = 0;
         %do g=1 %to &ng;
            %do i=1 %to &&ng&g;
               %if %length(%str(&&g&g._&i)) %then %do;
                  %if &gpos %then %do;
                     if not match and upcase(&domain.TESTCD)="&&g&g._&i" and upcase(&domain.POS)="&&gp&g._&i" then do;
                        match = 1;
                     end;                 
                  %end;
                  %else %do;
                     if not match and upcase(&domain.TESTCD)="&&g&g._&i" then do;
                        match = 1;
                     end;
                  %end;
               %end;
            %end;
         %end;
         * if no match then put the value into its own group at the end of the list;
         if not match then do;
            gx+1;
            call symput("ng"||compress(gx),compress(1));
            %if &gpos %then %do;
               call symput("g"||compress(gx),
                  cat(trim(left(&domain.TESTCD))," ",trim(left(&domain.POS)))
               );
               call symput("gp"||compress(gx)||"_1",trim(left(&domain.POS)));
            %end;
            %else %do;
               call symput("g"||compress(gx),trim(left(&domain.TESTCD)));
            %end;
            call symput("g"||compress(gx)||"_1",trim(left(&domain.TESTCD)));
         end;
         if _last then do;
            call symput("ng",compress(gx));
         end;
      run;
                  
      %put ng = &ng;
      %do g=1 %to &ng;
         %put g&g = &&g&g;
         %do i=1 %to &&ng&g;
            %put g&g._&i = &&g&g._&i;
            %if &gpos %then %do;
               %put gp&g._&i = &&g&g._&i;
            %end;
         %end;
      %end;
      %put maxglen = &maxglen;
      
      * define new variables based on the macro variables created above;
      data &outdat;
         set &indat;
         retain gx &ng;
         length %upcase(&domain.GRP) $ &maxglen;
         * determine matching tag;
         match = 0;
         %do g=1 %to &ng;
            %do i=1 %to &&ng&g;
               %if &gpos %then %do;
                  if not match and upcase(&domain.TESTCD)="&&g&g._&i" and upcase(&domain.POS)="&&gp&g._&i" then do;
                     %upcase(&domain.GRP) = "&&g&g";
                     %upcase(&domain.GRPN) = &g;
                     %upcase(&domain.SGRPN) = &i;
                     match = 1;
                  end;
               %end;
               %else %do;
                  if not match and upcase(&domain.TESTCD)="&&g&g._&i" then do;
                     %upcase(&domain.GRP) = "&&g&g";
                     %upcase(&domain.GRPN) = &g;
                     %upcase(&domain.SGRPN) = &i;
                     match = 1;
                  end;              
               %end;
            %end;
         %end;
         drop match gx;
      run;
         
   %end;
   %else %do;
      %put No grouping data set found for domain &domain.;
      data &outdat;
         set &indat;
      run;
   %end;
   %gexit:;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*** CHKDATE examines the validity of ISO8601 Dates;
%macro CHKDATE(date = , suffix = );
   %global DateErrFlg;
   errdt&suffix = " ";
   call symput("DateErrFlg", "N");
   if lengthn(compress(&date)) = 0 then errdt&suffix = " ";
   else if lengthn(&date) in (4,7,10,13,16,19) then do;
      if substr(&date,1,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if substr(&date,2,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if substr(&date,3,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if substr(&date,4,1) not in ("1","2","3","4","5","6","7","8","9","0") then do;
         errdt&suffix = "Y";
         call symput("DateErrFlg", "Y");
      end;
      else if lengthn(&date) >= 7 then do;
         if substr(&date,5,1) ne "-" then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,6,2) not in ("01","02","03","04","05","06","07","08","09","10","11","12") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
      end;
      else if lengthn(&date) >= 10 then do;
         if substr(&date,8,1) ne "-" then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,9,2) not in ("01","02","03","04","05","06","07","08","09","10","11","12",
                                           "13","14","15","16","17","18","19","20","21","22","23","24",
                                           "25","26","28","29","30","31") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
      end;
      else if lengthn(&date) >= 13 then do;
         if substr(&date,11,1) ne "T" then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,12,2) not in ("00","01","02","03","04","05","06","07","08","09","10","11",
                                            "12","13","14","15","16","17","18","19","20","21","22","23") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
      end;
      else if lengthn(&date) >= 16 then do;
         if substr(&date,14,1) ne ":" then do;
            call symput("DateErrFlg", "Y");
            errdt&suffix = "Y";
         end;
         else if substr(&date,15,2) not in ("00","01","02","03","04","05","06","07","08","09","10","11",
                                            "12","13","14","15","16","17","18","19","20","21","22","23",
                                            "24","25","26","27","28","29","30","31","32","33","34","35",
                                            "36","37","38","39","40","41","42","43","44","45","46","47",
                                            "48","49","50","51","52","53","54","55","56","57","58","59") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
     end;
     else if lengthn(&date) = 19 then do;
         if substr(&date,17,1) ne ":" then do;
           errdt&suffix = "Y";
           call symput("DateErrFlg", "Y");
         end;
         else if substr(&date,18,2) not in ("00","01","02","03","04","05","06","07","08","09","10","11",
                                            "12","13","14","15","16","17","18","19","20","21","22","23",
                                            "24","25","26","27","28","29","30","31","32","33","34","35",
                                            "36","37","38","39","40","41","42","43","44","45","46","47",
                                            "48","49","50","51","52","53","54","55","56","57","58","59") then do;
            errdt&suffix = "Y";
            call symput("DateErrFlg", "Y");
         end;
     end;
   end;
   else do;
      errdt&suffix = "Y"; 
      call symput("DateErrFlg", "Y");
   end;
%if &DateErrFlg = Y %then %put WARNING: %upcase(&dtcdate) has at least one instance that does not comply to ISO 8601 standards.;
%mend CHKDATE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;
/*Blocks to specify variables used for relative study day calulcations */
/*
$CVS_START$
$ID: CalcStudyDay_Findings$
$CMP: RSYS.Study Day Calculation$ $UE: FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_SV$
$CMP: RSYS.Study Day Calculation$ $UE: SV.SVSTDY or (SV.SVSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_SVO$
$CMP: OSYS.Study Day Calculation$ $UE: SV.SVSTDY or (SV.SVSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_FindingsV$
$CMP: RSYS.Findings Timing Calculation$ $UE: FINDINGS.VISIT or FINDINGS.VISITNUM or FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Findings Timing: Use of Visits or Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_FindingsO$
$CMP: OSYS.Findings Timing Calculation$ $UE: FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$ 
   $NOTES: Findings Timing: Calculate relative Study Day variable STDY. Partial date imputes to first moment.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_Interventions$
$CMP: RSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxSTDY or (INTERVENTIONS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment.$
$CMP: OSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxENDY or INTERVENTIONS.xxDUR or (INTERVENTIONS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_InterventionsO$
$CMP: OSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxSTDY or (INTERVENTIONS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment. Missing dates set STDY = 0$
$CMP: OSYS.Study Day Calculation$ $UE: INTERVENTIONS.xxENDY or INTERVENTIONS.xxDUR or (INTERVENTIONS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_Events$
$CMP: RSYS.Study Day Calculation$ $UE: EVENTS.xxSTDY or (EVENTS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment.$
$CMP: OSYS.Study Day Calculation$ $UE: EVENTS.xxENDY or EVENTS.xxDUR or (EVENTS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

$CVS_START$
$ID: CalcStudyDay_EventsO$
$CMP: OSYS.Study Day Calculation$ $UE: EVENTS.xxSTDY or (EVENTS.xxSTDTC and DM.RFSTDTC)$ 
   $NOTES: Calculate relative Start Study Day variable STDY. Partial date imputes to first moment.$
$CMP: OSYS.Study Day Calculation$ $UE: EVENTS.xxENDY or EVENTS.xxDUR or (EVENTS.xxENDTC and DM.RFSTDTC)$ 
   $NOTES: If ENDY cannot be calculated, set to STDY.$
$CVS_STOP$

*/
%macro calculateStudyDays(domain);
   %let domain = %upcase(&domain);
   %let useboth = 0;
   %if &domain=DS %then %do;
      %let daysuffix = STDY;
      %let timesuffix = STDTC;
      %let useboth = 1;
   %end;
   %else %if &domain=PP %then %do;
      %let daysuffix = DY;
      %let timesuffix = RFTDTC;
   %end;
   %else %if %sysfunc(findw(&findingsdomains, &domain)) %then %do;
      %let daysuffix = DY;
      %let timesuffix = DTC;
      %if &domain=MH %then %do;
         %let useboth = 1;
      %end;
   %end;
   %else %do;
      %let daysuffix = STDY;
      %let timesuffix = STDTC;
   %end;
   
   STDY = 0;
   if not missing(&domain.&daysuffix) then STDY = &domain.&daysuffix;
   else do;
      %calcStudyDay(&domain.&timesuffix,STDY);
      %if &useboth %then %do;
         if missing(STDY) then do;
            %calcStudyDay(&domain.STDTC,STDY);
         end;
      %end;
      &domain.&daysuffix = STDY;
   end;
   
   %if ^%sysfunc(findw(&findingsdomains, &domain)) or ^%sysfunc(findw(&&&domain.VARS, &domain.STRESN)) %then %do;
      ENDY = 0;
      if not missing(&domain.ENDY) then ENDY = &domain.ENDY;
      /* else if not missing(&domain.DUR) then ENDY = STDY + &domain.DUR;*/
      else do;
         %calcStudyDay(&domain.ENDTC,ENDY);
      end;
      * if missing(ENDY) then ENDY = STDY;
      if missing(&domain.DUR) and not missing(ENDY) then &domain.DUR = ENDY-STDY;
      LOG2_&domain.DUR = log2(max(1,&domain.DUR));    
   %end;
   if not missing(ENDY) and STDY > ENDY then STDY = ENDY - 1;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/**
Calculate Study Day
Directly from the SDTM IG
--DY = (date portion of --DTC) - (date portion of RFSTDTC) + 1 
if --DTC is on or after RFSTDTC --DY = (date portion of --DTC) - (date portion of RFSTDTC) 
if --DTC precedes RFSTDTC This algorithm should be used across all domains.
**/
%macro calcStudyDay(dtcdate,result);
   %CHKDATE(date = &dtcdate, suffix = &result);
   if not missing(&dtcdate) and not missing(rfstdtc) and errdt&result = " " then do;
      if find(&dtcdate,'T') then do;
         _cdy = mdy(substr(&dtcdate, 6, 2), substr(&dtcdate, 9, 2), substr(&dtcdate, 1, 4));
         partial_&dtcdate = 0;
      end;
      else if lengthn(scan(&dtcdate,3,'-')) then do;
         _cdy = mdy(substr(&dtcdate, 6, 2), substr(&dtcdate, 9, 2), substr(&dtcdate, 1, 4));
         partial_&dtcdate = 0;
      end;
      else if lengthn(scan(&dtcdate,2,'-')) then do;
         _cdy = mdy(substr(&dtcdate, 6, 2), 1, substr(&dtcdate, 1, 4));
         partial_&dtcdate = 1;
      end;
      else do;
         _cdy = mdy(1, 1, substr(&dtcdate, 1, 4));
         partial_&dtcdate = 1;
      end;
      if find(rfstdtc,'T') then do;
         _rdy = mdy(substr(rfstdtc, 6, 2), substr(rfstdtc, 9, 2), substr(rfstdtc, 1, 4));
      end;
      else if lengthn(scan(rfstdtc,3,'-')) then do;
         _rdy = mdy(substr(rfstdtc, 6, 2), substr(rfstdtc, 9, 2), substr(rfstdtc, 1, 4));
      end;
      else if lengthn(scan(rfstdtc,2,'-')) then do;
         _rdy = mdy(substr(rfstdtc, 6, 2), 1, substr(rfstdtc, 1, 4));
      end;
      else do;
         _rdy = mdy(1, 1, substr(rfstdtc, 1, 4));
      end;
      if _cdy >= _rdy then &result = _cdy - _rdy + 1;
      else &result = _cdy - _rdy;
      drop _cdy _rdy;
   end;
   else do;
      %if %index(%upcase(&result),STDY) %then %do;
         &result = 0;
      %end;
      %else %do;
         &result = .;
      %end;
      partial_&dtcdate = .;
   end;
***if errdt&result = "Y" then delete;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* create numeric days variable from iso 8601 variable;
%macro DATE9(var=,return=);
   if length(&var) = 4 then do;
      &return = mdy(1, 1, substr(&var, 1, 4));
      d9partial_&var = 1;
   end;
   else if length(&var) = 7 then do;
      &return = mdy(substr(&var, 6, 2), 1, substr(&var, 1, 4));
      d9partial_&var = 1;
   end;
   else if missing(&var) then do;
      &return = .;
      d9partial_&var = 1;
   end;
   else do;
      &return = mdy(substr(&var, 6, 2), substr(&var, 9, 2), substr(&var, 1, 4));
      d9partial_&var = 0;
   end;
   format &return &IntDateFormat..;
%mend DATE9;


*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* create numeric seconds variable from iso 6801 variable;
%macro TIME5(var=,return=);
   if index(&var,"T") then do;
      if lengthn(&var) = 19 then do;
         &return = hms(substr(&var, 12, 2), substr(&var, 15, 2), substr(&var, 18, 2));
      end;
      else if lengthn(&var) = 16 then do;
         &return = hms(substr(&var, 12, 2), substr(&var, 15, 2), 0);
      end;
      else if lengthn(&var) = 13 then do;
         &return = hms(substr(&var, 12, 2), 0, 0);
      end;
   end;
   else do;
      &return = .;
   end;
   format &return time5.;
%mend TIME5;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* add time window string to CatName and CatLabel if specified and/or create time window variable;
%macro AddTimeWindows(domain=,indat=,outdat=,twb=,tvar=,findingscall=0,blflag=0,errflag=1);

   %if &indat ne &outdat %then %do; 
      data &outdat;
         set &indat;
      run;
   %end;
   
   %if ^%symexist(CatName) %then %do;
      %global CatName;
      %let CatName = ;
   %end;
   %if ^%symexist(CatLabel) %then %do;
      %global CatLabel;
      %let CatLabel = &CatName;
   %end;
   %if ^%symexist(GroupVar) %then %do;
      %global GroupVar;
      %let GroupVar = ;
   %end;
   
      
   %global ntw maxtw CatKeepVars;
   %if %length(&twb) %then %do;
      %if ^&findingscall %then %do;
         data dm;
            set &in_dm;
            keep USUBJID rfstdtc;
         run;

         proc sort data=&outdat;
            by USUBJID;
         run;

         proc sort data=dm;
            by USUBJID;
         run;

         data &outdat;
            merge &outdat dm;
            by USUBJID;
            %if %length(&CatName) %then %do;
               if missing(&CatName) then delete;
            %end;
            %calculateStudyDays(&domain);
            if missing(ENDY) then ENDY = STDY;
            %if %Upcase(&TimeScale) = WEEKS %then %do;
               STWK = ceil(STDY/7);
               ENWK = ceil(ENDY/7);
               %let STvar = STWK;
               %let ENvar = ENWK;
            %end;
            %else %do;  
               %let STvar = STDY;
               %let ENvar = ENDY;
            %end;
         run;
      %end;

      * get time window boundary levels;
      %let i=1;
      %let maxtw = 0;
      %if %index(&twb,%str(])) %then %do;
         %do %while(%length(%scan(&twb,&i,']')));
            %global tw&i tws&i twe&i;
            %let tw&i = %qscan(&twb,&i,']')%str(]);
            %if %index(&&tw&i,%str(,)) %then %do;
               %let tws&i = %scan(&&tw&i,1,'[, ]');
               %let twe&i = %scan(&&tw&i,2,'[, ]');
               %let twv&i = 2;
            %end;
            %else %do;
               %let tws&i = %scan(&&tw&i,1,'[ ]');
               %let twe&i = %scan(&&tw&i,1,'[ ]');
               %let twv&i = 1;
            %end;
            %if %length(&&tw&i) > &maxtw %then %let maxtw = %length(&&tw&i);
            %let i=%eval(&i+1);
         %end;
      %end;
      %else %do;
         %do %while(%length(%scan(&twb,&i,' ')));
            %if %index(&twb,%str(,)) %then %do;
               %let exiterror=1;
               %put [MS_S_0089];
               %put ERROR: Commas must be used with brackets when specifying time windows.;
               %goto twexit;
            %end;
            %global tw&i tws&i twe&i;
            %let tw&i = %qscan(&twb,&i,' ');
            %let tws&i =%scan(&twb,&i,' ');
            %let twe&i =%scan(&twb,&i,' ');
            %let twv&i = 1;
            %if %length(&&tw&i) > &maxtw %then %let maxtw = %length(&&tw&i);
            %let i=%eval(&i+1);
         %end;
      %end; 
      %let ntw = %eval(&i-1);
      %put ntw = &ntw;
      %do i=1 %to &ntw;
         %let tw&i = %trim(&&tw&i);
         %put tw&i = &&tw&i;
         /*
         %put tws&i = &&tws&i;
         %put twe&i = &&twe&i;
         %put twv&i = &&twv&i;
         */
      %end;

      * get lengths;
      %if %length(&CatName) %then %do;
         %let _tid_ = %sysfunc(open(&outdat));
         %let vn = %sysfunc(varnum(&_tid_,&CatName));
         %let CatLen = %sysfunc(varlength(&_tid_,&vn));
         %let _tid_ = %sysfunc(close(&_tid_));
         %let CatLabelTW = &CatLabel with Time Window;
      %end;
      %if %length(&GroupVar) %then %do;
         %let _tid_ = %sysfunc(open(&outdat));
         %let vn = %sysfunc(varnum(&_tid_,&GroupVar));
         %let GroupLen = %sysfunc(varlength(&_tid_,&vn));
         %let GroupLabel = %sysfunc(varlabel(&_tid_,&vn));
         %let _tid_ = %sysfunc(close(&_tid_));
         %if ^%length(&GroupLabel) %then %let GroupLabel = &GroupVar;
         %let GroupLabelTW = &GroupLabel with Time Window;
      %end;

      * create a time variable and/or add time window strings to catname;
      data &outdat;
         %if %length(&CatName) %then %do;
            length &CatName.TW $ %eval(&CatLen+&maxtw+1);
         %end;
         %if %length(&GroupVar) %then %do;
            length &GroupVar.TW $ %eval(&GroupLen+&maxtw+1);
         %end;
         %if %length(&tvar) %then %do;
            length &tvar $ &maxtw;
         %end;
         set &outdat;
         gotoutput=0;
         %do i=1 %to &ntw;
            Domain = "&domain";
            TimeWindowNum = &i;
         
            if (&ENvar >= &&tws&i) and (&STvar <= &&twe&i) then do;
               %if %length(&CatName) %then %do;
                  &CatName.TW = cat(trim(left(&CatName))," &&tw&i");
               %end;             
               %if %length(&GroupVar) %then %do;
                  &GroupVar.TW = cat(trim(left(&GroupVar))," &&tw&i");
               %end;             
               %if %length(&tvar) %then %do;
                  &tvar = "&&tw&i";
               %end;
               output;
               gotoutput=1;               
            end;
         %end;
         %if %symexist(BaselineTimeWindow) %then %do;
            %if &blflag and ^%length(&BaselineTimeWindow) %then %do;
               if &domain.BLFL = 'Y' and gotoutput=0 then output;
            %end;
         %end;
         
         %if %length(&CatName) %then %do;
            label &CatName.TW = &CatLabelTW;
         %end;
         %if %length(&GroupVar) %then %do;
            label &GroupVar.TW = &GroupLabelTW;
         %end;
      run;

      %if %length(&CatName) %then %do;
         %global CatName0 CatLabel0;
         %let CatName0 = &CatName;
         %let CatLabel0 = &CatLabel;
         %let CatName = &CatName.TW;
         %let CatLabel = &CatLabelTW;
         %let CatKeepVars = &CatName0 &Tvar TimeWindowNum Domain;
      %end;
      %if %length(&GroupVar) %then %do;
         %global GroupVar0 GroupLabel0;
         %let GroupVar0 = &GroupVar;
         %let GroupLabel0 = &GroupLabel;
         %let GroupVar = &GroupVar.TW;
         %let GroupLabel = &GroupLabelTW;
         %if %length(&CatName) %then %do;
            %let CatKeepVars = &CatKeepVars &GroupVar0;
         %end;
      %end;
            
      %NObsVars(&outdat);
      %if ^&nobs and &errflag %then %do;
         %put [MS_S_0197];
         %put ERROR: No data found within the specified time windows. Please change the windows.;
         %let exiterror = 1;
      %end;
      
      /*
      data outlib.addtime;
         set &outdat;
      run;
      */
   %end;
   %else %do;
      %let ntw = 1;
      %let CatKeepVars = ;
   %end;
   %twexit:;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* create counts to be used in tree map;
%macro createCounts(domain,KeepVars=);

   %if ^%symexist(GroupVar) %then %do;
      %global GroupVar;
      %let GroupVar = ;
   %end;

   data c1;
      set &domain;
      keep USUBJID &GroupVar &CatName &KeepVars;
   run;

   proc sort data=c1 out=c2 nodupkey;
      by USUBJID &GroupVar &CatName &KeepVars;
   run;

   proc sort data=c2;
      by &GroupVar &CatName &KeepVars;
   run;

   data c2;
      set c2;
      one = 1;
   run;

   proc means data=c2 noprint;
      by &GroupVar &CatName &KeepVars;
      var one;
      output out=OutLib.&domain._counts(drop=_TYPE_ _FREQ_) n=Count;
   run;

   proc sort data=OutLib.&domain._counts;
      by &CatName;
   run;

%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* SAS macro for including drill-down buttons for Patient Profiler and Compare Subjects in tabbed report */
/*
   set _fCustomizedFunctions to 1 and define functions objRefNS:ppdd and objRefNS:cpdd prior to calling this macro
   if customized objRefNS:ppdd and objRefNS:cpdd functions are needed. Exposure Summary is one example that needs customized functions;
*/
%macro SubjectDrillDowns(_fCustomizedFunctions=0);

   %if ^&_fCustomizedFunctions %then %do;
      put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
      put "objRefNS:cpdd =  evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); clusterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""); clusterFunc(this:getParent());));";
      put "objRefNS:fpdd = evalexpr(Function({},{default local},  this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); reviewRefNS = namespace(reportRefNS:getParent()); createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< newContext); createFilterFunc(reviewRefNS << getName);));";  
      put "objRefNS:aendd = evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); aenFunc = Include(""$CLINICAL_HOME\ProcessLibrary\AENarrativeDrilldownFunction.jsl"",<< New Context,<< ""Names Default To Here""); aenFunc(this:getParent(), 1);));";
      put "objRefNS:spdd =  Function({}, {}, Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";
   %end;
   /*Kemicl add for new UI action button toolbar */
   put "    objRefNS:ActionButtonList= {{Expr(locale:get(""ProfileSubjects_Text"")), Expr(locale:get(""ShowSubjects_Text"")), Expr(locale:get(""ClusterSubjects_Text"")), expr(locale:get(""AENarrativeGeneration_Text"")), Expr(locale:get(""CreateSubjectFilter_Text""))}};";
   put "    objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"", ""&IconPath.DataSetUser.gif"", ""&IconPath.HierarchicalClustering.gif"", ""&IconPath.CreateNarratives.gif"",  ""&IconPath.AddFilter.gif""}};";
   put "    objRefNS:ActionSelectionCountList = {{1, 1, 3, 1, 1}};";
   put "    objRefNS:ActionRequirementFlagList = {{3, 0, 3, 3, 2}};";
   put "    objRefNS:ActionCodeList= {{objRefNS:ppdd(), objRefNS:spdd(), objRefNS:cpdd(), objRefNS:aendd(), objRefNS:fpdd()}};";
   /* Kemicl likely will go away */
   put "    objRefNS:ActionButtonDesc = {Expr(locale:get(""SelectSubjectsthenClick_Text""))};";

%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* set flags for demographic variables */
%macro DMFlags;

   %global cflg eflg rflg aflg sflg siflg tflg uflg wflg xflg yflg zflg;
   %global doAge doAgeTxt doSex doRace doTrt doStart doEnd doSite doStudy doCtry doEth doSpec doStrain doSbstrain;

   %let aflg = %sysfunc(indexw(&ADSLVARS, AGE, ' '));
   %let cflg = %sysfunc(indexw(&ADSLVARS, COUNTRY, ' '));
   %let eflg = %sysfunc(indexw(&ADSLVARS, ETHNIC, ' '));
   %let rflg = %sysfunc(indexw(&ADSLVARS, RACE, ' '));
   %let sflg = %sysfunc(indexw(&ADSLVARS, SITEID, ' '));
   %let siflg = %sysfunc(indexw(&ADSLVARS, STUDYID, ' '));
   %let tflg = %sysfunc(indexw(&ADSLVARS, RFSTDTC, ' '));
   %let uflg = %sysfunc(indexw(&ADSLVARS, RFENDTC, ' '));
   %let wflg = %sysfunc(indexw(&ADSLVARS, AGETXT, ' '));
   %let xflg = %sysfunc(indexw(&ADSLVARS, SPECIES, ' '));
   %let yflg = %sysfunc(indexw(&ADSLVARS, STRAIN, ' '));
   %let zflg = %sysfunc(indexw(&ADSLVARS, SBSTRAIN, ' '));
   
   %let doAge = 1;
   %let doAgeTxt = 1;
   %let doSex = 1;
   %let doRace = 1;
   %let doTrt = 1;
   %let doStart = 1;
   %let doEnd = 1;
   %let doSite = 1;
   %let doStudy = 1;
   %let doCtry = 1;
   %let doEth = 1;
   %let doSpec = 1;
   %let doStrain = 1;
   %let doSbstrain = 1;
   %let Country = %str(COUNTRY);
   %let Ethnic = %str(ETHNIC);
   %let Race = %str(RACE);
   %let Age = %str(AGE);
   %let StartDate = %str(RFSTDTC);
   %let EndDate = %str(RFENDTC);
   %let Site = %str(SITE);
   %let Species = %str(SPECIES);
   %let Strain = %str(STRAIN);
   %let Sbstrain = %str(SBSTRAIN);
   %let AgeTxt = %str(AGETXT);

   %if ^&aflg %then %do;
      %let doAge = 0;
      %let Age = ;
   %end;
   %if ^&sflg %then %do;
      %let doSite = 0;
      %let Site = ;
   %end;
   %if ^&siflg %then %do;
      %let doStudy = 0;
      %let Study = ;
   %end;   
   %if ^&cflg %then %do;
      %let doCtry = 0;
      %let Country = ;
   %end;
   %if ^&eflg %then %do;
      %let doEth = 0;
      %let Ethnic = ;
   %end;
   %if ^&rflg %then %do;
      %let doRace = 0;
      %let Race = ;
   %end;
   %if ^&tflg %then %do;
      %let doStart = 0;
      %let StartDate = ;
   %end;
   %if ^&uflg %then %do;
      %let doEnd = 0;
      %let EndDate = ;
   %end;
   %if ^&xflg %then %do;
      %let doSpec = 0;
      %let Species = ;
   %end;
   %if ^&yflg %then %do;
      %let doStrain = 0;
      %let Strain = ;
   %end;
   %if ^&zflg %then %do;
      %let doSbstrain = 0;
      %let Sbstrain = ;
   %end;
   %if ^&wflg %then %do;
      %let doAgeTxt = 0;
      %let AgeTxt = ;
   %end;

   %if %length(&DesignByVars) %then %do;
      %do i=1 %to %sysfunc(countw(&DesignByVars));
         %let bvi = %upcase(%scan(&DesignByVars,&i,' '));
         %if &bvi=AGE %then %let doAge = 0;
         %else %if &bvi=SEX %then %let doSex = 0;
         %else %if &bvi=RACE %then %let doRace = 0;
         %else %if &bvi=COUNTRY %then %let doCtry = 0;      
         %else %if &bvi=SITEID %then %let doSite = 0;
         %else %if &bvi=STUDYID %then %let doStudy = 0;
         %else %if &bvi=ETHNIC %then %let doEth = 0;     
         %else %if &bvi=SPECIES %then %let doSpec = 0;      
         %else %if &bvi=STRAIN %then %let doStrain = 0;
         %else %if &bvi=SBSTRAIN %then %let doSbstrain = 0;    
         %else %if &bvi=AGETXT %then %let doAgeTxt = 0;     
         %else %if &bvi=RFSTDTC %then %let doStart = 0;     
         %else %if &bvi=RFENDTC %then %let doEnd = 0;     
         %else %if &bvi=%upcase(&TrtName) %then %let doTrt = 0;   
      %end;
   %end;

   %if ^&doAge and ^&doSex and ^&doRace and ^&doTrt and ^&doCtry and ^&doSite and ^&doEth and 
      ^&doSpec and ^&doStrain and ^&doSbstrain and ^&doAgeTxt and ^&doStart and ^&doEnd %then %do;
      %put [MS_S_0198];
      %put ERROR: At least one of &Age SEX &Race &Site &Country &Ethnic &Species &Strain &Sbstrain &AgeTxt &StartDate &EndDate or &TrtName must not be specified as a By Variable.;
      %let exiterror = 1;
   %end;

   %if %upcase(&TrtVar) = SEX %then %let doSex = 0;
   %else %if %upcase(&TrtVar) = RACE %then %let doRace = 0;
   %else %if %upcase(&TrtVar) = AGE %then %let doAge = 0;
   %else %if %upcase(&TrtVar) = COUNTRY %then %let doCtry = 0;
   %else %if %upcase(&TrtVar) = SITEID %then %let doSite = 0;
   %else %if %upcase(&TrtVar) = SITEID %then %let doStudy = 0;
   %else %if %upcase(&TrtVar) = ETHNIC %then %let doEth = 0;
   %else %if %upcase(&TrtVar) = SPECIES %then %let doSpec = 0;    
   %else %if %upcase(&TrtVar) = STRAIN %then %let doStrain = 0;
   %else %if %upcase(&TrtVar) = SBSTRAIN %then %let doSbstrain = 0;     
   %else %if %upcase(&TrtVar) = RFSTDTC %then %let doSbstrain = 0;     
   %else %if %upcase(&TrtVar) = RFENDTC %then %let doSbstrain = 0;     
   %else %if %upcase(&TrtVar) = AGETXT %then %let doAgeTxt = 0;
   
   %global DMKeepVars;
   %let DMKeepVars = ;
   %if &doSex %then %let DMKeepVars = &DMKeepVars SEX;
   %if &doAge %then %let DMKeepVars = &DMKeepVars AGE;
   %if &doAgeTxt %then %let DMKeepVars = &DMKeepVars AGETXT;
   %if &doRace %then %let DMKeepVars = &DMKeepVars RACE;
   %if &doCtry %then %let DMKeepVars = &DMKeepVars COUNTRY;
   %if &doSite %then %let DMKeepVars = &DMKeepVars SITEID;
   %if &doStudy %then %let DMKeepVars = &DMKeepVars STUDYID;
   %if &doEth %then %let DMKeepVars = &DMKeepVars ETHNIC;
   %if &doSpec %then %let DMKeepVars = &DMKeepVars SPECIES;
   %if &doStrain %then %let DMKeepVars = &DMKeepVars STRAIN;
   %if &doSbstrain %then %let DMKeepVars = &DMKeepVars SBSTRAIN;
   %if &doStart %then %let DMKeepVars = &DMKeepVars RFSTDTC;
   %if &doEnd %then %let DMKeepVars = &DMKeepVars RFENDTC;
   
%mend;

/* Macro to check DM or ADSL to see if there is more than 1 StudyID value */
%macro CheckStudyLevels;
   %global StudyLevs;
   %if ^%symexist(TrtName) %then %let TrtName = ;
   %if ^%symexist(DesignByVars) %then %let DesignByVars = ;
   %let StudyLevs = 0;

   %if %sysfunc(findw(%upcase(&ADSLVARS), STUDYID)) > 0 %then %do;
      proc sql noprint; 
         select distinct STUDYID 
         into :uniquelevels separated by ' '
         from &in_adsl;
         %let StudyLevs = &sqlobs;
      quit;
   %end;

   %if (&StudyLevs > 1 and %upcase(STUDYID) ne %upcase(&TrtName) and ^%index(%Upcase(&DesignByVars),STUDYID)) %then %do;
      %put Data contains records from &StudyLevs Studies;
      %let doStudy = 1;
   %end;
   %else %let doStudy = 0;
%mend;


*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* set labels for demographic variables */
%macro DMLabels(OutData);

   %global AgeLabel AgeTxtLabel SexLabel  RaceLabel  SiteLabel  StudyLabel CtryLabel  EthLabel SpecLabel StrainLabel SbstrainLabel;

   %if &aflg %then %do;
      %let AgeLabel = AGE;
      %CheckLabel(&OutData,AgeLabel);
   %end;
   %if &wflg %then %do;
      %let AgeTxtLabel = AGETXT;
      %CheckLabel(&OutData,AgeTxtLabel);
   %end;
   %let SexLabel = SEX;
   %CheckLabel(&OutData,SexLabel);
   %if &rflg %then %do;
      %let RaceLabel = RACE;
      %CheckLabel(&OutData,RaceLabel);
   %end;
   %if &sflg %then %do;
      %let SiteLabel = SITEID;
      %CheckLabel(&OutData,SiteLabel);
   %end;
   %if &siflg %then %do;
      %let STUDYLabel = STUDYID;
      %CheckLabel(&OutData,STUDYLabel);
   %end;   
   %if &cflg %then %do;
      %let CtryLabel = COUNTRY;
      %CheckLabel(&OutData,CtryLabel);
   %end;
   %if &eflg %then %do;
      %let EthLabel = ETHNIC;
      %CheckLabel(&OutData,EthLabel);
   %end;
   %if &xflg %then %do;
      %let SpecLabel = SPECIES;
      %CheckLabel(&OutData,SpecLabel);
   %end;
   %if &yflg %then %do;
      %let StrainLabel = STRAIN;
      %CheckLabel(&OutData,StrainLabel);
   %end;
   %if &zflg %then %do;
      %let SbstrainLabel = SBSTRAIN;
      %CheckLabel(&OutData,SbstrainLabel);
   %end;

%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro COMPAREDATES(start = , stop = );
   _stmo = substr(&start, 6, 2) + 0;
   _stda = substr(&start, 9, 2) + 0;
   _stye = substr(&start, 1, 4) + 0;
   
   _enmo = substr(&stop, 6, 2) + 0;
   _enda = substr(&stop, 9, 2) + 0;
   _enye = substr(&stop, 1, 4) + 0;
   
   _start = mdy(_stmo, _stda, _stye);
   _end = mdy(_enmo, _enda, _enye);
   _orderr = "";
   
   if _stda > . and _enda > . then do;
      if _start > _end then _orderr = "Y";
   end;
   else if _stmo > . and _enmo > . then do;
      if _stye = _enye and _stmo > _enmo then _orderr = "Y";
   end;
   else if _stye > . and _enye > . then do;
      if _stye > _enye then _orderr = "Y";
   end;
%mend COMPAREDATES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro PROFILEDATEIMPUTE;
   *** Leaving out Medical History since it isnt in main profiler;
   /*
   %let implist = AE CE CM SU;
   %if %sysfunc(findw(&implist, &temp)) %then %do;
   */
   %if (%sysfunc(findw(&eventsdomains, %upcase(&temp))) and %upcase(&temp)^=MH) or %sysfunc(findw(&interventionsdomains, %upcase(&temp)))
      or %upcase(&temp)=CE or %upcase(&temp)=SU %then %do;
      
      *** Start Date ***;
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) %then %do;
         _impstart = "";
         %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STRF, ' ')) %then %do;
            if &temp.STDTC = "" and upcase(&temp.STRF) not in ("U","UNKNOWN") then do;
               &temp.STDTC = RFSTDTC;
               _impstart = "Y";
            end;
         %end;
         %else %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STRTPT, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STTPT, ' ')) %then %do;
            if &temp.STDTC = "" and upcase(&temp.STRTPT) not in ("U","UNKNOWN") then do;
               &temp.STDTC = &temp.STTPT;
               _impstart = "Y";
            end;           
         %end;
      %end;
      
      *** End Date ***;
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do;
         _impend = "";
         %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENRF, ' ')) %then %do;
            if &temp.ENDTC = "" and upcase(&temp.ENRF) not in ("U","UNKNOWN") then do;
               &temp.ENDTC = RFENDTC;
               _impend = "Y";
            end;
         %end;
         %else %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENRTPT, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENTPT, ' ')) %then %do;
            if &temp.ENDTC = "" and upcase(&temp.ENRTPT) not in ("U","UNKNOWN") then do;
               &temp.ENDTC = &temp.ENTPT;
               _impend = "Y";
            end;           
         %end;   
         %else %if %upcase(&temp) = AE and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)OUT, ' ')) %then %do;
            if &temp.ENDTC = "" and (upcase(AEOUT) in ("NOT RECOVERED/NOT RESOLVED", "NOT RECOVERED", "NOT RESOLVED", "RECOVERING/RESOLVING", "RECOVERING", "RESOLVING", "CONTINUING", "ONGOING") 
               or index(upcase(AEOUT), "CONTINUING")>0) then do;
               &temp.ENDTC = RFENDTC;
               _impend = "Y";
            end;
         %end;
      %end;
    
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do; 
         %COMPAREDATES(start = &temp.STDTC, stop = &temp.ENDTC);
         if _orderr = "Y" then do;
           if _impstart = "Y" then &temp.STDTC = "";
           if _impend = "Y" then &temp.ENDTC = "";
         end;
      %end;
      
      if _impstart = "Y" or _impend = "Y" then do;
         &temp.ENDY = .;
         &temp.DUR = .;
         %calculateStudyDays(&temp);
      end;
      
      *** Clean Up ***;
      /* %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) %then %do;
         drop _impstart;
      %end;
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do;
         drop _impend;
      %end; 
      */
      %if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STDTC, ' ')) and %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENDTC, ' ')) %then %do;
         drop _stmo _stda _stye _enmo _enda _enye _start _end _orderr;
      %end;     
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)STRF)) %then %do;
         drop &temp.STRF;
      %end;
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)ENRF)) %then %do;
         drop &temp.ENRF;
      %end;
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)STRTPT)) %then %do;
         drop &temp.STRTPT;
      %end;       
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)STTPT)) %then %do;
         drop &temp.STTPT;            
      %end;       
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)ENRTPT)) %then %do;
         drop &temp.ENRTPT;
      %end;       
      %if %sysfunc(findw(&&&temp.VARS, %upcase(&temp)ENTPT)) %then %do;
         drop &temp.ENTPT;
      %end;                   
   %end;
%mend PROFILEDATEIMPUTE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*** Get Variable Based on Drop Down Label ***;
%macro GETMEDDRAVAR(label = , column = );
   %if &sysscp = WIN %then %do;
      %let _slash_ = \;
   %end;
   %else %do;
      %let _slash_ = /;
   %end;

   %let studynameunder = %sysfunc(transtrn(&StudyName, %str( ), _));
   %global &column;
   
   %let flabel =;
   
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
   
   data _null_;
      length column $ 20 label $ 100;
      retain foundit 0;
      infile "%jg_getPath(/user/clinical/metadata/base, AEMEDDRA.txt, 0, 0, 3, &studynameunder)" delimiter = "~";
      input column $ label $;
      if (foundit < 1) then do;
         if (label = "&label") then foundit = 1;
         /* Disabling translated labels per S1043149
         else do;
            call symput("flabel", label);
            if resolve('%LocaleAreEquiv(&flabel, &label)') = 1 then foundit = 1;
         end;
         */
         if foundit = 1 then call symput("&column", upcase(trim(column)));
      end;
   run;     
%mend GETMEDDRAVAR;

/*kemicl Notes:  do we really want to categorize variables twice? Once because at least one is needed from the list supplied to the parameter and 
twice to be specific to the conditional parameter choice? */
/*
$CVS_START$
$ID: AETermValues$
$CMP: RSYS.Term Variable$ $UE: AE.AEDECOD or AE.AETERM or AE.AEBODSYS or AE.AELLT or AE.AEHLT or AE.AEHLGT or AE.AESOC$ 
   $NOTES: At least one variable that provides the adverse event term value must be present.$ 
$CMP: CPV.{"Term Level","Reported Term for the Adverse Event"}$
   $UE: AE.AETERM$
   $NOTES: Also supplies Group Level Parameter.$   
$CMP: CPV.{"Term Level","Lowest Level Term"}$
   $UE: AE.AELLT$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Term Level","Dictionary-Derived Term"}$
   $UE: AE.AEDECOD$
   $NOTES: Also supplies Group Level Parameter. Default value for Term Level if found in data.$
$CMP: CPV.{"Term Level","Body System or Organ Class"}$
   $UE: AE.AEBODSYS$
   $NOTES: Also supplies Group Level Parameter. Default value for Group Level if found in data$
$CMP: CPV.{"Term Level","High Level Term"}$
   $UE: AE.AEHLT$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CMP: CPV.{"Term Level","High Level Group Term"}$
   $UE: AE.AEHLGT$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$   
$CMP: CPV.{"Term Level","Primary System Organ Class"}$
   $UE: AE.AESOC$
   $NOTES: Also supplies Group Level Parameter. Term and Group Values must adhere to MedDRA controlled hierarchy.$
$CVS_STOP$
*/

*** Ensure that the Grouping Variable is at least the level of Term Variable ***;
%macro COMPAREMEDDRA(term = , group = );
   %if ^%symexist(DoubleFDR) %then %do;
      %Global DoubleFDR;
      %let DoubleFDR = no;
   %end;
   %if %upcase(&DoubleFDR) = YES %then %do;
      %if &term = AETERM or &term = AELLT %then %do;        
      %end;
      %else %if &term =  AEDECOD and (&group = AETERM or &group = AELLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &AETermLevel cannot have &AEGroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;    
      %else %if (&term = AEBODSYS or &term = AESOC) and (&group = AEDECOD or &group = AETERM or &group = AELLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &AETermLevel cannot have &AEGroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;
      %else %if &term = AEHLT and (&group = AEBODSYS or &group = AESOC or &group = AEDECOD or &group = AETERM or &group = AELLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &AETermLevel cannot have &AEGroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;    
      %else %if &term = AEHLGT and (&group = AEHLT or &group = AEBODSYS or &group = AESOC or &group = AEDECOD or &group = AETERM or &group = AELLT) %then %do;
         %put [MS_S_0199];
         %put ERROR: &AETermLevel cannot have &AEGroupLevel as a grouping variable.;
         %let exiterror = 1;
      %end;
   %end;
   %else %if %upcase(&DoubleFDR) = NO %then %do;
      %if &term = AETERM or &term = AELLT %then %do;        
      %end;
      %else %if &term =  AEDECOD and (&group = AETERM or &group = AELLT) %then %do;
         %put NOTE: &AETermLevel cannot have &AEGroupLevel as a grouping variable. Grouping variable set to &AETermLevel.;
         %let GroupVar = &CatName;
      %end;    
      %else %if (&term = AEBODSYS or &term = AESOC) and (&group = AEDECOD or &group = AETERM or &group = AELLT) %then %do;
         %put NOTE: &AETermLevel cannot have &AEGroupLevel as a grouping variable. Grouping variable set to &AETermLevel.;
         %let GroupVar = &CatName;
      %end;
      %else %if &term = AEHLT and (&group = AEBODSYS or &group = AESOC or &group = AEDECOD or &group = AETERM or &group = AELLT) %then %do;
         %put NOTE: &AETermLevel cannot have &AEGroupLevel as a grouping variable. Grouping variable set to &AETermLevel.;
         %let GroupVar = &CatName;
      %end;    
      %else %if &term = AEHLGT and (&group = AEHLT or &group = AEBODSYS or &group = AESOC or &group = AEDECOD or &group = AETERM or &group = AELLT) %then %do;
         %put NOTE: &AETermLevel cannot have &AEGroupLevel as a grouping variable. Grouping variable set to &AETermLevel.;
         %let GroupVar = &CatName;        
      %end; 
   %end;
%mend COMPAREMEDDRA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

* check AESerOnly and modify AEWhereStmt accordingly, accounting for translation of "Y";

%macro AESerOnlyToAEWhereStmt;
   %let trY = %nrbquote(%LocaleGet(Y_Text));
   %if %upcase(&AESerOnly)=YES %then %do;
      %if %length(&AEWhereStmt) %then %let AEWhereStmt = %nrbquote((&AEWhereStmt) and (AESER="Y" or AESER="&trY"));
      %else %let AEWhereStmt = %nrbquote(AESER="Y" or AESER="&trY");
   %end;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro NONNULLVARS(dataset = rbm_jmpc , varlistname = RBMVARS);
   options nonotes;
   ods exclude all;
   ods noresults;
   
   proc freq data = &dataset nlevels;
      ods output nlevels=temp;
   run;

   %let _tid_ = %sysfunc(open(temp));
   %let _vn_ = %sysfunc(varnum(&_tid_,nnonmisslevels));
   %let _tid_ = %sysfunc(close(&_tid_));  
   
   %global &varlistname;
   %let &varlistname = ;

   %if &_vn_ %then %do;   
      proc sql noprint;      
         select distinct upcase(tablevar)
            into :&varlistname separated by ' '
            from temp(where=(nnonmisslevels > 0))
            order by upcase(tablevar);
      quit;
   %end;
   %else %do;
      proc sql noprint;      
         select distinct upcase(tablevar)
            into :&varlistname separated by ' '
            from temp
            order by upcase(tablevar);
      quit;   
   %end;

   proc datasets nolist;
      delete temp;
   quit;
        
   options notes;
   ods exclude none;
   ods results;
%mend NONNULLVARS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro OUTNAMETRUNC(outdat =, truncnum = );    
   %if %length(&&&outdat)>&truncnum %then %let &outdat = %substr(&&&outdat,1,&truncnum);
   %if %length(&&&outdat) %then %let Outname = &&&outdat.._;
   %else %let Outname = &&&outdat;
   %let &outdat =;
%mend OUTNAMETRUNC;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro ADSLMergeDM (temp=adsl);
   %if %length(&SDTMMETA) & %length(&adslpresent) %then %do;
      proc sql noprint;
         create table &temp as
         select * from &temp a1 left join &in_DM a2
         on a1.USUBJID=a2.USUBJID;
         quit;
   %end;
%mend;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro PARSEFINDINGS(dlist = , flist = );
   *** initialize variable lists ***;
   %let i = 1;
   %do %while(%length(%scan(&dlist,&i,' ')));
      %let temp = %scan(&dlist,&i,' ');
     %global &temp.findlist;
     %let &temp.findlist =;
      %let i = %eval(&i+1);
   %end;

   *** fill variable lists ***;
   %let i = 1;
   %do %while(%length(%scan(&flist,&i,' ')));
      %let temp = %scan(&flist,&i,' ');
     %let prefix = %scan(&temp,1,':'); 
     %let suffix = %scan(&temp,2,':'); 
      %if ^%length(&&&prefix.findlist) %then %let &prefix.findlist = &&&prefix.findlist %nrbquote("&suffix");
     %else %let &prefix.findlist = &&&prefix.findlist,%nrbquote("&suffix");
      %let i = %eval(&i+1);
   %end;
%mend PARSEFINDINGS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CHECKOVERLAP(flist = , slist = );
   %global overlaplist;
   %let overlaplist =;
   %let i = 1;
   %do %while(%length(%scan(&flist,&i,' ')));
      %let temp = %scan(&flist,&i,' ');
     %if %sysfunc(findw(&slist, &temp)) %then %let overlaplist = &overlaplist &temp;
      %let i = %eval(&i+1);
   %end;
   %if %length(&overlaplist) %then %do;
      %put [MS_S_0200];
      %put ERROR: The following findings tests are listed as both categorical and continuous: &overlaplist..;
     %put ERROR: Ensure each test is listed as either categorical or continuous in the dialog and re-run.;
      %let exiterror = 1;
   %end;
%mend CHECKOVERLAP;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CHECKPRES(domain = , var = , adddomain = 0);
   %if ^&adddomain %then %do;
      %global %upcase(&var);
      %if %sysfunc(findw(%upcase(&&&domain.VARS), %upcase(&var))) %then %let %upcase(&var) = %upcase(&var);
      %else %let %upcase(&var) = ;
   %end;
   %else %do;
      %global %upcase(&domain&var);
      %if %sysfunc(findw(%upcase(&&&domain.VARS), %upcase(&var))) %then %let %upcase(&domain&var) = %upcase(&var);
      %else %let %upcase(&domain&var) = ;
   %end;
%mend CHECKPRES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro WHEREDELTA(deltastate =, wherestate =, deltaflag = );
   %if %kupcase(&&&deltastate) = %kupcase(%LocaleGet(New_Text)) or %kupcase(&&&deltastate) = %kupcase(%LocaleGet(Stable_Text)) %then %do;
      %if %length(&&&wherestate) %then %do;
         %let &wherestate = (&&&wherestate) and kupcase(&deltaflag) = "%kupcase(&&&deltastate)";
      %end;
      %else %do;
         %let &wherestate =  kupcase(&deltaflag) = "%kupcase(&&&deltastate)";     
      %end; 
   %end;
   /* %else %if %upcase(&&&deltastate) = OTHER THAN STABLE %then %do;
      %if %length(&&&wherestate) %then %do;
         %let &wherestate = (&&&wherestate) and upcase(&deltaflag) ne "STABLE";
      %end;
      %else %do;
         %let &wherestate = upcase(&deltaflag) ne "STABLE";     
      %end;    
   %end;*/     
%mend WHEREDELTA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro TEAETEXTFLAG(TEAEstring = &TEAEOnly);
   %global TEAEText;   
   %let trsent = %nrbquote(%LocaleGet(XdeterminedusingY_Text));   
   %let TEAEText = ;   
   %if %index(%upcase(&TEAEstring),EMERGENT) %then %do;
      %let trterm = %nrbquote(%LocaleGet(Treatmentemergence_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOM..&TEAEVARFLAG));
   %end;
   %else %if %index(%upcase(&TEAEstring),PRE) %then %do;
      %let trterm = %nrbquote(%LocaleGet(Pretreatment_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOMs..&TEAEVARFLAGs));
   %end;
   %else %if %index(%upcase(&TEAEstring),OFF) %then %do;
      %let trterm = %nrbquote(%LocaleGet(Offtreatmentfollowup_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOMe..&TEAEVARFLAGe));
   %end;
   %else %if %index(%upcase(&TEAEstring),ON) %then %do;
      %let trsent = %nrbquote(%LocaleGet(XdeterminedusingYandZ_Text));
      %let trterm = %nrbquote(%LocaleGet(Ontreatment_Text));    
      %let TEAEText = %nrbquote(%LocaleFill(&trsent, &trterm, &TEAEVARDOMs..&TEAEVARFLAGs, &TEAEVARDOMe..&TEAEVARFLAGe));
   %end;
%mend TEAETEXTFLAG;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;