/***************************************************************
JMP Clinical Macro

Name: Findings Macros
Support: Kelci Miclaus, SAS Institute Inc.
History:
   kemicl 15Oct10 Initial Coding
   
Description: 
      Macros for findings reports analyses. 

-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
****************************************************************/
*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;

/*
$CVS_START$
$ID: GeneralOptions$
$CMP: OSYS$ $UE: FINDINGS.xxPOS$ $NOTES: Position variable will be appended to xxTEST and/or xxTESTCD for analysis.$
$CMP: OOUT$ $UE: DM.AGE$ $NOTES: Variable will be shown in Data Filter selection and/or Distribution display).$
$CMP: OOUT$ $UE: DM.SEX$ $NOTES: Variable will be shown in Data Filter selection and/or Distribution display).$
$CMP: OOUT$ $UE: DM.RACE$ $NOTES: Variable will be shown in Data Filter selection and/or Distribution display).$
$CMP: OOUT$ $UE: DM.SITEID$ $NOTES: Variable will be shown in Data Filter selection and/or Distribution display).$
$CVS_STOP$
*/

/*
$CVS_START$
$ID: Visits$
$CMP: CPV.{"Time Scale","Visits"}$ $UE: FINDINGS.VISIT or FINDINGS.VISITNUM$
   $NOTES: Visits ordered by VisitNum if variable exists.$  
$CVS_STOP$
*/

%macro CheckFindingsData(_dom, _test=TESTCD, _errchk = 1);
   /***********************************************************
   Macro to load Findings data, apply domain filters, and create variables needed for processing
   
   INPUTS:
   - dom: Findings domain code
   
   OUTPUTS:
   - Initialized/Set Macro Variables for domain variable existence checks and processing
   - Dataset:  &_dom data set in work directory sorted by USUBJID (or POOLID)
   
   Assumes %CheckFindingsData has been run and standard findings parameter macro variables exist 
   ************************************************************/
   %put Entering CHECKFINDINGSDATA Macro;
   %let exerr = 0;
   
   %if ^%length(&_dom) or %sysfunc(indexw(&domains, &_dom, ' ')) = 0  %then %do;
      %put [MS_S_0111];
      %put ERROR: There is no findings domain to analyze for the current study.;
      %let exerr = 1;
   %end;
   %if &exerr %then %goto mexit;
   
   %if &_errchk %then %do;
      %if %sysfunc(indexw(&&&_dom.VARS, %upcase(&_dom)STRESN, ' ')) = 0 and %Upcase(&ProcessName) ne FINDINGSDISTRIBUTION %then %do;
         %put [MS_S_0211];
         %put ERROR: Analysis cannot be performed due to missing required variable %upcase(&_dom)STRESN or AVAL.;
         %let exerr=1;
         %goto mexit;
      %end; 

      %if %sysfunc(indexw(&&&_dom.VARS, %upcase(&_dom.&_test), ' ')) = 0 %then %do;
         %put [MS_S_0212];
         %put ERROR: Analysis cannot be performed due to missing required variable %upcase(&_dom.&_test).;
         %let exerr=1;
         %goto mexit;
      %end; 
   %end;
   
   %if %symexist(LogShft) and %symexist(NormRef) %then %do;
      %if %UPCASE(&LogShft)=NONE %then %do;
         %if %UPCASE(&NormRef) = GEOMETRIC %then %do;
            %put [MS_S_0067];
            %put ERROR: The Geometric normalization method can only be used when you specify a log transformation of the Lab measurements.;
            %let exerr=1;
            %goto mexit;
         %end;
         %let LogShft = ;
      %end;
      %if %UPCASE(&NormRef) = NONE %then %let NormRef = ;   
   %end;
   
   /* Create Where Statement based on dialog domain filters */
   
   %WHEREDELTA(deltastate = FDeltaDrop, wherestate = WhereTests, deltaflag = &_dom.RFlg);   
      
   /* Checks for presence of required and optional nonmissing variables */
   %global stdy_ fdtc_ fpool blflag blflagvar blchg_ fpos_ ftesn ftesc fsev fbodsys fspec funits
   fvisitn fvisit ftpt ftptn fcat fscat fhilowflag GroupSortVars _cd_ _test_; 
   %let stdy_ = 0;
   %let fpool = 0;
   %let fusubid = 0;
   %let ftesn = 0;
   %let ftesc = 0;
   %let fspec = 0;
   %let fsev = 0;
   %let fbodsys = 0;
   %let fpos_ = 0;
   %let blflag = 0;
   %let blchg_ = 0;
   %let funits = 0;
   %let BlFlagVar = ABLFL;
   %let fvisitn = 0;
   %let fvisit = 0;
   %let ftpt = 0;
   %let ftptn = 0;
   %let fcat = 0;
   %let fscat = 0;
   %let fhilowflag = 0;

   /* Search for variables present and NONMISSING */
   %let _cd_ = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)TESTCD, ' '));
   %let _test_ = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)TEST, ' '));
   %let stdy_ = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)DY, ' '));
   %let fdtc_ = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)DTC, ' '));
   %let fpos_   = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)POS, ' '));
   %let ftesn   = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)STRESN, ' '));
   %let ftesc   = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)STRESC, ' '));
   %let fspec   = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)SPEC, ' '));
   %let fsev    = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)SEV, ' '));
   %let funits  = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)STRESU, ' '));
   %let fbodsys    = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)BODSYS, ' '));
   %let fpool   = %sysfunc(indexw(&&&_dom.Vars, POOLID, ' '));
   %let fusubid = %sysfunc(indexw(&&&_dom.Vars, USUBJID, ' '));
   %let blchg_ = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)BLCHG, ' '));
   %let blflag = %sysfunc(indexw(&&&_dom.Vars, ABLFL, ' '));
   %let fvisitn = %sysfunc(indexw(&&&_dom.Vars, VISITNUM, ' '));
   %let fvisit = %sysfunc(indexw(&&&_dom.Vars, VISIT, ' '));
   %let ftptn = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)TPTNUM, ' '));
   %let ftpt = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)TPT, ' '));
   %let fcat = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)CAT, ' '));
   %let fscat = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)SCAT, ' '));
   %let fhilowflag = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)NRIND, ' '));

   %if ^&blflag %then %do;
      %let blflag    = %sysfunc(indexw(&&&_dom.Vars, %upcase(&_dom)BLFL, ' '));
      %let BlFLagVar = &_dom.BLFL;
   %end;
   
	/*Add Test Selection (if made) to Where Statement */
	%Global TestWhereList TestSelection;
	%let TestWhereList = ;   
	%if %symexist(FindDTests) %then %do;
		%if %length(&FindDTests) %then %do;
			%let i = 1;
			%do %while(%length(%scan(&FindDTests,&i,' ')));
				%let temp = %scan(&FindDTests,&i,' ');
				%if ^%length(&TestWhereList) %then %let TestWhereList = %nrbquote("&temp");
				%else %let TestWhereList = &TestWhereList,%nrbquote("&temp");
				%let i = %eval(&i+1);
			%end;
		%end;
	%end;

	%if %length(&TestWhereList) %then %do;
		%let TestSelection = &&_dom.TESTCD in (&TestWhereList);
		%if (^&fpos_ and ^&fspec) %then %do;
			%if %length(&WhereTests) %then %do;
					%let WhereTests = (&WhereTests) and &&_dom.TESTCD in (&TestWhereList); 
				%end;
				%else %do;
					%let WhereTests = &&_dom.TESTCD in (&TestWhereList);    
				%end
			%end;
		%end;
	%end;
      
   
   /* Load in Findings Domain for analysis */
   %LOADDATA(domlist = &_dom, checkForZeroObs=1);
   %put &_dom data set loaded into work directory;
   /* output data and macro variable &&in_&dom assigned to work.&_dom */
   
   /* check if results should be processed by poolid instead of usubjid */
   %if &fpool %then %do;
      %if %length(&InPath) & %sysfunc(fileexist(&InPath./pooldef.sas7bdat)) %then %do;
         /* check if the domain is still by USUBJID not poolid and if so use that */
         data temp;
            set &&in_&_dom;
            where not missing(USUBJID);
         run;
         %NObsVars(temp);
         %if &nobs %then %let fpool = 0;
      %end;
      %else %do;
         /* if there is no pooldef data set then don't use pooldef */
         %let fpool = 0;
      %end;
   %end;
   %put fpool = &fpool; 
   
   /* Apply domain where statement filters and sort data by USUBJID (or POOLID if exists) */
   %if &fpool %then %do;
      proc sort data=&&in_&_dom %if %length(&WhereTests) %then (where=(%unquote(&WhereTests))); 
         out=&&in_&_dom %if &fusubid %then (drop = USUBJID);;
         by POOLID ;
      run;
   %end;
   %else %if (&fpos_ or &fspec) & %Upcase(&ProcessName) ne HYSLAWSCREENING %then %do;
   /*Check if xxPOS and append to both xxTEST and xxTESTCD to use more informative test here */
      %let plen = 0;
      %if &fpos_ %then %VarAttributes(_Ds=&&in_&_dom,_Var=&_dom.POS,_Len=plen);;
      %let slen = 0;
      %if &fspec %then %VarAttributes(_Ds=&&in_&_dom,_Var=&_dom.SPEC, _Len = slen);;
      %if &_test_ %then %do;
         %let tlen = ;
         %VarAttributes(_Ds=&&in_&_dom,_Var=&_dom.TEST,_Len=tlen);
         %let newtlen = %eval(&plen+&slen+&tlen+1);
      %end;
      %if &_cd_ %then %do;
         %let tclen = ;
         %VarAttributes(_Ds=&&in_&_dom,_Var=&_dom.TESTCD,_Len=tclen);
         %let newtclen = %eval(&tclen+&slen+&plen+1);
      %end;

      data &&in_&_dom; 
         %if &_cd_ %then length &_dom.TESTCD $&newtclen;;
         %if &_test_ %then length &_dom.TEST $&newtlen;;
         set &&in_&_dom %if %length(&TestWhereList) %then (where = (%unquote(&TestSelection)));;
         %if &_cd_ %then %do;
            format &_dom.TESTCD;
            informat &_dom.TESTCD;
            %if &fpos_ %then &_dom.TESTCD = catx(" ",upcase(&_dom.TESTCD),upcase(&_dom.POS));;
            %if &fspec %then &_dom.TESTCD = catx(" ",upcase(&_dom.TESTCD),upcase(&_dom.SPEC));;
         %end;
         %if &_test_ %then %do;
            format &_dom.TEST;
            informat &_dom.TEST;
            %if &fpos_ %then &_dom.TEST = catx(" ",upcase(&_dom.TEST),upcase(&_dom.POS));;
            %if &fspec %then &_dom.TEST = catx(" ",upcase(&_dom.TEST),upcase(&_dom.SPEC));;
         %end;          
      run;
   %end;
   
   /*Create GroupSortVars for findings test results organization */
   %let GroupSortVars = ;
   %let GroupCatVars = ;
   %if &fcat %then %do;
      %let GroupSortVars = &_dom.CAT;
      %if &fscat %then %do;
         %let GroupSortVars = &GroupSortVars &_dom.SCAT;
      %end;
   %let GroupCatVars = &GroupSortVars;      
   %end;
   %if %length(&&ing_&_dom) %then %do;
      %GroupVars(&_dom,&&in_&_dom, &&in_&_dom);
      %let GroupSortVars = &GroupSortVars &_dom.GRPN &_dom.SGRPN;
   %end;
   
   /*Create unique test/testcd values across xxCAT and xxSCAT to avoid errors with non-unique columns when formatting data*/ 
   /*Make xxTESTCD a requirement for this findings generally */
   %if &_cd_ & %length(&GroupCatVars)  & %Upcase(&ProcessName) ne HYSLAWSCREENING %then %do;
      proc sort data = &&in_&_dom(keep=&GroupCatVars &_dom.TESTCD %if &_test_ %then &_dom.TEST;) out=tcatvals nodupkey;
         by &GroupCatVars &_dom.TESTCD;
      run;

      %nobsvars(tcatvals);
      %let ntottest = &nobs;

      proc sort data = tcatvals out=utests nodupkey;
         by &_dom.TESTCD;
      run;
      %nobsvars(utests);

      /*Less unique TESTCD only means TESTCD duplicated across CAT or SCAT. Trigger creation of unique tests by appending a numeric value. */
      %if &nobs < &ntottest %then %do;
         %let tclen = ;
         %let TCLabel = &_dom.TESTCD;
         %let tlen = ;
         %let TLabel = &_dom.TEST ;
         %VarAttributes(_Ds=&&in_&_dom,_Var=&_dom.TESTCD,_Len=tclen);
         %let newtclen = %eval(&tclen + 2);
         %CheckLabel(&&in_&_dom,TCLabel);
         %if &_test_ %then %do;
            %VarAttributes(_Ds=&&in_&_dom,_Var=&_dom.TEST,_Len=tlen);
            %let newtlen = %eval(&tlen +2);
            %CheckLabel(&&in_&_dom,TLabel);
         %end; 
         
         proc sort data=tcatvals; 
            by &_dom.TESTCD;
         run;
         
         data tcatvals;
            set tcatvals;
            length newtestcd $&newtclen;
            label newtestcd = "&TCLabel";
            label dupt = "Duplicate Test Code";
            %if &_test_ %then %do;
               length newtest $&newtlen;
               label newtest = "&TLabel";
            %end;
            by &_dom.TESTCD;
            retain dupt 0;
            if first.&_dom.TESTCD then do;
               dupt = 0;
               newtestcd = &_dom.TESTCD;
               %if &_test_ %then newtest = &_dom.TEST;;
            end;
            else do; 
               dupt = dupt + 1;
               newtestcd =  cats(&_dom.TESTCD,"_",dupt);
               %if &_test_ %then newtest = cats(&_dom.TEST,"_",dupt);;
            end;
         run;
         /*kemicl need to do a SQL join here and plunk in concatenated values for tests */
         
         proc sql;
            create table 
               &&in_&_dom(rename=(newtestcd = &_dom.TESTCD %if &_test_ %then newtest = &_dom.TEST) drop=oTESTCD nTESTCD %if &_test_ %then oTest nTest;) as
               select *
               from 
                  tcatvals(rename = (&_dom.TESTCD = nTESTCD %if &_test_ %then &_dom.TEST = nTest)) as _ds1,
                  &&in_&_dom(rename = (&_dom.TESTCD = oTESTCD %if &_test_ %then &_dom.TEST = oTest)) as _ds2
                  
               where %if &fcat %then _ds1.&_dom.CAT=_ds2.&_dom.CAT and; 
                     %if &fscat %then _ds1.&_dom.SCAT=_ds2.&_dom.SCAT and;
                     _ds1.nTESTCD = _ds2.oTESTCD
               order by USUBJID;
         quit;
      %end; 
   %end;
   /*End unique test creation */
   
   /*apply where statement and sort by USUBJID */
   %if ^&fpool %then %do;
         proc sort data=&&in_&_dom %if %length(&WhereTests) %then (where=(%unquote(&WhereTests))); out=&&in_&_dom;
            by USUBJID;
         run;
   %end;
   
   %nobsvars(&&in_&_dom);
   %if &nobs = 0 %then %do;
      %put [MS_S_0213];
      %put ERROR: There are no observations in the analysis data set after where clause filtering.;
      %put Check that the Filter for Tests matches valid values.;
      %let exerr=1;
      %goto mexit;
   %end;
   
   %if %symexist(NormRef) %then %do;
      /* kemicl add check for LLN or ULN missing or 0 in labs */
      %global chklln chkuln mlchklow mlchkhi nlev;
      %let chklln = ;
      %let chkuln = ;
      %let mlchklow = 0;
      %let mlchkhi = 0;
      %if %UPCASE(&_dom) = LB and (%UPCASE(&normref) = LLN or %UPCASE(&normref) = GEOMETRIC) %then %do;
         %CHKLLN(lb);   
      %end;
      %if &exiterror %then %goto mexit;
      %if %UPCASE(&_dom) = LB and (%UPCASE(&normref) = ULN or %UPCASE(&normref) = GEOMETRIC) %then %do;
         %CHKULN(lb);
      %end;
      %if &exiterror %then %goto mexit;   
   %end;
   
   /*Create or blank out variables that other macros may depend upon */
   %if ^%symexist(VMBaselines) %then %do;
      %Global VMBaselines;
      %let VMBaselines = No;
   %end;
   %else %do;
      %if &blflag=0 or (&fvisitn=0 and &fvisit=0) %then %do;
         %let VMBaselines = No;
      %end;
   %end;
   %if ^%symexist(BaselineTimeWindow) %then %do;
      %Global BaselineTimeWindow;
      %let BaselineTimeWindow = ;
   %end;
   %if ^%symexist(MeasurementValues) %then %do;
      %Global MeasurementValues;
      %let MeasurementValues= CHANGE FROM BASELINE;
   %end;
   %if ^%symexist(BaselineCalc) %then %do;
      %Global BaselineCalc;
      %let BaselineCalc=LAST;
   %end;
   %mexit:;
    %if &syserr > 4 or &exerr=1 %then %do;
      %let exiterror = 1;
    %end; 
    %put Exit CHECKFINDINGSDATA Macro;
%mend CheckFindingsData;

%macro CheckBLTWsyntax(tbw);
   %if %length(&tbw) %then %do;
      %let i=1;
      %if %index(&tbw,%str(])) %then %let delim = ']';
      %else %let delim = ' ';
      %do %while(%length(%scan(&tbw,&i,&delim)));
         %let i=%eval(&i+1);
      %end;
      %let ntwi = %eval(&i-1);
         * if only one window is specified, throw error;
      %if &ntwi > 1 %then %do;
         %put [MS_S_0214];
         %put ERROR: Only one Baseline Time Window can be specified;
         %let exiterror=1;
      %end;
   %end;
%mend CheckBLTWsyntax;

/* Create BaselineTimeWindow.  Expects macrovars BaselineTimeWindow, blflag, MinTime, TimeWindows*/
%macro CreateBaselineTW;
   /* calculate basline if not input and no flag */
   %if ^%length(&BaselineTimeWindow) and ^&blflag %then %do;
         %if &MinTime > 1 %then %let i1 = &MinTime;
         %else %let i1 = 1;
         %let i2 = %eval(&i1+1);
         %let BaselineTimeWindow = %str([&MinTime,&i1]);
         %if %length(&TimeWindows) and ^%index(&TimeWindows,%str(])) %then %do;
            %put [MS_S_0098];
            %put ERROR: You must use brackets to specify the Trial Time Windows, e.g. [2,5];
            %let exerr = 1;
         %end;
         %let MinTime =&i2;
   %end; 
   %else %do;
      /* both timewindows and baseline time windows exist, error check */
      %if %length(&TimeWindows) and %length(&BaselineTimeWindow) %then %do;
         %let ibw = %index(&BaselineTimeWindow,%str(]));
         %let itw = %index(&TimeWindows,%str(]));
         %if (&itw and ^&ibw) or (^&itw and &ibw) %then %do;
            %put [MS_S_0099];
            %put ERROR: Baseline Time Window and the Trial Time Windows must be specified with similar syntax.;
            %let exerr = 1;
         %end; 
      %end;
      %else %if %length(&BaselineTimeWindow) %then %do;
         %let ib = %index(&BaselineTimeWindow,%str(]));
         %if &ib = 0 %then %do;
            %put [MS_S_0098];
            %put ERROR: You must use brackets to specify the Baseline Time Window, e.g. [-7,1];
            %let exerr = 1;

         %end;             
         %let ic = %index(&BaselineTimeWindow,%str(,));
         %if &ic = 0 %then %do;
            %let ic = %index(&BaselineTimeWindow,%str([)); 
         %end;
         %let i2 = %eval(%substr(&BaselineTimeWindow,%eval(&ic+1),%eval(&ib-&ic-1))+1);
         %let MinTime = &i2;
      %end; 
   %end;
   %put Time Window for forming Baseline = &BaselineTimeWindow;
%mend CreateBaselineTW;

/*
$CVS_START$
$ID: BaselineMeasurements$
$CMP: CPV.{"Plot findings measurements as:","Change from baseline"}$ 
   $UE: FINDINGS.xxBLFL or FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$
   $NOTES: For FINDINGS.xxBLFL, Value of "Y" categorizes baseline records. For FINDINGS.xxDY and FINDINGS.xxDTC, Records with Study Day <= 1 categorize baseline.$
$CMP: CPV.{"Plot findings measurements as:","Percent change from baseline"}$ 
   $UE: FINDINGS.xxBLFL or FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$
   $NOTES: For FINDINGS.xxBLFL, Value of "Y" categorizes baseline records. For FINDINGS.xxDY and FINDINGS.xxDTC, Records with Study Day <= 1 categorize baseline.$
$CMP: CPV.{"Plot findings measurements as:","Percent of baseline"}$ 
   $UE: FINDINGS.xxBLFL or FINDINGS.xxDY or (FINDINGS.xxDTC and DM.RFSTDTC)$
   $NOTES: For FINDINGS.xxBLFL, Value of "Y" categorizes baseline records. For FINDINGS.xxDY and FINDINGS.xxDTC, Records with Study Day <= 1 categorize baseline.$   
$CVS_STOP$
*/
/* macro to split data sets when baseline Time window specified.  Stat needed from Shift plots. option to calculate blchg*/
%macro SplitBaselineTrial(domain, dset, TimeV=, Stat=,doblchg = 0,TrialMeasurements = 1);
   /***********************************************************
   INPUTS:
   - domain: findings domain code
   - dset:findings data set in stacked sdtm form
   - TimeV: bracketed study day/week time window if a user specified Baseline Time Window is used 
   - Stat:  Used in Shift Plots for naming variables 
   - doblchg:  0/1 flag to create a combined data set of baseline adjusted measurements
   
   OUTPUTS:
   - Datasets:  _std if &doblchg, otherwise d1m and d2 data sets of baseline and trial measurements
   
   Assumes %CheckFindingsData has been run and standard findings parameter macro variables exist 
   ************************************************************/
   
   
   %let trBaseline = %nrbquote(%LocaleGet(Baseline_Text));
   %let trTrialStat = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(TrialXns_Text)), %nrbquote(%LocaleEZX(%nrbquote(&Stat), EIA=1))));
   %let trChgfromBL = %nrbquote(%LocaleGet(ChangefromBaseline_Text));
   %let trPercChgfromBL = %nrbquote(%LocaleGet(PercentChangefromBaseline_Text));
   %let trPercofBL = %nrbquote(%LocaleGet(PercentofBaseline_Text));
   %let trRatioofBL = %nrbquote(%LocaleGet(RatioofBaseline_Text));
   
   /*Take input findings data set and split into d1 d2 by baseline flag or study day and/or specified baseline time window */
   data d1 %if &TrialMeasurements %then d2;;
      set &dset;
      length MFLAG $ 30;
      %if %length(&BaselineTimeWindow) and &blflag %then %do;
         if &TimeV ne "&tw1" and &BlFlagVar ='Y' then delete;
      %end; 
      %if %length(&Stat) %then %do;
         if missing(&TestVar) then delete;
      %end;
      %if &blflag %then %do;
         if &BLFlagVar = 'Y' then do;
      %end;
      %else %if ^%length(&BaselineTimeWindow) %then %do;
         if &domain.DY <= 1 then do;
      %end;
      %else %do;
         if &TimeV = "&tw1" then do;
      %end;
         MFLAG = "&trBaseline";
         output d1;
      end;
      %if &TrialMeasurements %then %do;
         else do;
            MFLAG = "&trTrialStat";
            output d2;
         end;
      %end;
   run;
   
   /*
   %put blflag = &blflag;
   %put BLFlagVar = &BLFlagVar;
   %put TimeV = &TimeV;
   %put tw1 = &tw1;
   data outlib.d1;
      set d1;
   run;
   */

   %nobsvars(d1);
   %if &nobs = 0 %then %do;
      %put [MS_S_0215];
      %put ERROR: No measurements are flagged as baseline for the trial. Changes from baseline cannot be computed and the findings report cannot be generated.;
      %let exerr = 1;
   %end;

   %if &TrialMeasurements %then %do;
      %nobsvars(d2);
      %if &nobs = 0 %then %do;
         %put [MS_S_0216];
         %put ERROR: All measurements are flagged as baseline for the trial or the baseline flag values are missing.;
         %put Changes from baseline cannot be computed and the findings report cannot be generated.;
         %let exerr = 1;
      %end;
   %end;
   /* kemicl add option of way to calculate baseline from time window
      options: last measured value (average of measurements of last BL day), mean across time window
   */
   /*set up macro variables for by group and keep processing */
   %let BLKeepVars = &domain.TESTCD USUBJID Baseline;
   %if %upcase(&VMBaselines) = YES %then %do;
      %let BLCByVars = MFLAG &GroupSortVars &domain.TESTCD USUBJID;
      %if &fvisit %then %do;
         %let BLCByVars = &BLCByVars VISIT; 
         %let BLKeepVars = &BLCByVars Baseline VISIT;
      %end;
      %if &fvisitn %then %do;
         %let BLCByVars = &BLCByVars VISITNUM;
         %let BLKeepVars = &BLKeepVars Baseline VISITNUM;
      %end;
      %let BLCByVarsm = &BLCByVars;
   %end;
   %else %do;
      %let BLCByVars = &GroupSortVars &domain.TESTCD USUBJID &domain.DY;
      %let BLCByVarsm = &GroupSortVars &domain.TESTCD USUBJID;
   %end;
   %if %length(&Stat) %then %do;
      %let BLKeepVars = &BLKeepVars &GroupSortVars MFLAG;
      %let BLCByVars = &BLCByVars MFLAG;
      %let BLCByVarsm = &BLCByVarsm MFLAG;
    %end;
    
   %if %upcase(&BaselineCalc) = LAST %then %do;
      proc sort data=d1;
         by &BLCByVars;
      run;

      /* average by day in case multiple bl measurements on last day to take */
      proc means data=d1 noprint;
         by &BLCByVars;
         var &ResponseVar;
         output out=d1m mean=Baseline;
      run;

      data d1m;
         set d1m;       
         %if %upcase(&VMBaselines) = NO %then %do; 
            by &GroupSortVars &domain.TESTCD USUBJID &domain.DY;
            if last.USUBJID;
         %end;
         keep &BLKeepVars;
         label Baseline="&trBaseline";
      run;
   %end;
   %else %do;
      proc sort data=d1; 
         by &BLCByVarsm;
      run;

      proc means data=d1 noprint;
         by &BLCByVarsm;
         var &ResponseVar;
         output out=d1m mean=Baseline;
      run;

      data d1m;
         set d1m;       
         keep &BLKeepVars;
         label Baseline="&trBaseline";
      run;
   %end;
   
   /* S1120210 kemicl add error trap for mis-use of Visit Matched Baselines option */
   %if %upcase(&VMBaselines) = YES %then %do;
      proc sort data=d1m out= vbcheckb nodupkey;
         by %if &fvisit %then VISIT; %else VISITNUM;;
      run;
      
      proc sort data=d2 out=vbcheckt nodupkey;
         by %if &fvisit %then VISIT; %else VISITNUM;;
      run; 
      
      data vbcheck;
         merge vbcheckb(in=a) vbcheckt(in=b);
         by %if &fvisit %then VISIT; %else VISITNUM;;
         if a and b;
      run;
      
      %nobsvars(vbcheck);
      %if &nobs < 2 %then %do;
         %put [MS_S_0217];
         %put ERROR: There are &nobs visits that have both baseline and trial measurements, the option to Calculate Visit-Matched baseline measurements cannot be performed without at least two visits with both baseline and trial values.;
         %let exerr = 1;
      %end;
   %end;
   /* if resulting data set should be a calculated change from baseline, create _std data set with calculated measurements */
   %if &doblchg %then %do;
      proc sql;
         create table _std as
            select *,  (_ds2.&ResponseVar - _ds1.BaseLine) as &domain.BLCHG  label "&trChgfromBL",
                       ((_ds2.&ResponseVar - _ds1.BaseLine)/_ds1.BaseLine)*100 as &domain.PBLCHG  label "&trPercChgfromBL",
                       ((_ds2.&ResponseVar/_ds1.BaseLine))*100 as &domain.PofBL label "&trPercofBL",
                       (_ds2.&ResponseVar/_ds1.BaseLine) as &domain.BL label "&trRatioofBL"
            from d1m _ds1, d2 _ds2
            %if %upcase(&VMBaselines) = YES %then %do;
               where _ds1.USUBJID=_ds2.USUBJID and _ds1.&domain.TESTCD=_ds2.&domain.TESTCD 
                  %if  &fvisit %then and  _ds1.VISIT = _ds2.VISIT;
                  %if &fvisitn %then and _ds1.VISITNUM = _ds2.VISITNUM;
               ;  
            %end;
            %else %do;
               where _ds1.USUBJID=_ds2.USUBJID and _ds1.&domain.TESTCD=_ds2.&domain.TESTCD;  
            %end;
      quit; 

      %nobsvars(_std);
      %if &nobs = 0 %then %do;
         %put [MS_S_0218];
         %put ERROR: There are no subjects that have both flagged baseline and trial measurements to calculate change from baseline, results cannot be plotted.;
         %let exerr = 1;
      %end;

      %global RVarLab; 
      %if %upcase(&MeasurementValues) = CHANGE FROM BASELINE %then %do;
         %let ResponseVar = &domain.BLCHG; 
         %let RVarLab = %nrbquote(&trChgfromBL);
      %end;
      %else %if %upcase(&MeasurementValues) = PERCENT CHANGE FROM BASELINE %then %do;
         %let ResponseVar = &domain.PBLCHG;
         %let RVarLab = %nrbquote(&trPercChgfromBL);
      %end;
      %else %if %upcase(&MeasurementValues) = PERCENT OF BASELINE %then %do;
         %let ResponseVar = &domain.PofBL;
         %let RVarLab = %nrbquote(&trPercofBL);
      %end;
      %else %if %upcase(&MeasurementValues) = BASELINE RATIO %then %do;
         %let ResponseVar = &domain.BL;
         %let RVarLab = %nrbquote(&trRatioofBL);
      %end; 
      
      proc datasets nolist;
         delete d1 d2 d1m;
      quit;
   %end;
%mend SplitBaselineTrial;
   
%macro CHKLLN(lbds);
   %if %sysfunc(findw(&LBVARS, LBSTNRLO)) = 0 %then %do;
      %put [MS_S_0219];
      %put ERROR: The LBSTNRLO variable was not found in the LB data set.;
      %let exiterror = 1;
   %end;
   proc freq noprint data = &lbds;
      tables lbtestcd * LBSTNRLO / out = chklln outpct;
   run;
      
   /* kemicl - first check if all missing or all 0 so exit from process */ 
   data chkall;
      set chklln;
      if LBSTNRLO=0 and PCT_ROW =100 then delete;
      if not missing(LBSTNRLO) and not missing(PCT_ROW);
   run;
   
   %nobsvars(chkall);
   %if &nobs = 0  %then %do;
      %put [MS_S_0220];
      %put ERROR: All LBSTNRLO values are either missing or zero for all LBTESTCD. Normalization cannot be performed.;
      %put ERROR: Please choose an alternate method of normalization and check the lb data set for accuracy.;
      %let exiterror = 1;
   %end;
   %else %do;
      proc sort data=chkall nodupkey;
         by LBTESTCD;
      run;
      
      proc sort data=chklln nodupkey;
         by LBTESTCD;
      run;
      
      data chklln; merge chkall(in=a) chklln(in=b);
         by LBTESTCD;
         if not a;
      run;
      data chklln(keep = lbtestcd chkllnlist);
         length chkllnlist $ 200;
         set chklln end = eof;
         where (LBSTNRLO = 0 and PCT_ROW = 100) or (missing(LBSTNRLO) and missing(PCT_ROW));
         retain chkllnlist;
         chkllnlist = catx(" ", lbtestcd, chkllnlist);
         if eof then CALL SYMPUT("chklln", chkllnlist);
      run;

      %let DSID = %sysfunc(open(chklln));      
      %if %sysfunc(attrn(&dsid, nobs)) > 0 %then %do;   
         %PUT Warning:  All LBSTNRLO are either missing or equal to zero for LBTESTCD = &chklln..;      
         %PUT Warning:  Results will not be generated for these tests.  Choose an alternate method of normalization if you wish to see these tests.;  
         %let mlchklow = 1;
      %end;
      %let DSID = %sysfunc(close(&dsid));
   %end;
%mend CHKLLN;

%macro CHKULN(lbds);
   %if %sysfunc(findw(&LBVARS, LBSTNRHI)) = 0 %then %do;
      %put [MS_S_0221];
      %put ERROR: The LBSTNRHI variable was not found in the LB data set.;
      %let exiterror = 1;
   %end;
   proc freq noprint data = &lbds;
      tables lbtestcd * LBSTNRHI / out = chkuln outpct;
   run;    
   /* kemicl - first check if all missing or all 0 so exit from process */ 
   data chkall;
      set chkuln;
      if LBSTNRHI=0 and PCT_ROW =100 then delete;
      if not missing(LBSTNRHI) and not missing(PCT_ROW);
   run;
   
   %nobsvars(chkall);
   %if &nobs = 0  %then %do;
      %if %index(&ProcessName,HysLaw) %then %do;
         %put [MS_S_0222];
         %put ERROR: All LBSTNRHI values are either missing or zero for all relevant Hys Law LBTESTCD. Normalization cannot be performed.;
         %put ERROR: LBSTNRHI values are required to perform Hys Law Screening, please check the lb data set for accuracy. ;
      %end;
      %else %do;
         %put [MS_S_0223];
         %put ERROR: All LBSTNRHI values are either missing or zero for all LBTESTCD. Normalization cannot be performed.;
         %put ERROR: Please choose an alternate method of normalization and check the lb data set for accuracy.;
      %end;
      %let exiterror = 1;
   %end;
   %else %do;
      proc sort data=chkall nodupkey;
         by LBTESTCD;
      run;
      
      proc sort data=chkuln nodupkey;
         by LBTESTCD;
      run;
      
      data chkuln; merge chkall(in=a) chkuln(in=b);
         by LBTESTCD;
         if not a;
      run;  
      data chkuln(keep = lbtestcd chkulnlist);
         length chkulnlist $ 200;
         set chkuln end = eof;
         where (LBSTNRHI = 0 and PCT_ROW = 100) or (missing(LBSTNRHI) and missing(PCT_ROW));
         retain chkulnlist;
         chkulnlist = catx(" ", lbtestcd, chkulnlist);
         if eof then CALL SYMPUT("chkuln", chkulnlist);
       run;

      %let DSID = %sysfunc(open(chkuln));      
      %if %sysfunc(attrn(&dsid, nobs)) > 0 %then %do;   
         %PUT Warning:  All LBSTNRHI are either missing or equal to zero for LBTESTCD = &chkuln..;      
         %PUT Warning:  Results will not be generated for these tests.  Choose an alternate method of normalization if you wish to see these tests.; 
         %let mlchkhi = 1;
      %end;
      %let DSID = %sysfunc(close(&dsid));
   %end;
%mend CHKULN;

/*
$CVS_START$
$ID: LabNormalization$
$CMP: CPV.{"Normalization of Lab Measurements","LLN"}$ 
   $UE: LB.LBSTNRLO$
   $NOTES: Lab tests with all missing or 0 values will be removed from analysis.$
$CMP: CPV.{"Normalization of Lab Measurements","ULN"}$ 
   $UE: LB.LBSTNRHI$
   $NOTES: Lab tests with all missing or 0 values will be removed from analysis.$   
$CVS_STOP$
*/
%macro LabsNormalize(Response,Norm, LogTrans);
   /* set up normalization and scale for Shift Plots */
      %if %UPCASE(&Norm)= ULN %then %do;
         if missing(LBSTNRHI) or missing(LBSTRESN) or LBSTNRHI <= 0 or LBSTRESN <= 0 then &Response = .;
         else &Response = &LogTrans.(LBSTRESN/LBSTNRHI);
      %end;
      %else %if %UPCASE(&Norm) = LLN %then %do;
         if missing(LBSTNRLO) or missing(LBSTRESN) or LBSTNRLO <= 0 or LBSTRESN <= 0 then &Response = .;
         else &Response = &LogTrans.(LBSTRESN/LBSTNRLO);
      %end;
      %else %if %UPCASE(&Norm) = GEOMETRIC %then %do;
         if missing(LBSTNRHI) or missing(LBSTNRLO) or missing(LBSTRESN) or
         LBSTNRHI <= 0 or LBSTNRLO <= 0 or LBSTRESN <= 0 then &Response = .;
         else do;
            HALFRL2 = (&LogTrans.(LBSTNRHI)-&LogTrans.(LBSTNRLO))/2;
            &Response = (&LogTrans.(LBSTRESN)-(&LogTrans.(LBSTNRLO)+HALFRL2))/HALFRL2;
         end;
      %end;
      %else %do;
         if missing(LBSTRESN) or LBSTRESN <= 0 then &Response = .;
         else &Response = &LogTrans.(LBSTRESN);
      %end; 
%mend;



%macro BubblePlotAnalysis(_dom,ResponseVar);
   %if (&TimeScale = Days) %then %do;
      %let TimeVar = &_dom.DY;
      %let TimeLabel = &_dom.DY;
      
      %if &stdy_>0 %then %do;
         %CheckLabel(&&in_&_dom,TimeLabel);
      %end;
   %end;
   %else %do;
      %let TimeVar = &_dom.WK;
      %let TimeLabel = Week;
   %end;
   
    data _std; 
      set &&in_&_dom end=_last;
      where not missing(&_dom.STRESN);
      retain _mintime 99999999 _maxtime -999999999;
      length _timelabel $32;
      * standardize lab values and compute max and min times;
      %if %UpCase(&_dom) = LB %then %do;
         %LabsNormalize(&ResponseVar, &NormRef, &LogShft);
      %end;    
      %if (&TimeScale = Weeks) %then %do;
         &TimeVar = &_dom.DY/7;
         label &TimeVar = "&TimeLabel";
      %end;
      if not missing(&TimeVar) then do;
         _mintime = min(_mintime,&TimeVar);
         _maxtime = max(_maxtime,&TimeVar);
      end;
      _timelabel = "&TimeLabel" || trim(left(&TimeVar));
      if _last then do;
         if &StartTime < _mintime then call symput('StartTime',trim(left(_mintime)));
         if &EndTime > _maxtime then call symput('EndTime',trim(left(_maxtime)));
      end;
    run;
    
    * censor times if necessary;
    data _std;
      set _std;
      if &TimeVar < &StartTime then delete;
      else if &TimeVar > &EndTime then delete;
    run;

    * get default X and Y Variables, try to start with Hys Law;
    %if ^%length(%trim(&XVar)) or ^%length(%trim(&YVar)) %then %do;
      proc freq data=_std noprint;
         tables &_dom.TESTCD / out=fl;
         where not missing(&_dom.TESTCD);
      run;

      %nobsvars(fl);
      %if &nobs < 2 %then %do;
         %if %length(&WhereTests) %then %do;
            %put [MS_S_0224];
            %put ERROR: There is &nobs distinct value(s) of the &_dom.TESTCD variable after Where clause filtering.;
         %end;
         %else %do;
            %put [MS_S_0225];
            %put ERROR: There is only &nobs distinct value(s) of the &_dom.TESTCD variable available for analysis.;
         %end;
         %put ERROR: This process requires at least 2 distinct values of the &_dom.TESTCD variable.;
         %let exiterror = 1;
         %goto bpexit;
      %end; 
            
      %if %UPCASE(&_dom)=LB %then %do; /* Set up for Labs */
         %let BiliLab = ;
         %let ALTLab = ;
         %let ASTLab = ;
         data _null_;
            set fl;
            if (_n_ = 1) then call symput("FirstLab",trim(left(&_dom.TESTCD)));
            if (_n_ = 2) then call symput("SecondLab",trim(left(&_dom.TESTCD)));
            if trim(upcase(LBTESTCD))="BILI" or trim(upcase(LBTESTCD))="TBIL" or trim(upcase(LBTESTCD))="TBILI" then call symput("BiliLab",trim(left(&_dom.TESTCD)));
            else if trim(upcase(LBTESTCD))="TBL" or trim(upcase(LBTESTCD))="BILT" or trim(upcase(LBTESTCD))="BILTOT" then call symput("BiliLab",trim(left(&_dom.TESTCD)));          
            if index(upcase(&_dom.TESTCD),"ALT") or index(upcase(&_dom.TESTCD),"SGPT") or index(upcase(LBTEST),"ALAT") or index(upcase(&_dom.TESTCD),"ALANINE") then call symput("ALTLab",trim(left(&_dom.TESTCD)));
            if index(upcase(&_dom.TESTCD),"AST") or index(upcase(&_dom.TESTCD),"SGOT") or index(upcase(LBTEST),"ASAT") or index(upcase(&_dom.TESTCD),"ASPARTATE") then call symput("ASTLab",trim(left(&_dom.TESTCD)));
         run;

         %if ^%length(%trim(&XVar)) %then %do;
            %if %length(%trim(&ALTLab)) %then %let XVar = &ALTLab;
            %else %if %length(%trim(&ASTLab)) %then %let XVar = &ASTLab;
            %else %let XVar = &FirstLab;
         %end;
         %if ^%length(%trim(&YVar)) %then %do;
            %if %length(%trim(&BiliLab)) %then %let YVar = &BiliLab;
            %else %let YVar = &SecondLab;
         %end;
      %end;
      %else %if %UPCASE(&_dom)=VS %then %do;  /* Set up for Vital Signs */
         %let SysBP = ;
         %let DiaBP = ;
         data _null_;
            set fl;
            if (_n_ = 1) then call symput("FirstVS",trim(left(&_dom.TESTCD)));
            if (_n_ = 2) then call symput("SecondVS",trim(left(&_dom.TESTCD)));
            %if &fpos_>0 %then %do;
               if lengthn(VSPOS) then do;
                  if (index(upcase(&_dom.TESTCD),"SYS") or index(upcase(&_dom.TESTCD),"SBP"))  and index(upcase(&_dom.TESTCD),"STAND") then call symput("SysBP",trim(left(&_dom.TESTCD)));
                  if (index(upcase(&_dom.TESTCD),"DIA") or index(upcase(&_dom.TESTCD),"DBP")) and index(upcase(&_dom.TESTCD),"STAND") then call symput("DiaBP",trim(left(&_dom.TESTCD)));
               end;
               else do;
                  if index(upcase(&_dom.TESTCD),"SYS") or index(upcase(&_dom.TESTCD),"SBP") then call symput("SysBP",trim(left(&_dom.TESTCD)));
                  if index(upcase(&_dom.TESTCD),"DIA") or index(upcase(&_dom.TESTCD),"DBP") then call symput("DiaBP",trim(left(&_dom.TESTCD)));
               end;
            %end;
            %else %do;           
               if index(upcase(&_dom.TESTCD),"SYS") or index(upcase(&_dom.TESTCD),"SBP") then call symput("SysBP",trim(left(&_dom.TESTCD)));
               if index(upcase(&_dom.TESTCD),"DIA") or index(upcase(&_dom.TESTCD),"DBP") then call symput("DiaBP",trim(left(&_dom.TESTCD)));
            %end;
         run;
   
         %put SysBP = &SysBP;
         %put DiaBP = &DiaBP;
         %if ^%length(%trim(&XVar)) %then %do;
            %if %length(%trim(&DiaBP)) %then %let XVar = &DiaBP;
            %else %let XVar = &FirstVS;
         %end;
         %if ^%length(%trim(&YVar)) %then %do;
            %if %length(%trim(&SysBP)) %then %let YVar = &SysBP;
            %else %let YVar = &SecondVS;
         %end;    
      %end;
      %else %if %UPCASE(&_dom)=EG %then %do;  /* Set up for Vital Signs */
         %let HeartR = ;
         %let QTcBAZ = ;
         %let QTcALT = ;
         data _null_;
            set fl;
            if (_n_ = 1) then call symput("FirstEG",trim(left(&_dom.TESTCD)));
            if (_n_ = 2) then call symput("SecondEG",trim(left(&_dom.TESTCD)));
            if index(upcase(&_dom.TESTCD),"HR") then call symput("HeartR",trim(left(&_dom.TESTCD)));
            if index(upcase(&_dom.TESTCD),"QTCB") then call symput("QTcBAZ",trim(left(&_dom.TESTCD)));
            /* backups for QTc */
            if index(upcase(&_dom.TESTCD),"QTC") then call symput("QTcALT",trim(left(&_dom.TESTCD)));
            else if index(upcase(&_dom.TESTCD), "QT") then call symput("QTcALT", trim(left(&_dom.TESTCD)));
         run;

         %if ^%length(%trim(&XVar)) %then %do;
            %if %length(%trim(&HeartR)) %then %let XVar = &HeartR;
            %else %let XVar = &FirstEG;
         %end;
         %if ^%length(%trim(&YVar)) %then %do;
            %if %length(%trim(&QTcBAZ)) %then %let YVar = &QTcBAZ;
            %else %if %length(%trim(&QTcALT)) %then %let YVar = &QTcALT;
            %else %let YVar = &SecondEG;
         %end;    
      %end; 
      %else %do;
         data _null_;
            set fl;
            if (_n_ = 1) then call symput("XVar",trim(left(&_dom.TESTCD)));
            if (_n_ = 2) then call symput("YVar",trim(left(&_dom.TESTCD)));
         run;
      %end;
    %end;
   
    * impute missing values with linear interpolations for each row and constant extrapolation;
    %if %upcase(&ImputeMissingValues)=YES %then %do;

      * make sure all times have at least one value by creating a dummy subject;
      data add;
         USUBJID = "000000                              ";
         &_dom.TESTCD = "AAA                                   ";
         do &TimeVar = &StartTime to &EndTime;
            output;
         end;
      run;

      data _std;
         set add _std;
      run;

      proc sort data=_std;
         by USUBJID &_dom.TESTCD &TimeVar;
      run;

      * compute averages for each unique time point;
      proc means data=_std noprint;
         by USUBJID &_dom.TESTCD &TimeVar;
         var &ResponseVar;
         output out=_stdm mean=&ResponseVar;
      run;

      * transpose to create missing values;
      proc transpose data=_stdm out=_stdmt prefix=_time;
         by USUBJID &_dom.TESTCD;
         var &ResponseVar;
         id &TimeVar;
         idlabel &TimeVar;
      run;

      data _stdmti;
         set _stdmt;
         array t{*} _time:;
         nnmi = 0;
         nnmv = .;
         do while(missing(nnmv) and nnmi < dim(t));
            nnmi+1;
            nnmv = t[nnmi];
         end;
         pnmi = 1;
         pnmv = nnmv;
         do i=1 to dim(t);
            if missing(t[i]) then do;
               if (pnmi=nnmi) then t[i] = pnmv;
               else t[i] = ((nnmi-i)/(nnmi-pnmi))*pnmv + ((i-pnmi)/(nnmi-pnmi))*nnmv;
            end;
            else do;
               pnmi = i;
               pnmv = t[i];
               nnmi = i;
               nnmv = .;
               do while(missing(nnmv) and nnmi < dim(t));
                  nnmi+1;
                  nnmv = t[nnmi];
               end;
               if missing(nnmv) then nnmv = pnmv;
            end;
         end;
         rowmean = mean(of _time:);
         if trim(left(USUBJID)) = "000000" then delete;
         drop i nnmi nnmv pnmi pnmv;
      run;

      * transpose to get into form required for JMP bubble plot;
      proc transpose data=_stdmti out=_bpt(drop=_name_);
         by USUBJID;
         var _time:;
         id &_dom.TESTCD;
         idlabel &_dom.TESTCD;
      run;

      * set label on time variable and convert to numeric;
      data _bpt;
         label &TimeVar="&TimeLabel";
         set _bpt;
         &TimeVar = 0 + _LABEL_;
         drop _LABEL_;
      run;
      /*
         * create transposed version in the output folder;
         proc sql;
            create table OutLib.lb_stdmti as
               select *
               from lb_stdmti d1, &in_adsl d2
               where d1.USUBJID=d2.USUBJID;
         quit;
         */

         /*
         data lb_stack;
            set lb_stdmti;
            array t{*} _time:;
            do &TimeVar = &StartTime to &EndTime;
               _lbval = t[&TimeVar-&StartTime+1];
               output;
            end;
            label &TimeVar = "&TimeLabel";
            drop _time:;
         run;

         proc sort data=lb_stack;
            by USUBJID &TimeVar;
         run;

         proc transpose data=lb_stack out=lb_bpt(drop=_name_);
            by USUBJID &TimeVar;
            var _lbval;
            id LBTESTCD;
            idlabel LBTESTCD;
         run;
      */
    %end;
    %else %do;
      proc sort data=_std;
         by USUBJID &TimeVar &_dom.TESTCD;
      run;

      * compute averages for each unique time point;
      proc means data=_std noprint;
         by USUBJID &TimeVar &_dom.TESTCD;
         var &ResponseVar;
         output out=_stdm mean=&ResponseVar;
      run;

      * transpose to get into form required for JMP bubble plot;
      proc transpose data=_stdm out=_bpt;
         by USUBJID &TimeVar;
         var &ResponseVar;
         id &_dom.TESTCD;
         idlabel &_dom.TESTCD;
      run;
    %end;
    
    %if &syserr > 4 %then %do;
      %let exiterror = 1;
    %end; 
    
    %bpexit:;
    
%mend;

%macro BPlot_JSL(_JSLFile, _dom, _DomName);

      %DMFlags;      
      %DMLabels(OutLib.&OutDataName);
      %CheckStudyLevels;
      
   * put JSL statements;
  data _null_;
   file "&_JSLFile" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newnamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";
      put "objRefNS:DataList = {};";
      put "objRefNS:TabButtonList = {};";
      put "objRefNS:TabCodeList = {};";
      put "objRefNS:TabAfterCodeList = {};";
      put "objRefNS:TabOpenList = {};";
      put "objRefNS:CreateReportList = {};";
      put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
      put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
      put "objRefNS:NoteDomain = ""&_dom"";";      
      
      put "objRefNS:includestring=""Findings Tests: "";";
      %let DomainWhereStmt = &WhereTests;
      put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";
            
      %if %length(&WhereForReports) %then %do;
         put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
         put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
         %if %length(&DomainWhereStmt) %then %do;
            put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
         %end;
      %end;
      %else %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;
      
      %if %upcase(&FDeltaDrop) = %upcase(%LocaleGet(New_Text)) or %upcase(&FDeltaDrop) = %upcase(%LocaleGet(Stable_Text)) %then %do;
         put"try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NInclude the following findings records: \[%nrbquote(&FDeltaDrop)]\"",";
         put"try(objRefNS:WhereFilters=""Include the following findings records:\[%nrbquote(&FDeltaDrop)]\""););";
      %end; 
      
      put "if(hostis(windows), _slash_ = ""\"", _slash_ = ""/"");";
      put "if(Host Is(Windows),";
      put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME\Clinical\"", absolute, windows),";
      put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME/Clinical/"", absolute, POSIX)";
      put ");";
   
      put "objRefNS:svar = ""&SubjectLabel"";";
      put "objRefNS:tvar = ""&TimeLabel"";";
      put "objRefNS:yvar = ""&YVar"";";
      put "objRefNS:xvar = ""&XVar"";";   
         
      put " objRefNS:openDTExpr = Expr(If(IsEmpty(objRefNS:bpd),";
      put "    objRefNS:bpd = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      put "    Column(""&SubjectLabel"") << Label(1);";
      put "    objRefNS:dbv = {};";
      %do i=1 %to &ndbv;
         put "    InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
      %end; 
      put "  ,";  /*else for isempty*/;
      put "  Current Data Table(objRefNS:bpd);";
      put " ););";   
      put " objRefNS:openDTExpr;";
      
      %if %symexist(mlchklow) or %symexist(mlchkhi) %then %do;
         %if &mlchklow or &mlchkhi %then %do;
            put "objRefNS:MLfunc=Function({},{default local},";
            put "    VListBox(";
            put "    objRefNS:mlwarn= TextBox(locale:get(""WarningResultsNotGenDueToMissValsGenByNorm_Text"")||"; 
            %if &mlchklow %then %do;
               put "    ""\!N\!N "" || locale:fill(locale:get(""AllLBSTNRLOMissOrZeroForLBTESTCDX_Text""), {""%trim(&chklln)""}) || ""\!N ""||";     
            %end;
            %if &mlchkhi %then %do;
               put "    ""\!N\!N "" || locale:fill(locale:get(""AllLBSTNRHIMissOrZeroForLBTESTCDX_Text""), {""%trim(&chkuln)""}) || ""\!N""||";      
            %end;    
            put "       "" "",";
            put "          <<Set Wrap(600), Set Font Size(11)";
            put "    );";  
            put "));";

            put 'InsertInto(objRefNS:DataList,{{""}});';
            put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XWarningMessage_Text""), {locale:get(""Findings_Text"")})));";
            put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:MLfunc()));';
            put 'InsertInto(objRefNS:TabAfterCodeList," ");';
            put 'InsertInto(objRefNS:TabOpenList,1);';
            put "InsertInto(objRefNS:CreateReportList,{{objRefNS:mlwarn}});";       
         %end;
      %end;       
      
      put "objRefNS:bpw=Function({},{default local},";
      put " objRefNS:openDTExpr;";
      put " If(!IsEmpty(objRefNS:bp),";
      put "    bprep = objRefNS:bp<<report;";
      put "    objRefNS:bptimeindx = bprep[sliderbox(1)]<<get;";
      put " ,";
      put "    objRefNS:bptimeindx=0;";
      put " );";  

      put "  VListBox(";
      %if %UPCASE(&Normref) = GEOMETRIC %then %do;
         put "   objRefNS:GeoTB = TextBox(""  "" || locale:get(""NoteWithGeoNormOneUnitChangeCorrOneGeoStd_Text""),<<Set Wrap(900)),";
         put "   TextBox(""  ""),";
      %end;
      put "     HListBox(";
      put "      objRefNS:bp = Bubble Plot(";
      put "         X( :Name( ""&XVar"" ) ),";
      put "         Y( :Name( ""&YVar"" ) ),";
      put "         Time( :Name( ""&TimeLabel"" ) ),";
      put "         Time Index(objRefNS:bptimeindx),";   
      put "         Coloring( :Name(""&TrtLabel"") ),";
      put "         ID( :Name(""&SubjectLabel"") ),";
      %if %upcase(&SplitResults)=YES %then %do;
         put "         By(eval(objRefNS:dbv)),";
      %end;
      put "         Trail Bubbles( 1 ),";
      put "         Trail Lines( 1 ),";
      put "         All Labels( 0 ),";
      put "         Selectable Across Gaps( 1 ),";
      put "         Show Roles( 1 ),";
      put "         Legend( 1 ),";
      put "         SendToReport(";
      put "            Dispatch( {}, ""Bubble Plot"", FrameBox,";
      put "               {Frame Size(650, 450) ";
      %if %UPCASE(&_dom)=LB and %UPCASE(&BPRefLines) = NO and %upcase(&LogShft) = LOG2 and %index(%UPCASE(&NormRef),LN) %then %do;
         put "                ,Add Graphics Script(";
         put "                   Back,";
         put "                   FillColor(""Gray"");";
         put "                   Transparency(0.4);";
         put "                   Rect({-1, 1}, {1, -1}, 1);";
         put "                   Transparency(0.3);";
         %if %UPCASE(&Normref) = GEOMETRIC %then %do;
            put "                   Rect({-2, 2}, {2, -2}, 1);";
            put "                   Transparency(0.2);";
            put "                   Rect({-3, 3}, {3, -3}, 1);";
            put "                   Transparency(0.1);";
            put "                   Rect({-4, 4}, {4, -4}, 1);";        
         %end;
         %else %do;
            put "                   Rect({-1.6, 1.6}, {1.6, -1.6}, 1);";
            put "                   Transparency(0.2);";
            put "                   Rect({-2, 2}, {2, -2}, 1);";
            put "                   Transparency(0.1);";
            put "                   Rect({-2.3, 2.3}, {2.3, -2.3}, 1);";
         %end;
         put "                )";
      %end;
      put "               }";
      
      put "            )";
      
      %if %UPCASE(&&BPRefLines) = YES %then %do;
         put "          ,";
         put "          Dispatch({},""1"",ScaleBox,";
         put "             {Add Ref Line(&XRefLine, Dashed, ""Medium Dark Gray"")}";
         put "          ),";
         put "          Dispatch({},""2"",ScaleBox,";
         put "             {Add Ref Line(&YRefLine, Dashed, ""Medium Dark Gray"")}";
         put "          )";
      %end;
      put "         )";
      put "      );";
    
      %if %UPCASE(&_dom)=LB and %UPCASE(&BPRefLines) = NO and %upcase(&LogShft) = LOG2 and %index(%UPCASE(&NormRef),LN) %then %do;
         put "  objRefNS:axis=    VListBox(";
         put "         TextBox(""\!N \!N \!N \!N \!N \!N \!N \!N \!N \!N \!N \!N"",<<SetFontSize(1)),";
         put "         PanelBox("""",";
         put "            TableBox(";
         %if %upcase(&LogShft) = LOG2 %then %do;
            %if(&NormRef) = ULN %then %do;
               put "          trULN = locale:get(""ULN_Text""); ";
               put "               NumberColBox(locale:get(""Axis_Text""), [-2.3,-2,-1.6,-1,-0.6,0,0.6,1,1.6,2,2.3]),";
               put "               StringColBox(locale:get(""Level_Text""),{trULN || "" / 5"", trULN || "" / 4"", trULN || "" / 3"", trULN || "" / 2"", trULN || "" / 1.5"", ";
               put "             trULN, ""1.5 x "" || trULN, ""2 x "" || trULN, ""3 x "" || trULN, ""4 x "" || trULN, ""5 x "" || trULN})";
            %end;
            %else %if(&NormRef) = LLN %then %do;
               put "          trLLN = locale:get(""LLN_Text""); ";
               put "               NumberColBox(locale:get(""Axis_Text""), [-2.3,-2,-1.6,-1,-0.6,0,0.6,1,1.6,2,2.3]),";
               put "               StringColBox(locale:get(""Level_Text""),{trLLN || "" / 5"", trLLN || "" / 4"", trLLN || "" / 3"", trLLN || "" / 2"", trLLN || "" / 1.5"", ";
               put "             trLLN, ""1.5 x "" || trLLN, ""2 x "" || trLLN, ""3 x "" || trLLN, ""4 x "" || trLLN, ""5 x "" || trLLN})";
            %end; 
         %end;
      put "            )";
         put "         )";
         put "      );";
      %end;
      put ")";
      put ")";  /* end vlist */
      put ");";  /* end function */
      
      put "objRefNS:CustomPanel = Expr(objRefNS:Filter=objRefNS:bpd<<Data Filter(<<Conditional(1),";
      put " Add Filter(";
      put "    Columns(";
      %if &doStudy %then %do;
         put " :Name(""&StudyLabel""),";
      %end;
      %if &doAge %then %do;
         put " :Name(""&AgeLabel""),";
      %end;
      %if &doSex %then %do;
         put " :Name(""&SexLabel""),";
      %end;
      %if &doRace %then %do;
         put " :Name(""&RaceLabel""), ";
      %end;
      %if &doSite %then %do;
         put "       :Name(""&SiteLabel""),";
      %end;
      put ":Name(""&TrtLabel""),";
      put "          )";
  
      put " ),";
      put " Mode(Show(1), Include(1))";
      put "););";    
         
      put 'InsertInto(objRefNS:DataList,{{"objRefNS:bpd"}});';
      put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""BubblePlotofX_Text""),{""&_DomName""})));";
      put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:bpw()));';
      put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
      put 'InsertInto(objRefNS:TabOpenList,1);';
      put "try(InsertInto(objRefNS:CreateReportList,{{";
      %if %UPCASE(&Normref) = GEOMETRIC %then %do;
         put "    objRefNS:GeoTB,";
      %end;
      put "    objRefNS:bp";
      %if %UPCASE(&_dom)=LB and %UPCASE(&BPRefLines) = NO and %upcase(&LogShft) = LOG2 and %index(%UPCASE(&NormRef),LN) %then %do;
         put "    ,objRefNS:axis"; 
      %end;
      put "    }}));"; 

      %SubjectDrillDowns; 
      put 'objRefNS:ActionDataTableList = {{{"objRefNS:bpd"},{"objRefNS:bpd"},{"objRefNS:bpd"},{"objRefNS:bpd"},{"objRefNS:bpd"}}};';

      
      put "include( JMPClinicalDir||""JSLFiles/Findings Drill Down Expressions.jsl"");";
      put "InsertInto(objRefNS:ActionButtonDesc , {Expr(locale:get(""ClicktoView_Text""))});";;
      put "objRefNS:ActionButtonList[2]={expr(locale:get(""GraphTimeTrends_Text""))};";
      put "objRefNS:ActionCodeList[2] ={objRefNS:BPGraphTimeTrends()};";
      put "objRefNS:ActionIconList[2] = {""&IconPath.LinesPlot.gif""};";
      put "objRefNS:ActionDataTableList[2]  = {{""objRefNS:bpd""}};";
      put "objRefNS:ActionSelectionCountList[2] = {1};";
      put "objRefNS:ActionRequirementFlagList[2] = {0};"; 
      
   run;
   
%mend;



%macro BoxPlotsAnalysis(_dom,ResponseVar);
      
      %let trTrialStats = %nrbquote(%LocaleGet(TrialStatistics_Text));
      %let trTrialMax = %nrbquote(%LocaleGet(TrialMax_Text));
      %let trTrialMean = %nrbquote(%LocaleGet(TrialMean_Text));
      %let trTrialMedian = %nrbquote(%LocaleGet(TrialMedian_Text));
      %let trTrialMin = %nrbquote(%LocaleGet(TrialMin_Text));
      %let trTimeWindows = %nrbquote(%LocaleGet(TimeWindows_Text));
      %let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));
      
      %if %UPCASE(&TimeScale) ne WEEKS %then %do;
         %let STvar = STDY;
         %let ENvar = STDY;
         %let TimeLabel = %nrbquote(%LocaleGet(StudyDay_Text));
      %end;
      %else %do;
         %let STvar = STWK;
         %let ENvar = STWK;
         %let TimeLabel = %nrbquote(%LocaleGet(StudyWeek_Text));
      %end;
      
      data _std; 
         set &_dom;
         %if (&fvisit and %upcase(&RemoveUVs) = YES and %upcase(&TimeScale)=VISITS) %then %do;
            where upcase(VISIT) ^contains "UNSCHEDULED" and kupcase(VISIT) ^contains "%kupcase(&trUnscheduled)";
         %end;
         if &ResponseVar = . then delete;;   
         %if %upcase(&TimeScale)=WEEKS %then %do;
            STWK = ceil(STDY/7);
         %end;
      run;
      %if &exerr %then %goto mexit;
      
      %nobsvars(_std);
      %if &nobs = 0 %then %do;
         %put [MS_S_0226];
         %put ERROR: There are no observations in the analysis data set after deleting missing values of &ResponseVar.;
         %let exiterror=1;
         %goto mexit;
      %end;
      
      proc means data=_std %if &blflag and &calcbl %then (where=(&BlFlagVar ne 'Y')); noprint;
         var &STvar;
         output out=mm min=MinTime max=MaxTime;
      run;
      
      %nobsvars(mm);
      %if &nobs = 0 %then %do;
         %put [MS_S_0227];
         %put ERROR: All measurements are flagged as baseline for the trial, change from baseline cannot be computed and box plots cannot be plotted.;
         %let exerr = 1;
         %goto mexit;
      %end;
      
      data _null_;
         set mm;
         call symput('MinTime',compress(MinTime));
         call symput('MaxTime',compress(MaxTime));
      run;
      
      %put MinTime = &MinTime;
      %put MaxTime = &MaxTime;
      %put BlFlag = &blflag;
      
      /*kemicl add test to group sort vars to be able to use it for Graph Labels */
      %if &_test_ %then %let GroupSortVars = &GroupSortVars &_dom.TEST;
      
      %if &calcbl %then %do;
         /* call macro to check and create Baseline Time window and prep for creating Time window*/
         %CreateBaselineTW;
         %if &exerr %then %goto mexit;
      %end;
      %else %let BaselineTimeWindow = ;
      
      /* set time points method*/
      %if ^%length(&TimeWindows) %then %do;
         %if %index(&TimeWindowMethod,Use) %then %do;
            %put [MS_S_0228];
            %put ERROR: A list of time windows must be specified;
            %let exerr = 1;
            %goto mexit;
         %end;
         %let _inc_ = %sysevalf(%sysfunc(ceil((&MaxTime - &MinTime)/&NWindows)));
         %let winbeg = &MinTime;
         %let winend = %eval(&MinTime+&_inc_);
         %let TimeWindows = %str(&TimeWindows [&winbeg,&winend]);

         %do i = 2 %to &NWindows;
            %let winbeg = %eval(&winend +1);
            %let winend = %eval(&winend + &_inc_);
            %let TimeWindows = %str(&TimeWindows [&winbeg,&winend]);
         %end;
      %end;
      
    %put Time Windows List = &TimeWindows;
    %let TimeWindows = %str(&BaselineTimeWindow &TimeWindows); 
    %global TimeVar;
     %let TimeVar = TimeWindow;
     %if &blflag %then %do;
         %AddTimeWindows(domain=&_dom, indat=_std,outdat=_std,twb=%nrbquote(&TimeWindows), tvar=&TimeVar,  findingscall=1,blflag=1);
     %end;
     %else %do;
         %AddTimeWindows(domain=&_dom, indat=_std,outdat=_std,twb=%nrbquote(&TimeWindows), tvar=&TimeVar,  findingscall=1);
     %end;     
      /* data outlib._std0; set _std; run; */
     
      %if &calcbl %then %do;
         %SplitBaselineTrial(&_dom, dset=_std, TimeV=&TimeVar, doblchg = 1);
         %if &exerr %then %goto mexit;
      %end;
      %else %do;
         %global RVarLab;
         %let RVarLab = %LocaleEZX(%nrbquote(&MeasurementValues), EIA=1);
      %end;
      
      %if %index(&TimeWindowMethod,None) %then %do;
         %let STvar = ;
         /* calculate  max and mins:*/
         proc sort data=_std;
            by USUBJID &GroupSortVars &_dom.TESTCD;
         run;
         
         *compute max, mean, median, and min for each subject/TESTCD across the whole trial;
         proc means data=_std noprint;
            by USUBJID &GroupSortVars &_dom.TESTCD;
            var &ResponseVar;
            output out=_max(drop=_Type_ _freq_) max=&ResponseVar;
            output out=_mean(drop =_Type_ _freq_) mean=&ResponseVar;
            output out=_median(drop =_Type_ _freq_) median=&ResponseVar;
            output out=_min(drop=_Type_ _freq_) min=&ResponseVar;
         run;

         data _bxall; 
            length &_dom.TESTCD $90 &TimeVar $25;
            set _max(in=a) _mean(in=b) _median(in=d) _min(in=e);
            label &TimeVar="&trTrialStats";
            if a then do;
               &TimeVar = "&trTrialMax";
            end;
            if b then do;
               &TimeVar = "&trTrialMean";
            end;
            if d then do;
               &TimeVar = "&trTrialMedian";
            end;
            if e then do;
               &TimeVar = "&trTrialMin";
            end;           
            &_dom.TESTCD = trim(left(&_dom.TESTCD))||" &RVarLab";
            &_dom.TESTCDL = &_dom.TESTCD;
         run;  
      %end; 
      %else %do; /* code for when using Time Windows */
         %if %upcase(&TimeScale) = VISITS %then %do;
            %let TimeVar = &VTimeVar;
         %end;
         
         proc sort data=_std;
            by USUBJID &TimeVar &GroupSortVars &_dom.TESTCD;
         run;
         
         * compute averages for each unique time point;
         proc means data=_std noprint;
            by USUBJID &TimeVar &GroupSortVars &_dom.TESTCD %if %upcase(&TimeScale) = VISITS %then &TimeVar;;
            var &ResponseVar &STVar;
            output out=_stdm &BPStat=&ResponseVar &STVar;
         run;
         
         %let trBPStat = %LocaleEZX(%nrbquote(&BPStat), EIA=1);
         
         %if %upcase(&TimeScale) ne VISITS %then %do;
            * calculate  max and mins;
            proc sort data=_std;
               by USUBJID &GroupSortVars &_dom.TESTCD;
            run;

            *compute max and min for each subject/TESTCD;
            proc means data=_std noprint;
               by USUBJID &GroupSortVars &_dom.TESTCD;
               var &ResponseVar;
               output out=_max(drop=_Type_ _freq_) max=&ResponseVar;
               output out=_min(drop=_Type_ _freq_) min=&ResponseVar;
            run;

            data _bxall; 
               length &_dom.TESTCD $90  &TimeVar $25;
               label &StVar = "&TimeLabel";
               set _stdm _min(in=a) _max(in=b);
               label &TimeVar="&trTimeWindows";
               if a then do;
                  &STvar = ceil(&MaxTime + ((&MaxTime - &MinTime)/5));
                  &TimeVar = "&trTrialMin";
               end;
               if b then do;
                  &STvar= ceil(&MaxTime + 2*((&MaxTime-&MinTime)/5));
                  &TimeVar = "&trTrialMax";
               end;
               &_dom.TESTCD = trim(left(&_dom.TESTCD))||" &trBPStat &RVarLab";
               &_dom.TESTCDL = &_dom.TESTCD;
            run;  
         %end;
         %else %do;
            data _bxall; 
               length &_dom.TESTCD $90;
               label &StVar = "&TimeLabel";
               set _stdm;
               &_dom.TESTCD = trim(left(&_dom.TESTCD))||" &trBPStat &RVarLab";
               &_dom.TESTCDL = &_dom.TESTCD;
            run;
         %end;
      %end;    

      proc sort data=_bxall;
         by USUBJID &StVar &TimeVar &GroupSortVars ;
      run;
       
      * transpose to get into form required;
      proc transpose data=_bxall out=_box;
         by USUBJID &StVar &TimeVar ;
         var &ResponseVar;
         id &_dom.TESTCD;
         idlabel &_dom.TESTCDL;
      run;
         
      %DMFlags;

      proc sql;
         create table OutLib.&OutDataName as
            select *
            from _box _ds1, &in_adsl _ds2
            where _ds1.USUBJID=_ds2.USUBJID;
      quit;
      
      %nobsvars(OutLib.&OutDataName);
      %if &nobs = 0 %then %do;
         %put [MS_S_0229];
         %put ERROR: The merge of &&in_&_dom and &in_adsl data sets resulted in 0 observations. No subject identifiers match between the two data sets found in the defined Study folders.;
         %put ERROR: Please check that the data sets in the Study folders are accurate.;
         %let exiterror = 1;
         %goto mexit;
      %end; 
      
      %if %upcase(&TimeScale) = VISITS %then %do;
         %if &fvisit %then %do;
            %let TimeVar = VISIT;
            %if &fvisitn %then %do;
               %let TimeLabel = VISITNUM;
               %CheckLabel(_bxall,TimeLabel);
            %end;
         %end;
         %else %let TimeVar = VISITNUM;
      %end;
         
      %global TimeWinLabel; 
      %let TimeWinLabel = &TimeVar;
      %CheckLabel(OutLib.&OutDataName,TimeWinLabel);
      %let SubjectLabel = USUBJID;
      %CheckLabel(OutLib.&OutDataName,SubjectLabel);

      %let odata = OutLib.&OutDataName;
      %include "&MacroPath./DesignByVars.sas";
      
      /* Kemicl added to check if there were time intervals with no data */
      %global misstimes;
      %let misstimes = 0;
      %if %index(&TimeWindowMethod,None)=0 %then %do;
         proc sort data=OutLib.&OutDataName out=gettw nodupkey;
            by &TimeVar;
         run;

         %nobsvars(gettw); 

         %if &nobs < &ntw %then %let misstimes = 1;
      %end;
      
   %mexit:;
    %if &syserr > 4 or &exerr=1 %then %do;
      %let exiterror = 1;
    %end; 
    
%mend;


%macro BoxPlot_JSL(_JSLFile, _dom, _DomName);

   *determine number of measurment levels and make macros for graphs;
   proc sort data=_bxall nodupkey out=vals(keep= &GroupSortVars &_dom.TESTCD) ;
      by &GroupSortVars &_dom.TESTCD;
   run;

   data vals;
      set vals end=_last;
      call symput("testlev"||trim(left(_n_)),trim(left(&_dom.TESTCD)));
      %if &_test_ %then call symput("testnamelev"||trim(left(_n_)),trim(left(&_dom.TEST)));;
      if _last then call symput("nvals", _n_);
   run;
   
   %if &funits or &fcat %then %do;
      /* get units of measurement */
      /* Get order of GroupVars */
      proc sort data = _std(keep=&GroupSortVars &_dom.TESTCD %if &funits %then &_dom.STRESU;) out=gunits nodupkey;
         by &GroupSortVars &_dom.TESTCD;
      run;

      /*kemicl:S1049452 - add check for unique values of test code across categories and group sort variables */
      %nobsvars(gunits);
      %let ntottest = &nobs;

      proc sort data = gunits out=uttests nodupkey;
         by &_dom.TESTCD;
      run;
      %nobsvars(uttests);

      %if &nobs < &ntottest %then %do;
         %put [MS_S_0110];
         %put ERROR: Analysis cannot be completed because the &TestVar values are not unique across values of test category and/or subcategory.;
         %let exiterror = 1;
         %goto mexit;
      %end;

      data gunits; 
         set gunits end=_last;
         %if &fcat %then %do;
            by &GroupSortVars;
         %end;
         %if &funits %then call symput("testunits"||trim(left(_n_)),trim(left(&Fdomain.STRESU)));;
         %if &fcat %then %do;
            if first.&FDomain.CAT then call symput("Cat"||trim(left(_n_)),trim(left(&FDomain.CAT)));
            %if &fscat %then %do;
               if first.&FDomain.SCAT then call symput("SCat"||trim(left(_n_)),trim(left(&FDomain.SCAT)));
            %end;
         %end;
      run;
   %end; 

   %let Limitlines = 0;
   %if %upcase(&DisplayLimits) = YES %then %do;
      %if %sysfunc(findw(&LBVARS, LBSTNRHI)) > 0 and  %sysfunc(findw(&LBVARS, LBSTNRLO)) > 0 %then %do;
         %let LimitLines = 1;
         proc sort data = _std(keep=&GroupSortVars &_dom.TESTCD LBSTNRHI LBSTNRLO) out=limits;
            by &GroupSortVars &_dom.TESTCD;
         run;
         
         proc means data = limits noprint;
            by &GroupSortVars &_dom.TESTCD;
            var LBSTNRHI LBSTNRLO;
            output out=minmaxs min(LBSTNRHI)=MinHi min(LBSTNRLO)=MinLo max(LBSTNRHI)=maxhi  max(LBSTNRLO) = maxlo;
         run;
         
         data minmaxs; 
            set minmaxs end=_last;
            call symput("minHi"||trim(left(_n_)),trim(left(MinHi)));
            call symput("maxHi"||trim(left(_n_)),trim(left(MaxHi)));
            call symput("minLo"||trim(left(_n_)),trim(left(MinLo)));
            call symput("maxLo"||trim(left(_n_)),trim(left(MaxLo)));
         run;
         
      %end;
   %end;

   %if %upcase(&DisplaySummaryStats) = YES & %upcase(&TimeScale) = VISITS %then %do;
      %if &fvisit & &fvisitn %then %do;
         proc sort data=_bxall(keep = VISIT VISITNUM) out=visnums nodupkey;
            by VisitNum Visit;
         run;
         
         data visnums; 
             set visnums end=_last;
             where not missing(VISIT);
             call symput("oVisits"||trim(left(_n_)),trim(left(VISIT)));
             if _last then call symput("nvisits", _n_);
         run;  
         
         %let _visitList_ = ""%trim(&oVisits1)"";
         %do v = 1 %to %eval(&nvisits);
            %let _visitList_ = &_visitList_ , ""%trim(&&oVisits&v)"";
         %end; 
      %end;
   %end;   
         
   %DMLabels(OutLib.&OutDataName);

   %CheckStudyLevels;

   %let trULN = %nrbquote(%LocaleGet(ULN_Text));
   %let trLLN = %nrbquote(%LocaleGet(LLN_Text));
   %let trMin = %nrbquote(%LocaleGet(Min_Text));
   %let trMax = %nrbquote(%LocaleGet(Max_Text));
   
   * put JSL statements;
  data _null_;
   file "&_JSLFile" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";
      put "objRefNS:DataList = {};";
      put "objRefNS:TabButtonList = {};";
      put "objRefNS:TabCodeList = {};";
      put "objRefNS:TabAfterCodeList = {};";
      put "objRefNS:TabOpenList = {};";
      put "objRefNS:CreateReportList = {};";
      put "objRefNS:NoteDomain = ""&_dom"";";      
      
      put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
      put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
      
      %if %upcase(&ProcessName) = FINDINGSBOXPLOTS %then %do;
               put "objRefNS:includestring=""Findings Tests: "";";
               %let DomainWhereStmt = &WhereTests;
      %end;
      
      put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";
            
      %if %length(&WhereForReports) %then %do;
         put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
         put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
         %if %length(&DomainWhereStmt) %then %do;
            put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
         %end;
      %end;
      %else %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;
      put "   objRefNS:openDTExpr = Expr(If(IsEmpty(objRefNS:bpd),";
      put "objRefNS:bpd = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      put "    Column(""&SubjectLabel"")<<Label(1);";
      put "    Column(""&SubjectLabel"")<< Set Modeling Type( Nominal);";
      %if &fvisitn %then %do;
         put "    Column(""&TimeWinLabel"")<< Set Modeling Type( Ordinal);";
      %end; 
      %if (%upcase(&DisplaySummaryStats) = YES & %upcase(&TimeScale) = VISITS & &fvisit & &fvisitn) %then %do;
        put "  Column(""&TimeWinLabel"")<< Set Property(""Value Ordering"",{&_visitList_});";
      %end;
      put "    ,"; /*else*/
      put "     Current Data Table(objRefNS:bpd);";
      put "    ););";
      put " objRefNS:openDTExpr;";
      put "objRefNS:fsize = 75;";
      
      put "objRefNS:bpfunc=Function({},{default local},";
      put "objRefNS:fsize = 75;";   
      put " if(""&MeasurementValues"" == ""Observed Result"", objRefNS:trMeasurementValues = locale:get(""ObservedResult_Text""),";
      put "    ""&MeasurementValues"" == ""Change from Baseline"", objRefNS:trMeasurementValues = locale:get(""ChangefromBaseline_Text""),";
      put "    ""&MeasurementValues"" == ""Percent Change from Baseline"", objRefNS:trMeasurementValues = locale:get(""PercentChangefromBaseline_Text""),";
      put "    ""&MeasurementValues"" == ""Percent of Baseline"", objRefNS:trMeasurementValues = locale:get(""PercentofBaseline_Text""),";
      put "    objRefNS:trMeasurementValues = ""&MeasurementValues"");";   
         
      %if &misstimes %then %do;
         put "If(IsEmpty(objRefNS:mtnw),";
         put "objRefNS:mtnw = New Window(locale:fill(locale:get(""XMessage_Text""), {locale:get(""FindingsBoxPlots_Text"")}),showmenu(0),showtoolbars(0),";
         put " TextBox(locale:get(""NoteSomeTimeWindowsNotDispifNoFindMeasinTimeInt_Text""),";
         put "    <<Font Color(5),<<Set Font Size(11),<<Set Wrap(400)";
         put " )";
         put "));";
      %end;
      put " objRefNS:openDTExpr;";
      put "   VListBox(";
      put " objRefNS:vlb = VListBox(";
      %do i=1 %to &nvals;
         %if &fcat %then %do;
            %if %symexist(Cat&i) %then %do;
               put " OutlineBox(locale:fill(locale:get(""XTestCategory_Text""), {""&FDomain""}) || "" = &&Cat&i"",";
            %end;
            %if %symexist(SCat&i) %then %do;
               %if %length(&&SCat&i) %then %do;
                  put " TextBox(locale:fill(locale:get(""XTestSubcategory_Text""), {""&FDomain""}) || "" = &&Scat&i""),";
               %end;
            %end;
         %end;    
         put "    objRefNS:vgb&i = VListBox(";
         put "    objRefNS:gb&i = Graph Builder(";
         %if %upcase(&SplitResults)=YES %then %do;
            put "       Size(750,550),";
         %end;
         %else %do;
            put "       Size(750,415),";
         %end; 
         put "       Show Control Panel(0), AutoStretching(0),";
         put "       Where(!IsMissing(:Name(""&&testlev&i""))),";
         put "       X Group Edge(""Bottom""),";
         put "       Variables(";
         put "          Y(Column(""&&testlev&i"")),";
         put "          Group X(";
         put "             :&TimeWinLabel";
         %if %index(&TimeWindowMethod,None)=0 and %upcase(&TimeVar) ne VISITNUM %then %do;
            put "             ,Order By(Column(""&TimeLabel""), Ascending, Order Statistic(""Mean""))";
         %end;
         put "          ),";
         %if %upcase(&SplitResults)=YES %then %do;
            %do j=1 %to &ndbv;
               put "       Group Y(Column(""&&dbv&j"")),";
            %end;
         %end;
         put "          Overlay(Column(""&TrtLabel""))";
         put "       ),";
         put "       Elements(";
         put "          Box Plot(";
         put "             Y, Legend(1), Jitter(0), Outliers(1), Box Style(""Solid"")";
         put "          )";
         put "       )";
         put "       ,SendToReport(";
         %if &_test_ %then %do;
            put "    Dispatch({},";
            put "       ""Graph Builder"",";
            put "       OutlineBox,";
            put "       {Set Title( locale:fill(locale:get(""XforX_Text""), {objRefNS:trMeasurementValues, ""&&testnamelev&i""}) )}";
            put "    ),";
         %end;            
         %if &funits %then %do;
            put "       Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""&&testlev&i (&&testunits&i)"" )} )";
         %end;
         put "    ),"; 
         %if &LimitLines %then %do;
            %if &&minHi&i ne . or &&maxHi&i ne . %then %do;
               %if &&minHi&i = &&maxHi&i %then %do;
                  put "    ,Dispatch( {}, ""&&testlev&i"", ScaleBox, {Add Ref Line( &&minHi&i, Dashed, ""Medium Dark Gray"", ""&trULN"" )})";
               %end;
               %else %do;
                  /*
                  %if &&minHi&i ne . %then %do;
                     put "    ,Dispatch( {}, ""&&testlev&i"", ScaleBox, {Add Ref Line( &&minHi&i, Dashed, ""Medium Dark Gray"", ""&trMin &trULN"" )})";
                  %end;
                  */
                  %if &&maxHi&i ne . %then %do;
                     put "    ,Dispatch( {}, ""&&testlev&i"", ScaleBox, {Add Ref Line( &&maxHi&i, Dashed, ""Medium Dark Gray"", ""&trMax &trULN"" )})";
                  %end; 
               %end;
            %end;
            %if &&minLo&i ne . or &&maxLo&i ne . %then %do;
               %if &&minLo&i = &&maxLo&i %then %do;
                  put "    ,Dispatch( {}, ""&&testlev&i"", ScaleBox, {Add Ref Line( &&minLo&i, Dashed, ""Medium Dark Gray"", ""&trLLN"" )})";
               %end;
               %else %do;
                  %if &&minLo&i ne . %then %do;
                     put "    ,Dispatch( {}, ""&&testlev&i"", ScaleBox, {Add Ref Line( &&minLo&i, Dashed, ""Medium Dark Gray"", ""&trMin &trLLN"" )})";
                  %end;
                  /*
                  %if &&maxLo&i ne . %then %do;
                     put "    ,Dispatch( {}, ""&&testlev&i"", ScaleBox, {Add Ref Line( &&maxLo&i, Dashed, ""Medium Dark Gray"", ""&trMax &trLLN"" )})";
                  %end;
                  */
               %end;             
            %end;
         %end;          
         put "    ),"; /* end for each graph builder plot and */
         %if %upcase(&DisplaySummaryStats) = YES %then %do;
            put " OutlineBox(locale:fill(locale:get(""XforXSumStatTables_Text""), {objRefNS:trMeasurementValues, ""&&testnamelev&i""}),";
            put " SpacerBox(),";
            put " Tabulate(Show Control Panel( 0 ),";
            put "    Set Format(";
            put "       Mean( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       StdDev( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       Min( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       Max( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       StdErr( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       CV( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       Median( :Name(""&&testlev&i"")( 10, &NumDecimals ) ),";
            put "       Quantiles( :Name(""&&testlev&i"")( 10, &NumDecimals ) )";
            put "    ),";
            put "    By(:Name(""&trtLabel"")),";
            put "       Where(!IsMissing(:Name(""&&testlev&i""))),";          
            put "    Add Table(";
            put "       Column Table(Grouping Columns(:Name(""&TimeWinLabel""))),";
            put "       Row Table(Analysis Columns(:Name(""&&testlev&i"")),";
            put "          Statistics(";
            %let jj = 1;
            %do %while(%length(%scan(&SummaryStatList,&jj,' ')));
               %let currstat = %scan(&SummaryStatList,&jj,' ');
               put "          &currstat,";
               %let jj = %eval(&jj+1);
            %end;
            put " ))))";
            put " )"; 
         %end;         
         put "),";  /* end tab vlistbox*/
         %if &i < &nvals and &fcat %then %do;
               %let ni = %eval(&i+1);
               %if %symexist(Cat&ni) %then %do;
                  put "   ),";  /* end each outline box for test category*/
               %end;
            %end;
         %if &i = &nvals and &fcat %then %do;
            put "    )";  /* close last outline box if xxCat used */
         %end;       
      %end;
      put " )";  /*end vlist*/
      put "));";
      
      put "objRefNS:CustomPanel = Expr(objRefNS:Filter=  objRefNS:bpd<<Data Filter(<<Conditional(1),";
      put "    Add Filter(";
      put "       Columns(";
      %if &doStudy %then %do;
         put " :Name(""&StudyLabel""),";
      %end;
      %if &doAge %then %do;
         put " :Name(""&AgeLabel""),";
      %end;
      %if &doSex %then %do;
         put " :Name(""&SexLabel""),";
      %end;
      %if &doRace %then %do;
         put " :Name(""&RaceLabel""), ";
      %end;
      %if &doSite %then %do;
         put "       :Name(""&SiteLabel""),";
      %end;      
      put" :Name(""&TrtLabel"")),";
      put "    ),";
      put "    Mode(Show(1), Include(1))";
      put " )";
      put ");";      

      put "  objRefNS:reportACupdates = Expr(";
      %do i = 1 %to &nvals;
         put "if (System Service( Get Language Code ) != ""zh-CN"",";
            put "    try(objRefNS:vgb&i[Text Box(1)]<<Delete;);";
         put ");";      
      %end;
      put " );";

      put 'InsertInto(objRefNS:DataList,{{"objRefNS:bpd"}});';
      put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XBoxPlots_Text""),{""&_DomName""})));";
      put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:bpfunc()));';
      put 'InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:reportACupdates));';
      put 'InsertInto(objRefNS:TabOpenList,1);';
      put "InsertInto(objRefNS:CreateReportList, {{objRefNS:vlb}});";
      
      %SubjectDrillDowns;
   run;
   
   %mexit:;
    %if &syserr > 4 or &exerr=1 %then %do;
      %let exiterror = 1;
    %end; 
%mend;
