/*
Name: SAS Clinical Toolkit Macros
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   1) CSTMETADATA: Creates metadata files from a directory of SDTM or ADAM data
   2) CSTCRTDDS:   Creates CRTDDS Version 1.0 files from meta data files
   3) DEFINEXML:   Creates Define.xml from CRTDDS files
   
The above are modified driver programs provided with the SAS Clinical Toolkit.  These macros have been removed.

   4) MAKEDIR:          Checks for existence of a directory and creates if not available
   5) CREATEMETADATA:   Calls CST macros to generate metadata and define.xml files for SDTM and ADaM 
   6) IDENTIFYDOMAINS:  Generates a list of all domains from the SDTM/ADAM directories that are of 
                        length 2 and are not ADaM BDS.  A flag indicating presence or  absence of ADSL 
                        is also generated.  Domain Class is derived within this macro
   7) MISSVARS:         Creates data set of null variables                        
   8) METADATAOUTFILES: Creates text files of DM and ADSL Variables, Findings Domains Test Codes,
                        Domain Classes (Events, Interventions and Findings), and CSTMacros.sas which
                        contains macro variable lists of a) domains and b) variables present within
                        each data set considering whether domain is represented by ADAM or SDTM and
                        whether supplemental qualifier data is present from SUPPXX data sets. Also
                        assigns domain libraries to each available domain.
   9) CREATEFILTER      Creates a SAS data set that can be used to create where statements on the fly
  10) GENERATEMETA      Creates all needed metadata for a study
  11) CHKSTUDYNAME      Checks for Special Characters
  12) KEYS              Derives keys based on data set sort or SDTM IG
  13) COMPARE           When NewSnap = Yes, the current data directories will be compared to previous
                        directories to identify any new or modified records
  14) GETPREVDATA       Creates a DeltaFile# file which contains libnames and data set macro variables
                        of previous snapshot
  15) MAKENOTES         Creates notes directory, ADSLRS and ADSLRSL and updates ADSLRS with new subjects
  16) UPDATESUBJECTFLAGS Changes flags in ADSLRS
  17) TOOLKITME         Gets necessary study metadata
*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

*options nomprint nomlogic nosymbolgen;

%macro TOOLKITME;
   %if %length(&InPath) %then %do;
      proc contents data = sdtm._all_ noprint out = sdtm(keep = memname memlabel name label nobs where = (nobs > 0));
      run;
      %if %symexist (SDTMDom) %then %do;
          %if %length(&SDTMDom) %then %do;
            %let i = 1;
            %let quotedSDTM =;
            %do %while(%length(%scan(&SDTMDom,&i,' ')));
               %let SDTMDomV&i = %upcase(%scan(&SDTMDom,&i,' '));
               %let quotedSDTM = &quotedSDTM "&&SDTMDomV&i";
               %let i = %eval(&i+1);
            %end;
            data sdtm;
               set sdtm;   
               if upcase(memname) in (&quotedSDTM);;
            run;
          %end;
      %end;
      ods exclude all;
      ods noresults;
      proc datasets lib=sdtm memtype=data nolist;
         ods output members=sdtmnousedDom (keep=Name MemType);
         contents data=_all_;
      quit;
      ods exclude none;
      ods results; 
      data sdtmnousedDom;
         set sdtmnousedDom;
         Folder = "SDTM";
      run;
   %end;
   %if %length(&AInPath) %then %do;
      proc contents data = adam._all_ noprint out = adam(keep = memname memlabel name label nobs where = (nobs > 0));
      run;
      %if %symexist (ADaMDom) %then %do;
          %if %length(&ADaMDom) %then %do;
            %let i = 1;
            %let quotedADaM =;
            %do %while(%length(%scan(&ADaMDom,&i,' ')));
               %let ADaMDomV&i = %upcase(%scan(&ADaMDom,&i,' '));
               %let quotedADaM = &quotedADaM "&&ADaMDomV&i";
               %let i = %eval(&i+1);
            %end;
            data adam;
               set adam;   
               if upcase(memname) in (&quotedADaM);;
            run;
          %end;
      %end;
      ods exclude all;
      ods noresults;
      proc datasets lib=adam memtype=data nolist;
         ods output members=adamnousedDom (keep=Name MemType);
         contents data=_all_;
      quit;
      ods exclude none;
      ods results; 
      data adamnousedDom;
         set adamnousedDom;
         Folder = "ADaM";
      run;
   %end;
    *** Determine if SEND model based on presence of SETCD ***;
    data dm;
         set 
      %if %length(&InPath) %then %do; sdtm(where = (memname = "DM" and name = "SETCD")) %end;
      %if %length(&AInPath) %then %do; adam(where = (memname in ("ADSL", "ADDM") and name = "SETCD")) %end;
      ;
    run;
   %NObsVars(dm);
    %if &nobs %then %let SEND = 1;
   %else %let SEND = 0;

   proc datasets nolist;
      delete dm;
   quit;
   
   %let trDemographics = %nrbquote(%LocaleGet(Demographics_Text));
   %let trSubjectElements = %nrbquote(%LocaleGet(SubjectElements_Text));
   %let trComments = %nrbquote(%LocaleGet(Comments_Text));
   %let trExposure = %nrbquote(%LocaleGet(Exposure_Text));
   %let trDisposition = %nrbquote(%LocaleGet(Disposition_Text));
   %let trLaboratoryTestResults = %nrbquote(%LocaleGet(LaboratoryTestResults_Text));
   %let trVitalSigns = %nrbquote(%LocaleGet(VitalSigns_Text));
   %let trECGTestResults = %nrbquote(%LocaleGet(ECGTestResults_Text));
   %let trPKConcentrations = %nrbquote(%LocaleGet(PKConcentrations_Text));
   %let trPKParameters = %nrbquote(%LocaleGet(PKParameters_Text));
   %let trSubjectCharacteristics = %nrbquote(%LocaleGet(SubjectCharacteristics_Text));
   %let trTrialArms = %nrbquote(%LocaleGet(TrialArms_Text));
   %let trTrialElements = %nrbquote(%LocaleGet(TrialElements_Text));
   %let trTrialSummary = %nrbquote(%LocaleGet(TrialSummary_Text));
   %let trRelatedRecords = %nrbquote(%LocaleGet(RelatedRecords_Text));
   %let trSupplementalQualifiers = %nrbquote(%LocaleGet(SupplementalQualifiers_Text));
   %let trBodyWeights = %nrbquote(%LocaleGet(BodyWeights_Text));
   %let trBodyWeightGains = %nrbquote(%LocaleGet(BodyWeightGains_Text));
   %let trClinicalObservations = %nrbquote(%LocaleGet(ClinicalObservations_Text));
   %let trDeathDiagnosis = %nrbquote(%LocaleGet(DeathDiagnosis_Text));
   %let trFoodandWaterConsumption = %nrbquote(%LocaleGet(FoodandWaterConsumption_Text));
   %let trMacroscopicFindings = %nrbquote(%LocaleGet(MacroscopicFindings_Text));
   %let trMicroscopicFindings = %nrbquote(%LocaleGet(MicroscopicFindings_Text));
   %let trOrganMeasurements = %nrbquote(%LocaleGet(OrganMeasurements_Text));
   %let trPalpableMasses = %nrbquote(%LocaleGet(PalpableMasses_Text));
   %let trTumorFindings = %nrbquote(%LocaleGet(TumorFindings_Text));
   %let trTrialSets = %nrbquote(%LocaleGet(TrialSets_Text));
   %let trPooling = %nrbquote(%LocaleGet(Pooling_Text));
   %let trSubjectVisits = %nrbquote(%LocaleGet(SubjectVisits_Text));
   %let trConcomitantMedications = %nrbquote(%LocaleGet(ConcomitantMedications_Text));
   %let trSubstanceUse = %nrbquote(%LocaleGet(SubstanceUse_Text));
   %let trAdverseEvents = %nrbquote(%LocaleGet(AdverseEvents_Text));
   %let trMedicalHistory = %nrbquote(%LocaleGet(MedicalHistory_Text));
   %let trClinicalEvents = %nrbquote(%LocaleGet(ClinicalEvents_Text));
   %let trProtocolDeviations = %nrbquote(%LocaleGet(ProtocolDeviations_Text));
   %let trQuestionnaires = %nrbquote(%LocaleGet(Questionnaires_Text));
   %let trMicrobiologySpecimen = %nrbquote(%LocaleGet(MicrobiologySpecimen_Text));
   %let trInclusionExclusionCritNotMet = %nrbquote(%LocaleGet(InclusionExclusionCriterionNotMet_Text));
   %let trPhysicalExamination = %nrbquote(%LocaleGet(PhysicalExamination_Text));
   %let trDrugAccountability = %nrbquote(%LocaleGet(DrugAccountability_Text));
   %let trMicrobiologySuscTest = %nrbquote(%LocaleGet(MicrobiologySusceptibilityTest_Text));
   %let trFindingsAbout = %nrbquote(%LocaleGet(FindingsAbout_Text));
   %let trTrialVisits = %nrbquote(%LocaleGet(TrialVisits_Text));
   %let trTrialInclusionExclusionCrit = %nrbquote(%LocaleGet(TrialInclusionExclusionCriteria_Text));
   %let trEvents = %nrbquote(%LocaleGet(Events_Text));
   %let trInterventions = %nrbquote(%LocaleGet(Interventions_Text));
   %let trFindings = %nrbquote(%LocaleGet(Findings_Text));
   %let trSpecial = %nrbquote(%LocaleGet(Special_Text));
   %let trTrialDesign = %nrbquote(%LocaleGet(TrialDesign_Text));
   %let trRelationship = %nrbquote(%LocaleGet(Relationship_Text));
   %let trTableName = %nrbquote(%LocaleGet(TableName_Text));
   %let trLabel = %nrbquote(%LocaleGet(Label_Text));
   %let trObservationClasswithinStand = %nrbquote(%LocaleGet(ObservationClasswithinStandard_Text));
   %let trColumnName = %nrbquote(%LocaleGet(ColumnName_Text));
   %let trSubjectLevel = %nrbquote(%LocaleGet(SubjectLevel_Text));
   
   %let trTrialDiseaseAssess = %nrbquote(%LocaleGet(TrialDiseaseAssessment_Text));
   %let trSkinResponse = %nrbquote(%LocaleGet(SkinResponse_Text));
   %let trTumorResults = %nrbquote(%LocaleGet(TumorResults_Text));
   %let trSubjectStatus = %nrbquote(%LocaleGet(SubjectStatus_Text));
   %let trDiseaseResponse = %nrbquote(%LocaleGet(DiseaseResponse_Text));
   %let trDeathDetails = %nrbquote(%LocaleGet(DeathDetails_Text));
   %let trTumorIdentification = %nrbquote(%LocaleGet(TumorIdentification_Text));
   %let trReproductiveSysFind = %nrbquote(%LocaleGet(ReproductiveSystemFindings_Text));
   %let trMorphology = %nrbquote(%LocaleGet(Morphology_Text));
   %let trImmunoSpecimenAssess = %nrbquote(%LocaleGet(ImmunogenicitySpecimenAssessments_Text));
   %let trProcedures = %nrbquote(%LocaleGet(Procedures_Text));
   %let trExposureasCollected = %nrbquote(%LocaleGet(ExposureasCollected_Text));
   %let trHealthcareEncounters = %nrbquote(%LocaleGet(HealthcareEncounters_Text));
   
   %if %length(&InPath) %then %do;  
      %if %sysfunc(fileexist(&InPath.adsl.sas7bdat)) %then %do;
         %put [MS_S_0201];
         %put ERROR: ADSL detected in the SDTM folder. Please move to an ADaM folder distinct from the SDTM folder.;
         %let CSTERROR = 1;
         %goto tkexit;
      %end;

      data sdtm;
            set sdtm(drop = nobs);
         if not (length(trim(memname)) = 2 or (substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6) or memname in ("RELREC", "POOLDEF")) then delete;
            *** Labels ***;
         if memname = "DM" and memlabel = "" then memlabel = "&trDemographics";
         else if memname = "SE" and memlabel = "" then memlabel = "&trSubjectElements";
         else if memname = "CO" and memlabel = "" then memlabel = "&trComments";
         else if memname = "EX" and memlabel = "" then memlabel = "&trExposure";
         else if memname = "DS" and memlabel = "" then memlabel = "&trDisposition";
         else if memname = "LB" and memlabel = "" then memlabel = "&trLaboratoryTestResults";
         else if memname = "VS" and memlabel = "" then memlabel = "&trVitalSigns";
         else if memname = "EG" and memlabel = "" then memlabel = "&trECGTestResults";
         else if memname = "PC" and memlabel = "" then memlabel = "&trPKConcentrations";
         else if memname = "PP" and memlabel = "" then memlabel = "&trPKParameters";
         else if memname = "SC" and memlabel = "" then memlabel = "&trSubjectCharacteristics";
         else if memname = "TA" and memlabel = "" then memlabel = "&trTrialArms";
         else if memname = "TE" and memlabel = "" then memlabel = "&trTrialElements";
         else if memname = "TS" and memlabel = "" then memlabel = "&trTrialSummary";                                            
         else if memname = "RELREC" and memlabel = "" then memlabel = "&trRelatedRecords";
         else if substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6 and memlabel = "" then memlabel = catx(" ", "&trSupplementalQualifiers", substr(memname,5,2));
         %if &SEND = 1 %then %do;
            else if memname = "BW" and memlabel = "" then memlabel = "&trBodyWeights";
            else if memname = "BG" and memlabel = "" then memlabel = "&trBodyWeightGains";
            else if memname = "CL" and memlabel = "" then memlabel = "&trClinicalObservations";
            else if memname = "DD" and memlabel = "" then memlabel = "&trDeathDiagnosis";
            else if memname = "FW" and memlabel = "" then memlabel = "&trFoodandWaterConsumption";
            else if memname = "MA" and memlabel = "" then memlabel = "&trMacroscopicFindings";
            else if memname = "MI" and memlabel = "" then memlabel = "&trMicroscopicFindings";
            else if memname = "OM" and memlabel = "" then memlabel = "&trOrganMeasurements";
            else if memname = "PM" and memlabel = "" then memlabel = "&trPalpableMasses";
            else if memname = "TF" and memlabel = "" then memlabel = "&trTumorFindings";
            else if memname = "TX" and memlabel = "" then memlabel = "&trTrialSets";
            else if memname = "POOLDEF" and memlabel = "" then memlabel = "&trPooling";
         %end;
         %if &SEND = 0 %then %do;
            else if memname = "SV" and memlabel = "" then memlabel = "&trSubjectVisits";
            else if memname = "CM" and memlabel = "" then memlabel = "&trConcomitantMedications";
            else if memname = "SU" and memlabel = "" then memlabel = "&trSubstanceUse";
            else if memname = "AE" and memlabel = "" then memlabel = "&trAdverseEvents";
            else if memname = "MH" and memlabel = "" then memlabel = "&trMedicalHistory";
            else if memname = "CE" and memlabel = "" then memlabel = "&trClinicalEvents";
            else if memname = "DV" and memlabel = "" then memlabel = "&trProtocolDeviations";            
            else if memname = "QS" and memlabel = "" then memlabel = "&trQuestionnaires";
            else if memname = "MB" and memlabel = "" then memlabel = "&trMicrobiologySpecimen";
            else if memname = "IE" and memlabel = "" then memlabel = "&trInclusionExclusionCritNotMet";
            else if memname = "PE" and memlabel = "" then memlabel = "&trPhysicalExamination";
            else if memname = "DA" and memlabel = "" then memlabel = "&trDrugAccountability";   
            else if memname = "MS" and memlabel = "" then memlabel = "&trMicrobiologySuscTest";
            else if memname = "FA" and memlabel = "" then memlabel = "&trFindingsAbout";
            else if memname = "TV" and memlabel = "" then memlabel = "&trTrialVisits";
            else if memname = "TI" and memlabel = "" then memlabel = "&trTrialInclusionExclusionCrit";
            else if memname = "EC" and memlabel = "" then memlabel = "&trExposureasCollected";
            else if memname = "PR" and memlabel = "" then memlabel = "&trProcedures";
            else if memname = "HO" and memlabel = "" then memlabel = "&trHealthcareEncounters";
            else if memname = "IS" and memlabel = "" then memlabel = "&trImmunoSpecimenAssess";
            else if memname = "MO" and memlabel = "" then memlabel = "&trMorphology";
            else if memname = "RP" and memlabel = "" then memlabel = "&trReproductiveSysFind";
            else if memname = "TU" and memlabel = "" then memlabel = "&trTumorIdentification";
            else if memname = "DD" and memlabel = "" then memlabel = "&trDeathDetails";
            else if memname = "RS" and memlabel = "" then memlabel = "&trDiseaseResponse";
            else if memname = "SS" and memlabel = "" then memlabel = "&trSubjectStatus";
            else if memname = "TR" and memlabel = "" then memlabel = "&trTumorResults";
            else if memname = "SR" and memlabel = "" then memlabel = "&trSkinResponse";     
            else if memname = "TD" and memlabel = "" then memlabel = "&trTrialDiseaseAssess";                 
         %end;
         if memlabel = "" then memlabel = memname;
      run;      
      
      *** Define user-defined classes ***;
      data sdtm_type_other(keep = memname class2);
         length Class2 $ 15;
            set sdtm(where = (substr(upcase(name),3) in ("TERM","TRT","TESTCD")));
            if substr(upcase(name),3) = "TERM" then Class2 = "&trEvents";
            else if substr(upcase(name),3) = "TRT" then Class2 = "&trInterventions";
            else if substr(upcase(name),3) = "TESTCD" then Class2 = "&trFindings";
         *** delete if term does not begin with domain prefix ***;
         if substr(upcase(name),1,2) ne memname then delete;
      run;

      *** Define Model Classes ***;
      data sdtm_type(keep = memname memlabel class);
         length Class $ 50;
         set sdtm;
         by memname;
         if first.memname;
         %if &SEND = 0 %then %do;
            if memname in ("DM","SE","CO","SV") then class = "&trSpecial";
            else if memname in ("CM","SU","EX") then class = "&trInterventions";
            else if memname in ("AE","MH","CE","DS","DV") then class = "&trEvents";
            else if memname in ("EG","LB","QS","VS","MB","PC","IE","PE","SC","DA","MS","PP") then class = "&trFindings";
            else if memname in ("FA") then class = "&trFindingsAbout";
            else if memname in ("TA","TV","TS","TE","TT") then class = "&trTrialDesign";
            else if memname = "RELREC" or (substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6) then class = "&trRelationship";
         %end;
         %else %if &SEND = 1 %then %do;
            if memname in ("DM","SE","CO") then class = "&trSpecial";
            else if memname in ("EX") then class = "&trInterventions";
            else if memname in ("DS") then class = "&trEvents";
            else if memname in ("BW","BG","CL","DD","FW","LB","MA","MI","OM","PM","PC","PP","SC","TF","VS","EG") then class = "&trFindings";
            else if memname in ("TE","TA","TX","TS") then class = "&trTrialDesign";
            else if memname = "RELREC" or (substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6) then class = "&trRelationship";
         %end;
      run;

      *** Fill in user-defined classes ***;
      *** Delete where class missing ***;
      data sdtm_type;
         merge sdtm_type sdtm_type_other;
         by memname;
         if class = "" and class2 ne "" then class = class2;
         if class = "" then delete;
      run;

      data sdtm_type;
         set sdtm_type;
         by memname;
         retain count;
         if first.memname then count = 1;
         else count = count + 1;
         if last.memname;
      run;

      *** Delete user-defined domains where class cannot be determined ***;
      data sdtm_type;
         set sdtm_type;
         if count > 1 then delete;
      run;
      
      *** Keep variables for domains with defined classes ***;
      data sdtm;
         merge sdtm_type(keep = memname in = ina) sdtm;
         by memname;
         if ina;
      run;

      data sdtmmeta.source_tables(keep = memname memlabel class rename = (memname = Table memlabel = Label));
         retain memname memlabel class;
         set sdtm_type(where = (upcase(memname) not in ("POOLDEF","RELREC")));
         label memname = "&trTableName" memlabel = "&trLabel" class = "&trObservationClasswithinStand";
      run;

      data sdtmmeta.source_columns(keep = memname name label rename = (memname = Table name = Column));
         retain memname name label;
         set sdtm(where = (upcase(memname) not in ("POOLDEF","RELREC")));
         name = upcase(name);
         label memname = "&trTableName" label = "&trLabel" name = "&trColumnName";
      run;

      proc datasets nolist;
         delete sdtm sdtm_type sdtm_type_other;
      quit;
      
      *** Findings Check. Drop those without xxTESTCD ***;  
      proc sort data = sdtmmeta.source_tables out = sdtm1;
         by table;
      run;

      proc sort data = sdtmmeta.source_columns out = sdtm2;
         by table column;
      run;
      
      data findchk1;
         set sdtm1(where = (class = "&trFindings"));
         by table;
         if first.table;
         keep table;
      run;
      
      data findchk2;
         set sdtm2(where = (index(column,"TESTCD") or index(column,"TEST")));
         by table;
         if first.table;
         keep table;
      run;
      
      data findchk1;
         merge findchk1(in = ina) findchk2(in = inb);
         by table;
         if ina and not inb;
      run;
      %Nobsvars(findchk1);   
      %if &nobs %then %do;
         proc sql noprint;         
            select table
               into :findchk separated by ' '
               from findchk1
               order by table;
         quit;
         
         data sdtmmeta.source_tables;
            set sdtmmeta.source_tables;
            if
            %let i = 1;
            %do %while(%length(%scan(&findchk,&i,' ')));
               %let temp = %scan(&findchk,&i,' ');
               %if &i = 1 %then %do;  table = "&temp" %end;
               %else %do; or table = "&temp" %end;
               %let i = %eval(&i+1);
            %end;
            then delete;
         run;

         data sdtmmeta.source_columns;
            set sdtmmeta.source_columns;
            if
            %let i = 1;
            %do %while(%length(%scan(&findchk,&i,' ')));
               %let temp = %scan(&findchk,&i,' ');
               %if &i = 1 %then %do;  table = "&temp" %end;
               %else %do; or table = "&temp" %end;
               %let i = %eval(&i+1);
            %end;
            then delete;
         run;

      %end;
      proc datasets nolist;
         delete sdtm1 sdtm2 findchk1 findchk2;
      quit;                
   %end;

   %if %length(&AInPath) %then %do;
      data adam;
            set adam(drop = nobs);
         if not ((substr(memname,1,2) = "AD" and 4 <= length(trim(memname)) <= 8) or (substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6) 
               or memname in ("RELREC", "POOLDEF")) then delete;
            *** Labels ***;
         if memname = "ADDM" and memlabel = "" then memlabel = "&trDemographics";
         else if memname = "ADSL" and memlabel = "" then memlabel = "&trSubjectLevel";
         else if memname = "ADSE" and memlabel = "" then memlabel = "&trSubjectElements";
         else if memname = "ADCO" and memlabel = "" then memlabel = "&trComments";
         else if memname = "ADEX" and memlabel = "" then memlabel = "&trExposure";
         else if memname = "ADDS" and memlabel = "" then memlabel = "&trDisposition";
         else if memname = "ADLB" and memlabel = "" then memlabel = "&trLaboratoryTestResults";
         else if memname = "ADVS" and memlabel = "" then memlabel = "&trVitalSigns";
         else if memname = "ADEG" and memlabel = "" then memlabel = "&trECGTestResults";
         else if memname = "ADPC" and memlabel = "" then memlabel = "&trPKConcentrations";
         else if memname = "ADPP" and memlabel = "" then memlabel = "&trPKParameters";
         else if memname = "ADSC" and memlabel = "" then memlabel = "&trSubjectCharacteristics";
         else if memname = "TA" and memlabel = "" then memlabel = "&trTrialArms";
         else if memname = "TE" and memlabel = "" then memlabel = "&trTrialElements";
         else if memname = "TS" and memlabel = "" then memlabel = "&trTrialSummary";
         else if memname = "RELREC" and memlabel = "" then memlabel = "&trRelatedRecords";
         else if substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6 and memlabel = "" then memlabel = catx(" ", "&trSupplementalQualifiers", substr(memname,5,2));
         %if SEND = 1 %then %do;
            else if memname = "ADBW" and memlabel = "" then memlabel = "&trBodyWeights";
            else if memname = "ADBG" and memlabel = "" then memlabel = "&trBodyWeightGains";
            else if memname = "ADCL" and memlabel = "" then memlabel = "&trClinicalObservations";
            else if memname = "ADDD" and memlabel = "" then memlabel = "&trDeathDiagnosis";
            else if memname = "ADFW" and memlabel = "" then memlabel = "&trFoodandWaterConsumption";
            else if memname = "ADMA" and memlabel = "" then memlabel = "&trMacroscopicFindings";
            else if memname = "ADMI" and memlabel = "" then memlabel = "&trMicroscopicFindings";
            else if memname = "ADOM" and memlabel = "" then memlabel = "&trOrganMeasurements";
            else if memname = "ADPM" and memlabel = "" then memlabel = "&trPalpableMasses";
            else if memname = "ADTF" and memlabel = "" then memlabel = "&trTumorFindings";
            else if memname = "TX" and memlabel = "" then memlabel = "&trTrialSets";
            else if memname = "POOLDEF" and memlabel = "" then memlabel = "&trPooling";
         %end;
         %if SEND = 0 %then %do;
            else if memname = "ADSV" and memlabel = "" then memlabel = "&trSubjectVisits";
            else if memname = "ADCM" and memlabel = "" then memlabel = "&trConcomitantMedications";
            else if memname = "ADSU" and memlabel = "" then memlabel = "&trSubstanceUse";
            else if memname = "ADAE" and memlabel = "" then memlabel = "&trAdverseEvents";
            else if memname = "ADMH" and memlabel = "" then memlabel = "&trMedicalHistory";
            else if memname = "ADCE" and memlabel = "" then memlabel = "&trClinicalEvents";
            else if memname = "ADDV" and memlabel = "" then memlabel = "&trProtocolDeviations";          
            else if memname = "ADQS" and memlabel = "" then memlabel = "&trQuestionnaires";
            else if memname = "ADMB" and memlabel = "" then memlabel = "&trMicrobiologySpecimen";
            else if memname = "ADIE" and memlabel = "" then memlabel = "&trInclusionExclusionCritNotMet";
            else if memname = "ADPE" and memlabel = "" then memlabel = "&trPhysicalExamination";
            else if memname = "ADDA" and memlabel = "" then memlabel = "&trDrugAccountability"; 
            else if memname = "ADMS" and memlabel = "" then memlabel = "&trMicrobiologySuscTest";
            else if memname = "FA" and memlabel = "" then memlabel = "&trFindingsAbout";
            else if memname = "TV" and memlabel = "" then memlabel = "&trTrialVisits";
            else if memname = "TI" and memlabel = "" then memlabel = "&trTrialInclusionExclusionCrit";
            else if memname = "ADEC" and memlabel = "" then memlabel = "&trExposureasCollected";
            else if memname = "ADPR" and memlabel = "" then memlabel = "&trProcedures";
            else if memname = "ADHO" and memlabel = "" then memlabel = "&trHealthcareEncounters";
            else if memname = "ADIS" and memlabel = "" then memlabel = "&trImmunoSpecimenAssess";
            else if memname = "ADMO" and memlabel = "" then memlabel = "&trMorphology";
            else if memname = "ADRP" and memlabel = "" then memlabel = "&trReproductiveSysFind";
            else if memname = "ADTU" and memlabel = "" then memlabel = "&trTumorIdentification";
            else if memname = "ADDD" and memlabel = "" then memlabel = "&trDeathDetails";
            else if memname = "ADRS" and memlabel = "" then memlabel = "&trDiseaseResponse";
            else if memname = "ADSS" and memlabel = "" then memlabel = "&trSubjectStatus";
            else if memname = "ADTR" and memlabel = "" then memlabel = "&trTumorResults";
            else if memname = "ADSR" and memlabel = "" then memlabel = "&trSkinResponse";     
            else if memname = "TD" and memlabel = "" then memlabel = "&trTrialDiseaseAssess";                 
         %end;
         if memlabel = "" then memlabel = memname;
      run;

      *** Define user-defined classes ***;
      data adam_type_other(keep = memname class2);
         length Class2 $ 15;
         set adam(where = (substr(upcase(name),3) in ("TERM","TRT","TESTCD") or name = "PARAMCD"));
         if memname = "ADSL" then delete;
         if substr(upcase(name),3) = "TERM" then Class2 = "&trEvents";
         else if substr(upcase(name),3) = "TRT" then Class2 = "&trInterventions";
         else if substr(upcase(name),3) = "TESTCD" then Class2 = "&trFindings";
         else if name = "PARAMCD" then class2 = "BDS";
         if substr(upcase(name),3) in ("TERM","TRT","TESTCD") then do;
         if substr(upcase(name),1,2) ne substr(upcase(memname),3,2) then delete;
         end;
      run;

      *** Define Model Classes ***;
      data adam_type(keep = memname memlabel class);
         length Class $ 15;
         set adam;
         by memname;
         if first.memname;
         %if &SEND = 0 %then %do;
            if memname in ("ADSL","ADDM","ADSE","ADCO","ADSV") then class = "&trSpecial";
            else if memname in ("ADCM","ADSU","ADEX") then class = "&trInterventions";
            else if memname in ("ADAE","ADMH","ADCE","ADDS","ADDV") then class = "&trEvents";
            else if memname in ("ADEG","ADLB","ADQS","ADVS","ADMB","ADPC","ADIE","ADPE","ADSC","ADDA","ADMS","ADPP") then class = "&trFindings";
            else if memname = "RELREC" or (substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6) then class = "&trRelationship";
         %end;
         %else %if &SEND = 1 %then %do;
            if memname in ("ADSL","ADDM","ADSE","ADCO") then class = "&trSpecial";
            else if memname in ("ADEX") then class = "&trInterventions";
            else if memname in ("ADDS") then class = "&trEvents";
            else if memname in ("ADBW","ADBG","ADCL","ADDD","ADFW","ADLB","ADMA","ADMI","ADOM","ADPM","ADPC","ADPP","ADSC","ADTF","ADVS","ADEG") then class = "&trFindings";
            else if memname = "RELREC" or (substr(memname,1,4) = "SUPP" and length(trim(memname)) = 6) then class = "&trRelationship";
         %end;
      run;

      *** Fill in user-defined classes ***;
      *** Delete where class missing ***;
      data adam_type;
            merge adam_type adam_type_other;
            by memname;
            if class = "" and class2 ne "" then class = class2;
            if class = "" then delete;
      run;
   
      data adam_type;
            set adam_type;
            by memname;
            retain count;
            if first.memname then count = 1;
         else count = count + 1;
         if last.memname;
      run;

      *** Delete user-defined domains where class cannot be determined ***;
      data adam_type;
         set adam_type;
         if count > 1 then delete;
      run;
      
      data bds(keep = memname);
         set adam_type(where = (class2 = "BDS"));
      run;
      
      *** determining requirements for BDS ***;
      data bds;
         merge bds(in = ina) adam(keep = memname name where = (upcase(name) in ("PARAMCD","AVISITN","AVAL","AVALC","ADTM","ADT","AVISIT","ADY")));
         by memname;
         if ina;
         avail = 1;
      run;
      
      proc transpose data = bds out = bds;
         by memname;
         var avail;
         id name;
      run;
      
      data bds(keep = memname keepit);
         set bds;
         if paramcd and (aval or avalc) then keepit = 1;    
         else keepit = 0;
      run;
      
      *** Delete BDS domains ***;
      data adam_type(drop = keepit);
         merge adam_type(in = ina) bds(in = inb);
         by memname;
         BDSFlag = 0;
         if inb then do;
             if keepit = 0 then delete;
             else if keepit = 1 then do;
                Class = "&trFindings";
                BDSFlag = 1;
             end;
         end;
      run;
      
      *** Keep variables for domains with defined classes ***;
      data adam;
         merge adam_type(keep = memname bdsflag in = ina) adam;
         by memname;
         if ina;
      run;

      data adammeta.source_tables(keep = memname memlabel class rename = (memname = Table memlabel = Label));
         retain memname memlabel class;
         set adam_type(where = (upcase(memname) not in ("POOLDEF","RELREC")));
         label memname = "&trTableName" memlabel = "&trLabel" class = "&trObservationClasswithinStand";
      run;
 
      data adammeta.source_columns(keep = memname name label rename = (memname = Table name = Column));
         retain memname name label;
         set adam(where = (upcase(memname) not in ("POOLDEF","RELREC")));
         name = upcase(name);
         if BDSFlag = 1 then do;
            if substr(name,3) in ("TESTCD", "TEST") then delete;
         end;         
         label memname = "&trTableName" label = "&trLabel" name = "&trColumnName";
      run;

      proc datasets nolist;
         delete adam adam_type adam_type_other;
      quit;
      
      *** Findings Check. Drop those without xxTESTCD or PARAMCD ***;   
      proc sort data = adammeta.source_tables out = adam1;
         by table;
      run;
      
      proc sort data = adammeta.source_columns out = adam2;
         by table column;
      run;     
      
      data findchk1;
         set adam1(where = (class = "&trFindings"));
         by table;
         if first.table;
         keep table;
      run;
      
      data findchk2;
         set adam2(where = (index(column,"TESTCD") or column = "PARAMCD"));
         by table;
         if first.table;
         keep table;
      run;
      
      data findchk1;
         merge findchk1(in = ina) findchk2(in = inb);
         by table;
         if ina and not inb;
      run;
      %Nobsvars(findchk1);   
      %if &nobs %then %do;
         proc sql noprint;         
            select table
               into :findchk separated by ' '
               from findchk1
               order by table;
         quit;
         
         data adammeta.source_columns;
            set adammeta.source_columns;
            if
            %let i = 1;
            %do %while(%length(%scan(&findchk,&i,' ')));
               %let temp = %scan(&findchk,&i,' ');
               %if &i = 1 %then %do;  table = "&temp" %end;
               %else %do; or table = "&temp" %end;
               %let i = %eval(&i+1);
            %end;
            then delete;
         run;
      %end;      
      proc datasets nolist;
         delete adam1 adam2 findchk1 findchk2;
      quit;
   %end;
   %tkexit:;
%mend TOOLKITME;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro MAKEDIR(dir = , varname =, parent = ) ; 
   %local rc fileref;
   %let rc = %sysfunc(filename(fileref,&parent.&dir)); 
   %if %sysfunc(fexist(&fileref)) %then %do;
      %put NOTE: The directory "&parent.&dir" exists;
   %end;
   %else %do; 
      %let rc = %sysfunc(dcreate(&dir, &parent));
      %if %length(&rc) %then %put NOTE: The directory &varname (&parent.&dir) has been created; 
      %else %do;
         %put [MS_S_0202];
         %put ERROR: Unable to create directory &parent.&dir;
         %let CSTERROR = 1;
      %end;
   %end; 
   %if &Server ne Local %then %do;
      %global &&varname;
      %let &&varname = &parent.&dir;
      libname &&varname "&parent.&dir";
   %end;
   %else %if &Server = Local %then %do;
      %global &varname;
      %let &varname = &parent.&dir;
      libname &varname "&parent.&dir";
   %end;   
   %let rc=%sysfunc(filename(fileref)); 
%mend MAKEDIR;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;
/* fdomain:          Input data: each finding domain;
   fdBaseline:    Outputput data: baseline ;
*/
%macro ComputeBaseline(fdomain,fdbaseline);

   proc datasets nolist;
      delete &fdbaseline;
   quit;
   %let fdomainsn = ;
   %if %sysfunc(findw(&&&fdomain.VARS, &fdomain.STRESN)) or %sysfunc(findw(&&&fdomain.VARS, AVAL)) %then %do;
      %let fdomainsn = &fdomainsn &fdomain;

      %let modlistB = POS LOC FAST SPEC METHOD TPT TPTNUM;    
      %let modvlistB = ;
      %do jj=1 %to %sysfunc(countw(&modlistB));
         %let modifier = %scan(&modlistB,&jj,' ');
         %if %sysfunc(findw(&&&fdomain.VARS, &fdomain.&modifier)) %then %do;
            %let modvlistB = &modvlistB &fdomain.&modifier;
         %end;
      %end;
      %put modvlistB=&modvlistB;
      * flag to precompute change from baseline, percent of baseline and percent change from baseline;
      %if %sysfunc(indexw(&&&fdomain.VARS, &fdomain.BLFL, ' ')) %then %let &fdomain.BlFlag = &fdomain.BLFL;
      %else %let &fdomain.BlFlag = ;

      * create ordinal variable corresponding to sorted TPTNUM variable;
      %if %sysfunc(findw(&&&fdomain.VARS, &fdomain.TPTNUM)) %then %do;
         proc sort data=&&in_&fdomain(keep=&fdomain.TPTNUM) out=tpt nodupkey;
            by &fdomain.TPTNUM;
         run;
         
         data tpt;
            set tpt;
            &fdomain.TPTN_ = _n_;
         run;
         
         proc sql noprint;
            create table i&fdomain as
            select * from &&in_&fdomain as a left join tpt as b
            on a.&fdomain.TPTNUM = b.&fdomain.TPTNUM;
         *  order by USUBJID &fdomain.testcd &&&fdomain.BlFlag &fdomain.dy;;
         quit;
         %let fdata = i&fdomain;
      %end;
      %else %do;
         %let fdata = &&in_&fdomain;
         %if %length(&fdata) > 5 and %upcase(%substr(&fdata,1,5)) = WORK. %then %do;
            %let fdata = %substr(&fdata,6);
         %end;
      %end;
      
      %* run this if there is BDS OR there is no DY;
      %if %sysfunc(findw(&bdsdomains, &fdomain)) or ^%sysfunc(findw(&&&fdomain.VARS, &fdomain.DY)) %then %do;
         data i&fdomain /* (keep = USUBJID &modvlistB &fdomain.testcd &&&fdomain.BlFlag &fdomain.dy &fdomain.STRESN) */;
            set &fdata;
            %if %sysfunc(findw(&bdsdomains, &fdomain)) %then %do;
               rename PARAMCD=&fdomain.testcd aval=&fdomain.STRESN;;
            %end;
            %if ^%sysfunc(findw(&&&fdomain.VARS, &fdomain.DY)) %then %do;
               %calculateStudyDays(&fdomain);
            %end;
         run;
         %let fdata = i&fdomain;
      %end;
      %put fdomain = &fdomain     fdata = &fdata;
      
      proc sort data=&fdata /* (keep = USUBJID &modvlistB &fdomain.testcd &&&fdomain.BlFlag &fdomain.dy &fdomain.STRESN) */ out=b&fdomain;
         by USUBJID &fdomain.testcd &&&fdomain.BlFlag &fdomain.dy;
      run;
   
      data b&fdomain;
         set b&fdomain;
         %if %length(&&&fdomain.BlFlag) %then %do;
            if index(&&&fdomain.BlFlag,"Y") > 0;
         %end;
         %else %do;
            if . < &fdomain.dy <= 1;
         %end;
      run;

      data b&fdomain(keep = USUBJID &modvlistB &fdomain.testcd _BASE);
         set b&fdomain;
         by USUBJID &fdomain.testcd;
         _BASE = &fdomain.stresn;
         if last.&fdomain.testcd;
      run;    

      proc sql noprint;
         create table &fdbaseline(drop = USUBJID1 USUBJID2 &fdomain.testcd1 &fdomain.testcd2 rfstdtc) as
         select coalesce(aa.USUBJID1, bb.USUBJID2) as USUBJID, coalesce(aa.&fdomain.testcd1, bb.&fdomain.testcd2) as &fdomain.TESTCD, aa.*, bb.*, 
         case 
            when &fdomain.dy > 1 then &fdomain.stresn - _BASE 
            else .
         end as BLCHG,
         case 
            when &fdomain.dy > 1 then round((&fdomain.stresn - _BASE) / _BASE * 100, 0.1) 
            else .
         end as PBLCHG,
         case 
            when &fdomain.dy > 1 then round(&fdomain.stresn / _BASE * 100, 0.1) 
            else .
         end as PofBL
         from &fdata(rename=(USUBJID = USUBJID1 &fdomain.testcd = &fdomain.testcd1)) as aa 
         left join b&fdomain(rename=(USUBJID = USUBJID2 &fdomain.testcd = &fdomain.testcd2)) as bb
         on aa.USUBJID1 = bb.USUBJID2 and aa.&fdomain.testcd1 = bb.&fdomain.testcd2
         order by USUBJID, &fdomain.testcd, &fdomain.dy;
      quit; 
      proc datasets nolist;
         delete b&fdomain;
      quit;
   %end;
   %cmbexit:;
%mend ComputeBaseline;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;
/* fdomain:       Input data: each finding domain;
   fdBaseline:    Input data: baseline from output of %ComputeBaseline;
   findm:         Output data: in order to write to metadata base directory;
*/
%macro ComputeQuantiles(fdomain,fdBaseline,findQuant);
   * search for test code modifier variables and build a list of them;
   /* %let modlist = POS LOC FAST SPEC METHOD TPT TPTNUM; */
   %let modlist = POS LOC FAST SPEC METHOD;
   %let modvlist = ;
   %let modvlistc = ;
   %let modvlist1 = ;
   %let modvlist2 = ;
   %do jj=1 %to %sysfunc(countw(&modlist));
      %let modifier = %scan(&modlist,&jj,' ');
      %if %sysfunc(findw(&&&fdomain.VARS, &fdomain.&modifier)) %then %do;
         %if &modifier = TPT %then %do;
            %if %sysfunc(findw(&&&fdomain.VARS, &fdomain.TPTNUM)) %then %do;
               %let modvlist = &modvlist &fdomain.TPTNUM;
               %if ^%length(&modvlistc) %then %do;
                  %let modvlistc = &fdomain.TPTNUM;
               %end;
               %else %do;
                  %let modvlistc = &modvlistc, &fdomain.TPTNUM;
               %end;
            %end;
            %else %do;
               %let modvlist = &modvlist &fdomain.&modifier;
               %if ^%length(&modvlistc) %then %do;
                  %let modvlistc = &fdomain.&modifier;
               %end;
               %else %do;
                  %let modvlistc = &modvlistc, &fdomain.&modifier;
               %end;
            %end;
            %if ^%length(&modvlist2) %then %do;
               %let modvlist2 = &fdomain.&modifier;
            %end;
            %else %do;
               %let modvlist2 = &modvlist2 &fdomain.&modifier;
            %end;
         %end;
         %else %do;
            %let modvlist = &modvlist &fdomain.&modifier;
            %if ^%length(&modvlistc) %then %do;
               %let modvlistc = &fdomain.&modifier;
            %end;
            %else %do;
               %let modvlistc = &modvlistc, &fdomain.&modifier;
            %end;
            %if ^%length(&modvlist1) %then %do;
               %let modvlist1 = &fdomain.&modifier;
            %end;
            %else %do;
               %let modvlist1 = &modvlist1 &fdomain.&modifier;
            %end;
            %if ^%length(&modvlist2) %then %do;
               %let modvlist2 = &fdomain.&modifier;
            %end;
            %else %do;
               %let modvlist2 = &modvlist2 &fdomain.&modifier;
            %end;
         %end;
      %end;
   %end;
   %put modvlist = &modvlist;
   %put modvlistc = &modvlistc;
   %put modvlist1 = &modvlist1;
   %put modvlist2 = &modvlist2;
   data &fdomain.tmp;
      set &fdBaseline end=eof_ds2;
      if missing(&fdomain.STRESN) or index(upcase(&fdomain.CAT),"QUALITATIVE") then delete;
         rename &fdomain.RFLG = RevFlag;
   run;
   %NObsVars(&fdomain.tmp);
   %if &nobs =0 %then %do;
      %goto cmqexit;
   %end;

   proc sort data=&fdomain.tmp;
      by &fdomain.testcd &modvlist;
   run;
      
   * compute quantiles for all subjects;
   data allsubs;
      set &fdBaseline(keep=&temp.testcd &modvlist &fdomain.STRESN BLCHG PBLCHG PofBL);
      if missing(&fdomain.STRESN) or index(upcase(&fdomain.CAT),"QUALITATIVE") then delete;
   run;
      
   proc sort data=allsubs;
      by &fdomain.testcd &modvlist;
   run;

   proc means data=allsubs noprint;
      by &fdomain.testcd &modvlist;
      var &fdomain.STRESN BLCHG PBLCHG PofBL;
      output out=&findQuant(keep=&fdomain.testcd &modvlist YP:) 
         p10=YP10_1 YP10_2 YP10_3 YP10_4 
         p25=YP25_1 YP25_2 YP25_3 YP25_4 
         p50=YP50_1 YP50_2 YP50_3 YP50_4 
         p75=YP75_1 YP75_2 YP75_3 YP75_4
         p90=YP90_1 YP90_2 YP90_3 YP90_4;
   run;
   proc datasets nolist;
      delete &fdomain.tmp allsubs;
   quit;
   %cmqexit:;
%mend ComputeQuantiles;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CREATEMETADATA;
   %let nonstandard = ;
   %global domaindups;
   %let domaindups = ;

   %TOOLKITME; 
   %if &CSTERROR %then %goto cmdexit;
   
   %if %length(&InPath) %then %do;
      proc sql noprint;
         *** Check for duplicated domains;
         create table dups as
            select substr(table,1,2) as dupdomain, freq(calculated dupdomain) as ndomain
            from sdtmmeta.source_tables(where = (index(table, "GRP") = 0 and index(table, "SUPP") = 0 and class ne ""))
            group by calculated dupdomain
            having ndomain > 1;
         %let numdups = &sqlobs;
         %if &numdups %then %do;
            select dupdomain
               into :domaindups separated by ' '
               from dups
               order by dupdomain;
         %end;
         select table
            into :nonstandard separated by ' '
            from sdtmmeta.source_tables(where = (class = ""))
            order by table;
      quit;

      proc datasets nolist;
         delete dups;
      quit;

      %if %length(&domaindups) %then %do;
         %goto cmdexit;
      %end;

      *** Delete any GRP or POOLDEF metadata;
      data sdtmmeta.source_tables;
         retain table label class;
         length Flag $4.;        
         set sdtmmeta.source_tables;
         if index(upcase(table), "GRP") > 0 or upcase(table) = "POOLDEF" or upcase(table) = "RELREC" then delete;
         %if %length(&nonstandard) %then %do;
            %PURGE(list = &nonstandard, flag = );
         %end;
         flag ="SDTM";
         label flag="SDTM/ADAM Flag";
      run;

      data sdtmmeta.source_columns;
         set sdtmmeta.source_columns;
         if index(upcase(table), "GRP") > 0 or upcase(table) = "POOLDEF" or upcase(table) = "RELREC" then delete;
         %if %length(&nonstandard) %then %do;
            %PURGE(list = &nonstandard, flag = );
         %end;
      run;
   %end;
   
   %if %length(&AInPath) %then %do;
      *** Delete any GRP metadata;
      data adammeta.source_tables;
         retain table label class;
         length Flag $4.;
         set adammeta.source_tables;
         if index(upcase(table), "GRP") > 0 or not (substr(table,1,2) = "AD" and 4 <= length(table) <= 8) then delete;
         %if %length(&nonstandard) %then %do;
            %PURGE(list = &nonstandard, flag = AD);
         %end;
         flag ="ADAM";
         label flag="SDTM/ADAM Flag";
      run;
      
      data adammeta.source_columns;
         set adammeta.source_columns;
         if index(upcase(table), "GRP") > 0 or not (substr(table,1,2) = "AD" and 4 <= length(table) <= 8) then delete;
         %if %length(&nonstandard) %then %do;
            %PURGE(list = &nonstandard, flag = AD);
         %end;
      run;
   %end;
   %cmdexit:;
%mend CREATEMETADATA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro IDENTIFYDOMAINS;
    options nonotes;
    %global domains domainclass adslpresent findingsdomains eventsdomains interventionsdomains suppdomains cdiscmodel bdsdomains;

   %let findingsdomains = ;
   %let eventsdomains = ;
   %let interventionsdomains = ;
   %let adslpresent =; 
   %let suppdomains = ;
   %let cdiscmodel = ;
   %let bdsdomains = ;
   
   %let trSpecial = %nrbquote(%LocaleGet(Special_Text));
   %let trEvents = %nrbquote(%LocaleGet(Events_Text));
   %let trInterventions = %nrbquote(%LocaleGet(Interventions_Text));
   %let trFindings = %nrbquote(%LocaleGet(Findings_Text));
   %let trBodyWeights = %nrbquote(%LocaleGet(BodyWeights_Text));
   %let trBodyWeightGains = %nrbquote(%LocaleGet(BodyWeightGains_Text));
   %let trClinicalObservations = %nrbquote(%LocaleGet(ClinicalObservations_Text));
   %let trDeathDiagnosis = %nrbquote(%LocaleGet(DeathDiagnosis_Text));
   %let trFoodandWaterConsumption = %nrbquote(%LocaleGet(FoodandWaterConsumption_Text));
   %let trMacroscopicFindings = %nrbquote(%LocaleGet(MacroscopicFindings_Text));
   %let trMicroscopicFindings = %nrbquote(%LocaleGet(MicroscopicFindings_Text));
   %let trOrganMeasurements = %nrbquote(%LocaleGet(OrganMeasurements_Text));
   %let trPalpableMasses = %nrbquote(%LocaleGet(PalpableMasses_Text));
   %let trTumorFindings = %nrbquote(%LocaleGet(TumorFindings_Text));
        
    proc sql noprint;
      %if %length(&InPath) %then %do;
            *** Identify all non-supp SDTM domains;
            %let sdtmdomains = ;
            select table
               into :sdtmdomains separated by ' '
               from sdtmmeta.source_tables
               where index(upcase(table), "SUPP") = 0
               order by table;
            %let numdomains = &sqlobs;
            %let sdtmdomainsclass = ;
            select scan(class, 1, " ")
               into :sdtmdomainsclass separated by ' '
               from sdtmmeta.source_tables
               where index(upcase(table), "SUPP") = 0
               order by table;
            select table
               into :suppdomains separated by ' '
               from sdtmmeta.source_tables
               where index(upcase(table), "SUPP") > 0;                   
            select column
               into :cdiscmodel separated by ' '
               from sdtmmeta.source_columns
               where upcase(table) = "DM" and upcase(column) = "SETCD";
            %if %length(&cdiscmodel) %then %let cdiscmodel = SEND;
            %else %let cdiscmodel = SDTM;
      %end;
       %if %length(&AInPath) %then %do;
            %if cdiscmodel =  %then %let cdiscmodel = SDTM;
         *** Identify ADSL;
                   select table
            into :adslpresent separated by ' '
            from adammeta.source_tables
                 where upcase(table) = "ADSL"
                      order by table;
              *** Identify all non-ADSL ADAM domains;
                   %let alladamdomains =;
                   select table
            into :alladamdomains separated by ' '
            from adammeta.source_tables
                 where index(upcase(table), "ADSL") = 0;
         %let numadam = &sqlobs;
         *** Identify all non-ADSL ADAM BDS domains;
              select table
            into :bdsdomains separated by ' '
            from adammeta.source_columns
                 where upcase(column) = "PARAMCD" and index(upcase(table), "ADSL") = 0
                      order by table;
         *** Derive domain types for ADAM domains;
         create table vartype as
            select table, substr(upcase(column),3) as type,
              case calculated type
                when "TERM" then "&trEvents"
               when "TRT" then "&trInterventions"
               when "TESTCD" then "&trFindings"
               when "RAMCD" then "&trFindings"
               else "UNKNOWN"
             end as class
            from adammeta.source_columns
            where calculated type in ("TERM","TRT","TESTCD","RAMCD")
              order by table, calculated type;
      %end;
   quit;

    *** Add missing SEND labels;
    %if &cdiscmodel = SEND %then %do;
    data sdtmmeta.source_tables;
       set sdtmmeta.source_tables;
       if label = "" then do;
          if table = "BW" then label = "&trBodyWeights";
          else if table = "BG" then label = "&trBodyWeightGains";
          else if table = "CL" then label = "&trClinicalObservations";
          else if table = "DD" then label = "&trDeathDiagnosis";         
          else if table = "FW" then label = "&trFoodandWaterConsumption";         
          else if table = "MA" then label = "&trMacroscopicFindings";          
          else if table = "MI" then label = "&trMicroscopicFindings";
          else if table = "OM" then label = "&trOrganMeasurements";                  
          else if table = "PM" then label = "&trPalpableMasses";                  
          else if table = "TF" then label = "&trTumorFindings";                                     
       end;
    run;
    %end;

    %if %length(&AInPath) %then %do;
    *** Define Known Types;
        data adamtype(where = (class = "&trSpecial"));
           set clinical.adamdomainclass;
        run;

      *** Select only those derived domain classes of a single type;
       data vartype;
          set vartype;
          by table;
          if first.table = 1 and last.table = 1;
       run;

      proc sql noprint;
         create table vartype2 as
            select coalesce(adamtype.table, vartype.table) as table, coalesce(adamtype.class, vartype.class) as class
            from adamtype full join vartype
            on adamtype.table = vartype.table
            where adamtype.table is null;
         create table vartype3 as
               select * from adamtype union
            select * from vartype2;
          %let adamdomainclasstable = ;
          select table
            into :adamdomainclasstable separated by ' '
            from vartype3
            order by table;
          %let adamdomainclass = ;
          select class
            into :adamdomainclass separated by ' '
            from vartype3
            order by table;
          quit;
   %end;

   %if %length(&InPath) %then %do;
       %let domains = &sdtmdomains;
      %let domainclass = &sdtmdomainsclass;
   %end;
   %else %do;
       %let domains = ;
       %let domainclass = ;
   %end;
   
   * check to see if the parent of SUPP domains exists, if not, delete it from suppdomains list;
   %let i = 1;
   %let suppdomainsTemp =;
   %let suppDelDomains =;
   %do %while(%length(%scan(&suppdomains,&i,' ')));
      %let tempSupp = %scan(%upcase(&suppdomains),&i," ");
      %let tempSupp2 = %substr(&tempSupp,5);
      %put domains=&domains tempSupp2=&tempSupp2;
      %if %sysfunc(findw(&domains, &tempSupp2)) %then %do;
         %let suppdomainsTemp = &suppdomainsTemp &tempSupp;
      %end;
      %else %do;
         %let suppDelDomains=&suppDelDomains &tempSupp;
      %end;
      %let i = %eval(&i+1);
   %end;
   %let suppdomains=&suppdomainsTemp;
   %if %length(&InPath) %then %do;
      data sdtmmeta.source_tables;
         set sdtmmeta.source_tables;
         %let _i_ =1;
         %do %while(%length(%scan(&suppDelDomains,&_i_,' ')));
            %let temp = %scan(%upcase(&suppDelDomains),&_i_," ");
            if table ="&temp" then delete;
            %let _i_ = %eval(&_i_+1);
         %end;
      run;
      data sdtmmeta.source_columns;
         set sdtmmeta.source_columns;
         %let _i_ =1;
         %do %while(%length(%scan(&suppDelDomains,&_i_,' ')));
            %let temp = %scan(%upcase(&suppDelDomains),&_i_," ");
            if table ="&temp" then delete;
            %let _i_ = %eval(&_i_+1);
         %end;
      run;
   %end;
   %if %length(&AInPath) %then %do;
      data adammeta.source_tables;
         set adammeta.source_tables;
         %let _i_ =1;
         %do %while(%length(%scan(&suppDelDomains,&_i_,' ')));
            %let temp = %scan(%upcase(&suppDelDomains),&_i_," ");
            if table ="&temp" then delete;
            %let _i_ = %eval(&_i_+1);
         %end;
      run;
      data adammeta.source_columns;
         set adammeta.source_columns;
         %let _i_ =1;
         %do %while(%length(%scan(&suppDelDomains,&_i_,' ')));
            %let temp = %scan(%upcase(&suppDelDomains),&_i_," ");
            if table ="&temp" then delete;
            %let _i_ = %eval(&_i_+1);
         %end;
      run;
   %end;
   
    *** this is for old code to remove BDS when available which is now removed in TOOLKITME ***;
    %let adambdsdomains = ;
    %if %length(&AInPath) %then %do;          
      %if %length(&alladamdomains) %then %do;
         %let templist = ;
         %let i = 1;
         %do %while(%length(%scan(&alladamdomains,&i,' ')));         
            %let temp = %substr(%scan(&alladamdomains,&i,' '),3);           
            %let templist = &templist &temp;
            %let i = %eval(&i+1);
         %end;
         %let alladamdomains = &templist;
      %end;      

      %if %length(&adambdsdomains) %then %do;
         %let templist = ;
         %let i = 1;
         %do %while(%length(%scan(&adambdsdomains,&i,' ')));         
            %let temp = %substr(%scan(&adambdsdomains,&i,' '),3);           
            %let templist = &templist &temp;
            %let i = %eval(&i+1);
         %end;
         %let adambdsdomains = &templist;
      %end;
      
      %if %length(&adamdomainclasstable) %then %do;
         %let templist = ;
         %let i = 1;
         %do %while(%length(%scan(&adamdomainclasstable,&i,' ')));         
            %let temp = %substr(%scan(&adamdomainclasstable,&i,' '),3);           
            %let templist = &templist &temp;
            %let i = %eval(&i+1);
         %end;
         %let adamdomainclasstable = &templist;
      %end;
      
      %if %length(&bdsdomains) %then %do;
         %let templist = ;
         %let i = 1;
         %do %while(%length(%scan(&bdsdomains,&i,' ')));         
            %let temp = %substr(%scan(&bdsdomains,&i,' '),3);           
            %let templist = &templist &temp;
            %let i = %eval(&i+1);
         %end;
         %let bdsdomains = &templist;
      %end;      
    
        %do i = 1 %to &numadam;
         %let class = ;
          %let temp = %scan(&alladamdomains,&i,' ');
          %if %sysfunc(findw(&adamdomainclasstable, &temp)) %then %let class = %scan(&adamdomainclass,%sysfunc(findw(&adamdomainclasstable, &temp, ' ', E)),' '); 
          %if ^%sysfunc(findw(&adambdsdomains, &temp)) and ^%sysfunc(findw(&domains, &temp)) and %length(&class) > 0 %then %do;
            %let domains = &domains &temp;
            %let domainclass = &domainclass &class;
         %end;
        %end; 
   %end;

   %let i = 1;
   %do %while(%length(%scan(&domains,&i,' ')));
      %let temp = %scan(&domains,&i,' ');
      %let class = %scan(&domainclass,&i,' ');
      %if %upcase(&class) = %upcase(&trEvents) %then %let eventsdomains = &eventsdomains &temp;
        %else %if %upcase(&class) = %upcase(&trFindings) %then %let findingsdomains = &findingsdomains &temp; 
        %else %if %upcase(&class) = %upcase(&trInterventions) %then %let interventionsdomains = &interventionsdomains &temp;
        %let i = %eval(&i+1);
   %end; 

   options notes;

   %put NOTE: Available Domains for Analysis: &domains;
   %put NOTE: Domains with Supplemental Qualifiers: &suppdomains;
   %put NOTE: Events Domains: &eventsdomains;
   %put NOTE: Findings Domains: &findingsdomains;
   %put NOTE: Interventions Domains: &interventionsdomains;
   %put NOTE: ADaM BDS Domains: &bdsdomains;
   %if %length(&adslpresent) %then %put NOTE: ADSL Available;
   %else %put NOTE: ADSL NOT Available;

   proc datasets nolist;
      delete adamtype vartype vartype2 vartype3;
   quit;
%mend IDENTIFYDOMAINS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro MISSVARS(model = SDTM, dir = , progressStart = 0.10, progressStop = 0.35);
   %put Entering MISSVARS;
   %ClockStart(MISSVARS);     
   options nonotes;
   ods exclude all;
   ods noresults;

   proc datasets library = &model memtype = data nolist;
     ods output variables = varlist(keep = member);
     contents data = _all_;
   quit;

   %let SubList =;
   %let i = 1;
   %do %while(%length(%scan(&domains,&i,' ')));
      %let temp = %scan(&domains,&i,' ');
      %if %sysfunc(findw(&alladamdomains, AD&temp)) and &model = ADAM %then %do;
         %let SubList = &SubList AD&temp;
      %end;
      %else %if ^%sysfunc(findw(&alladamdomains, AD&temp)) and &model = SDTM %then %do;
         %let SubList = &SubList &temp;
         %if %sysfunc(findw(&suppdomains, %upcase(SUPP&temp))) %then %do;
            %let SubList = &SubList SUPP&temp;
         %end;
      %end;
      %let i = %eval(&i+1);
   %end;
   %if %length(&adslpresent) and &model = ADAM %then %do; 
      %let SubList = &SubList ADSL;
   %end;

   %let quotedSubList =;   
   %let i = 1;
   %do %while(%length(%scan(&SubList,&i,' ')));
      %let temp = %upcase(%scan(&SubList,&i,' '));
      %let quotedSubList = &quotedSubList "&temp";
      %let i = %eval(&i+1);
   %end;
   
   %if %length(&quotedSubList) %then %do;
      data varlist;
         set varlist;
         if substr(member, index(member,'.')+1) in (&quotedSubList);
      run;
   %end;
   
   %let datasets = ;
   %GetLevels(varlist,member,datasets,_nLvls=sqlobs,_vType=vtype,_dropmiss = 1);
   
   %let first = 0;
   %let i = 1;
   %do %while(%length(%scan(&datasets,&i,' ')));
      %let temp = %scan(%upcase(&datasets),&i," ");

      %let progressCurrent = %sysevalf(&progressStart + %sysevalf(%sysevalf(%eval(&i - 1) / %sysfunc(countw(&datasets, ' '))) * %sysevalf(&progressStop - &progressStart)));
      %put [AddStudyLocal_~MISSVARS Datasets &temp~_PROGRESS_INDICATOR_ESTIMATED_FRACTION=&progressCurrent];

      %nObsVars(&temp);
      %if &nobs > 0 %then %do;
         *** checking first row to remove variables with data ***;
         %let _tid_ = %sysfunc(open(&temp));
         %let _vn_ = %sysfunc(varnum(&_tid_,_name_));
         %let _tid_ = %sysfunc(close(&_tid_));  

         data temp;
            set &temp(obs = 1);
            %if &_vn_ %then %do;
              drop _name_; 
            %end;
         run;                
         
         proc transpose data = temp out = tempn(where = (missing(col1)));
            var _numeric_;
         run;
         proc transpose data = temp out = tempc(where = (missing(col1)));
            var _character_;
         run;
         
         %let missvarsn =;
         %GetLevels(tempn,_NAME_,missvarsn,_nLvls=sqlobs,_vType=vtype,_dropmiss = 1);
         %let missvarsc =;
         %GetLevels(tempc,_NAME_,missvarsc,_nLvls=sqlobs,_vType=vtype,_dropmiss = 1);
                  
         %if %length(&missvarsn) or %length(&missvarsc) %then %let missvars = %sysfunc(strip(&missvarsn &missvarsc));
         %else %let missvars =;               
      
         %if %length(&missvars) %then %do;
            proc sort data = &temp(keep = &missvars) nodupkey out = temp;
               by &missvars;
            run;

            proc freq data = temp nlevels;
               ods output nlevels=temp1;
            run;

            proc datasets nolist;
               delete temp;
            quit;

            %let _tid_ = %sysfunc(open(temp1));
            %let _vn_ = %sysfunc(varnum(&_tid_,nnonmisslevels));
            %let _tid_ = %sysfunc(close(&_tid_));  

            %if &_vn_ %then %do;
               data temp1(keep = column1 table1);
                  length column1 $ 8 table1 $ 32;
                  set temp1(where=(nnonmisslevels eq 0));
                  column1 = upcase(tablevar);
                  table1 = upcase("%scan(&temp,2,.)");
               run;

               %if ^&first %then %do;
                  data alltemp;
                    set temp1;
                  run;
                  %let first = 1;
               %end;
               %else %do;
                  proc append force base = alltemp data = temp1;
                  run;
               %end;

               proc datasets nolist;
                  delete temp temp1;
               quit;
            %end;
         %end;
      %end;
      %let i = %eval(&i+1);
   %end;
   
   %if ^&first %then %do;
      data alltemp;
         length column1 $ 8 table1 $ 32;
         stop;
      run;
   %end;
   
   proc sort data = alltemp;
      by table1 column1;
   run;
   
   data &dir..missvars&model(keep = column table);
      set alltemp;
      column = trim(column1);
      table = trim(table1);
   run;

   proc datasets nolist;
     delete alltemp;
   quit;
        
   %mvexit:; 
   options notes;
   ods exclude none;
   ods results;
   %put Exiting MISSVARS;
   %ClockStop(MISSVARS);        
%mend MISSVARS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro METADATAOUTFILES;
    %put Entering METADATAOUTFILES;
    %if %length(&InPath) %then %do;
       *** Removes null variables from SDTM;
      proc sql noprint;
         create table sdtm_source_columns as
         select *
         from sdtmmeta.source_columns(keep = table column where = (index(upcase(table), "SUPP") = 0)) 
              as A natural left join sdtmmeta.missvarssdtm(where = (index(upcase(table), "SUPP") = 0)) as B
         where B.column is null;
      quit;
    %end;

    %if %length(&AInPath) %then %do;
      *** Removes null variables from ADAM;
      proc sql noprint;
         create table adam_source_columns as
         select *
         from adammeta.source_columns(keep = table column) as A natural left join adammeta.missvarsadam as B
         where B.column is null;
      quit;
    %end;

    %let sdtmdomains = ;
    %let sdtmsuppdomains = ;     
    %let alladamdomains = ;
    %let adambdsdomains = ;
    
   * options nonotes;
   proc sql noprint;
      %if %length(&InPath) %then %do;
          *** Identify all non-supp SDTM domains;
          select table
            into :sdtmdomains separated by ' '
            from sdtmmeta.source_tables
            where index(upcase(table), "SUPP") = 0;
            %let numdomains = &sqlobs;
          *** Identify all supp SDTM domains;
          select table
            into :sdtmsuppdomains separated by ' '
            from sdtmmeta.source_tables
            where index(upcase(table), "SUPP") > 0;
            %let numsupp = &sqlobs;
          *** Identify all variables within each SDTM domain;
          %do i = 1 %to &numdomains;
             %let temp = %scan(&sdtmdomains,&i,' ');
                       %let &temp.var = ; 
            select column
               into :&temp.var separated by ' '
               from sdtm_source_columns
               where upcase(table) = "&temp";
         %end;
         *** Identify variables from Supp Domains;
         %do i = 1 %to &numsupp;
             %let temp = %scan(&sdtmsuppdomains,&i,' ');
                  %let &temp.var = ; 
                  select distinct qnam
                into :&temp.var separated by ' '
           from sdtm.&temp;
         %end;
         %if %sysfunc(findw(&domains, DM)) %then %do;
            *** Create DM variable data set;
            create table DMVARLIST as
               select column
               from sdtmmeta.source_columns
               where upcase(table) = "DM"
                order by column;
         %end;
      %end;
      %if %length(&AInPath) %then %do;
         *** Identify all ADAM domains;
         select table
            into :alladamdomains separated by ' '
            from adammeta.source_tables;
         %let numadam = &sqlobs;
         *** Identify all ADAM BDS domains;
      select table
            into :adambdsdomains separated by ' '
            from adammeta.source_columns
            where upcase(column) = "PARAMCD";
          *** Identify all variables within each ADAM domain;
          %do i = 1 %to &numadam;
             %let temp = %scan(&alladamdomains,&i,' ');
             %let &temp.var = ; 
                  select column
               into :&temp.var separated by ' '
               from adam_source_columns
               where upcase(table) = "&temp";
         %end;
         %if %length(&adslpresent) %then %do;
            *** Create ADSL variable data set;
            create table ADSLVARLIST as
               select column
               from adammeta.source_columns
               where upcase(table) = "ADSL"
                order by column;
         %end;
      %end;
    quit;
    
   %if %length(&InPath) and %sysfunc(findw(&domains, DM)) %then %do;
      *** Create TXT files of DM Variables;
      data _null_;
         set DMVARLIST;
         file "&basemeta&_slash_.DMVARLIST.txt" encoding="utf-8";
         put column;
      run;
   %end;

   %if %length(&adslpresent) %then %do;
      *** Create TXT files of ADSL Variables;
      data _null_;
         set ADSLVARLIST;
         file "&basemeta&_slash_.ADSLVARLIST.txt" encoding="utf-8";
         put column;
      run; 
   %end;

   *** Initialize macrovariable file;
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas";
      put "%nrstr(%macro) libassign;";
      put "   %nrstr(%global) SDTMMETA ADAMMETA BASEMETA;";
      put "   %nrstr(%let) SDTMMETA = &SDTMMETA;";
      put "   %nrstr(%let) ADAMMETA = &ADAMMETA;";
      put "   %nrstr(%let) BASEMETA = &BASEMETA;";
      put "   libname sdtmmeta;";
      put "   %nrstr(%if %length(&SDTMMETA) %then %do;)";
      put "      %nrstr(%if %sysfunc(fileexist(&SDTMMETA)) %then %do;)"; 
      put "         libname sdtmmeta '&SDTMMETA';";
      put "      %nrstr(%end;)";
      put "   %nrstr(%end;)";
      put "   libname adammeta;";
      put "   %nrstr(%if %length(&ADAMMETA) %then %do;)";
      put "      %nrstr(%if %sysfunc(fileexist(&ADAMMETA)) %then %do;)"; 
      put "         libname adammeta '&ADAMMETA';";
      put "      %nrstr(%end;)";
      put "   %nrstr(%end;)";
      put "   libname basemeta;";
      put "   %nrstr(%if %length(&BASEMETA) %then %do;)";
      put "      %nrstr(%if %sysfunc(fileexist(&BASEMETA)) %then %do;)"; 
      put "         libname basemeta '&BASEMETA';";
      put "      %nrstr(%end;)";
      put "   %nrstr(%end;)";
      put "   libname sdtm;";
      put "   %nrstr(%if %length(&InPath) %then %do;)";
      put "      %nrstr(%if %sysfunc(fileexist(%"&InPath%")) %then %do;)"; 
      put "         libname sdtm cvp '&InPath';";
      put "      %nrstr(%end;)";
      put "   %nrstr(%end;)";
      put "   libname adam;";
      put "   %nrstr(%if %length(&AInPath) %then %do;)";
      put "      %nrstr(%if %sysfunc(fileexist(%"&AInPath%")) %then %do;)"; 
      put "         libname adam cvp '&AInPath';"; 
      put "      %nrstr(%end;)";
      put "   %nrstr(%end;)";
      put "%nrstr(%mend;)";
      put "%nrstr(%libassign;)";
   run;

   %macro ALPHABETIZE(varlist = );      
      %if %length(&&&varlist) %then %do;
         data alph;
            length domain $ 8;
            %let i = 1;
            %do %while(%length(%scan(&&&varlist,&i,' ')));
               %let temp = %scan(%upcase(&&&varlist),&i," ");
               domain = "&temp";
               output;
              %let i = %eval(&i+1);
            %end; 
         run;

         proc sort data = alph;
            by domain;
         run;

         proc sql noprint;
            select domain
               into :&varlist separated by ' '
               from alph;
         quit;
         
         proc datasets nolist;
            delete alph;
         quit;
    %end;
   %mend ALPHABETIZE;
   
   %ALPHABETIZE(varlist = domains);
   %ALPHABETIZE(varlist = eventsdomains);
   %ALPHABETIZE(varlist = findingsdomains);
   %ALPHABETIZE(varlist = interventionsdomains);
   %ALPHABETIZE(varlist = suppdomains);
   %ALPHABETIZE(varlist = bdsdomains);
   
   *** Create Domain Lists;
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) domains adslpresent findingsdomains eventsdomains interventionsdomains suppdomains bdsdomains cdiscmodel;";
      put "%nrstr(%let) cdiscmodel = &cdiscmodel;";
      put "%nrstr(%let) domains = &domains;";
      put "%nrstr(%let) adslpresent = &adslpresent;";
      put "%nrstr(%let) findingsdomains = &findingsdomains;";
      put "%nrstr(%let) eventsdomains = &eventsdomains;";
      put "%nrstr(%let) interventionsdomains = &interventionsdomains;";
      put "%nrstr(%let) suppdomains = &suppdomains;";
      put "%nrstr(%let) bdsdomains = &bdsdomains;";
   run;

   %let i = 1;
   *** Creates domain data set specifications;
   %do %while(%length(%scan(&domains,&i,' ')));
      %let temp = %scan(&domains,&i,' ');
      %if %sysfunc(findw(&alladamdomains, AD&temp)) %then %do;
         %let in_&temp = adam.AD&temp;
         data _null_;
           file "&basemeta&_slash_.CSTMacroVars.sas" mod;
           put "%nrstr(%global) in_&temp;";
          put "%nrstr(%let) in_&temp = adam.AD&temp;";
        run;   
      %end;
      %else %if ^%sysfunc(findw(&alladamdomains, AD&temp)) %then %do;
         %let in_&temp = sdtm.&temp;
          data _null_;
           file "&basemeta&_slash_.CSTMacroVars.sas" mod;
           put "%nrstr(%global) in_&temp;";
          put "%nrstr(%let) in_&temp = sdtm.&temp;";
        run;   
      %end;
      %let i = %eval(&i+1);
   %end;
   *** Creates adsl data set specifications;
   %if %length(&adslpresent) > 0 %then %do;
      %let in_ADSL = adam.ADSL;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) in_ADSL;";
        put "%nrstr(%let) in_ADSL = adam.ADSL;";
      run;  
   %end;
   %else %if %length(&adslpresent) = 0 and %sysfunc(findw(&domains, DM)) %then %do;
      %let in_ADSL = &in_dm;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) in_ADSL;";
        put "%nrstr(%let) in_ADSL = &in_dm;";
      run;
      %end;

   %let _tid_ = %sysfunc(open(&in_adsl));
   %let vn = %sysfunc(varnum(&_tid_,Country));
   %let _tid_ = %sysfunc(close(&_tid_));        
                        
   %if &vn %then %do;   
      proc sql noprint;
         create table countries(where = (country ne "")) as
            select distinct country
            from &in_adsl
            order by country;
      quit;
      
      %if &Server = Local %then %do;
         %let CPath = &USER_HOME.RBM/;
      %end;
      %else %do;
         %let CPath = &MetaPath.RBM/;
      %end;
      libname CPath "&CPath";
      %if %sysfunc(fileexist(&CPath.countries.sas7bdat)) %then %do;
         %let _tid_ = %sysfunc(open(CPath.countries));
         %let vn = %sysfunc(varnum(&_tid_,Country));
         %let CLen0 = %sysfunc(varlength(&_tid_,&vn));
         %let _tid_ = %sysfunc(close(&_tid_));
         %let _tid_ = %sysfunc(open(countries));
         %let vn = %sysfunc(varnum(&_tid_,Country));
         %let CLen1 = %sysfunc(varlength(&_tid_,&vn));
         %let _tid_ = %sysfunc(close(&_tid_));                                             
         
         data countries;
            length Country $ %sysfunc(max(&CLen0,&CLen1));
            set CPath.countries countries;
         run;                    

         proc sql noprint;
            create table countries2(where = (country ne "")) as
               select distinct country
               from countries
               order by country;
         quit;       

         data CPath.countries;
            set countries;
         run;     

         proc datasets nolist;
            delete countries countries2;
         quit;

      %end;
      %else %do;
         data CPath.countries;
            set countries;
         run;
          
         proc datasets nolist;
            delete countries;
         quit;
      %end;
   %end; 

   data bothlabels;
      length table $ 8 label $ 100;
      set %if %length(&InPath) %then %do; sdtmmeta.source_tables(keep = table label in = ina) %end;
          %if %length(&AInPath) %then %do; adammeta.source_tables(keep = table label in = inb) %end;;
      if label = "" then do;
        if ina then label = table;
        else if inb then label = substr(table,3);
      end;
      label = TRANSTRN(label,"'",TRIMN(""));
      label = TRANSTRN(label,'"',TRIMN(""));
   run;

   *** Create TXT files of Domains;
    data findings;
      length domain $ 8 table $ 8;
      %let i = 1;
      %do %while(%length(%scan(&findingsdomains,&i,' ')));
         %let temp = %scan(%upcase(&findingsdomains),&i," ");
         %let prefix = %scan(&&in_&temp, 1, ".");
         domain = "&temp"; 
         %if %upcase(&prefix) = ADAM %then %do;
            table = "AD&temp";
         %end;
         %else %do;
            table = "&temp";
         %end;
         output;
         %let i = %eval(&i+1);
      %end; 
   run; 

    data events;
      length domain $ 8 table $ 8;
      %let i = 1;
      %do %while(%length(%scan(&eventsdomains,&i,' ')));
         %let temp = %scan(%upcase(&eventsdomains),&i," ");
         %let prefix = %scan(&&in_&temp, 1, ".");
         domain = "&temp"; 
         %if %upcase(&prefix) = ADAM %then %do;
            table = "AD&temp";
         %end;
         %else %do;
            table = "&temp";
         %end;
         output;
         %let i = %eval(&i+1);
      %end; 
   run; 

   data interventions;
      length domain $ 8 table $ 8;
      %let i = 1;
      %do %while(%length(%scan(&interventionsdomains,&i,' ')));
         %let temp = %scan(%upcase(&interventionsdomains),&i," ");
         %let prefix = %scan(&&in_&temp, 1, ".");
         domain = "&temp"; 
         %if %upcase(&prefix) = ADAM %then %do;
            table = "AD&temp";
         %end;
         %else %do;
            table = "&temp";
         %end;
         output;
         %let i = %eval(&i+1);
      %end; 
   run;
   
   *** Prepare to create a txt file to have all of the domains;
   data Alldomains;
      length Name $8;
      set %if %length(&InPath) %then %do; sdtmmeta.source_tables %end; %if %length(&AInPath) %then %do; adammeta.source_tables %end;;
   run;
   data AllnoUsedDom;
      length Name $8;
      set %if %length(&AInPath) %then %do; adamnousedDom %end; %if %length(&InPath) %then %do; sdtmnousedDom %end;;
   run;
   
   %let findingsdomainslabel = ;
   %let eventsdomainslabel = ;
   %let interventionsdomainslabel = ;
   
   proc sql noprint;
      create table findings2(drop = table1 table2) as
         select coalesce(A.table1, B.table2) as table, A.*, B.*
         from findings(rename=(table=table1)) as A left join bothlabels(rename=(table=table2)) as B
         on A.table1 = B.table2
         order by domain;
      select label
         into :findingsdomainslabel separated by '~'
         from findings2
         order by domain;
      create table events2(drop = table1 table2) as
         select coalesce(A.table1, B.table2) as table, A.*, B.*
         from events(rename=(table=table1)) as A left join bothlabels(rename=(table=table2)) as B
         on A.table1 = B.table2
         order by domain;
      select label
         into :eventsdomainslabel separated by '~'
         from events2
         order by domain;
      create table interventions2(drop = table1 table2) as
         select coalesce(A.table1, B.table2) as table, A.*, B.*
         from interventions(rename=(table=table1)) as A left join bothlabels(rename=(table=table2)) as B
         on A.table1 = B.table2
         order by domain;
      select label
         into :interventionsdomainslabel separated by '~'
         from interventions2
         order by domain;
   quit;
   
   data _null_;
      set findings2(drop = table);
      file "&basemeta&_slash_.FINDINGS.txt" delimiter = "~" encoding="utf-8";
      put domain label;
   run;

   data _null_;
      set events2;
      file "&basemeta&_slash_.EVENTS.txt" delimiter = "~" encoding="utf-8";
      put domain label;
   run; 

   data _null_;
      set interventions2;
      file "&basemeta&_slash_.INTERVENTIONS.txt" delimiter = "~" encoding="utf-8";
      put domain label;
   run; 
   
   data _null_;
      set Alldomains(drop = label class);
      file "&basemeta&_slash_.ALLDOMAINS.txt" delimiter = "~" encoding="utf-8";
      put table flag;
   run;

   proc datasets nolist;
      delete findings events interventions events2 interventions2 ;
   quit;

   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) findingsdomainslabel eventsdomainslabel interventionsdomainslabel;";
      put "%nrstr(%let) findingsdomainslabel = &findingsdomainslabel;";
      put "%nrstr(%let) eventsdomainslabel = &eventsdomainslabel;";
      put "%nrstr(%let) interventionsdomainslabel = &interventionsdomainslabel;";
   run;

   *** Creates data set variables lists based on specifications.  Adds SUPP variables if available;
   %let i = 1;
   %global DMVariables;
   %let DMVariables=;
   %do %while(%length(%scan(&domains,&i," ")));
      %let temp = %scan(&domains,&i," ");
      %let prefix = %scan(&&in_&temp, 1, ".");
      %let supp =;
      %if %upcase(&prefix) ne ADAM and %sysfunc(findw(&sdtmsuppdomains, %upcase(SUPP&temp))) %then %let supp = &&supp&temp.var;
      %global &temp.VARS;
      %if %upcase(&prefix) = ADAM %then %let &temp.VARS = %upcase(&&AD&temp.var);
      %else %let &temp.VARS = %upcase(&&&temp.var);
      %put i=&i temp=&temp;
      %if %upcase(&temp) = DM %then %let DMVariables=&&&temp.VARS;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) &temp.VARS;";
         %if %upcase(&prefix) = ADAM %then %do;
            put "%nrstr(%let) %sysfunc(trim(&temp.VARS = %upcase(&&AD&temp.var)));";
         %end;
         %else %do;
            put "%nrstr(%let) %sysfunc(trim(&temp.VARS = %upcase(&&&temp.var)));";
         %end;
         %if %length(&supp) %then %do;
            put "%nrstr(%global) &temp.SUPPVARS;";
            put "%nrstr(%let) %sysfunc(trim(&temp.SUPPVARS = %upcase(&supp)));";
         %end;
      run;
      %let i = %eval(&i+1);
   %end;

   %if %upcase(%scan(&in_ADSL, 1, ".")) = ADAM %then %do;
       %global ADSLVARS;
      %let ADSLVARS = &ADSLVAR;
      * add variables in DM, but not in ADSL to ADSLVARS list;
      %let _i_=1;
      %do %while(%length(%scan(&DMVariables,&_i_," ")));
         %let DMtemp=%scan(&DMVariables,&_i_," ");
         %if %sysfunc(findw(&ADSLVARS, &DMtemp))=0 %then %let ADSLVAR = &ADSLVAR &DMtemp;
         %let _i_ = %eval(&_i_+1);
      %end;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) ADSLVARS;";
        put "%nrstr(%let) %sysfunc(trim(ADSLVARS = %upcase(&ADSLVAR)));";
      run;
   %end;
   %else %if %upcase(%scan(&in_ADSL, 1, ".")) = SDTM %then %do;
      %let supp = ;
      %if %sysfunc(findw(&sdtmsuppdomains, SUPPDM)) %then %let supp = &suppdmvar;
       %global ADSLVARS;
      %let ADSLVARS = &dmvar;
      %if %length(&supp) %then %do;
         %global ADSLSUPPVARS;
         %let ADSLSUPPVARS = %upcase(&supp);
      %end;      
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) ADSLVARS;";
         put "%nrstr(%let) %sysfunc(trim(ADSLVARS = %upcase(&dmvar)));";
         %if %length(&supp) %then %do;
            put "%nrstr(%global) ADSLSUPPVARS;";
            put "%nrstr(%let) %sysfunc(trim(ADSLSUPPVARS = %upcase(&supp)));";
         %end;
      run;
   %end;  

   *** Create separate findings domain for character and numeric;
   %global findingsdomainsnum findingsdomainschar;
   %let findingsdomainsnum = ;
   %let findingsdomainschar = ;
   %let i = 1;
   %do %while(%length(%scan(&findingsdomains,&i," ")));
      %let temp = %scan(&findingsdomains,&i," ");     
      %if %index(&&&temp.VARS,&temp.STRESN) %then %let findingsdomainsnum = &findingsdomainsnum &temp;
      %if %index(&&&temp.VARS,&temp.STRESC) and ^%index(&&&temp.VARS,&temp.STRESN) %then %let findingsdomainschar = &findingsdomainschar &temp;
      %let i = %eval(&i+1);
   %end;
   
   %if %length(&findingsdomainschar) %then %do;
      data findingschar;
         length domain $ 8 table $ 8;
         %let i = 1;
         %do %while(%length(%scan(&findingsdomainschar,&i,' ')));
            %let temp = %scan(%upcase(&findingsdomainschar),&i," ");
            domain = "&temp"; 
            output;
            %let i = %eval(&i+1);
         %end;
      run;
      
      data _null_;
         merge findingschar(in = ina) findings2(drop = table);
         by domain;
         if ina;
         file "&basemeta&_slash_.FINDINGSCHAR.txt" delimiter = "~" encoding="utf-8";
         put domain label;
      run;  
   %end;
   
   %if %length(&findingsdomainsnum) %then %do;
      data findingsnum;
         length domain $ 8 table $ 8;
         %let i = 1;
         %do %while(%length(%scan(&findingsdomainsnum,&i,' ')));
            %let temp = %scan(%upcase(&findingsdomainsnum),&i," ");
            domain = "&temp"; 
            output;
            %let i = %eval(&i+1);
         %end;
      run;

      data _null_;
         merge findingsnum(in = ina) findings2(drop = table);
         by domain;
         if ina;
         file "&basemeta&_slash_.FINDINGSNUM.txt" delimiter = "~";
         put domain label;
      run;
   %end;

   %if &cdiscmodel = SEND %then %do;
      %let pooliddomains =;
      %let i = 1;   
      %do %while(%length(%scan(&findingsdomains,&i,' ')));
         %let temp = %scan(%upcase(&findingsdomains),&i," ");
         %if %index(&&&temp.VARS,POOLID) %then %let pooliddomains = &pooliddomains &temp;
         %let i = %eval(&i+1);
      %end;
   %end;
   
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) findingsdomainsnum findingsdomainschar;";
      put "%nrstr(%let) findingsdomainsnum = &findingsdomainsnum;";
      put "%nrstr(%let) findingsdomainschar = &findingsdomainschar;";
      %if &cdiscmodel = SEND %then %do;
         put "%nrstr(%let) pooliddomains = &pooliddomains;";
      %end;
   run;
   
   %MAKENOTES;
   %if &exiterror %then %goto mdoexit;
   
   proc datasets nolist;
      delete findings2 findingschar findingsnum;
   quit;
   
   *** Create Treatment Macro Vars and Data Sets;
   %let TRTLIST = ;
   %let TRTLISTQ = ;

   %if %sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
      %let TRTLIST = &TRTLIST ARM;
      %let TRTLISTQ = &TRTLISTQ "ARM";
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), ACTARM)) %then %do;
      %let TRTLIST = &TRTLIST ACTARM;
      %if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) %then %do;
         %let TRTLISTQ = "ACTARM";
      %end;
      %else %do;
         %let TRTLISTQ = &TRTLISTQ ,"ACTARM";
      %end;
   %end;
   %let i = 1;
   %let trtflg = 0;
   %do %while(&trtflg = 0);
      %if %index(&ADSLVARS, TRT0&i.P) > 0 %then %do;
         %let TRTLIST = &TRTLIST TRT0&i.P;
         %if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM)) and ^%sysfunc(findw(%upcase(&ADSLVARS), ACTARM)) %then %do;
            %let TRTLISTQ = "TRT0&i.P";
         %end;
         %else %do;
            %let TRTLISTQ = &TRTLISTQ ,"TRT0&i.P";
         %end;
      %end;
      %else %let trtflg = 1;
      %let i = %eval(&i+1);
   %end;
   %let i = 1;
   %let trtflg = 0;
   %do %while(&trtflg = 0);
      %if %index(&ADSLVARS, TRT0&i.A) > 0 %then %do;
         %let TRTLIST = &TRTLIST TRT0&i.A;
         %if %length(&TRTLISTQ) %then %let TRTLISTQ = &TRTLISTQ ,"TRT0&i.A";
         %else %let TRTLISTQ = "TRT0&i.A";
      %end;
      %else %let trtflg = 1;
      %let i = %eval(&i+1);
   %end;
   
   proc sql noprint;
      %let i = 1;
      %do %while(%length(%scan(&TRTLIST,&i," ")));
         %let temp = %scan(&TRTLIST,&i," ");
              select distinct &temp
                 into :&temp.CDS separated by '~'
                 from &in_ADSL;
         %let i = %eval(&i+1);                 
           %end;
      %if %length(&TRTLISTQ) %then %do;      
         create table treatvars as 
            select distinct column, label
            from %scan(&in_adsl, 1, ".")meta.source_columns
            where upcase(table) = "%scan(&in_adsl, 2, ".")" and upcase(column) in (&TRTLISTQ);
      %end;
      %let i = 1;
      %do %while(%length(%scan(&TRTLIST,&i," ")));
         %let temp = %scan(&TRTLIST,&i," ");
         create table &temp.CDS as 
            select distinct &temp
            from &in_ADSL;
         %let i = %eval(&i+1);                 
           %end;           
   quit;

   *** Text File of Treatment Variables;
   %if %length(&TRTLISTQ) %then %do;      
      data _null_;
         set treatvars;
         file "&basemeta&_slash_.TREATVARS.txt" delimiter = "~" encoding="utf-8";
         put column label;
      run;
   %end;
   %else %do;
      data treatvars;
      run;
   
      data _null_;
         set treatvars;
         file "&basemeta&_slash_.TREATVARS.txt" delimiter = "~" encoding="utf-8";
      run;  
   %end;
   
   *** Create TXT files of Treatment Values;
   %let i = 1;
   %do %while(%length(%scan(&TRTLIST,&i,' ')));
      %let temp = %scan(&TRTLIST,&i,' ');
      data _null_;
         set &temp.CDS;
         file "&basemeta&_slash_.&temp.CDS.txt" delimiter = "~" encoding="utf-8";
         put &temp;
      run;
      %let i = %eval(&i+1); 
   %end;
      
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) TRTLIST;";
           put "%nrstr(%let) TRTLIST = &TRTLIST;";
      %let i = 1;
      %do %while(%length(%scan(&TRTLIST,&i," ")));
         %let temp = %scan(&TRTLIST,&i," ");
            put "%nrstr(%global) &temp.CDS;";
              put "%nrstr(%let) &temp.CDS = %nrstr(%nrbquote)(%nrbquote(&&&temp.CDS));";
         %let i = %eval(&i+1);                 
           %end;
   run;

   proc sql noprint;
      *** Create Findings Test Code data sets;  
      %let i = 1;
      %do %while(%length(%scan(&findingsdomains,&i,' ')));
         %let temp = %scan(&findingsdomains,&i,' ');
            %if %sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
            create table &temp.TESTCD as
           select distinct PARAMCD %if %sysfunc(findw(&&&temp.VARS, PARAM)) > 0 %then %do; , PARAM %end;
         from &&in_&temp;
         %end;
         %else %do;
            create table &temp.TESTCD as
            select distinct &temp.TESTCD %if %sysfunc(findw(&&&temp.VARS, &temp.TEST)) > 0 %then %do; , &temp.TEST %end;
            from &&in_&temp;
         %end;                
           %let i = %eval(&i+1);
       %end;
   quit;

   *** Create TXT files of Findings Test Codes;
   %let i = 1;
   %do %while(%length(%scan(&findingsdomains,&i,' ')));
       %let temp = %scan(&findingsdomains,&i,' ');
      data _null_;
         %if %sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
            set &temp.testcd(rename = (paramcd = &temp.testcd param = &temp.test));
         %end;
         %else %do;  
            set &temp.testcd;
         %end;
         file "&basemeta&_slash_.&temp.TESTCD.txt" delimiter = "~" encoding="utf-8";
         put &temp.testcd &temp.test;
      run;
      
      %* compute quantile during add study process;
      %ComputeBaseline(&temp,_blD&temp);
      %if %sysfunc(exist(work._blD&temp)) %then %do;
         %NObsVars(_blD&temp); 
         %if &nobs >0 %then %do;
            %ComputeQuantiles(&temp,_blD&temp,fdQuantile);
            %if %sysfunc(exist(work.fdQuantile)) %then %do;
               %NObsVars(fdQuantile); 
               %if &nobs >0 & (&ProcessName = AddStudyLocal or &ProcessName = ManageStudies) %then %do;
                  data basemeta.&temp.QUANTILES;
                     set fdQuantile;
                  run;
                  proc datasets nolist;
                     delete fdQuantile;
                  quit;
               %end;
            %end;
         %end;
         proc datasets nolist;
            delete _blD&temp;
         quit;
      %end;
      %let i = %eval(&i+1); 
   %end;
   %if &exiterror %then %goto mdoexit;

   *** Population codes;
   %if %symexist(ADSLSUPPVARS) %then %let POPFULLLIST = &ADSLVARS &ADSLSUPPVARS;
   %else %let POPFULLLIST = &ADSLVARS;
      
   *** Create Treatment Macro Vars and Data Sets;
   %let POPLISTQ = All Subjects Excluding Screen Failures~All Subjects;
   %let POPLISTVAR = ;

   *** Full Set;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), FASFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Full Analysis Set;
      %let POPLISTVAR = &POPLISTVAR FASFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), FULLSET)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Full Analysis Set;
      %let POPLISTVAR = &POPLISTVAR FULLSET;
   %end;
   
   *** Safety;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), SAFFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Safety;
      %let POPLISTVAR = &POPLISTVAR SAFFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), SAFETY)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Safety;
      %let POPLISTVAR = &POPLISTVAR SAFETY;
   %end; 

   *** ITT;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), ITTFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Intent-to-Treat;
      %let POPLISTVAR = &POPLISTVAR ITTFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), ITT)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Intent-to-Treat;
      %let POPLISTVAR = &POPLISTVAR ITT;
   %end;
   
   *** PPROT;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), PPROTFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Per-Protocol;
      %let POPLISTVAR = &POPLISTVAR PPROTFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), PPROT)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Per-Protocol;
      %let POPLISTVAR = &POPLISTVAR PPROT;
   %end; 
   
   *** Completer;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), COMPFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Completers;
      %let POPLISTVAR = &POPLISTVAR COMPFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), COMPLETED)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Completers;
      %let POPLISTVAR = &POPLISTVAR COMPLETED;
   %end; 
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), COMPLT)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Completers;
      %let POPLISTVAR = &POPLISTVAR COMPLT;
   %end;    

   *** Randomized;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), RANDFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Randomized;
      %let POPLISTVAR = &POPLISTVAR RANDFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), RANDOMIZED)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Randomized;
      %let POPLISTVAR = &POPLISTVAR RANDOMIZED;
   %end; 
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), RANDOM)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Randomized;
      %let POPLISTVAR = &POPLISTVAR RANDOM;
   %end;    

   *** Enrolled;
   %if %sysfunc(findw(%upcase(&POPFULLLIST), ENRLFL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Enrolled;
      %let POPLISTVAR = &POPLISTVAR ENRLFL;
   %end;
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), ENROLLED)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Enrolled;
      %let POPLISTVAR = &POPLISTVAR ENROLLED;
   %end; 
   %else %if %sysfunc(findw(%upcase(&POPFULLLIST), ENRL)) > 0 %then %do;
      %let POPLISTQ = &POPLISTQ ~Enrolled;
      %let POPLISTVAR = &POPLISTVAR ENRL;
   %end;
   
   /* 
   %let POPLISTQ = %SUBSTR(%quote(&POPLISTQ) ,2);
   %if %upcase(&in_adsl) = SDTM.DM and %sysfunc(findw(%upcase(&suppdomains), SUPPDM)) > 0 %then %do;
      proc sql noprint;
         create table popdata2 as 
            select column, label
               from %scan(&in_adsl, 1, ".")meta.source_columns(where = (upcase(table) = "%scan(&in_adsl, 2, ".")"))
            union
            select column, label
               from sdtm.suppdm(keep = qnam qlabel rename = (qnam = column qlabel = label))
            order by column, label;          
         create table popdata as
            select distinct column, label
            from popdata2(where = (upcase(column) in (&POPLISTQ)))
            order by column, label;          
      quit;
      
      proc datasets nolist;
         delete popdata2;
      quit;
   %end;
   %else %do;
      proc sql noprint;
         create table popdata as 
            select distinct column, label
               from %scan(&in_adsl, 1, ".")meta.source_columns(where = (upcase(table) = "%scan(&in_adsl, 2, ".")" and upcase(column) in (&POPLISTQ)))
            order by column, label;          
      quit;   
   %end;

   data popdata;
      set popdata;
      if label = "" then label = column;
   run;
   */
   
   data popdata;
      length varname $ 8 populate $ 38;
      %let i = 1;
      %do %while(%length(%scan(&POPLISTQ,&i,'~')));
         %let temp = %scan(&POPLISTQ,&i,"~");
         varname = cats("FakeVar","&i");
         populate = "&temp";
         output;
        %let i = %eval(&i+1);
      %end; 
   run;
   /*
   data popdata;
      length varname $ 8;
      set popdata;
      varname = cats("FakeVar",_n_);
   run;      
   */
   *** Text File of Population Variables;
   data _null_;
      set popdata;
      file "&basemeta&_slash_.POPVARS.txt" delimiter = "~" encoding="utf-8";
      put varname populate;
   run;
   
   proc datasets nolist;
      delete popdata;
   quit;
   
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) poplistvar;";
      put "%nrstr(%let) poplistvar = &poplistvar;";
   run;  

   *** MedDRA codes ***;
   %if %sysfunc(findw(%upcase(&domains), AE)) %then %do; 
      proc sql noprint;
         create table aemeddra as 
            select distinct column, label
            from %scan(&in_ae, 1, %str(.))meta.source_columns
            where upcase(table) = "%scan(&in_ae, 2, %str(.))";
      quit;

      /*
      data outlib.aemeddra;
         set aemeddra;
      run;
      */

      %let _tid_ = %sysfunc(open(aemeddra));
      %let labelfl = %sysfunc(varnum(&_tid_,label));     
      %let _tid_ = %sysfunc(close(&_tid_));
         
      data aemeddra;
         %if ^&labelfl %then %do; length label $ 8; %end;
         set aemeddra;
         where upcase(column) in ("AETERM","AELLT","AEDECOD","AEBODSYS","AESOC","AEHLT","AEHLGT");
         %if ^&labelfl %then %do; label = "";  %end;        
      run;
      
      data aemeddra;
         set aemeddra;
         if label = "" then label = column;
      run;
      
      data _null_;
         set aemeddra;
         file "&basemeta&_slash_.AEMEDDRA.txt" delimiter = "~" encoding="utf-8";
         put column label;
      run;     

      proc datasets nolist;
         delete aemeddra;
      quit;    
   %end;
   %else %do;
      data _null_;
         file "&basemeta&_slash_.AEMEDDRA.txt" delimiter = "~" encoding="utf-8";
      run;   
   %end;
   
   %if ^%symexist(NeedDelta) %then %do;
      %global NeedDelta;
      %let NeedDelta = NO;
   %end;
   
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
   
   %if &ProcessName = AddStudyLocal %then %do;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) ReviewsEnabled ;";
         %if &SnpComp = Yes %then %do;          
            put "%nrstr(%let) ReviewsEnabled = 1;";         
         %end;
         %else %do;
            put "%nrstr(%let) ReviewsEnabled = 0;";    
         %end;
      run;
      
      *create a .sas dataset to save the snapshot comparison information for managestudies;
      libname metalib "%jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder)";
      data metalib.SNAPSHOT;
         %if &SnpComp = Yes %then %do; 
            ReviewsEnabled= 1;
         %end;
         %else %do;
            ReviewsEnabled= 0;
         %end; 
      run;
   %end;
   %if &ProcessName = AddStudyLocal or (&ProcessName = ManageStudies and &NeedDelta = YES) %then %do;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         %if (&ProcessName = ManageStudies and &NeedDelta = YES) %then %do;
            put "%nrstr(%global) ReviewsEnabled ;";         
            put "%nrstr(%let) ReviewsEnabled = &ReviewsEnabled;";
         %end;
         put "%nrstr(%global) ReviewNumber;";
         put "%nrstr(%let) ReviewNumber = 0;";
      run;   
   %end;
   %else %if &ProcessName = ManageStudies %then %do;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) ReviewsEnabled ;";         
         put "%nrstr(%let) ReviewsEnabled = &ReviewsEnabled;";
      run;
      %if &NewSnap = YES %then %let CurrentReview = %eval(&CurrentReview + 1);
      %else %if &NewSnap = NO %then %let CurrentReview = &CurrentReview;
      data _null_;
         file "&basemeta&_slash_.CSTMacroVars.sas" mod;
         put "%nrstr(%global) ReviewNumber;";
         put "%nrstr(%let) ReviewNumber = &CurrentReview;";
      run;   
   %end;   

   *** Generate Data Set Keys ***;
   %KEYS;
   %if &exiterror %then %goto mdoexit;

   /* %MERGESUPP(addstudy = 1); */
   
   %if &ProcessName = AddStudyLocal or (&ProcessName = ManageStudies and &NeedDelta = YES) %then %do;
      %if &ProcessName = AddStudyLocal %then %do; 
         %if &SnpComp =Yes %then %do;
            %COMPAREADD;   
         %end;
      %end;
      %else %do;
         %let meta=%jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder);
         libname metalib "&meta";
         %let snapfile=SNAPSHOT.sas7bdat;
         %if %sysfunc(fileexist(&meta.&snapfile)) %then %do;
            %global ReviewsEnabled;
            data _null_;
               set metalib.SNAPSHOT;
               call symput('ReviewsEnabled',ReviewsEnabled);
            run;
         %end;
         %else %do;
            * for previous added study: there is no ReviewsEnabled variable;
            %if ^%symexist(ReviewsEnabled) %then %do;
               %global ReviewsEnabled;
               %let ReviewsEnabled=1;
            %end;
         %end;
         %if &ReviewsEnabled %then %do;
            %COMPAREADD; 
         %end;
      %end;
      %DOMAINLISTADD;
   %end; 
   %else %if &ProcessName = ManageStudies %then %do;
      %if &NewSnap = YES %then %do;
         *** Copy File to Metadata Folder ***;
         data getprev;
            set RevLib.DeltaFile&CurrentReview;
         run;
         
         data aa;
            length text $ 200;
            file "%jg_getPath(/user/clinical/metadata/base, DeltaFile&CurrentReview..sas, 0, 0, 3, &studynameunder)";
            set getprev;
            put text;
         run;  
         
         proc datasets nolist;
            delete getprev;
         quit;
         
         *** Delete File from Output Folder ***;
         data _null_;
            fname = "tempfile";
            rc = filename(fname,"&ServerOutPath.DeltaFile&CurrentReview..sas7bdat");
            if rc = 0 and fexist(fname) then rc = fdelete(fname);
            rc = filename(fname);
         run;        

         %COMPARE;
      %end;
      %else %do;
         data _null_;
            file "&basemeta&_slash_.CSTMacroVars.sas" mod;
            put "libname DeltaLib %nrstr(%')%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder)%nrstr(%');";
            put "%nrstr(%let) DeltaDir = %jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder);";
         run;  
      %end;
      %if %index(&Task,Update) or %index(&Task,Refresh) %then %do;
         %DOMAINLISTADD;
      %end;
   %end;

   *** Generate Filter Data Sets;
   /*
   %let i = 1;
   %do %while(%length(%scan(&domains,&i,' ')));
      %let temp = %scan(&domains,&i,' ');
         %CREATEFILTER(dtemp = &temp);
      %let i = %eval(&i+1);
   %end; 

   %CREATEFILTER(dtemp = ADSL);*/
   /* %PATCH31; */
   options notes;
   %mdoexit:;
   %put exiting METADATAOUTFILES;

%mend METADATAOUTFILES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CREATEFILTER(dtemp = &temp);
   %let genvars = STUDYID SITEID ARM ARMCD ACTARM ACTARMCD TRT01P TRT01A SEX RACE ETHNIC COUNTRY VISIT VISITNUM FASFL SAFFL ITTFL PPROTFL COMPLFL RANDFL ENRLFL;
   %let specvars = BLFL BODSYS CAT CLAS DECOD DOSFRM FAST INDC LOC METHOD NRIND OUT POS REL RESCAT ROUTE SCAN SCAT SDTH SER SEV SHOSP SLIFE SMIE SOD SPEC TERM TEST TESTCD TOX TOXGR TPT TRT;

    %let varlist = ;
  
    %let j = 1;
    %do %while(%length(%scan(&genvars,&j,' ')));
      %let temp2 = %scan(&genvars,&j,' ');
      %if %sysfunc(indexw(&&&dtemp.VARS, &temp2, ' ')) > 0 %then %let varlist = &varlist &temp2;
        %let j = %eval(&j+1);
    %end;
   %if &dtemp ne ADSL %then %do; 
      %let k = 1;
       %do %while(%length(%scan(&specvars,&k,' ')));
         %let temp3 = &dtemp%scan(&specvars,&k,' ');
         %if %sysfunc(indexw(&&&dtemp.VARS, &temp3, ' ')) > 0 %then %let varlist = &varlist &temp3;
           %let k = %eval(&k+1);
       %end;
   %end;
   proc sort data = &&in_&dtemp(keep = &varlist) nodupkey out = &dtemp.FILTER;
      by &varlist;
   run;
   %if %sysfunc(indexw(&&&dtemp.VARS, AGE, ' ')) > 0 %then %do;
       proc means nmiss min max noprint data = &&in_&dtemp;
         var age;
         output out = stats min = min max = max nmiss = nmiss;
      run;

      proc transpose data = stats out = stats(rename = (col1 = age));
      run;

      data stats(where = (upcase(_NAME_) in ("NMISS", "MAX", "MIN")));
         set stats;
         if upcase(_NAME_) = "NMISS" then do;
            if age > 0 then age = .;
            else if age = 0 then delete;
         end;
      run;
      
      proc sql noprint undo_policy = none;
         create table &dtemp.FILTER as
           select * from &dtemp.FILTER, stats(drop = _NAME_);
        quit; 
    %end;
    %if %sysfunc(indexw(&&&dtemp.VARS, &dtemp.STRESN, ' ')) > 0 %then %do;
       proc means nmiss min max noprint data = &&in_&dtemp;
         var &dtemp.STRESN;
         output out = stats min = min max = max nmiss = nmiss;
      run;

      proc transpose data = stats out = stats(drop = _label_ rename = (col1 = &dtemp.STRESN));
      run;

      data stats(where = (upcase(_NAME_) in ("NMISS", "MAX", "MIN")));
         set stats;
         if upcase(_NAME_) = "NMISS" then do;
            if &dtemp.STRESN > 0 then &dtemp.STRESN = .;
            else if &dtemp.STRESN = 0 then delete;
         end;
      run;
      
      proc sql noprint undo_policy = none;
         create table &dtemp.FILTER as
           select * from &dtemp.FILTER, stats(drop = _NAME_);
        quit; 
    %end;

   data basemeta.&dtemp.FILTER;
      set &dtemp.FILTER;
   run;

   proc datasets nolist;
      delete &dtemp.FILTER;
   quit;
%mend CREATEFILTER;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro PATCH31;
   %let list31 = AE CM CO DM DS EG EX LB MH SU SV VS;
   %let i = 1;
   %do %while(%length(%scan(&list31,&i,' ')));
      %let temp = %scan(&list31,&i,' ');
      %if ^%sysfunc(findw(&domains, &temp)) %then %do;
         data _null_;
            file "&basemeta&_slash_.CSTMacroVars.sas" mod;
            put "%nrstr(%global) in_&temp;";
            put "%nrstr(%let) in_&temp = ;";
         run;
      %end;
      %let i = %eval(&i+1);
   %end;       
%mend PATCH31;
   
*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro PURGE(list = , flag = );
   %let i = 1;
   %do %while(%length(%scan(&list ,&i,' ')));
      %let temp = %scan(&list ,&i,' ');
      if upcase(table) = "%upcase(&temp)" then delete;
      %if &flag = AD and &temp ne SL %then %do;
         if upcase(table) = "AD.%upcase(&temp)" then delete;
      %end;
      %let i = %eval(&i+1);
   %end;    
%mend PURGE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro GENERATEMETA(studname = &studyname);
   %global CSTERROR;
   %let CSTERROR = 0;

   %if %length(&InPath) %then %do;
      libname sdtm cvp "&InPath";
   %end; 
   %if %length(&AInPath) %then %do;    
      libname adam cvp "&AInPath";
   %end;
   libname clinical "&CLINICAL_HOME";
   *** Replaces spaces in studname with underscores;
   %let studynameunder = %sysfunc(transtrn(&studname, %str( ), _));
   
   %put Metapath = &Metapath;

      %include "&MacroPath./UtilityPathMacros.sas" / nosource;
      %jg_getPath(/user/clinical/metadata,,3,0,0,&studynameunder);  
   
   %if %length(&InPath) %then %do;
      %MAKEDIR(dir=SDTM, varname = SDTMMETA, parent = %jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder));
   %end;
   %else %do;
      %global SDTMMETA;
      %let SDTMMETA = ;
   %end;
   %if %length(&AInPath) %then %do;
      %MAKEDIR(dir=ADAM, varname = ADAMMETA, parent = %jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder)); 
   %end;
   %else %do;
      %global ADAMMETA;
      %let ADAMMETA = ;
   %end;
   %MAKEDIR(dir=BASE, varname = BASEMETA, parent = %jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder)); 
   %if %symexist(STUDY_HOME)=0 %then %do;
      %global STUDY_HOME;
      %let STUDY_HOME = %jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder);
   %end;
   
   %CREATEMETADATA;
   
   %if &CSTERROR %then %goto mdexit;
   %if %length(&domaindups) %then %do;
      %let CSTERROR = 1;
      %put [MS_S_0203];
      %put ERROR: The following domains are duplicated in at least two datasets: &domaindups;
      %goto mdexit;
   %end;

   %IDENTIFYDOMAINS;
   
   %if %length(&InPath) %then %do;
      %MISSVARS(model = SDTM, dir = SDTMMETA); 
      %if &CSTERROR %then %goto mdexit;
   %end;
   %if %length(&AInPath) %then %do;
      %MISSVARS(model = ADAM, dir = ADAMMETA); 
      %if &CSTERROR %then %goto mdexit;
   %end;
   %METADATAOUTFILES;
   %if &exiterror %then %goto mdexit;
   %put CSTERROR = &CSTERROR;
   %mdexit:;   
%mend GENERATEMETA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro CHKSTUDYNAME;
   %let StudyName = %nrbquote(&StudyName);
   %let numlist = 0 1 2 3 4 5 6 7 8 9;
   %if %index(&numlist, %substr(&StudyName, 1, 1)) %then %do;
      %put [MS_S_0204];
      %put ERROR: Study Name must not begin with a number.;
      %let exiterror = 1;
   %end;

   %let symlist = %nrstr(< > { } [ ] * | ? / \ , : = ; ` ! $ ^);
   %do i = 1 %to 25;
      %if &i < 20 %then %let temp = %scan(&symlist,&i,' ');
      %else %if &i = 20 %then %let temp = %nrstr(&);
      %else %if &i = 21 %then %let temp = %nrstr(%%);
      %else %if &i = 22 %then %let temp = %nrstr(%');
      %else %if &i = 23 %then %let temp = %nrstr(%");
      %else %if &i = 24 %then %let temp = %nrstr(%();
      %else %if &i = 25 %then %let temp = %nrstr(%));
      %if %index(&StudyName, %nrbquote(&temp)) %then %do;
         %put [MS_S_0205];
         %put ERROR: Study Name must not contain the symbol &temp.;
         %let exiterror = 1;
      %end;
   %end;
   
   %if &StudyName = %nrstr(.) or &StudyName = %nrstr(..) %then %do;
      %put ERROR: Study Name cannot be . or .. .;
      %let exiterror = 1;
   %end;
%mend CHKSTUDYNAME;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

/* %include "&MacroPath./MergeSupp.sas" / nosource; */

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro KEYS(domlist = DM CO SE SV &findingsdomains &eventsdomains &interventionsdomains &adslpresent);

   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
    %let notesDataset =;
    %jg_getNotesDatasetName(notesDataset);
   %let domlist = %sysfunc(compbl(&domlist));
   %let i = 1;
   %let dflg = 0;
   
   %let trSortedby = %nrbquote(%LocaleGet(SortedbyNS_Text));
    %let notesDataset =;
    %jg_getNotesDatasetName(notesDataset);       
   
   %do %while(%length(%scan(&domlist,&i,' ')));
      %let temp = %scan(&domlist,&i,' ');
      %put getting keys for &temp;
      %if %sysfunc(findw(%upcase(&domains), &temp)) or &temp = ADSL %then %do;   
         %global &temp.KEYS &temp.KEYSMISS;
         
         %let &temp.KEYSMISS =; 
   
         proc contents data = &&in_&temp;
            ods exclude all;
            ods output sortedby = sortvars(where = (upcase(label1) = upcase("&trSortedby")));         
         run;
         ods select all;
         * Chinese study may not get sorted. RZ suggests to sort to minimize the number of duplications and notes;
         %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH %then %do;
            %if ^%sysfunc(exist(sortvars)) & %sysfunc(findw(%upcase(&findingsdomains), &temp)) %then %do;
               proc sort data = &&in_&temp out = sorted&temp;
                  by _Character_ %if %upcase(&temp)=VS & %sysfunc(findw(%upcase(&VSVARS.vars), VSSEQ)) %then VSSEQ ;;
               run;
               proc contents data = sorted&temp;
                  ods exclude all;
                  ods output sortedby = sortvars(where = (upcase(label1) = upcase("&trSortedby")));         
               run;
               ods select all;
            %end;
         %end;
         %let _tid_ = %sysfunc(open(&&in_&temp));
         %let tCRDate = %sysfunc(attrn(&_tid_,CRDTE));      
         %let _tid_ = %sysfunc(close(&_tid_));
         
         %if ^&dflg %then %do; 
            %let CRDate = &tCRDate;
         %end;
         %else %if &dflg %then %do;
            %if %sysevalf(&tCRDate > &CRDate) %then %let CRDate = &tCRDate;         
         %end;
         
         **** S0992169 for Chinese, data sortvars is generated,but no observations ****;
         %let nobsSortVars=0;
         %if %sysfunc(exist(sortvars)) %then %do;
            %NObsVars(sortvars); 
            %let nobsSortVars=&nobs;   
         %end;
         
         %if %sysfunc(exist(sortvars)) & &nobs>0 %then %do;
            data _null_;
               set sortvars;
               cvalue1 = compbl(transtrn(cvalue1, "DESCENDING", ""));
               call symput("&temp.KEYS", strip(cvalue1));
            run;

            proc datasets nolist;
               delete sortvars;
            quit;
         %end;
         %else %do;
            %let &temp.KEYSMISS =; 
            %if %sysfunc(findw(%upcase(&&&temp.vars), STUDYID)) %then %let _STUDYID = STUDYID;
            %else %do;
               %let _STUDYID =;
               %let &temp.KEYSMISS = &&&temp.KEYSMISS STUDYID; 
            %end;
            %if %sysfunc(findw(%upcase(&&&temp.vars), USUBJID)) %then %let _USUBJID = USUBJID;
            %else %do;
               %let _USUBJID =;
               %let &temp.KEYSMISS = &&&temp.KEYSMISS USUBJID; 
            %end;
            %if &temp = SV %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), VISITNUM)) %then %let _VISITNUM = VISITNUM;
               %else %do;
                  %let _VISITNUM =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS VISITNUM; 
               %end;
            %end;          
            %else %if &temp = CO %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), COSEQ)) %then %let _COSEQ = COSEQ;
               %else %do;
                  %let _COSEQ =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS COSEQ; 
               %end;
            %end;
            %else %if &temp = SE %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), SESTDTC)) %then %let _SESTDTC = SESTDTC;
               %else %do;
                  %let _SESTDTC =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS SESTDTC; 
               %end;
               %if %sysfunc(findw(%upcase(&&&temp.vars), ETCD)) %then %let _ETCD = ETCD;
               %else %do;
                  %let _ETCD =;              
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS ETCD; 
               %end;
            %end;
            %else %if %sysfunc(findw(%upcase(&interventionsdomains), &temp)) %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.STDTC)) %then %let _&temp.STDTC = &temp.STDTC;
               %else %do;
                  %let _&temp.STDTC =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.STDTC; 
               %end;
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.TRT)) %then %let _&temp.TRT = &temp.TRT;
               %else %do;
                  %let _&temp.TRT =;                                       
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.TRT; 
               %end;
            %end;
            %else %if %sysfunc(findw(%upcase(&eventsdomains), &temp)) %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.STDTC)) %then %let _&temp.STDTC = &temp.STDTC;
               %else %do;
                  %let _&temp.STDTC =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.STDTC; 
               %end;             
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.TERM)) %then %let _&temp.TERM = &temp.TERM;
               %else %do;
                  %let _&temp.TERM =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.TERM; 
               %end;
            %end;          
            %else %if %sysfunc(findw(%upcase(&findingsdomains), &temp)) and ^%sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.TESTCD)) %then %let _&temp.TESTCD = &temp.TESTCD;
               %else %do;
                  %let _&temp.TESTCD =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.TESTCD; 
               %end;
               %if &temp ne IE and &temp ne DA %then %do;   
                  %if %sysfunc(findw(%upcase(&&&temp.vars), VISITNUM)) %then %let _VISITNUM = VISITNUM;
                  %else %do;
                     %let _VISITNUM =;          
                     %let &temp.KEYSMISS = &&&temp.KEYSMISS VISITNUM; 
                  %end;
                  %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.TPTREF)) %then %let _&temp.TPTREF = &temp.TPTREF;
                  %else %do;
                     %let _&temp.TPTREF =;
                     %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.TPTREF; 
                  %end;
                  %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.TPTNUM)) %then %let _&temp.TPTNUM = &temp.TPTNUM;
                  %else %do;
                     %let _&temp.TPTNUM =;
                     %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.TPTNUM; 
                  %end;
               %end;
            %end;
            %else %if %sysfunc(findw(%upcase(&findingsdomains), &temp)) and %sysfunc(findw(%upcase(&bdsdomains), &temp)) %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), PARAMCD)) %then %let _&temp.TESTCD = PARAMCD;
               %else %do;
                  %let _&temp.TESTCD =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS PARAMCD; 
               %end;
               %if &temp ne IE and &temp ne DA %then %do;   
                  %if %sysfunc(findw(%upcase(&&&temp.vars), AVISITN)) %then %let _VISITNUM = AVISITN;
                  %else %do;
                     %let _VISITNUM =;          
                     %let &temp.KEYSMISS = &&&temp.KEYSMISS AVISITN; 
                  %end;
                  %let _&temp.TPTREF =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.TPTREF; 
                  %if %sysfunc(findw(%upcase(&&&temp.vars), ATPTN)) %then %let _&temp.TPTNUM = ATPTN;
                  %else %do;
                     %let _&temp.TPTNUM =;
                     %let &temp.KEYSMISS = &&&temp.KEYSMISS ATPTN; 
                  %end;
               %end;
            %end;          
            %if &temp = FA %then %do;
               %if %sysfunc(findw(%upcase(&FAVARS), FAOBJ)) %then %let _FAOBJ = FAOBJ;
               %else %do;
                  %let _FAOBJ =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS FAOBJ;
               %end;
            %end;
            %if &temp = LB %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.SPEC)) %then %let _&temp.SPEC = &temp.SPEC;
               %else %do;
                  %let _&temp.SPEC =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.SPEC;
               %end;
            %end;
            %if &temp = DA %then %do;
               %if %sysfunc(findw(%upcase(&FAVARS), DADTC)) %then %let _DADTC = DADTC;
               %else %do;
                  %let _DADTC =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS DADTC;
               %end;
            %end;
            %if &temp = QS or &temp = PP %then %do;
               %if %sysfunc(findw(%upcase(&&&temp.vars), &temp.CAT)) %then %let _&temp.CAT = &temp.CAT;
               %else %do;
                  %let _&temp.CAT =;
                  %let &temp.KEYSMISS = &&&temp.KEYSMISS &temp.CAT;
               %end;             
            %end; 
            
            %if &temp = DM %then %let DMKEYS = &_STUDYID &_USUBJID;
            %else %if &temp = CO %then %let COKEYS = &_STUDYID &_USUBJID &_COSEQ;
            %else %if &temp = SE %then %let SEKEYS = &_STUDYID &_USUBJID &_ETCD &_SESTDTC;
            %else %if &temp = SV %then %let SVKEYS = &_STUDYID &_USUBJID &_VISITNUM;
            %else %if %sysfunc(findw(%upcase(&interventionsdomains), &temp)) %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TRT &&_&temp.STDTC;                
            %else %if %sysfunc(findw(%upcase(&eventsdomains), &temp)) %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TERM &&_&temp.STDTC;
            %else %if &temp = ADSL %then %let ADSLKEYS = &_STUDYID &_USUBJID;
            %else %if %sysfunc(findw(%upcase(&findingsdomains), &temp)) %then %do;
               %if &temp = FA %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TESTCD &&_&temp.OBJ &_VISITNUM &&_&temp.TPTREF &&_&temp.TPTNUM;
               %else %if &temp = LB %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TESTCD &&_&temp.SPEC &_VISITNUM &&_&temp.TPTREF &&_&temp.TPTNUM;
               %else %if &temp = IE %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TESTCD;
               %else %if &temp = DA %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TESTCD &_DADTC;
               %else %if &temp = QS %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.CAT &&_&temp.TESTCD &_VISITNUM &&_&temp.TPTREF &&_&temp.TPTNUM;
               %else %if &temp = PP %then %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TESTCD &&_&temp.CAT &_VISITNUM &&_&temp.TPTREF &&_&temp.TPTNUM;
               %else %let &temp.KEYS = &_STUDYID &_USUBJID &&_&temp.TESTCD &_VISITNUM &&_&temp.TPTREF &&_&temp.TPTNUM;
            %end;          
            
            %let &temp.KEYS = %sysfunc(compbl(&&&temp.KEYS));
         %end; 

         data _null_;
            file "&basemeta&_slash_.CSTMacroVars.sas" mod;
            put "%nrstr(%global) &temp.KEYS;";
            put "%nrstr(%let) &temp.KEYS = &&&temp.KEYS;";
         run;
         %if ^&dflg %then %let dflg = 1;
      %end;
      %let i = %eval(&i+1);
   %end;

   data _null_;
      date = &CRDate;
      call symput("CRDateC", put(date, DATETIME19.));
      call symput("RDateC", put(datetime(), DATETIME19.));
   run;
   
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) CRDate CRDateC RDateC;";
      put "%nrstr(%let) CRDate = &CRDate;";
      put "%nrstr(%let) CRDateC = %nrstr(%nrbquote)(&CRDateC);";
      put "%nrstr(%let) RDateC = %nrstr(%nrbquote)(&RDateC);";
   run;
   
   %global USUBJIDLabel COUNTRYLabel SITEIDLabel USUBJIDlength SITEIDlength COUNTRYlength SITEID COUNTRY;
   %let _tid_ = %sysfunc(open(&in_adsl));
   %let vn = %sysfunc(varnum(&_tid_,USUBJID));
   %let vn2 = %sysfunc(varnum(&_tid_,SITEID));
   %let vn3 = %sysfunc(varnum(&_tid_,COUNTRY));
   %let USUBJIDlength = %sysfunc(varlength(&_tid_,&vn));
   %if &vn2 %then %let SITEIDlength = %sysfunc(varlength(&_tid_,&vn2));
   %else %let SITEIDlength = 0;
   %if &vn3 %then %let Countrylength = %sysfunc(varlength(&_tid_,&vn3));
   %else %let COUNTRYlength = 0;
   %let _tid_ = %sysfunc(close(&_tid_));
   %let USUBJIDLabel = USUBJID;
   %CheckLabel(&in_ADSL,USUBJIDLabel);
   %let SITEID = ;
   %let COUNTRY = ;
   %if &vn2 %then %do;
      %let SITEIDLabel = SITEID;
      %CheckLabel(&in_ADSL,SITEIDLabel);
      %let SITEID = SITEID;
   %end;
   %if &vn3 %then %do;
      %let COUNTRYLabel = COUNTRY;
      %CheckLabel(&in_ADSL,COUNTRYLabel);
      %let COUNTRY = COUNTRY;
   %end;
   *** get labels and lengths from DM in case not in ADSL ***;
   %if %upcase(&in_ADSL) ne ADAM.ADSL and %sysfunc(findw(%upcase(&domains), DM)) %then %do;
      %if ^&SITEIDlength or ^&COUNTRYlength %then %do;
         %let _tid_ = %sysfunc(open(&in_dm));
            %if ^&SITEIDlength %then %do;
               %let vn2 = %sysfunc(varnum(&_tid_,SITEID));
               %if &vn2 %then %let SITEIDlength = %sysfunc(varlength(&_tid_,&vn2));
               %else %let SITEIDlength = 0;
               %if &vn2 %then %do;
                  %let SITEIDLabel = SITEID;
                  %CheckLabel(&in_ADSL,SITEIDLabel);
                  %let SITEID = SITEID;
               %end;
            %end;
            %if ^&COUNTRYlength %then %do;
               %let vn3 = %sysfunc(varnum(&_tid_,COUNTRY));
               %if &vn3 %then %let COUNTRYlength = %sysfunc(varlength(&_tid_,&vn3));
               %else %let COUNTRYlength = 0;            
               %if &vn3 %then %do;
                  %let COUNTRYLabel = COUNTRY;
                  %CheckLabel(&in_ADSL,COUNTRYLabel);
                  %let COUNTRY = COUNTRY;
               %end;      
            %end;
         %let _tid_ = %sysfunc(close(&_tid_));
      %end;   
   %end;
   
   %if &ProcessName = AddStudyLocal %then %do;
      %let trProcessName = %nrbquote(%LocaleGet(&ProcessName._Label));
      %let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
      %let trSAtJC = %nrbquote(%LocaleGet(StudyAddedtoJMPClinical_Text));
      %let trMRDoIDC = %nrbquote(%LocaleGet(MostRecentDatetimeofInputDataCreation_Text));
      %let trSnapEnabled = %nrbquote(%LocaleGet(SnapShotComparisonisEnabled_Text));
      %let trSnapDisabled = %nrbquote(%LocaleGet(SnapShotComparisonisDisabled_Text));
      %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
      %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
      %let trReport = %nrbquote(%LocaleGet(Report_Text));
      %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
      %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
      %let trSnapshot = %nrbquote(%LocaleGet(Snapshot_Text));
      %let trType = %nrbquote(%LocaleGet(Type_Text));
      %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
      %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
      %let trUser = %nrbquote(%LocaleGet(User_Text));
      %let trNote = %nrbquote(%LocaleGet(Note_Text));
      %let trSDTMFolder = %nrbquote(%LocaleGet(SDTMorSENDFolder_Text));
      %let trADaMFolder = %nrbquote(%LocaleGet(ADaMFolder_Text));
      %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));
      
         data notes;
            length usubjid $ &USUBJIDlength 
               %if %length(&SiteID) %then %do; siteid $ &SITEIDlength %end; 
               %if %length(&country) %then %do; country $ &COUNTRYlength %end; 
               Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
            Study_Name = "&StudyName";
            Report = "&trProcessName";
            User = "&sysuserid";
            Date_of_Note = datetime();
            SnapshotNumber = 0;
            Type = "&trSnapshot";
            Note = "&trSAtJC: &RDateC";
            SystemGenerated = "Y";
            output;
            Note = "&trMRDoIDC: &CRDateC";
            output;
            %if &SnpComp =Yes %then %do;
               Note = "&trSnapEnabled";
            %end;
            %else %do;
               Note = "&trSnapDisabled";
            %end;
            output;
            %if %length(&InPath) %then %do;
               Note = "&trSDTMFolder: &InPath";
               output;
            %end;
            %if %length(&AInPath) %then %do;
               Note = "&trADaMFolder: &AInPath";
               output;
            %end;
            format Date_of_Note DATETIME19.;
            label usubjid = "&USUBJIDLabel" 
               %if %length(&SiteID) %then %do; siteid = "&SITEIDlabel" %end; 
               %if %length(&Country) %then %do; Country = "&Countrylabel" %end;  
               Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" 
               Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" 
               Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
         run;   
      %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;
         data NotesLib.&notesDataset;
            retain SnapshotNumber Type usubjid %if %length(&SiteID) %then %do; siteid %end; 
               %if %length(&Country) %then %do; Country %end; Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
            set notes;
            %if %length(&SiteID) %then %do; 
               label siteid = "&SiteIDLabel";
            %end; 
            %if %length(&Country) %then %do; 
               label country = "&CountryLabel";
            %end; 
         run;
      %end;
      %else %do;
         %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
            %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
            %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
            %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
            %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
            %else %let Ulength_ = 0;
            %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
            %else %let Slength_ = 0;         
            %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
            %else %let Clength_ = 0;         
            %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
            %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
            %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
         %let _tid_ = %sysfunc(close(&_tid_));

         data notes;
            retain SnapshotNumber Type usubjid %if &notessiteid or %length(&SiteID) %then %do; siteid %end; 
               Study_Name %if &notescountry or %length(&Country) %then %do; country %end; Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
            length usubjid $ &USUBJIDlength 
                   %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                   %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                   ;               
            set notes NotesLib.&notesDataset;
            %if &notessiteid or %length(&SiteID) %then %do; 
               label siteid = "&SiteIDLabel";
            %end;
            %if &notescountry or %length(&Country) %then %do;
               label country = "&CountryLabel";
            %end;
         run;
         
         proc sort data = notes out = NotesLib.&notesDataset;
            by SnapshotNumber Date_of_Note;
         run;
      %end;    
         
      proc datasets nolist;
         delete notes;
      quit;                      
   %end;
   %else %if &ProcessName = ManageStudies and &NewSnap = YES %then %do;
      %let trProcessName = %nrbquote(%LocaleGet(&ProcessName._Label));
      %let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
      %let trSnapshot = %nrbquote(%LocaleGet(Snapshot_Text));
      %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
      %let trUSAtJC = %nrbquote(%LocaleGet(UpdatedSnapshotAppliedtoJMPClinical_Text));
      %let trReviewCR = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(ReviewX_Text)), f1=%nrbquote(&CurrentReview)));
      %let trMRDoIDC = %nrbquote(%LocaleGet(MostRecentDatetimeofInputDataCreation_Text));
      %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
      %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
      %let trReport = %nrbquote(%LocaleGet(Report_Text));
      %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
      %let trType = %nrbquote(%LocaleGet(Type_Text));
      %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
      %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
      %let trUser = %nrbquote(%LocaleGet(User_Text));
      %let trNote = %nrbquote(%LocaleGet(Note_Text));
      %let trSDTMFolder = %nrbquote(%LocaleGet(SDTMorSENDFolder_Text));
      %let trADaMFolder = %nrbquote(%LocaleGet(ADaMFolder_Text));
      %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));
      
         data notes;
            length usubjid $ &USUBJIDlength %if %length(&SiteID) %then %do; siteid $ &SITEIDlength %end; %if %length(&Country) %then %do; Country $ &Countrylength %end; 
               Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
            Study_Name = "&StudyName";
            Report = "&trProcessName";
            User = "&sysuserid";
            Date_of_Note = datetime();
            SnapshotNumber = &CurrentReview;
            Type = "&trSnapshot";
            SystemGenerated = "Y";
            Note = "&trUSAtJC (&trReviewCR): &RDateC";
            output;
            Note = "&trMRDoIDC (&trReviewCR): &CRDateC";
            output;
            %if %length(&InPath) %then %do;
               Note = "&trSDTMFolder: &InPath";
               output;
            %end;
            %if %length(&AInPath) %then %do;
               Note = "&trADaMFolder: &AInPath";
               output;
            %end;            
            format Date_of_Note DATETIME19.;
            label usubjid = "&USUBJIDLabel" %if %length(&SiteID) %then %do; siteid = "&SITEIDlabel" %end; %if %length(&Country) %then %do; Country = "&Countrylabel" %end; 
               Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" Date_of_Note = "&trDateofNote" Type = "&trType" SnapshotNumber = "&trSnapshotNumber"
               Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
         run;   
         
      %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;
         data NotesLib.&notesDataset;
            retain SnapshotNumber Type usubjid %if %length(&SiteID) %then %do; siteid %end; %if %length(&Country) %then %do; country %end; 
               Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
            set notes;
            %if %length(&SiteID) %then %do; 
               label siteid = "&SiteIDLabel";
            %end;                   
            %if %length(&Country) %then %do; 
               label Country = "&CountryLabel";
            %end;                   
         run;
      %end;
      %else %do;
         %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
            %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
            %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
            %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
            %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
            %else %let Ulength_ = 0;
            %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
            %else %let Slength_ = 0;         
            %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
            %else %let Clength_ = 0;         
            %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
            %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
            %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
         %let _tid_ = %sysfunc(close(&_tid_));
      
         data notes;
            retain SnapshotNumber Type usubjid %if &notessiteid or %length(&SiteID) %then %do; siteid %end; %if &notescountry or %length(&country) %then %do; country %end; 
               Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
            length usubjid $ &USUBJIDlength 
                   %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                   %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                   ;               
            set notes NotesLib.&notesDataset;
            %if &notessiteid or %length(&SiteID) %then %do; 
               label siteid = "&SiteIDLabel";
            %end;
            %if &notescountry or %length(&country) %then %do; 
               label Country = "&CountryLabel";
            %end;    
         run;
         
         proc sort data = notes out = NotesLib.&notesDataset;
            by SnapshotNumber Date_of_Note;
         run;
      %end;
         
      proc datasets nolist;
         delete notes;
      quit;
   %end;
   %else %if &ProcessName = ManageStudies and &MigrateFlag = Y %then %do;
      %let trProcessName = %nrbquote(%LocaleGet(&ProcessName._Label));
      %let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
      %let trSMtNVoJC = %nrbquote(%LocaleGet(StudyMigratedtoNewVersionofJMPClinical_Text));
      %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
      %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
      %let trReport = %nrbquote(%LocaleGet(Report_Text));
      %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
      %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
      %let trSnapshot = %nrbquote(%LocaleGet(Snapshot_Text));
      %let trType = %nrbquote(%LocaleGet(Type_Text));
      %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
      %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
      %let trUser = %nrbquote(%LocaleGet(User_Text));
      %let trNote = %nrbquote(%LocaleGet(Note_Text));
      %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));
      
         data notes;
            length usubjid $ &USUBJIDlength %if %length(&SiteID) %then %do; siteid $ &SITEIDlength %end; %if %length(&Country) %then %do; Country $ &Countrylength %end; 
               Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
            Study_Name = "&StudyName";
            Report = "&trProcessName";
            User = "&sysuserid";
            SystemGenerated = "Y";
            Date_of_Note = datetime();
            SnapshotNumber = 0;
            Type = "&trSnapshot";
            Note = "&trSMtNVoJC: &RDateC";
            output;
            format Date_of_Note DATETIME19.;
            label usubjid = "&USUBJIDLabel" %if %length(&SiteID) %then %do; siteid = "&SITEIDlabel" %end; %if %length(&Country) %then %do; Country = "&Countrylabel" %end; 
               Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" Date_of_Note = "&trDateofNote" 
               SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
         run;   
         
      %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;
         data NotesLib.&notesDataset;
            retain SnapshotNumber Type usubjid %if %length(&SiteID) %then %do; siteid %end; %if %length(&country) %then %do; country %end; 
               Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
            set notes;
            %if %length(&SiteID) %then %do; 
               label siteid = "&SiteIDLabel";
            %end;
            %if %length(&Country) %then %do; 
               label Country = "&CountryLabel";
            %end;          
         run;
      %end;
      %else %do;
         %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
            %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
            %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
            %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
            %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
            %else %let Ulength_ = 0;
            %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
            %else %let Slength_ = 0;         
            %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
            %else %let Clength_ = 0;         
            %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
            %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
            %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
         %let _tid_ = %sysfunc(close(&_tid_));

         data notes;
            retain SnapshotNumber Type usubjid %if &notessiteid or %length(&SiteID) %then %do; siteid %end; %if &notescountry or %length(&country) %then %do; country %end; 
               Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
            length usubjid $ &USUBJIDlength 
                   %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                   %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                   ;               
            set notes NotesLib.&notesDataset;
            %if &notessiteid or %length(&SiteID) %then %do; 
               label siteid = "&SiteIDLabel";
            %end;
            %if &notescountry or %length(&country) %then %do;
               label Country = "&CountryLabel";
            %end;
         run;
         
         proc sort data = notes out = NotesLib.&notesDataset;
            by SnapshotNumber Date_of_Note;
         run;
      %end;
               
      proc datasets nolist;
         delete &notesDataset;
      quit;    
   %end;
   %exit:;
%mend KEYS; 

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro UPDATESUBJECTFLAGS(domlist = DM CO SE SV &findingsdomains &eventsdomains &interventionsdomains &adslpresent);
      
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
    %let notesDataset =;
    %jg_getNotesDatasetName(notesDataset);
   %let domlist = %sysfunc(compbl(&domlist));
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
   %let availdom = ;
   %let i = 1;
   
   %let trNew = %nrbquote(%LocaleGet(New_Text));
   %let trModified = %nrbquote(%LocaleGet(Modified_Text));
   %let trStable = %nrbquote(%LocaleGet(Stable_Text));
   %let trDropped = %nrbquote(%LocaleGet(Dropped_Text));
   %let trNonUnique = %nrbquote(%LocaleGet(NonUnique_Text));
   %let trIntroduced = %nrbquote(%LocaleGet(Introduced_Text));
   
   *** identify subjects with new or modified data ***; 
   %do %while(%length(%scan(&domlist,&i,' ')));
      %let temp = %scan(&domlist,&i,' ');
         %if %sysfunc(findw(%upcase(&domains), &temp)) or &temp = ADSL %then %do;
            %let availdom = &availdom &temp;
            
            proc sort data = DeltaLib.&temp(keep = usubjid &temp.RFlg) out = &temp(rename = (&temp.RFlg = Rflg));
               by usubjid &temp.RFlg;
            run;

            data &temp;
               set &temp(where = (RFlg in ("&trNew", "&trModified")));
               by usubjid RFlg;
               /*%if &temp = DM or &temp = ADSL %then %do;
                  if RFlg = "&trNew" then delete;
               %end; */
               if last.usubjid;
            run;                                
         %end;
      %let i = %eval(&i+1);
   %end;
   
   data alldom;
      set &availdom;
   run;
   
   proc datasets nolist;
      delete &availdom;
   quit;
   
   proc sort data = alldom;
      by usubjid RFlg;
   run;
   
   data alldom;
      set alldom;
      by usubjid RFlg;
      if last.usubjid;
   run;
   
   proc sort data = NotesLib.adslrs out = adslrs;
      by usubjid;
   run;
   
   data _null_;
      set NotesLib.adslrsl(obs=1);
      call symput('defaultrs',trim(left(RSTATUS)));
   run;  
   
   data adslrs(drop = RFlg);
      merge adslrs(in = ina) alldom;
      by usubjid;
      if ina;
      if ReviewFlag ne "Introduced" & ReviewFlag ne "&trIntroduced" then do;
         if RFlg ne "" then do;
            ReviewFlag = catx(" ", RFlg, "Records");
            RSTATUS = "&defaultrs";
            DateUpdated = put(datetime(),E8601DT19.);
            User = "&sysuserid";
         end;
         else if RFlg = "" then ReviewFlag = "&trStable";
      end;
   run;
   
   %let trNewRecords = %nrbquote(%LocaleGet(NewRecords_Text));
   %let trManageStudies = %nrbquote(%LocaleGet(ManageStudies_Text));
   %let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
   %let trSubjHasNewRecs = %nrbquote(%LocaleGet(Subjecthasnewrecords_Text));
   %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
   %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
   %let trReport = %nrbquote(%LocaleGet(Report_Text));
   %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
   %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
   %let trType = %nrbquote(%LocaleGet(Type_Text));
   %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
   %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
   %let trUser = %nrbquote(%LocaleGet(User_Text));
   %let trNote = %nrbquote(%LocaleGet(Note_Text));
   %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));
   
   data snotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated);
      length Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
      set adslrs(where = (ReviewFlag = "&trNewRecords"));
      Study_Name = "&StudyName";
      Report = "&trManageStudies";
      SystemGenerated = "Y";
      Domain = "";
      Keys = "";
      SnapshotNumber = &CurrentReview;
      Type = "&trNewRecords";
      Keys_Values = "";
      User = "&sysuserid";
      Note = "&trSubjHasNewRecs";
      Date_of_Note = datetime();             
      format Date_of_Note DATETIME19.;
      label usubjid = "&USUBJIDLabel" Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" 
         Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
   run;

   %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;
      data NotesLib.&notesDataset;
         retain SnapshotNumber Type usubjid %if %length(&SiteID) %then %do; siteid %end; %if %length(&country) %then %do; country %end; Study_Name Keys Keys_Values 
            Domain Report User Note Date_of_Note SystemGenerated;
         set snotes;
         %if %length(&SiteID) %then %do; 
            label siteid = "&SiteIDLabel";
         %end;                
         %if %length(&country) %then %do; 
            label country = "&countryLabel";
         %end;                
      run;
   %end;
   %else %do;
      %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
         %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
         %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
         %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
         %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
         %else %let Ulength_ = 0;
         %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
         %else %let Slength_ = 0;         
         %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
         %else %let Clength_ = 0;         
         %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
         %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
         %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
      %let _tid_ = %sysfunc(close(&_tid_));

      data snotes;
         retain SnapshotNumber Type usubjid %if %length(&SiteID) or &notessiteid %then %do; siteid %end; %if %length(&country) or &notescountry %then %do; country %end; 
            Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
         length usubjid $ &USUBJIDlength 
                %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                ;            
         set snotes NotesLib.&notesDataset;
         %if &notessiteid or %length(&SiteID) %then %do; 
            label siteid = "&SiteIDLabel";
         %end;
         %if %length(&country) or &notescountry %then %do;
            label country = "&countryLabel";
         %end;
      run;

      proc sort data = snotes out = NotesLib.&notesDataset;
         by SnapshotNumber Date_of_Note;
      run;  
   %end;    
         
   proc datasets nolist;
      delete alldom;
   quit;
   
   *** Check if Dropped ***;
   proc sort data = &in_adsl out = adsl(keep = usubjid);
      by usubjid;
   run;
   
   data NotesLib.adslrs;
      retain USUBJID RSTATUS ReviewFlag DateAdded;
      merge adslrs(in = ina) adsl(in = inb);
      by usubjid;
      if ina;
      if ina and not inb then ReviewFlag = "&trDropped";
   run;     
   
   proc datasets nolist;
      delete adsl;
   quit;
   
%mend UPDATESUBJECTFLAGS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro COMPARE(domlist = DM CO SE SV &findingsdomains &eventsdomains &interventionsdomains &adslpresent);

   %let domlist = %sysfunc(compbl(&domlist));
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
   %let notesDataset =;
   %jg_getNotesDatasetName(notesDataset);
   %let mbd = %jg_getPath(/user/clinical/metadata/base, DeltaFile&CurrentReview..sas, 0, 0, 3, &studynameunder);
   %include "&mbd" / nosource;
   
   %let trNew = %nrbquote(%LocaleGet(New_Text));
   %let trModified = %nrbquote(%LocaleGet(Modified_Text));
   %let trStable = %nrbquote(%LocaleGet(Stable_Text));
   %let trDropped = %nrbquote(%LocaleGet(Dropped_Text));
   %let trNonUnique = %nrbquote(%LocaleGet(NonUnique_Text));
   %let trYes = %nrbquote(%LocaleGet(Yes_Text));
   %let trPrevSnapFreq = %nrbquote(%LocaleGet(PreviousSnapshotFrequency_Text));
   %let trNewSnapFreq = %nrbquote(%LocaleGet(NewSnapshotFrequency_Text));
   %let trManageStudies = %nrbquote(%LocaleGet(ManageStudies_Text));
   %let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
   %let trMissing = %nrbquote(%LocaleGet(Missing_Text));
   %let trVciXfXtX = %nrbquote(%LocaleGet(ValuechangeinXfromXtoX_Text));
   %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
   %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
   %let trReport = %nrbquote(%LocaleGet(Report_Text));
   %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
   %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
   %let trType = %nrbquote(%LocaleGet(Type_Text));
   %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
   %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
   %let trUser = %nrbquote(%LocaleGet(User_Text));
   %let trNote = %nrbquote(%LocaleGet(Note_Text));
   %let trDuplicate = %nrbquote(%LocaleGet(Duplicate_Text));
   %let trDeftpk = %nrbquote(%LocaleGet(Duplicatesexistforthisparticularkey_Text));
   %let trprevious = %nrbquote(%LocaleGet(previous_Text));
   %let trtimes = %nrbquote(%LocaleGet(times_Text));
   %let trlcnew = %nrbquote(%LocaleGet(new_Text));
   %let trNCPBS = %nrbquote(%LocaleGet(Nocomparisonsperformedbetweensnapshots_Text));
   %let trVariable = %nrbquote(%LocaleGet(Variable_Text));
   %let trLabel = %nrbquote(%LocaleGet(Label_Text));
   %let trTPKWAIPDBNND = %nrbquote(%LocaleGet(ThisparticularkeywasAvailinPrevdatabutnotnewdata_Text));
   %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));

   %let trRepX1 = %nrbquote(%LocaleGet(XColonDiffsinVarsBetwNewSnapAndPrevForModKeys_Text));
   %let trRepX2 = %nrbquote(%LocaleGet(XColonKeysWithDupsNoCompWereMadeifXNonUniq_Text));
   %let trRepX3 = %nrbquote(%LocaleGet(XColonDroppedKeysAvailInPrevDataButNotNewSnap_Text));
   %let trRepX4 = %nrbquote(%LocaleGet(NoteTheXdomainisnewlyavailable_Text));
   
   *** Check for Directories ***;
   %jg_getPath(/user/clinical/delta,,3,0,0,&studynameunder);
   *** Delete Previous if Available ***;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder))) ;    
   %end;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder))) ;   
   %end;
   %RemoveDir(%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder));

   %jg_getPath(/user/clinical/delta,,3,0,0,&studynameunder);  
   libname DeltaLib "%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder)";

   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "libname DeltaLib %nrstr(%')%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder)%nrstr(%');";
      put "%nrstr(%let) DeltaDir = %jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder);";
   run;  

   options orientation = landscape nodate missing = "" sysprint="";
   ods escapechar = "^";
   ods listing close;
   ods pdf file = "&ServerOutPath.&StudyName SnapShotDiffs.pdf" style = journal;
   %let ADSLpre = ;
   %let ADSLpost = ;
   
   %let i = 1;
   %let first = 0;
   
   %let trXReviewFlag = %nrbquote(%LocaleGet(XReviewFlag_Text));
   
   %do %while(%length(%scan(&domlist,&i,' ')));
      %let temp = %scan(&domlist,&i,' ');
      %let trxrf = %nrbquote(%LocaleFill(%nrbquote(&trXReviewFlag), f1=%nrbquote(&temp)));
      %let predupcnt = 0;
      %let postdupcnt = 0;
      %if %sysfunc(findw(%upcase(&domains), &temp)) or &temp = ADSL %then %do;
         %if &temp = ADSL %then %do;
            %if %scan(&in_adsl,2,'.') = DM %then %let ADSLpost = DM;
            %if %symexist(prev_adsl) %then %do;
               %if %scan(&prev_adsl,2,'.') = DM %then %let ADSLpre = DM;
            %end;
         %end;
         
         %if %symexist(in_&temp) and %symexist(prev_&temp) and ^%length(&ADSLpre) and ^%length(&ADSLpost) %then %do;

            %let lib1 = %scan(&&in_&temp, 1, ".");
            %let lib0 = %scan(&&prev_&temp, 1, ".");
            
            proc sort data = &&in_&temp out = post;
               by &&&temp.keys;
            run;

            proc sort data = &&prev_&temp out = pre;
               by &&&temp.keys;
            run;           

            /*
            *** Merge in SUPP info if available for BOTH and SDTM ***;
            %if %upcase(&lib1) = SDTM and %upcase(&lib0) = PREVSDTM and %sysfunc(exist(SDTM.%upcase(SUPP&temp))) and %sysfunc(exist(PREVSDTM.%upcase(SUPP&temp))) %then %do;

                  %let IDVAR1 =; 
                  %let IDVARVAL1 =;
                  %let IDVARTYPE1 =;
                  %let IDVAR0 =; 
                  %let IDVARVAL0 =;
                  %let IDVARTYPE0 =;
                  %let postfail = 0;
                  %let prefail = 0;

                  %let _tid_ = %sysfunc(open(&lib1..SUPP&temp));
                  %if %sysfunc(varnum(&_tid_,IDVAR1)) %then %let IDVAR1 = IDVAR;
                  %if %sysfunc(varnum(&_tid_,IDVARVAL1)) %then %let IDVARVAL1 = IDVARVAL;
                  %let _tid_ = %sysfunc(close(&_tid_));

                  %if %length(&IDVAR1) %then %do;
                     %if %nrbquote(&SYSENCODING) eq %nrbquote(euc-cn) %then %do;
                        proc freq noprint data = &lib1..SUPP&temp(encoding=ANY);
                           tables &IDVAR1) / missing out = temp1(rename = (&IDVAR1 = valuec));
                        run;
                     %end;
                     %else %do;
                        proc freq noprint data = &lib1..SUPP&temp;
                           tables &IDVAR1 / missing out = temp1(rename = (&IDVAR1 = valuec));
                        run;
                     %end;

                     data _null_;
                        set temp1;
                        if missing(valuec) and round(percent) = 100 then do;
                           call symput("IDVAR1", "");
                        end;
                     run;

                     proc datasets nolist;
                        delete temp1;
                     quit; 
                  %end;

                  %if %length(&IDVARVAL1) %then %do;
                     %if %nrbquote(&SYSENCODING) eq %nrbquote(euc-cn) %then %do;
                        proc freq noprint data = &lib1..SUPP&temp(encoding=ANY);
                           tables &IDVARVAL1) / missing out = temp2(rename = (&IDVARVAL1 = valuec));
                        run;
                     %end;
                     %else %do;
                        proc freq noprint data = &lib1..SUPP&temp;
                           tables &IDVARVAL1 / missing out = temp2(rename = (&IDVARVAL1 = valuec));
                        run;
                     %end;

                     data _null_;
                        set temp2;
                        if missing(valuec) and round(percent) = 100 then do;
                           call symput("IDVARVAL1", "");
                        end;
                     run;

                     proc datasets nolist;
                        delete temp2;
                     quit;
                  %end;

                  %let _tid_ = %sysfunc(open(&lib0..SUPP&temp));
                  %if %sysfunc(varnum(&_tid_,IDVAR0)) %then %let IDVAR0 = IDVAR;
                  %if %sysfunc(varnum(&_tid_,IDVARVAL0)) %then %let IDVARVAL0 = IDVARVAL;
                  %let _tid_ = %sysfunc(close(&_tid_));

                  %if %length(&IDVAR0) %then %do;
                     %if %nrbquote(&SYSENCODING) eq %nrbquote(euc-cn) %then %do;
                        proc freq noprint data = &lib0..SUPP&temp(encoding=ANY);
                           tables &IDVAR0) / missing out = temp1(rename = (&IDVAR0 = valuec));
                        run;
                     %end;
                     %else %do;
                        proc freq noprint data = &lib0..SUPP&temp;
                           tables &IDVAR0 / missing out = temp1(rename = (&IDVAR0 = valuec));
                        run;
                     %end;

                     data _null_;
                        set temp1;
                        if missing(valuec) and round(percent) = 100 then do;
                           call symput("IDVAR0", "");
                        end;
                     run;

                     proc datasets nolist;
                        delete temp1;
                     quit; 
                  %end;

                  %if %length(&IDVARVAL0) %then %do;
                     %if %nrbquote(&SYSENCODING) eq %nrbquote(euc-cn) %then %do;
                        proc freq noprint data = &lib0..SUPP&temp(encoding=ANY);
                           tables &IDVARVAL0) / missing out = temp2(rename = (&IDVARVAL0 = valuec));
                        run;
                     %end;
                     %else %do;
                        proc freq noprint data = &lib0..SUPP&temp;
                           tables &IDVARVAL0 / missing out = temp2(rename = (&IDVARVAL0 = valuec));
                        run;
                     %end;

                     data _null_;
                        set temp2;
                        if missing(valuec) and round(percent) = 100 then do;
                           call symput("IDVARVAL0", "");
                        end;
                     run;

                     proc datasets nolist;
                        delete temp2;
                     quit;
                  %end;

                  *** only if both variables are present or absent ***;
                  %if (%length(&IDVAR0) and %length(&IDVARVAL0) and %length(&IDVAR1) and %length(&IDVARVAL1)) or 
                      (^%length(&IDVAR0) and ^%length(&IDVARVAL0) and ^%length(&IDVAR1) and ^%length(&IDVARVAL1)) %then %do;
                     %let IDVARACTUAL1 =;                         
                     %if %length(&IDVAR1) %then %do;
                        data _null_;
                           set &lib1..supp&temp(obs=1 keep = IDVAR1 where = (IDVAR1 ne ""));
                           if length(IDVAR1) then call symput("IDVARACTUAL1",IDVAR1);
                        run;
                     %end;             
                     %if %length(&IDVARACTUAL1) %then %do;
                        %let _tid_ = %sysfunc(open(&lib1..&temp));
                        %let _vn_ = %sysfunc(varnum(&_tid_,&IDVARACTUAL1));
                        %let IDVARTYPE1 = %sysfunc(vartype(&_tid_,&_vn_)); 
                        %let _tid_ = %sysfunc(close(&_tid_));  
                     %end;

                     %let IDVARACTUAL0 =;                         
                     %if %length(&IDVAR0) %then %do;
                        data _null_;
                           set &lib0..supp&temp(obs=1 keep = IDVAR0 where = (IDVAR0 ne ""));
                           if length(IDVAR0) then call symput("IDVARACTUAL0",IDVAR0);
                        run;
                     %end;             
                     %if %length(&IDVARACTUAL0) %then %do;
                        %let _tid_ = %sysfunc(open(&lib0..&temp));
                        %let _vn_ = %sysfunc(varnum(&_tid_,&IDVARACTUAL0));
                        %let IDVARTYPE0 = %sysfunc(vartype(&_tid_,&_vn_)); 
                        %let _tid_ = %sysfunc(close(&_tid_));  
                     %end;                            

                     *** divert errors on merge with supp info ***;
                     %if ^&first %then %do; 
                        proc printto log = "&OutPath.SUPPMergeComp.log" new;
                        run;
                     %end;
                     %else %do;
                        proc printto log = "&OutPath.SUPPMergeComp.log";
                        run;           
                     %end;
                     *** Removes duplicate qnams based on STUDYID, DOMAIN, USUBJID, IDVAR, IDVARVAL;   
                     proc sql noprint;
                        create table postsupp&temp(rename = (rdomain = DOMAIN) where = (IDVAR1 = "&IDVARACTUAL1")) as
                           select *, FREQ(qnam) AS numqnam
                           from &lib1..supp&temp
                           %if (%length(&IDVAR1) and %length(&IDVARVAL1)) %then %do;
                              group by studyid, rdomain, usubjid, idvar1, idvarval1, qnam
                           %end;
                           %else %do;
                              group by studyid, rdomain, usubjid, qnam
                           %end;
                           having numqnam = 1
                           %if (%length(&IDVAR1) and %length(&IDVARVAL1)) %then %do;
                              order by studyid, rdomain, usubjid, idvar1, idvarval1, qnam;
                           %end;
                           %else %do;
                              order by studyid, rdomain, usubjid, qnam;
                           %end;
                     quit;

                     proc sql noprint;
                        create table presupp&temp(rename = (rdomain = DOMAIN) where = (IDVAR0 = "&IDVARACTUAL0")) as
                           select *, FREQ(qnam) AS numqnam
                           from &lib0..supp&temp
                           %if (%length(&IDVAR0) and %length(&IDVARVAL0)) %then %do;
                              group by studyid, rdomain, usubjid, idvar0, idvarval0, qnam
                           %end;
                           %else %do;
                              group by studyid, rdomain, usubjid, qnam
                           %end;
                           having numqnam = 1
                           %if (%length(&IDVAR0) and %length(&IDVARVAL0)) %then %do;
                              order by studyid, rdomain, usubjid, idvar0, idvarval0, qnam;
                           %end;
                           %else %do;
                              order by studyid, rdomain, usubjid, qnam;
                           %end;
                     quit;                         

                     *** This works if IDVAR and IDVARVAL are NULL;
                     %if (%length(&IDVAR1) and %length(&IDVARVAL1)) %then %do; 
                        proc transpose data = postsupp&temp out = postsupp&temp(drop = _name_ _label_ idvar1 idvarval1);
                           by studyid domain usubjid idvar1 idvarval1 qnam;
                           var idvarval1;
                           id idvar1;
                           copy qval qlabel;
                        run;
                     %end;

                     %if (%length(&IDVAR0) and %length(&IDVARVAL0)) %then %do; 
                        proc transpose data = presupp&temp out = presupp&temp(drop = _name_ _label_ idvar0 idvarval0);
                           by studyid domain usubjid idvar0 idvarval0 qnam;
                           var idvarval0;
                           id idvar0;
                           copy qval qlabel;
                        run;
                     %end;

                     proc transpose data = postsupp&temp out = postsupp&temp(drop = _name_ _label_);
                        %if %length(&IDVARACTUAL1) %then %do;
                           by studyid domain usubjid &IDVARACTUAL1;
                        %end;
                        %else %do;
                           by studyid domain usubjid;
                        %end;
                        var qval;
                        id qnam;
                        idlabel qlabel;
                     run;

                     proc transpose data = presupp&temp out = presupp&temp(drop = _name_ _label_);
                        %if %length(&IDVARACTUAL0) %then %do;
                           by studyid domain usubjid &IDVARACTUAL0;
                        %end;
                        %else %do;
                           by studyid domain usubjid;
                        %end;
                        var qval;
                        id qnam;
                        idlabel qlabel;
                     run;

                     %if &idvartype1 = N %then %do;
                        data postsupp&temp(rename = (N&IDVARACTUAL1 = &IDVARACTUAL1));
                           set postsupp&temp;
                           N&IDVARACTUAL1 = &IDVARACTUAL1 + 0;
                           drop &IDVARACTUAL1;
                        run;
                     %end;

                     %if &idvartype0 = N %then %do;
                        data presupp&temp(rename = (N&IDVARACTUAL0 = &IDVARACTUAL0));
                           set presupp&temp;
                           N&IDVARACTUAL0 = &IDVARACTUAL0 + 0;
                           drop &IDVARACTUAL0;
                        run;
                     %end;

                     proc sql noprint;
                        create table post as
                        select * from &lib1..&temp as aa left join postsupp&temp(drop = domain) as bb
                        %if %length(&IDVARACTUAL1) %then %do;
                           on aa.studyid = bb.studyid and aa.usubjid = bb.usubjid and aa.&IDVARACTUAL1 = bb.&IDVARACTUAL1
                           order by %sysfunc(transtrn(&&&temp.keys, %str( ), %str(,)));
                        %end;
                        %else %do;
                           on aa.studyid = bb.studyid and aa.usubjid = bb.usubjid
                           order by %sysfunc(transtrn(&&&temp.keys, %str( ), %str(,)));
                        %end; 
                     quit;
                     %if %eval(&syserr > 4) %then %do;
                        %let postfail = 1;
                     %end;                         

                     proc sql noprint;
                        create table pre as
                        select * from &lib0..&temp as aa left join presupp&temp(drop = domain) as bb
                        %if %length(&IDVARACTUAL0) %then %do;
                           on aa.studyid = bb.studyid and aa.usubjid = bb.usubjid and aa.&IDVARACTUAL0 = bb.&IDVARACTUAL0
                           order by %sysfunc(transtrn(&&&temp.keys, %str( ), %str(,)));
                        %end;
                        %else %do;
                           on aa.studyid = bb.studyid and aa.usubjid = bb.usubjid
                           order by %sysfunc(transtrn(&&&temp.keys, %str( ), %str(,)));
                        %end; 
                     quit;
                     %if %eval(&syserr > 4) %then %do;
                        %let prefail = 1;
                     %end;                         

                     %if &postfail or &prefail %then %do;
                        proc sort data = &&in_&temp out = post;
                           by &&&temp.keys;
                        run;

                        proc sort data = &&prev_&temp out = pre;
                           by &&&temp.keys;
                        run;
                     %end;
                     %let first = 1;
                  %end;            
            %end;
            */
            %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH %then %do;
               proc sort data =pre force;
                  by &&&temp.keys;
               run;
               proc sort data =post force;
                  by &&&temp.keys;
               run;
            %end;
            *** Remove Duplicates ***;
            data predup pre;
               set pre;
               by &&&temp.keys;
               if first.%scan(&&&temp.keys, -1, ' ') ne 1 or last.%scan(&&&temp.keys, -1, ' ') ne 1 then output predup;
               else output pre;
            run;

            *** Get Count of Previous Duplicates ***;
            data predup;
               set predup;
               by &&&temp.keys;
               retain PreCount;
               if first.%scan(&&&temp.keys, -1, ' ') then PreCount = 1;
               else PreCount = PreCount + 1;
               if last.%scan(&&&temp.keys, -1, ' ');
               keep &&&temp.keys PreCount;
               label PreCount = "&trPrevSnapFreq";
            run;
            %Nobsvars(predup);
            %let predupcnt = &nobs;

            proc transpose data = pre out = prec(rename = (col1 = prec) drop=col2);
               by &&&temp.keys;
               var _character_;
            run;

            proc transpose data = pre out = pren(rename = (col1 = pren) drop=col2);
               by &&&temp.keys;
               var _numeric_;
            run;

            *** Unique Set of Previous Keys (Excluding Dups) ***;
            data prekeys;
               set pre;
               keep &&&temp.keys;
            run;

            *** Previous Data ***;
            data pre;
               set prec(in = ina) pren(in = inb);
               if ina then Ptype = "C";
               else if inb then Ptype = "N";
               label prec = "Previous Character Values" pren = "Previous Numeric Values" _name_ = "Variable Name" 
                  _label_ = "Variable Label" Ptype = "Previous Type";
            run;

            proc datasets nolist;
               delete prec pren;
            quit;

            proc sort data = pre;
               by &&&temp.keys _name_;
            run;

            *** Remove Duplicates ***;
            data postdup post;
               set post;
               by &&&temp.keys;
               if first.%scan(&&&temp.keys, -1, ' ') ne 1 or last.%scan(&&&temp.keys, -1, ' ') ne 1 then output postdup;
               else output post;
            run;

            *** Get Count of Post Duplicates ***;
            data postdup;
               set postdup;
               by &&&temp.keys;
               retain PostCount;
               if first.%scan(&&&temp.keys, -1, ' ') then PostCount = 1;
               else PostCount = PostCount + 1;
               if last.%scan(&&&temp.keys, -1, ' ');
               keep &&&temp.keys PostCount;
               label PostCount = "&trNewSnapFreq";
            run;
            %Nobsvars(postdup);
            %let postdupcnt = &nobs;            

            proc transpose data = post out = postc(rename = (col1 = postc) drop=col2);
               by &&&temp.keys;
               var _character_;
            run;

            proc transpose data = post out = postn(rename = (col1 = postn) drop=col2);
               by &&&temp.keys;
               var _numeric_;
            run;

            *** Unique Set of Post Keys (Excluding Dups) ***;
            data postkeys;
               set post;
               keep &&&temp.keys;
            run;

            *** Post Data ***;
            data post;
               set postc(in = ina) postn(in = inb);
               if ina then Ntype = "C";
               else if inb then Ntype = "N";
               label postc = "New Character Values" postn = "New Numeric Values" _name_ = "Variable Name" 
                   _label_ = "Variable Label" Ntype = "New Type";
            run;

            proc datasets nolist;
               delete postc postn;
            quit;

            proc sort data = post;
               by &&&temp.keys _name_;
            run;

            %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %do;
            * In Chinese, the column length for prekeys and postkeys are different, then create problem S1010544;
               %let tempkeyflag=0;
                  %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(prekeys));
                  %let PreLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                  %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end; 
               %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(postkeys));
                  %let PostLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end;
               %let tempLen=;
               %let _i_=1; 
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %if &&tempKeysType&_i_ = C %then %do;
                     %if %length(&&PreLen&_i_) & %length(&&PostLen&_i_) & &&PreLen&_i_ < &&PostLen&_i_ %then %do;
                        %let tempLen=&tempLen  %scan(&&&temp.keys,&_i_);
                        %let tempLen= &tempLen $ &&PostLen&_i_;
                        %let tempkeyflag=1;
                     %end;
                  %end;
                  %let _i_= %eval(&_i_+1);
               %end;
               data keys;
                  length &temp.RFlg $ 16 %if &tempkeyflag %then &tempLen;;
                     merge prekeys(in = ina) postkeys(in = inb);
                  by &&&temp.keys;
                  if ina and not inb then &temp.RFlg = "&trDropped";
                  else if inb and not ina then &temp.RFlg = "&trNew"; 
                  label &temp.RFlg = "&trxrf";
               run;
            %end;
            %else %do;
               *** Identify Keys that are New or Dropped (Excluding Dups)***;
               data keys;
                  length &temp.RFlg $ 16;
                     merge prekeys(in = ina) postkeys(in = inb);
                  by &&&temp.keys;
                  if ina and not inb then &temp.RFlg = "&trDropped";
                  else if inb and not ina then &temp.RFlg = "&trNew"; 
                  label &temp.RFlg = "&trxrf";
               run;
            %end;
            %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %do;
            * In Chinese, the column length for prepost and keys are different, then create problem S1010544;
               %let tempkeyflg=0;
                  %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(pre));
                  %if %sysfunc(varnum(&ds,&tempkeyScan)) %then %do;
                     %let PreLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                     %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %end;
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end; 
               %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(post));
                  %if %sysfunc(varnum(&ds,&tempkeyScan)) %then %do;
                     %let PostLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                     %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %end;
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end;
               %let tempLen=;
               %let _i_=1; 
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %if &&tempKeysType&_i_ = C %then %do;
                     %if %symexist(PreLen&_i_) & %symexist(PostLen&_i_) %then %do;
                        %if %length(&&PreLen&_i_) & %length(&&PostLen&_i_) & &&PreLen&_i_ < &&PostLen&_i_ %then %do;
                           %let tempLen=&tempLen  %scan(&&&temp.keys,&_i_);
                           %let tempLen= &tempLen $ &&PostLen&_i_;
                           %let tempkeyflg=1;
                        %end;
                     %end;
                  %end;
                  %let _i_= %eval(&_i_+1);
               %end;
               data prepost;
                  %if &tempkeyflg %then %do;
                     length &tempLen;
                  %end;
                  merge pre(in = ina) post(in = inb);
                  by &&&temp.keys _name_;
                  if (ntype = "C" and prec ne postc) or (ntype = "N" and pren ne postn);
                  %if &ExclTrt = YES %then %do;
                     if _name_ in ("ARM", "ARMCD", "ACTARM", "ACTARMCD") then delete;
                     %if &temp = EX %then %do;
                        if _name_ = "EXTRT" then delete;
                     %end;
                     %if &temp = ADSL %then %do;
                        if substr(_name_,1,3) = "TRT" and substr(_name_,6,1) in ("A","P") then delete;
                        if substr(_name_,1,6) = "TRTSEQ" then delete;
                        if substr(_name_,1,2) = "TR" and substr(_name_,5,2) in ("AG","PG") then delete;
                     %end;                               
                  %end;
                  %if &ExclSpid = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.SPID)) %then %do;
                        if _name_ = "&temp.SPID" then delete;
                     %end;                                       
                  %end;
                  %if &ExclSeq = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.SEQ)) %then %do;
                        if _name_ = "&temp.SEQ" then delete;
                     %end;                                       
                  %end;                  
                  %if &ExclDom = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), DOMAIN)) %then %do;
                        if _name_ = "DOMAIN" then delete;
                     %end;                                       
                  %end;                  
                  %if &ExclGrp = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.GRPID)) %then %do;
                        if _name_ = "&temp.GRPID" then delete;
                     %end;                                       
                  %end;
                  %if &ExclRef = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.REFID)) %then %do;
                        if _name_ = "&temp.REFID" then delete;
                     %end;                                       
                  %end;                                    
                  %if &ExclLnk = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.LNKID)) %then %do;
                        if _name_ = "&temp.LNKID" then delete;
                     %end;                                       
                  %end;
                  %if &ExclLnkg = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.LNKGRP)) %then %do;
                        if _name_ = "&temp.LNKGRP" then delete;
                     %end;                                       
                  %end;                  
                  
               run;
            %end;
            %else %do;
               *** Identify Any Differences Between Variables ***;
               data prepost;
                  merge pre(in = ina) post(in = inb);
                  by &&&temp.keys _name_;
                  if (ntype = "C" and prec ne postc) or (ntype = "N" and pren ne postn);
                  %if &ExclTrt = YES %then %do;
                     if _name_ in ("ARM", "ARMCD", "ACTARM", "ACTARMCD") then delete;
                     %if &temp = EX %then %do;
                        if _name_ = "EXTRT" then delete;
                     %end;
                     %if &temp = ADSL %then %do;
                        if substr(_name_,1,3) = "TRT" and substr(_name_,6,1) in ("A","P") then delete;
                        if substr(_name_,1,6) = "TRTSEQ" then delete;
                        if substr(_name_,1,2) = "TR" and substr(_name_,5,2) in ("AG","PG") then delete;
                     %end;                               
                  %end;
                  %if &ExclSpid = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.SPID)) %then %do;
                        if _name_ = "&temp.SPID" then delete;
                     %end;                                       
                  %end;
                  %if &ExclSeq = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.SEQ)) %then %do;
                        if _name_ = "&temp.SEQ" then delete;
                     %end;                                       
                  %end;                  
                  %if &ExclDom = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), DOMAIN)) %then %do;
                        if _name_ = "DOMAIN" then delete;
                     %end;                                       
                  %end;                  
                  %if &ExclGrp = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.GRPID)) %then %do;
                        if _name_ = "&temp.GRPID" then delete;
                     %end;                                       
                  %end;
                  %if &ExclRef = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.REFID)) %then %do;
                        if _name_ = "&temp.REFID" then delete;
                     %end;                                       
                  %end;                                    
                  %if &ExclLnk = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.LNKID)) %then %do;
                        if _name_ = "&temp.LNKID" then delete;
                     %end;                                       
                  %end;
                  %if &ExclLnkg = YES %then %do;
                     %if ^%sysfunc(findw(%upcase(&&&temp.keys), &temp.LNKGRP)) %then %do;
                        if _name_ = "&temp.LNKGRP" then delete;
                     %end;                                       
                  %end;                                    
               run;
            %end;

            *** Identify Keys with Differences Between Variables ***;
            data ppkeys;
               set prepost;
               by &&&temp.keys;
               if first.%scan(&&&temp.keys, -1, ' ');
               keep &&&temp.keys;
            run;

            *** Combine Duplicates to get Pre-Post Counts ***;
            %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %do;
               %let tempkeyflg=0;
               %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(predup));
                  %if %sysfunc(varnum(&ds,&tempkeyScan)) %then %do;
                     %let PreLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                     %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %end;
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end; 
               %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(postdup));
                  %if %sysfunc(varnum(&ds,&tempkeyScan)) %then %do;
                     %let PostLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                     %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %end;
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end;
               %let tempLen=;
               %let _i_=1; 
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %if &&tempKeysType&_i_ = C %then %do;
                     %if %symexist(PreLen&_i_) & %symexist(PostLen&_i_) %then %do;
                        %if %length(&&PreLen&_i_) & %length(&&PostLen&_i_) & &&PreLen&_i_ < &&PostLen&_i_ %then %do;
                           %let tempLen=&tempLen  %scan(&&&temp.keys,&_i_);
                           %let tempLen= &tempLen $ &&PostLen&_i_;
                           %let tempkeyflg=1;
                        %end;
                     %end;
                  %end;
                  %let _i_= %eval(&_i_+1);
               %end;
               data dupkeys;
                  %if &tempkeyflg %then %do;
                     length &tempLen;
                  %end;
                  merge predup postdup;
                  by &&&temp.keys;
                  if PreCount = . then PreCount = 0;
                  if PostCount = . then PostCount = 0;
               run;
            %end;
            %else %do;
               data dupkeys;
                  merge predup postdup;
                  by &&&temp.keys;
                  if PreCount = . then PreCount = 0;
                  if PostCount = . then PostCount = 0;
               run;
            %end;

            *** Identify Any Differences Between Variables (Excluding Dups) ***;
            data keys;
               merge keys(in = ina) ppkeys(in = inb);
               by &&&temp.keys;
               if inb and &temp.RFlg = "" then &temp.RFlg = "&trModified";
               else if not inb and &temp.RFlg = "" then &temp.RFlg = "&trStable";
            run;

            *** Define Duplicate Type ***;
            data keys;
               retain Domain &&&temp.keys PreCount PostCount &temp.RFlg;
               length domain $ 8 &temp.RFlg $ 16;
               merge keys(in = ina) dupkeys(in = inb);
               by &&&temp.keys;
               Domain = "&temp";
               if inb then do;
                 if PreCount = 0 then &temp.RFlg = "&trNew";
                 else if PostCount = 0 then &temp.RFlg = "&trDropped";
                 else &temp.RFlg = "&trNonUnique";
               end;
               %if ^&predupcnt %then %do;
                  drop PreCount;
               %end;
               %if ^&postdupcnt %then %do;
                  drop PostCount;
               %end;
            run;

            proc datasets nolist;
               delete postkeys post postdup prekeys pre predup ppkeys dupkeys;
            quit;
         %end;
         %else %if (&temp ne ADSL and ^%symexist(prev_&temp)) or (&temp = ADSL and %length(&ADSLpre) and ^%length(&ADSLpost)) or 
            (&temp = ADSL and ^%length(&ADSLpre) and ^%length(&ADSLpost)) %then %do;
         
            %if &temp = ADSL %then %do;         
               data _null_;
                  file "&basemeta&_slash_.CSTMacroVars.sas" mod;
                  put "%nrstr(%global) ADSLNewFlag;";
                  put "%nrstr(%let) ADSLNewFlag = &trYes;";
               run;
            %end;
         
            proc sort data = &&in_&temp out = post;
               by &&&temp.keys;
            run;

            *** Remove Duplicates ***;
            data postdup post;
               set post;
               by &&&temp.keys;
               if first.%scan(&&&temp.keys, -1, ' ') ne 1 or last.%scan(&&&temp.keys, -1, ' ') ne 1 then output postdup;
               else output post;
            run;

            *** Get Count of Post Duplicates ***;
            data postdup;
               set postdup;
               by &&&temp.keys;
               retain PostCount;
               if first.%scan(&&&temp.keys, -1, ' ') then PostCount = 1;
               else PostCount = PostCount + 1;
               if last.%scan(&&&temp.keys, -1, ' ');
               keep &&&temp.keys PostCount;
               label PostCount = "&trNewSnapFreq";
            run;
            %Nobsvars(postdup);
            %let postdupcnt = &nobs;                  

            *** Unique Set of Post Keys (Excluding Dups) ***;
            data postkeys;
               set post;
               keep &&&temp.keys;
            run;

            *** Identify Keys that are New (Excluding Dups)***;
            data keys;
               length &temp.RFlg $ 16;
                  set postkeys;
               &temp.RFlg = "&trNew"; 
               label &temp.RFlg = "&trxrf";
            run;

            *** Define Duplicate Type ***;
            data keys;
               retain Domain &&&temp.keys PostCount &temp.RFlg;
               length domain $ 8 &temp.RFlg $ 16;
               merge keys(in = ina) postdup(in = inb);
               by &&&temp.keys;
               Domain = "&temp";
               if inb then &temp.RFlg = "&trNew";
               %if ^&postdupcnt %then %do;
                  drop PostCount;
               %end;
            run;

            proc datasets nolist;
               delete postkeys post postdup;
            quit;
         %end;

         %if (&temp ne ADSL) or (&temp = ADSL and %length(&ADSLpre) and ^%length(&ADSLpost)) or (&temp = ADSL and ^%length(&ADSLpre) and ^%length(&ADSLpost)) %then %do;
            *** Save Keys ***;         
            data DeltaLib.&temp;
               set keys;
            run;

            %let rschk = %sysfunc(open(keys));
            %let prechk = %sysfunc(varnum(&rschk,PreCount));
            %let postchk = %sysfunc(varnum(&rschk,PostCount));
            %let rschk = %sysfunc(close(&rschk));
            %let PreCount =;
            %let PostCount =;
            %if &PreChk %then %let PreCount = PreCount; 
            %if &PostChk %then %let PostCount = PostCount; 

            data dropped;
               set keys(where = (&temp.RFlg = "&trDropped"));
            run; 
            %Nobsvars(dropped);

            %if &nobs %then %do;
               %jg_getPath(/user/clinical/delta/dropped,,3,0,0,&studynameunder);  
               libname Dropped "%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder)";

               proc sort data = &&prev_&temp out = pre;
                  by &&&temp.keys;
               run;

               data Dropped.&temp;
                  merge dropped(in = ina keep = &&&temp.keys) pre;
                  by &&&temp.keys;
                  if ina;
               run;

               proc datasets nolist;
                  delete pre;
               quit;
            %end;

            proc datasets nolist;
               delete dropped;
            quit;
         %end;

         %if %symexist(in_&temp) and %symexist(prev_&temp) and ^%length(&ADSLpre) and ^%length(&ADSLpost) %then %do;
            %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %do;
            * In Chinese, the column length for prepost and keys are different, then create problem S1010544;
               %let tempkeyflg=0;
                  %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(prepost));
                  %if %sysfunc(varnum(&ds,&tempkeyScan)) %then %do;
                     %let PreLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                     %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %end;
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end; 
               %let _i_=1;             
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %let tempkeyScan = %scan(&&&temp.keys,&_i_);
                  %let ds = %sysfunc(open(keys));
                  %if %sysfunc(varnum(&ds,&tempkeyScan)) %then %do;
                     %let PostLen&_i_ = %sysfunc(varlen(&ds,%sysfunc(varnum(&ds,&tempkeyScan))));
                     %let tempKeysType&_i_=%sysfunc(vartype(&ds,&_i_));
                  %end;
                  %let rc = %sysfunc(close(&ds)); 
                  %let _i_ = %eval(&_i_+1);
               %end;
               %let tempLen=;
               %let _i_=1; 
               %do %while (%length(%scan(&&&temp.keys,&_i_)));
                  %if &&tempKeysType&_i_ = C %then %do;
                     %if %symexist(PreLen&_i_) & %symexist(PostLen&_i_) %then %do;
                        %if %length(&&PreLen&_i_) & %length(&&PostLen&_i_) & &&PreLen&_i_ < &&PostLen&_i_ %then %do;
                           %let tempLen=&tempLen  %scan(&&&temp.keys,&_i_);
                           %let tempLen= &tempLen $ &&PostLen&_i_;
                           %let tempkeyflg=1;
                        %end;
                     %end;
                  %end;
                  %let _i_= %eval(&_i_+1);
               %end;
               data prepost;
                  %if &tempkeyflg %then %do;
                  length &tempLen;
                  %end;
                  merge prepost keys(keep = &&&temp.keys &temp.RFlg);
                  by &&&temp.keys;
               run;
            %end;
            %else %do;
               data prepost;
                  merge prepost keys(keep = &&&temp.keys &temp.RFlg);
                  by &&&temp.keys;
               run;
            %end;

            %let _tid_ = %sysfunc(open(prepost));
            %let vn = %sysfunc(varnum(&_tid_,_label_));
            %let _label_len = %sysfunc(varlength(&_tid_,&vn));
            %let _tid_ = %sysfunc(close(&_tid_));

            %let trXDiffsVarsNPMK = %nrbquote(%LocaleFill(%nrbquote(&trRepX1), f1=%nrbquote(&temp)));
            title "&trXDiffsVarsNPMK";
            proc print width = min noobs label data = prepost 
               style(bylabel) = [fontsize = 7pt] style(header) = [fontsize = 7pt] style(obs) = [font_size = 7pt] style(obsheader) = [font_size = 7pt];
               var &&&temp.keys _name_ %if %length(&_label_len) %then _label_; prec postc pren postn / style(data) = [font_size = 7pt] style(header) = [background=white font_size = 7pt];                           
               where &temp.RFlg = "&trModified";
            run;           

            %let trXKWDupsNCXNU = %nrbquote(%LocaleFill(%nrbquote(&trRepX2), f1=%nrbquote(&temp), f2=%nrbquote(&temp.RFlg)));
            title "&trXKWDupsNCXNU";
            proc print width = min noobs label data = keys 
               style(bylabel) = [font_size = 7pt] style(obs) = [font_size = 7pt] style(obsheader) = [font_size = 7pt];
               var &&&temp.keys &temp.RFlg &PreCount &PostCount / style(data) = [font_size = 7pt] style(header) = [background=white font_size = 7pt];          
               where index(&temp.RFlg, "&trNonUnique")
               %if %length(&PostCount) %then %do;
                  or &PostCount > .
               %end;          
               %if %length(&PreCount) %then %do;
                  or &PreCount > .
               %end;          
               ;
            run;           

            %let trXDroppedKeys = %nrbquote(%LocaleFill(%nrbquote(&trRepX3), f1=%nrbquote(&temp)));
            title "&trXDroppedKeys";
            proc print width = min noobs label data = keys 
               style(bylabel) = [font_size = 7pt] style(obs) = [font_size = 7pt] style(obsheader) = [font_size = 7pt];
               var &&&temp.keys &temp.RFlg / style(data) = [font_size = 7pt] style(header) = [background=white font_size = 7pt];          
               where index(&temp.RFlg, "&trDropped");
            run;

            *** Save Notes of Modifications ***;
            
            data mnotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated &&&temp.keys _name_ _label_);
               length Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
               set prepost(where = (&temp.RFlg = "&trModified"));
               Study_Name = "&StudyName";
               Report = "&trManageStudies";
               Domain = "&temp";
               Keys = "&&&temp.keys";
               SnapshotNumber = &CurrentReview;
               Type = "&trModified";
               Keys_Values = catx("~", %sysfunc(transtrn(%str(&&&temp.keys), %str( ), %str(,))));   
               User = "&sysuserid";               
               if prec = "" then prec = "&trMissing";
               if postc = "" then postc = "&trMissing";
               call symput('flcn', catx(" ", _label_, cats("(",_name_,")")));
               if ntype = "C" then do;
                  call symput('fprec', prec);
                  call symput('fpostc', postc);
                  Note = resolve('%LocaleFill(&trVciXfXtX, &flcn, &fprec, &fpostc)');
               end;
               else if ntype = "N" and pren = . and postn > . then do;
                  call symput('flpostn', left(postn));
                  Note = resolve('%LocaleFill(&trVciXfXtX, &flcn, &trMissing, &flpostn)');
               end;
               else if ntype = "N" and postn = . and pren > . then do;
                  call symput('flp', left(pren));
                  Note = resolve('%LocaleFill(&trVciXfXtX, &flcn, &flp, &trMissing)'); 
               end;
               else if ntype = "N" and postn > . and pren > . then do;
                  call symput('flp', left(pren));
                  call symput('flpostn', left(postn));
                  Note = resolve('%LocaleFill(&trVciXfXtX, &flcn, &flp, &flpostn)');
               end;
               SystemGenerated = "Y";
               Date_of_Note = datetime();             
               format Date_of_Note DATETIME19.;
               label usubjid = "&USUBJIDLabel" Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" 
                  Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
            run;
            %Nobsvars(mnotes);
            
            %if &nobs %then %do;
               %jg_getPath(/user/clinical/delta/modified,,3,0,0,&studynameunder);
               libname modified "%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder)";
               data modified.&temp;
                  set mnotes(keep = usubjid study_name &&&temp.keys _name_ _label_ rename = (_name_ = Variable _label_ = Label));
                  label variable = "&trVariable";
                  label label = "&trLabel";
               run;
            %end;
            
            data dnotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated);
               length Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
               set keys(where = (index(&temp.RFlg,"&trNonUnique") 
                  %if %length(&PostCount) %then %do;
                     or &PostCount > .
                  %end;          
                  %if %length(&PreCount) %then %do;
                     or &PreCount > .
                  %end;          
               ));
               Study_Name = "&StudyName";
               Report = "&trManageStudies";
               Domain = "&temp";
               Keys = "&&&temp.keys";
               SnapshotNumber = &CurrentReview;
               Type = "&trDuplicate";
               %if ^%length(&PreCount) %then %do;
                  PreCount = 0;
               %end;
               %if ^%length(&PostCount) %then %do;
                  PostCount = 0;
               %end;                             
               Keys_Values = catx("~", %sysfunc(transtrn(%str(&&&temp.keys), %str( ), %str(,))));   
               User = "&sysuserid";
               SystemGenerated = "Y";
               if PreCount = 0 or PostCount = 0 then Note = catx(" ", "&trDeftpk [&trprevious:",left(PreCount),"&trtimes, &trlcnew:", cats(left(PostCount), "]."));
               else Note = catx(" ", "&trDeftpk [&trprevious:",left(PreCount),"&trtimes, &trlcnew:", cats(left(PostCount), "]."), "&trNCPBS");
               Date_of_Note = datetime();             
               format Date_of_Note DATETIME19.;
               label usubjid = "&USUBJIDLabel" Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" 
                  Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
            run;

            data drnotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated);
               length Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
               set keys(where = (&temp.RFlg = "&trDropped"));
               Study_Name = "&StudyName";
               Report = "&trManageStudies";
               Domain = "&temp";
               Keys = "&&&temp.keys";
               SnapshotNumber = &CurrentReview;
               Type = "&trDropped";
               Keys_Values = catx("~", %sysfunc(transtrn(%str(&&&temp.keys), %str( ), %str(,))));   
               User = "&sysuserid";
               Note = "&trTPKWAIPDBNND";
               SystemGenerated = "Y";
               Date_of_Note = datetime();             
               format Date_of_Note DATETIME19.;
               label usubjid = "&USUBJIDLabel" Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" 
                  Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
            run;
            
            %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;
               data NotesLib.&notesDataset;
                  retain SnapshotNumber Type usubjid %if %length(&Siteid) %then %do; siteid %end; %if %length(&country) %then %do; country %end; Study_Name Keys Keys_Values 
                     Domain Report User Note Date_of_Note SystemGenerated;
                  set mnotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated) dnotes drnotes;
                  %if %length(&Siteid) %then %do; 
                     label siteid = "&SiteIDLabel";
                  %end;                
                  %if %length(&country) %then %do; 
                     label country = "&countryLabel";
                  %end;                
               run;           
            %end;
            %else %do;
               %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
                  %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
                  %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
                  %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
                  %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
                  %else %let Ulength_ = 0;
                  %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
                  %else %let Slength_ = 0;         
                  %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
                  %else %let Clength_ = 0;         
                  %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
                  %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
                  %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
               %let _tid_ = %sysfunc(close(&_tid_));
               
               data mnotes;
                  retain SnapshotNumber Type usubjid %if %length(&Siteid) or %length(&notessiteid) %then %do; siteid %end; %if %length(&country) or %length(&notescountry) %then %do; country %end; 
                     Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
                  length usubjid $ &USUBJIDlength 
                         %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                         %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                         ;                     
                  set mnotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated) dnotes drnotes NotesLib.&notesDataset;
                  %if %length(&notessiteid) or %length(&SiteID) %then %do; 
                     label siteid = "&SiteIDLabel";
                  %end;
                  %if %length(&country) or %length(&notescountry) %then %do;
                     label country = "&countryLabel";
                  %end;
               run;

               proc sort data = mnotes out = NotesLib.&notesDataset;
                  by SnapshotNumber Date_of_Note;
               run;           
            %end;
            
            proc datasets nolist;
               delete mnotes dnotes drnotes;
            quit;
         %end;
         %else %if (&temp ne ADSL and ^%symexist(prev_&temp)) or (&temp = ADSL and %length(&ADSLpre) and ^%length(&ADSLpost)) %then %do;
            %let trNXDomNewAv = %nrbquote(%LocaleFill(%nrbquote(&trRepX4), f1=%nrbquote(&temp)));
            title;
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
            ods pdf text = "^S={font=('Arial') just=center} &trNXDomNewAv";
            ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial')}";           
         %end;
         proc datasets nolist;
            delete keys pre post prepost;
         quit;
      %end;
      %let i = %eval(&i+1);
   %end; 
   ods pdf close;
   ods listing;   
   
   %if ^%index(&Task,Rename) %then %do;   
      %UPDATESUBJECTFLAGS;
   %end;
%exitcomp:;
%mend COMPARE;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro COMPAREADD(domlist = DM CO SE SV &findingsdomains &eventsdomains &interventionsdomains &adslpresent);

   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
    %let notesDataset =;
    %jg_getNotesDatasetName(notesDataset);
   %let domlist = %sysfunc(compbl(&domlist));
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
   
   %let trNew = %nrbquote(%LocaleGet(New_Text));
   %let trModified = %nrbquote(%LocaleGet(Modified_Text));
   %let trStable = %nrbquote(%LocaleGet(Stable_Text));
   %let trDropped = %nrbquote(%LocaleGet(Dropped_Text));
   %let trNonUnique = %nrbquote(%LocaleGet(NonUnique_Text));

   *** Check for Directories ***;
   %jg_getPath(/user/clinical/delta,,3,0,0,&studynameunder);
   *** Delete Previous if Available ***;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder))) ;    
   %end;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder))) ;   
   %end; 
   %RemoveDir(%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder)); 
   libname DeltaLib "%jg_getPath(/user/clinical/delta,,3,0,3,&studynameunder)";

   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "libname DeltaLib %nrstr(%')%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder)%nrstr(%');";
      put "%nrstr(%let) DeltaDir = %jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder);";
   run;  
   
   %let firstflag = 0;
   
   options orientation = landscape nodate missing = "" sysprint="";
   ods escapechar = "^";
   ods listing close;
   ods pdf file = "&ServerOutPath.&StudyName SnapShotDups.pdf" style = journal;
   
   %let trNewSnapFreq = %nrbquote(%LocaleGet(NewSnapshotFrequency_Text));
   %let trDKRforStudy = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DuplicateandKeysReportforX_Text)), f1=%nrbquote(&StudyName)));
   %let trDKR1 = %nrbquote(%LocaleGet(DupKeysReportIntroMsgPart1_Text));
   %let trDKR2 = %nrbquote(%LocaleGet(DupKeysReportIntroMsgPart2_Text));
   %let trDKR3 = %nrbquote(%LocaleGet(DupKeysReportIntroMsgPart3_Text));
   %let trDKR4 = %nrbquote(%LocaleGet(DupKeysReportIntroMsgPart4_Text));
   %let trDKR5 = %nrbquote(%LocaleGet(DupKeysReportIntroMsgPart5_Text));
   %let trDKR6 = %nrbquote(%LocaleGet(DupKeysReportIntroMsgPart6_Text));
   %let trDKRIntro = &trDKR1 &trDKR2 &trDKR3 &trDKR4 &trDKR5 &trDKR6;
   %let trManageStudies = %nrbquote(%LocaleGet(ManageStudies_Text));
   %let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
   %let trDuplicate = %nrbquote(%LocaleGet(Duplicate_Text));
   %let trDeftpk = %nrbquote(%LocaleGet(Duplicatesexistforthisparticularkey_Text));
   %let trprevious = %nrbquote(%LocaleGet(previous_Text));
   %let trtimes = %nrbquote(%LocaleGet(times_Text));
   %let trlcnew = %nrbquote(%LocaleGet(new_Text));
   %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
   %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
   %let trReport = %nrbquote(%LocaleGet(Report_Text));
   %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
   %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
   %let trType = %nrbquote(%LocaleGet(Type_Text));
   %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
   %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
   %let trUser = %nrbquote(%LocaleGet(User_Text));
   %let trNote = %nrbquote(%LocaleGet(Note_Text));
   %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));
   
   %let trRep10 = %nrbquote(%LocaleGet(NoteKeysfortheXdomainColon_Text));
   %let trRep11 = %nrbquote(%LocaleGet(NoteKeysSuggAndUnavailForXDomainColon_Text));
   %let trRep12 = %nrbquote(%LocaleGet(XColonKeysWithDupsNoCompWereMadeifXNonUniq_Text));
   %let trRep13 = %nrbquote(%LocaleGet(NoteTherearenoduplicaterecordsfortheXdomain_Text));
   
   %let i = 1;
   
   %let trXReviewFlag = %nrbquote(%LocaleGet(XReviewFlag_Text));
   
   %*if &SnpComp =Yes %then %do;
   %do %while(%length(%scan(&domlist,&i,' ')));
      %let temp = %scan(&domlist,&i,' ');
      %let trxrf = %nrbquote(%LocaleFill(%nrbquote(&trXReviewFlag), f1=%nrbquote(&temp)));
      %let predupcnt = 0;
      %let postdupcnt = 0;
      %if %sysfunc(findw(%upcase(&domains), &temp)) or &temp = ADSL %then %do;
         
         %if (&temp ne ADSL) or (&temp = ADSL and %length(&adslpresent)) %then %do;
            proc sort data = &&in_&temp out = post;
               by &&&temp.keys;
            run;
            
            %if %upcase(%substr(%sysfunc(getoption(locale)),1,2)) = ZH %then %do;
               proc sort data =post force;
                  by &&&temp.keys;
               run;
            %end;
            *** Remove Duplicates ***;
            data postdup post;
               set post;
               by &&&temp.keys;
               if first.%scan(&&&temp.keys, -1, ' ') ne 1 or last.%scan(&&&temp.keys, -1, ' ') ne 1 then output postdup;
               else output post;
            run;

            *** Get Count of Post Duplicates ***;
            data postdup;
               set postdup;
               by &&&temp.keys;
               retain PostCount;
               if first.%scan(&&&temp.keys, -1, ' ') then PostCount = 1;
               else PostCount = PostCount + 1;
               if last.%scan(&&&temp.keys, -1, ' ');
               keep &&&temp.keys PostCount;
               label PostCount = "&trNewSnapFreq";
            run;
            %Nobsvars(postdup);
            %let postdupcnt = &nobs;
            %*put the number of duplicates=&postdupcnt;
            * if there are too many duplicates and snpshot comparison is checked, an error message will threw;
            %if &postdupcnt > 2000 %then %do;
               %global snpCompDup;
               %let snpCompDup=&postdupcnt;
            %end;
            *** Unique Set of Post Keys (Excluding Dups) ***;
            data postkeys;
               set post;
               keep &&&temp.keys;
            run;

            *** Identify Keys that are New (Excluding Dups)***;
            data keys;
               length &temp.RFlg $ 16;
                  set postkeys;
               &temp.RFlg = "&trNew"; 
               label &temp.RFlg = "&trxrf";
            run;

            *** Define Duplicate Type ***;
            data keys;
               retain Domain &&&temp.keys PostCount &temp.RFlg;
               length domain $ 8 &temp.RFlg $ 16;
               merge keys(in = ina) postdup(in = inb);
               by &&&temp.keys;
               Domain = "&temp";
               if inb then &temp.RFlg = "&trNew";
               %if ^&postdupcnt %then %do;
                  drop PostCount;
               %end;
            run;

            proc datasets nolist;
               delete postkeys post postdup;
            quit;

            *** Save Keys ***;         
            data DeltaLib.&temp;
               set keys;
            run;

            %let rschk = %sysfunc(open(keys));
            %let prechk = %sysfunc(varnum(&rschk,PreCount));
            %let postchk = %sysfunc(varnum(&rschk,PostCount));
            %let rschk = %sysfunc(close(&rschk));
            %let PreCount =;
            %let PostCount =;
            %if &PreChk %then %let PreCount = PreCount; 
            %if &PostChk %then %let PostCount = PostCount;
            

            %if ^&firstflag %then %do;
               title;
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
               ods pdf text = "^S={font=('Arial') just=center} &trDKRforStudy";
               ods pdf text = "^S={font=('Arial')}";
               %LocaleWrap(%nrbquote(&trDKRIntro), _width=65, _lineprefix=%nrbquote(^S={font=('Arial') just=center} ), _format=pdf);
               ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial')}";  
               %let firstflag = 1;
            %end;

            %let trNKFXD = %nrbquote(%LocaleFill(%nrbquote(&trRep10), f1=%nrbquote(&temp)));
            %let trNKSAUFXD = %nrbquote(%LocaleFill(%nrbquote(&trRep11), f1=%nrbquote(&temp)));
            %let trXKWDupsNCXNU = %nrbquote(%LocaleFill(%nrbquote(&trRep12), f1=%nrbquote(&temp), f2=%nrbquote(&temp.RFlg)));
            title;
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
            ods pdf text = "^S={font=('Arial') just=center} &trNKFXD";
            ods pdf text = "^S={font=('Arial') just=center} &&&temp.keys";
            %if %length(&&&temp.keysmiss) %then %do;           
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial') just=center} &trNKSAUFXD";
               ods pdf text = "^S={font=('Arial') just=center} &&&temp.keysmiss";            
            %end;
            ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
            ods pdf text = "^S={font=('Arial')}";
            ods pdf text = "^S={font=('Arial')}";              

            title "&trXKWDupsNCXNU";
            proc print width = min noobs label data = keys 
               style(bylabel) = [font_size = 7pt] style(obs) = [font_size = 7pt] style(obsheader) = [font_size = 7pt];
               var &&&temp.keys &temp.RFlg &PreCount &PostCount / style(data) = [font_size = 7pt] style(header) = [background=white font_size = 7pt];
               where index(&temp.RFlg, "&trNonUnique") 
               %if %length(&PostCount) %then %do;
                  or &PostCount > .
               %end;
               ;
            run;           
            
            data dnotes(keep = SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated);
               length Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
               set keys(where = (index(&temp.RFlg,"&trNonUnique")
                  %if %length(&PostCount) %then %do;
                     or &PostCount > .
                  %end;          
                  %if %length(&PreCount) %then %do;
                     or &PreCount > .
                  %end;                                        
               ));
               Study_Name = "&StudyName";
               Report = "&trManageStudies";
               Domain = "&temp";
               Keys = "&&&temp.keys";
               SnapshotNumber = 0;
               Type = "&trDuplicate";
               Keys_Values = catx("~", %sysfunc(transtrn(%str(&&&temp.keys), %str( ), %str(,))));   
               User = "&sysuserid";
               SystemGenerated = "Y";
               Note = catx(" ", "&trDeftpk [&trlcnew:", left(PostCount), "&trtimes].");            
               Date_of_Note = datetime();             
               format Date_of_Note DATETIME19.;
               label usubjid = "&USUBJIDLabel" Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" 
                  Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote" SystemGenerated = "&trSystemGenerated";
            run;
            %NObsVars(dnotes);
            
            %if ^&nobs %then %do;
               %let trNNdupsXdom = %nrbquote(%LocaleFill(%nrbquote(&trRep13), f1=%nrbquote(&temp)));
               title;
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
               ods pdf text = "^S={font=('Arial') just=center} &trNNdupsXdom";
               ods pdf text = "^S={font=('Arial') just=center} ----------------------------------------------------------------------";
               ods pdf text = "^S={font=('Arial')}";
               ods pdf text = "^S={font=('Arial')}";              
            %end;

            %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;           
               data NotesLib.&notesDataset;
                  retain SnapshotNumber Type usubjid %if %length(&Siteid) %then %do; siteid %end; %if %length(&country) %then %do; country %end; Study_Name Keys Keys_Values 
                     Domain Report User Note Date_of_Note SystemGenerated;
                  set dnotes;
                  %if %length(&SiteID) %then %do; 
                     label siteid = "&SiteIDLabel";
                  %end;                         
                  %if %length(&country) %then %do; 
                     label country = "&countryLabel";
                  %end;                         
               run;
            %end;
            %else %do;  
               %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
                  %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
                  %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
                  %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
                  %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
                  %else %let Ulength_ = 0;
                  %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
                  %else %let Slength_ = 0;         
                  %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
                  %else %let Clength_ = 0;         
                  %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
                  %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
                  %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
               %let _tid_ = %sysfunc(close(&_tid_));
            
               data dnotes;
                  retain SnapshotNumber Type usubjid %if %length(&Siteid) or &notessiteid %then %do; siteid %end; %if %length(&country) or &notescountry %then %do; country %end; 
                     Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
                  length usubjid $ &USUBJIDlength 
                         %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                         %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                         ;                     
                  set dnotes NotesLib.&notesDataset;
                  %if &notessiteid or %length(&SiteID) %then %do; 
                     label siteid = "&SiteIDLabel";
                  %end;
                  %if %length(&country) or &notescountry %then %do; 
                     label country = "&countryLabel";
                  %end;
               run;

               proc sort data = dnotes out = NotesLib.&notesDataset;
                  by SnapshotNumber Date_of_Note;
               run;

               proc datasets nolist;
                  delete dnotes ;
               quit;      
            %end;
         %end;

         proc datasets nolist;
            delete keys post;
         quit;
      %end;
      %let i = %eval(&i+1);
   %end; 
   ods pdf close;
   ods listing;
/* %end;
   %else %do;
      %do %while(%length(%scan(&domlist,&i,' ')));
         %let temp = %scan(&domlist,&i,' ');
         %if %sysfunc(findw(%upcase(&domains), &temp)) or &temp = ADSL %then %do;
            data DeltaLib.&temp;
               set &&in_&temp;
            run;
         %end;
         %let i = %eval(&i+1);
      %end;
   %end;*/
   
%exitcomp:;
%mend COMPAREADD;

%macro DOMAINLISTADD;
   proc sort data=AllDomains;
      by table;
   run;
   data AllnoUsedDom;
      set AllnoUsedDom;
      rename Name = table;
   run;
   proc sort data=AllnoUsedDom;
      by table;
   run;
   data AllnoUsedDom2;
      length table $8.;
      merge AllDomains(in=a) AllnoUsedDom(in=b);
      by table;
      if (not a and b);
   run;

   options orientation = landscape nodate missing = "" sysprint="";
   ods escapechar = "^";
   ods listing close;
   ods pdf file = "&ClientOutPath.&StudyName DomainList.pdf" notoc style = statdoc;
   
   %let trIncludedDoms = %nrbquote(%LocaleGet(ListofIncludedDomainsinJMPClinicalStudy_Text));
   %let trExcludedDoms = %nrbquote(%LocaleGet(ListofExcludedDomainsinJMPClinicalStudy_Text));

   %let titDomList= %nrbquote(%LocaleFill(&trIncludedDoms, %nrbquote(&StudyName)));
   %let titNoUsedDomList = %nrbquote(%LocaleFill(&trExcludedDoms, %nrbquote(&StudyName)));
   %let domListDoclink = %nrbquote(%LocaleGet(PleaseRefToHDJMPCIDForInfWhyDomMayExc_Text));
   %let trAvailableDomains = %nrbquote(%LocaleGet(AvailableDomains_Text));
   %let trFolders = %nrbquote(%LocaleGet(Folders_Text));
   %let trDomainsnotusedinJMPC = %nrbquote(%LocaleGet(DomainsnotusedinJMPC_Text));
   %let trClickHere = %nrbquote(%LocaleGet(ClickHere_Text));
   
   title height=17pt "&titDomList";
   proc report nowindows missing nofs colwidth=15 style = {frame = void rules=none} data = AllDomains;
      column table Flag;
      define table / left width = 15 "&trAvailableDomains";
      define Flag / left width = 15 "&trFolders";
   run;
   
   title height=17pt "&titNoUsedDomList";
   proc report nowindows missing nofs colwidth=15 style = {frame = void rules=none} data = AllnoUsedDom2;
      column table Folder;
      define table / left width = 15 "&trDomainsnotusedinJMPC";
      define Folder / left width = 15 "&trFolders";
   run;  
   title ;
   ods pdf text= "^S={font=('Arial')}";
   ods pdf text = "^S={font=('Arial')}";
   ods pdf text = "^S={font=('Arial') FONTSIZE=4 just=center} &domListDoclink";         
   ods escapechar="^";
   ods pdf text="^S={font=('Arial') FONTSIZE=3  just=center URL='&DocLinkIdDomsIncStudy'} &trClickHere";
   
   ods pdf close;
   ods listing;
   proc datasets nolist;
      delete Alldomains AllnoUsedDom AllnoUsedDom2;
   quit;
   %exitdomadd:;
%mend DOMAINLISTADD;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro GETPREVDATA;
   %let studynameunder = %sysfunc(transtrn(&StudyName, %str( ), _));
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
   
   data getprev;     
      infile "%jg_getPath(/user/clinical/metadata/base, CSTMacroVars.sas, 0, 0, 3, &studynameunder)" linesize = 200 pad;
      input text $ 1-200;
         if substr(left(text),1,8) = "%nrstr(%let) in_" or substr(left(text),1,13) = "libname sdtm "
      or substr(left(text),1,13) = "libname adam ";
   run;

   data getprev;
      set getprev;
      text = trim(text);
      text = transtrn(text, "%nrstr(%let) in_", "%nrstr(%let) prev_");
      text = transtrn(text, "libname sdtm", "libname prevsdtm");
      text = transtrn(text, "libname adam", "libname prevadam");
      text = transtrn(text, "= sdtm.", "= prevsdtm.");
      text = transtrn(text, "= adam.", "= prevadam.");
   run;

   %let ReviewPlus = %eval(&CurrentReview+1);
   libname RevLib "&ServerOutPath";
   *** Temporarily Save in Output Folder ***;   
   data RevLib.DeltaFile&ReviewPlus;
      *file "&ServerOutPath.DeltaFile&ReviewPlus..sas";
      set getprev;
      put text;
   run;  

   proc datasets nolist;
      delete getprev;
   quit;

%mend GETPREVDATA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro MAKENOTES;

   %let trrflg = %nrbquote(%LocaleGet(ReviewFlag_Text));
   %let trReviewed = %nrbquote(%LocaleGet(Reviewed_Text));
   %let trUnreviewed = %nrbquote(%LocaleGet(Unreviewed_Text));
   %let trIntroduced = %nrbquote(%LocaleGet(Introduced_Text));
   %let trPreviouslyAvailable = %nrbquote(%LocaleGet(PreviouslyAvailable_Text));
   %let trUser = %nrbquote(%LocaleGet(User_Text));   
   
   %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
   *** if notes directory doesnt exist create one ***;
   %include "&MacroPath./UtilityPathMacros.sas" / nosource;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder))); 
   %if ^%sysfunc(fexist(&fileref)) %then %do;
      %jg_getPath(/user/clinical/notes,,3,0,0,&studynameunder);
   %end;
      
   %if &sysscp = WIN %then %do;
      %let _slash_ = \;
   %end;
   %else %do;
      %let _slash_ = /;
   %end;

   *** if notes directory for study doesnt exist create one ***;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder))); 
   %if ^%sysfunc(fexist(&fileref)) %then %do;
      %jg_getPath(/user/clinical/notes,,3,0,0,&studynameunder);
   %end;
   
   libname NotesLib "%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder)";
   %global NotesDir; 
   %let NotesDir = %jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder);

   * for Review packaging used (T.C. 06/21/13 S0984129);
   %if ^%symexist(AddinPackageFlag) %then %do;
      %global AddinPackageFlag;
      %let AddinPackageFlag = 0;
   %end;

/* 
   *** Kemicl: No longer delete adslrs on AddStudy to accommodate Shared notes ***;
   *** If adding a study start the review data set anew ***;
   %if &ProcessName = AddStudyLocal & &AddinPackageFlag = 0 %then %do;     
      data _null_;
         fname = "tempfile";
         rc = filename(fname,"&NotesDir.adslrs.sas7bdat");
         if rc = 0 and fexist(fname) then rc = fdelete(fname);
         rc = filename(fname);
      run;  
      
      data _null_;
         fname = "tempfile";
         rc = filename(fname,"&NotesDir.adslrsl.sas7bdat");
         if rc = 0 and fexist(fname) then rc = fdelete(fname);
         rc = filename(fname);
      run;        
   %end;
*/    
   * assign macro for review status and create default data sets if necessary;   
   %if ^%sysfunc(exist(NotesLib.adslrsl)) %then %do;
      data NotesLib.adslrsl;
         RSTATUS = "&trUnreviewed";
         output;
         RSTATUS = "&trReviewed";
         output;
      run;
   %end;
   %if ^%sysfunc(exist(NotesLib.adslrs)) %then %do;
      data _null_;
         set NotesLib.adslrsl(obs=1);
         call symput('defaultrs',trim(left(RSTATUS)));
      run;
      data NotesLib.adslrs;
         retain USUBJID RSTATUS ReviewFlag DateAdded DateUpdated User ;
         length ReviewFlag $ 20 DateAdded $ 19 DateUpdated $ 19   User $ 50 ;
         set &in_adsl;
         RSTATUS = "&defaultrs";
         ReviewFlag = "&trIntroduced"; 
         DateAdded = put(datetime(),E8601DT19.);
         DateUpdated = put(datetime(),E8601DT19.);
         User = "&sysuserid";         
         label DateAdded = "DateTime Subject Added to Data Set" ReviewFlag = "&trrflg" User = "&trUser" 
         DateUpdated = "DateTime of Last Status Update";
         keep USUBJID RSTATUS ReviewFlag User DateAdded DateUpdated;
      run;
   %end;
   %else %do;
      %if &ProcessName = AddStudyLocal %then %do;
         proc sort data=NotesLib.adslrs(keep= USUBJID) out=currSubList;
            by USUBJID;
         run;
         
         proc sort data=&in_adsl(keep=USUBJID) out=newSubList;
            by USUBJID;
         run;
         
         data checksubs;
            merge currSubList(in=a) newSubList(in=b);
            by USUBJID;
            if (a and not b) or (not a and b);
         run;
         
         %nobsvars(checksubs);
         %if &nobs %then %do; 
            %put ERROR: A notes directory exists for a study previously defined as &StudyName and the subject list for the study you are adding does not match...Please choose a different study name or move/remove the notes directory.;
            %put The notes directory is located at %jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder);
            %let exiterror = 1;
            %goto mnexit;
         %end;
      %end;
      %else %do;
         *** Patch to move NOTES to notes dataset ***;      
         %let rschk = %sysfunc(open(NotesLib.adslrs));
         %let notenum = %sysfunc(varnum(&rschk,Notes));
         %let rschk = %sysfunc(close(&rschk));

         %if &notenum %then %do;
            data notes;
               set NotesLib.adslrs;
               where notes ne "";
            run;
            %Nobsvars(notes);
            %if &nobs=0 %then %do;
               proc datasets nolist;
                  delete notes;
               quit;
            %end;
            %else %do;
               %let trPatientProfiler = %nrbquote(%LocaleGet(PatientProfiler_Text));
               %let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
               %let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
               %let trReport = %nrbquote(%LocaleGet(Report_Text));
               %let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
               %let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
               %let trSnapshot = %nrbquote(%LocaleGet(Snapshot_Text));
               %let trType = %nrbquote(%LocaleGet(Type_Text));
               %let _trKeys = %nrbquote(%LocaleGet(Keys_Text));
               %let trDomain = %nrbquote(%LocaleGet(Domain_Text));
               %let trUser = %nrbquote(%LocaleGet(User_Text));
               %let trNote = %nrbquote(%LocaleGet(Note_Text));
               %let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));
               
               data notes;
                  length Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Type $ 24 ;
                  set notes(rename = (notes = Note) drop = rstatus);
                  Report = "&trPatientProfiler";
                  Study_Name = "&StudyName";
                  SnapshotNumber = 0;
                  User = "&sysuserid";
                  Type = "&trSnapshot";
                  Date_of_Note = dhms(mdy(substr(DateAdded, 6, 2), substr(DateAdded, 9, 2), substr(DateAdded, 1, 4)), 
                     substr(DateAdded, 12, 2), substr(DateAdded, 15, 2), substr(DateAdded, 18, 2));
                  drop DateAdded;
                  format Date_of_Note DATETIME19.;
                  label Date_of_Note = "&trDateofNote" Report = "&trReport" Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" 
                     SnapshotNumber = "&trSnapshotNumber" Type = "&trType" Keys = "&_trKeys" Domain = "&trDomain" User = "&trUser" Note = "&trNote";
               run;

               %if ^%sysfunc(fileexist(&NotesDir.&notesDataset..sas7bdat)) %then %do;
                  data NotesLib.&notesDataset;
                     retain SnapshotNumber Type usubjid %if %length(&Siteid) %then %do; siteid %end; %if %length(&country) %then %do; country %end; Study_Name Keys Keys_Values 
                        Domain Report User Note Date_of_Note SystemGenerated;
                     set notes;
                     %if %length(&SiteID) %then %do; 
                        label siteid = "&SiteIDLabel";
                     %end;                         
                     %if %length(&Country) %then %do; 
                        label Country = "&CountryLabel";
                     %end;                         
                  run;
               %end;
               %else %do;
                  %let _tid_ = %sysfunc(open(noteslib.&notesDataset));
                     %let notesusubjid = %sysfunc(varnum(&_tid_,USUBJID));         
                     %let notessiteid = %sysfunc(varnum(&_tid_,SITEID));
                     %let notescountry = %sysfunc(varnum(&_tid_,COUNTRY));
                     %if &notesusubjid %then %let Ulength_ = %sysfunc(varlength(&_tid_,&notesusubjid));
                     %else %let Ulength_ = 0;
                     %if &notessiteid %then %let Slength_ = %sysfunc(varlength(&_tid_,&notessiteid));
                     %else %let Slength_ = 0;         
                     %if &notescountry %then %let Clength_ = %sysfunc(varlength(&_tid_,&notescountry));
                     %else %let Clength_ = 0;         
                     %let USUBJIDlength = %sysfunc(max(&USUBJIDlength,&Ulength_));
                     %let SITEIDlength = %sysfunc(max(&SITEIDlength,&Slength_));
                     %let COUNTRYlength = %sysfunc(max(&COUNTRYlength,&Clength_));         
                  %let _tid_ = %sysfunc(close(&_tid_));

                  data notes;
                     retain SnapshotNumber Type usubjid %if &notessiteid or %length(&SiteID) %then %do; siteid %end; %if &notescountry or %length(&country) %then %do; country %end; 
                        Study_Name Keys Keys_Values Domain Report User Note Date_of_Note SystemGenerated;
                     length usubjid $ &USUBJIDlength 
                            %if &notessiteid or %length(&SiteID) %then %do; siteid $ &SITEIDlength %end;
                            %if &notescountry or %length(&Country) %then %do; country $ &COUNTRYlength %end;
                            ;                     
                     set notes NotesLib.&notesDataset;
                     %if &notessiteid or %length(&SiteID) %then %do; 
                        label siteid = "&SiteIDLabel";
                     %end;
                     %if &notescountry or %length(&country) %then %do;
                        label Country = "&CountryLabel";
                     %end;
                  run;

                  proc sort data = notes out = NotesLib.&notesDataset;
                     by SnapshotNumber Date_of_Note;
                  run;
               %end;

               proc datasets nolist;
                  delete &notesDataset;
               quit;
            %end;

            data NotesLib.adslrs;
            retain USUBJID RSTATUS ReviewFlag DateAdded DateUpdated  User;
            length ReviewFlag $ 20 DateAdded $ 19 DateUpdated $ 19   User $ 50 ;
               set NotesLib.adslrs;
               ReviewFlag = "&trIntroduced";     
               DateUpdated = put(datetime(),E8601DT19.);    
               User = "&sysuserid";   
               drop notes;
            run;
         %end;    

         *** Add New Subjects ***;
         data _null_;
            set NotesLib.adslrsl(obs=1);
            call symput('defaultrs',trim(left(RSTATUS)));
         run;

         data new(keep = usubjid);
            set &in_adsl;
         run;

         proc sort data = new;
            by usubjid;
         run;

         data old;
            set NotesLib.adslrs;
         run;

         proc sort data = old;
            by usubjid;
         run;

         data new;
            length ReviewFlag $ 20 DateAdded $ 19 DateUpdated $ 19 User $ 50   ;
            merge old(in = ina) new(in = inb);
            by usubjid;
            if inb and not ina;
            RSTATUS = "&defaultrs";
            ReviewFlag = "&trIntroduced";    
            DateAdded = put(datetime(),E8601DT19.);       
            DateUpdated = put(datetime(),E8601DT19.);
            User = "&sysuserid";         
            label DateAdded = "DateTime Subject Added to Data Set" ReviewFlag = "&trrflg" User = "&trUser" 
            DateUpdated = "DateTime of Last Status Update";
            keep USUBJID RSTATUS ReviewFlag DateAdded User DateUpdated;
         run;

         data old;
            retain USUBJID RSTATUS ReviewFlag User DateAdded DateUpdated;
            set old(in = ina) new;
            %if &ProcessName = ManageStudies %then %do;
               %if ^%index(&Task,Rename) %then %do;   
                  if ina and (ReviewFlag = "Introduced" or ReviewFlag = "&trIntroduced") then do;
                     ReviewFlag = "&trPreviouslyAvailable";
                  end;
               %end;
            %end;
         run;   

         proc sort data = old out = NotesLib.adslrs;
            by usubjid;
         run;

         proc datasets nolist;
            delete old new;
         quit; 
      %end;
   %end;
   
   data _null_;
      file "&basemeta&_slash_.CSTMacroVars.sas" mod;
      put "%nrstr(%global) NotesDir in_adslrs in_adslrsl;";
      put "%nrstr(%let) NotesDir = &NotesDir;";
      put "libname NotesLib '&NotesDir';";
      put "%nrstr(%let) in_adslrs = NotesLib.adslrs;";
      put "%nrstr(%let) in_adslrsl = NotesLib.adslrsl;";      
   run;  
   %mnexit:;
%mend MAKENOTES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;