/*
JMP Genomics Macros

Name: MixedModelMacros
Author: Tzu-Ming Chu, SAS Institute Inc.
Support: Tzu-Ming Chu, SAS Institute Inc.
History:
   tzuchu 28Aug02 initial coding
   sasrdw 01Jul03 added multiple testing methods
   sasrdw 24Mar04 added processing for lsmeans diff,
                  slice, and labels
   tzuchu 09Dec04 added processing for contrast
   juzhan 02Feb05 updated WebSearchJSL macro call
   sasrdw 11Apr05 converted from stacked to tall 
   sasrdw 18Sep07 moved OneWay code here so Bivariate One-Way can also call it

Description: 
   MixedModelMacros contains the primary computational
   macros for the following APs:

   ANOVA
   ANOVA Normalization
   Bivariate One-Way ANOVA
   Mixed Model Analysis
   Mixed Model Normalization
   One-Way ANOVA
   Findings ANOVA
*/

/* main macro for mixed model normalization */
%macro MixedNormalization(_htmlbody=, _htmlframe=, _htmlcontents=);;

* check input data sets;
%CheckInData(_ExpFlag=1);

* create stacked data set;
%Stack(_Out=InData);

%let ds = InData;

* filter responses individually;
%if (%length(&Filter)) %then %do;
   data &ds;
      set &ds;
      if &Filter then delete;
   run;
%end;

%if &syserr %then %goto exit;

* lsmeans variables to keep;
%let LsmToKeepVars = ;
%if %length(&LsmeansEffectToKeep) %then %do;
   %put LsmeansEffectToKeep = &LsmeansEffectToKeep;
   %let i = 1;
   %do %while(%length(%scan(&LsmeansEffectToKeep,&i,'*()')));
      %let LsmToKeepVars = &LsmToKeepVars %scan(&LsmeansEffectToKeep,&i,'*()');
      %let i = %eval(&i+1);
   %end;
   %put LsmToKeepVars = &LsmToKeepVars;
   %put;
%end;

* add outp= option to model statement;
%let flag_rownum = 1;
%CheckVarExist(_rownum,&ds,Row Number,_ShowError=0);
%if &exiterror %then %do;
   %let exiterror = 0;
   %let flag_rownum = 0;
%end;
%let flag_ColumnName = 1;
%CheckVarExist(ColumnName,&ds,Column Name,_ShowError=0);
%if &exiterror %then %do;
   %let exiterror = 0;
   %let flag_ColumnName = 0;
%end;
%MixedAddModOpt(%str(outp=residuals(keep=
   %if &flag_rownum %then _rownum
   %if &flag_ColumnName %then ColumnName
   resid &LsmToKeepVars)));

* normalization model fit;
ods html 
   body = "&&&_htmlbody" (url="&bodyfile")
   frame = "&&&_htmlframe"  (url="&framefile")
   contents = "&&&_htmlcontents" (url="&contentsfile")
;

%put;
%put Proc Mixed Statements = &ProcMixedStatements;

ods noresults;
ods listing close;

proc mixed data=&ds &ProcMixedOptions namelen=32;
   &ProcMixedStatements
   %if %length(&LsmeansEffectToKeep) %then %do;
      ods output lsmeans=_lsm;
   %end;
run;
%if &syserr %then %goto exit;

ods html close;
ods listing;
ods results;
run;

%NObsVars(residuals);
%if &nobs = 0 %then %do;
   %put ERROR:  PROC MIXED did not converge.  The model may be over- or mis-specified or fitting assumptions may be violated.;
   %let exiterror = 1;
   %PackageInsertHtml(html from &ProcessName, &OutPath, &OutdataName.body.html,
      &OutDataName.frame.html, &OutDataName.contents.html);
   %goto exit;
%end;

* add lsmeans effect back into residuals;
%if %length(&LsmeansEffectToKeep) %then %do;

   data _lsm1;
      set _lsm;
      where upcase(effect) = "%upcase(&LsmeansEffectToKeep)";
      _LSM = estimate;
      keep &LsmToKeepVars _LSM; 
   run;
   
   %NObsVars(_lsm1);
   %if (&nobs = 0) %then %do;
      %put ERROR: The LSMeans Effect to Keep &LsmeansEffectToKeep was not found in the LSMeans output table.;  
      %put Please respecify it to match one of the LSMeans effects in this table.;
      %put If it is an interaction effect, the order of the variables must be the same as in the Class Variables list.;
      %put WARNING: The output normalized data set does not retain any LSMeans effects.;
   %end;
   %else %do;
      proc sort data=_lsm1;
         by &LsmToKeepVars;
      run;

      proc sort data=residuals;
         by &LsmToKeepVars;
      run;

      data residuals;
         merge residuals _lsm1;
         by &LsmToKeepVars;
         resid = resid + _LSM;
         drop _LSM &LsmToKeepVars;
      run;
   %end;
%end;

* apply normalized filter;
%if (%length(&NormFilter)) %then %do;
   data residuals;
      set residuals;
      if resid %substr(&NormFilter,%index(%upcase(&NormFilter),RESPONSE)+8) then resid = .;
   run;
%end;

%Stack_to_Tall(_InData=residuals,_Out=&OutData,_Responses=resid);

/*
* transform back to tall form;
proc sort data=residuals;
   by _rownum;
run;

proc transpose data=residuals out=outd(drop=_name_ _label_);
   by _rownum;
   var resid;
   id ColumnName;
run;

data &OutData;
   set InLib.&InData;
   _rownum = _n_;
   drop %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
run;

data &OutData;
   merge &OutData outd;
   by _rownum;
   drop _rownum;
run;
*/

* Packaging;
%PackageInsertHtml(html from &ProcessName, &OutPath, &OutdataName.body.html,
   &OutDataName.frame.html, &OutDataName.contents.html);
%PackageInsertDataset(Normalized data from &ProcessName, outlib, &OutDataName);

/*
%PackagePublish(&OutPath, to_requester);
*/

%exit:;

%mend MixedNormalization;



/*---add an outp= option to the model statement---*/
%macro MixedAddModOpt(opt);

%put Adding &opt to ProcMixedStatements;
%let pmst = %qleft(&ProcMixedStatements);
%let stmts = %upcase(&pmst);
%let iv = 1;
%do %while (%length(%qscan(%nrbquote(&stmts),&iv,%str(;))));
   %let stmt = %nrbquote(%qscan(%nrbquote(&stmts),&iv,%str(;)));
   %let first = %qscan(&stmt,1);
   %let fn = %eval(%index(&stmt,&first) + %length(&first));
   %let iv = %eval(&iv+1);
   %if %index(&first,MODEL) %then %do; 
      %let iv = 999;
   %end;
%end;

%let midx = %index(%quote(%upcase(&pmst)),&stmt);
%if (&midx = 0) %then %do;
   %let exiterror = 1;
   %put ERROR: MODEL statement does not exist.; 
   %goto exit;
%end;
%let pmst1 = %substr(%quote(&pmst),&midx);
%let midx1 = %index(%quote(&pmst1),%str(;));
%let pmst2 = %substr(%quote(&pmst1),1,&midx1);
%let midx2 = %index(%quote(&pmst2),%quote(&opt));
%if &midx2 = 0 %then %do;
   %let firstpart = %substr(%quote(&pmst),1,%eval(&midx-1)) %substr(%quote(&pmst1),1,%eval(&midx1-1));
   %let secondpart = %substr(%quote(&pmst1),&midx1);
   /*
   %let firstpart = %substr(%quote(&pmst),1,%eval(&midx+&midx1-1));
   %let secondpart = %substr(%quote(&pmst),%eval(&midx+&midx1));
   */
   %put pmst = &pmst;
   %put midx = &midx;
   %put midx1 = &midx1;
   %put firstpart = &firstpart;
   %put secondpart = &secondpart;
   %if (%index(%quote(&pmst2),%str(/))) %then %let slashchar = ;
   %else %let slashchar = /;
   %let ProcMixedStatements = &firstpart &slashchar &opt &secondpart;
%end;

%exit:;

%mend MixedAddModOpt;


/*---extract ClassVars, FixedEffects, and RandomEffects from ProcMixedStatements---*/
%macro MixedGetEffects;

%global ClassVars FixedEffects RandomEffects;
%let ClassVars = ;
%let FixedEffects = ;
%let RandomEffects = ;

%let pmst = %qleft(&ProcMixedStatements);
%let stmts = %upcase(&pmst);
%let iv = 1;
%do %while (%length(%qscan(%nrbquote(&stmts),&iv,%str(;))));
   %let stmt = %nrbquote(%qscan(%nrbquote(&stmts),&iv,%str(;)));
   %let first = %qscan(&stmt,1);
   %let fn = %eval(%index(&stmt,&first) + %length(&first));
   %let midx = %index(%quote(%upcase(&pmst)),&stmt);
   %if %index(&first,MODEL) %then %do; 
      %let pmst1 = %substr(%quote(&pmst),&midx);
      %let midx1 = %index(%quote(&pmst1),%str(;));
      %let pmst2 = %substr(%quote(&pmst1),1,%eval(&midx1-1));
      %let midx2 = %index(%quote(&pmst2),%str(/));
      %if &midx2 %then %do;
         %let pmst2 = %substr(%quote(&pmst2),1,%eval(&midx2-1));
      %end;
      %let midx3 = %index(%quote(&pmst2),%str(=));
      %let FixedEffects = %substr(%quote(&pmst2),%eval(&midx3+1));
   %end;
   %else %if %index(&first,RANDOM) %then %do; 
      %let pmst1 = %substr(%quote(&pmst),%eval(&midx+7));
      %let midx1 = %index(%quote(&pmst1),%str(;));
      %let pmst2 = %substr(%quote(&pmst1),1,%eval(&midx1-1));
      %let midx2 = %index(%quote(&pmst2),%str(/));
      %if &midx2 %then %do;
         %let pmst2 = %substr(%quote(&pmst2),1,%eval(&midx2-1));
      %end;
      %let RandomEffects = &pmst2;
   %end;
   %else %if %index(&first,CLASS) %then %do; 
      %let pmst1 = %substr(%quote(&pmst),%eval(&midx+6));
      %let midx1 = %index(%quote(&pmst1),%str(;));
      %let pmst2 = %substr(%quote(&pmst1),1,%eval(&midx1-1));
      %let midx2 = %index(%quote(&pmst2),%str(/));
      %if &midx2 %then %do;
         %let pmst2 = %substr(%quote(&pmst2),1,%eval(&midx2-1));
      %end;
      %let ClassVars = &pmst2;
   %end;
   %let iv = %eval(&iv+1);
%end;

%put ClassVars = &ClassVars;
%put FixedEffects = &FixedEffects;
%put RandomEffects = &RandomEffects;
%put;

%exit:;
%mend MixedGetEffects;



/*---check input data sets---*/
/*
%macro MixedInDataCheck;

%put ExpInData = &ExpInData;
%CheckForBlank(&ExpInData,Input dataset);
%if %sysfunc(fileexist(&ExpInData.*)) = 0 %then %do; 
   %let exiterror = 1;
   %put ERROR: Specified Experimental Design Data Set does not exist.;
%end;
%PathName(&ExpInData);
%if &exiterror %then %do;
   %put ERROR: Experimental Design Data Set specification is invalid.;
   %goto exit;
%end;
%let ExpInData = &TmpName;
libname ExpInLib "&TmpPath";
%if %sysfunc(fileexist(&InData.*)) = 0 %then %do; 
   %let exiterror = 1;
   %put ERROR: Specified Input Data Set does not exist.;
%end;
%PathName(&InData);
%if &exiterror %then %do;
   %put ERROR: Input Data Set specification is invalid.;
   %goto exit;
%end;
%let InData = &TmpName;
libname InLib "&TmpPath";

%exit:;

%mend;
*/

/*---check mixed model parameter specifications---*/
%macro MixedParmCheck;

%CheckForBlank(&OutPath,Output path);
libname OutLib "&OutPath";

%CheckForBlank(&ProcMixedStatements,Proc Mixed statements);
%if &exiterror %then %goto exit;

/* check output data set names */
%if %length(&OutData) and (%length(&OutDataComp) or %length(&MMEIData)) %then %do;
   %if (&OutData = &OutDataComp) or (&OutData = OutLib.&MMEIData) or
      (&OutDataComp = OutLib.&MMEIData)%then %do;
      %put ERROR: Output data sets cannot have the same name.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end;

/* check annotation */
%CheckAnno;
%if &exiterror %then %goto exit;
%if %length(&AnnoInData) and %length(&AnnoWhereStmt) %then %do;
   data OutLib._AnnoData1_;
      set AnnoLib.&AnnoData;
      where %unquote(&AnnoWhereStmt);
   run;
   %let AnnoData = _AnnoData1_;
   %let AnnoPath = &OutPath;
   libname AnnoLib "&OutPath";
%end;
%global Anno2Data Anno2Path;
%if %length(&Anno2InData) %then %do;
   * save and reassign certain macro variables so OnlyAnno works;
   %let AnnoData0 = &AnnoData;
   %let AnnoMergeVars0 = &AnnoMergeVars;
   %let BaseMergeVars0 = &BaseMergeVars;
   %let AnnoPath = %sysfunc(pathname(AnnoLib));

   %let AnnoInData = &Anno2InData;
   %let AnnoMergeVars = &Anno2MergeVars;
   %let BaseMergeVars = &BaseMergeVars2;
   %CheckAnno;
   %let Anno2MergeVars = &AnnoMergeVars;
   %let BaseMergeVars2 = &BaseMergeVars;
   %let Anno2Data = &AnnoData;
   %let Anno2Path = %sysfunc(pathname(AnnoLib));
   libname Anno2Lib "&Anno2Path";
 
   %if %length(&Anno2WhereStmt) %then %do;
      data OutLib._AnnoData2_;
         set AnnoLib.&AnnoData;
         where %unquote(&Anno2WhereStmt);
      run;
      %let Anno2Data = _annoData2_;
      %let Anno2Path = &OutPath;
      libname Anno2Lib "&OutPath";
   %end;

   * revert anno macro assignments;
   %let AnnoData = &AnnoData0;
   %let AnnoMergeVars = &AnnoMergeVars0;
   %let BaseMergeVars = &BaseMergeVars0;
   libname AnnoLib "&AnnoPath";
   /*
   %PathName(&Anno2InData);
   %let Anno2Data = &TmpName;
   %let Anno2Path = &TmpPath;
   libname Anno2Lib "&Anno2Path"; 
   %if %length(&Anno2WhereStmt) %then %do;
      data OutLib.&Anno2Data;
         set Anno2Lib.&Anno2Data;
         where %unquote(&Anno2WhereStmt);
      run;
      %let Anno2Path = &OutPath;
      libname Anno2Lib "&OutPath";
   %end;
   */
%end;
%else %do;
   %let Anno2Data = ;
   %let Anno2Path = ;
%end;

/* check by variables */
%if ^%length(&ByVars) %then %do;
   %let ByVars = _Row;
   %let _KeepVar_ = _Row &_KeepVar_;
   %put _KeepVar_ = &_KeepVar_;
%end;
%global OrigByVars;
%let OrigByVars = &ByVars;
%let ByVars = &DesignByVars &ByVars;
%global LastByVar _ByVars nby _MergeNonByVars;
%let _ByVars = ;
%let _MergeNonByVars = ;
%let nby = 1;
%CheckByVars(_OldList=&ByVars,_NewList=_ByVars,_nVar=nby);
%do iv = 1 %to &nby;
   %let ByVar&iv = %scan(&_ByVars,&iv,' ');
%end;
%let LastByVar = &&ByVar&nby;
%if %length(&BaseMergeVars) %then %do;
   %let i = 1;
   %do %while(%length(%scan(&BaseMergeVars,&i,' ')));
      %let bmvi = %upcase(%trim(%left(%scan(&BaseMergeVars,&i,' '))));
      %if ^%index(%upcase(&_ByVars),&bmvi) %then %do;
         %let _MergeNonByVars = &_MergeNonByVars %scan(&BaseMergeVars,&i);
      %end;
      %let i = %eval(&i+1);
   %end;
%end;

* check for duplicate fixed and random effects;
%if (&ProcessName = MixedModelAnalysis) or ^%symexist(FixedEffects) %then %do;
   %MixedGetEffects;
%end;

%if %length(%trim(&FixedEffects)) and %length(%trim(&RandomEffects)) %then %do;
   %let i = 1;
   %do %while(%length(%scan(&FixedEffects,&i,' ')));
      %let fei = %qupcase(%scan(&FixedEffects,&i,' '));
      %put fei = &fei;
      %if %length(&fei) %then %do;
         %let j = 1;
         %do %while(%length(%scan(&RandomEffects,&j,' ')));
            %let rej = %qupcase(%scan(&RandomEffects,&j,' '));
            %if %length(&rej) %then %do;
               %put rej = &rej;
               %if &fei = &rej %then %do;
                  %put ERROR: The effect &fei is specified as both fixed and random.;
                  %put It must be one or the other, not both.;
                  %let exiterror = 1;
                  %goto exit;
               %end;
            %end;
            %let j=%eval(&j+1);
         %end;
      %end;
      %let i=%eval(&i+1);
   %end;
%end;

* add this code for handling GLIMMIX updates in ANOVA AP (TC 2/2/11);
%if ^%symexist(WhichProc) %then %do;
   %global WhichProc;
   %let WhichProc = MIXED;
%end;
%if ^%symexist(DataDistribution) %then %do;
   %global DataDistribution;
   %let DataDistribution = NORMAL;
%end;

/* add outp=StdRes to end of model statement if not found */ 

%let KeepResVars = &ByVars &_MergeNonByVars Response Resid;
%if (%upcase(&PlotResiduals)=YES) or (%length(&MMEIData)) %then %do;
   %let KeepResVars = &KeepResVars pred _rownum ColumnName;
%end;
%else %do;
   %if (&WhichProc = GLIMMIX) %then %do;
      %let KeepResVars = &KeepResVars pred;
   %end;
%end;

%if &WhichProc = MIXED %then %do;
   * add studentized residual to StdRes output dataset 2/9/11 TC;
   /* %MixedAddModOpt(%str(outp=StdRes(keep=&KeepResVars) Residual)); */
   /* removing Residual option due to S0974338: AE Severity ANOVA gives integer overflow error on metadata server */
   %MixedAddModOpt(%str(outp=StdRes(keep=&KeepResVars)));
%end;
%if &WhichProc = GLIMMIX %then %do;
   %let KeepResVars = &KeepResVars StuResidual;
   %if %qupcase(&DataDistribution)=%qupcase(Generalized Poisson) %then %do;
      %MixedAddModOpt(%str(Link=&LinkFunction));
   %end;
   %else %do;
      %MixedAddModOpt(%str(Dist=&DataDistribution Link=&LinkFunction));
   %end;
   %global OutputStatement;
   %let OutputStatement = %str(Output out=StdRes(keep=&KeepResVars) pred=Pred residual=Resid student=StuResidual;);
   * reset "outer" iteration convergence cutoff;
   %let ProcMixedOptions = &ProcMixedOptions pconv=%sysevalf(&PConv/100000000);
%end;
%if &WhichProc = LOGISTIC %then %do;
   %MixedAddModOpt(%str(Link=&LinkFunction)));
%end;
/*
%MixedAddModOpt(%str(outp=StdRes(drop=StdErrPred DF Alpha Lower Upper)));
*/

%global FilterResiduals;
%if (&FilterType = None) %then %let FilterResiduals = No;
%else %let FilterResiduals = Yes;

/* verify filter residuals specification */
%if %upcase(&FilterResiduals) = YES %then %do;
   %if %length(&ResidualFalsePositive) | %length(&ResidualCutoff) %then
      %put NOTE: Both ResidualFalsePositive and ResidualCutoff are specified. Only ResidualFalsePositive will be applied.;
   %if (&ResidualFalsePositive = 0) %then %do;
      %put ERROR: ResidualFalsePositive must be positive.;
     %let exiterror = 1;
     %goto exit;
   %end;
%end;

%if %upcase(&LComponents)=YES %then %do;
   %MixedAddModOpt(%str(lcomponents solution));
%end;

%exit:;

%mend MixedParmCheck;





/*---main entry point for mixed model fit---*/
%macro MixedFit(_htmlbody=, _htmlframe=, _htmlcontents=);

/*---determine experimental design variables to keep---*/

/*---loop through statements and extract information, code scarfed from %glimmix macro---*/
%let stmts = %upcase(&ProcMixedStatements);
%let spec = ;
%let class = ;
%let parms = ;
%let id = ;
%let rndlst = ;
%let replst = ;
%let intopt = ;
%let iv = 1;

%do %while (%length(%qscan(%nrbquote(&stmts),&iv,;)));

   %let stmt = %nrbquote(%qscan(%nrbquote(&stmts),&iv,%str(;)));
   %let first = %qscan(&stmt,1);
   %let fn = %eval(%index(&stmt,&first) + %length(&first));

   /*---check RANDOM options and extract random effects list---*/
   %if %index(&first,RANDOM) or %index(&first,REPEATED) %then %do; 
      %let intopt = &intopt COVMOD;
   %end;
   %if %index(&first,RANDOM) %then %do;
      %let i = %index(&stmt,/);
      %if &i = 0 %then %do;
         %let i = %length(&stmt);
         %let i = %eval(&i + 1); 
      %end;
      %else %do;  
         %let rndopt = %substr(&stmt,&i); 
         %if %index(&rndopt,%str( S )) or %index(&rndopt,SOLUTION) or
            %index(&rndopt,%str( CL )) or %index(&rndopt,ALPHA) %then %do;
            %let intopt = &intopt SOLNR;
         %end;
      %end;
      %let rndlst = &rndlst %substr(&stmt,&fn,%eval(&i-&fn));
   %end;

   /*---check REPEATED options and extract repeated effects list---*/
   %if %index(&first,REPEATED) %then %do;
    %let i = %index(&stmt,/);
    %if &i = 0 %then %let i = %length(&stmt);
    %else %do; 
       %let repopt = %substr(&stmt,&i); 
       %let j = %index(&repopt,EXP);
       %if &j ne 0 %then %do;
          %let k = %index(&repopt,EXP);
          %let repexp = %bquote(%substr(&repopt,&k));
          %let k1 = %index(&repexp,%str(%());
          %let k1 = %eval(&k1 + 1);
          %let repexp1 = %bquote(%substr(&repexp,&k1));
          %let k2 = %index(&repexp1,%str(%)));
          %let replst = &replst %substr(&repexp,&k1,
             %eval(&k2-1));
       %end;
       %let j = %index(&repopt,TYPE=SP);
       %if &j ne 0 %then %do;
          %let k = %index(&repopt,TYPE=SP);
          %let repexp = %bquote(%substr(&repopt,&k));
          %let k1 = %eval(%index(&repexp,%str(%))) + 1);
          %let repexp = %bquote(%substr(&repexp,&k1));
          %let k2 = %eval(%index(&repexp, %str(%()) + 1);
          %let k3 = %eval(%index(&repexp, %str(%))) - 1);
          %let replst = &replst %substr(&repexp,&k2,
             %eval(&k3-&k2+1));
       %end;
    %end;
    %let j = %eval(&i-&fn);
    %if &j > 0 %then 
       %let replst = &replst %substr(&stmt,&fn,&j);
   %end;

   /*---save statements---*/
   %if %index(&first,CLASS) %then %do;
      %let class = %qsubstr(&stmt,&fn);
   %end;
   %else %if %index(&first,MODEL) %then %do;
      %let model = %qsubstr(&stmt,&fn);
   %end;
   %else %if %index(&first,ID) %then %do;
      %let id = %qsubstr(&stmt,&fn);
   %end;
   %else %if %index(&first,PARMS) %then %do;
      %let parms = %qsubstr(&stmt,&fn);
   %end;
   %else %let spec = &spec &stmt %str(;) ;

   /*
   %put;
   %put iv = &iv;
   %put stmt = &stmt;
   %put first = &first;
   %put fn = &fn;
   %put class = &class;
   %put;
   */
   
   %let iv = %eval(&iv + 1);

%end;


/*---get response---*/
%let response = %scan(&model,1,=);
%let eqidx = %eval(%index(&model,=)+1);
%if (&eqidx > %length(&model)) %then %let mdl = %str();
%else %let mdl = %str( ) %qsubstr(&model,&eqidx);
%if %index(&mdl,/) %then %do;
    %let mdlspec = %qscan(&mdl,1,/);  
%end;
%else %do;
    %let mdlspec = &mdl;
%end;

/*---get variable list and trim it---*/
%let varlst0 = %trim(&DesignByVars);
%let mdllst = &mdlspec;
%let varlst = ;

/*---strip out @ signs---*/
%if %index(&mdllst,@) %then %do;
 %let j = 1;
 %let mdl = &mdllst;
 %let mdllst=;
 %do %while(%length(%scan(&mdl,&j,' ')));
    %let var=%scan(&mdl,&j,' ');
    %if %index(&var,@) %then %do;
       %let b = %eval(%index(&var,@)-1);
       %let mdllst = &mdllst %substr(%quote(&var),1,&b);
    %end;
    %else %let mdllst = &mdllst &var;
    %let j = %eval(&j+1);
 %end;
%end;
%if %index(&rndlst,@) %then %do;
 %let j = 1;
 %let mdl = &rndlst;
 %let rndlst=;
 %do %while(%length(%scan(&mdl,&j,' ')));
    %let var=%scan(&mdl,&j,' ');
    %if %index(&var,@) %then %do;
       %let b = %eval(%index(&var,@)-1);
       %let rndlst = &rndlst %substr(%quote(&var),1,&b);
    %end;
    %else %let rndlst = &rndlst &var;
    %let j = %eval(&j+1);
 %end;
%end;

/*---get variables from fixed effects---*/
%let iv = 1;
%do %while (%length(%scan(&mdllst,&iv,%str( ) | * %( %) )));
 %let varlst = &varlst %scan(&mdllst,&iv,%str( ) | * %( %) );
 %let iv = %eval(&iv + 1);
%end;

/*---get variables from random effects---*/
%let iv = 1;
%do %while (%length(%scan(&rndlst,&iv,%str( ) | * %( %))));
 %let temp = %scan(&rndlst,&iv,%str( ) | * %( %) );
 %if %index(%quote(&temp),-) %then %do;
   %let varlst = &varlst &temp;
 %end;
 %else %do;
    %if &temp ne INT and &temp ne INTERCEPT %then %let varlst = &varlst &temp;
 %end;
 %let iv = %eval(&iv + 1);
%end;  

/*---get variables from repeated effects---*/
%let iv = 1;
%do %while (%length(%scan(&replst,&iv,%str( ) | * %( %))));
 %let temp = %scan(&replst,&iv,%str( ) | * %( %) );
 %if &temp ne DIAG %then %let varlst = &varlst &temp;
 %let iv = %eval(&iv + 1);
%end;  

/*---if there is any overlap between by variables and effects, throw an error---*/
%if %length(&varlst0) %then %do;
   %let iv0 = 1;
   %do %while (%length(%scan(&varlst0,&iv0,%str( ))));
      %let temp0 = %upcase(%scan(&varlst0,&iv0,%str( )));
      %let iv = 1;
      %do %while (%length(%scan(&varlst,&iv,%str( ))));
         %let temp = %upcase(%scan(&varlst,&iv,%str( )));
         %if (&temp0 = &temp) %then %do;
            %put ERROR: By Variable &temp occurs as a modeling effect, resulting in singularities.;
            %put Please remove the variable from the By Variables list or from modeling effects.;
            %let exiterror = 1;
            %goto mfexit;
         %end;
         %let iv = %eval(&iv + 1);
      %end;  
      %let iv0 = %eval(&iv0 + 1);
   %end;  
%end;

%let varlst = &varlst0 &varlst &class &id; 

/*---add InData variables---*/
%let varlst = &OrigByVars &BaseMergeVars &BaseMergeVars2 &ToKeepVars &varlst;
%if (%upcase(&PlotResiduals)=YES) or (%length(&MMEIData)) %then %do;
   %let varlst = &varlst _rownum ColumnName;
%end;

/*---add Baseline Covariate variables---*/
%if ^%symexist(BaseSubjectVar) %then %let BaseSubjectVar = ;
%if ^%symexist(BaselineVar) %then %let BaselineVar = ;
%if ^%symexist(BaselineValue) %then %let BaselineValue = ;
%if %length(&BaseSubjectVar) and %length(&BaselineVar) and %length(&BaselineValue) %then %do;
   %let varlst = &varlst %upcase(&BaseSubjectVar &BaselineVar);
   %let _tid_ = %sysfunc(open(ed));
   %if ^&_tid_ %then %let _tid_ = %sysfunc(open(InLib.&InData));
   %let bvarnum = %sysfunc(varnum(&_tid_,&BaselineVar));
   %let bvartype = %sysfunc(vartype(&_tid_,&bvarnum));
   %let _tid_ = %sysfunc(close(&_tid_));

   %if (&bvartype=C) %then %do;
      %let bcheck = %str(&BaselineVar = "&BaselineValue");
   %end;
   %else %do;
      %let bcheck = %str(&BaselineVar = &BaselineValue);
   %end;

   %put bvarnum = &bvarnum;
   %put bvartype = &bvartype;
   %put bcheck = &bcheck;
%end;

/*---remove duplicates in varlst---*/
%let i1 = 1;
%let tname =;
%do %while (%length(%scan(&varlst,&i1,%str( ))));
   %let first = %scan(&varlst,&i1,%str( ));
   %let i2 = %eval(&i1 + 1);
   %do %while (%length(%scan(&varlst,&i2,%str( ))));
      %let next = %scan(&varlst,&i2,%str( ));
      %if %quote(&first) = %quote(&next) %then %let i2=10000;
      %else %let i2 = %eval(&i2 + 1);
   %end;
   %if (&i2<10000) %then %let tname = &tname &first;
   %let i1 = %eval(&i1 + 1);
%end;
%let varlst = &tname;
%put varlst = &varlst;
%put ByVars = &ByVars;

/*---drop data with missing by variables---*/
%let Mixed_Indata = InLib.&InData;
%if %symexist(Miss_Flag) %then %do; 
   * using the data after filtering from ANOVA AP; 
   %if %upcase(&Miss_Flag) = YES %then %let Mixed_Indata = NonMissInData;
%end;

data InDataAll;
   set &Mixed_Indata;
   %if %index(&ByVars,_Row) %then %do;
      _Row = _n_;
   %end;
   %if (&ByVars ^= _Row) %then %do;
      %let i=1;
      %do %while(%length(%scan(&OrigByVars,&i,' ')));
         %let bvi = %scan(&OrigByVars,&i,' ');
         %if &bvi ^= _Row %then %do;
            where not missing(&bvi);
         %end;
         %let i=%eval(&i+1);
      %end;
   %end;
run;
%if %symexist (WhereInmt) %then %do;
   %if %length(&WhereInmt) %then %do;
       data InDataAll;
         set InDataAll;
          where %unquote(&WhereInmt);
       run;
       
       %Nobsvars(InDataAll);
       %if &syserr>4 or &nobs=0 %then %do;
         %if &nobs=0 %then %put ERROR: There are no observations that satisfy the Filter to Include Observations.;
         %let exiterror = 1;
         %goto mfexit;
       %end;       
   %end;
%end;

%NObsVars(InDataAll);
%if &nobs=0 %then %do;
   %put ERROR: The input data set has no valid observations.;
   %let exiterror = 1;
   %goto mfexit;
%end;


/*--drop unannotated data if necessary---*/
%if %length (&AnnoInData) and (&AnnoOnly=yes) %then %do;
   %OnlyAnno;
   %NObsVars(InDataAll);
   %if &nobs=0 %then %do;
      %put ERROR: Merge with annotation data yielded no observations.;
      %let exiterror = 1;
      %goto mfexit;
   %end;
%end;

%if %length (&Anno2InData) and (&Anno2Only=yes) %then %do;
   * save and reassign certain macro variables so OnlyAnno works;
   %let AnnoData0 = &AnnoData;
   %let AnnoMergeVars0 = &AnnoMergeVars;
   %let BaseMergeVars0 = &BaseMergeVars;

   %let AnnoData = &Anno2Data;
   %let AnnoMergeVars = &Anno2MergeVars;
   %let BaseMergeVars = &BaseMergeVars2;
   libname AnnoLib "&Anno2Path";

   %OnlyAnno;
   %if &exiterror %then %goto mfexit;
   %NObsVars(InDataAll);
   %if &nobs=0 %then %do;
      %put ERROR: Merge with secondary annotation data yielded no observations.;
      %let exiterror = 1;
      %goto mfexit;
   %end;

   * revert anno macro assignments;
   %let AnnoData = &AnnoData0;
   %let AnnoMergeVars = &AnnoMergeVars0;
   %let BaseMergeVars = &BaseMergeVars0;
   libname AnnoLib "&AnnoPath";
%end;


/*---break by groups into chunks to avoid building one huge stacked data set---*/
proc sort data=InDataAll;
   by &OrigByVars;
run;

data ByG;
   set InDataAll;
   keep &OrigByVars;
run;

proc sort data=ByG nodupkey;
   by &OrigByVars;
run;

data ByG;
   set ByG end=_end_;      
   _chunk = ceil(_n_/&ChunkSize);
   if _end_ then call symput('nchunk',trim(left(_chunk)));
run;

%if (&nchunk > 1) %then %do;
   %let varlst = &varlst _chunk;
%end;
%else %do;
   data ByG;
      set ByG;
      drop _chunk;
   run;
%end;

data InDataAll;
   merge InDataAll ByG;
   by &OrigByVars;
run;

* remove spaces around character by variables;
%let _tid_ = %sysfunc(open(InDataAll));
%let numbyv = 0;
%let gotchar = 0;
%let i=1;
%do %while(%length(%scan(&OrigByVars,&i)));
   %let byv&i = %scan(&OrigByVars,&i);
   %let byn&i = %sysfunc(varnum(&_tid_,&&byv&i));
   %put byv&i = &&byv&i;
   %put byn&i = &&byn&i;
   %if (&&byn&i) %then %do;
      %let byt&i = %sysfunc(vartype(&_tid_,&&byn&i));
      %if (&&byt&i) = C %then %do;
         %let gotchar = 1;
      %end;
   %end;
   %else %do;
      %let byt&i = ;
   %end;
   %put byt&i = &&byt&i;
   %let i=%eval(&i+1);
%end;
%let _tid_ = %sysfunc(close(&_tid_));
%let numbyv = %eval(&i-1);
%put gotchar = &gotchar;
%put numbyv = &numbyv;

%if &gotchar and &numbyv %then %do;
   data InDataAll;
      set InDataAll;
      %do i=1 %to &numbyv;
         %if (&&byt&i) = C %then %do;
            &&byv&i = trim(left(&&byv&i));
         %end;
      %end;
   run;
%end;
      
%if (%upcase(&ListAll) = YES) %then %do;
   ods html 
   body = "&&&_htmlbody" (url="&bodyfile")
   frame = "&&&_htmlframe"  (url="&framefile")
   contents = "&&&_htmlcontents" (url="&contentsfile")
;
%end;
%else %if &LS_Product ^= Clinical %then %do;
   * turn listing off;
   ods exclude all;
   ods noresults; 
   options nonotes;
   run;
%end;

%if ^%symexist(AlreadyStacked) %then %do;
   %global AlreadyStacked;
   %let AlreadyStacked = 0;
%end;

%put AlreadyStacked = &AlreadyStacked;
%put;
   
* loop over chunks;  
%do _chunk=1 %to &nchunk;

   %put Processing chunk &_chunk of &nchunk....;
   
   %if (&nchunk > 1) %then %do;
      data InChunk;
         set InDataAll;
         where _chunk = &_chunk;
      run;
      %let InChunk = InChunk;
   %end;
   %else %do;
      %let InChunk = InDataAll;
   %end;

   %if %length(&BaseSubjectVar) and %length(&BaselineVar) and %length(&BaselineValue) %then %do;
      data &InChunk;
         set &InChunk;
         BaseCov = .;
      run;
   %end;
   %put varlst = &varlst;
   
   * create stacked data set for this chunk;
   %if &AlreadyStacked %then %do;
      data InData;
         set &InChunk;
      run;
   %end;
   %else %do;
      %Stack(_In=&InChunk,_Out=InData,_IKeep=&varlst);
   %end;
      
   * for baseline covariate, restack the data so those responses are now covariates;
   %if %length(&BaseSubjectVar) and %length(&BaselineVar) and %length(&BaselineValue) %then %do;
      
      %if (%upcase(&ListAll) ^= YES) %then %do;
         options notes;
      %end;
      
      %if ^%symexist(BaseByVars) %then %let BaseByVars = &ByVars;
      %if ^%symexist(BaselineTimeVar) %then %let BaselineTimeVar = ;
      
      data InData CovData(keep=&BaseByVars &BaseSubjectVar Response &BaselineTimeVar);
         set InData;
         if &bcheck then output CovData;
         else output InData;
      run;
      
      %NObsVars(CovData);
      %if (&nobs = 0) %then %do;
         %put ERROR: No valid observations found for baseline covariate &bcheck..;
         %let mixederr = 1;
         %let exiterror = 1;
         %goto mfexit;
      %end;
         
      proc sort data=CovData;
         by &BaseByVars &BaseSubjectVar &BaselineTimeVar;
      run;
      %if %upcase(&BaselineCalc) = LAST %then %do;
         data MeanCovData;
            set CovData;
            by  &BaseByVars &BaseSubjectVar &BaselineTimeVar;
            if last. &BaseSubjectVar;
            BaseCov = Response;
            keep &BaseByVars &BaseSubjectVar BaseCov;
         run;
      %end;
      %else %do;
         proc means data=CovData noprint;
            by &BaseByVars &BaseSubjectVar;
            var Response;
            output out=MeanCovData mean=BaseCov;
         run;
      %end;
         
      proc sort data=InData out=InData(drop=BaseCov);
         by &BaseByVars &BaseSubjectVar;
      run;
      
      data InData;
         merge InData MeanCovData;
         by &BaseByVars &BaseSubjectVar;
      run;
      
      %if (&BaseByVars ne &ByVars) %then %do;
         data InData;
            set InData;
            %let i=1;
            %do %while(%length(%scan(&ByVars,&i,' ')));
               if missing(%scan(&ByVars,&i,' ')) then delete;
               %let i=%eval(&i+1);
            %end;
         run;
         proc sort data=InData;
            by &ByVars;
         run;
      %end;
         
      %if (%upcase(&ListAll) ^= YES) and &LS_Product ^= Clinical %then %do;
         options nonotes;
      %end;
   %end;
   %else %do;
      proc sort data=InData;
         by &ByVars;
      run;
   %end;

   %let QC = 0;
   %if %upcase(&FilterResiduals) = YES %then %do;
      %let QC = 1;
   %end;

   %put QC = &QC;
   %let mixederr = 0;
/*    
   %if ^%length(&ResidualFalsePositive) %then %let ResidualFalsePositive=.01;
   %put ResidualFalsePositive = &ResidualFalsePositive;
*/ 
   /* call outlier screening loop */
   %OutlierScreen(Method=&QC,
                  Alpha=&ResidualFalsePositive,
                  CutOff=&ResidualCutoff,
                  MaxIteration=&MaxLoop,
                  InputData=InData,
                  OutStdRes=_OutStdRes,
                  OutResults=_OutResults,
                  OutLComp=_OutLComp               
   );
   %if &exiterror %then %goto mfexit;
%*let exiterror = 1; %*goto mfexit;
   %if ^%symexist(LabelVar) %then %do;
      %global LabelVar;
      %let LabelVar = ;
   %end;

   %if (&mixederr = 0) %then %do;
   
      * merge label,chromosome, and position variables with results;
      %if %length(&LabelVar) or %length(&ChrVar) or %length(&PosVar) or %length(&ToKeepVars) %then %do;
         data chrpos;
            set &InChunk;
            keep &OrigByVars &LabelVar &ChrVar &PosVar &ToKeepVars;
         run;
         
         proc sort data=chrpos nodupkey;
            by &OrigByVars;
         run;
         
         %if (&OrigByVars ^= &ByVars) %then %do;
            proc sort data=_OutResults;
               by &OrigByVars;
            run;
         %end;
         
         data _OutResults;
            merge chrpos _OutResults;
            by &OrigByVars;
            /*
            %if %length(&ChrVar) %then %do;
               if missing(&ChrVar) then delete;
            %end;
            %if %length(&PosVar) %then %do;
               if missing(&PosVar) then delete;
            %end; 
            */
         run;
         
         %if (&OrigByVars ^= &ByVars) %then %do;
            proc sort data=_OutResults;
               by &ByVars;
            run;
         %end;

         %if %upcase(&LComponents)=YES %then %do;
            %if ^%symexist(InClassVars) %then %let InClassVars = ;
            %if %length(&InClassVars) %then %do;
               data chrpos;
                  set &InChunk;
                  keep &OrigByVars &LabelVar &ChrVar &PosVar &InClassVars;
               run;
         
               proc sort data=chrpos;
                  by &OrigByVars &InClassVars;
               run;
            %end;

            proc sort data=_OutLComp;
               by &OrigByVars &InClassVars;
            run;
            
            data _OutLComp;
               merge chrpos _OutLComp;
               by &OrigByVars &InClassVars;
               if missing(Estimate) then delete;
            run;

            proc sort data=_OutLComp;
               by &ByVars EIndex LIndex;
            run;
         %end;
      %end;
            
      %if (&_chunk=1) %then %do;
         data &OutData;
            set _OutResults;
         run;

         data &OutDataRes;
            set _OutStdRes;
         run;
         
         %if %upcase(&LComponents)=YES %then %do;
            data &OutDataComp;
               set _OutLcomp;
            run;
         %end;
      %end;
      %else %do;
         %* the change is for defect S0840483: it seems not appending;
         data OutdataTmp;
            set &OutData;
         run;
         proc append base=OutdataTmp data=_OutResults force;
         run;
         data &OutData;
            set OutdataTmp;
         run;

         proc append base=&OutDataRes data=_OutStdRes force;
         run;
         
         %if %upcase(&LComponents)=YES %then %do;
            proc append base=&OutDataComp data=_OutLComp force;
            run;
         %end;
      %end;
      %if %length(&AnnoChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         %MergeAnno(&OutDataRes);
         %if &exiterror %then %goto mfexit;
      %end;
   
   %end;
         
%end;

* merge annotation data into main results output data set;
%global ChrVar0 PosVar0;
%let ChrVar0 = &ChrVar;
%let PosVar0 = &PosVar;
%if %length (&AnnoInData) %then %do;
   %*MergeAnno(&OutData,_NoMissKeys=Variance_Residual); * S0863167 - some non-convergent rows will be removed;
   %MergeAnno(&OutData);
   %if &exiterror %then %goto mfexit;
   %NObsVars(&OutData);
   %put Number of observations in &OutDataName = &nobs;
   %if &nobs=0 %then %do;
      %put ERROR: Merge with annotation data produced zero observations.;
      %put The merge key variables may be incompatible.;
      %let exiterror = 1;
      %goto mfexit;
   %end;
   %let cp = 0;
   %if %length(&AnnoChrVar) %then %do;
      %put Setting Chromosome variable to &AnnoChrVar..;
      %let ChrVar = &AnnoChrVar;
      %let cp = 1;
   %end;
   %if %length(&AnnoPosVar) %then %do;
      %put Setting Position variable to &AnnoPosVar..;
      %let PosVar = &AnnoPosVar;
      %let cp = 1;
   %end;
   %put ChrVar = &ChrVar;
   
   * convert PosVar to numeric if necessary;
   %if %length(&PosVar) %then %do;
      %let pvt = ;
      %VarAttributes(&OutData,&PosVar,_Type=pvt);
      %if &pvt=C %then %do;
         data &OutData;
            length _pvn 8;
            set &OutData;
            _pvn = 0 + &PosVar;
            drop &PosVar;
         run;
         data &OutData;
            set &OutData;
            rename _pvn = &PosVar;
         run;
      %end;
   %end;
   
   * delete rows with missing chromosome or position information;
   /*
   %if (&cp) %then %do;
      data &OutData;
         set &OutData;
         %if %length(&ChrVar) %then %do;
            if missing(&ChrVar) then delete;
         %end;
         %if %length(&PosVar) %then %do;
            if missing(&PosVar) then delete;
         %end;    
      run;
      %NObsVars(&OutData);
      %put Number of observations in &OutDataName after deleting missing chrom and pos = &nobs;
      %if &nobs=0 %then %do;
         %put ERROR: Deleting output with missing chromosome or position data produced zero observations.;
         %put Please check that chromosome and position variable specifications are correct.;
         %let exiterror = 1;
         %goto mfexit;
      %end;
   %end;
   */
   %if %symexist(AnnoLabResid) %then %do;
      %if %upcase(&AnnoLabResid) = YES %then %do;
         %MergeAnno(&OutDataRes);
         %if &exiterror %then %goto mfexit;
      %end;
   %end;
   %if %upcase(&LComponents) = YES %then %do;
      %MergeAnno(&OutDataComp,LComFlag=1);
      %if &exiterror %then %goto mfexit;
      %NObsVars(&OutDataComp);
      %if &nobs=0 %then %do;
         %put ERROR: Merge with annotation data yielded no observations.;
         %let exiterror = 1;
         %goto mfexit;
      %end;    
   %end;
%end;

* merge secondary annotation data into components output data set;
%if %length(&Anno2Data) and %upcase(&LComponents)=YES %then %do;
   * make sure secondary merge keys exist in component data set;
   %let dsid=%sysfunc(open(&OutDataComp));
   %let gotem = 1;
   %if &dsid %then %do;
      %let i = 1;
      %do %while(%length(%scan(&Anno2MergeVars,&i)));
         %let vnum = %sysfunc(varnum(&dsid,%scan(&Anno2MergeVars,&i)));
         %if ^&vnum %then %let gotem = 0;
         %let i = %eval(&i+1);
      %end;
      %let rc=%sysfunc(close(&dsid));
   %end;
   %else %let gotem = 0;
   %if &gotem %then %do;
   
      * save and reassign certain macro variables so MergeAnno works;
      %let AnnoData0 = &AnnoData;
      %let AnnoMergeVars0 = &AnnoMergeVars;
      %let BaseMergeVars0 = &BaseMergeVars;
   
      %let AnnoData = &Anno2Data;
      %let AnnoMergeVars = &Anno2MergeVars;
      %let BaseMergeVars = &Anno2MergeVars;
      libname AnnoLib "&Anno2Path";
      
      %MergeAnno(&OutDataComp);
      %if &exiterror %then %goto mfexit;

      proc sort data=&OutDataComp;
         by &ByVars EIndex LIndex;
      run;
      
      * revert anno macro assignments;
      %let AnnoData = &AnnoData0;
      %let AnnoMergeVars = &AnnoMergeVars0;
      %let BaseMergeVars = &BaseMergeVars0;
      libname AnnoLib "&AnnoPath";
      
      * assign secondary chromosome and position variables;
      %let Chr2Var = &ChrVar;
      %let Pos2Var = &PosVar;
      %if %length(&Anno2ChrVar) %then %do;
         %put Setting secondary chromosome variable to &Anno2ChrVar..;
         %let Chr2Var = &Anno2ChrVar;
      %end;
      %if %length(&Anno2PosVar) %then %do;
         %put Setting secondary position variable to &Anno2PosVar..;
         %let Pos2Var = &Anno2PosVar;
      %end;
      
      * convert Pos2Var to numeric if necessary;
      %if %length(&Pos2Var) %then %do;
         %let pvt = ;
         %VarAttributes(&OutDataComp,&Pos2Var,_Type=pvt);
         %if &pvt=C %then %do;
            data &OutDataComp;
               length _pvn 8;
               set &OutDataComp;
               _pvn = 0 + &Pos2Var;
               drop &Pos2Var;
            run;
            data &OutDataComp;
               set &OutDataComp;
               rename _pvn = &Pos2Var;
            run;
         %end;
      %end;
   %end;
   %else %do;
      %put Secondary annotation merge key variables not found in fixed effects component data set.;
   %end;
%end;

%if (%upcase(&ListAll) = YES) %then %do;
   ods html close;
   run;
%end;
%else %do;
   * turn listing back on;
   ods exclude none;
   ods results;
   options notes;
   run;
%end;

%mfexit:;
                  

%mend MixedFit;



/* code for a single mixed model fit */
%macro MixedCode(dsin,Loop=0);
   * added option &Loop parameter for conditional breaking submission 
      when testing code (TC 03/04/11);
%ClockStart(MixedCode Chunk=&_chunk of &nchunk);

* scan statements and check for lsmeans, diff, and estimate;
%global HasT1 Hast2 HasT3 HasLsm HasDiff HasSlice HasEst HasCon; 
%let HasT1 = 0;
%let HasT2 = 0;
%let HasT3 = 0;
%let HasLsm = 0;
%let HasDiff = 0;
%let HasSlice = 0;
%let HasEst = 0;
%let HasCon = 0;
%let iv = 1;

%do %while (%length(%qscan(%quote(&ProcMixedStatements),&iv,%str(;))));
   %let stmt = %upcase(%qscan(%quote(&ProcMixedStatements),&iv,%str(;)));
   %let first = %qscan(%quote(&stmt),1);
   %if %index(%quote(&first),MODEL) %then %do;
      * the htype specification must not contain any spaces;
      %let hi = %index(%quote(&stmt),HTYPE);
      %if (&hi > 0) %then %do;
         %let hsub = %qsubstr(%quote(&stmt),%eval(&hi+6));
         %let htype = %qscan(%quote(&hsub),1,' ');
         %put htype = &htype;
         %if %index(%quote(&htype),1) %then %let HasT1 = 1;
         %if %index(%quote(&htype),2) %then %let HasT2 = 1;
         %if %index(%quote(&htype),3) %then %let HasT3 = 1;
      %end;
      %else %do;
         %let HasT3 = 1;
      %end;
   %end;
   %if %index(%quote(&first),LSMEANS) %then %do;
      %let HasLsm = 1;
      %if %index(%quote(&stmt),DIFF) %then %do;
         %let HasDiff = 1;
      %end;
      %if %index(%quote(&stmt),SLICE) %then %do;
         %let HasSlice = 1;
      %end;
   %end;
   %if %index(%quote(&first),ESTIMATE) %then %do;
      %let HasEst = 1;
   %end;
   %if %index(%quote(&first),CONTRAST) %then %do;
      %let HasCon = 1;
   %end;
   %let iv = %eval(&iv + 1);
%end;

%put ProcMixedStatements = &ProcMixedStatements;
%global mixed_namelen;
%let mixed_namelen = 24;
%if &WhichProc=GLIMMIX %then %do;
   %put OutputStatement=&OutputStatement;
%end;

%RemoveData(StdRes);
/*
%*put ERROR: Input dataset: &dsin;
* For testing; 
options notes;
data outlib.Stack; set &dsin; run;
%let mixederr = 1;
%let exiterror = 1;
%goto mixedexit;
*/

* options notes mprint;
* gene-by-gene model fits;
proc &WhichProc data=&dsin
   %if &WhichProc=GLIMMIX %then %do;
      %if %qupcase(&DataDistribution)=%qupcase(Generalized Poisson) %then %do;
         method=laplace
      %end;
   %end;
   &ProcMixedOptions namelen=&mixed_namelen
;
   by &ByVars;
   %unquote(&ProcMixedStatements)
   %if ^%index(%upcase(&DataDistribution),NORMAL) & ^%index(%upcase(&DataDistribution),BETA)
      & ^%index(%upcase(&DataDistribution),NB) & ^%index(%upcase(&DataDistribution),GENERALIZED) %then %do; 
      random _residual_; 
   %end;
   %if &WhichProc=GLIMMIX %then %do;
      NLOPTIONS gconv=%sysevalf(&GConv/100000000);

      %if %qupcase(&DataDistribution)=%qupcase(Generalized Poisson) %then %do;
         xi = (1 - 1/exp(_phi_));
         _variance_ = _mu_ / (1-xi)/(1-xi);
         if (_mu_=.) or (_linp_ = .) then _logl_ = .;
         else do;
            mustar = _mu_ - xi*(_mu_ - response);
            if (mustar < 1E-12) or (_mu_*(1-xi) < 1e-12) then
               _logl_ = -1E20;
            else do;
               _logl_ = log(_mu_*(1-xi)) + (response-1)*log(mustar) - 
                  mustar - lgamma(response+1);
            end;
         end;
      %end;

      &OutputStatement
   %end;

   ods output covparms=cp FitStatistics=fs 
      %if (&HasT1) %then %do;
         tests1=t1
      %end;
      %if (&HasT2) %then %do;
         tests2=t2
      %end;
      %if (&HasT3) %then %do;
         tests3=t3
      %end;
      %if (&HasLsm) %then %do; 
         lsmeans=lsm 
         %if (&HasDiff) %then %do; 
            diffs=diffs
         %end;
         %if (&HasSlice) %then %do; 
            slices=slices
         %end;
      %end;
      %if (&HasEst) %then %do; 
         estimates=ests
      %end;
      %if (&HasCon) %then %do; 
         contrasts=cons
      %end;
      %if %upcase(&LComponents)=YES %then %do;
         lcomponents=lcomp
         solutionf=sf(drop=estimate stderr tvalue probt);
      %end;
      ;
run;
* for case of dummy Warning when PCONV does not meet;
%if &syserr=1012 %then %do; 
   data _null_; if (0); run;
%end;
%if &syserr>4 %then %put ERROR: syserr=&syserr;
/*
%let mixederr = 1;
%let exiterror = 1;
%goto mixedexit;
*/

%if ^%sysfunc(exist(fs)) %then %do;
	%put ERROR The requested ANOVA models could not be fit to the input data.  This may be due to missing data or not enough data for models to converge. The model fit statistics data was not created.;
	%let mixederr = 1;
	%let exiterror = 1;
	%goto mixedexit;	
%end;
* handling model fitness information;
%if &WhichProc=MIXED %then %do;
   proc transpose data=fs 
      out=fs2(drop=_NAME_ %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_; rename=(COL1=N2RLL COL2=AIC COL3=AICC COL4=BIC));
      by &ByVars;
      idlabel Descr;
   run;
%end;
%else %do;
   proc transpose data=fs 
      out=fs2(drop=_NAME_ %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_; rename=(COL1=N2LL COL2=AIC COL3=AICC COL4=BIC 
         COL5=CAIC COL6=HQIC COL7=PCS COL8=PCSDDF));
      by &ByVars;
      idlabel Descr;
   run;
%end;
/*
data outlib.fs ; set fs ; run;
data outlib.fs2; set fs2; run;
*/
%global vcFlag;
%let vcFlag = 0;
%if &WhichProc=GLIMMIX & ^%sysfunc(exist(cp)) %then %do;
   * no random statement in &ProcMixedStatements;
   * create cp data set for _rasidual_;
   %let vcFlag = 1; 
   data cp;
      set fs;
      length CovParm $13;
      if Descr = "Pearson Chi-Square / DF" | Descr = "Generalized Chi-Square / DF";
      CovParm = "Residual (VC)";
      drop Descr;
      rename Value = Estimate;
      label CovParm = "Parameter";
   run;
%end;

* screening case of no valid fitting for all features;
%nObsVars(StdRes);
%global Flag_no_valid_fit;
%let Flag_no_valid_fit = 0;
%if ^&nobs %then %do;
   %let Flag_no_valid_fit = 1;
   %goto mixedexit;
%end;

* check if CP table was created, if not, then the run bombed and so exit here;
%if ^%sysfunc(exist(cp)) %then %do;
   %put ERROR: PROC &WhichProc did not produce any covariance parameter estimates.  Ceasing processing.;
   %put The model is likely misspecified or confounding is too severe within each by group.;
   %let mixederr = 1;
   %let exiterror = 1;
   %goto mixedexit;
%end;

* check if diffs table was created, if not, then the run bombed and so exit here;
%if (&HasDiff) %then %do;
   %if ^%sysfunc(exist(diffs)) %then %do;
      %put ERROR: PROC &WhichProc did not produce any difference estimates.  Ceasing processing.;
      %put The model is likely misspecified or confounding is too severe within each by group.;
      %let mixederr = 1;
      %let exiterror = 1;
      %goto mixedexit;
   %end;
%end;

%if &syserr>4 %then %do;
   %let mixederr = 1;
   %let exiterror = 1;
   %goto mixedexit;
%end;


* check if Type3 table was created;
%if ^%sysfunc(exist(t3)) %then %let HasT3 = 0;
%put HasT3 = &HasT3;

%if &syserr>4 %then %goto mixedexit;

* options notes;

* merge class levels in with lcomponent tests;
%if %upcase(&LComponents)=YES %then %do;
   data lcomp;
      set lcomp;
      retain EIndex 1;
      if &LastByVar ne lag(&LastByVar) then EIndex = 0;
      if Effect ne lag(Effect) or EIndex = 0 then EIndex = EIndex+1;
   run;
   
   data sf;
      set sf;
      if effect="Intercept" or missing(df) then delete;
   run;
   
   * assumes an intercept is in the model;
   data sf;
      set sf;
      retain EIndex 0 LIndex 0;
      if &LastByVar ne lag(&LastByVar) then do;
         EIndex = 0;
         LIndex = 1;
      end;
      if Effect ne lag(Effect) or EIndex = 0 then do;
         EIndex = EIndex+1;
         LIndex = 1;
      end;
      else do;
         LIndex = LIndex+1;
      end;
   run;  
   
   /*
   data outlib.lcomp;
      set lcomp;
   run;
   
   data outlib.sf;
      set sf;
   run;
   */
   
   data lcomp;
      merge sf lcomp;
      by &ByVars EIndex LIndex;
      if missing(probt) then NegLog10Probt = .;
      else if (probt > 10**(-310)) then NegLog10Probt = -log10(probt);/* 1E-310 didn't work on client server. so change it to 10**(-310);*/
      else NegLog10Probt = 310;
      label EIndex="E Index" NegLog10Probt="-log10(Pr > |t|)";
   run;
%end;

* create datasets for plots;
* R-square values;
%if &whichProc = GLIMMIX %then %do;
   * convert response through link function and calculate R2 at that scale;
   data StdRes;
      set StdRes;
      response_link = pred + resid;
   run;
%end;
proc means data=StdRes mean var noprint;
   by &ByVars;
   %if %length(&_MergeNonByVars) %then %do;
      id &_MergeNonByVars;
   %end;
   where (response ne .) and (resid ne .);
   %if &whichProc = GLIMMIX %then %do;
      var response_link resid;
   %end;
   %else %do;
      var response resid;
   %end;
   output out=r2 var=vlogir vresid;
run;

* compute rsquare;
data RSquare;
   set r2;
   if vresid = 0 then RSquare = 1;
   else RSquare = 1 - vresid/vlogir;
   if (RSquare < 0) & RSquare ne . then RSquare = 0;
   keep &ByVars RSquare &_MergeNonByVars;
run;

* check group and subject variables;
proc contents data=cp out=tmp noprint;
run;

%let HasSub = 0;
%let HasGrp = 0;

%if ^%index(%upcase(&ByVars),SUBJECT) and ^%index(%upcase(&ByVars),GROUP) %then %do;
   data _null_ ;
      set tmp;
      if upcase(name) = 'SUBJECT' then call symput('HasSub',compress(1));
      if upcase(name) = 'GROUP' then call symput('HasGrp',compress(1));
   run;
%end;

%put HasSub = &HasSub;
%put HasGrp = &HasGrp;

%if &HasSub | &HasGrp %then %do;
   %let CovParm = trim(covparm);
   %if &HasSub %then %do;
      %let CovParm = &CovParm || "_" || trim(subject);
   %end;
   %if &HasGrp %then %do;
      %let CovParm = &CovParm || "_" || trim(translate(group,"_"," "));
   %end;

   data cp;
      set cp; 
      drop covparm;
      %if (&HasSub) %then %do;
         if (Subject ne '') then covparm1 = &CovParm;
         else covparm1 = covparm;
         drop subject;
      %end;
      %if (&HasGrp) %then %do;
         if (Group ne '') then covparm1 = &CovParm;
         else covparm1 = covparm;
         drop group;
      %end;
      rename covparm1=CovParm;
   run;
%end;

* variance component estimates;
proc transpose data=cp out=VarCompEsts(drop=_name_ %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_;) prefix=Variance_;
   by &ByVars;
   var estimate;
   id covparm;
run;

proc transpose data=cp out=PropVarCompEsts(drop=_name_ %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_;) prefix=PropVar_;
   by &ByVars;
   var estimate;
   id covparm;
run;

* determine if model contains a single residual variance;
%let dsid = %sysfunc(open(varcompests));
%let nvr = 0;
%if &dsid %then %do;
   %let nvr = %sysfunc(varnum(&dsid,Variance_Residual));
   %if &nvr=0 & %upcase(&FilterResiduals)=YES %then
      %put WARNING: Residuals are not standardized for this model because no single residual variance is available.;
   %let rc = %sysfunc(close(&dsid));
%end;

%if &WhichProc ne GLIMMIX %then %do;
   data VarCompEsts;
      set VarCompEsts;
      SumVariance = sum(of Variance_:);
   run;
%end;
%else %do;
   * there is no residual variance;
   * add generalized chi-square statistic and scale parameter if existing;
   * may need to be updated for methods are LAPLASE or QUAD for conditional 
      Pearson chi-square statistics; 
   data _null_;
      set fs;
      by &ByVars;
      if Descr = "Generalized Chi-Square" | Descr = "Pearson Chi-Square"
         then call symput("Descr",Descr);
      if last.&ByVars then stop;
   run;

   data fs;
      set fs;
      retain GenChiSq 0;
      by &ByVars;
      if Descr = "Generalized Chi-Square" | Descr = "Pearson Chi-Square" 
         then GenChiSq = Value;
      if last.&ByVars;
      _DF_ = GenChiSq / Value; 
      keep &ByVars Value _DF_;
      rename Value=Chi_Sq_Div_DF;
      label Chi_Sq_Div_DF = "&Descr Divided by DF";
   run;
%end;

data VarCompEsts;
   merge VarCompEsts %if &WhichProc=GLIMMIX %then fs; r2(keep=&ByVars vlogir) PropVarCompEsts;
   by &ByVars;
   %if (&nvr = 0) %then %do;
      Variance_Residual = 1;
   %end;
   array PropVars{*} PropVar_:;
   if SumVariance > 0 then do;
      do i = 1 to dim(PropVars); 
         PropVars{i} = PropVars{i} / SumVariance;
      end;
   end;
   else do;
      do i = 1 to dim(PropVars); 
         PropVars{i} = .;
      end;
   end;
   drop i;
   rename vlogir=SampleVariance;
run;

* compute standardized residuals;
data vcr;
   set VarCompEsts(keep=&ByVars Variance_Residual);
run;

data StdRes;
   merge StdRes vcr;
   by &ByVars;
   * if Pred ne . and (Variance_Residual > 1e-12);
   if (Variance_Residual > 0) then do;
      * Note: this standardized residual is not suitable for GLIMMIX case (2/10/11 TC);
      StdResidual = resid / sqrt(Variance_Residual);
   end;
   else do;
      StdResidual = .;
   end;
   drop Variance_Residual resid;
run;

* create macros for design-level by variables;
%global ndbv;
%let ndbv = 0;
%if %length(&DesignByVars) %then %do;
   %let i=1;
   %do %while(%length(%scan(&DesignByVars,&i)));
      %global dbv&i;
      %let dbv&i = %scan(&DesignByVars,&i);
      %let i = %eval(&i+1);
   %end;
   %let ndbv = %eval(&i-1);
   %put ndbv = &ndbv;
   %if &ndbv > 1 %then %do;
      %let LastDBV = %scan(&DesignByVars,&ndbv);
   %end;
   %else %do;
      %let LastDBV = &DesignByVars;
   %end;
%end;

* Contrasts;
%let ConData = ;
%if (&HasCon) %then %do;
   %let clabel = label;
   %if %upcase(&SplitResults)=YES %then %do;
      %do i=&ndbv %to 1 %by -1;
         %let clabel = trim(left(&&dbv&i)) || " " || &clabel;
      %end;
   %end;
   data cons;
      set cons;
      by &ByVars;
      lid = trim(left(&clabel));
      pclabel = "ProbF for Contrast " || trim(left(&clabel));
   run;

   proc transpose data=cons out=cons1(drop=_name_ _label_) prefix=PrF_Con_;
      by &ByVars;
      var probf;
      id lid;
      idlabel pclabel;
   run;

   %let ConData = cons1;

   * save transposed first obs for label processing;
   data tmp;
      set cons1(obs=1);
      keep PrF_Con_: ;
   run;

   proc transpose data=tmp out=tmpt;
   run;

   data consb1;
      set tmpt;
      pclabel = _label_;
      keep pclabel;
   run;
 
   /*
   * save labels from first by group for later jsl processing;
   %if %upcase(&SplitResults)=YES %then %do;
      data cons2;
         set cons;
         keep &ByVars pclabel;
      run;
      %let cbyvars = &OrigByVars &DesignByVars;
      proc sort data=cons2;
         by &cbyvars;
      run;
      %let cdata = cons2;
   %end;
   %else %do;
      %let cdata = cons;
      %let cbyvars = &ByVars;
   %end;

   data consb1;
      set &cdata;
      by &cbyvars;
      output;
      if (last.&LastByVar = 1) then stop;
      keep pclabel;
   run;
   */
   
%end;

* Estimates;
%let EstData = ;
%if (&HasEst) %then %do;
   %let elabel = label;
   %if %upcase(&SplitResults)=YES %then %do;
      %do i=&ndbv %to 1 %by -1;
         %let elabel = trim(left(&&dbv&i)) || " " || &elabel;
      %end;
   %end;
   data ests_in; set ests; run;
   data ests;
      set ests;
      by &ByVars;
      lid = trim(left(&elabel));
      elabel = "Estimate of " || trim(left(&elabel));
      %if (%upcase(&UDiffs) = YES) %then %do;
         %if (%upcase(&WhichProc) = MIXED) %then %do;
            if (Estimate < 0) then do;
               Exp2Est = -2**(-Estimate);
            end;
            else do;
               Exp2Est = 2**Estimate;
            end;
            udlab = "Exp2 Estimate of " || trim(left(&elabel));
         %end;
         %else %do;
            %if(%upcase(&LinkFunction) ne LOG) %then %do;
               if (Estimate < 0) then do;
                  Exp2Est = -2**(-Estimate);
               end;
               else do;
                  Exp2Est = 2**Estimate;
               end;
            %end;
            %else %do;
               if (Estimate < 0) then do;
                  Exp2Est = -2.718282**(-Estimate);
               end;
               else do;
                  Exp2Est = 2.718282**Estimate;
               end;
            %end;
            udlab = "Exp e Estimate of " || trim(left(&elabel));
         %end;
      %end;
      /*
      %if (%upcase(&UDiffs) = YES) %then %do;
         if (Estimate < 0) then do;
            Exp2Est = -2**(-Estimate);
         end;
         else do;
            Exp2Est = 2**Estimate;
         end;
         uelab = "Exp2 Estimate of " || trim(left(&elabel));
      %end;
      */
      %if (%upcase(&TStats) = YES) %then %do;
         telab = "t-Statistic for Estimate of " || trim(left(&elabel));
      %end;
      %if (%upcase(&PValues) = YES) %then %do;
         pelab = "p-Value for Estimate of " || trim(left(&elabel));
      %end;
      nlpelab = "-log10(p-Value) for Estimate of " || trim(left(&elabel));
      if missing(probt) then NegLog10p = .;
      else do; 
         if probt < 10**-300 then NegLog10p = 300;
         else NegLog10p = -log10(Probt);
      end;
   run;
   data ests_out; set ests; run;

   proc transpose data=ests out=ests1(drop=_name_) prefix=Est_;
      by &ByVars;
      var Estimate;
      id lid;
      idlabel elabel;
   run;

   %if (%upcase(&UDiffs) = YES) %then %do;
      proc transpose data=ests out=ests1u(drop=_name_ _label_) prefix=Exp2Est_;
         by &ByVars;
         var Exp2Est;
         id lid;
         idlabel uelab;
      run;
      %let ests1u = ests1u;
   %end;
   %else %do;
      %let ests1u = ;
   %end;
   
   %if (%upcase(&TStats) = YES) %then %do;
      proc transpose data=ests out=ests1t(drop=_name_ _label_) prefix=TStatEst_;
         by &ByVars;
         var tValue;
         id lid;
         idlabel telab;
      run;
      %let ests1t = ests1t;
   %end;
   %else %do;
      %let ests1t = ;
   %end;
   
   %if (%upcase(&PValues) = YES) %then %do;
      proc transpose data=ests out=ests1p(drop=_name_ _label_) prefix=PValEst_;
         by &ByVars;
         var probt;
         id lid;
         idlabel pelab;
      run;
      %let ests1p = ests1p;
   %end;
   %else %do;
      %let ests1p = ;
   %end;

   proc transpose data=ests out=ests2(drop=_name_) prefix=NLPE_;
      by &ByVars;
      var NegLog10p;
      id lid;
      idlabel nlpelab;
   run;

   %let EstData = ests1 &ests1u &ests1t &ests1p ests2;


   * save transposed first obs for label processing;
   data tmp;
      set ests1(obs=1);
      keep Est_: ;
   run;

   proc transpose data=tmp out=tmpt;
   run;

   data estsb1;
      set tmpt;
      elabel = _label_;
      keep elabel;
   run;
   
   data tmp;
      set ests2(obs=1);
      keep NLPE_: ;
   run;

   proc transpose data=tmp out=tmpt;
   run;
   
   data tmpt;
      set tmpt;
      nlpelab = _label_;
   run;

   data estsb1;
      merge estsb1 tmpt;
   run;

   /*
   * save labels from first by group for later jsl processing;
   %if %upcase(&SplitResults)=YES %then %do;
      data ests3;
         set ests;
         keep &ByVars elabel nlpelab;
      run;
      %let ebyvars = &OrigByVars &DesignByVars;
      proc sort data=ests3;
         by &ebyvars;
      run;
      %let edata = ests3;
   %end;
   %else %do;
      %let edata = ests;
      %let ebyvars = &ByVars;
   %end;
   
   data estsb1;
      set &edata;
      by &ebyvars;
      output;
      if (last.&LastByVar = 1) then stop;
      keep elabel nlpelab;
   run;
   */
%end;

* Lsmeans;
%let LsmData = ;
%let DiffData = ;
%let SliceData = ;
%if (&HasLsm) %then %do;
   * create label from all variables before Estimate;
   proc contents data=lsm out=lcon noprint;
   run;

   data lcon;
      set lcon;
      lvn = "lv" || trim(left(varnum));
      call symput(lvn,trim(left(name)));
      if name = "Effect" then call symput("nlvbeg",trim(left(varnum)));
      if name = "Estimate" then call symput("nlvend",trim(left(varnum-1)));
   run;

   %let llabel = trim(left(&&lv&nlvbeg));
   %do v=%eval(&nlvbeg+1) %to &nlvend;
      %let llabel = &llabel || trimn(" " || trim(left(&&lv&v)));
   %end;
   %let lid = &llabel;
   %let llabel = "Lsmean " || &llabel;
   
   %if %upcase(&SplitResults)=YES %then %do;
      %do i=&ndbv %to 1 %by -1;
         %let llabel = trim(left(&&dbv&i)) || " " || &llabel;
      %end;
   %end;
   %put llabel = &llabel;

   * create numeric suffixes to ensure unique varnames;
   proc sort data=lsm out=lsm_order nodupkey;
      by &DesignByVars %do i = %eval(&nlvbeg+1) %to &nlvend; &&lv&i %end;;
   run;
   
   data lsm_order;
      set lsm_order;
      lid = _n_;
      llabel = trim(left(&llabel));
      keep &DesignByVars %do i = %eval(&nlvbeg+1) %to &nlvend; &&lv&i %end; lid llabel;
   run;

   data lsm;
      if 0 then set lsm_order;
      if _n_ = 1 then do;
         declare hash h(dataset: "work.lsm_order", hashexp: 6);
            h.defineKey(
               %if &ndbv %then %do;
                  "&dbv1",
                  %do i = 2 %to &ndbv;
                     "&&dbv&i",
                  %end;
               %end;
               %do i = %eval(&nlvbeg+1) %to &nlvend;
                  %if (&i < &nlvend) & (%eval(&nlvbeg+1)<&nlvend) %then "&&lv&i",;
                  %else "&&lv&i";
               %end; 
            );
            h.defineData('lid','llabel');
            h.defineDone();
      end;
      
      set lsm;
      if (h.find()=0) then output;
   run;
/*
   data lsm;
      set lsm;
      by &ByVars;
      retain lid 0;
      %if %upcase(&SplitResults)=YES %then %do;
         retain lid0 1;
      %end;
      lid = lid + 1;
      if first.&LastByVar=1 then do;
         %if %upcase(&SplitResults)=YES %then %do;
            if first.&LastDBV=1 then do;
               lid0 = lid;
            end;
            else do;
               lid = lid0;
            end;
         %end;
         %else %do;
            lid = 1;
         %end;
      end;
      llabel = trim(left(&llabel));
   run;
*/   
   /*
   ods exclude none;
   ods results;
   proc print;
   run;
   */
   
   proc transpose data=lsm out=lsm1(drop=_name_ %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_;) prefix=LSMean;
      by &ByVars;
      var estimate;
      id lid;
      idlabel llabel;
   run;
   
   * create labels for JSL processing; 
   %*if ^%sysfunc(exist(lsmb1)) %then %do;
      /* %*LabelTable(_data=lsm1,_drop=&_ByVars,_out=lsmb1,_label=llabel); */
      proc sort data=lsm(keep=&DesignByVars llabel) out=lsmb1 nodupkey;
         by %if %upcase(&SplitResults)=YES %then &DesignByVars; llabel;
      run;
   %*end;
      
   %if ^%length(&LSMeanStdMethod) %then %do;
      %let LsmData = lsm1;
   %end;
   %else %do;
      * standardize the lsmeans within each gene;
      proc stdize data=lsm out=lsms method=&LSMeanStdMethod;
         by &ByVars;
         var estimate;
      run;

      data lsms;
         set lsms;
         lslabel = "Std " || llabel;
      run;

      proc transpose data=lsms out=lsms1(drop=_name_ %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_;) prefix=StdLSMean;
         by &ByVars;
         var estimate;
         id lid;
         idlabel lslabel;
      run;
      %let LsmData = lsm1 lsms1;

      * create labels for JSL processing; 
      %*LabelTable(_data=lsms1,_drop=&_ByVars,_out=lsmsb1,_label=lslabel);
      proc sort data=lsms(keep=&DesignByVars lslabel) out=lsmsb1 nodupkey;
         by %if %upcase(&SplitResults)=YES %then &DesignByVars; lslabel;
      run;
   %end;
   %if (&HasDiff) %then %do;
      
      * create label from all variables before Estimate;
      proc contents data=diffs out=dcon noprint;
      run;

      data dcon;
         set dcon;
         dvn = "dv" || trim(left(varnum));
         call symput(dvn,trim(left(name)));
         if name = "Estimate" then do;
            call symput("nv1",trim(left((varnum-&nby-2)/2)));
            call symput("nv2",trim(left(varnum-1)));
         end;
      run;
      
      /* Add option to only include LSmeans differences from a dataset created by Difference Chooser AP*/
      %if ^%symexist(DiffInData) %then %let DiffInData= ;
      %if %length(&DiffInData) %then %do;
         %PathName(&DiffInData);
         %let DiffData = &TmpName;
         %let DiffPath = &TmpPath;
         libname DiffLib "&DiffPath";
         %let effv = %eval(&nby + 1);
         %let startv = %eval(&nby + 2);
         
         * preserve order of specified diffs, also protects against reordering when residual filtering;
         data diffind;
            set DiffLib.&DiffData;
            _rowidx = _n_;
         run;
         
         /*
         data outlib.diffs;
            set diffs;
         run;
         */
         
         proc sql;
            create table diffnew as
            select * from diffs as d, diffind as dc
            where %do v=&effv %to &nv2;
               d.&&dv&v=dc.&&dv&v and
               %end; 1;
         quit;
         
         proc sort data=diffnew;
            by &ByVars _rowidx;
         run;
         
         /*
         %do v=&effv %to &nv2;
            %put dv&v = &&dv&v;
         %end;
         */
         
         %let _dsdiff = %sysfunc(open(diffnew));
         %let nobsd = %sysfunc(attrn(&_dsdiff,NOBS));
         %let rc=%sysfunc(close(&_dsdiff));
         %if (&nobsd=0) %then %do;
            %put ERROR: None of the differences in the LSMeans Difference Data Set Specified match those produced by PROC &WhichProc.. Please check the order of Class Variables and LSMeans Effect Parameters.;
            %let mixederr = 1;
            %let exiterror=1;
            %goto mixedexit;
         %end;
          
         /* kemicl 6/3/09 - added to reverse differences and reverse levels */
         data diffs;
            set diffnew;
            length tmp $64; 
            if reverse=1 then do;
               %do v=&startv %to %eval(&startv+&nv1-1);
                  tmp=&&dv&v;
                  %let v2=%eval(&v+&nv1);
                  &&dv&v=&&dv&v2;
                  &&dv&v2=tmp;
               %end;
               estimate=-estimate;
               tvalue=-tvalue;   
            end;
            drop _rowidx;
         run;
      %end;          
      
      %let effv = %eval(&nby + 1);
      %let startv = %eval(&nby + 2);
      %let dlabel = trim(left(trim(left(&&dv&startv));
      %do v=%eval(&startv+1) %to %eval(&startv+&nv1-1);
         %let dlabel = &dlabel || trimn(" " || trim(left(&&dv&v)));
      %end;
      %let dlabel = &dlabel )) || ")-(" || trim(left(trim(left(&&dv&v));
      %do v=%eval(&v+1) %to &nv2;
         %let dlabel = &dlabel || trimn(" " || trim(left(&&dv&v)));
      %end;
      %let dlabel = &dlabel )) || ")" ;
      %let did = &dlabel;
      %let dlabel = "%nrbquote(%LocaleGet(DiffofX_Text)) = (" || &dlabel;
      %let dlabel = %sysfunc(tranwrd(&dlabel, {{^1}}, " || ktrim(kleft(&&dv&effv)) || "));
      %if %upcase(&SplitResults)=YES %then %do;
         %do i=&ndbv %to 1 %by -1;
            %let dlabel = trim(left(&&dbv&i)) || " " || &dlabel;
         %end;
      %end;
      
      %if (%upcase(&UDiffs) = YES) %then %do;
         %let udid = "exp2_" || &dlabel;
         %let udlab = "Exp2 " || &dlabel;
      %end;
      %if (%upcase(&TStats) = YES) %then %do;
         %let tdid = "t_" || &dlabel;
         %let tdlab = "t-Statistic for " || &dlabel;
      %end;
      %if (%upcase(&PValues) = YES) %then %do;
         %let pdid = "p_" || &dlabel;
         %let pdlab = "p-Value for " || &dlabel;
      %end;
 
      %let nlpdid = "nlp_" || &dlabel;
      %let nlpdlab = "-log10(p-value) for " || &dlabel;
      
      data diffs;
         set diffs;
         by &ByVars;
         retain did 0;
         %if %upcase(&SplitResults)=YES %then %do;
            retain did0 1;
         %end;
         %if %index(&LSMeansDiffSet,Simple) %then %do;
            _nchanged = 0;
            %do v=&startv %to %eval(&startv+&nv1-1);
               %let v2 = %eval(&v+&nv1);
               if &&dv&v ne &&dv&v2 then _nchanged = _nchanged + 1;
            %end;
            if _nchanged > 1 then delete;
            else did = did + 1;
         %end;
         %else %do;
            did = did + 1;
         %end;
         if first.&LastByVar=1 then do;
            %if %upcase(&SplitResults)=YES %then %do;
               if first.&LastDBV=1 then do;
                  did0 = did;
               end;
               else do;
                  did = did0;
               end;
            %end;
            %else %do;
               did = 1;
            %end;
         end;
         /*
         did = &did;
         %TrimVarName(did,26);
         */
         dlabel = &dlabel;
         %if (%upcase(&UDiffs) = YES) %then %do;
            %if (%upcase(&WhichProc) = MIXED) %then %do;
               if (Estimate < 0) then do;
                  Exp2Est = -2**(-Estimate);
               end;
               else do;
                  Exp2Est = 2**Estimate;
               end;
               udlab = &udlab;
            %end;
            %else %do;
               %if(%upcase(&LinkFunction) ne LOG) %then %do;
                  if (Estimate < 0) then do;
                     Exp2Est = -2**(-Estimate);
                  end;
                  else do;
                     Exp2Est = 2**Estimate;
                  end;
               %end;
               %else %do;
                  if (Estimate < 0) then do;
                     Exp2Est = -2.718282**(-Estimate);
                  end;
                  else do;
                     Exp2Est = 2.718282**Estimate;
                  end;
               %end;
               udlab = "Exp_e_" || &dlabel;
            %end;
         %end;
         %if (%upcase(&TStats) = YES) %then %do;
            tdlab = &tdlab;
         %end;
         %if (%upcase(&PValues) = YES) %then %do;
            pdlab = &pdlab;
         %end;
         nlpdlab = &nlpdlab;
         if missing(probt) then NegLog10p = .;
         else do; 
            if probt < 10**-300 then NegLog10p = 300;
            else NegLog10p = -log10(Probt);
         end;
      run;
          
      /*
      options notes;
      ods exclude none;
      ods results;
      proc print data=diffs(obs=10);
      run;
      */
      
      /* SS0943396 Any missng value of LSmeans will cause the sequential "did" assignment in previous
      data step mis-aligns data after transpose. Check on the first 5000 rows to re-asign "did" based
      on the unique "dlabel" to avoid the mis-alignment.    
      */ 
      data _diffs_5000;
         set diffs;
         _order_ = _n_;
         if _n_ > 5000 then stop;
         keep dlabel _order_;
      run;
      proc sort data=_diffs_5000 nodupkey;
         by dlabel;
      run;
      proc sort data=_diffs_5000;
         by _order_;
      run;
      data _diffs_5000;
         set _diffs_5000;
         did = _n_;
         drop _order_;
      run;
      data diffs;
         if 0 then set _diffs_5000;
         if _n_ = 1 then do;
            declare hash h(dataset: "work._diffs_5000", hashexp:6);
            h.defineKey("dlabel");
            h.defineData("did");
            h.defineDone();
         end;
         set diffs;
         rc = h.find();
         if (rc = 0) then output;
         drop rc;
      run;

      proc transpose data=diffs out=diffs1(drop=_name_  %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) ne en %then _label_;) prefix=Diff;
         by &ByVars;
         var Estimate;
         id did;
         idlabel dlabel;
      run;
      
      /*
      data outlib.diffst;
         set diffs1;
      run;
      */
      
      * create labels for JSL processing; 
      %LabelTable(_data=diffs1,_drop=&_ByVars,_out=diffsb1,_label=dlabel);
               
      %if (%upcase(&UDiffs) = YES) %then %do;
         proc transpose data=diffs out=diffs1u(drop=_name_ _label_) prefix=Exp2Diff;
            by &ByVars;
            var Exp2Est;
            id did;
            idlabel udlab;
         run;
         %let diffs1u = diffs1u;
      %end;
      %else %do;
         %let diffs1u = ;
      %end;

      %if (%upcase(&TStats) = YES) %then %do;
         proc transpose data=diffs out=diffs1t(drop=_name_ _label_) prefix=TDiff;
            by &ByVars;
            var tValue;
            id did;
            idlabel tdlab;
         run;
         %let diffs1t = diffs1t;
      %end;
      %else %do;
         %let diffs1t = ;
      %end;
      
      %if (%upcase(&PValues) = YES) %then %do;
         proc transpose data=diffs out=diffs1p(drop=_name_ _label_) prefix=PDiff;
            by &ByVars;
            var probt;
            id did;
            idlabel pdlab;
         run;
         %let diffs1p = diffs1p;
      %end;
      %else %do;
         %let diffs1p = ;
      %end;

      proc transpose data=diffs out=diffs2(drop=_name_) prefix=NLPDiff;
         by &ByVars;
         var NegLog10p;
         id did;
         idlabel nlpdlab;
      run;

      * create labels for JSL processing; 
      %LabelTable(_data=diffs2,_drop=&_ByVars,_out=diffsb2,_label=nlpdlab);

      %let DiffData = diffs1 &diffs1u &diffs1t &diffs1p diffs2;
      data diffsb1;
         merge diffsb1 diffsb2;
      run;  
   %end; * end of HasDiff;
   %if (&HasSlice) %then %do;
      * create label from all variables before Estimate;
      proc contents data=slices out=scon noprint;
      run;

      data scon;
         set scon;
         svn = "sv" || trim(left(varnum));
         call symput(svn,trim(left(name)));
         if name = "Effect" then call symput("nsvbeg",trim(left(varnum)));
         if name = "NumDF" then call symput("nsvend",trim(left(varnum-1)));
      run;

      %let pslabel = trim(left(&&sv&nsvbeg));
      %do v=%eval(&nsvbeg+1) %to &nsvend;
         %let pslabel = &pslabel || trimn(" " || "&&sv&v=" || trim(left(&&sv&v)));
      %end;
      %let psid = &pslabel;
      %let pslabel = "ProbF for Slice of " || &pslabel;
      %if %upcase(&SplitResults)=YES %then %do;
         %do i=&ndbv %to 1 %by -1;
            %let pslabel = trim(left(&&dbv&i)) || " " || &pslabel;
         %end;
      %end;

      data slices;
         set slices;
         by &ByVars;
         retain psid 0;
         psid = psid + 1;
         %if %upcase(&SplitResults)=YES %then %do;
            retain psid0 1;
         %end;
         psid = psid + 1;
         if first.&LastByVar=1 then do;
            %if %upcase(&SplitResults)=YES %then %do;
               if first.&LastDBV=1 then do;
                  psid0 = psid;
               end;
               else do;
                  psid = psid0;
               end;
            %end;
            %else %do;
               psid = 1;
            %end;
         end;
         /*
         psid = trim(left(&psid));
         %TrimVarName(psid,25);
         */
         pslabel = trim(left(&pslabel));
      run;

      proc transpose data=slices out=SliceData(drop=_name_ _label_) Prefix=PrF_Sli_;
         by &ByVars;
         var ProbF;
         id psid;
         idlabel pslabel;
      run;

      %let SliceData = SliceData;

      * save transposed first obs for label processing;
      data tmp;
         set SliceData(obs=1);
         keep PrF_Sli_: ;
      run;
      
      proc transpose data=tmp out=tmpt;
      run;
      
      data slicesb1;
         set tmpt;
         pslabel = _label_;
         keep pslabel;
      run;
      
   %end; * end of HasSlice;
%end; * end of HasLsm;

* F tests;
%do i=1 %to 3;
   %let Tests&i.Data = ;
   %if &&HasT&i %then %do;
      proc transpose data=t&i out=T&i.F(drop=_name_ _label_) Prefix=F_T&i._;
         by &ByVars;
         var FValue;
         id Effect;
      run;
      proc transpose data=t&i out=T&i.NDF(drop=_name_ _label_) Prefix=NDF_T&i._;
         by &ByVars;
         var NumDF;
         id Effect;
      run;
      proc transpose data=t&i out=T&i.DDF(drop=_name_ _label_) Prefix=DDF_T&i._;
         by &ByVars;
         var DenDF;
         id Effect;
      run;
      proc transpose data=t&i out=T&i.P(drop=_name_ _label_) Prefix=PrF_T&i._;
         by &ByVars;
         var ProbF;
         id Effect;
      run;
      %if ^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice %then %do;
         data t&i;
            set t&i;
            if missing(ProbF) then NLProbF = .;
            else if (ProbF > 10**(-310)) then NLProbF = -log10(ProbF); /* 1E-310 didn't work on client server. so change it to 10**(-310);*/
            else NLProbF = 310;
         run;
         proc transpose data=t&i out=T&i.NLP(drop=_name_ _label_) Prefix=NLP_T&i._;
            by &ByVars;
            var NLProbF;
            id Effect;
         run;
         %let nlpdata = T&i.NLP;
      %end;
      %else %do;
         %let nlpdata = ;
      %end;
      data T&i.Data;
         merge T&i.F T&i.NDF T&i.DDF T&i.P &nlpdata;
         by &ByVars;
      run;
      %let Tests&i.Data = T&i.Data;
   %end;
%end;

data _mixed_results;
   merge VarCompEsts RSquare fs2 &Tests1Data &Tests2Data &Tests3Data &LsmData &DiffData &EstData &ConData &SliceData;
   by &ByVars;
   %if &vcFlag %then drop Variance_Residual__VC_;
run;

%mixedexit:;

%ClockStop(MixedCode);

%mend MixedCode;



/* ensure a name is of a certain length; if it is too long, truncate component words; */
%macro TrimVarName(vname,len);
   if length(&vname) > &len then do;
      /* count number of words */
      _moretext = 1;
      _i = 1;
      do while(_moretext=1);
         _wi = scan(&vname,_i,'* -.');
         if lengthn(_wi) = 0 then _moretext = 0;
         else _i = _i + 1;
      end;
      _nw = _i - 1;
      if _nw < 1 then _nw = 1;
      
      /* truncate component words in attempt to produce a meaningful name less than len */
      _wl = max(1,floor(&len/_nw)-1);
      _tv = " " %do j=1 %to %eval(&len-1); || " " %end; ;
      _loc = 1;
      _sw = 1;
      if (_wl = 1) and (2*_nw > &len) then _sw = _sw + max(1,floor((2*_nw - &len)/2));
      do _i=_sw to _nw;
         _wi = trim(left((scan(&vname,_i,'* -.'))));
         _wli = length(_wi);
         if (_wli > _wl) then do;
            _wi = substr(_wi,1,_wl);
            _wli = _wl;
         end;
         substr(_tv,_loc,_wli+1) = _wi || "_";
         _loc = _loc + _wli + 1;
      end;
      &vname = _tv;

      /* remove middle portion */
      /*
      _len0 = length(&vname);
      _mid = floor(_len0/2);
      _chop1 = floor((_len0-&len)/2);
      _chop2 = ceil((_len0-&len)/2);
      _vnm1 = substr(&vname,1,_mid-_chop1);
      _vnm2 = substr(&vname,_mid+_chop2+1);
      &vname = trimn(_vnm1) || trimn(_vnm2);
      */
      * drop _len0 _mid _cho1 _chop2 _vnm1 _vnm2;
   end;
%mend TrimVarName;



*******************************;
*****  outlier detection: *****;
*******************************;
%macro OutlierScreen(Method=1,
                     Alpha=,
                     CutOff= ,
                     MaxIteration= ,
                     InputData= ,
                     OutStdRes= ,
                     OutResults= ,
                     OutLComp=
                    );
   * Method - 0. No outlier screening;
   *          1. Multiple outlier screening;
   * Alpha - Familywise error rate;
   * CutOff - Cutoff for Residual Filter;
   * MaxIteration - Maximum number of iteration for outlier screening;
   * InputData - Input data set;
   * OutStdRes - Output standardized residuals data set;
   * OutResults - Output mixed model results data set;
   * OutLComp - Output L components data set;

%put ;
%ClockStart(OutlierScreen Chunk=&_chunk of &nchunk);

%put Residual Outlier Filter Type = &FilterType;

%if &Method %then %do;
   %if %upcase(&FilterType) = CUTOFF %then %do;
      %if ^%length(&Cutoff) %then %let Cutoff = 4;
      %let MaxRes = &CutOff;
   %end;
   %else %do;
      %if ^%length(&Alpha) %then %let Alpha = 0.01;
      %if &WhichProc ne GLIMMIX %then %do;
         %let MaxRes = %str(abs(probit(&Alpha/count/2)));
      %end;
      %else %do;
         %let MaxRes = %str(abs(quantile('T',&Alpha/count/2,_DF_)));
      %end;
   %end;
%end;

* Number of obs of each gene;
data NumGene;
   set &InputData;
   by &ByVars;
   retain count 0;
   if first.&LastByVar=1 then count = 0;
   if response ne . then count = count + 1;
   if last.&LastByVar=1;
   keep &_ByVars count;
run;

* clean up data sets;
proc datasets nolist nodetails;
   delete cp ests lsm t3;
quit;

%let mixederr = 0;
* Run mixed model;
%put Run Proc the first time.;
%MixedCode(dsin=&InputData);
%if (&mixederr=1 | &syserr | &exiterror) %then %goto outlierscreenexit;

data &OutStdRes;
   set stdres;
*   if pred ne .;
run;

data &OutResults;
   set _mixed_results;
*   if PropVar_Residual ne .;
run;

%if %upcase(&LComponents)=YES and (&OutLComp ^= lcomp) %then %do;
   data &OutLComp;
      set lcomp;
   run;
%end;

%put Method = &Method;
%put Alpha = &Alpha;
%put CutOff = &CutOff;
%if &Method %then %put MaxRes = &MaxRes;

%if &Method %then %do;
   %* Check Residuals;
   %let TotOut = 0;
   %let OutFlag = 0;
   data OutTmp;
      merge StdRes NumGene %if &WhichProc=GLIMMIX %then FS(drop=Chi_Sq_Div_DF); 
         END=eof;
      by &ByVars;
      retain _nonMissing_ 0;
      out = 0;
      if response ne . then do;
         _nonMissing_ + 1;
      %if &WhichProc ne GLIMMIX %then %do;
         * apply standardized residual;
         if StdResidual ne . & abs(StdResidual) >= &MaxRes then do;
      %end;
      %else %do;
         * apply studentized residual;
         if StuResidual ne . & abs(StuResidual) >= &MaxRes then do;
      %end;
            response = .;
            out = 1;
            if &OutFlag = 0 then call symput("OutFlag",1);
         end; 
      end;
      if eof then call symput("TotObs",_nonMissing_);
      drop count _nonMissing_;
   run;
   %if ^&OutFlag %then %put No significant large residual found. No further screening will be performed.; 

   %let LoopNo = 0; 
   %do %while (&OutFlag = 1 and &LoopNo < &MaxIteration);
      %put Outlier Screening Loop: %eval(&LoopNo+1);
      data tmp1;
         set OutTmp;
         if out = 1;
         keep &_ByVars out;
      run;

      proc sort data=Tmp1 out=Tmp2(rename=(out=out2)) NoDup;
         by &ByVars;
      run;

      data NewInput;
         merge &InputData tmp2;
         by &ByVars;
         if out2 = 1;
         drop out2 response;
      run; 
      
      data tmp3;
         merge OutTmp tmp2;
         by &ByVars;
         if out2 = 1;
         keep response;
      run;
      
      data NewInput;
         merge NewInput tmp3;
      run;

      %MixedCode(DSIN=NewInput,Loop=%eval(&LoopNo+1));
      %if (&syserr | &exiterror) %then %goto outlierscreenexit;

      * if no valid fitting for all data then step out of screening;
      %if &Flag_no_valid_fit %then %do;
         %put WARNNG: No valid fitting for all the input data. No further screening will be performed.;
         %put The preceding information is from OutlierScreen iteration %eval(&LoopNo + 1).;
         %let LoopNo = &MaxIteration;
         %goto _Skip_Outlier_Screening_;
      %end;

      * check residuals;
      %let OutFlag = 0;
      data OutTmp;
         merge StdRes(in=_SR_) NumGene %if &WhichProc=GLIMMIX %then FS(drop=Chi_Sq_Div_DF);;
         by &ByVars;
         if _SR_;
      %if &WhichProc ne GLIMMIX %then %do;
         * apply standardized residual;
         if abs(StdResidual) >= &MaxRes then do;
      %end;
      %else %do;
         * apply studentized residual;
         if abs(StuResidual) >= &MaxRes then do;
      %end;
            response = .;
            out = 1;
            if &OutFlag = 0 then call symput("OutFlag",1);
         end; 
         else out = 0;
         drop count;
      run;

      * update output data sets;
      data &OutStdRes;
         merge &OutStdRes stdres;
         by &ByVars;
      run;

      data &OutResults;
         merge &OutResults _mixed_results;
         by &ByVars;
      run;

      %if %upcase(&LComponents) = YES %then %do;
         data &OutLComp;
            merge &OutLComp lcomp;
            by &ByVars;
         run;
      %end;

      %if ^&OutFlag %then %put No significant large residual found. No further screening will be performed.; 
      %let LoopNo = %eval(&LoopNo + 1);
      %put The preceding information is from OutlierScreen iteration &LoopNo .;

      %_Skip_Outlier_Screening_:;
   %end;

   data tmp;
      set &OutStdRes;
      if response = .;
   run;

   data _null_;
      set tmp END=EOF;
      if EOF then call symput("TotOut",_n_);
   run;

   %let OutRate = %sysevalf(&TotOut/&TotObs);
   %put The fraction of screened outliers is &OutRate .;
%end;

%if %length(&MMEIData) %then %do;
   %MMEI;
%end;

* convert OutStdRes to tall format;
%if (%upcase(&PlotResiduals)=YES) and ^&AlreadyStacked %then %do;
   %let StdResVarNames = ;
   %let StdResVarLabels = ;
   %if &WhichProc = GLIMMIX %then %do;
      %let StuResVarNames = ;
      %let StuResVarLabels = ;
   %end;
   %let PredVarNames = ;
   %let PredVarLabels = ;

   %do _c_=1 %to &_nTallCol_;
      %let StdResVarNames = &StdResVarNames StdRes&_c_;
      %let StdResVarLabels = &StdResVarLabels StRes_&&_ColName_&_c_._ ||;
      %if &WhichProc = GLIMMIX %then %do;
         %let StuResVarNames = &StuResVarNames StuRes&_c_;
         %let StuResVarLabels = &StuResVarLabels StuRes_&&_ColName_&_c_._ ||;
      %end;
      %let PredVarNames = &PredVarNames Pred&_c_;
      %let PredVarLabels = &PredVarLabels Pred_&&_ColName_&_c_._ ||;
   %end;


   %if &WhichProc ne GLIMMIX %then %do;
      %Stack_to_Tall(_InData=&OutStdRes,_InTallData=&InChunk,_Out=&OutStdRes,_Responses=StdResidual Pred,
         _VarLists=StdResVarNames PredVarNames,_LabLists=StdResVarLabels PredVarLabels);
   %end;
   %else %do;
      %Stack_to_Tall(_InData=&OutStdRes,_InTallData=&InChunk,_Out=&OutStdRes,_Responses=StuResidual Pred,
         _VarLists=StuResVarNames PredVarNames,_LabLists=StuResVarLabels PredVarLabels);
   %end;
%end; 

* stop timing;
%if &Method %then %ClockStop(OutlierScreen,_Notes="The fraction of screened outliers is &OutRate");
            %else %ClockStop(OutlierScreen,_Notes="No outlier screened");

%outlierscreenexit:;

%mend OutlierScreen;





%macro MixedCreateJSL;

%ClockStart(MixedCreateJSL);

%if ^%symexist(mixed_namelen) %then %let mixed_namelen = 24;
%if ^%symexist(HasT1) %then %let HasT1 = 0;
%if ^%symexist(HasT2) %then %let HasT2 = 0;
%if ^%symexist(LComponents) %then %let LComponents = no;

* create macro variables to use in JSL code;
%if %upcase(&SplitResults)=YES %then %do;
   %let i = 1;
   %let TotLvls = 0;
   %do %while(%length(%scan(&DesignByVars,&i)));
      %let dbv&i = %scan(&DesignByVars,&i);
      %CheckLabel(ExpInLib.&ExpIndata,dbv&i);
      %if &i = 1 %then %let jslDBV = Column(""%str(&&dbv&i)"");
      %else %let jslDBV = &jslDBV , Column(""%str(&&dbv&i)"");
      %global DBVList&i DBVnL&i DBVType&i;
      %let DBVList&i = ;
      %GetLevels(OutLib.&ExpIndata,%scan(&DesignByVars,&i),
                 DBVList&i,_nLvls=DBVnL&i,_vType=DBVType&i);
      %if &i = 1 %then %let TotLvls = &&DBVnL&i;
      %else %let TotLvls = %eval(&TotLvls+&&DBVnL&i);
      %let i = %eval(&i+1);
   %end;
   %let ndbv = %eval(&i-1);
   %if &ndbv>1 %then %do;
      %let FirstDBV = %scan(&DesignByVars,1);
      %let LastDBV = %scan(&DesignByVars,&ndbv);
   %end;
   %else %do;
      %let FirstDBV = &DesignByVars;
      %let LastDBV = &DesignByVars;
   %end;

   proc sort data=OutLib.&ExpIndata out=_exp_ nodupkey;
      by &DesignByVars;
   run;

   data _null_;
      set _exp_ end=_e_;
      %do i = 1 %to &ndbv;
         name = "Ftr&i._Lvl" || trim(left(_n_));
         call symput(name,trim(left(%scan(&DesignByVars,&i))));
      %end;
      if _e_ then call symput('TotalCLvl_DBV',trim(left(_n_)));
   run;
   %put TotalCLvl_DBV = &TotalCLvl_DBV;
%end;

%if &HasLsm %then %do;
   %if %upcase(&SplitResults) ne YES %then %do;
      data _null_;
         set lsmb1 end=_last;
         macvar = "llabel" || trim(left(_n_));
         call symput(macvar,trim(left(llabel)));
         if _last then call symput("nLsm",trim(left(_n_)));
      run;
      %if (&nLsm = 1) %then %let HasLsm = 0;
      %else %if (&nLsm = 2) %then %let LSMeanStdMethod = ;
   %end;
   %else %do;
      data _null_;
         set lsmb1 end=_end;
         by &DesignByVars;
         retain _c_ _nDBV_ 0;
         if first.&LastDBV then do;
            _c_ = 1;
            _nDBV_ = _nDBV_ + 1;
         end;
         else do;
            _c_ = _c_ + 1;
         end;
         macvar = "llabel" || trim(left(_nDBV_)) ||'_'|| trim(left(_c_));
         call symput(macvar,llabel);
         macvar2 = "nLsm" || trim(left(_nDBV_));
         if last.&LastDBV then call symput(macvar2,trim(left(_c_)));
         if (_end) then call symput("nldbv",trim(left(_nDBV_)));
      run;
      %put nldbv = &nldbv;
      %let all1 = 1;
      %let all2 = 1;
      %do i=1 %to &nldbv;
         %if &&nLsm&i ^= 1 %then %let all1 = 0;
         %if &&nLsm&i ^= 2 %then %let all2 = 0;
      %end;
      %if (&all1) %then %let HasLsm = 0;
      %else %if (&all2) %then %let LSMeanStdMethod = ;
   %end;
%end;
%else %do;
   %let nLsm = 0;
%end;
%if &HasLsm and %length(&LSMeanStdMethod) %then %do;
   %if %upcase(&SplitResults) ne YES %then %do;
      data _null_;
         set lsmsb1;
         macvar = "lslabel" || trim(left(_n_));
        call symput(macvar,trim(left(lslabel)));
      run;
   %end;
   %else %do;
      data _null_;
         set lsmsb1;
         by &DesignByVars;
         retain _c_ _nDBV_ 0;
         if first.&LastDBV then do;
            _c_ = 1;
            _nDBV_ = _nDBV_ + 1;
         end;
         else do;
            _c_ = _c_ + 1;
         end;
         macvar = "lslabel" || trim(left(_nDBV_)) ||'_'|| trim(left(_c_));
         call symput(macvar,trim(left(lslabel)));
      run;
   %end;
%end;

%if (&HasDiff | &HasSlice | &HasEst | &HasCon) %then %do;
   %if &HasDiff %then %do;
      data _null_;
         set diffsb1 end=_last;
         macvar = "dlabel" || trim(left(_n_));
         call symput(macvar,trim(left(dlabel)));
         macvar = "nlpdlab" || trim(left(_n_));
         call symput(macvar,trim(left(nlpdlab)));
         if _last then call symput("nDiff",trim(left(_n_)));
      run;
   %end;
   %else %do;
      %let nDiff = 0;
   %end;
   %if &HasSlice %then %do;
      data _null_;
         set slicesb1 end=_last;
         macvar = "pslabel" || trim(left(_n_));
         call symput(macvar,trim(left(pslabel)));
         if _last then call symput("nSlice",trim(left(_n_)));
      run;
   %end;
   %else %do;
      %let nSlice = 0;
   %end;
   %if &HasCon %then %do;
      data _null_;
         set consb1 end=_last;
         macvar = "pclabel" || trim(left(_n_));
         call symput(macvar,trim(left(pclabel)));
         if _last then call symput("nCon",trim(left(_n_)));
      run;
   %end;
   %else %do;
      %let nCon = 0;
   %end;
   %if &HasEst %then %do;
      data _null_;
         set estsb1 end=_last;
         macvar = "elabel" || trim(left(_n_));
         call symput(macvar,trim(left(elabel)));
         macvar = "nlpelab" || trim(left(_n_));
         call symput(macvar,trim(left(nlpelab)));
         if _last then call symput("nEst",trim(left(_n_)));
      run;
   %end;
   %else %do;
      %let nEst = 0;
   %end;
   %put nDiff = &nDiff;
   %put nSlice = &nSlice;
   %put nCon = &nCon;
   %put nEst = &nEst;
%end;
%else %do;
   %if %length(&MultipleTestingFixedEffects) %then %do;
      /*
      data mte;
         length effect$ 100;
         one = 1;
         %let i = 1;
         %do %while(%length(%scan(&MultipleTestingFixedEffects,&i,' ')));
            effect = "%scan(&MultipleTestingFixedEffects,&i,' ')";
            output;
            %let i = %eval(&i + 1);
         %end;
      run;
      proc transpose data=mte out=mtet(drop=_name_) prefix=PrF_T3_;
         var one;
         id Effect;
      run;
      */
      
      * get effect names by running proc mixed, covers the case where long effect names get shortened;
      %let cvars = ;
      %let i= 1;
      %do %while(%length(%scan(&MultipleTestingFixedEffects,&i,' *()|@')));
         %let cvar&i = %scan(&MultipleTestingFixedEffects,&i,' *()|@');
         %let cvars = &cvars &&cvar&i;
         %let i = %eval(&i+1);
      %end;
      %let ncvars = %eval(&i-1);
      
      data dummy;
         do i=1 to 50;
            dummy = rannor(0);
            %do i=1 %to &ncvars;
               &&cvar&i = ranbin(0,1,0.5);
            %end;
            output;
         end;
      run;
      
      ods exclude all;
      ods noresults;
      proc mixed data=dummy namelen=&mixed_namelen;
         class &cvars;
         model dummy = &MultipleTestingFixedEffects;
         ods output tests3=t3m;
      run;
      ods exclude none;
      ods results;
      
      proc transpose data=t3m out=mtet(drop=_name_ _label_) Prefix=PrF_T3_;
         var ProbF;
         id Effect;
      run;
      %MakeVarList(_data=mtet,_nvar=nFstat,_prefix=PrF_T3_);

      data t3m;
         set t3m;
         if (ProbF > 10**(-310)) then NLProbF = -log10(ProbF); /* 1E-310 didn't work on client server. so change it to 10**(-310);*/
         else NLProbF = 310;
      run;
      proc transpose data=t3m out=mtet(drop=_name_ _label_) Prefix=NLP_T3_;
         var NLProbF;
         id Effect;
      run;
      %MakeVarList(_data=mtet,_nvar=nNLP,_prefix=NLP);
   %end;
   %else %do;
      %MakeVarList(_data=&OutData,_nvar=nFstat,_prefix=PrF_T3_);
      %if &HasT1 or &HasT2 or &HasT3 %then %do;
         %MakeVarList(_data=&OutData,_nvar=nNLP,_prefix=NLP);
      %end;
   %end;
%end;

/*
%MakeVarList(_data=&OutData,_nvar=nVariance,_prefix=PropVar);
*/

%MakeVarList(_data=&OutData,_nvar=nVariance,_prefix=Variance_);
/*
%if &WhichProc = GLIMMIX %then %do;
   * replace Variance_Residual by Chi_Sq_Div_DF;
   %let Variance_&nVariance = Chi_Sq_Div_DF;
%end;
*/
%if ^%length(&MultipleTestingMethod) and ^%length(&NegLog10pCutoff) %then %do;
   %let MultipleTestingMethod = AdaptiveHolm;
%end;
%put MultipleTestingMethod = &MultipleTestingMethod;

*adjustment for separate tests;
%if ^%symexist (indTestAdj) %then %let indTestAdj=no;

* determine multiple testing p-value cutoff;
%if %length(&MultipleTestingMethod) and (&HasDiff | &HasEst | &HasCon | &HasSlice | &HasT3) %then %do;
   %if %length(&NegLog10pCutoff) %then 
      %put NOTE: Both MultipleTestingMethod and NegLog10pCutoff are specified. Only MultipleTestingMethod will be applied.;
   %if ^%length(&Alpha) %then %let Alpha = 0.05;
   %let alphaerror = 0;
   data _null_;
      if &Alpha <= 0 or &Alpha >= 1 then call symput("alphaerror",left(trim(1)));
   run;
   %if &alphaerror = 1 %then %do;
      %put ERROR: Invalid value for alpha.;
   %end;
   
   * create p-value data set by stacking variables;
   data tmpOutData; *dummy data set for extracting variable information;
      set &OutData;
      if _n_ > 1 then stop;
   run;
   %MakeVarList(_data=tmpOutData,_nvar=_d_,_prefix=NLPD,_labelflag=1);
   %MakeVarList(_data=tmpOutData,_nvar=_e_,_prefix=NLPE,_labelflag=1);
   %MakeVarList(_data=tmpOutData,_nvar=_c_,_prefix=PrF_Con_,_labelflag=1);
   %MakeVarList(_data=tmpOutData,_nvar=_s_,_prefix=PrF_Sli_,_labelflag=1);
   %MakeVarList(_data=tmpOutData,_nvar=nFstat,_prefix=PrF_T3_,_labelflag=1);

   data pd;
      set &OutData;
      length _Name_ $32 _Label_ $100;
      %if (&HasDiff | &HasEst | &HasCon) %then %do;
         %if (&HasDiff) %then %do;
            array npd[&_d_] %do i = 1 %to &_d_; &&NLPD&i %end; ;
            %do i = 1 %to &_d_;
               raw_p = 10**(-npd[&i]);
               _Name_ = "Adj_" || substr("&&NLPD&i",4);
               %if %length(&&NLPD_label_&i) %then %do;
                  _Label_ = "Adjusted p-Value" || substr("&&NLPD_label_&i",16);
               %end;
               %else %do;
                  _Label_ = _Name_;
               %end;
               _ColumnOrder_ = &i;
               _RowOrder_ = _n_;
               output;
            %end;
         %end;
         %if (&HasEst) %then %do;
            array npe[&_e_] %do i = 1 %to &_e_; &&NLPE&i %end; ;
            %do i = 1 %to &_e_;
               raw_p = 10**(-npe[&i]);
               _Name_ = "Adj_" || substr("&&NLPE&i",4);
               %if %length(&&NLPE_label_&i) %then %do;
                  _Label_ = "Adjusted p-Value" || substr("&&NLPE_label_&i",16);
               %end;
               %else %do; 
                  _Label_ = _Name_;
               %end;
               _ColumnOrder_ = &i;
               _RowOrder_ = _n_;
               output;
            %end;
         %end;
         %if (&HasCon) %then %do;
            array npc[&_c_] %do i = 1 %to &_c_; &&PrF_Con_&i %end; ;
            %do i = 1 %to &_c_;
               raw_p = npc[&i];
               _Name_ = "Adj_&&PrF_Con_&i";
               %if %length(&&PrF_Con__label_&i) %then %do; 
                  _Label_ = "Adjusted &&PrF_Con__label_&i";
               %end;
               %else %do;
                  _Label_ = _Name_;
               %end;
               _ColumnOrder_ = &i;
               _RowOrder_ = _n_;
               output;
            %end;
         %end;
      %end;
      %else %if &HasSlice %then %do;
         array nps[&_s_] %do i = 1 %to &_s_; &&PrF_Sli_&i %end; ;
         %do i = 1 %to &_s_;
            raw_p = nps[&i];
            _Name_ = "Adj_&&PrF_Sli_&i";
            %if %length(&&PrF_Sli__label_&i) %then %do;
               _Label_ = "Adjusted &&PrF_Sli__label_&i";
            %end;
            %else %do;
               _Label_ = _Name_;;
            %end;
            _ColumnOrder_ = &i;
            _RowOrder_ = _n_;
            output;
         %end;
      %end;
      %else %do;
         array npt[&nFstat] %do i=1 %to &nFstat; &&PrF_T3_&i %end; ;
         %do i=1 %to &nFstat;
            raw_p = npt[&i];
            _Name_ = "Adj_&&PrF_T3_&i";
            %if %length(&&PrF_T3__label_&i) %then
               _Label_ = "Adjusted &&PrF_T3__label_&i";
            %else 
               _Label_ = _Name_;;
            _ColumnOrder_ = &i;
             _RowOrder_ = _n_;
            output;
         %end;
      %end;
      keep raw_p _Name_ _Label_ _ColumnOrder_ _RowOrder_;
   run;

   %include "&MacroPath.\MultipleTestingAdjustment.sas";

   %if &indTestAdj=yes %then %do;
      %SliceData (pd,_Name_,SnameLvl,%sysfunc(pathname(work)),_SubDataList=sList);
      %AdjPValueName(method=&MultipleTestingMethod);
      %NObsVars(SnameLvl);
      %let nmtests = &nobs;
      
      data tmp;
         set pd;
         _cOrder_ = _n_;
         if _RowOrder_ > 1 then stop;
         keep _Name_ _cOrder_;
      run;

      proc sort data=tmp;
         by _Name_;
      run;
      
      data _null_;
         set tmp;
         length name $20;
         name  = "nlps" || trim(left(_n_));
         call symput(name,"NegLog10pCutoff" || trim(left(_cOrder_)));
      run;
      
      %do _i_ =1 %to &nmtests;
         %MultipleTestingAdjustment(data=pd&_i_,method=&MultipleTestingMethod,out=opd);
      
         %put Alpha = &Alpha;
         %put ntests = &ntests;

         * determine cutoff for raw p-values empirically from the adjusted p-values, use Bonferroni as upper bound;
         data _null_;
            cutoff = -log10(&Alpha/&ntests);
            call symput("NegLog10pCutoff",trim(left(round(cutoff,0.01))));
         run;
   
         %put Initial -log10 cutoff (Bonferroni) = &NegLog10pCutoff;

         data _null_;
            set opd;
            if not missing(&mtpvar) and (&mtpvar <= &Alpha) then do;
               cutoff = -log10(raw_p);
               /*
               call symput("NegLog10pCutoff",trim(left(round(cutoff,0.01)))); *can have rounding issue;
               */
               call symput("NegLog10pCutoff",trim(left(cutoff)));
               stop;
            end;
         run;
         %let &&nlps&_i_= &NegLog10pCutoff;
         %*put i=&_i_,NegLog10pCutoff&_i_ = &&nlps&_i_, nlp=&&&nlps&_i_;
         %put NegLog10pCutoff = &NegLog10pCutoff;

         %if %upcase(&AdjPValues)=YES %then %do;
            * transform stack data into tall data after multiple testing adjustment; 
            proc sort data=opd;
               by _RowOrder_ _ColumnOrder_;
            run;
      
            proc transpose data=opd out=topd(drop=_RowOrder_ _NAME_ _Label_);
               var &mtpvar;
               id _Name_;
               idlabel _label_;
               by _RowOrder_;
            run;
   
            data &OutData;
               set &OutData;
               set topd;
            run;
         %end;
      %end;
   %end;
   %else %do;
      %AdjPValueName(method=&MultipleTestingMethod);
      %MultipleTestingAdjustment(data=pd,method=&MultipleTestingMethod,out=opd);
      %*if &exiterror %then %goto mcjexit;

      %put Alpha = &Alpha;
      %put ntests = &ntests;

      * determine cutoff for raw p-values empirically from the adjusted p-values, use Bonferroni as upper bound;
      data _null_;
         cutoff = -log10(&Alpha/&ntests);
         call symput("NegLog10pCutoff",trim(left(round(cutoff,0.01))));
      run;
   
      %put Initial -log10 cutoff (Bonferroni) = &NegLog10pCutoff;

      data _null_;
         set opd;
         if not missing(&mtpvar) and (&mtpvar <= &Alpha) then do;
            cutoff = -log10(raw_p);
            /*
            call symput("NegLog10pCutoff",trim(left(round(cutoff,0.01)))); *can have rounding issue;
            */
            call symput("NegLog10pCutoff",trim(left(cutoff)));
            stop;
         end;
      run;
      %put NegLog10pCutoff = &NegLog10pCutoff;

      %if %upcase(&AdjPValues)=YES %then %do;
         * transform stack data into tall data after multiple testing adjustment; 
         proc sort data=opd;
            by _RowOrder_ _ColumnOrder_;
         run;

         proc transpose data=opd out=topd(drop=_RowOrder_ _NAME_ _Label_);
            var &mtpvar;
            id _Name_;
            idlabel _label_;
            by _RowOrder_;
         run;

         data &OutData;
            set &OutData;
            set topd;
         run;
      %end;
   %end;
%end;

/* subset output for EZ wizard */
%if (&HasLSM) and (&HasDiff) %then %do;
%*if %symexist(SigDataFlag) %then %do;
   %*if &SigDataFlag %then %do;
      %if %length(%trim(&OutDataName)) > 28 %then 
         %let _name = %substr(&OutDataName,1,28);
      %else %let _name = &OutDataName;
      data Outlib.&_name._sig;
         set OutLib.&OutDataName;
         array _sig {*} NLPDiff:;
         _flag = 0;
         do _i = 1 to dim(_sig);
            if _sig(_i) > &NegLog10pCutoff then do;
               _flag = 1;
               _i = dim(_sig);
            end;
         end;
         if _flag;
         drop _flag _i;
      run;
   %*end;
%*end;
%end;

* Find the min and max y-coordinate in volcanos;
%let nNLPs = 0;

%if (&HasDiff | &HasEst) & (%upcase(&VolcanoUnifiedY)=YES) %then %do;
   %let ds = %sysfunc(open(&OutData));
   %do i = 1 %to %sysfunc(attrn(&ds,NVARS));
      %let ckVar = %upcase(%sysfunc(varname(&ds,&i)));
      %if %length(&ckVar) >= 3 %then %do;
         %if %qsubstr(&ckVar,1,3) = NLP %then %do;
            %let nNLPs = %eval(&nNLPs+1);
            %let NLPVar = &ckVar;
         %end;
      %end;
   %end;
   %let rc=%sysfunc(close(&ds));
   %put nNLPs = &nNLPs;
   
   %if (&nNLPs > 0) %then %do;
      proc means data=&OutData max noprint;
         var NLP:;
         output out=o;
      run;

      data _null_;
         set o(where=(_STAT_='MAX'));
         %if (&nNLPs > 1) %then %do;
            max = max(of NLP:);
         %end;
         %else %do;
            max = &NLPVar;
         %end;
         call symputx('NLG_max',ceil(max*10.5)/10);
         call symputx('NLG_min',ceil(max*-0.5)/10);
         if max <= 5 then call symputx('NLG_inc',0.5);
         else if max <= 10 then call symputx('NLG_inc',1);
         else if max <= 100 then do;
            if max/ceil(max/10) < 8 then call symputx('NLG_inc',floor(max/10));
           else call symputx('NLG_inc',ceil(max/10));
         end;
         else do;
           if max/(ceil(max/100)*10) < 8 then call symputx('NLG_inc',floor(max/100)*10);
           else call symputx('NLG_inc',ceil(max/100)*10);
        end;
      run;
      %put NLG_min = &NLG_min;
      %put NLG_max = &NLG_max;
      %put NLG_inc = &NLG_inc;
   %end;
%end;

* extract effects for drill down;
%global nfef maxelen;
%let nfef = 0;
%let maxelen = 0;
%put HasT3 = &HasT3;
%if &HasT3 %then %do;
   data t3check;
      set t3;
      if _n_ > 1 and (&LastByVar ne lag(&LastByVar)) then do;
         stop;
      end;
   run;
   data t3check;
      set t3check end=_last;
      retain maxelen 0;
      maxelen = max(maxelen,length(trim(effect)));
      call symputx('fef'||trim(left(_n_)),effect);
      if _last then do;
         call symputx('nfef',trim(left(_n_)));
         call symputx('maxelen',trim(left(maxelen)));
      end;
   run;
%end;
%put nfef = &nfef;
%do i=1 %to &nfef;
   %put fef&i = &&fef&i;
%end;
%put maxelen = &maxelen;
%put mixed_namelen = &mixed_namelen;

%if ^%symexist(InClassVars) %then %let InClassVars = ;
%let AllClassVars = &ClassVars &InClassVars;

   * save truncated names;
   %do i=1 %to &nfef;
      %let feft&i = &&fef&i;
   %end;
/* if maximum effect name length is achieved, run another mixed model with larger namelen to
   avoid truncated effect names in drill down */
%if &maxelen = &mixed_namelen %then %do;

   * set up class vars;
   %let cvars = ;
   %let i = 1;
   %do %while(%length(%scan(&FixedEffects,&i,' *()|@')));
      %let cvar&i = %scan(&FixedEffects,&i,' *()|@');
      %let cvars = &cvars &&cvar&i;
      %let i = %eval(&i+1);
   %end;
   %let ncvars = %eval(&i-1);

   data dummy;
      do i=1 to 50;
         dummy = rannor(7483201);
         %do i=1 %to &ncvars;
            &&cvar&i = ranbin(0,1,0.5);
         %end;
         output;
      end;
   run;

   %put Fixed effects to parse = &FixedEffects;
   ods exclude all;
   ods noresults;
   * 200 is the max allowed value of namelen;
   proc mixed data=dummy namelen=200;
      class &cvars;
      model dummy = &FixedEffects;
      ods output tests3=t3t;
   run;
   ods exclude none;
   ods results;
   
   data _null_;
      set t3t end=last_one;
      retain maxelen 0;
      maxelen = max(maxelen,length(trim(effect)));
      call symputx('fef'||trim(left(_n_)),effect);
      if last_one then do;
         call symputx('nfef',_n_);
         call symputx('maxelen',maxelen);
         stop;
      end;
   run;
   
   %put nfef = &nfef;
   %put maxelen = &maxelen;
   %do i=1 %to &nfef;
      %put effect&i = &&fef&i;
   %end;
%end; 

%if ^%symexist(nDiff) %then %let nDiff = 0;
%if ^%symexist(nEst) %then %let nEst = 0;

%if %index(&ProcessName,OneWay) & &nDiff > 0  & %length(&ChrVar) & (%upcase(&JournalChrom) = YES) %then %do; 
   %PrepOutFile(jslfile,jslfilename,&ProcessName Script for &sname..jsl);
%end;
%else %if %index(&ProcessName,FindingsANOVA) %then %do;
   %PrepOutFile(jslfile,jslfilename,&StudyName &ProcessName Script for &FDomain..jsl);
%end;
%else %do;
   %PrepOutFile(jslfile,jslfilename,&ProcessName Script for &OutDataName..jsl);
%end;

%if ^%symexist(ChrVar) %then %let ChrVar = ;
%if ^%symexist(ChrVar0) %then %let ChrVar0 = &ChrVar;
%if ^%symexist(PosVar) %then %let PosVar = ;
%if ^%symexist(PosVar0) %then %let PosVar0 = &PosVar;
%if ^%symexist(JournalChrom) %then %let JournalChrom = no;
%put ChrVar = &ChrVar;
%put PosVar = &PosVar;
%put JournalChrom = &JournalChrom;

%global trSigIndex;
%let trSigIndex = %nrbquote(%LocaleGet(SigIndex_Text));

* separate and journal results by chromosome;
%if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;

   * find unique values of chromosome variable;
   data cvu;
      set &OutData;
      keep &ChrVar;
   run;
   
   proc sort data=cvu nodupkey;
      by &ChrVar;
   run;
   
   data cvu;
      set cvu;
      if &ChrVar = "chr1" then _chrvar = "chr01";
      else if &ChrVar = "chr2" then _chrvar = "chr02";
      else if &ChrVar = "chr3" then _chrvar = "chr03";
      else if &ChrVar = "chr4" then _chrvar = "chr04";
      else if &ChrVar = "chr5" then _chrvar = "chr05";
      else if &ChrVar = "chr6" then _chrvar = "chr06";
      else if &ChrVar = "chr7" then _chrvar = "chr07";
      else if &ChrVar = "chr8" then _chrvar = "chr08";
      else if &ChrVar = "chr9" then _chrvar = "chr09";
      else _chrvar = &ChrVar;
   run;
   
   proc sort data=cvu;
      by _chrvar;
   run;
   
   data _null_;
      set cvu end=last_one;
      retain maxlen 0;
      len = lengthn(&ChrVar);
      if (len > maxlen) then maxlen = len;
      call symputx("chr"||trim(left(_n_)),trim(left(&ChrVar)));
      if last_one then do;
         call symputx("maxlenchr",trim(left(maxlen)));
         call symputx("nchr",trim(left(_n_)));
         call symputx("typechr",vtype(&ChrVar));;
      end;
   run;
   
   %put nchr = &nchr;
   %put maxlenchr = &maxlenchr;
   %put typechr = &typechr;
      
   * create separate output data sets;
   %let maxon = %eval(31-&maxlenchr);
   %if %length(%trim(&OutDataName)) > &maxon %then 
      %let _name = %substr(&OutDataName,1,&maxon);
   %else %let _name = &OutDataName;
   %do i=1 %to &nchr;
      %let OName&i=&_name._&&chr&i;
      %CheckDataName(_name=OName&i);
      data OutLib.&&OName&i;
         set &OutData;
         %if (&typechr = N) %then %do;
            if &ChrVar = &&chr&i;
         %end;
         %else %do;
            if &ChrVar = "&&chr&i";
         %end;
      run;
   %end;

   %if (%upcase(&PlotResiduals) = YES) %then %do; 
      %if %length(%trim(&OutDataResName)) > &maxon %then 
         %let _resname = %substr(&OutDataResName,1,&maxon);
      %else %let _resname = &OutDataResName;
      %do i=1 %to &nchr;
         %let RName&i=&_resname._&&chr&i;
         %CheckDataName(_name=RName&i);
         data OutLib.&&RName&i;
            set &OutDataRes;
            %if (&typechr = N) %then %do;
               if &ChrVar = &&chr&i;
            %end;
            %else %do;
               if &ChrVar = "&&chr&i";
            %end;
         run;
      %end;
   %end;

   %if %upcase(&LComponents)=YES %then %do;
      %if %length(%trim(&OutDataCompName)) > &maxon %then  %do;
         %if %substr(&OutDataCompName,1,&maxon) = &_name %then %do;
            %let _compname = %substr(&OutDataCompName,1,&maxon-5)%substr(&OutDataCompName,%length(&OutDataCompName)-3,4);

         %end;
      %end;
      %else %let _compname = &OutDataCompName;
      %do i=1 %to &nchr;
         %let CName&i=&_compname._&&chr&i;
         %CheckDataName(_name=CName&i);
         data OutLib.&&CName&i;
            set &OutDataComp;
            %if (&typechr = N) %then %do;
               if &ChrVar = &&chr&i;
            %end;
            %else %do;
               if &ChrVar = "&&chr&i";
            %end;
         run;
      %end;
   %end;
   
   %global sname;
   * create a summary table of sig indices for the graph by &ChrVar;
   %if (&nDiff > 0) %then %do;
      data sigind;
         set OutLib.&OutDataName;
         *array pvalvars{*} &NegLog10p;
         array neglogp {*} NLP:;
         array sigIndex{&nDiff};
         do _i=1 to &nDiff;
            if missing(neglogp[_i]) then sigIndex[_i]=.;
            else if (neglogp[_i] > &NegLog10pCutoff) then sigIndex[_i]=1;
            else sigIndex[_i]=0;
         end;
      run;

      data  sigind2;
         set sigind;
         keep &ChrVar sigIndex1-sigIndex&nDiff; 
      run;

      proc sort data=sigind2;
         by &ChrVar;
      run;

      %if %length(&_name) > 23 %then %let sname = %substr(&_name,1,23);
      %else %let sname = &_name.&AnnoChrVar;
      %if %length(&sname) > 28 %then %let sname = %substr(&8name,1,28);
      
      proc means data=sigind2 noprint;
         var sigIndex1-sigIndex&nDiff;
            by &ChrVar;
         output out=OutLib.&sname._sum sum(sigIndex1-sigIndex&nDiff)=
            %do i=1 %to &nDiff; sigIndex&i %end; ;
      run;
      
      data OutLib.&sname._sum;
         set OutLib.&sname._sum;
         %do i=1 %to &nDiff; 
            /*
            %let a=%scan(&&dlabel&i,1,')');
            %let b&i=%scan(%bquote(&a),2,'(');
            rename sigIndex&i=&&b&i;
            */
            label sigIndex&i="&trSigIndex %substr(&&nlpdlab&i,17)";
         %end;
      run;

      /*------ get top ranked samples for summary graph----*/  
      %if &nDiff >= 50 %then %do;

         proc means data =OutLib.&sname._sum sum noprint;
         /* var &&b1-&&b&nDiff;*/
            output out=pp1 sum= ;
         run;

         data pp1;
            set pp1;
            &ChrVar = "S";
         run;


         proc transpose data=pp1
            out=pp2;
         run;

         proc sort data=pp2;
            by descending COL1 ;
         run;

         proc transpose data=pp2
                  out=pp3;
         run;
         data pp3;
            set pp3;
            &ChrVar = "S";
            drop _Name_ _FREQ_ _TYPE_;
         run;  
         
         /* 
         data OutLib.&sname._sum2;
            set pp3 OutLib.&sname._sum ;
         run;
         data OutLib.&sname._sum2(where=(&ChrVar ne "S"));
            set OutLib.&sname._sum2;
         run;
         */

         %let ssid = %sysfunc(open(pp3));

         %do _s_=1 %to 50;
            %let ssname&_s_ = %sysfunc(varname(&ssid,&_s_));
            %put ssname&_s_=&&ssname&_s_;
         %end;
         %let src = %sysfunc(close(&ssid));
      %end;
   /*--------------------------*/   
   %end;
   %else %do;
      %let sname = ;
   %end;
   
   %if ^%symexist (indTestAdj) %then %let indTestAdj=no;
   
   * macro names for journal and secondary script;
   %let jrnfile = ;
   %if %index(&ProcessName,OneWay) & &nDiff > 0  %then %do;
      %PrepOutFile(jrnfile,jrnfilename,&ProcessName Journal for &sname &_nThePro_..jrn);
   %end;
   %else %do;
      %PrepOutFile(jrnfile,jrnfilename,&ProcessName Journal for &InData &_nThePro_..jrn);
   %end;
   %let jslfile2 = ;
   %PrepOutFile(jslfile2,jslfile2name,&ProcessName Secondary Script &_nThePro_..jsl);
   %let jslfile3 = ;
   %PrepOutFile(jslfile3,jslfile3name,Main Tabbed Script for Secondary &_nThePro_..jsl);
   
   * main script now just opens the journal;
   data _null_;
      file "&JSLFile" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newnamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";    
      * main results;
      put "open(""&ClientOutPath.&jrnfilename"");";
      put "objRefNS:outds = ""&OutDataName..sas7bdat"";";
      put "objRefNS:cutoff = &NegLog10pCutoff;";   
      
      * summary bar charts of number of differences;
      %if &nDiff > 0 %then %do;
      
         %if &nDiff > 50 %then %let nDiffShow = 50;
         %else %let nDiffShow = &nDiff;
      
         put "objRefNS:data=open(""&ClientOutPath.&sname._sum.sas7bdat"",invisible);";;
         put "foundit=Try(ccol = Column(""&ChrVar"");1,0);";
         put "if(foundit,ccol << Set Property(""Value Ordering"",{""chr1"",""chr2"",""chr3"",""chr4"",""chr5"",";
         put "   ""chr6"",""chr7"",""chr8"",""chr9""});";
         put "   ccol << label(1));";
         put "found1 = 0;";
         %do i = 1 %to &nDiff;
            put "if(ColSum(:Name(""&trSigIndex %substr(&&nlpdlab&i,17)""))>0,found1=1);";
         %end;
      
         put "if(found1==0,";
         put "   Dialog(title(""Message""),""No significant differences were found at cutoff = &NegLog10pCutoff.""),";
         put "   mmchart=Chart(";      
         put "    X( :&ChrVar ),";
         put "    Y(" @;
            
         %do i = 1 %to &nDiffShow;
            put "    :Name(""&trSigIndex %substr(&&nlpdlab&i,17)"")," @;                     
         %end;
            
         put "    ),";
         put "   Bar Chart( 1 ),";
         put "   SendToReport(";
         %if &nDiff <= 50 %then %do;
            put  "     Dispatch( {}, ""Chart"", OutlineBox, Set Title(""Number of Significant Differences by &ChrVar"")";
         %end;
         %else %do;
            put  "     Dispatch( {}, ""Chart"", OutlineBox, Set Title(""Number of Significant Differences by &ChrVar (only 50 shown)"")";
         %end;
         put "       ),";
         * put '   Dispatch( {}, "", AxisBox( 2 ), Add Axis Label( "Significant Samples" ) ),';
         put "    )";
         put "   );";
         put ");";
            
         put "objRefNS:data << Minimize Window;";
         put "mmchart<<Journal Window;"; 
            /*save chart script to journal file*/
            /*
            put "_JrnTxt1=LoadTextFile(""&OutPath.&jrnfilename"");";
            put "_JrnTxt2=mmchart<<GetJournal;";
            put "_JT1a=Munger(_JrnTxt1,1,length(trim(_JrnTxt1))-2);";
            put "_JT1b=Munger(_JT1a,1,length(trim(_JT1a))-1);";      
            put "_intermediatetext = "",ListBox(vertical,near,sub("";";
            put "_endtext = ""))))"";";
            put "SaveTextFile(""&OutPath.&jrnfilename"",_JT1b||_intermediatetext||_JrnTxt2||_endtext);";
            */
         put "mmchart<<Close Window;"; 
      %end;
      
   run;
   
   * construct journal;
   data _null_;
      file "&JRNFile" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      
      put "List Box(";
      put "   vertical,";
      put "   near,";
      put "   sub(";
      put "      Outline Box( Open(1),";
      put "         title(""&ProcessName Results""),";
      put "         sub(";
      %do i=1 %to &nchr;
         %if %length(&&chr&i) %then %do;
            put "            Button Box(title(""&ChrVar &&chr&i""),";
            put "               Script Text("" ";
            put "                  objRefNS:outds = \!""&&OName&i...sas7bdat\!"";";
            put " objRefNS:ChrVarInd=\!""&&chr&i\!"";";

            %if (%upcase(&PlotResiduals) = YES) %then %do; 
               put "                  objRefNS:outresds = \!""&&RName&i...sas7bdat\!"";";
            %end;
            %if %upcase(&LComponents)=YES %then %do;        
               put "                  jg:outcompds = \!""&&CName&i...sas7bdat\!"";";
            %end;
            put "                  include(\!"".\&jslfile2name\!"",<<new context);";
            put "               "")";
            put "            ),";
         %end;
      %end;
      put "            Button Box(title(""Combined Results""),";
      put "               Script Text("" ";
      put "                  objRefNS:outds = \!""&OutDataName..sas7bdat\!"";";
      put ' objRefNS:ChrVarInd=\!"Combined\!";';

      %if (%upcase(&PlotResiduals) = YES) %then %do; 
         put "                  objRefNS:outresds = \!""&OutDataResName..sas7bdat\!"";";
      %end;
      %if %upcase(&LComponents)=YES %then %do;
         put "                  jg:outcompds = \!""&OutDataCompName..sas7bdat\!"";";
      %end;
      put "                  include(\!"".\&jslfile2name\!"",<<new context);";
      put "               "")";
      put "            )";
      put "         )";
      put "      )";
      put "   )";
      put ");";
   run;

   * packaging;
   %if &Server^=Local %then %do;
      %if ^%symexist(sname) %then %let sname = ;
      * create string of dependent data sets;
      %do i=1 %to &nchr;
         %if (&i=1) %then %do;
            %let depdata = &_name._%lowcase(&&chr&i).sas7bdat;
         %end;
         %else %do;
            %let depdata = &depdata, &_name._%lowcase(&&chr&i).sas7bdat;
         %end;
         %if (%upcase(&PlotResiduals) = YES) %then %do; 
            %let depdata = &depdata, &_resname._%lowcase(&&chr&i).sas7bdat;
         %end;
         %if %upcase(&LComponents)=YES %then %do;
            %let depdata = &depdata, &_compname._%lowcase(&&chr&i).sas7bdat;
         %end;
      %end;
      %if &nDiff > 0 %then %do;
         %let depdata = &depdata, &sname._sum.sas7bdat;
      %end;
      %if %index(&ProcessName,OneWay) & &nDiff > 0  %then %do;
         %PackageInsertFile(Journal for &sname from &ProcessName, &OutPath, &JRNFileName, "&depdata", 
            text/plain, yes, no);
      %end;
      %else %do;
         %PackageInsertFile(Journal for &InData from &ProcessName, &OutPath, &JRNFileName, "&depdata", 
            text/plain, yes, no);
      %end;

      %PackageInsertFile(Secondary JSL code from &ProcessName, &OutPath, &JSLFile2Name, , 
         text/plain, yes, no);

      /* these are not inserted here because they are listed as dependencies in the main jsl file later
      %PackageInsertDataset(Output data set for &ProcessName, outlib, &OutDataName);
      %if (%upcase(&PlotResiduals) = YES) %then %do; 
         %PackageInsertDataset(Residuals data set for &ProcessName, outlib, &OutDataResName);
      %end;
      %if %upcase(&LComponents)=YES %then %do;
         %PackageInsertDataset(Components data set for &ProcessName, outlib, &OutDataCompName);
      %end;
      */
   %end;

%end;

%global _mainjslfile_ OpenSASOutputTab;
%if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
   %let _mainjslfile_ = %quote(&JSLFile3);
   data _null_;
      file "&JSLFile3";
      put " ";
   run;
%end;
%else %do;
   %let _mainjslfile_ = %quote(&JSLFile);
%end;

%if ^%symexist(FullAPName) %then %do;
   %global FullAPName;
   %let FullAPName = %str(&ProcessName);
%end;
%let OpenSASOutputTab=0;

* create JMP script;


%let BA_index=%index(&ProcessName,FindingsANOVA);
%let SA_index=%index(&ProcessName,Severity);

data _null_;
   %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
      file "&JSLFile2" lrecl=32767 %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   %end;
   %else %do;
      file "&JSLFile" lrecl=32767 %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   %end;
   
   put "//!";
   put "Names Default to Here(1);";
   %if (%upcase(&JournalChrom) ne YES) %then %do;  
      put "objRefNS = newnamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";    
   %end;
   put "objRefNS:DataList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   
   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{},{}};";
   put "objRefNS:ActionCodeList ={{},{}};";
   put "objRefNS:ActionDataTableList = {{},{}};";
   put "objRefNS:ActionIconList = {{},{}};";
   put "objRefNS:ActionRequirementFlagList =  {{},{}};";   
   put "objRefNS:ActionButtonsperRow = {};";

   put "objRefNS:TabsSectionFlag = 1;";
   put "objRefNS:ArryIndex = 1;";
    
   %if (%upcase(&ProcessName) = ANOVA) | %upcase(&ProcessName) = ONEWAYANOVA | %upcase(&ProcessName) = MIXEDMODELANALYSIS  %then %do;
   %end;
   %else %do;
      put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
      put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
      put "try(objRefNS:AEWhereStmt=""\[&AEWhereStmt]\"");";

      %if %index(&ProcessName,FindingsANOVA) %then %do;
         put "objRefNS:NoteDomain = ""&FDomain"";";
         put "objRefNS:includestring=""Findings Tests: "";";
         %let DomainWhereStmt = &WhereTests;
      %end;
      %else %if &ProcessName = AESeverityANOVA %then %do;
         put "objRefNS:NoteDomain = ""AE"";";      
      %end;       
      put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";

      %if %length(&WhereForReports) %then %do;
         put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
         put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
      %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
         %end;
      %end;
      %else %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;    
   %end;
   
   %let opentabflag =0;
   
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"Greetings Scientist.");';
      put 'speak(wait(1),"Here are the mixed model results you requested.");';
   %end;
   
   %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
      put "objRefNS:adata1 = open(objRefNS:outds,invisible);";
      put "objRefNS:FullApName=objRefNS:ChrVarInd;";
      put "objRefNS:outdsname = word(1,objRefNS:outds,""."");";
   %end;
   %else %do;
      put "objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      %if %length(%trim(&OutDataName)) > 28 %then %do;
            %let _name = %substr(&OutDataName,1,28);
            put "objRefNS:outdsname = ""&_name"";";
      %end;
      %else %do;
         put "objRefNS:outdsname = ""&OutDataName"";";
      %end;
   %end;
   /* put "for each row(color of(rowstate())=1);"; */
  
   put " ";
   
   %if &BA_index %then %do;  
      %if %symexist(mlchklow) or %symexist(mlchkhi) %then %do;
         %if &mlchklow or &mlchkhi %then %do;
            put "objRefNS:MLfunc=Function({},{default local},";
            put "    VListBox(";
            put "    objRefNS:mlwarn= TextBox(locale:get(""WarningResultsNotGenDueToMissValsGenByNorm_Text"")||"; 
            %if &mlchklow %then %do;
               put "    ""\!N\!N "" || locale:fill(locale:get(""AllLBSTNRLOMissOrZeroForLBTESTCDX_Text""), {""%trim(&chklln)""}) || ""\!N ""||";     
            %end;
            %if &mlchkhi %then %do;
               put "    ""\!N\!N "" || locale:fill(locale:get(""AllLBSTNRHIMissOrZeroForLBTESTCDX_Text""), {""%trim(&chkuln)""}) || ""\!N""||";      
            %end;    
            put "       "" "",";
            put "          <<Set Wrap(600), Set Font Size(11)";
            put "    );";  
            put "));";

            put 'InsertInto(objRefNS:DataList,{{""}});';
            put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XWarningMessage_Text""), {locale:get(""Findings_Text"")})));";
            put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:MLfunc()));';
            put 'InsertInto(objRefNS:TabAfterCodeList," ");';
            put 'InsertInto(objRefNS:TabOpenList,1);';      
         %end;
      %end; 
    %end;
   
   %if &HasDiff | &HasEst | &HasCon | &HasSlice | &HasT3 %then %do;   
      %if %length(&MultipleTestingMethod) %then %do;
         %put &MultipleTestingMethod NegLog10p-value cutoff using alpha = &Alpha over &ntests tests = &NegLog10pCutoff;
         put "/*---&MultipleTestingMethod NegLog10p-value cutoff using alpha = &Alpha over &ntests tests---*/";
      %end;
      %else %do;
         %put Specified NegLog10p-value cutoff = &NegLog10pCutoff;
         put "/*---specified NegLog10p-value cutoff---*/";
      %end;
      put "objRefNS:cutoff = &NegLog10pCutoff;";
      /*kemicl - create a JMP variable filter for pos and neg values of x axis */
      %if %symexist(UseXaxisFilter) %then %do;
         %if %upcase(&UseXaxisFilter)=YES %then %do;
            put "objRefNS:xaxispos = abs(&XaxisCutoff);";
            %if %index(%upcase(&XaxisDirection),POSITIVE)=0 %then %do;
               put "objRefNS:xaxisneg = -objRefNS:xaxispos;";
            %end;
         %end;
      %end;
   %end;
   put "/*---window frame size and position offset---*/";
   put "objRefNS:hfsize = 180;";
   put "objRefNS:pfsize = 180;";
   put "objRefNS:vfsize = 240;";
   put " ";
%if %upcase(&Whichproc) ne GLIMMIX %then %do;   
   * variability estimates;
   put " ";
   put "current data table(objRefNS:adata1);";
   put " ";
   /*
   %if (&nVariance = 1) %then %do;
      %let VarName = Variance_Residual, ;
   %end;
   %else %do;
      %let VarName = ;
      %do i = 1 %to &nVariance;
         %let VarName = &VarName :&&PropVar&i, ;
      %end;
   %end;
   */
   %let VarName = :&Variance_1;
   %do i = 2 %to &nVariance;
      %let VarName = &VarName, :&&Variance_&i ;
   %end;
   %let VarName = &VarName, :RSquare;
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"variability estimates");';
   %end;
   put 'include("$LS_HOME\Generator\DialogHelperFunctions.jsl");';
   put 'objRefNS:ve=Function({}, {default local},';
   put "If(IsEmpty(objRefNS:adata1),";
   %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
      put " objRefNS:adata1 = open(objRefNS:outds,invisible);";
   %end;
   %else %do;
      put " objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   %end;
   put ");";
   put "current data table(objRefNS:adata1);";

   put '   VListBox(';
   put '      HListBox(';
   put "         Distribution(";
   %if (&nVariance = 1) %then %do; 
      put "            Continuous Distribution(Column( :&Variance_1)),";
   %end;
   %else %do;
      *put "            Uniform Scaling(1),";
      %do i = 1 %to &nVariance;
         put "            Continuous Distribution(Column( :&&Variance_&i)),";
      %end; 
   %end;
   %if %upcase(&SplitResults)=YES %then %do;
      put "            Continuous Distribution(Column( :RSquare)), by(&jslDBV)";
    %end;
   %else %do;
      put "            Continuous Distribution(Column( :RSquare))";
   %end;
   put '         )';
   put "      ),";
   put '      HListBox(';
   put "         Multivariate(Y( &VarName ), Scatterplot Matrix(Density Ellipses(1),";
   %if %upcase(&SplitResults)=YES %then %do;
      put "            Ellipse Color(3)), by(&jslDBV)";
   %end;
   %else %do;
      put "            Ellipse Color(3))";
   %end;
   put '         )';
   put '      )';
   put '   )';
   put ');';
   %*end;
   put " ";
%end; 
%else %do;
   %if ^%symexist(vcFlag) %then %let vcFlag = 0;
   %if ^&vcFlag %then %do; 
   * variability estimates;
   put " ";
   put "current data table(objRefNS:adata1);";
   put " ";

   %let VarName = :&Variance_1;
   %do i = 2 %to &nVariance;
      %let VarName = &VarName, :&&Variance_&i ;
   %end;
   %*let VarName = &VarName, :RSquare;
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"variability estimates");';
   %end;
   put 'include("$LS_HOME\Generator\DialogHelperFunctions.jsl");';
   put 'objRefNS:ve=Function({}, {default local},';
   put "If(IsEmpty(objRefNS:adata1),";
   %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
      put " objRefNS:adata1 = open(objRefNS:outds,invisible);";
   %end;
   %else %do;
      put " objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   %end;
   put ");";
   put "current data table(objRefNS:adata1);";

   put '   VListBox(';
   put '      HListBox(';
   put "         Distribution(";
   %if (&nVariance = 1) %then %do; 
      put "            Continuous Distribution(Column( :&Variance_1))";
   %end;
   %else %do;
      *put "            Uniform Scaling(1),";
      %do i = 1 %to %eval(&nVariance-1);
         put "            Continuous Distribution(Column( :&&Variance_&i)),";
      %end; 
      put "            Continuous Distribution(Column( :&&Variance_&i))";
   %end;
   put '         )';
   put "      ),";
   put '      HListBox(';
   put "         Multivariate(Y( &VarName ), Scatterplot Matrix(Density Ellipses(1),";
   %if %upcase(&SplitResults)=YES %then %do;
      put "            Ellipse Color(3)), by(&jslDBV)";
   %end;
   %else %do;
      put "            Ellipse Color(3))";
   %end;
   put '         )';
   put '      )';
   put '   )';
   put ');';
   %*end;
   put " ";
   %end;

   
   * goodness of fit; *************not finished!*****************************
   put " ";
   put "current data table(objRefNS:adata1);";
   put " ";

   %let ODLab = Chi_Sq_Div_DF;
   %CheckLabel(&OutData,ODLab);
   %let VarName = :RSquare, :&ODLab;
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"goodness of fit");';
   %end;
   put 'include("$LS_HOME\Generator\DialogHelperFunctions.jsl");';
   put 'objRefNS:gf=Function({}, {default local},';
   put "If(IsEmpty(objRefNS:adata1),";
   /*%if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
      put " objRefNS:adata1 = open(objRefNS:outds,invisible);";
   %end;
   %else %do;*/
      put " objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   %*end;
   put ");";
   put "current data table(objRefNS:adata1);";

   put '   VListBox(';
   put '      HListBox(';
   put "         Distribution(";
   put "            Continuous Distribution(Column( :&ODLab )),";
   put "            Continuous Distribution(Column( :RSquare ))";
   put '         )';
   put "      ),";
   put '      HListBox(';
   put "         Multivariate(Y( &VarName ), Scatterplot Matrix(Density Ellipses(1),";
   %if %upcase(&SplitResults)=YES %then %do;
      put "            Ellipse Color(3)), by(&jslDBV)";
   %end;
   %else %do;
      put "            Ellipse Color(3))";
   %end;
   put '         )';
   put '      )';
   put '   )';
   put ');';
   %*end;
   put " ";
%end;



   %if ^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice and (&HasT1 or &HasT2 or &HasT3) %then %do;
      put 'objRefNS:fet=Function({}, {default local},';
      put "If(IsEmpty(objRefNS:adata1),";
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put " objRefNS:adata1 = open(objRefNS:outds,invisible);";
      %end;
      %else %do;
         put " objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      %end;
       put ");";
      put "current data table(objRefNS:adata1);";
      put '   VListBox(';
      put '      HListBox(';
      put "         Distribution(";
      %do i = &nNLP %to 1 %by -1;
         %if %upcase(&SplitResults)=YES %then %do;
            put "            Continuous Distribution(Column( :&&NLP&i)), by(&jslDBV)" @;
         %end;
         %else %do;
            put "            Continuous Distribution(Column( :&&NLP&i))" @;
         %end;
         %if (&i > 1) %then %do;
            put ",";
         %end;
         %else %do;
            put " ";
         %end;
      %end; 
      put '         )';
      put '      )';
      put "   )";
      put ');';
      
      put " ";
   %end;
   
   * web search prep;
   %include "&MacroPath./WebSearchJSL.sas";
   
   * JMP label variables;
   
   %if %length(&AnnoInData) and %length(&AnnoLabel) %then %AnnoLabelJSL(Ds=&OutData);
   %if ^%symexist(LabelVar) %then %let LabelVar = ;
   %let LabelVar0 = &LabelVar;
   %if %length(&LabelVar) %then %do;   
      %CheckLabel(&OutData,LabelVar);
      put "%str(column(%"&LabelVar%") << label(1);)";
   %end;
   %else %if %length(&ByVars) %then %do;
      %do i = 1 %to &nby;
         %let ByLab = %scan(&_ByVars,&i);
         %CheckLabel(&OutData,ByLab);
         put "%str(column(%"&ByLab%") << label(1);)";
     %end;
   %end;
   %else %if %length(&BaseMergeVars) %then %do;
      %let i = 1;
      %do %while(%length(%scan(&BaseMergeVars,&i)));
         %let BaseLab = %scan(&BaseMergeVars,&i);
         %CheckLabel(&OutData,BaseLab);
         put "%str(column(%"&BaseLab%") << label(1);)";
         %let i=%eval(&i+1);
     %end;
   %end;
   %let clientPath=&clientoutpath;

   * web search expressions;
   %WebSearchJSL(Ds=&OutData,clientPath=&clientPath,MakeWindow=0);

   * drill down expressions;
   %put ByVars = &ByVars;
   put "objRefNS:lsproduct = ""&LS_Product"";";
   put "objRefNS:origbyvars = ""&OrigByVars"";";
   put "objRefNS:byvars = ""&ByVars"";";
   put "objRefNS:lastbyvar = ""&LastByVar"";";
   put "objRefNS:bvlist = {" @@;
   %let i=1;
   %do %while(%length(%scan(&ByVars,&i)));
      %let ByLab = %scan(&ByVars,&i);
      %checklabel(&OutData,ByLab);
      put "column(""&ByLab"")" @@;
      %if (&i<&nby) %then %do;
         put "," @@;
      %end;
      %let i=%eval(&i+1);
   %end;
   put "};";
   put "objRefNS:belist = {" @@;
   %let i=1;
   %do %while(%length(%scan(&ByVars,&i)));
      %let ByLab = %scan(&ByVars,&i);
      /* %checklabel(&OutData,ByLab); */
      put " ""&ByLab"" " @@;
      %if (&i<&nby) %then %do;
         put "," @@;
      %end;
      %let i=%eval(&i+1);
   %end;
   put "};";
   put 'objRefNS:bvmatch = "' @@;
   %let i=1;
   %do %while(%length(%scan(&ByVars,&i)));
      %let ByLab = %scan(&ByVars,&i);
      %checklabel(&OutData,ByLab);
      put "column(\!""&ByLab\!"")==column(\!""&ByLab\!"")" @@;
      %if (&i<&nby) %then %do;
         put "," @@;
      %end;
      %let i=%eval(&i+1);
   %end;
   put '";';
   put "objRefNS:alreadystacked = &AlreadyStacked;";
   %if ^%symexist(ResponseVar) %then %let ResponseVar = ;
   put "objRefNS:responsevar = ""&ResponseVar"";";
   %if ^%symexist(ResponseVar0) %then %let ResponseVar0 = ;
   put "objRefNS:responsevar0 = ""&ResponseVar0"";";
   put "objRefNS:inpath = ""&InPath"";";
   put "objRefNS:indata = ""&InData"";";
   put "objRefNS:expinpath = ""&outpath"";";
   put "objRefNS:expindata = ""&ExpInData"";";
   %if ^%symexist(Anno2InData) %then %let Anno2InData = ;
   %if %length(&Anno2InData) %then %do;
      put "objRefNS:annoindata = ""&Anno2InData"";";
      put "objRefNS:annopath = ""&Anno2Path"";";
      put "objRefNS:annodata = ""&Anno2Data"";";
      put "objRefNS:annomergevars = ""&Anno2MergeVars"";";
      put "objRefNS:basemergevars = ""&Anno2MergeVars"";";
      %if ^%length(&Anno2Label) %then %do;
         %if %length(&Anno2MergeVars) %then %do;
            %let Anno2Label = %scan(&Anno2MergeVars,1,' ');
         %end;
         %else %if %length(&Anno2PosVar) %then %do;
            %let Anno2Label = &Anno2PosVar;
         %end;
         %else %let Anno2Label = &Anno2ChrVar;
      %end;
      put "objRefNS:annolabel = ""&Anno2Label"";";
      put "objRefNS:annochrvar = ""&Anno2ChrVar"";";
      put "objRefNS:annoposvar = ""&Anno2PosVar"";";
   %end;
   %else %do;
      put "objRefNS:annoindata = ""&AnnoInData"";";
      put "objRefNS:annopath = ""&AnnoPath"";";
      put "objRefNS:annodata = ""&AnnoData"";";
      put "objRefNS:annomergevars = ""&AnnoMergeVars"";";
      put "objRefNS:basemergevars = ""&BaseMergeVars"";";
      put "objRefNS:annolabel = ""&AnnoLabel"";";
      put "objRefNS:annoGeneSymbol = ""&AnnoGeneSymbol"";";
      put "objRefNS:annochrvar = ""&AnnoChrVar"";";
      put "objRefNS:annoposvar = ""&AnnoPosVar"";";
   %end;
   put "objRefNS:chrvar = ""&ChrVar0"";";
   put "objRefNS:chrvar0 = ""&ChrVar0"";";
   put "objRefNS:chrvalues = {};";
   put "objRefNS:posvar = ""&PosVar0"";";
   put "objRefNS:posvar0 = ""&PosVar0"";";
   put "jg:trackfolder = ""&TrackPath"";";
   put "jg:trackfiles = ""&TrackFiles"";";
   put "jg:trackadd = &TrackAdd;";
   %if %length(&TrackPath) %then %do;
      put "plot_tracks = 1;";
   %end;
   %else %do;
      put "plot_tracks = 0;";
   %end;
   put "objRefNS:outpath = ""&OutPath"";";
   put "objRefNS:serveroutpath = ""&ServerOutPath"";";
   put "objRefNS:clientoutpath = ""&ClientOutPath"";";
   /*
   %if %substr(&MacroPath,%length(&MacroPath),1) ^= \ %then %do;
      %let MacroPath = &MacroPath.\;
   %end;
   */
   put "objRefNS:macropath = ""&MacroPath"";";
   put "objRefNS:processname = ""&ProcessName"";";
   put "objRefNS:ProcUsed = ""&WhichProc"";";
   put "objRefNS:DataDistribution = ""&DataDistribution"";";
   put "objRefNS:ModelStatement=""\[%bquote(&ProcMixedStatements)]\"";";
   put "objRefNS:classvars0 = {" @@;
   %let i=1;
   %do %while(%length(%scan(&ClassVars,&i)));
      %let cvi = %scan(&ClassVars,&i);
      %if (&i > 1) %then %do;
         put ", " @@;
      %end;
      put "Column(""&cvi"")" @@;
      %let i=%eval(&i+1);
   %end;
   put "};";
   put "objRefNS:classvars = {" @@;
   %let i=1;
   %do %while(%length(%scan(&AllClassVars,&i)));
      %let cvi = %scan(&AllClassVars,&i);
      %if (&i > 1) %then %do;
         put ", " @@;
      %end;
      put "Column(""&cvi"")" @@;
      %let i=%eval(&i+1);
   %end;
   put "};";
   put "objRefNS:effects = {" @@;
   %if &nfef %then %do;
      %do i=1 %to &nfef;
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         put " &&fef&i " @@;
      %end;       
   %end;
   %else %do;
      %let i=1;
      %do %while(%length(%scan(&FixedEffects,&i,' ')));
         %let fei = %scan(&FixedEffects,&i,' ');
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         %if %quote(&fei) =%quote(%str(/)) %then %do;
            put " ""&fei"" " @@; 
         %end;
         %else %do;
            put " &fei " @@;        
         %end;
         %let i=%eval(&i+1);
      %end;
   %end;
   %let i=1;
   %do %while(%length(%scan(&RandomEffects,&i,' ')));
      %let fei = %scan(&RandomEffects,&i,' ');
      %if &nfef or %length(&FixedEffects) or (&i > 1) %then %do;
         put ", " @@;
      %end;
      put "&fei %str(&) Random" @@;
      %let i=%eval(&i+1);
   %end;
   put "};";
   put "objRefNS:fixedeffects = {" @@;
   %if &nfef %then %do;
      %do i=1 %to &nfef;
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         put " ""&&fef&i"" " @@;
      %end;       
   %end;
   %else %do;
      %let i=1;
      %do %while(%length(%scan(&FixedEffects,&i,' ')));
         %let fei = %scan(&FixedEffects,&i,' ');
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         put " ""&fei"" " @@;
         %let i=%eval(&i+1);
      %end;
   %end;
   put "};";
   put "objRefNS:fixedeffectstrunc = {" @@;
   %if &nfef %then %do;
      %do i=1 %to &nfef;
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         put " ""&&feft&i"" " @@;
      %end;       
   %end;
   %else %do;
      %let i=1;
      %do %while(%length(%scan(&FixedEffects,&i,' ')));
         %let fei = %scan(&FixedEffects,&i,' ');
         %if (&i > 1) %then %do;
            put ", " @@;
         %end;
         put " ""&fei"" " @@;
         %let i=%eval(&i+1);
      %end;
   %end;
   put "};";
   put "objRefNS:inclassvars = ""&InClassVars"";";
   put 'objRefNS:timevar = "";';
   %if ^%symexist(TrtName) %then %let TrtName = ;
   put "objRefNS:trtname = ""&TrtName"";";
   put 'objRefNS:censorvar = "";';
   put 'objRefNS:censorvalues = "";';
   put "objRefNS:nstrata = 0;";
   put "objRefNS:basesubjectvar = ""&BaseSubjectVar"";";
   put "objRefNS:baselinevar = ""&BaselineVar"";"; 
   put "objRefNS:baselinevalue = ""&BaselineValue"";"; 
   put "objRefNS:volcanotab =1;";
   put "objRefNS:clustertab =2;";
   put "objRefNS:ppcatab =3;";
   %if %symexist(GConv) %then %do;
      put "objRefNS:gconv = %sysevalf(&GConv/100000000);";
   %end;
   %if %symexist(PConv) %then %do;
      put "objRefNS:pconv = %sysevalf(&PConv/100000000);";
   %end;
   
/* KELCI need to evaluate tab selection/index dependency */
   put "objRefNS:RefreshTabExpr = Expr(";
   put '   program = "';
   put '  objRefNS:tabfound=objRefNS:trtb << Index of Tab(objRefNS:TabButtonList["||char(eval(objRefNS:tabloc))||"]);';
   put "    if(objRefNS:tabfound >0,";
   
   put "       objRefNS:trtb << Delete(objRefNS:tabfound);";
   put "    );";
   put '  objRefNS:tabfound=objRefNS:trtb << Index of Tab(objRefNS:TabButtonList["||char(eval(objRefNS:tabloc))||"]);';
   put "    if(objRefNS:tabfound == 0,  ";
   %if %upcase(&SplitResults)=YES %then put 'if (tSig > 0,';
   %else put '          if (objRefNS:Nsig > 0,';;
   put '         objRefNS:trtb << Append( objRefNS:TabButtonList["||char(eval(objRefNS:tabloc))||"], Eval(objRefNS:TabCodeList["||char(eval(objRefNS:tabloc))||"]) );';
   put "          if(!isEmpty(objRefNS:TabAfterCodeList),";
   put '               Eval(objRefNS:TabAfterCodelist["||char(eval(objRefNS:tabloc))||"])';
   put "       );";

   put ",";
   put "    if (objRefNS:tabvolcano >0,";
   put '         objRefNS:trtb << Append( objRefNS:TabButtonList["||char(eval(objRefNS:tabloc))||"], Eval(objRefNS:TabCodeList["||char(eval(objRefNS:tabloc))||"]) );';
   put "          if(!isEmpty(objRefNS:TabAfterCodeList),";
   put '               Eval(objRefNS:TabAfterCodelist["||char(eval(objRefNS:tabloc))||"])';
   put "       );";
   put ");";
   
   put ");// end of objRefNS:Nsig";
   put "       , //else - only allow 1 tab open per button";
   put "          objRefNS:trtb << Set Selected(objRefNS:tabfound) ";
   put ');";';
   
   
   put "    program = eval(parse(program));  ";
   
   put " );";

   %if %upcase(&LComponents)=YES %then %do;
      put "objRefNS:lcomp = 1;";
      put "objRefNS:outcomp = ""&OutDataCompName"";";
   %end;
   %else %do;
      put "objRefNS:lcomp = 0;";
   %end;
   put "objRefNS:vennFlag = 1;";
   %if %upcase(&ProcessName) = ANOVA | %upcase(&ProcessName) = MIXEDMODELANALYSIS | %upcase(&ProcessName) = ONEWAYANOVA %then %do;
      %if (&HasDiff | &HasEst) %then %do;
         put "objRefNS:foldfilter = 1;";
      %end;
      %else %do;
         put "objRefNS:foldfilter = 0;";
      %end;
   %end;
   %else %do;
      put "objRefNS:foldfilter = 0;";
   %end;
   put "include(""$LS_HOME\JSLFiles\Mixed Model Drill Down Expressions.JSL"");";
   
   
   * Action Buttons window;
   /* put 'abw=New Window("Action Buttons",';*/
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"action buttons");';
   %end;
   %if &LS_Index=2 %then %do;
      put 'objRefNS:ActionButtonDesc = {"Search the Web: Select Points or Rows then Click a Button"};';
      put "objRefNS:ActionButtonsperRow = {0};";

      %WebButtons;
      %WebButtons_individual;
   %end;
   
   put "InsertInto(objRefNS:ActionButtonDesc , {expr(locale:get(""DrillDownSelectPointsorRowsthenClickaButton_Text""))});";
   put "InsertInto(objRefNS:ActionButtonsperRow ,0);";
   
   put "include(""$LS_HOME\JSLFiles\Mixed Model Drill Down Buttons.JSL"");";

   * add ipathwayGuide drill down;
   %if &LS_Index=2 %then %do;
      %if %symexist(AnnoGeneSymbol) %then %do;
         %if %length(&AnnoGeneSymbol) %then %do;
            put "InsertInto(objRefNS:ActionButtonDesc , ""Drill Down: Click a Button for Pathway Analysis"");";
            put "InsertInto(objRefNS:ActionButtonsperRow, 0);";      
            put "objRefNS:ArryIndex = 1;";
            put " InsertInto(objRefNS:ActionButtonList,{{""iPathwayGuide""}});";
            put " InsertInto(objRefNS:ActionCodeList,{{ipg}}); ";
            put " objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);";
            put "ipg=expr(";
            put "iPathwayOutput;";
            put ");";
         %end;
      %end;
   %end;
   
   /* Add for Box and Shift Plot drill downs for Clinical */
   %if &BA_index %then %do;     
      /* grab the prefix to know domain */
      %let Domain_dd = %substr(&ProcessName,1,%eval(&BA_Index-1)); 
      /* put in drill down buttons for Shift and Box Plot APs */
      put "include(""$LS_HOME\JSLFiles\BaseLine ANOVA Drill Downs.JSL"");";
      %if ^%symexist(BaselineTimeWindow0) %then %let BaselineTimeWindow0 =;
      %if ^%symexist(TimeWindows0) %then %let TimeWindows0 = ;
      put "objRefNS:tpdb = expr(objRefNS:BPSPfctn(""&FDomain"",""TimeTrends"",""&TestVar"",""&TestVarLabel"",""&BaselineTimeWindow0"", ""&TimeWindows0"",locale:get(""TrendPlots_Text"")));";
      put "objRefNS:spdb = expr(objRefNS:BPSPfctn(""&FDomain"",""ShiftPlots"",""&TestVar"",""&TestVarLabel"",""&BaselineTimeWindow0"", ""&TimeWindows0"",locale:get(""ShiftPlots_Text"")));";
      put "objRefNS:bpdb = expr(objRefNS:BPSPfctn(""&FDomain"",""BoxPlots"",""&TestVar"",""&TestVarLabel"",""&BaselineTimeWindow0"", ""&TimeWindows0"",locale:get(""BoxPlots_Text"")));";
      put "objRefNS:wpdb = expr(objRefNS:BPSPfctn(""&FDomain"",""WaterfallPlots"",""&TestVar"",""&TestVarLabel"",""&BaselineTimeWindow0"", ""&TimeWindows0"",locale:get(""WaterFallPlots_Text"")));";
    
      put "InsertInto(objRefNS:ActionButtonDesc , {""Related Reports Drill Downs""});";
      put "InsertInto(objRefNS:ActionButtonsperRow ,2);";
      put "objRefNS:ArryIndex = 2;";
      
      put "InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{expr(locale:get(""TrendPlots_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],Expr(objRefNS:tpdb()));";      
      put "InsertInto(objRefNS:ActionDataTableList[objRefNS:ArryIndex], {{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:ActionIconList[objRefNS:ArryIndex],""&IconPath.LinesPlot.gif"");";
      put "InsertInto(objRefNS:ActionRequirementFlagList[objRefNS:ArryIndex],3);";

      put "InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{expr(locale:get(""ShiftPlots_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],Expr(objRefNS:spdb()));";
      put "InsertInto(objRefNS:ActionDataTableList[objRefNS:ArryIndex], {{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:ActionIconList[objRefNS:ArryIndex],""&IconPath.LinearRegression.gif"");";
      put "InsertInto(objRefNS:ActionRequirementFlagList[objRefNS:ArryIndex],3);";
      
            
      put "InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{expr(locale:get(""BoxPlots_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],Expr(objRefNS:bpdb()));";
      put "InsertInto(objRefNS:ActionDataTableList[objRefNS:ArryIndex], {{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:ActionIconList[objRefNS:ArryIndex],""&IconPath.BoxPlotBlue.gif"");";
      put "InsertInto(objRefNS:ActionRequirementFlagList[objRefNS:ArryIndex],3);";

      put "InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{expr(locale:get(""WaterFallPlots_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],Expr(objRefNS:wpdb()));";
      put "InsertInto(objRefNS:ActionDataTableList[objRefNS:ArryIndex], {{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:ActionIconList[objRefNS:ArryIndex],""&IconPath.PowerCurve.gif"");";
      put "InsertInto(objRefNS:ActionRequirementFlagList[objRefNS:ArryIndex],3);";
      /*
      put "InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{ expr(locale:get(""TrendPlots_Text"")), expr(locale:get(""ShiftPlots_Text"")), expr(locale:get(""BoxPlots_Text"")),  expr(locale:get(""WaterFallPlots_Text""))},1);";
      put "InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],{objRefNS:tpdb,objRefNS:spdb,objRefNS:bpdb,objRefNS:wpdb},1);";
      put "InsertInto(objRefNS:ActionDataTableList[objRefNS:ArryIndex], {{""objRefNS:adata1""},{""objRefNS:adata1""},{""objRefNS:adata1""},{""objRefNS:adata1""}},1);";
      */
   %end;
   
   %if &LS_Index=2 %then %do;
      put "InsertInto(objRefNS:ActionButtonDesc ,""Launch Process: Select Points or Rows then Click a Button"");";
      put "InsertInto(objRefNS:ActionButtonsperRow ,0);";
      put "include(""$LS_HOME\JSLFiles\Mixed Model Drill Down Launch.JSL"");";
   %end;
   %if &indTestAdj=no %then %do;
      put "InsertInto(objRefNS:ActionButtonDesc , {expr(locale:get(""DetermineSignificance_Text""))});";
      put "InsertInto(objRefNS:ActionButtonsperRow,1);";
      %if &LS_Product ^= Clinical %then %do;
         put "if(isEmpty(objRefNS:ActionButtonList),";
         put "if (objRefNS:foldfilter,";
         put "objRefNS:ActionButtonList = {{""Create Subset with Mean Difference and P-value Criteria""}};";
         put "objRefNS:ActionCodeList = {{objRefNS:fpub}};";
         put "   objRefNS:ArryIndex = 1;";
         put "   InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{expr(locale:get(""Refresh_Text""))});";
         put "   InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],{objRefNS:getcuttoff});";
         put "   objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);";
         put ",";
         put "   objRefNS:ActionButtonList = {{expr(locale:get(""Refresh_Text""))}};";
         put "   objRefNS:ActionCodeList = {{objRefNS:getcuttoff}};";
         put "   objRefNS:ArryIndex = 1;";
         put ");";
         put "   , // else";
         put "if (objRefNS:vennFlag,";
         put "   InsertInto(objRefNS:ActionButtonList,{{expr(locale:get(""VennDiagram_Text""))}});";
         put "   InsertInto(objRefNS:ActionCodeList,{{objRefNS:venn}});";
         put "   objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);";
         put "if (objRefNS:foldfilter,";
         put "   InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{""Create Subset with Mean Difference and P-value Criteria""});";
         put "   InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],{objRefNS:fpub});";
         put "   objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);";
         put ");";
         put "   InsertInto(objRefNS:ActionButtonList[objRefNS:ArryIndex],{expr(locale:get(""Refresh_Text""))});";
         put "   InsertInto(objRefNS:ActionCodeList[objRefNS:ArryIndex],{objRefNS:getcuttoff});";
         put "   objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);";
         put ",";
         put "   InsertInto(objRefNS:ActionButtonList,{{expr(locale:get(""Refresh_Text""))}});";
         put "   InsertInto(objRefNS:ActionCodeList,{{objRefNS:getcuttoff}});";
         put "   objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);";
         put ");";
         put ");";
         put 'objRefNS:getcuttoff = expr(';
         put 'objRefNS:cutoff = Num(Eval(objRefNS:vteb)<<GetText);'; 
         put '            if (objRefNS:cutoff>=0,';
         put " // objRefNS:DataList = {};";
         put " // objRefNS:TabButtonList = {};";
         put " // objRefNS:TabCodeList = {};";
         put " // objRefNS:TabAfterCodeList = {};";
         put " // objRefNS:TabOpenList = {};";
         put '             objRefNS:adata1<<clear row states;';
         put '             objRefNS:sigrun;';
         %if &BA_index %then %do;
            put "    objRefNS:tabloc=Loc(objRefNS:TabButtonList, locale:fill(locale:get(""XResults_Text""), {""&FDomain""}));";
         %end;
         %else %do;
            put '    objRefNS:tabloc=Loc(objRefNS:TabButtonList, locale:get("Results_Text"));';
         %end;
         put " if( NRow(objRefNS:tabloc) != 0,";
         put "    objRefNS:tabloc1 =objRefNS:tabloc[1];";
         put "    objRefNS:tabloc =objRefNS:tabloc1;";
         put "    objRefNS:tabvolcano =1;";
         put "    objRefNS:RefreshTabExpr;";
         put "    );";
         put '   objRefNS:tabloc=Loc(objRefNS:TabButtonList, "Chromosome Position Plots");';
         put "if( NRow(objRefNS:tabloc) != 0,";
         put "    objRefNS:tabloc1 =objRefNS:tabloc[1];";
         put "    objRefNS:tabloc =objRefNS:tabloc1;";
         put "    objRefNS:tabvolcano =0;";
         put "    objRefNS:RefreshTabExpr;";
         put "    );";
         put ",";
         put '               objRefNS:ActionButtonMessage=NewWindow("Action Buttons Message",';
         put '                  VListBox(';
         put '                     objRefNS:tb=TextBox("Please specify a nonnegative (>=0) objRefNS:cutoff."),';
         put '                     TextBox(" "),';
         put '                     ButtonBox("OK",objRefNS:ActionButtonMessage<<CloseWindow);';
         put '                  );';
         put '               );';
         put '               objRefNS:tb << Font Color(3);';
         put '               objRefNS:tb << Set Font Size(11);';
        put '               objRefNS:tb << Set Wrap(1000)';
        put '            );';
        put '         );';
      %end;
      /*
      %else %do;
         put "if (objRefNS:vennFlag,";
               put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""DetermineSignificance_Text""))});";   
               put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""VennDiagram_Text""))});";
               put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:venn()));";    
               put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:adata1""}});";
               put "InsertInto(objRefNS:ActionButtonsperRow,1);";
               put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.VennDiagramAnd.gif"");";               
         put "   InsertInto(objRefNS:ActionCodeList,{{objRefNS:venn}});"
         put "   objRefNS:ArryIndex = NItems(objRefNS:ActionButtonList);"; 
         put ");";
      %end;
      ;*/
      put " ";
   %end;

   * objRefNS:sigrun: encapsulate code below in an expression so it can be re-executed from Action Buttons;
   put " ";
   put "jg:ncall_sigrun = 0;";
   put "/*--expression to rerun from Action Buttons---*/";
   put 'objRefNS:sigrun = expr(';
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"Greetings Scientist.");';
      put 'speak(wait(1),"Here are the mixed model results you requested.");';
   %end;
   
   put "if (isempty(ncall_sigrun),jg:ncall_sigrun=0;);";
   put 'jg:ncall_sigrun++;';
   put 'if(jg:ncall_sigrun==1,jg:ncall_string="",jg:ncall_string=" "||char(jg:ncall_sigrun));';
   
   * significant differences indicator variables and table;
   %if (&HasDiff | &HasEst | &HasSlice | &HasT3) %then %do;
      put "/*---Corresponding p-value cutoff---*/"; 
      put "objRefNS:pcutoff = 10^(-objRefNS:cutoff);";
      put "cutoffAlias = objRefNS:cutoff;";
      put "pcutoffAlias = objRefNS:pcutoff;";
      put 'current data table(objRefNS:adata1);';
      put 'objRefNS:adata1<< Clear Column Selection;';
      * create significance indicator variables;
      %if (%upcase(&Speak) = YES) %then %do;
         put 'speak(wait(1),"selecting significant rows");';
      %end;
      %let nCat = 0;
      put '_colnames_ = Char(objRefNS:adata1<<Get Column Names());';
      %if &HasDiff | &HasEst | &HasCon %then %do;
         %if &HasDiff %then %do;
            %let nCat = &nDiff;
            %do i = 1 %to &nDiff;
               %let SigLabel = &trSigIndex %substr(&&nlpdlab&i,17);
               put "If(!Contains(_colnames_,""&SigLabel""),";
               put '  objRefNS:adata1 << NewColumn(' @@;
               put "  %str(%"&SigLabel%")," @@;
               put "  Nominal, Width(1), " @@;
               put "   <<Set Property(""SAS Name"",Sig_Index_for_Diff&i)," ; 
               put "   <<Set Property(""SAS Label"",:Name(""&SigLabel"")))" @@; 
               put ");" ; /* close If stmnt */
               put "Eval(Substitute(Expr(Column(""&SigLabel"")<<set formula( if (Is Missing(:Name("" " @@;
               put "%str(&&nlpdlab&i)" @@;
               put '"))==1,.,';
               put ' :Name("' @@;
               put "%str(&&nlpdlab&i)" @@;
               put '") > _cutoffAlias' @@;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                        put " & abs(:Name(""&&dlabel&i"")) > abs(_xaxis)";
                     %end;
                     %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                        put " & :Name(""&&dlabel&i"") > abs(_xaxis)";
                     %end;
                     %else %do;
                        put " & :Name(""&&dlabel&i"") < -abs(_xaxis)";
                     %end; 
                  %end;
               %end;
               put ', 1,0) ))'; /* end expr */ 
               %if &indTestAdj=no %then %do;
                  put ", Expr(_cutoffAlias), cutoffAlias" @@;
               %end;
               %else %do;
                  put ", Expr(_cutoffAlias), &&NegLog10pCutoff&i" @@;
               %end;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     put ", Expr(_xaxis), objRefNS:xaxispos";
                  %end;
               %end;               
               put "));";
               /* kemicl - end of eval substitute */ 
               put "Column(""&SigLabel"")<<Eval Formula;";
            %end;
            * compute sum and average of sig indices;
            %if &nDiff > 1 %then %do;
               put "If(!Contains(_colnames_, ""Sum Sig Index for Diffs""),";
               put " objRefNS:adata1 << NewColumn(""Sum Sig Index for Diffs"");"; 
               put ");";
               put "Eval(Substitute(Expr(Column(""Sum Sig Index for Diffs"")<<set formula(Sum(";
               %do i = 1 %to &nDiff;
                  put "   if (Is Missing(:Name(""%str(&&nlpdlab&i)""))==1,.,";
                  put "      :Name(""%str(&&nlpdlab&i)"") > " @@;
                  %if &indTestAdj = no %then %do;
                     put " _cutoffAlias" @@;
                  %end;
                  %else %do;
                     put " _cutoffAlias&i" @@;
                  %end;
                  %if %symexist(UseXaxisFilter) %then %do;
                     %if %Upcase(&UseXaxisFilter)=YES %then %do;
                        %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                           put " & abs(:Name(""&&dlabel&i"")) > abs(_xaxis)";
                        %end;
                        %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                           put " & :Name(""&&dlabel&i"") > abs(_xaxis)";
                        %end;
                        %else %do;
                           put " & :Name(""&&dlabel&i"") < -abs(_xaxis)";
                        %end; 
                     %end;
                  %end;
                   put ', 1,0)';
                  %if &i < &nDiff %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put ')))';
               %if &indTestAdj=no %then %do;
                  put ", Expr(_cutoffAlias), cutoffAlias" @@;
               %end;
               %else %do;
                  %do i = 1 %to &nDiff;
                     put ", Expr(_cutoffAlias&i), &&NegLog10pCutoff&i" @@;
                  %end;
               %end; 
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     put ", Expr(_xaxis), objRefNS:xaxispos";
                  %end;
               %end;                   
               put "));"; 
                put "Column(""Sum Sig Index for Diffs"")<<Eval Formula;";
               /* kemicl - remove and change if contains check to only create the column
               put "Column(""Sum Sig Index for Diffs"")<<set each value(Sum(";
                  %do i = 1 %to &nDiff;
                     put "   if (Is Missing(:Name(""%str(&&nlpdlab&i)""))==1,.,";
                     put "      :Name(""%str(&&nlpdlab&i)"") > cutoffAlias,1,0)" @@;
                     %if &i < &nDiff %then %do;
                        put ",";
                     %end;
                  %end;
               put " ";
               put ")));";
               */
               put "If(!Contains(_colnames_, ""Avg Sig Index for Diffs""),";
               put " objRefNS:adata1 << NewColumn(""Avg Sig Index for Diffs"");";
               put ");";
               put "Eval(Substitute(Expr(Column(""Avg Sig Index for Diffs"")<<set formula(Mean(";
               %do i = 1 %to &nDiff;
                  put "   if (Is Missing(:Name(""%str(&&nlpdlab&i)""))==1,.,";
                  put "      :Name(""%str(&&nlpdlab&i)"") >  " @@;
                  %if &indTestAdj = no %then %do;
                     put " _cutoffAlias" @@;
                  %end;
                  %else %do;
                     put " _cutoffAlias&i" @@;
                  %end;
                  %if %symexist(UseXaxisFilter) %then %do;
                     %if %Upcase(&UseXaxisFilter)=YES %then %do;
                        %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                           put " & abs(:Name(""&&dlabel&i"")) > abs(_xaxis)";
                        %end;
                        %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                           put " & :Name(""&&dlabel&i"") > abs(_xaxis)";
                        %end;
                        %else %do;
                           put " & :Name(""&&dlabel&i"") < -abs(_xaxis)";
                        %end; 
                     %end;
                  %end;
                   put ', 1,0)';
                  %if &i < &nDiff %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put ')) )';
               %if &indTestAdj=no %then %do;
                  put ", Expr(_cutoffAlias), cutoffAlias" @@;
               %end;
               %else %do;
                  %do i = 1 %to &nDiff;
                     put ", Expr(_cutoffAlias&i), &&NegLog10pCutoff&i" @@;
                  %end;
               %end; 
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     put ", Expr(_xaxis), objRefNS:xaxispos";
                  %end;
               %end;    
               put "));";
               put "Column(""Avg Sig Index for Diffs"")<<Eval Formula;";
               /* kemicl - remove and change if contains check to only create the column
                  put 'Column("Avg Sig Index for Diffs")<<set each value(Mean(';
                     %do i = 1 %to &nDiff;
                        put "   if (Is Missing(:Name(""%str(&&nlpdlab&i)""))==1,.,";
                        put "      :Name(""%str(&&nlpdlab&i)"") > cutoffAlias,1,0)" @@;
                        %if &i < &nDiff %then %do;
                           put ",";
                        %end;
                     %end;
                  put ' ';
                  put ')));'; 
               */
               %if &LS_Index=3 %then %do;
                  put "currentdatatable(objRefNS:adata1);";
                  put "for each row(";
                  put "   if (:Name(""Sum Sig Index for Diffs"")>0,labeled(rowstate())=1)";
                  put ");";
               %end;
            %end;
            %else %if &LS_Index=3 %then %do;
               put "currentdatatable(objRefNS:adata1);";
               put "for each row(";
               put "   if (:Name(""&trSigIndex %substr(&nlpdlab1,17)"")>0,labeled(rowstate())=1)";
               put ");";
            %end;
         %end;
         %if &HasEst %then %do;
            %do i = 1 %to &nEst;
               %let nCat = %eval(&nCat+1);
               %let SigLabel = &trSigIndex %substr(&&nlpelab&i,17);
               put "If(!Contains(_colnames_,""&SigLabel""),";
               put '  objRefNS:adata1 << NewColumn(' @@;
               put "  %str(%"&SigLabel%")," @@;
               put "  Nominal, Width(1), " @@;
               put "   <<Set Property(""SAS Name"",Sig_Index_for_Est&i)," ; 
               put "   <<Set Property(""SAS Label"",:Name(""&SigLabel"")))" @@; 
               put ");" ; /* close If stmnt */
               put "Eval(Substitute(Expr(Column(""&SigLabel"")<<set formula( if (Is Missing(:Name("" " @@;
               put "%str(&&nlpelab&i)" @@;
               put '"))==1,., :Name("' @@;
               put "%str(&&nlpelab&i)" @@;
               put '") > _cutoffAlias' @@;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                        put " & abs(:Name(""&&elabel&i"")) > abs(_xaxis)";
                     %end;
                     %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                        put " & :Name(""&&elabel&i"") > abs(_xaxis)";
                     %end;
                     %else %do;
                        put " & :Name(""&&elabel&i"") < -abs(_xaxis)";
                     %end; 
                  %end;
               %end;
               put ', 1,0) ))'; /* end expr */ 
               %if &indTestAdj=no %then %do;
                  put ",Expr(_cutoffAlias), cutoffAlias" @@;
               %end;
               %else %do;
                  %let j = %eval(&i+&nDiff);
                  put ",Expr(_cutoffAlias), &&NegLog10pCutoff&j" @@;
               %end;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     put ", Expr(_xaxis), objRefNS:xaxispos";
                  %end;
               %end;               
               put "));";
               put "Column(""&SigLabel"")<<Eval Formula;";
            %end;
            * compute sum and average of sig indices;
            %if &nEst > 1 %then %do;
               put "If(!Contains(_colnames_, ""Sum Sig Index for Estimates""),";          
               put " objRefNS:adata1 << NewColumn(""Sum Sig Index for Estimates"");";
               put ");";
               put "Eval(Substitute(Expr(Column(""Sum Sig Index for Estimates"")<<set formula(Sum(";
               %do i = 1 %to &nEst;
                  put "   if (Is Missing(:Name(""%str(&&nlpelab&i)""))==1,.,";
                  put "      :Name(""%str(&&nlpelab&i)"") >  " @@;
                  %if &indTestAdj = no %then %do;
                     put " _cutoffAlias" @@;
                  %end;
                  %else %do;
                     put " _cutoffAlias&i" @@;
                  %end;
                  %if %symexist(UseXaxisFilter) %then %do;
                     %if %Upcase(&UseXaxisFilter)=YES %then %do;
                        %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                           put " & abs(:Name(""&&elabel&i"")) > abs(_xaxis)";
                        %end;
                        %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                           put " & :Name(""&&elabel&i"") > abs(_xaxis)";
                        %end;
                        %else %do;
                           put " & :Name(""&&elabel&i"") < -abs(_xaxis)";
                        %end; 
                     %end;
                  %end;
                   put ', 1,0)';
                  %if &i < &nEst %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put ')))';
               %if &indTestAdj=no %then %do;
                  put ", Expr(_cutoffAlias), cutoffAlias" @@;
               %end;
               %else %do;
                  %do i = 1 %to &nEst;
                     %let j = %eval(&i+&nDiff);
                     put ", Expr(_cutoffAlias&i), &&NegLog10pCutoff&j" @@;
                  %end;
               %end;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     put ", Expr(_xaxis), objRefNS:xaxispos";
                  %end;
               %end;                   
               put "));";
               put "Column(""Sum Sig Index for Estimates"")<<Eval Formula;";
               /* end of Sum sig index column */
               put "If(!Contains(_colnames_, ""Avg Sig Index for Estimates""),";
               put " objRefNS:adata1 << NewColumn(""Avg Sig Index for Estimates"");";
               put ");";
               put "Eval(Substitute(Expr(Column(""Avg Sig Index for Estimates"")<<set formula(Mean(";
               %do i = 1 %to &nEst;
                  put "   if (Is Missing(:Name(""%str(&&nlpelab&i)""))==1,.,";
                  put "      :Name(""%str(&&nlpelab&i)"") >  " @@;
                  %if &indTestAdj = no %then %do;
                     put " _cutoffAlias" @@;
                  %end;
                  %else %do;
                     put " _cutoffAlias&i" @@;
                  %end;
                  %if %symexist(UseXaxisFilter) %then %do;
                     %if %Upcase(&UseXaxisFilter)=YES %then %do;
                        %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                           put " & abs(:Name(""&&elabel&i"")) > abs(_xaxis)";
                        %end;
                        %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                           put " & :Name(""&&elabel&i"") > abs(_xaxis)";
                        %end;
                        %else %do;
                           put " & :Name(""&&elabel&i"") < -abs(_xaxis)";
                        %end; 
                     %end;
                  %end;
                   put ', 1,0)';
                  %if &i < &nEst %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put ')))';
               %if &indTestAdj=no %then %do;
                  put ", Expr(_cutoffAlias), cutoffAlias" @@;
               %end;
               %else %do;
                   %do i = 1 %to &nEst;
                     %let j = %eval(&i+&nDiff);
                     put ", Expr(_cutoffAlias&i), &&NegLog10pCutoff&j" @@;
                   %end;
               %end; 
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     put ", Expr(_xaxis), objRefNS:xaxispos";
                  %end;
               %end;                   
               put "));"; 
               put "Column(""Avg Sig Index for Estimates"")<<Eval Formula;";
            %end;
         %end;
         %if &HasCon %then %do;
            %do i = 1 %to &nCon;
               %let nCat = %eval(&nCat+1);
               %let SigLabel = &trSigIndex %substr(&&pclabel&i,7);
               put "If(!Contains(_colnames_,""&SigLabel""),";
               put '  objRefNS:adata1 << NewColumn(' @@;
               put "  %str(%"&SigLabel%")," @@;
               put "  Nominal, Width(1), " @@;
               put "   <<Set Property(""SAS Name"",Sig_Index_for_Con&i)," @@; 
               put "   <<Set Property(""SAS Label"",:Name(""&SigLabel"")))" @@; 
               put ");" ; /* close If stmnt */
               put "Column(""&SigLabel"")<<set each value( if (Is Missing(:Name("" " @@;
               put "%str(&&pclabel&i)" @@;
               put '"))==1,., :Name("' @@;
               put "%str(&&pclabel&i)" @@;
               put '") < pcutoffAlias,1,0) );';
            %end;
            * compute sum and average of sig indices;
            %if &nCon > 1 %then %do;
               put "If(!Contains(_colnames_, ""Sum Sig Index for Contrasts""),";
               put 'objRefNS:adata1 << NewColumn("Sum Sig Index for Contrasts", set each value(Sum(';
               %do i = 1 %to &nCon;
                  put "   if (Is Missing(:Name(""%str(&&pclabel&i)""))==1,.,";
                  put "      :Name(""%str(&&pclabel&i)"") < pcutoffAlias,1,0)" @@;
                  %if &i < &nCon %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put '))));';
               put "If(!Contains(_colnames_, ""Avg Sig Index for Contrasts""),";
               put 'objRefNS:adata1 << NewColumn("Avg Sig Index for Contrasts", set each value(Mean(';
               %do i = 1 %to &nCon;
                  put "   if (Is Missing(:Name(""%str(&&pclabel&i)""))==1,.,";
                  put "      :Name(""%str(&&pclabel&i)"") < pcutoffAlias,1,0)" @@;
                  %if &i < &nCon %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put '))));';
            %end;
         %end;
      %end;
      %else %if &HasSlice %then %do;
         %let nCat = &nSlice;
         %do i = 1 %to &nSlice;
            %let SigLabel = &trSigIndex %substr(&&pslabel&i,7);
            put "If(!Contains(_colnames_,""&SigLabel""),";
            put '  objRefNS:adata1 << NewColumn(' @@;
            put "  %str(%"&SigLabel%")," @@;
            put "  Nominal, Width(1), " @@;
            put "   <<Set Property(""SAS Name"",Sig_Index_for_Slice&i)," @@; 
            put "   <<Set Property(""SAS Label"",:Name(""&SigLabel"")))" @@; 
            put ");" ; /* close If stmnt */
            put "Column(""&SigLabel"")<<set each value( if (Is Missing(:Name("" " @@;
            put "%str(&&pslabel&i)" @@;
            put '"))==1,., :Name("' @@;
            put "%str(&&pslabel&i)" @@;
            put '") < pcutoffAlias,1,0) );';
         %end;
         * compute sum and average of sig indices;
         %if &nSlice > 1 %then %do;
            put "If(!Contains(_colnames_, ""Sum Sig Index for Slices""),";
            put 'objRefNS:adata1 << NewColumn("Sum Sig Index for Slices", set each value(Sum(';
            %do i = 1 %to &nSlice;
               put "   if (Is Missing(:Name(""%str(&&pslabel&i)""))==1,.,";
               put "      :Name(""%str(&&pslabel&i)"") < pcutoffAlias,1,0)" @@;
               %if &i < &nSlice %then %do;
                  put ",";
               %end;
            %end;
            put ' ';
            put '))));';
            put "If(!Contains(_colnames_, ""Avg Sig Index for Slices""),";
            put 'objRefNS:adata1 << NewColumn("Avg Sig Index for Slices", set each value(Mean(';
            %do i = 1 %to &nSlice;
               put "   if (Is Missing(:Name(""%str(&&pslabel&i)""))==1,.,";
               put "      :Name(""%str(&&pslabel&i)"") < pcutoffAlias,1,0)" @@;
               %if &i < &nSlice %then %do;
                  put ",";
               %end;
            %end;
            put ' ';
            put '))));';
         %end;
      %end;
      %else %if &HasT3 %then %do;
         %let nCat = &nFstat;
         %if %index(&ProcessName,OneWay) %then %do;
            %let SigLabel = &trSigIndex PrF_T3_&gName;
            put "If(!Contains(_colnames_,""&SigLabel""),";
            put '  objRefNS:adata1 << NewColumn(' @@;
            put "  %str(%"&SigLabel%")," @@;
            put "  Nominal, Width(1), " @@;
            put "   <<Set Property(""SAS Name"",Sig_Index_for_F_Test)," @@; 
            put "   <<Set Property(""SAS Label"",:Name(""&SigLabel"")))" @@; 
            put ");" ; /* close If stmnt */
            put "Column(""&SigLabel"")<<set each value( if (Is Missing(:Name("" " @@;
            put "%str(PrF_T3_&gName)" @@;
            put '"))== 1,., :Name("' @@;
            put "%str(PrF_T3_&gName)" @@;
            put '") < pcutoffAlias,1,0) );'; 
         %end;
         %else %do;
            %do i = 1 %to &nFstat;
               %let SigLabel = &trSigIndex &&PrF_T3_&i;
               put "If(!Contains(_colnames_,""&SigLabel""),";
               put '  objRefNS:adata1 << NewColumn(' @@;
               put "  %str(%"&SigLabel%")," @@;
               put "  Nominal, Width(1), " @@;
               put "   <<Set Property(""SAS Name"",Sig_Index_for_F_Test&i)," @@; 
               put "   <<Set Property(""SAS Label"",:Name(""&SigLabel"")))" @@; 
               put ");" ; /* close If stmnt */
               put "Column(""&SigLabel"")<<set each value( if (Is Missing(:Name("" " @@;
               put "%str(&&PrF_T3_&i)" @@;
               put '"))== 1,., :Name("' @@;
               put "%str(&&PrF_T3_&i)" @@;
               put '") < pcutoffAlias,1,0) );';
            %end;
            * compute sum and average of sig indices;
            %if &nFStat > 1 %then %do;
               put "If(!Contains(_colnames_, ""Sum Sig Index for F-Tests""),";
               put 'objRefNS:adata1 << NewColumn("Sum Sig Index for F-Tests", set each value(Sum(';
               %do i = 1 %to &nFStat;
                  put "   if (Is Missing(:Name(""%str(&&PrF_T3_&i)""))==1,.,";
                  put "      :Name(""%str(&&PrF_T3_&i)"") < pcutoffAlias,1,0)" @@;
                  %if &i < &nFStat %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put '))));';
               put "If(!Contains(_colnames_, ""Avg Sig Index for F-Tests""),";
               put 'objRefNS:adata1 << NewColumn("Avg Sig Index for F-Tests", set each value(Mean(';
               %do i = 1 %to &nFStat;
                  put "   if (Is Missing(:Name(""%str(&&PrF_T3_&i)""))==1,.,";
                  put "      :Name(""%str(&&PrF_T3_&i)"") < pcutoffAlias,1,0)" @@;
                  %if &i < &nFStat %then %do;
                     put ",";
                  %end;
               %end;
               put ' ';
               put '))));';
            %end;
         %end;
      %end;

      %if %upcase(&SplitResults)=YES %then %do;
         * define list for &DesignByVars;
         %do i = 1 %to &ndbv;
            put "LvlOfFtr&i = {" @@;
            %do j = 1 %to &&DBVnL&i;
               put '"' "%scan(&&DBVList&i,&j)" '"'@@;
               %if &j < &&DBVnL&i %then put ',' @@;;
            %end;
            put '};';
         %end; 
         put "objRefNS:combinedLvl = {" @@; *list in list;
            %do i = 1 %to &TotalCLvl_DBV;
               put "{" @@;
               %do j = 1 %to &ndbv;
                  put '"' "&&Ftr&j._Lvl&i" '"'@@; 
                  %if &j ne &ndbv %then put ', ' @@;; 
               %end;
               put "}" @@; 
               %if &i ne &TotalCLvl_DBV %then put ', ' @@;; 
            %end;
         put '};';
         put 'objRefNS:sigdata = {' @@;
         %do j = 1 %to &TotalCLvl_DBV;
            put '"' "objRefNS:data&j" '"' @@;
            %if &j < &TotalCLvl_DBV %then put ',' @@;;
         %end;
         put '};';
         
         put 'objRefNS:Nsigl = {' @@; 
         %do j = 1 %to &TotalCLvl_DBV;
            put '"' "nSig&j" '"' @@;
            %if &j < &TotalCLvl_DBV %then put ',' @@;;
         %end;
         put '};';
                  
         put 'tSig = 0;';
         *put "for (i= 1, i <= &ndbv, i++,"; * limit to one factor for now; *allow multiple in v4.0;
         *put "   for (j= 1, j<=N Items(LvlOfFtr1), j++,";
         put "for (j = 1, j <= &TotalCLvl_DBV, j++,";
      %end;
      %else %do;
         %let TotLvls = 1;
      %end;
      
      put "   objRefNS:Nsig = 0;";
      put "   for each row( Selected( Rowstate() )=((";
      %if (&HasDiff | &HasEst | &HasCon) %then %do;
         %if &HasDiff %then %do;
            %if &nDiff > 1 %then %do;
               put "    :Name(""Sum Sig Index for Diffs"") > 0 ";
            %end;
            %else %do;
               put " ( " @@;  /* for and statement */
               %if &indTestAdj=no %then %do;
                  put "    :Name(""&nlpdlab1"") > objRefNS:cutoff";
               %end;
               %else %do;
                  put "    :Name(""&nlpdlab1"") > &NegLog10pCutoff1";
               %end;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                        put " & abs(:Name(""&dlabel1"")) > abs(objRefNS:xaxispos)";
                     %end;
                     %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                        put " & :Name(""&dlabel1"") > abs(objRefNS:xaxispos)";
                     %end;
                     %else %do;
                        put " & :Name(""&dlabel1"") < -abs(objRefNS:xaxispos)";
                     %end; 
                  %end;
               %end;
               put " ) ";
            %end;          
            /* kemicl - use the Sum of sig indicators instead 
            %if &indTestAdj=no %then %do;
               put '    :Name("' "&nlpdlab1" '") > objRefNS:cutoff';
            %end;
            %else %do;
               put '    :Name("' "&nlpdlab1" '") > ';
               put "&NegLog10pCutoff1";
            %end;
            %do i = 2 %to &nDiff;
               %if &indTestAdj=no %then %do;
                  put '  | :Name("' "&&nlpdlab&i" '") > objRefNS:cutoff';
               %end;
               %else %do;
                  put '  | :Name("' "&&nlpdlab&i" '") > ';
                  put "&&NegLog10pCutoff&i";
               %end;
            %end;
            */
         %end;
         %if &HasEst %then %do;
            %if &HasDiff %then %do;
               put "  | " @@;
            %end;
            %if &nEst > 1 %then %do;
               put "    :Name(""Sum Sig Index for Estimates"") > 0 ";
            %end;
            %else %do;
               put " ( " @@;  /* for and statement */
               %if &indTestAdj=no %then %do;
                  put "    :Name(""&nlpelab1"") > objRefNS:cutoff";
               %end;
               %else %do;
                  %let j = %eval(1+&nDiff);
                  put "    :Name(""&nlpelab1"") > &&NegLog10pCutoff&j";
               %end;
               %if %symexist(UseXaxisFilter) %then %do;
                  %if %Upcase(&UseXaxisFilter)=YES %then %do;
                     %if %index(%Upcase(&XaxisDirection),ABSOLUTE) %then %do;
                        put " & abs(:Name(""&elabel1"")) > abs(objRefNS:xaxispos)";
                     %end;
                     %else %if %index(%Upcase(&XaxisDirection),POSITIVE) %then %do;
                        put " & :Name(""&elabel1"") > abs(objRefNS:xaxispos)";
                     %end;
                     %else %do;
                        put " & :Name(""&elabel1"") > -abs(objRefNS:xaxispos)";
                     %end; 
                  %end;
               %end;
               put " ) ";
            %end;
            /* kemicl - use the Sum of sig indicators instead 
            %do i = 2 %to &nEst;
               %let j = %eval(&i+&nDiff);
               %if &indTestAdj=no %then %do;
                  put '  | :Name("' "&&nlpelab&i" '") > objRefNS:cutoff';
               %end;
               %else %do;
                  put '  | :Name("' "&&nlpelab&i" '") > ';
                  put "&&NegLog10pCutoff&j";
               %end;
            %end;
            */
         %end;
         %if &HasCon %then %do;
            %if &HasDiff | &HasEst %then %do;
               put "  | " @@;
            %end;
            put '    :Name("' "&pclabel1" '") < objRefNS:pcutoff';
            %do i = 2 %to &nCon;
               put '  | :Name("' "&&pclabel&i" '") < objRefNS:pcutoff';
            %end;
         %end;
      %end;
      %else %if &HasSlice %then %do;
         put '   :Name("' "&pslabel1" '") < objRefNS:pcutoff';
         %do i = 2 %to: &nSlice;
            put  '  | :Name("' "&&pslabel&i" '") < objRefNS:pcutoff';
         %end;
      %end;
      %else %if &HasT3 %then %do;
         * select using type 3 tests of fixed effects;
         %if %index(&ProcessName,OneWay) %then %do;
            put '  :Name("' "PrF_T3_&gName" '") <= objRefNS:pcutoff';
         %end;
         %else %do;
            put '  :Name("' "&PrF_T3_1" '") <= objRefNS:pcutoff';
            %if &nFstat > 1 %then %do;
               %do i = 2 %to &nFstat;
                  put '   | :Name("' "&&PrF_T3_&i" '") < objRefNS:pcutoff';
               %end;
            %end;
         %end;
      %end;       
      put '      )';
      
      %if %upcase(&SplitResults)=YES %then %do;
         %do i = 1 %to &ndbv;
            put '      & char(:Name("' "&&dbv&i" '")[])==objRefNS:combinedLvl[j]' "[&i]"; 
         %end;
      %end; 
      put "      );";
      put "      objRefNS:Nsig = objRefNS:Nsig + Selected(Rowstate());";
      put "   );";
      
      %if %upcase(&SplitResults)=YES %then %do;
         put "   tSig = tSig + objRefNS:Nsig;";
         put "   objRefNS:Nsigl[j]=objRefNS:Nsig;";
         put "   show(j,objRefNS:Nsig);";
      %end;
      
      put " ";
      

      put "   /*---create sig diffs table and graphs only if there is at least one significant gene---*/";
      
      put '   if (objRefNS:Nsig==0,';
      %if ^&BA_Index & ^&SA_Index %then %do;
         put 'nw = New Window(locale:get("JMPLifeSciencesMessage_Text") || "' @@;
         %if %upcase(&SplitResults)=YES %then %do;
            put ': "'@@;
            %do i = 1 %to &ndbv; 
               put '||"' " &&dbv&i=" '"||objRefNS:combinedLvl[j]' "[&i]" @@; 
            %end;
            put ',';
         %end;
         %else %do;
            put '",';
         %end;
         put '      TextBox(locale:fill(locale:get("NoteNoSigDiffsFoundAtCutoffXMsg_Text"), {char(objRefNS:cutoff)})),';
         put '      );';
         put '      //nw << SetWindowIcon("$LS_HOME/Documentation/Icons/JMPClinicalApp.ico");';
      %end;
      put '      , // else';
      %if %upcase(&SplitResults)=YES %then %do;
         put '      objRefNS:sigdata[j] = objRefNS:adata1<<Subset(output table name( "Significant rows with at least one -log10(p) >" || char(objRefNS:cutoff) '@@;
         put '||", "' @@; 
         %do i = 1 %to &ndbv;
            put '||"' " &&dbv&i=" '"||objRefNS:combinedLvl[j]' "[&i]" @@; 
         %end;
         put '), invisible,Linked);'; 
         put '      if (length(objRefNS:outdsname) >21,';
         put '         objRefNS:outdsname = substr(objRefNS:outdsname,1,length(objRefNS:outdsname)-17)||substr(objRefNS:outdsname,-6);';
         put '      );';
         put "      if(j==1,if(isEmpty(jg:JMPClinicalLiteMode),SAS Export Data(objRefNS:adata1,""Work"",objRefNS:outdsname,ReplaceExisting)););";
         put "      if(isEmpty(jg:JMPClinicalLiteMode),SAS Export Data(objRefNS:sigdata[j],""Work"",objRefNS:outdsname||""_sig""||char(j),ReplaceExisting););";
         put "      if(jg:sasserver!=""Local"",";
         put "         if(j==1,jg:SASGetFile2(objRefNS:serveroutpath||objRefNS:outdsname||"".sas7bdat"",objRefNS:clientoutpath||objRefNS:outdsname||"".sas7bdat""));";
         put "         jg:SASGetFile2(objRefNS:serveroutpath||objRefNS:outdsname||""_sig""||char(j)||"".sas7bdat"",objRefNS:clientoutpath||objRefNS:outdsname||""_sig""||char(j)||"".sas7bdat"");";
         put "       ,";
         put '       current data table(objRefNS:sigdata[j]);';
         put "       current data table() <<save(objRefNS:clientoutpath||objRefNS:outdsname||""_sig""||char(j)||"".sas7bdat"");";
         put "      );";
         put '      objRefNS:sigdata[j]<<clear select;';
         put '      objRefNS:sigdata[j]<<Minimize Window;';
         put '      current data table(objRefNS:adata1);';
         put '      // current data table() <<save(objRefNS:clientoutpath||objRefNS:outdsname||".sas7bdat");';
         put '      objRefNS:adata1<<clear select;';
         put '   )'; 
         put ');'; * end of loop j in jsl;
      %end;
      %else %do;
         put '   objRefNS:sigdata = objRefNS:adata1<<Subset(output table name( "Significant rows with at least one -log10(p) >" || char(objRefNS:cutoff) ), invisible,Linked);';
         put '   if (length(objRefNS:outdsname) >28,';
         put '      objRefNS:outdsname = substr(objRefNS:outdsname,1,length(objRefNS:outdsname)-12)||substr(objRefNS:outdsname,-6);';
         put '      objRefNS:outdsname = substr(objRefNS:outdsname,1,28);'; 
         put '   );';
         put '   // show(objRefNS:outdsname);';
         put '   if(isEmpty(jg:JMPClinicalLiteMode),SAS Export Data(objRefNS:adata1,"Work",objRefNS:outdsname,ReplaceExisting););';
         put '   if(isEmpty(jg:JMPClinicalLiteMode),SAS Export Data(objRefNS:sigdata,"Work",objRefNS:outdsname||"_sig",ReplaceExisting););';
         put '   // print("after export");';
         put '   if(jg:sasserver!="Local",';
         put '       jg:SASGetFile2(objRefNS:serveroutpath||objRefNS:outdsname||".sas7bdat",objRefNS:clientoutpath||objRefNS:outdsname||".sas7bdat");';
         put '       jg:SASGetFile2(objRefNS:serveroutpath||objRefNS:outdsname||"_sig.sas7bdat",objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat");';
         put '    ,';
         put '    current data table(objRefNS:sigdata);';
         put '    objRefNS:sigdata <<save(objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat");';
         put '    current data table(objRefNS:adata1);';
         put '    objRefNS:adata1 <<save(objRefNS:clientoutpath||objRefNS:outdsname||".sas7bdat");';
         put '   );';
         put '   objRefNS:adata1<<clear select;';
         put '   current data table(objRefNS:sigdata);';
         put ');';
      %end;
      put ' ';
   
      * parallel, pca, and clustering;
      %if &HasLsm and %upcase(&ClusterLSM)=YES %then %do;
         %if %upcase(&SplitResults)=YES %then put 'if (tSig > 0,';
         %else put 'if (objRefNS:Nsig > 0,';;
         %if %length(&LSMeanStdMethod) %then %do;
            %let SpeakDes = hierarchical clustering of standardized least squares means;
            %let TitleDes = Hierarchical Clustering of Standardized LSMeans;
            %let mVar = lslabel;
         %end;
         %else %do;
            %let SpeakDes = hierarchical clustering of least squares means;
            %let TitleDes = Hierarchical Clustering of LSMeans;
            %let mVar = llabel;
         %end;
         %if (%upcase(&Speak) = YES) %then %do;
            put "   speak(wait(1),""&SpeakDes"");";
         %end;

         %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
            put '   textboxChr= TextBox("Chromosome "|| objRefNS:ChrVarInd);';
            put '   textboxChr << set font size(11);';
         %end;
         
         put " if(""&TitleDes"" == ""Hierarchical Clustering of LSMeans"", objRefNS:TransTitleDes = locale:get(""HierarchicalClusteringofLSMeans_Text""), ";
         put "    ""&TitleDes"" == ""Hierarchical Clustering of Standardized LSMeans"", objRefNS:TransTitleDes = locale:get(""HierarchicalClusteringofStandardizedLSMeans_Text""), ";
         put "    objRefNS:TransTitleDes = ""&TitleDes""); ";
         
         %if %upcase(&SplitResults) ne YES %then %do;
            put "objRefNS:vb = Function({},{default local}," ;
            
            put "   If(IsEmpty(objRefNS:sigdata),";
            put '      objRefNS:sigdata=open(objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat",invisible);';
            put '   );';
            put "   Current Data Table(objRefNS:sigdata);";
            
            put "   objRefNS:hcVlist=Vlistbox(";
            put "      objRefNS:hfsize = 180;";
            put "      objRefNS:hiclus = Hierarchical Cluster(Y(";
            %do i = 1 %to %eval(&nLsm - 1);
               put '         :Name("' "&&&mVar.&i" '"),';
            %end;
            put '         :Name("' "&&&mVar.&i" '")),';
            %if %length(&AnnoLabel) %then %do;
               %if %length(%trim(%quote(&ALabel))) = 0 %then %do;
                  put '         Label(:Name("'"&AnnoLabVar"'")),';
               %end;
               %else %do;
                  put '         Label(:Name("'":&ALabel"'")),'; 
               %end;
            %end;
            put "            Method(Fast Ward), Standardize(0), Missing value imputation(1), Geometric X Scale(1), Color Clusters(1),";
            put '            Title(objRefNS:TransTitleDes), Two way clustering,';
            put "            Number of Clusters(5), Dendrogram Scale(""Distance Scale""), Color Map(Blue to Gray to Red), Legend(1),";
            put "             SendToReport(";
            put "                Dispatch(";
            put "                   {""Dendrogram""},""Clust Dendro"", FrameBox(2),";
            put "                   {Frame Size(130, 0.75*objRefNS:hfsize)}";
            put "                ),";
            put "                Dispatch(";
            put "                   {""Dendrogram""},""Clust Dendro"", FrameBox(5),";
            put "                   {Frame Size(130, 55)}";
            put "                )";
            put "             )";            
            put "         );";
            put '   );';
            
            
            
            put "   objRefNS:hctt=Vlistbox(";
            
            
            put "  clsterevi=EvalInsert(""\[";
            put "      objRefNS:clstdiv=objRefNS:hiclus<<report;";
            put "      objRefNS:clstdiv[CellPlotBox(1)]<<FrameSize(objRefNS:hfsize,0.75*objRefNS:hfsize);";
            put '      objRefNS:ncframeSizeSlider = SliderBox(num(100),num(700),objRefNS:hfsize,';
            put '         objRefNS:clstdiv[CellPlotBox(1)]<<FrameSize(objRefNS:hfsize,0.75*objRefNS:hfsize);';
            put '            );';
            put '  ]\"); ';
            put '           try( HListBox(TextBox(" " || locale:get("FrameSizeforGraphsBelow_Text")),';
            put "    Eval(Parse(clsterevi)));";
            put "    // Eval(Parse(clstering)));";
            put '            ),';
            put "      objRefNS:hcVlist;";
            put "    );";
            put "   );";
         %end;
         %else %do; 
            put "objRefNS:vb = Function({},{default local}," ;
            
            put '      objRefNS:hcVlist=VListBox(';
            put '      VListBox(';
            %do nCL = 1 %to &nldbv;
               put "         if(objRefNS:sigdata[&nCL]!=""objRefNS:data&nCL"",";
               put "            current data table(objRefNS:sigdata[&nCL]);";
               put "            objRefNS:hiclus&nCL = Hierarchical Cluster(Y(";
               %do i = 1 %to %eval(&&nLsm&nCL - 1);
                  put '               :Name("' "&&&mVar.&nCL._&i" '"),';
               %end;
               put '               :Name("' "&&&mVar.&nCL._&i" '")),';
               %if %length(&AnnoLabel) %then %do;
                  %if %length(%trim(%quote(&ALabel))) = 0 %then %do;
                     put '               Label(:Name("'"&AnnoLabVar"'")),';
                  %end;
                  %else %do;
                     put '               Label(:Name("'":&ALabel"'")),'; 
                  %end;
               %end;
               put "               Method(Fast Ward), Standardize(0), Missing value imputation(1), Geometric X Scale(1), Color Clusters(1),";
               
               put '               Title(objRefNS:TransTitleDes || ", "' @@;
               %do i = 1 %to &ndbv;
                  put '||"' "&&dbv&i=" '"||' "objRefNS:combinedLvl[&nCL][&i]" '||" "' @@;
               %end; 
               put '),Two way clustering,';
               put "            Number of Clusters(5), Dendrogram Scale(""Distance Scale""), Color Map(Blue to Gray to Red), Legend(1),";
               put "             SendToReport(";
               put "                Dispatch(";
               put "                   {""Dendrogram""},""Clust Dendro"", FrameBox(2),";
               put "                   {Frame Size(130, 0.75*objRefNS:hfsize)}";
               put "                ),";
               put "                Dispatch(";
               put "                   {""Dendrogram""},""Clust Dendro"", FrameBox(5),";
               put "                   {Frame Size(130, 55)}";
               put "                )";
               put "             )";
               put ")";  /* end cluster */         
               %if &nCL < &nldbv %then %do;
                  put "         ),";
               %end;
               %else %do;
                  put "         )";
               %end;
            %end;
            put '      );';
            put '   );';
            put "   objRefNS:hctt=Vlistbox(";
            
            put '            VListBox(';
            
            put "  clsterevi=EvalInsert(""\[";
            %do nCL = 1 %to &nldbv;
               put "if(objRefNS:Nsigl[&nCL]>0,";
               put "      objRefNS:clstdiv&nCL=objRefNS:hiclus&nCL<<report;";
               put "      objRefNS:clstdiv&nCL[CellPlotBox(1)]<<FrameSize(objRefNS:hfsize,0.75*objRefNS:hfsize);";
               put '            );';
            %end;
            put '               objRefNS:ncframeSizeSlider = SliderBox(num(100),num(700),objRefNS:hfsize,';
            %do nCL = 1 %to &nldbv;
               put "if(objRefNS:Nsigl[&nCL]>0,";
               put "      objRefNS:clstdiv&nCL[CellPlotBox(1)]<<FrameSize(objRefNS:hfsize,0.75*objRefNS:hfsize);";
               put '            );';
            %end;
            put '            );';
            put '  ]\"); ';
            
            put '     try( HListBox(TextBox(" " || locale:get("FrameSizeforGraphsBelow_Text")),';
            put "     Eval(Parse(clsterevi)));";
            put "    // Eval(Parse(clstering)));";
            put '            ),';
            put "      objRefNS:hcVlist;";
            put "    );";
            
            put '   ));';
         %end;
         
         put " "; 
         
         put "objRefNS:ppca = Function({},{default local}," ;
         %if (%upcase(&Speak) = YES) %then %do;
            put '      speak(wait(1),"parallel and principal components plots");';
         %end;
         %if %upcase(&SplitResults) ne YES %then %do;
            put "   If(IsEmpty(objRefNS:sigdata),";
            put '      objRefNS:sigdata=open(objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat",invisible);';
            put '   );';
            put '   current data table(objRefNS:sigdata);';
         %end;
         put '      objRefNS:paralLB=VListBox(';
         %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
            put '   textboxChr= TextBox("Chromosome "|| objRefNS:ChrVarInd),';
            put '   textboxChr << set font size(11),';
         %end;

         %if %upcase(&SplitResults) ne YES %then %do;
            %if %length(&LSMeanStdMethod) %then %do;
               put "            objRefNS:pplotstdlsm = Parallel Plot(Y(";
               %do i = 1 %to %eval(&nLsm - 1);
                  put '            :Name("' "&&lslabel&i" '"),';
               %end;
               put '               :Name("' "&&lslabel&i" '")),';
               put "               Scale Uniformly,";
               put "               SendToReport(";
               put "                  Dispatch(";
               put "                     {},";
               put "                     ""Parallel Coord"",";
               put "                     FrameBox,";
               put "                     Marker Drawing Mode( ""Fast"" )";
               put "                  )";
               put "               ),";
               put '               Title(locale:get("ParallelPlotofStdLSMeans_Text"))';
               put '            ),';
            %end;
            put "            objRefNS:pplotlsm = Parallel Plot(Y(";
            %do i = 1 %to %eval(&nLsm - 1);
               put '            :Name("' "&&llabel&i" '"),';
            %end;
            put '               :Name("' "&&llabel&i" '")),';
            put "               Scale Uniformly,";
            put "               SendToReport(";
            put "                  Dispatch(";
            put "                     {},";
            put "                     ""Parallel Coord"",";
            put "                     FrameBox,";
            put "                     Marker Drawing Mode( ""Fast"" )";
            put "                  )";
            put "               ),";
            put '               Title(locale:get("ParallelPlotofLSMeans_Text"))';
            put '            );'; 
         %end;
         %else %do;
            
            %if %length(&LSMeanStdMethod) %then %do;
               put '         HListBox(';
               %do nCL = 1 %to  &nldbv;
                  put "            if(objRefNS:sigdata[&nCL]!=""objRefNS:data&nCL"",";
                  put "               current data table(objRefNS:sigdata[&nCL]);";
                  put "               objRefNS:pplotstdlsm&nCL = Parallel Plot(Y(";
                  %do i = 1 %to %eval(&&nLsm&nCL - 1);
                     put '                  :Name("' "&&lslabel&nCL._&i" '"),';
                  %end;
                  put '                  :Name("' "&&lslabel&nCL._&i" '")),';
                  put "                  Scale Uniformly,";
                  put "                  SendToReport(";
                  put "                     Dispatch(";
                  put "                        {},";
                  put "                        ""Parallel Coord"",";
                  put "                        FrameBox,";
                  put "                        Marker Drawing Mode( ""Fast"" )";
                  put "                     )";
                  put "                  ),";
                  put '                  Title(locale:get("ParallelPlotofStdLSMeans_Text")||", "' @@;
                  %do i = 1 %to &ndbv;
                     put '||"' "&&dbv&i=" '"||' "objRefNS:combinedLvl[&nCL][&i]" '||" "' @@;  
                  %end;
                  put ')';
                  put '               )';
                  %if &nCL < &nldbv %then %do;
                     put '            ),';
                  %end;
                  %else %do;
                     put '            )';
                  %end;
               %end; 
               put '      ),';
            %end;
            put '         HListBox(';
            %do nCL = 1 %to &nldbv;
               put "            if(objRefNS:sigdata[&nCL]!=""objRefNS:data&nCL"",";
               put "               current data table(objRefNS:sigdata[&nCL]);";
               put "               objRefNS:pplotlsm&nCL  = Parallel Plot(Y(";
               %do i = 1 %to %eval(&&nLsm&nCL - 1);
                  put '               :Name("' "&&llabel&nCL._&i" '"),';
               %end;
               put '               :Name("' "&&llabel&nCL._&i" '")),';
               put "               Scale Uniformly,";
               put "               SendToReport(";
               put "                  Dispatch(";
               put "                     {},";
               put "                     ""Parallel Coord"",";
               put "                     FrameBox,";
               put "                     Marker Drawing Mode( ""Fast"" )";
               put "                  )";
               put "               ),";
               put '               Title(locale:get("ParallelPlotofLSMeans_Text")||", "' @@;
               %do i = 1 %to &ndbv;
                  put '||"' "&&dbv&i=" '"||' "objRefNS:combinedLvl[&nCL][&i]" '||" "' @@; 
               %end;
               put ')';
               put '               )';
               %if &nCL < &nldbv %then %do;
                  put '            ),';
               %end;
               %else %do;
                  put '            )';
               %end;
            %end; 
            put '         ),'; * end of HListBox;
         %end;

         
         %if %length(&LSMeanStdMethod) %then %do;
            %let mVar = lslabel;
            %let PCADes = Principal Components of Std LSMeans;
         %end;
         %else %do;
            %let mVar = llabel;
            %let PCADes = Principal Components of LSMeans;
         %end;
         
         put " if(""&PCADes"" == ""Principal Components of LSMeans"", TransPCADes = locale:get(""PrincipalComponentsofLSMeans_Text""), ";
         put "    ""&PCADes"" == ""Principal Components of Std LSMeans"", TransPCADes = locale:get(""PrincipalComponentsofStdLSMeans_Text""), ";
         put "    TransPCADes = ""&PCADes""); ";
         
         %if %upcase(&SplitResults) ne YES %then %do;
            put '               VListBox(';
            put '                  Multivariate(Y(';
            %do i = 1 %to %eval(&nLsm - 1);
               put '                  :Name("' "&&&mVar.&i" '"),';
            %end;
            put '                  :Name("' "&&&mVar.&i" '")),';
            put "                  Principal Components(on Correlations, Save Principal Components(2)) ));";
            put "               // set names of pc columns to override regional settings";
            put "               cdt = CurrentDataTable();";
            put "               nc = NCol(cdt);";
            put "               Column(cdt,nc-1) << SetName(""Prin1"");";
            put "               Column(cdt,nc) << SetName(""Prin2"");";
            put '               if(isEmpty(jg:JMPClinicalLiteMode),SAS Export Data(objRefNS:sigdata,"Work",objRefNS:outdsname||"_sig",ReplaceExisting););';
            put '               if(jg:sasserver!="Local",';
            put '                   jg:SASGetFile2(objRefNS:serveroutpath||objRefNS:outdsname||"_sig.sas7bdat",objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat");';
            put "                ,";
            put '                cdt <<save(objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat");';
            put '               );';
            put '               current data table(objRefNS:sigdata);';
            put '               objRefNS:pcabiv=Bivariate(X(:Prin1),Y(:Prin2),Title(TransPCADes));';
            put "     //        objRefNS:ppcabiv[framebox(1)]<<MarkerSize(4);";
            put "     //        objRefNS:ppcabiv[framebox(1)]<<Marker Drawing Mode(Outlined);";
            put '      );'; * end of VListBox;
            
            put "   objRefNS:pptt=Vlistbox(";
            put '            VListBox(';
            put "  pplot=EvalInsert(""\[";
            put '               objRefNS:ppframeSizeSlider = SliderBox(num(100),num(700),objRefNS:pfsize,';
            %if %length(&LSMeanStdMethod) %then %do;
                  put "      objRefNS:pplotstdiv= objRefNS:pplotstdlsm<<report;";
                  put '      objRefNS:pplotstdiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);';
            %end;
            put "      objRefNS:pplotiv=objRefNS:pplotlsm<<report;";
            put "      objRefNS:ppcabiv= objRefNS:pcabiv<<report;";
            put '                  objRefNS:pplotiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);';
            put '                  objRefNS:ppcabiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);';
            put '            );';
            put '  ]\"); ';
            put "  ppevi=EvalInsert(""\[";
            %if %length(&LSMeanStdMethod) %then %do;
               put "      objRefNS:pplotstdiv= objRefNS:pplotstdlsm<<report;";
               put "      objRefNS:pplotstdiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            %end;
            put "      objRefNS:pplotiv=objRefNS:pplotlsm<<report;";
            put "      objRefNS:pplotiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            put "      objRefNS:ppcabiv= objRefNS:pcabiv<<report;";
            put "      objRefNS:ppcabiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            %if &LS_Index=3 %then %do;
               put "      objRefNS:ppcabiv[framebox(1)]<<MarkerSize(3);";
               put "      objRefNS:ppcabiv[framebox(1)]<<Marker Drawing Mode(Outlined);";
            %end;
            put '  ]\"); ';
            put '           try( HListBox(TextBox(" " || locale:get("FrameSizeforGraphsBelow_Text")),';
            put "  Eval(Parse(ppevi));";
            put "  Eval(Parse(pplot)));";
            put '            ),';
            put "      objRefNS:paralLB;";
            put '            ),';
            put "   );"; * end of VListBox;
            put '   );'; * end of ppca;
         %end;
         %else %do;
            put '         HListBox(';
            %do nCL = 1 %to &nldbv;
               put "            if(objRefNS:sigdata[&nCL]!=""objRefNS:data&nCL"",";
               put "               current data table(objRefNS:sigdata[&nCL]);";
               put '               VListBox(';
               put '                  Multivariate(Y(';
               %do i = 1 %to %eval(&&nLsm&nCL - 1);
                  put '                  :Name("' "&&&mVar.&nCL._&i" '"),';
               %end;
               put '                  :Name("' "&&&mVar.&nCL._&i" '")),';
               put "                  Principal Components(on Correlations, Save Principal Components(2)) ));";
               put "               if(isEmpty(jg:JMPClinicalLiteMode),SAS Export Data(objRefNS:sigdata[&nCL],""Work"",objRefNS:outdsname||""_sig&nCL"",ReplaceExisting););";
               put "               if(jg:sasserver!=""Local"",";
               put "                  jg:SASGetFile2(objRefNS:serveroutpath||objRefNS:outdsname||""_sig&nCL..sas7bdat"",objRefNS:clientoutpath||objRefNS:outdsname||""_sig&nCL..sas7bdat"");";
               put "                ,";
               put "                current data table() <<save(objRefNS:clientoutpath||objRefNS:outdsname||""_sig&nCL..sas7bdat"");";
               put "               );";               
               put "               objRefNS:pcabiv&nCL=Bivariate(X(:Prin1),Y(:Prin2),Title(" '"' "&PCADes" '"||","' @@;
               %do i = 1 %to &ndbv;
                  put '||"' "&&dbv&i=" '"||' "objRefNS:combinedLvl[&nCL][&i]" '||" "' @@; 
               %end;
               put '));';
               %if &LS_Index = 3 %then %do;
                  put "            (objRefNS:pcabiv&nCL<<report)[framebox(1)]<<MarkerSize(4);";
                  put "            (objRefNS:pcabiv&nCL<<report)[framebox(1)]<<Marker Drawing Mode(Outlined);";
               %end;
               %if &nCL < &nldbv %then %do;
                  put '            ),';
               %end;
               %else %do;
                  put '            ),';
               %end;
            %end;
            put '         );'; * end of HListBox;
            put '            );';
            put "   objRefNS:pptt=Vlistbox(";
            put "  pplot=EvalInsert(""\[";
            put '               objRefNS:ppframeSizeSlider = SliderBox(num(100),num(700),objRefNS:pfsize,';
            %do nCL = 1 %to  &nldbv;
               put "if(objRefNS:Nsigl[&nCL]>0,";
               %if %length(&LSMeanStdMethod) %then %do;
                  put "      objRefNS:pplotstdiv&nCL= objRefNS:pplotstdlsm&nCL<<report;";             
                  put "                  objRefNS:pplotstdiv&nCL[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               %end;
               put "                  objRefNS:pplotiv&nCL=objRefNS:pplotlsm&nCL<<report;";
               put "                  objRefNS:ppcabiv&nCL= objRefNS:pcabiv&nCL<<report;";
               put "                  objRefNS:pplotiv&nCL[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               put "                  objRefNS:ppcabiv&nCL[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               put '            );';
            %end;
            put '            );';
            put '  ]\"); ';
               
            /*
            %do nCL = 1 %to  &nldbv;
               %if %length(&LSMeanStdMethod) %then %do;
                  put "                  (objRefNS:pplotstdlsm&nCL<<report)[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               %end;
               put "                  (objRefNS:pplotlsm&nCL<<report)[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               put "                  (objRefNS:pcabiv&nCL<<report)[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            %end;
            put '            )),';
            */
                  
            put "  ppevi=EvalInsert(""\[";
            %do nCL = 1 %to &&TotalCLvl_DBV;
               put "if(objRefNS:Nsigl[&nCL]>0,";
               %if %length(&LSMeanStdMethod) %then %do;
                  put "      objRefNS:pplotstdiv&nCL= objRefNS:pplotstdlsm&nCL<<report;";
                  put "      objRefNS:pplotstdiv&nCL[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               %end;
               put "      objRefNS:pplotiv&nCL=objRefNS:pplotlsm&nCL<<report;";
               put "      objRefNS:pplotiv&nCL[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               put "      objRefNS:ppcabiv&nCL= objRefNS:pcabiv&nCL<<report;";
               put "      objRefNS:ppcabiv&nCL[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
               put "      objRefNS:ppcabiv&nCL[framebox(1)]<<MarkerSize(4);";
               put "      objRefNS:ppcabiv&nCL[framebox(1)]<<Marker Drawing Mode(Outlined);"; 
               put '            );';
            %end;
            put '  ]\"); ';
            put '            HListBox(TextBox(" " || locale:get("FrameSizeforGraphsBelow_Text")),';
            put "  try(Eval(Parse(ppevi));";
            put "  Eval(Parse(pplot)));";
            put '            ),';
            put "      objRefNS:paralLB;";
            
            put '      );'; * end of VListBox;
            put '   );'; * end of objRefNS:ppca;
         %end;
         
         put "objRefNS:aftercode = Function({},{default local}," ;
         put "  pplotvei=EvalInsert(""\[";      
         %if %upcase(&SplitResults)=YES %then %do;

         %end;
   /*    %else %do;
            %if %length(&LSMeanStdMethod) %then %do;
               put "      objRefNS:pplotstdiv= objRefNS:pplotstdlsm<<report;";
               put "      objRefNS:pplotstdiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            %end;
            put "      objRefNS:pplotiv=objRefNS:pplotlsm<<report;";
            put "      objRefNS:pplotiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            put "      objRefNS:ppcabiv= objRefNS:pcabiv<<report;";
            put "      objRefNS:ppcabiv[framebox(1)]<<FrameSize(objRefNS:pfsize,0.75*objRefNS:pfsize);";
            put "      objRefNS:ppcabiv[framebox(1)]<<MarkerSize(4);";
            put "      objRefNS:ppcabiv[framebox(1)]<<Marker Drawing Mode(Outlined);";
         %end;*/
         put '  ]\"); ';
         put "  Eval(Parse(pplotvei));";
         
   /*    put "outpath = ""&OutPath"";";
         %if %upcase(&SplitResults)=YES %then %do;
            %do nCL = 1 %to &&TotalCLvl_DBV;
               put "current data table(objRefNS:sigdata[&nCL]);";
               put 'if (length(objRefNS:outdsname) >21,';
               put '   objRefNS:outdsname = substr(objRefNS:outdsname,1,length(objRefNS:outdsname)-17)||substr(objRefNS:outdsname,-6);';
               put ');';
               put "current data table() <<save(objRefNS:clientoutpath||objRefNS:outdsname||""_sig&nCL..sas7bdat"");";
            %end;
         %end;
         %else %do;
            put 'current data table(objRefNS:sigdata);';
            put 'if (length(objRefNS:outdsname) >26,';
            put '   objRefNS:outdsname = substr(objRefNS:outdsname,1,length(objRefNS:outdsname)-12)||substr(objRefNS:outdsname,-6);';
            put ');';
            put 'objRefNS:sigdata <<save(objRefNS:clientoutpath||objRefNS:outdsname||"_sig.sas7bdat");';
         %end;*/
         put "    //       objRefNS:RefreshTabExpr;";
         put ');';
         put ');'; * end of if objRefNS:Nsig > 0 statement in jsl;
      %end;
   %end;

   * chromosome position plots;
   %if %length(&ChrVar) and %length(&PosVar) and (&HasDiff or &HasEst or 
     (^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice and (&HasT1 or &HasT2 or &HasT3))) %then %do;

      %checklabel(&OutData,ChrVar);
      %checklabel(&OutData,PosVar);
      put " ";
      put 'current data table(objRefNS:adata1);';
      put "ccol = Column(""&ChrVar"");";
      put "ccol << Set Property(""Value Ordering"",{""chr1"",""chr2"",""chr3"",""chr4"",""chr5"",";
      put "   ""chr6"",""chr7"",""chr8"",""chr9""});";
      put "ccol << label(1);";
      %if (%upcase(&Speak) = YES) %then %do;
         put 'speak(wait(1),"chromosome position plots");';
      %end;
      
      put "objRefNS:cp = Function({},{default local}," ;
      put "   If(IsEmpty(objRefNS:adata1),";
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put " objRefNS:adata1 = open(objRefNS:outds,invisible);";
      %end;
      %else %do;
         put "objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      %end;
       put ");";
       put 'current data table(objRefNS:adata1);';
      put '   VListBox(';
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put '  textboxChr= TextBox("Chromosome "|| objRefNS:ChrVarInd),';
         put 'textboxChr << set font size(11),';
      %end;
      put '   TextBox(" " || locale:fill(locale:get("HorizontalreferencelinedrawnatMinusLog10pX_Text"), {char(format(objRefNS:cutoff,"Fixed Dec",3))})),';
      
      %if ^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice and (&HasT1 or &HasT2 or &HasT3) %then %do;

         put "   HListBox(dcbiv&i = Overlay Plot(";
         put "      X( :Name(""&PosVar"")),";
         put "      Y(" @@;
         %do i = &nNLP %to 1 %by -1;
            put ":Name(""&&NLP&i"")" @@;
            %if (&i > 1) %then %do;
               put "," @@;
            %end;
            %else %do;
               put "),";
            %end;
         %end;
         %if %upcase(&SplitResults)=YES %then %do;
            put "      By(&jslDBV,:Name(""&ChrVar"")),";
         %end;
         %else %do;
            put "      By(:Name(""&ChrVar"")),";
         %end;
         %if &nNLP = 1 %then %let astring = Y Axis;
         %else %let astring = Left Axis;
         put "      &astring << {{Scale( Linear ), Format( ""Best"" ),";
         put "         Min(-0.5), Max(32), Inc(8), Minor Ticks(1),";
         put '         Add Ref Line( objRefNS:cutoff, Dashed, "Red" )}},';
         %do i = &nNLP %to 1 %by -1;         
            put "      :Name(""&&NLP&i"")(Connect Points( 1 )),";
         %end;
   /*    put "      SendToReport(";
         put "         Dispatch(";
         put "            {},";
         put "            ""Overlay Plot"",";
         put "            FrameBox,";
         put "            {Frame Size( 400, 200 )}";
         put "         )";
         put "      )";*/
         put "   ))";
      %end;
      %else %if (&HasDiff | &HasEst) %then %do;
         %do i = 1 %to &nDiff;

            put "   HListBox(dcbiv&i = Overlay Plot(X( :Name(""&PosVar"")),";
            put "      Y( :Name(""&&dlabel&i""),:Name(""&&nlpdlab&i"")),";
            %if %upcase(&SplitResults)=YES %then %do;
               put "      By(&jslDBV,:Name(""&ChrVar"")),";
            %end;
            %else %do;
               put "      By(:Name(""&ChrVar"")),";
            %end;
            put "      Y Scale( Left, Right ),";
            put "      Left Axis << {{Scale( Linear ), Format( ""Best"" ), Min( -4.1 ),";
            put "         Max( 4.1 ), Inc( 1 ), Add Ref Line( 0, Dashed, ""Black"" )}},";
            put "      Right Axis << {{Scale( Linear ), Format( ""Best"" ), Min( -0.5 ),";
            put "         Max( 96 ), Inc( 16 ), Minor Ticks( 1 ),";
            put "         Add Ref Line( objRefNS:cutoff, Dashed, ""Red"" )}},";
            put "      :Name(""&&dlabel&i"")(Connect Points( 1 )),";
            put "      :Name(""&&nlpdlab&i"")(Connect Points( 1 )),";
      /*    put "      SendToReport(";
            put "         Dispatch(";
            put "            {},";
            put "            ""Overlay Plot"",";
            put "            FrameBox,";
            put "            {Frame Size( 400, 200 )}";
            put "          )";
            put "       )";*/

            /* code for bivariate instead of overlay
            put "   HListBox(dcbiv&i = Bivariate(X( :Name("'"' "&PosVar" '")),';
            put '      Y( :Name("' "&&nlpdlab&i" '")),';
            %if %upcase(&SplitResults)=YES %then %do;
               put "      By(&jslDBV,:Name(""&ChrVar""))," ' Title(" " ' @@;
               %do i = 1 %to &ndbv;
                  %let tmp = %substr(%scan(%nrbquote(&jslDBV),&i,','));
                  put '||"' "&&dbv&i=" '"||' "char(&tmp)" @@; 
               %end;
               put '),';
            %end;
            %else %do;
               put "      By(:Name(""&ChrVar"")),";
            %end;
            put "      SendToReport(";
            put "         Dispatch(";
            put "            {},";
            put "            ""2"",";
            put "            ScaleBox, {";
            put "            Add Ref Line( objRefNS:cutoff, Dashed, Red )";
            %if %upcase(&VolcanoUnifiedY)=YES %then %do;
               put "            ,Min(&NLG_min),Max(&NLG_max),Inc(&NLG_inc),Minor Ticks(0)";
            %end;
            put "            }";
            put "         )";
            put "      )";
            */

            %if (&i < &nDiff) or (&nEst > 0) %then %do;
               put '   )),';
            %end;
            %else %do;
               put '   ))';
            %end;
         %end;
         %do i = 1 %to &nEst;

            put "   HListBox(dcbiv&i = Overlay Plot(X( :Name(""&PosVar"")),";
            put "      Y( :Name(""&&elabel&i""),:Name(""&&nlpelab&i"")),";
            %if %upcase(&SplitResults)=YES %then %do;
               put "      By(&jslDBV,:Name(""&ChrVar"")),";
            %end;
            %else %do;
               put "      By(:Name(""&ChrVar"")),";
            %end;
            put "      Y Scale( Left, Right ),";
            put "      Left Axis << {{Scale( Linear ), Format( ""Best"" ), Min( -4.1 ),";
            put '         Max( 4.1 ), Inc( 1 ), Add Ref Line( 0, Dashed, "Black" )}},';
            put "      Right Axis << {{Scale( Linear ), Format( ""Best"" ), Min( -0.5 ),";
            put "         Max( 96 ), Inc( 16 ), Minor Ticks( 1 ),";
            put '         Add Ref Line( objRefNS:cutoff, Dashed, "Red" )}},';
            put "      :Name(""&&elabel&i"")(Connect Points( 1 )),";
            put "      :Name(""&&nlpelab&i"")(Connect Points( 1 )),";
      /*    put "       SendToReport(";
            put "          Dispatch(";
            put "             {},";
            put "             ""Overlay Plot"",";
            put "             FrameBox,";
            put "             {Frame Size( 400, 200 )}";
            put "          )";
            put "       )";*/

            /* code for bivariate instead of overlay
            put "   HListBox(ecbiv&i = Bivariate(X( :Name(""&PosVar"")),";
            put "      Y( :Name(""&&nlpelab&i"")),";
            %if %upcase(&SplitResults)=YES %then %do;
               put "      By(&jslDBV, :Name(""&ChrVar""))," ' Title(" " ' @@;
               %do i = 1 %to &ndbv;
                  %let tmp = %substr(%scan(%nrbquote(&jslDBV),&i,','));
                  put '||"' "&&dbv&i=" '"||' "char(&tmp)" @@; 
               %end;
               put '),';
            %end;
            %else %do;
               put "      By(:Name(""&ChrVar"")),";
            %end;
            put "      SendToReport(";
            put "         Dispatch(";
            put "            {},";
            put "            ""2"",";
            put "            ScaleBox, {";
            put "            Add Ref Line( objRefNS:cutoff, Dashed, Red )";
            %if %upcase(&VolcanoUnifiedY)=YES %then %do;
               put "            ,Min(&NLG_min),Max(&NLG_max),Inc(&NLG_inc),Minor Ticks(0)";
            %end;
            put "            }";
            put "         )";
            put "      )";
            */

            %if &i < &nEst %then %do;
               put '   )),';
            %end;
            %else %do;
               put '   ))';
            %end;
         %end;
      %end;
      put '));';


      * plot average significance index;
      %if (&nDiff > 1) or (&nEst > 1) %then %do;
         put 'objRefNS:as=Function({}, {default local},';
         put ' VListBox(';
         %if %index(&ProcessName,OneWay) %then %do;
            %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
               put '  textboxChr= TextBox("Chromosome "|| objRefNS:ChrVarInd),';
               put 'textboxChr << set font size(11),';
            %end;
            put '   TextBox(" " || locale:fill(locale:get("HorizontalreferencelinedrawnatMinusLog10pX_Text"), {char(format(objRefNS:cutoff,"Fixed Dec",3))})),';
         %end;
         %if (&nDiff > 1) %then %do;
            put "   HListBox(dcbiv&i = Overlay Plot(X( :Name(""&PosVar"")),";
            %if %index(&ProcessName,OneWay) %then %do;
               put "      Y( Column(""Avg Sig Index for Diffs""||jg:ncall_string),";
               put "         :Name(""NLP_T3_&gName"")";
               put "      ),";
            %end;
            %else %do;
               put "      Y( Column(""Avg Sig Index for Diffs""||jg:ncall_string)),";
            %end;
            %if %upcase(&SplitResults)=YES %then %do;
               put "      By(&jslDBV,:Name(""&ChrVar"")),";
            %end;
            %else %do;
               put "      By(:Name(""&ChrVar"")),";
            %end; 
            %if %index(&ProcessName,OneWay) %then %do;
               put "      Y Scale( Left, Right ),";
               put "      Left Axis << {{Scale( Linear ), Format( ""Best"" ), Min( -0.6 ),";
               put "         Max( 1.1 ), Inc( 1 )}},";
               put "      Right Axis << {{Scale( Linear ), Format( ""Best"" ), Min( -0.5 ),";
               put "         Max( 96 ), Inc( 16 ), Minor Ticks( 1 ),";
               put '         Add Ref Line( objRefNS:cutoff, Dashed, "Red" )}},';
               put "      Column(""Avg Sig Index for Diffs""||jg:ncall_string)(Connect Points( 1 )),";
               %if %index(&ProcessName,OneWay) %then %do;
                  put "      :Name(""NLP_T3_&gName"")(Connect Points( 1 )),";
               %end;
         /*    put "       SendToReport(";
               put "          Dispatch(";
               put "             {},";
               put "             ""Overlay Plot"",";
               put "             FrameBox,";
               put "             {Frame Size( 400, 200 )}";
               put "          )";
               put "       )";*/
            %end;
            %else %do;
               put "      Y Axis[1] << {{Min(-0.1), Max(1.1), Inc(0.2), Minor Ticks(1)}}";
            %end;
         %end;
         %if (&nEst > 1) %then %do;
            %if (&nDiff > 1) %then %do;
               put '   )),';
            %end;
            put "   HListBox(dcbiv&i = Overlay Plot(X( :Name(""&PosVar"")),";
            put "      Y( Column(""Avg Sig Index for Estimates""||jg:ncall_string)),";
            %if %upcase(&SplitResults)=YES %then %do;
               put "      By(&jslDBV,:Name(""&ChrVar"")),";
            %end;
            %else %do;
               put "      By(:Name(""&ChrVar"")),";
            %end; 
            put "      Y Axis[1] << {{Min(-0.1), Max(1.1), Inc(0.2), Minor Ticks(1)}}";
         %end;
         put '   ))';
         put '));';
      %end;
      
      put " ";
   %end; 
   
   * volcano plots;
   %if (&HasDiff | &HasEst) %then %do;
      put 'current data table(objRefNS:adata1);';
      %if (%upcase(&Speak) = YES) %then %do;
         put 'speak(wait(1),"volcano plots");';
      %end;
      put 'objRefNS:vp=Function({}, {default local},';
      put "   If(IsEmpty(objRefNS:adata1),";
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put " objRefNS:adata1 = open(objRefNS:outds,invisible);";
      %end;
      %else %do;
         put "objRefNS:adata1 = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
      %end;
      put ");";
      put 'current data table(objRefNS:adata1);';
      
       put '   objRefNS:voldiff=VListBox(';
      %do i = 1 %to &nEst;
         %let j = %eval(&i+&nDiff);
         put '   HListBox(';
         put "      objRefNS:ebiv&i = Bivariate(X( :Name("'"' "&&elabel&i" '")),';
         put '         Y( :Name("' "&&nlpelab&i" '")),';
         put '         Title("' "&&elabel&i" '"),';
         put '         SendToReport(';
         %if &LS_Index=3 %then %do;
            put "          Dispatch(";
            put "             {}, ""Bivar Plot"", FrameBox,";
            put "             {Marker Size(3), Marker Drawing Mode(""Outlined"")}";
            put "          ),";
         %end; 
         %if %symexist(UseXaxisFilter) %then %do;
            %if %Upcase(&UseXaxisFilter)=YES %then %do;
               put '            Dispatch(';
               put '               {},"1", ScaleBox,{';
               %if %index(%Upcase(&XaxisDirection),NEGATIVE) = 0 %then %do;
                  put "               Add Ref Line( objRefNS:xaxispos, Dashed, ""Red"" )";
               %end;
               %if %index(%Upcase(&XaxisDirection), POSITIVE) = 0 %then %do;
                  put "                ,";
                  put "               Add Ref Line( objRefNS:xaxisneg, Dashed, ""Red"" )";
               %end;
               put '             }),';
            %end;
         %end;       
         put '            Dispatch(';
         put '               {},';
         put '               "2",';
         put '               ScaleBox,';
         put '               {Scale( Linear ), Format( "Best" ),';
         %if %upcase(&VolcanoUnifiedY)=YES %then %do;
            put "               Min(&NLG_min),Max(&NLG_max),Inc(&NLG_inc),Minor Ticks(0),";
         %end;
         %if &indTestAdj=yes %then %do;
            put "               Add Ref Line( &&NegLog10pCutoff&j, Dashed, ""Red"" )}";
         %end;
         %else %do;
            put '               Add Ref Line( objRefNS:cutoff, Dashed, "Red" )}';
         %end;
         put '            )';
         put '         )';
         put '      )';
         %if &i < &nEst %then %do;
            put '   ),';
         %end;
         %else %do;
            put '   ),';
         %end;
      %end; 
      %do i = 1 %to &nDiff;
         
         put '   HListBox(';
         put "      objRefNS:dbiv&i = Bivariate(X( :Name("'"' "&&dlabel&i" '")),';
         put '         Y( :Name("' "&&nlpdlab&i" '")),';
         put '         Title("' "&&dlabel&i" '"),';
         put '         SendToReport(';
         %if &LS_Index=3 %then %do;
            put "          Dispatch(";
            put "             {}, ""Bivar Plot"", FrameBox,";
            put "             {Marker Size(3), Marker Drawing Mode(""Outlined"")}";
            put "          ),";
         %end; 
         %if %symexist(UseXaxisFilter) %then %do;
            %if %Upcase(&UseXaxisFilter)=YES %then %do;
               put '            Dispatch(';
               put '               {},"1", ScaleBox,{';
               %if %index(%Upcase(&XaxisDirection),NEGATIVE) = 0 %then %do;
                  put "               Add Ref Line( objRefNS:xaxispos, Dashed, ""Red"" )";
               %end;
               %if %index(%Upcase(&XaxisDirection), POSITIVE) = 0 %then %do;
                  put "                ,";
                  put "               Add Ref Line( objRefNS:xaxisneg, Dashed, ""Red"" )";
               %end;
               put '             }),';
            %end;
         %end;
         put '            Dispatch(';
         put '               {},';
         put '               "2",';
         put '               ScaleBox,';
         put '               {Scale( Linear ), Format( "Best" ),';
         %if %upcase(&VolcanoUnifiedY)=YES %then %do;
            put "               Min(&NLG_min),Max(&NLG_max),Inc(&NLG_inc),Minor Ticks(0),";
         %end;
         %if &indTestAdj=yes %then %do;
            put "               Add Ref Line( &&NegLog10pCutoff&i, Dashed, ""Red"" )}";
         %end;
         %else %do;
            put '               Add Ref Line( objRefNS:cutoff, Dashed, "Red" )}';
         %end;
         put '            )';

         put '         )';
         put '      )';
         %if (&i < &nDiff) or (&nEst > 0) %then %do;
            put '   ),';
         %end;
         %else %do;
            put '   )';
         %end;
      %end;*end of graphic definition;

      put '   );';
   
      
      put '   objRefNS:tt=VListBox(';
      
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put '  textboxChr= TextBox("Chromosome "|| objRefNS:ChrVarInd),';
         put 'textboxChr << set font size(11),';
      %end;
         
      put '   HListBox(TextBox(" " || locale:get("FrameSizeforGraphsBelow_Text")),';
      put "  objRefNS:ppframeSizeSlider=EvalInsert(""\[";
      put '      objRefNS:vframesizeslider = SliderBox(num(100),num(700),objRefNS:vfsize,';
      
      %do i = 1 %to &nDiff;
         put "         objRefNS:rdbiv&i[framebox(1)]<<FrameSize(objRefNS:vfsize,0.75*objRefNS:vfsize);";
      %end;
      %do i = 1 %to &nEst;
         put "         objRefNS:rebiv&i[framebox(1)]<<FrameSize(objRefNS:vfsize,0.75*objRefNS:vfsize);";
      %end;
      put '      )';
      put '  ]\"); ';
      put "  objRefNS:volevi=EvalInsert(""\[";
      %do i = 1 %to &nDiff;
         put "      objRefNS:rdbiv&i=objRefNS:dbiv&i<<report;";
         put "      objRefNS:rdbiv&i[framebox(1)]<<FrameSize(objRefNS:vfsize,0.75*objRefNS:vfsize);";
      %end;
      %do i = 1 %to &nEst;
         put "      objRefNS:rebiv&i=objRefNS:ebiv&i<<report;";
         put "      objRefNS:rebiv&i[framebox(1)]<<FrameSize(objRefNS:vfsize,0.75*objRefNS:vfsize);";
      %end;
      put '  ]\"); ';
      put "  Eval(Parse(objRefNS:volevi));";
      put "  Eval(Parse(objRefNS:ppframeSizeSlider));";
      put '   ),';
      %if &indTestAdj=no %then %do;
         put '   TextBox(" " || locale:fill(locale:get("HorizontalreferencelinedrawnatMinusLog10pX_Text"), {char(format(objRefNS:cutoff,"Fixed Dec",3))})),';
      %end;
      put "      objRefNS:voldiff;";

      
   /* put '   msize = 0;';
      put '   HListBox(TextBox("Marker Size for Graphs Below"),';
      put '      objRefNS:framesizeslider = SliderBox(num(-1),num(6.5),msize,';
      %do i = 1 %to &nDiff;
         put "         objRefNS:rdbiv&i[framebox(1)]<<MarkerSize(msize);";
         put "         objRefNS:rdbiv&i[framebox(1)]<<Marker Drawing Mode(Outlined);";
      %end;
      %do i = 1 %to &nEst;
         put "         objRefNS:rebiv&i[framebox(1)]<<MarkerSize(msize);";
         put "         objRefNS:rebiv&i[framebox(1)]<<Marker Drawing Mode(Outlined);";
      %end;
      put '      )';
      put '   ),';
      */

      
      put '));';
      
      /*---need to determine number of design-level by groups for this displaybox, 
           constraining to 1 for now---*/
      /*
      %if %upcase(&SplitResults)=YES %then %do;
         put "ndbyg = 1;";
      %end;
      %else %do;
         put "ndbyg = 1;";
      %end;
      %do i = 1 %to &nDiff;
         put "objRefNS:rdbiv&i = objRefNS:dbiv&i << report;";
         put "for(i=1, i<=ndbyg, i++,";
         put "   objRefNS:rdbiv&i[axisbox(i)]<<AddRefLine(objRefNS:cutoff,2,3);";
         %if %upcase(&VolcanoUnifiedY)=YES %then %do;
            put "   objRefNS:rdbiv&i[axisbox(i)]<<Min(&NLG_min);";
            put "   objRefNS:rdbiv&i[axisbox(i)]<<Max(&NLG_max);";
            put "   objRefNS:rdbiv&i[axisbox(i)]<<Inc(&NLG_inc);";
            put "   objRefNS:rdbiv&i[axisbox(i)]<<Minor Ticks(0);";
         %end;
         put ");";
      %end;
      %do i = 1 %to &nEst;
         put "objRefNS:rebiv&i = objRefNS:ebiv&i << report;";
         put "for(i=1, i<=ndbyg, i++,";
         put "   objRefNS:rebiv&i[axisbox(i)]<<AddRefLine(objRefNS:cutoff,2,3);";
         %if %upcase(&VolcanoUnifiedY)=YES %then %do;
            put "   objRefNS:rebiv&i[axisbox(i)]<<Min(&NLG_min);";
            put "   objRefNS:rebiv&i[axisbox(i)]<<Max(&NLG_max);";
            put "   objRefNS:rebiv&i[axisbox(i)]<<Inc(&NLG_inc);";
            put "   objRefNS:rebiv&i[axisbox(i)]<<Minor Ticks(0);";
         %end;
         put ");";
      %end;
      */
   /* put " ";
      put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:TabButtonList, ""Volcano Plots"");";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:vp()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      %let opentabflag =1;*/
   %end; 
   %if %upcase(&SplitResults) ne YES %then %do;
      %if (&HasDiff | &HasEst | &HasSlice | &HasT3) %then %do;
         %if &HasLsm and %upcase(&ClusterLSM)=YES %then %do;   
            put 'objRefNS:combineTabs=Function({}, {default local},';
            %if (&HasDiff | &HasEst) %then %do;
               put ' objRefNS:vp();';
            %end;
         
            %if %upcase(&SplitResults)=YES %then put 'if (tSig > 0,';
            %else put 'if (objRefNS:Nsig > 0,';;         
            put "objRefNS:sigobs = N Row(objRefNS:sigdata);";
            put "if (objRefNS:sigobs>1,";
            put '    objRefNS:vb();';
            put ");";
            put ' objRefNS:ppca();';
            %if %length(&TEAEText) %then %do;
               put "VlistBox(";
               put "objRefNS:TEAEText = TextBox(""&TEAEText"", <<Set Wrap(1000)), Text Box(""""),";
            %end;                        
            put ' HListBox(';
            %if (&HasDiff | &HasEst) %then %do;
               put'  objRefNS:tt,';
            %end;
            put "if (objRefNS:sigobs>1,";
            put '    objRefNS:hctt,';
            put "),";
            put '    objRefNS:pptt,';
            put'     ),';
            %if %length(&TEAEText) %then %do;
               put "),";
            %end;                                    
            %if (&HasDiff | &HasEst) %then %do;
               put'  objRefNS:tt';
            %end;
            put " )";
            put ");";
            put 'if (objRefNS:Nsig > 0,';
            put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
            put ",";
            put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}}),";
            put ");";
            %if &BA_index %then %do;
               put "InsertInto(objRefNS:TabButtonList, locale:fill(locale:get(""XResults_Text""), {""&FDomain""}));";
            %end;
            %else %do;
               put "InsertInto(objRefNS:TabButtonList, locale:get(""Results_Text""));";
            %end;          
            put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:combineTabs()));";
            put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
            put "InsertInto(objRefNS:TabOpenList,1);";
            %let opentabflag =1;
         %end;
         %else %do;
            %if (&HasDiff | &HasEst) %then %do;
               put 'objRefNS:combineTabs=Function({}, {default local},';
               put ' objRefNS:vp();';
               put " );";
               put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
               %if &BA_index %then %do;
                  put "InsertInto(objRefNS:TabButtonList, locale:fill(locale:get(""XResults_Text""), {""&FDomain""}));";
               %end;
               %else %do;
                  put "InsertInto(objRefNS:TabButtonList, locale:get(""Results_Text""));";
               %end;
               put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:combineTabs()));";
               put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
               put "InsertInto(objRefNS:TabOpenList,1);";
               %let opentabflag =1;
            %end;
         %end;       
      %end;
   %end;
   %else %do;
      %if (&HasDiff | &HasEst | &HasSlice | &HasT3) %then %do;
         %if &HasLsm and %upcase(&ClusterLSM)=YES %then %do;
            put 'objRefNS:combineTabs=Function({}, {default local},';
            %if (&HasDiff | &HasEst) %then %do;
            put ' objRefNS:vp();';
            %end;
            %if %upcase(&SplitResults)=YES %then put 'if (tSig > 0,';
            %else put 'if (objRefNS:Nsig > 0,';;
            put 'HListBox(';
            %if (&HasDiff | &HasEst) %then %do;
               put'objRefNS:tt,';
            %end;
            put ' objRefNS:vb(),';
            put ' objRefNS:ppca()';
            put'),';
            %if (&HasDiff | &HasEst) %then %do;
            put'objRefNS:vp()';
            %end;
            put ");";
            put ");";
            
            put "InsertInto(objRefNS:DataList,{{";
            put'"objRefNS:adata1",'; 
            %do nCL = 1 %to &TotalCLvl_DBV;
               put'"objRefNS:sigdata'"[&nCL]"'"'; 
               %*if &nCL ne &TotalCLvl_DBV %then %do;
                  put ',';
               %*end;
            %end;
            %do nCL = 1 %to &TotalCLvl_DBV;
               put'"objRefNS:sigdata'"[&nCL]"'"'; 
               %if &nCL ne &TotalCLvl_DBV %then %do;
                  put ',';
               %end;
            %end;
            put '}});';

            %if &BA_index %then %do;
               put "InsertInto(objRefNS:TabButtonList, locale:fill(locale:get(""XResults_Text""), {""&FDomain""}));";
            %end;
            %else %do;
               put "InsertInto(objRefNS:TabButtonList, locale:get(""Results_Text""));";
            %end;
            put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:combineTabs()));";
            put "if(objRefNS:Nsig >0,";
            put "InsertInto(objRefNS:TabAfterCodeList,  "" "");";
            put ",";
            put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
            put ");";
            put "InsertInto(objRefNS:TabOpenList,1);";
            %let opentabflag =1;
         %end;
         %else %do;
            %if (&HasDiff | &HasEst) %then %do;
               put 'objRefNS:combineTabs=Function({}, {default local},';
               put ' objRefNS:vp();';
               put " );";
               put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
               %if &BA_index %then %do;
                  put "InsertInto(objRefNS:TabButtonList, locale:fill(locale:get(""XResults_Text""), {""&FDomain""}));";
               %end;
               %else %do;
                  put "InsertInto(objRefNS:TabButtonList, locale:get(""Results_Text""));";
               %end;
               put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:combineTabs()));";
               put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
               put "InsertInto(objRefNS:TabOpenList,1);";
            %end;
         %end;
      %end;
   %end;
   %if %length(&ChrVar) and %length(&PosVar) and (&HasDiff or &HasEst or 
     (^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice and (&HasT1 or &HasT2 or &HasT3))) %then %do;
     %if (&nDiff > 1) or (&nEst > 1) %then %do;
         put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
         put "InsertInto(objRefNS:TabButtonList, ""Avg Sig Index Chromosome Plots"");";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:as()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
     %end;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:TabButtonList, ""Chromosome Position Plots"");";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:cp()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      %let opentabflag =1;    
   %end;
   %if ^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice and (&HasT1 or &HasT2 or &HasT3) %then %do;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
      put "InsertInto(objRefNS:TabButtonList, ""Fixed Effect Tests"");";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fet()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      %if &opentabflag %then %do;
         put "InsertInto(objRefNS:TabOpenList,0);";
      %end;
      %else %do;
         put "InsertInto(objRefNS:TabOpenList,1);";
         %let opentabflag =1; 
      %end;
   %end;
%if ^%symexist(vcFlag) %then %let vcFlag = 0;
%if ^&vcFlag %then %do;
   put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""VariabilityEstimates_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ve()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   %if &opentabflag %then %do;
      put "InsertInto(objRefNS:TabOpenList,0);";
   %end;
   %else %do;
      put "InsertInto(objRefNS:TabOpenList,1);";
   %end;
%end;
%if %upcase(&Whichproc) = GLIMMIX  %then %do;
   put "InsertInto(objRefNS:DataList,{{""objRefNS:adata1""}});";
   put "InsertInto(objRefNS:TabButtonList, ""Goodness of Fit"");";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:gf()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   %if &opentabflag %then %do;
      put "InsertInto(objRefNS:TabOpenList,0);";
   %end;
   %else %do;
      put "InsertInto(objRefNS:TabOpenList,1);";
   %end;
%end;
   * end of objRefNS:sigrun expression definition;
   put ');'; 
   put 'objRefNS:sigrun;';
   
   %*if %length(&ChrVar) and %length(&PosVar) and (&HasDiff or &HasEst or 
     (^&HasCon and ^&HasEst and ^&HasDiff and ^&HasSlice and (&HasT1 or &HasT2 or &HasT3))) %then %do;

   %*end;
   * residual plotting;
   %if (%upcase(&PlotResiduals) = YES) %then %do; 
      %if (%upcase(&Speak) = YES) %then %do;
         put 'speak(wait(1),"standardized residuals table");';
      %end;
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put " objRefNS:resdata = open(objRefNS:outresds,invisible);";
      %end;
      %else %do;
         put "objRefNS:resdata=open(""&ClientOutPath.&OutDataResName..sas7bdat"",invisible);";
      %end;
      /*
      put "for each row(color of(rowstate())=6);";
      */
      %if %upcase(&AnnoLabResid) = YES %then %do;
         %AnnoLabelJSL(Ds=&OutDataRes);
      %end;
      %else %do;
         %if (%upcase(&ProcessName) ne ONEWAYANOVA) %then %do; 
            %let i = 1;
            %do %while(%length(%scan(&OrigByVars,&i)));
               %let ByLab = %scan(&OrigByVars,&i);
               %checklabel(&OutDataRes,ByLab);
               put "%str(column(%"&ByLab%") << label(1);)";
               %let i=%eval(&i+1);
            %end;
            %if &AlreadyStacked and %length(&BaseMergeVars) %then %do;
               %let i = 1;
               %do %while(%length(%scan(&BaseMergeVars,&i)));
                  %let BMLab = %scan(&BaseMergeVars,&i);
                  %checklabel(&OutDataRes,BMLab);
                  put "%str(column(%"&BMLab%") << label(1);)";
                  %let i=%eval(&i+1);
               %end;
            %end;
         %end;
      %end;
      
      %if (%upcase(&Speak) = YES) %then %do;
         %if &WhichProc ne GLIMMIX %then %do;
            put 'speak(wait(1),"standardized residuals plots");';
         %end;
         %else %do;
            put 'speak(wait(1),"studentized residuals plots");';
         %end;
      %end;
      put 'objRefNS:stdresidualplots=Function({}, {default local},';
      put "   If(IsEmpty(objRefNS:resdata),";
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put " objRefNS:resdata = open(objRefNS:outresds,invisible);";
      %end;
      %else %do;
         put "objRefNS:resdata=open(""&ClientOutPath.&OutDataResName..sas7bdat"",invisible);";
      %end;
        put ");";
        put 'current data table(objRefNS:resdata);';
      put 'VListBox(';
      %if &AlreadyStacked %then %do;
         %if &WhichProc ne GLIMMIX %then %do;
            put "      Bivariate( X( :Predicted), Y( :StdResidual),";
         %end;
         %else %do;
            put "      Bivariate( X( :Predicted), Y( :StuResidual),";
         %end;
         %if %upcase(&SplitResults) = YES %then %do;
            put "         By(&jslDBV),";
         %end;
         put "         Histogram Borders(1), Separate Axes(1), Fit Spline(1)";
         put "      )" @@;
      %end;
      %else %do;
         %do _c_=1 %to &_nTallCol_;
            put "   HListBox(";
            put "      Distribution(Continuous Distribution(";
            %if &WhichProc ne GLIMMIX %then %do;
               put "         Column(:StRes_&&_ColName_&_c_._), More Moments(1),";
            %end;
            %else %do;
               put "         Column(:StuRes_&&_ColName_&_c_._), More Moments(1),";
            %end;
            put "         Horizontal Layout(1), Count Axis(1),";  
            put "         Normal Quantile Plot(1))),";
            %if &WhichProc ne GLIMMIX %then %do;
               put "      Bivariate( X( :Pred_&&_ColName_&_c_._), Y( :StRes_&&_ColName_&_c_._),";
            %end;
            %else %do;
               put "      Bivariate( X( :Pred_&&_ColName_&_c_._), Y( :StuRes_&&_ColName_&_c_._),";
            %end;
            put "         Separate Axes(1), Fit Spline(1) )";
            put "   )" @@;
            %if (&_c_ < &_nTallCol_) %then %do;
               put ",";
            %end;
         %end;
      %end;
      put "));";  
      put " ";
   %end;
   %if (%upcase(&Speak) = YES) %then %do;
      put 'speak(wait(1),"you are now ready to go.  have fun in jump!");';
   %end;
   /*
   put 'Main Menu("View:Log");';
   */
   * residual plotting;
   %if (%upcase(&PlotResiduals) = YES) %then %do; 
      put "InsertInto(objRefNS:DataList,{{""objRefNS:resdata""}});";
      %if &WhichProc ne GLIMMIX %then %do;
         put "InsertInto(objRefNS:TabButtonList, locale:get(""StandardizedResidualsPlots_Text""));";
      %end;
      %else %do;
         put "InsertInto(objRefNS:TabButtonList, locale:get(""StudentizedResidualsPlots_Text""));";
      %end;
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:stdresidualplots()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
   %end;
      %if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
         put "include(""&ClientOutPath.&jslfile3name"");";
      %end; 
   * buttons to launch GSE aps for ANOVA;
   %if &ProcessName=ANOVA %then %do;
      put "gselaunchbuttons=Function({},{default local},";
      put "   VListBox(";
      put "       ButtonBox(""Gene Set Enrichment"",";
      put "          if(host is(Windows),";
      put "             Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/GeneSetEnrichment.jsl"", absolute, windows)),";
      put "             Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/GeneSetEnrichment.jsl"", absolute, POSIX) ));";
      put " ";
      put "          jg:setCurrentParmValue(2,""GeneSetEnrichment"", jg:getParmNum(""GeneSetEnrichment"",""InData""),""&ClientOutPath.&OutDataName..sas7bdat"");";
      %if %length(&AnnoInData) %then %do;
         put "          jg:setCurrentParmValue(2,""GeneSetEnrichment"", jg:getParmNum(""GeneSetEnrichment"",""AnnoInData""),""&AnnoInData"");";    
      %end;
      put "          jg:GeneSetEnrichment_updateDeps;";  
      put "          jg:setCurrentParmValue(2,""GeneSetEnrichment"", jg:getParmNum(""GeneSetEnrichment"",""OutPath""),""&ClientOutPath"");";

      put "       ) ,";    
      put " ";
      put "   )";
      put ");";   
      put "objRefNS:LaunchCode = gselaunchbuttons();";
   %end;
         

   
run;
/*
%if &indTestAdj=no %then %do;
   %global jsl_after_tabbedreport;
   %let jsl_after_tabbedreport = %quote($LS_HOME/JSLFiles/Mixed Model Drill Down Recompute.JSL);
%end;
*/

* packaging;
%if ^%symexist(PlotResiduals) %then %let PlotResiduals = No;
%if ^%symexist(LComponents) %then %let LComponents = No;
%let outds = &OutDataName..sas7bdat;
%if (%upcase(&PlotResiduals) = YES) %then %do; 
   %let outds = &outds,&OutDataResName..sas7bdat;
%end;
%if %upcase(&LComponents)=YES %then %do;
   %let outds = &outds,&OutDataCompName..sas7bdat;
%end;
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &JSLFileName, "&outds", text/plain, yes);
%if %length(&ChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
   %PackageInsertFile(Main Tabbed Script for Secondary from &ProcessName, &OutPath, &JSLFile3Name, , 
      text/plain, yes, no);
%end;

%mcjexit:;
%ClockStop(MixedCreateJSL);
%mend MixedCreateJSL;




* Calculate MMEI;
%macro MMEI;
%ClockStart(MMEI Chunk=&_chunk of &nchunk);

%CheckDataName(_name=MMEIData);
proc means data=&OutStdRes nway noprint mean;
   class &byvars ColumnName;
   var pred;
   output out=mmei(drop=_type_ _freq_) mean=mmei;   
run;

%*if %sysfunc(exist(outlib.&mmeidata)) %then %do;
%if &_chunk>1 %then %do;
   proc transpose data=mmei out=tmmei(drop=_name_ _label_);
      by &byvars;
      var mmei;
      id ColumnName;
   run;

   proc append base=outlib.&mmeidata data=tmmei force;
   run;

   proc datasets library=work nolist;
      delete tmmei;
   run;
%end;
%else %do;
   proc transpose data=mmei out=outlib.&mmeidata(drop=_name_ _label_);
      by &byvars;
      var mmei;
      id ColumnName;
   run;
%end;

%ClockStop(MMEI);
%mend MMEI;


%macro MixedPackage;
%ClockStart(MixedPackage);
* Packaging;
* package _sig data set;
%if %length(%trim(&OutDataName)) > 28 %then 
   %let _name = %substr(&OutDataName,1,28);
%else %let _name = &OutDataName; 

%if %sysfunc(exist(Outlib.&_name._sig)) %then %do;
	*check if there is any significant results;
	%NObsVars(Outlib.&_name._sig);
	%if &nobs>0 %then %do;
   	%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(SignificantDifferencesDataFromX_Text)),
   	   %nrbquote(%LocaleGet(&ProcessName._Label)))), outlib, &_name._sig,yes);
	%end;
%end;

%if %symexist(Miss_Flag) %then %do;
   %if %upcase(&Miss_Flag) = YES %then %do; 
      %PackageInsertDataset(Filtered data from &ProcessName, outlib, &OutDataMissName,yes);
   %end;
%end;

%if %symexist(ListAll) %then %do;
   %if (%upcase(&ListAll) = YES) %then %do;
      %PackageInsertHtml(html from &ProcessName, &OutPath, &bodyfile, &framefile, &contentsfile);
   %end;
%end;

%if %symexist(MMEIData) %then %do;
   %if %length(&MMEIData) %then %do;
      %PackageInsertDataset(Mixed Model Expression Index data from &ProcessName, outlib, &mmeidata,yes);
%end;
%end;

%ClockStop(MixedPackage);
%mend MixedPackage;




/*------------------------------------*/
/*---ONE WAY ANOVA IN THE DATA STEP---*/
/*------------------------------------*/
%macro OneWay;

%put Starting OneWay;

%if ^%sysfunc(fileexist(&InData)) %then %do;
   %put ERROR: Physical file does not exist, &InData.; 
   %goto exit;
%end;

%GetOrigFileName(InData,OrigInData);

%global WhichProc;
%let WhichProc = MIXED; 

* check name for OutData;
%if %symexist(OutDataPre) %then %do;
   %if %length(&OutDataPre) %then %do;
      %if %length(&OutDataPre) < 29 %then %let OID = &OutDataPre;
      %else %let OID = %substr(&OutDataPre,1,28);
   %end;
   %else %do;
      %if %length(&OrigInData) < 29 %then %let OID = &OrigInData;
      %else %let OID = %substr(&OrigInData,1,28);
   %end;
%end;
%else %do;
   %if %length(&OrigInData) < 29 %then %let OID = &OrigInData;
   %else %let OID = %substr(&OrigInData,1,28);
%end;
%PrepOutDS(OutData,OutDataName,&OID,&suffix);
%PrepOutDS(OutDataRes,OutDataResName,&OID,_ars);

* check experimental design file;
%if ^%symexist(CheckDesign) %then %let CheckDesign = 1;
%CheckInData(_ExpFlag=1,_CheckDesign=&CheckDesign);
%if (&syserr | &exiterror) %then %goto exit;

* check annotation file;
%CheckAnno;
%if &exiterror %then %goto exit;
%if %length(&AnnoInData) and %length(&AnnoWhereStmt) %then %do;
   data OutLib._AnnoData1_;
      set AnnoLib.&AnnoData;
      where &AnnoWhereStmt;
   run;
   %let AnnoData = _AnnoData1_;
   %let AnnoPath = &OutPath;
   libname AnnoLib "&OutPath";
%end;
%if ^%symexist(LabelVar) %then %let LabelVar = ;

* drop unannotated data if necessary;
%let Mixed_Indata = InLib.&InData;
%if %symexist(Miss_Flag) %then %do; 
   * using the data after filtering from ANOVA AP; 
   %if %upcase(&Miss_Flag) = YES %then %let Mixed_Indata = NonMissInData;
%end;

%if %length(&AnnoInData) and (&AnnoOnly=yes) %then %do;
   data InDataAll;
      set &Mixed_Indata;
   run;
   %OnlyAnno;
   %if &exiterror %then %goto exit;
   %NObsVars(InDataAll);
   %if &nobs=0 %then %do;
      %put ERROR: Merge with annotation data yielded no observations.;
      %let exiterror = 1;
      %goto exit;
   %end;
%end;
%else %do;
   %let indat = &Mixed_Indata;
%end;

/* comment out this part since there is no second anno table for OneWay currently;
* handeling for second annotation; 
%global Anno2Data Anno2Path;
%if %length(&Anno2InData) %then %do;
   * save and reassign certain macro variables so OnlyAnno works;
   %let AnnoData0 = &AnnoData;
   %let AnnoMergeVars0 = &AnnoMergeVars;
   %let BaseMergeVars0 = &BaseMergeVars;
   %let AnnoPath = %sysfunc(pathname(AnnoLib));

   %let AnnoInData = &Anno2InData;
   %let AnnoMergeVars = &Anno2MergeVars;
   %let BaseMergeVars = &BaseMergeVars2;
   %CheckAnno;
   %let Anno2MergeVars = &AnnoMergeVars;
   %let BaseMergeVars2 = &BaseMergeVars;
   %let Anno2Data = &AnnoData;
   %let Anno2Path = %sysfunc(pathname(AnnoLib));
   libname Anno2Lib "&Anno2Path";
 
   %if %length(&Anno2WhereStmt) %then %do;
      data OutLib._AnnoData2_;
         set AnnoLib.&AnnoData;
         where %unquote(&Anno2WhereStmt);
      run;
      %let Anno2Data = _annoData2_;
      %let Anno2Path = &OutPath;
      libname Anno2Lib "&OutPath";
   %end;

   %if (&Anno2Only=yes) %then %do;
      %if %length(&Anno2WhereStmt) %then %do;
         %let AnnoData = &Anno2Data;
         libname AnnoLib "&Anno2Path";
      %end;
      %OnlyAnno;
      %if &exiterror %then %goto exit;
      %NObsVars(InDataAll);
      %if &nobs=0 %then %do;
         %put ERROR: Merge with secondary annotation data yielded no observations.;
         %let exiterror = 1;
         %goto exit;
      %end;
   %end;

   * revert anno macro assignments;
   %let AnnoData = &AnnoData0;
   %let AnnoMergeVars = &AnnoMergeVars0;
   %let BaseMergeVars = &BaseMergeVars0;
   libname AnnoLib "&AnnoPath";
%end;
%else %do;
   %let Anno2Data = ;
   %let Anno2Path = ;
%end;
*/

%if ^%symexist(AlreadyStacked) %then %do;
   %global AlreadyStacked;
   %let AlreadyStacked = 0;
%end;

%put AlreadyStacked = &AlreadyStacked;
%put;

* create macro variables corresponding to groups;
data ed;
   set OutLib.&ExpInData;
run;

%global OrigByVars;
%let OrigByVars =_Row;

%GroupMacroVars(data=ed,vars=&GroupVars,prefix=g,maxnamelen=24);

* parse ESTIMATE statements;
%if ^%symexist(EstInFile) %then %let EstInFile = ;
%if %length(&EstInFile) %then %do;
   data _tmp_;
      length instring $ 32767;
      infile "&EstInFile" truncover lrecl=32767;
      input instring $1-32767;
      nidx = trim(left(_n_));
      estv = "est"|| nidx;
      call symput(estv,trim(left(instring)));
      call symput("nelines",nidx);
   run;

   %let allest = ;
   %do e=1 %to &nelines;
      %let allest = &allest &&est&e;
   %end;
   
   %let FullEffect = ;
   %let i = 1;
   %do %while(%length(%scan(&GroupVars,&i)));
      %if (&i > 1) %then %do;
         %let FullEffect = &FullEffect.*;
      %end;
      %let FullEffect = &FullEffect.%scan(&GroupVars,&i);
      %let i = %eval(&i+1);
   %end;
   %let FullEffect = %upcase(&FullEffect);
   %put FullEffect = &FullEffect;
   
   %let nEst = 0;
   %let another = 1;
   %do %while(&another);
      %let semidx = %index(&allest,%str(;));
      %if &semidx = 0 %then %do;
         %let another = 0;
      %end;
      %else %do;
         %let nEst = %eval(&nEst+1);
         %let currest = %substr(&allest,1,%eval(&semidx-1));
         %do %while(%index(%upcase(&currest),ESTIMATE));
            %let eidx = %index(%upcase(&currest),ESTIMATE);
            %let currest = %substr(&currest,%eval(&eidx+9));
         %end;
         %put est&nEst = &currest;
         %let estlab&nEst = %scan(&currest,1,%str(%'));
         %let lenlab = %length(&&estlab&nEst);
         %let currest = %substr(&currest,%eval(&lenlab+3));
         %let effect = %scan(&currest,1,' ');
         %if (%quote(%upcase(&effect)) ^= %quote(&FullEffect)) %then %do;
            %put ERROR: The effect &effect in ESTIMATE statement &&estlab&nEst does not match the One-Way effect &FullEffect..;
            %put The two effects must agree exactly in order.;
            %let exiterror = 1;
         %end;
         
         %let divisor = 1;
         %let didx = %index(%upcase(&currest),DIVISOR);
         %if &didx %then %do;
            %let dest = %substr(&currest,%eval(&didx+7));
            %let deidx = %index(&dest,=);
            %if &deidx %then %do;
               %let dest = %substr(&dest,%eval(&deidx+1));
               %let divisor = %scan(&dest,1,' ');
               %put divisor = &divisor;
            %end;
         %end;
         
         %let ns = 2;
         %let nc = 1;
         %do %while(%length(%scan(&currest,&ns,' ')));
            %let coef = %scan(&currest,&ns,' ');
            %if %quote(&coef) = %quote(%str(/)) %then %do;
               %let ns = 99999;
            %end;
            %else %do;
               %if (&divisor ^= 1) %then %do;
                  %let coef = %sysevalf(&coef/&divisor);
               %end;
               %let estcoef&nEst._&nc = &coef;
               %let ns = %eval(&ns+1);
               %let nc = %eval(&nc+1);
            %end;
         %end;
         %let nc = %eval(&nc-1);
         %if &nc ^= &gNum %then %do;
            %put ERROR: The number of coefficients in ESTIMATE statement &&estlab&nEst, &nc, does not equal the number of groups, &gNum..; 
            %put The two numbers must be equal.;
            %let exiterror = 1;
         %end;
         
         %if (&semidx >= %length(&allest)) %then %do;
            %let another = 0;
         %end;
         %else %do;
            %let allest = %substr(&allest,%eval(&semidx+1));
         %end;
      %end;
   %end;
   
   %do e=1 %to &nEst;
      %put estimate &e of &nEst label = &&estlab&e;
      %put estimate &e of &nEst coefficients:;
      %do c = 1 %to &gNum;
         %put &&estcoef&e._&c;
      %end;
   %end;
   
   %if &exiterror %then %goto exit;
   
%end;
%else %do;
   %let nEst = 0;
%end;

* set up variables for drill down, must build full factorial for JMP Fit Model;
%global ClassVars FixedEffects RandomEffects;
%let ClassVars = &GroupVars &BlockVars;
%let i = 1;
%do %while(%length(%scan(&GroupVars,&i)));
   %let i = %eval(&i+1);
%end;
%let ngVars = %eval(&i-1);

%if &ngVars = 1 %then %do;
   %let FixedEffects = &GroupVars;
%end;
%else %do;
   data _null_;
      call symputx('two_to_ng_m1',2**&ngVars - 1);
   run;
   data b;
      do i=1 to &two_to_ng_m1;
         call symputx('binary'||trim(left(i)),put(i,binary&ngVars..));
      end;
   run;
   %let FixedEffects = ;
   %do i=1 %to &two_to_ng_m1;
      %let bstr = &&binary&i;
      %let ThisEffect = ;
      %do j=1 %to &ngVars;
         %if %substr(&bstr,%eval(&nGVars - &j + 1),1) = 1 %then %do;
            %if %length(&ThisEffect) %then %do;
               %let ThisEffect = &ThisEffect.*;
            %end;
            %let ThisEffect = &ThisEffect.%scan(&GroupVars,&j,' ');
         %end;
      %end;
      %let FixedEffects = &FixedEffects &ThisEffect;
   %end;
%end;
%put FixedEffects = &FixedEffects;

%let RandomEffects = ;
%let i = 1;
%do %while(%length(%scan(&BlockVars,&i)));
   %if (&i > 1) %then %do;
      %let RandomEffects = &RandomEffects.*;
   %end;
   %let RandomEffects = &RandomEffects.%scan(&BlockVars,&i);
   %let i = %eval(&i+1);
%end;
%let nbVars = %eval(&i-1);
%put RandomEffects = &RandomEffects;
%let BaseSubjectVar = ;
%let BaselineVar = ;
%let BaselineValue = ;

* determine reference mean value;
%if %index(&LSMeansDiffSet,Control) %then %do;
   %if %length(&LSMeansControlValues) %then %do;
      * strip off quotes;
      %let refval = ;
      %do i=1 %to %eval(&ngVars*2-1); 
         %let refval = &refval %unquote(%scan(&LSMeansControlValues,&i,'"'));
      %end;
      %put;
      %put LSMeans Control Value = &refval;
      %let refset = 0;

      %do i=1 %to &gNum; 
         %if (%nrbquote(&refval) = %nrbquote(&&gID&i)) %then %do;
            %let refset = &i;
         %end;
      %end;
      %if ^&refset %then %do;
         %put ERROR: Means Control Value &LSMeansControlValues does not match any of those found in the data.;
         %put Valid Means Control Values are listed as gID values in the SAS log.;
         %let exiterror = 1;
         %goto exit;
      %end;
   %end;
   %else %do;
      %let refset = 1;
   %end;
%end;

* create macro variables corresponding to blocks; 
%if %length(&BlockVars) %then %do;

   %GroupMacroVars(data=ed,vars=&BlockVars,prefix=b,maxnamelen=23);

   * precompute quantities that depend only on X and Z, assuming no missing data;
   
   * XtZ matrix;
   %do i=1 %to &gNum;
      %do j=1 %to &bNum;
         %let gbNum_&i._&j = 0;
      %end;
   %end;
   
   proc sort data=ed;
      by gNum bNum;
   run;
   
   data ed;
      set ed;
      _one = 1;
      _y = rannor(12308478);
   run;
   
   proc means noprint;
      by gNum bNum;
      var _one;
      output out=edm n=n;
   run;
   
   data edm;
      set edm;
      mname = "gbNum_" || trim(left(gNum)) || "_" || trim(left(bNum));
      call symputx(mname,trim(left(n)));
   run;
   
   /*
   %do i=1 %to &gNum;
      %do j=1 %to &bNum;
         %put gbNum_&i._&j = &&gbNum_&i._&j;
      %end;
   %end;
   */
   
   * rank(X || Z);
   data t3;
      if (0);
   run;
   ods exclude all;
   ods noresults;
   proc mixed data=ed;
      class gNum bNum;
      model _y = gNum;
      random bNum;
      ods output tests3=t3;
   run;
   ods exclude none;
   ods results;
   
   %NObsVars(t3);
   %if (&nobs=0) %then %do;
      %put WARNING: The oneway model appears to be saturated.;
      %put Residual degrees of freedom is set to 1.;
      %let resdf = 1;
   %end;
   %else %do;
      data t3;
         set t3;
         if _n_ = 1 then call symputx('resdf',dendf);
      run;
   %end;

   %let rankxz = %eval(&_nTallCol_ - &resdf);
   %put resdf = &resdf;
   %put rankxz = &rankxz;
   %put;
   
   
   * MIVQUE(0) elements via formulas from Wolfinger, Tobias, and Sall (1994);
   /*
   proc mixed data=ed method=mivque0 noprofile pder plik;
      class gNum bNum;
      model _y = gNum;
      random bNum;
   run;
   
   %put ERROR: Not!;
   %let exiterror = 1;
   %goto exit;
   */

   %let g1 = 0;
   %let H1 = 0;
   %do i=1 %to &bNum;
      %let g1 = %eval(&g1 + &&bNum&i);
      %let H1 = %eval(&H1 - &&bNum&i * &&bNum&i);
   %end;
   %let H3_1 = 0;
   %do i=1 %to &gNum;
      %let ht = 0;
      %do j=1 %to &bNum;
         %let ht = %sysevalf(&ht + &&gbNum_&i._&j * &&gbNum_&i._&j * &&bNum&j);
         /* %put i=&i j=&j ht=&ht H3_1=&H3_1; */
      %end;
      %let H3_1 = %sysevalf(&H3_1 + &ht / &&gNum&i);
   %end;
   %let H3_1 = %sysevalf(&H3_1 * 2);
   %let g3 = 0;
   %let H3_2 = 0;
   %do i1=1 %to &gNum;
      %do i2=1 %to &gNum;
         %let ht = 0;
         %do j=1 %to &bNum;
            %let ht = %sysevalf(&ht + &&gbNum_&i1._&j * &&gbNum_&i2._&j);
            /* %put i1=&i1 i2=&i2 j=&j ht=&ht H3_2=&H3_2; */
         %end;
         %if (&i1=&i2) %then %do;
            %let g3 = %sysevalf(&g3 - &ht / &&gNum&i1);
         %end;
         %let H3_2 = %sysevalf(&H3_2 - &ht * &ht / &&gNum&i1 / &&gNum&i2);
      %end;
   %end;
   %let H3 = %sysevalf(&H3_1 + &H3_2);
   %let MVQ1 = %sysevalf(-&H1-&H3);
   %let MVQ2 = %sysevalf(&g1+&g3);
   %let MVQ3 = %eval(&_nTallCol_ - &gNum);
   %let MVQDet = %sysevalf(&MVQ1*&MVQ3-&MVQ2*&MVQ2);
   
   %put;
   %put H1=&H1 H3_1=&H3_1 H3_2=&H3_2 H3=&H3 g1=&g1 g3=&g3;
   %put MVQ1=&MVQ1 MVQ2=&MVQ2 MVQ3=&MVQ3 MVQDet=&MVQDet;
   
   %if (&MVQ2 = 0) %then %do;
      %put ERROR: The oneway residual error estimate is zero.  The model is likely misspecified.;
      %put Make sure that the Group Variables allow for some replication and that the Block;
      %put Variables are not contained in the Group Variables.;
      %let exiterror = 1;
      %goto exit;
   %end;
   
   %if (&MVQDet = 0) %then %do;
      %put ERROR: The oneway estimating equations are singular.  The model is likely misspecified.;
      %put Make sure that the Group Variables allow for some replication and that the Block;
      %put Variables are not contained in the Group Variables.;
      %let exiterror = 1;
      %goto exit;
   %end;

   %let np = &gNum;
   %let npSym = %eval(&np*(&np+1)/2);
   %let np1 = %eval(&np+1);
   %let np1Sym = %eval(&npSym+&np1);
   %let npm1 = %eval(&np-1);
   %let npm1Sym = %eval(&npSym-&np);
   %put np = &np;
   %put npSym = &npSym;
   %put np1 = &np1;
   %put np1Sym = &np1Sym;

%end;
%if ^%symexist(PlotResiduals) %then %let PlotResiduals = No;
%if (%upcase(&PlotResiduals) = YES) %then %do;
   data stdres;
      _l2 = 0;
      set &indat end=_last;
      %do i=1 %to &gNum;
         _Mean&i = mean(of %do k=1 %to &&gNum&i; &&G&i.R&k %end;);
         * replace data with residuals;
         %do k=1 %to &&gNum&i;
            &&G&i.R&k = &&G&i.R&k - _Mean&i;
            _l2 = _l2 + (&&G&i.R&k)**2; 
         %end;
      %end;
   run;
   options nolabel;
   data OutLib.&OutDataResName;  
      set stdres;
      %do i=1 %to &gNum;
         %do k=1 %to &&gNum&i;
            &&G&i.R&k = &&G&i.R&k/SQRT(_l2);
            Pred_&&G&i.R&k = _Mean&i;
            rename &&G&i.R&k=StRes_&&G&i.R&k;
         %end;
      %end;
      drop _Mean1-_Mean&gNum _l2;
   run;
   options label;
   %if %length(&AnnoChrVar) and (%upcase(&JournalChrom) = YES) %then %do;
      %MergeAnno(&OutDataRes);
      %if &exiterror %then %goto mfexit;
   %end;
%end;
* for Empirical Bayes shrinkage, make first pass and compute estimated inverted gamma prior;
%if (&EBShrink = yes) %then %do;
   data ebs;
      set &indat end=_last;
      retain nzvr 0;

      %if %length(&BlockVars) %then %do;
         * ANOVA with one random blocking factor;

         * 1. compute MIVQUE(0) estimates; 
         _l2 = 0;
         %do i=1 %to &gNum;
            _Mean&i = mean(of %do k=1 %to &&gNum&i; &&G&i.R&k %end;);
            * replace data with residuals;
            %do k=1 %to &&gNum&i;
               &&G&i.R&k = &&G&i.R&k - _Mean&i;
               _l2 = _l2 + (&&G&i.R&k)**2; 
            %end;
         %end;
         _g2 = 0;
         %do j=1 %to &bNum;
            * note &&B&j.R&k references the same residuals but with different indices;
            _wzrj = sum(of %do k=1 %to &&bNum&j; &&B&j.R&k %end;);
            _g2 = _g2 + _wzrj*_wzrj;
         %end;

         Variance_&bName = (&MVQ3 * _g2 - &MVQ2 * _l2)/&MVQDet;
         Variance_Residual = (-&MVQ2 * _g2 + &MVQ1 * _l2)/&MVQDet;
         
         * use MIVQUE0 transform as in PROC MIXED PRIOR statement;
         AB = Variance_&bName * &MVQ1 / &MVQ2 + Variance_Residual;
         if (AB > 0) then OneOverAB = 1/AB;
         else OneOverAB = .;
      %end;
      %else %do;
         * simple one-way ANOVA with no blocking;
         GrandMean = mean(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
         ErrorSS = 0;
         %do i=1 %to &gNum;
            Mean&i = mean(of %do k=1 %to &&gNum&i; &&G&i.R&k %end;);
            %do k=1 %to &&gNum&i;
               ErrorSS = ErrorSS + (&&G&i.R&k-Mean&i)**2; 
            %end;
         %end;
         TotalSS = css(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
         ModelSS = TotalSS - ErrorSS;
         NDF_&gName = &gNum - 1;
         DDF_&gName = &_nTallCol_ - &gNum;
         if (DDF_&gName < 1) then DDF_&gName = 1;
         Variance_Residual = ErrorSS/DDF_&gName;
      %end;
      if (Variance_Residual > 0) then do;
         OneOverVR = 1/Variance_Residual;
         nzvr = nzvr+1;
      end;
      else OneOverVR = .;
      drop %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
      if (_last) then call symput("nzvr",trim(left(nzvr)));
   run;
   
   %if (&nzvr = 0) %then %do;
      %put ERROR: Every row has zero or missing residual error variance.  The model may be completely saturated.;
      %let exiterror = 1;
      %goto exit;
   %end;
   
   %if (&nzvr < 3) %then %do;
      %put ERROR: There are less than 3 observations with valid data.  Empirical Bayes shrinkage cannot be performed.;
      %let exiterror = 1;
      %goto exit;
   %end;
         
   proc univariate data=ebs noprint;
      var OneOverVR %if %length(&BlockVars) %then %do; OneOverAB %end; ;
      where OneOverVR ne .;
      histogram / gamma noplot;
      ods output parameterestimates=pe;
   run;
      
   %if &syserr %then %goto exit;
   
   data _null_;
      set pe;
      if VarName="OneOverVR" and Parameter="Scale" then call symput("VR_Scale",trim(left(put(1/Estimate,12.6))));
      if VarName="OneOverVR" and Parameter="Shape" then call symput("VR_Shape",trim(left(put(Estimate,12.6))));
      if VarName="OneOverAB" and Parameter="Scale" then call symput("AB_Scale",trim(left(put(1/Estimate,12.6))));
      if VarName="OneOverAB" and Parameter="Shape" then call symput("AB_Shape",trim(left(put(Estimate,12.6))));
   run;
   
   %put VR_Scale=&VR_Scale   VR_Shape=&VR_Shape;
   %if %length(&BlockVars) %then %do;
      %put AB_Scale=&AB_Scale   AB_Shape=&AB_Shape;
   %end;

%end;


/*---Compute One-Way ANOVA statistics directly in a data step, creating a results data set similar
     to that produced by ANOVA and Mixed Model Analysis.  This method can run quite a bit faster
     than the same models in ANOVA and Mixed Model Analysis because it avoids the I/O overhead of 
     creating stacked data sets, calling Proc Mixed, and merging output tables---*/
data &OutData;
   set &indat;
   _Row = _n_;
   %if %length(&BlockVars) %then %do;
      * ANOVA with one random blocking factor;
      
      * 1. compute MIVQUE(0) estimates; 
      _l2 = 0;
      
      * make copy of data;
      %do i=1 %to &gNum;
         %do k=1 %to &&gNum&i;
            _y_&i._&k = &&G&i.R&k;
         %end;
      %end;
      
      %do i=1 %to &gNum;
         Mean&i = mean(of %do k=1 %to &&gNum&i; &&G&i.R&k %end;);
         * replace data with residuals;
         %do k=1 %to &&gNum&i;
            &&G&i.R&k = &&G&i.R&k - Mean&i;
            _l2 = _l2 + (&&G&i.R&k)**2; 
         %end;
      %end;
      _g2 = 0;
      %do j=1 %to &bNum;
         * note &&B&j.R&k references the same residuals but with different indices;
         _wzrj = sum(of %do k=1 %to &&bNum&j; &&B&j.R&k %end;);
         _g2 = _g2 + _wzrj*_wzrj;
      %end;
      
      * restore data;
      %do i=1 %to &gNum;
         %do k=1 %to &&gNum&i;
            &&G&i.R&k = _y_&i._&k;
         %end;
      %end;

      NDF_&gName = &npm1;
      DDF_&gName = &_nTallCol_ - &rankxz;
      DF_&bName = &bNum - 1;
      Variance_&bName = (&MVQ3 * _g2 - &MVQ2 * _l2)/&MVQDet;
      Variance_Residual = (-&MVQ2 * _g2 + &MVQ1 * _l2)/&MVQDet;   
      %if (&EBShrink=yes) %then %do;
         Variance_&bName.0 = Variance_&bName;
         if (Variance_&bName.0 > 0) then do;
            OneOverVB0 = 1/Variance_&bName.0;
         end;
         else do;
            OneOverVB0 = .;
         end;
         Variance_Residual0 = Variance_Residual;
         if (Variance_Residual0 > 0) then do;
            OneOverVR0 = 1/Variance_Residual0;
         end;
         else do;
            OneOverVR0 = .;
         end;
         DDF_&gName.0 = DDF_&gname;
         DF_&bName.0 = DF_&bname;
         * might should add 1 for Jeffreys, and also need to decide between mean and mode;
         AB_Shape = DF_&bName/2 + &AB_Shape; 
         AB_Scale = (Variance_&bName * &MVQ1 / &MVQ2 + Variance_Residual) * DF_&bName / 2 + &AB_Scale;
         AB_Estimate = AB_Scale / AB_Shape;
         Variance_&bName = (AB_Estimate - Variance_Residual) * &MVQ2 / &MVQ1;
         DF_&bName = 2*AB_Shape;
         Variance_Residual = (DDF_&gName*Variance_Residual + 2*&VR_Scale)/(DDF_&gName + 2*&VR_Shape);    
         DDF_&gName = DDF_&gName + 2*&VR_Shape;
      %end;
      if (Variance_Residual > 0) then do;
         VarRatio = Variance_&bName / Variance_Residual;
         SumVariance = Variance_&bName + Variance_Residual;
      end;
      else do;
         VarRatio = .;
         SumVariance = .;
      end;
      SampleVariance = var(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
      if (SumVariance > 0) then do;
         PropVar_&bName = Variance_&bName / SumVariance;
         PropVar_Residual = Variance_Residual / SumVariance;
      end;
      else do;
         PropVar_&bName = .;
         PropVar_Residual = .;
      end;        
      
      
      * 2. compute (Xt Vinv X) || (Xt Vinv y);
      array xpx[&np1Sym] _temporary_;
      
      %let idx = 0;
      %do i1=1 %to &np;
         %do i2=1 %to &i1;
            _ht = 0;
            %do j=1 %to &bNum;
               _ht = _ht - &&gbNum_&i1._&j * &&gbNum_&i2._&j / (&&bNum&j + 1/VarRatio);
            %end;
            %if (&i1=&i2) %then %do;
               _ht = _ht + &&gNum&i1;
            %end;
            xpx[&idx+1] = _ht/Variance_Residual;
            %let idx = %eval(&idx+1);
         %end;
      %end;
      
      array zsum[&bNum] _temporary_;
      %do j=1 %to &bNum;
         zsum[&j] = sum(of %do k=1 %to &&bNum&j; &&B&j.R&k %end;);
      %end;
      
      %do i=1 %to &np;
         _ht = Mean&i * &&gNum&i;
         %do j=1 %to &bNum;
            _ht = _ht - &&gbNum_&i._&j * zsum[&j] / (&&bNum&j + 1/VarRatio);
         %end;
         if (Variance_Residual > 0) then xpx[&idx+1] = _ht/Variance_Residual;
         else xpx[&idx+1] = .;
         %let idx = %eval(&idx+1);
      %end;
      
      xpx[&np1Sym] = 0;
      
      * 3. sweep (X'invV X) || (X' invV y) to obtain fixed effects solution and cov matrix;
      array work[&np1] _temporary_;
      %sweep(xpx,1,&np,&np1,work);
      
      * 4. compute means, residuals, blups, and RSquare;
      GrandMean = mean(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
      %do i=1 %to &gNum;
         Mean&i = xpx[&npSym+&i];
      %end;
      
      * new residuals;
      %do i=1 %to &gNum;
         %do k=1 %to &&gNum&i;
            &&G&i.R&k = _y_&i._&k - Mean&i;
         %end;
      %end;
      
      * blup, note &&B&j.R&k references the same residuals but with different indices;
      * error sum of squares;
      ErrorSS = 0;
      %do j=1 %to &bNum;
         blupj = VarRatio * (1 - &&bNum&j / (&&bNum&j + 1/VarRatio) ) * 
                  sum(of %do k=1 %to &&bNum&j; &&B&j.R&k %end;);
         %do k=1 %to &&bNum&j;
            * note &&B&j.R&k references the same residuals but with different indices;
            &&B&j.R&k = &&B&j.R&k - blupj;
            ErrorSS = ErrorSS + (&&B&j.R&k)**2; 
         %end;
      %end;
      
      * RSquare;
      if (SampleVariance > 0) then do;
         RSquare = 1 - ErrorSS/(SampleVariance*(&_nTallCol_-1));
         if RSquare < 0 then RSquare = 0;
      end;
      else RSquare = .;
      
      * 5. compute F-test;
      array fcon[&npSym] _temporary_;
      
      * use last level as reference level for F-test contrast matrix;
      %let ij = 1;
      %do i=1 %to &npm1;
         %do j=1 %to &i;
            fcon[&ij] = xpx[&ij] + xpx[&npSym] - xpx[&npm1Sym + &i] - xpx[&npm1Sym + &j];
            %let ij = %eval(&ij+1);
         %end;
         fcon[&npm1Sym+&i] = Mean&i - Mean&gNum;
      %end;
      
      %sweep(fcon,1,&npm1,&np,work);
      
      _ht = 0;
      %do i=1 %to &npm1;
         _ht = _ht + (Mean&i-Mean&gNum)*fcon[&npm1Sym+&i];
      %end;
      
      if (NDF_&gName > 0) then do;
         F_T3_&gName = _ht/NDF_&gName;
         if (F_T3_&gName > 0) then do;
            PrF_T3_&gName = 1 - probf(F_T3_&gName,NDF_&gName,DDF_&gName);
            if (PrF_T3_&gName < 1e-32) then NLP_T3_&gName = 32;
            else NLP_T3_&gName = -log10(PrF_T3_&gName);
         end;
         else do;
            PrF_T3_&gName = 1;
            NLP_T3_&gname = 0;
         end;
      end;
      else do;
         F_T3_&gName = .;
         PrF_T3_&gName = .;
         NLP_T3_&gName = .;
      end;
      
      * 6. clean up;
      drop %do i=1 %to &gNum; %do k=1 %to &&gNum&i; _y_&i._&k %end; %end;
         _l2 _g2 _wzrj _ht i j;
   
   %end;
   %else %do;
      * simple one-way ANOVA with no blocking;
      GrandMean = mean(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
      ErrorSS = 0;
      %do i=1 %to &gNum;
         Mean&i = mean(of %do k=1 %to &&gNum&i; &&G&i.R&k %end;);
         %do k=1 %to &&gNum&i;
               ErrorSS = ErrorSS + (&&G&i.R&k-Mean&i)**2; 
         %end;
      %end;
      TotalSS = css(of %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;);
      ModelSS = TotalSS - ErrorSS;
      NDF_&gName = &gNum - 1;
      DDF_&gName = &_nTallCol_ - &gNum;
      if (DDF_&gName > 0) then Variance_Residual = ErrorSS/DDF_&gName;
      else Variance_Residual = 0;
      %if (&EBShrink=yes) %then %do;
         Variance_Residual0 = Variance_Residual;
         if (Variance_Residual0 > 0) then do;
            OneOverVR0 = 1/Variance_Residual0;
         end;
         else do;
            OneOverVR0 = .;
         end;
         DDF_&gName.0 = DDF_&gName;
         * might should add 1 for Jeffreys, and also need to decide between mean and mode;
         DDF_&gName = &_nTallCol_ - &gNum + 2*&VR_Shape;
         if (DDF_&gName > 0) then do;
            Variance_Residual = (ErrorSS + 2*&VR_Scale)/DDF_&gName;
         end;
         else do;
            Variance_Residual = 0;
         end;
      %end;
      SampleVariance = TotalSS/(&_nTallCol_ - 1);
      if (Variance_Residual > 0) then PropVar_Residual = 1;
      else PropVar_Residual = 0;
      if (TotalSS > 0) then RSquare = ModelSS/TotalSS;
      else RSquare = .;
      if (NDF_&gName > 0) and (Variance_Residual > 0) then do;
         F_T3_&gName = ModelSS/NDF_&gName/Variance_Residual;
         if (F_T3_&gName > 0) then do;
            PrF_T3_&gName = 1 - probf(F_T3_&gName,NDF_&gName,DDF_&gName);
            if (PrF_T3_&gName < 1e-32) then NLP_T3_&gName = 32;
            else NLP_T3_&gName = -log10(PrF_T3_&gName);
         end;
         else do;
            PrF_T3_&gName = 1;
            NLP_T3_&gname = 0;
         end;
      end;
      else do;
         F_T3_&gName = .;
         PrF_T3_&gName = .;
         NLP_T3_&gname = .;
      end;
      
   %end;
   
   * compute standardized means;
   %if (&gNum = 2) %then %let LSMeanStdMethod = ;
   %if %length(&LSMeanStdMethod) %then %do;
      MeanMean = mean(of Mean1-Mean&gNum);
      StdDevMean = std(of Mean1-Mean&gNum);
      %do i=1 %to &gNum;
         %if (&LSMeanStdMethod=MEAN) %then %do;
            StdMean&i = Mean&i - MeanMean;
         %end;
         %else %do;
            if (StdDevMean > 0) then StdMean&i = (Mean&i - MeanMean)/StdDevMean;
            else StdMean&i = .;
         %end;
      %end;
   %end;

   * compute differences, std errors, and t-tests;
   %if &LSMeansDiffSet ^= None or &nEst %then %do;

      * use length statements to sort variables by type;
      %if (&LSMeansDiffSet ^= None) and &nEst %then %let nt = 2;
      %else %let nt = 1;
      %do v=1 %to 5;
         %do t=1 %to &nt;
            %if (&LSMeansDiffSet ^= None) and &t=1 %then %let tpref = Diff;
            %else %let tpref = Est;

            %if &v=1 %then %let vpref = &tpref;
            %else %if &v=2 %then %let vpref = StdErr&tpref;
            %else %if &v=3 %then %let vpref = t&tpref;
            %else %if &v=4 %then %let vpref = Probt&tpref;
            %else %if &v=5 %then %let vpref = NLP&tpref;

            %if ((&LSMeansDiffSet ^= None) and &t=1) %then %do;
               %do i=1 %to &gNum;
                  %if %index(&LSMeansDiffSet,Pairwise) %then %do;
                     %let jfirst=%eval(&i+1);
                     %let jlast=&gNum;
                  %end;
                  %else %do;
                     %let jfirst=&refset;
                     %if (&i=&refset) %then %do;
                        %let jlast=-1;
                     %end;
                     %else %do;
                        %let jlast=&refset;
                     %end;
                  %end;
                  %do j=&jfirst %to &jlast;
                     length &vpref._&i._&j 8;
                     %if (&v=2) and (&StdErrs=no) %then %do;
                        drop &vpref._&i._&j;
                     %end;
                     %if (&v=3) and (&TStats=no) %then %do;
                        drop &vpref._&i._&j;
                     %end;
                     %if (&v=4) and (&PValues=no) %then %do;
                        drop &vpref._&i._&j;
                     %end;
                  %end;
               %end;
            %end;
            %else %do;
               %do e=1 %to &nEst;
                  length &vpref._&e 8;
                  %if (&v=2) and (&StdErrs=no) %then %do;
                     drop &vpref._&e;
                  %end;
                  %if (&v=3) and (&TStats=no) %then %do;
                     drop &vpref._&e;
                  %end;
                  %if (&v=4) and (&PValues=no) %then %do;
                     drop &vpref._&e;
                  %end;
               %end;
            %end;
         %end;
      %end;

      * compute stats;
      %if (&LSMeansDiffSet ^= None) %then %do;
         %do i=1 %to &gNum;
            %if %index(&LSMeansDiffSet,Pairwise) %then %do;
               %let jfirst=%eval(&i+1);
               %let jlast=&gNum;
            %end;
            %else %do;
               %let jfirst=&refset;
               %if (&i=&refset) %then %do;
                  %let jlast=-1;
               %end;
               %else %do;
                  %let jlast=&refset;
               %end;
            %end;
            %if %length(&BlockVars) %then %do;
               %let isym = %eval(&i*(&i+1)/2);
            %end;
            %do j=&jfirst %to &jlast;
               Diff_&i._&j = Mean&i - Mean&j; 
               %if %length(&BlockVars) %then %do;
                  %let jsym = %eval(&j*(&j+1)/2);
                  %if (&i < &j) %then %do;
                     %let ijsym = %eval(&jsym - &j + &i);
                  %end;
                  %else %do;
                     %let ijsym = %eval(&isym - &i + &j);
                  %end;
                  StdErrDiff_&i._&j = sqrt(max(xpx[&isym] + xpx[&jsym] - 2*xpx[&ijsym],0));
               %end;
               %else %do;
                  if (&&gNum&i>0) and (&&gNum&j>0) then 
                     StdErrDiff_&i._&j = sqrt(Variance_Residual*(1/&&gNum&i + 1/&&gNum&j)); 
                  else StdErrDiff_&i._&j = .;
               %end;
               if (StdErrDiff_&i._&j > 0) then do;
                  tDiff_&i._&j = Diff_&i._&j/StdErrDiff_&i._&j; 
                  * ProbtDiff_&i._&j = min(2*probt(-abs(tDiff_&i._&j),DDF_&gName),1);
                  ProbtDiff_&i._&j = min(2*cdf('T',-abs(tDiff_&i._&j),DDF_&gName),1);
                  if missing(ProbtDiff_&i._&j) then NLPDiff_&i._&j = .;
                  else do;
                     if (ProbtDiff_&i._&j <= 1e-32) then NLPDiff_&i._&j = 32;
                     else NLPDiff_&i._&j = -log10(ProbtDiff_&i._&j); 
                  end;
               end;
               else do;
                  tDiff_&i._&j = .; 
                  ProbtDiff_&i._&j = .;
                  NLPDiff_&i._&j = .;
               end;
            %end;
         %end;
      %end;
      %do e=1 %to &nEst;
         %put e = &e;
         Est_&e = 0;
         varest = 0;
         * compute estimate and quadratic form for variance;
         %do i=1 %to &gNum;
            %if (%quote(&&estcoef&e._&i) ^= %quote(0)) %then %do;
               Est_&e = Est_&e + &&estcoef&e._&i * Mean&i; 
               %if %length(&BlockVars) %then %do;
                  %let isym = %eval(&i*(&i+1)/2);
                  %do j=1 %to &gNum;
                     %if (%quote(&&estcoef&e._&j) ^= %quote(0)) %then %do;
                        %if (&j = &i) %then %do;
                           %let ijsym = &isym;
                        %end;
                        %else %if (&i < &j) %then %do;
                           %let jsym = %eval(&j*(&j+1)/2);
                           %let ijsym = %eval(&jsym - &j + &i);
                        %end;
                        %else %do;
                           %let ijsym = %eval(&isym - &i + &j);
                        %end;
                        varest = varest + (&&estcoef&e._&i * &&estcoef&e._&j * xpx[&ijsym]);
                     %end;
                  %end;
               %end;
               %else %do;
                  if (&&gNum&i>0) then 
                     varest = varest + (&&estcoef&e._&i * &&estcoef&e._&i * Variance_Residual / &&gNum&i); 
                  else varest = .;
               %end;
            %end;
         %end;
         if (varest > 0) then StdErrEst_&e = sqrt(varest);
         else StdErrEst_&e = .;
         drop varest;

         if not missing(StdErrEst_&e) and (StdErrEst_&e > 0) then do;
            tEst_&e = Est_&e/StdErrEst_&e; 
            * ProbtEst_&e = min(2*probt(-abs(tEst_&e),DDF_&gName),1);
            ProbtEst_&e = min(2*cdf('T',-abs(tEst_&e),DDF_&gName),1);
            if missing(ProbtEst_&e) then NLPEst_&e = .;
            else do;
               if (ProbtEst_&e <= 1e-32) then NLPEst_&e = 32;
               else NLPEst_&e = -log10(ProbtEst_&e); 
            end;
         end;
         else do;
            tEst_&e = .; 
            ProbtEst_&e = .;
            NLPEst_&e = .;
         end;

      %end;
   %end;

   * labels for JMP;
   label    
      %do i=1 %to &gNum;
         Mean&i="Mean &gEffect &&gID&i" 
         %if %length(&LSMeanStdMethod) %then %do;
            StdMean&i="Std Mean &gEffect &&gID&i" 
         %end;
         %if &LSMeansDiffSet ^= None %then %do;
            %if %index(&LSMeansDiffSet,Pairwise) %then %do;
               %let jfirst=%eval(&i+1);
               %let jlast=&gNum;
            %end;
            %else %do;
               %let jfirst=&refset;
               %if (&i=&refset) %then %do;
                  %let jlast=-1;
               %end;
               %else %do;
                  %let jlast=&refset;
               %end;
            %end;
            %do j=&jfirst %to &jlast;
               Diff_&i._&j="%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)" 
               StdErrDiff_&i._&j="StdErr for %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)"
               tDiff_&i._&j="t Value for %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)"
               ProbtDiff_&i._&j="Prob>|t| for %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)"
               NLPDiff_&i._&j="-log10(p-value) for %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)"
            %end;
         %end;
      %end;
      %do e=1 %to &nEst;
         Est_&e="Estimate of &&estlab&e" 
         StdErrEst_&e="StdErr for Estimate of &&estlab&e"
         tEst_&e="t Value for Estimate of &&estlab&e"
         ProbtEst_&e="Prob>|t| for Estimate of &&estlab&e"
         NLPEst_&e="-log10(p-value) for Estimate of &&estlab&e"
      %end;
         
   ;
   drop %do i = 1 %to &_nAllColChunk_; &&_AllCol_&i._ %end;;
run;

%if (&syserr > 4) %then %do;
   %put ERROR: Unable to complete OneWay calculations.  Please scan the log for reasons.;
   %let exiterror = 1;
   %goto exit;
%end;

* merge annotation data;
%global ChrVar0 PosVar0;
%let ChrVar0 = &ChrVar;
%let PosVar0 = &PosVar;
%if %length (&AnnoInData) %then %do;
   /*
   data &OutData;
      set &OutData;
      %if %length(&ChrVar) %then %do;
         if missing(&ChrVar) then delete;
      %end;
      %if %length(&PosVar) %then %do;
         if missing(&PosVar) then delete;
      %end;
   run;
   */
   %MergeAnno(&OutData);
   %if &exiterror %then %goto exit;
   %let cp = 0;
   %if ^%length(&ChrVar) and %length(&AnnoChrVar) %then %do;
      %put Setting Chromosome variable to &AnnoChrVar..;
      %let ChrVar = &AnnoChrVar;
      %let cp = 1;
   %end;
   %if ^%length(&PosVar) and %length(&AnnoPosVar) %then %do;
      %put Setting Position variable to &AnnoPosVar..;
      %let PosVar = &AnnoPosVar;
      %let cp = 1;
   %end;
   * convert PosVar to numeric if necessary;
   %if %length(&PosVar) %then %do;
      %let pvt = ;
      %VarAttributes(&OutData,&PosVar,_Type=pvt);
      %if &pvt=C %then %do;
         data &OutData;
            set &OutData;
            _pvn = 0 + &PosVar;
            drop &PosVar;
         run;
         data &OutData;
            set &OutData;
            rename _pvn = &PosVar;
         run;
      %end;
   %end;
   * delete rows with missing position information;
   /*
   %if (&cp) %then %do;
      data &OutData;
         set &OutData;
         %if %length(&ChrVar) %then %do;
            if missing(&ChrVar) then delete;
         %end;
         %if %length(&PosVar) %then %do;
            if missing(&PosVar) then delete;
         %end; 
      run;
   %end;
   */
   %if %symexist(AnnoLabResid) %then %do;
      %if %upcase(&AnnoLabResid) = YES %then %do;
         %MergeAnno(&OutDataRes);
         %if &exiterror %then %goto mfexit;
      %end;
   %end;
%end;

* set up macro variables and label data sets for the MixedCreateJSL macro and drill down;
%let ByVars = _Row;
%let _ByVars = _Row;
%let ByVar1 = _Row;
%let LastByVar = _Row;
%let nby = 1;

%let HasT3 = 1;
%let HasLsm = 1;
%let nLsm = &gNum;

data t3;
   Effect = "&gEffect";
   _Row = 1;
   output;
   _Row = 2;
   output;
run;

data lsmb1;
   length llabel $ 256;
   %do i=1 %to &gNum;
      llabel = "Mean &gEffect &&gID&i";
      output;
   %end;
run;

%if %length(&LSMeanStdMethod) %then %do;
   data lsmsb1;
      length lslabel $ 256;
      %do i=1 %to &gNum;
         lslabel = "Std Mean &gEffect &&gID&i";
         output;
      %end;
   run;
%end;

%if (&LSMeansDiffSet ^= None) %then %do;
   %let HasDiff = 1;
   %let nLsm = &gNum;
   data diffsb1;
      length dlabel $ 256 nlpdlab $ 256;
      %do i=1 %to &gNum;
         %if %index(&LSMeansDiffSet,Pairwise) %then %do;
            %let jfirst=%eval(&i+1);
            %let jlast=&gNum;
         %end;
         %else %do;
            %let jfirst=&refset;
            %if (&i=&refset) %then %do;
               %let jlast=-1;
            %end;
            %else %do;
               %let jlast=&refset;
            %end;
         %end;
         %do j=&jfirst %to &jlast;
            dlabel = "%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)";
            nlpdlab = "-log10(p-value) for %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DiffofX_Text)), %nrbquote(&gEffect))) = (&&gID&i) - (&&gID&j)";
            output;
         %end;
      %end;
   run;
%end;
%else %let HasDiff = 0;

%if (&nEst) %then %do;
   %let HasEst = 1;
   data estsb1;
      length elabel $ 256 nlpelab $ 256;
      %do e=1 %to &nEst;
         elabel = "Estimate of &&estlab&e";
         nlpelab = "-log10(p-value) for Estimate of &&estlab&e";
         output;
      %end;
   run;
%end;
%else %let HasEst = 0;
 
%let SplitResults = no;
%let MultipleTestingFixedEffects = ;

%let HasCon = 0;
%let HasSlice = 0;

* set up JSL for drill down to plot means by chromosome and position;
%if ^%symexist(OneWayChrPosJSL) %then %let OneWayChrPosJSL = ;
%if %length(&OneWayChrPosJSL) %then %do;

   * create numerical index for chromosome by groups;
   proc sort data=&OutData;
      by &ChrVar &PosVar;
   run;
   data &OutData;
      set &OutData;
      retain _ByGroup 0;
      if &ChrVar ne lag(&ChrVar) then _ByGroup = _ByGroup + 1;
   run;

   * save number of obs in each by group;
   proc means data=&OutData noprint;
      by &ChrVar _ByGroup;
      var &PosVar;
      output out=nb n=nr;
   run;

   data _null_;
      dt = floor(datetime()) - 1551163755;
      call symput("dt",trim(left(dt)));
   run;

   data _null_;
      set nb end=last;
      call symputx("chrvar"||trim(left(_n_)),trim(left(&ChrVar)));
      call symputx("nr&dt._"||trim(left(_n_)),trim(left(nr)));
      if last then call symputx("nr&dt",trim(left(_n_)));
   run;
               
   %include "&MacroPath./WebSearchJSL.sas";

   data _null_;
      file "&OutPath.&OneWayChrPosJSL";
      %if %length(&AnnoInData) and %length(&AnnoLabel) %then %AnnoLabelJSL(Ds=&OutData);
      %let JMPPosVar = &PosVar;
      %CheckLabel(&OutData,JMPPosVar);
      put "objRefNS:posvar = ""&JMPPosVar"";";
      put "objRefNS:xvar = {Column(""&JMPPosVar"")};";
      put "objRefNS:yvar = ""Mean &gEffect &gID1"";";
      put "objRefNS:mvars = {";
      %do i=1 %to &gNum;
         put "   Column(""Mean &gEffect &&gID&i"")" @@;
         %if (&i < &gNum) %then %do;
            put ",";
         %end;
      %end;
      put "};";
      %let JMPChrVar = &ChrVar;
      %CheckLabel(&OutData,JMPChrVar);
      put "objRefNS:chrvar = ""&JMPChrVar"";";
      put "objRefNS:cvar = {Column(""&JMPChrVar"")};";
      put "objRefNS:dvar = {Column(""DDF_&gName"")};";
      put "objRefNS:nm = &gNum;";
      put "objRefNS:gsize = {" @@;
      %do i=1 %to &gNum;
         put "&&gNum&i" @@;
         %if (&i < &gNum) %then %do;
            put "," @@;
         %end;
      %end;
      put "};";
      put "objRefNS:nby = &&nr&dt;";
      put "objRefNS:nr&dt = {" @@;
      %do i=1 %to &&nr&dt;
         put "&&nr&dt._&i" @@;
         %if (&i < &&nr&dt) %then %do;
            put "," @@;
         %end;
      %end;
      put "};";
      %if %length(&TrackPath) %then %do;
         put "objRefNS:plot_tracks = 1;";
         put "objRefNS:chrvar = ""&ChrVar"";";
         put "objRefNS:trackfolder = ""&TrackPath"";";
         put "objRefNS:trackfiles = ""&TrackFiles"";";
         put "objRefNS:trackadd = &TrackAdd;";
         put "objRefNS:intervening = 1;";
      %end;
      %else %do;
         put "objRefNS:plot_tracks = 0;";
      %end;
      put "objRefNS:dt = &dt;";
   run; 

%end;


%if ^%symexist(OneWayLaunchJSL) %then %let OneWayLaunchJSL = 1;
%if (&OneWayLaunchJSL) %then %do;
   %MixedCreateJSL;
%end;

%MixedPackage;

%exit:;

%if ^%symexist(OneWayLaunchJSL) %then %let OneWayLaunchJSL = 1;
%if (&OneWayLaunchJSL) %then %do;
   %if ^%index(&ProcessName,Bivariate) %then 
      %ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%end;

%mend OneWay;




